@@ -84,7 +84,6 @@
 #include "nsXULTabAccessible.h"
 #include "nsXULTextAccessible.h"
 #include "nsXULTreeAccessible.h"
-#include "nsIAccessible.h"
 #include "nsCaretAccessible.h"
 #include "nsIAccessibleCaret.h"
 
@@ -434,28 +433,27 @@ nsAccessibilityService::CreateHTMLAreaAccessible(nsIWeakReference *aShell, nsIDO
   return NS_OK;
 }
 
-NS_IMETHODIMP 
-nsAccessibilityService::CreateHTMLBlockAccessible(nsIDOMNode* aDOMNode, nsISupports* aDocument, nsIAccessible **_retval)
+NS_IMETHODIMP
+nsAccessibilityService::CreateHTMLBlockAccessible(nsISupports *aFrame, nsIAccessible **_retval)
 {
-  nsCOMPtr<nsIDocument> document (do_QueryInterface(aDocument));
-  if (!document)
-    return NS_ERROR_FAILURE;
+#ifndef MOZ_ACCESSIBILITY_ATK
+  *_retval = nsnull;
+  return NS_ERROR_FAILURE;
+#else
+  nsIFrame* frame;
+  nsCOMPtr<nsIDOMNode> node;
+  nsCOMPtr<nsIWeakReference> weakShell;
+  nsresult rv = GetInfo(aFrame, &frame, getter_AddRefs(weakShell), getter_AddRefs(node));
+  if (NS_FAILED(rv))
+    return rv;
 
-#ifdef DEBUG
-  PRInt32 shells = document->GetNumberOfShells();
-  NS_ASSERTION(shells > 0,"Error no shells!");
-#endif
-
-  nsCOMPtr<nsIPresShell> tempShell;
-  document->GetShellAt(0, getter_AddRefs(tempShell));
-  nsCOMPtr<nsIWeakReference> weakShell = do_GetWeakReference(tempShell);
-
-  *_retval = new nsAccessible(aDOMNode, weakShell);
+  *_retval = new nsHTMLBlockAccessible(node, weakShell);
   if (! *_retval) 
     return NS_ERROR_OUT_OF_MEMORY;
 
   NS_ADDREF(*_retval);
   return NS_OK;
+#endif
 }
 
 NS_IMETHODIMP
@@ -610,7 +608,7 @@ nsAccessibilityService::CreateHTMLListboxAccessible(nsIDOMNode* aDOMNode, nsISup
 
   nsCOMPtr<nsIWeakReference> weakShell = do_GetWeakReference(presShell);
 
-  *_retval = new nsHTMLListboxAccessible(aDOMNode, weakShell);
+  *_retval = new nsHTMLSelectListAccessible(aDOMNode, weakShell);
   if (! *_retval) 
     return NS_ERROR_OUT_OF_MEMORY;
 
@@ -686,7 +684,10 @@ nsAccessibilityService::CreateHTMLRadioButtonAccessibleXBL(nsIDOMNode *aNode, ns
 }
 
 NS_IMETHODIMP 
-nsAccessibilityService::CreateHTMLSelectOptionAccessible(nsIDOMNode* aDOMNode, nsIAccessible *aAccParent, nsISupports* aPresContext, nsIAccessible **_retval)
+nsAccessibilityService::CreateHTMLSelectOptionAccessible(nsIDOMNode* aDOMNode, 
+																												 nsIAccessible *aAccParent, 
+																												 nsISupports* aPresContext, 
+																												 nsIAccessible **_retval)
 {
   nsCOMPtr<nsIPresContext> presContext(do_QueryInterface(aPresContext));
   NS_ASSERTION(presContext,"Error non prescontext passed to accessible factory!!!");
@@ -696,7 +697,7 @@ nsAccessibilityService::CreateHTMLSelectOptionAccessible(nsIDOMNode* aDOMNode, n
 
   nsCOMPtr<nsIWeakReference> weakShell = do_GetWeakReference(presShell);
 
-  *_retval = new nsHTMLSelectOptionAccessible(aAccParent, aDOMNode, weakShell);
+  *_retval = new nsHTMLSelectOptionAccessible(aDOMNode, weakShell);
   if (! *_retval) 
     return NS_ERROR_OUT_OF_MEMORY;
 
@@ -747,6 +748,10 @@ nsAccessibilityService::CreateHTMLTableCaptionAccessible(nsIDOMNode *aDOMNode, n
 NS_IMETHODIMP
 nsAccessibilityService::CreateHTMLTableHeadAccessible(nsIDOMNode *aDOMNode, nsIAccessible **_retval)
 {
+#ifndef MOZ_ACCESSIBILITY_ATK
+  *_retval = nsnull;
+  return NS_ERROR_FAILURE;
+#else
   NS_ENSURE_ARG_POINTER(aDOMNode);
 
   nsresult rv = NS_OK;
@@ -764,6 +769,7 @@ nsAccessibilityService::CreateHTMLTableHeadAccessible(nsIDOMNode *aDOMNode, nsIA
   NS_IF_ADDREF(*_retval);
 
   return rv;
+#endif
 }
 
 NS_IMETHODIMP
@@ -805,7 +811,7 @@ nsAccessibilityService::CreateHTMLTextAccessible(nsISupports *aFrame, nsIAccessi
     if (textLength == 1) {
       const PRUnichar NBSP = 160;
       const nsTextFragment *textFrag;
-      nsresult rv = textContent->GetText(&textFrag);
+      textContent->GetText(&textFrag);
       PRUnichar theChar = textFrag->CharAt(0);
       if (theChar == NBSP)
         return NS_ERROR_FAILURE;
@@ -843,6 +849,24 @@ nsAccessibilityService::CreateHTMLTextFieldAccessible(nsISupports *aFrame, nsIAc
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsAccessibilityService::CreateHTMLLabelAccessible(nsISupports *aFrame, nsIAccessible **_retval)
+{
+  nsIFrame* frame;
+  nsCOMPtr<nsIDOMNode> node;
+  nsCOMPtr<nsIWeakReference> weakShell;
+  nsresult rv = GetInfo(aFrame, &frame, getter_AddRefs(weakShell), getter_AddRefs(node));
+  if (NS_FAILED(rv))
+    return rv;
+
+  *_retval = new nsHTMLLabelAccessible(node, weakShell);
+  if (! *_retval) 
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*_retval);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsAccessibilityService::CreateHTMLHRAccessible(nsISupports *aFrame, nsIAccessible **_retval)
 {
@@ -1371,6 +1395,7 @@ NS_IMETHODIMP nsAccessibilityService::CreateXULToolbarSeparatorAccessible(nsIDOM
 NS_IMETHODIMP nsAccessibilityService::CreateXULTooltipAccessible(nsIDOMNode *aNode, nsIAccessible **_retval)
 {
 #ifdef MOZ_XUL
+#ifndef MOZ_ACCESSIBILITY_ATK
   nsCOMPtr<nsIWeakReference> weakShell;
   GetShellFromNode(aNode, getter_AddRefs(weakShell));
 
@@ -1379,6 +1404,9 @@ NS_IMETHODIMP nsAccessibilityService::CreateXULTooltipAccessible(nsIDOMNode *aNo
     return NS_ERROR_OUT_OF_MEMORY;
 
   NS_ADDREF(*_retval);
+#else
+  *_retval = nsnull;
+#endif // MOZ_ACCESSIBILITY_ATK
 #else
   *_retval = nsnull;
 #endif // MOZ_XUL
@@ -1500,7 +1528,7 @@ NS_IMETHODIMP nsAccessibilityService::GetAccessibleFor(nsIDOMNode *aNode,
   // Please leave this in for now, it's a convenient debugging method
   nsAutoString name;
   aNode->GetLocalName(name);
-  if (name.Equals(NS_LITERAL_STRING("INPUT"))) 
+  if (name.EqualsIgnoreCase("LABEL")) 
     printf("## aaronl debugging tag name\n");
 
   nsAutoString attrib;
@@ -1513,10 +1541,22 @@ NS_IMETHODIMP nsAccessibilityService::GetAccessibleFor(nsIDOMNode *aNode,
 #endif
   nsCOMPtr<nsIAccessibleProvider> accProv(do_QueryInterface(aNode));
   if (accProv) {
-    accProv->GetAccessible(_retval); 
-    if (*_retval)
-      return NS_OK;
-    return NS_ERROR_FAILURE;
+    accProv->GetAccessible(getter_AddRefs(newAcc)); 
+    if (! newAcc)
+      return NS_ERROR_FAILURE;
+#ifdef MOZ_ACCESSIBILITY_ATK
+    PRUint32 role, state;
+    newAcc->GetAccRole(&role);
+    // don't create the accessible object for popup widget when it's not visible
+    if (role == nsIAccessible::ROLE_MENUPOPUP) {
+      newAcc->GetAccState(&state);
+      if (state & (nsIAccessible::STATE_INVISIBLE | nsIAccessible::STATE_OFFSCREEN))
+        return NS_ERROR_FAILURE;
+    }
+#endif // MOZ_ACCESSIBILITY_ATK
+    *_retval = newAcc;
+    NS_ADDREF(*_retval);
+    return NS_OK;
   }
 
 #ifdef MOZ_XUL
@@ -1595,6 +1635,7 @@ NS_IMETHODIMP nsAccessibilityService::GetAccessibleFor(nsIDOMNode *aNode,
 
   frame->GetAccessible(getter_AddRefs(newAcc));
 
+#ifndef MOZ_ACCESSIBILITY_ATK
   // ---- If link, create link accessible ----
   if (!newAcc) {
     // is it a link?
@@ -1604,6 +1645,7 @@ NS_IMETHODIMP nsAccessibilityService::GetAccessibleFor(nsIDOMNode *aNode,
       newAcc = new nsHTMLLinkAccessible(aNode, weakShell);
     }
   }
+#endif //MOZ_ACCESSIBILITY_ATK
 
   // ---- If <select> <option>, create select option accessible
     
@@ -1611,7 +1653,7 @@ NS_IMETHODIMP nsAccessibilityService::GetAccessibleFor(nsIDOMNode *aNode,
     nsCOMPtr<nsIDOMHTMLOptionElement> optionElement(do_QueryInterface(aNode));
     if (optionElement) {
       nsCOMPtr<nsIWeakReference> weakShell(do_GetWeakReference(shell));
-      newAcc = new nsHTMLSelectOptionAccessible(nsnull, aNode, weakShell);
+      newAcc = new nsHTMLSelectOptionAccessible(aNode, weakShell);
     }
   }
   // See if this is an <optgroup>, 
@@ -1620,7 +1662,7 @@ NS_IMETHODIMP nsAccessibilityService::GetAccessibleFor(nsIDOMNode *aNode,
     nsCOMPtr<nsIDOMHTMLOptGroupElement> optGroupElement(do_QueryInterface(aNode));
     if (optGroupElement) {
       nsCOMPtr<nsIWeakReference> weakShell(do_GetWeakReference(shell));
-      newAcc = new nsHTMLSelectOptGroupAccessible(nsnull, aNode, weakShell);
+      newAcc = new nsHTMLSelectOptGroupAccessible(aNode, weakShell);
     }
   }
 
