@@ -36,55 +36,42 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#include "nsIAccessible.h"
-#include "nsRootAccessible.h"
-#include "nsCOMPtr.h"
-#include "nsIDocument.h"
-#include "nsIDOMNSDocument.h"
-#include "nsIPresShell.h"
-#include "nsIPresContext.h"
-#include "nsIContent.h"
-#include "nsIFrame.h"
-#include "nsIDOMEventTarget.h"
-#include "nsIDOMElement.h"
-#include "nsIDOMNSEvent.h"
-#include "nsIDOMEventReceiver.h"
-#include "nsIDOMEventListener.h"
-#include "nsReadableUtils.h"
-#include "nsILink.h"
-#include "nsHTMLFormControlAccessible.h"
-#include "nsHTMLLinkAccessible.h"
-#include "nsIURI.h"
-#include "nsIDocShell.h"
-#include "nsIDocShellTreeItem.h"
-#include "nsIWebNavigation.h"
-#include "nsIXULDocument.h"
+// NOTE: alphabetically ordered
+#include "nsAccessibilityService.h"
+#include "nsAccessibleEventData.h"
+#include "nsHTMLSelectAccessible.h"
+#include "nsIAccessibleCaret.h"
+#include "nsIAccessibleHyperText.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMDocumentType.h"
-#include "nsINameSpaceManager.h"
-#include "nsIDOMNSHTMLSelectElement.h"
-#include "nsString.h"
-#include "nsXPIDLString.h"
-#include "nsIAccessibilityService.h"
-#include "nsIServiceManager.h"
-#include "nsHTMLSelectAccessible.h"
+#include "nsIDOMElement.h"
+#include "nsIDOMEventListener.h"
+#include "nsIDOMEventTarget.h"
+#include "nsIDOMHTMLAnchorElement.h"
+#include "nsIDOMHTMLInputElement.h"
 #include "nsIDOMHTMLSelectElement.h"
-#include "nsCURILoader.h"
-#include "nsIInterfaceRequestorUtils.h"
-#include "nsIScriptGlobalObject.h"
-#include "nsIDOMWindow.h"
-#include "nsIViewManager.h"
-#include "nsIScrollableView.h"
+#include "nsIDOMNSDocument.h"
+#include "nsIDOMNSEvent.h"
 #include "nsIDOMXULSelectCntrlEl.h"
 #include "nsIDOMXULSelectCntrlItemEl.h"
-#include "nsXULTreeAccessible.h"
+#include "nsIDocShell.h"
+#include "nsIDocShellTreeItem.h"
+#include "nsIDocument.h"
+#include "nsIFrame.h"
+#include "nsIInterfaceRequestorUtils.h"
+#include "nsINameSpaceManager.h"
+#include "nsIScriptGlobalObject.h"
+#include "nsIScrollableView.h"
+#include "nsIServiceManager.h"
 #include "nsITreeSelection.h"
-#include "nsAccessibilityService.h"
-#include "nsISelectionPrivate.h"
-#include "nsICaret.h"
-#include "nsIAccessibleCaret.h"
-#include "nsIDOMHTMLInputElement.h"
-#include "nsAccessibleEventData.h"
+#include "nsIURI.h"
+#include "nsIViewManager.h"
+#include "nsIWebNavigation.h"
+#include "nsIXULDocument.h"
+#include "nsLayoutAtoms.h"
+#include "nsReadableUtils.h"
+#include "nsRootAccessible.h"
+#include "nsXULTreeAccessible.h"
 
 NS_INTERFACE_MAP_BEGIN(nsRootAccessible)
   NS_INTERFACE_MAP_ENTRY(nsIAccessibleDocument)
@@ -262,8 +249,7 @@ void nsRootAccessible::ScrollTimerCallback(nsITimer *aTimer, void *aClosure)
     // We only want to fire accessibilty scroll event when scrolling stops or pauses
     // Therefore, we wait for no scroll events to occur between 2 ticks of this timer
     // That indicates a pause in scrolling, so we fire the accessibilty scroll event
-    if (rootAcc->mListener)
-      rootAcc->mListener->HandleEvent(nsIAccessibleEventListener::EVENT_SCROLLINGEND, rootAcc, nsnull);
+    rootAcc->HandleEvent(nsIAccessibleEventListener::EVENT_SCROLLINGEND, rootAcc, nsnull);
     rootAcc->mScrollPositionChangedTicks = 0;
     rootAcc->mScrollWatchTimer->Cancel();
     rootAcc->mScrollWatchTimer = nsnull;
@@ -348,11 +334,11 @@ NS_IMETHODIMP nsRootAccessible::AddAccessibleEventListener(nsIAccessibleEventLis
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
 
     // add ourself as a CheckboxStateChange listener (custom event fired in nsHTMLInputElement.cpp)
-    rv = target->AddEventListener(NS_LITERAL_STRING("CheckboxStateChange"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
+    rv = target->AddEventListener(NS_LITERAL_STRING("CheckboxStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
 
     // add ourself as a RadioStateChange Listener ( custom event fired in in nsHTMLInputElement.cpp  & radio.xml)
-    rv = target->AddEventListener(NS_LITERAL_STRING("RadioStateChange"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
+    rv = target->AddEventListener(NS_LITERAL_STRING("RadioStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
 
     rv = target->AddEventListener(NS_LITERAL_STRING("ListitemStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
@@ -390,14 +376,14 @@ NS_IMETHODIMP nsRootAccessible::RemoveAccessibleEventListener()
     if (target) { 
       target->RemoveEventListener(NS_LITERAL_STRING("focus"), NS_STATIC_CAST(nsIDOMFocusListener*, this), PR_TRUE);
       target->RemoveEventListener(NS_LITERAL_STRING("select"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
-      target->RemoveEventListener(NS_LITERAL_STRING("CheckboxStateChange"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
+      target->RemoveEventListener(NS_LITERAL_STRING("CheckboxStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
+      target->RemoveEventListener(NS_LITERAL_STRING("RadioStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
+      target->RemoveEventListener(NS_LITERAL_STRING("ListitemStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
       target->RemoveEventListener(NS_LITERAL_STRING("popupshowing"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
       target->RemoveEventListener(NS_LITERAL_STRING("popuphiding"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
       target->RemoveEventListener(NS_LITERAL_STRING("DOMMenuItemActive"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
       target->RemoveEventListener(NS_LITERAL_STRING("DOMMenuBarActive"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
       target->RemoveEventListener(NS_LITERAL_STRING("DOMMenuBarInactive"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
-      target->RemoveEventListener(NS_LITERAL_STRING("RadioStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
-      target->RemoveEventListener(NS_LITERAL_STRING("ListitemStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
     }
  
     if (mScrollWatchTimer) {
@@ -419,6 +405,9 @@ NS_IMETHODIMP nsRootAccessible::RemoveAccessibleEventListener()
     mCaretAccessible->RemoveSelectionListener();
     mCaretAccessible = nsnull;
   }
+
+  mAccService = nsnull;
+
   return NS_OK;
 }
 
@@ -432,7 +421,7 @@ NS_IMETHODIMP nsRootAccessible::GetCaretAccessible(nsIAccessibleCaret **aCaretAc
 void nsRootAccessible::FireAccessibleFocusEvent(nsIAccessible *focusAccessible, nsIDOMNode *focusNode)
 {
   if (focusAccessible && focusNode && gLastFocusedNode != focusNode) {
-    mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, focusAccessible, nsnull);
+    HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, focusAccessible, nsnull);
     NS_IF_RELEASE(gLastFocusedNode);
     PRUint32 role = ROLE_NOTHING;
     focusAccessible->GetAccRole(&role);
@@ -471,6 +460,16 @@ NS_IMETHODIMP nsRootAccessible::HandleEvent(nsIDOMEvent* aEvent)
       optionTargetNode = do_QueryInterface(selectItem);
     }
 
+#ifdef MOZ_ACCESSIBILITY_ATK
+    nsCOMPtr<nsIDOMHTMLAnchorElement> anchorElement(do_QueryInterface(targetNode));
+    if (anchorElement) {
+      nsCOMPtr<nsIDOMNode> blockNode;
+      // For ATK, we don't create any individual object for hyperlink, use its parent who has block frame instead
+      nsAccessible::GetParentBlockNode(targetNode, getter_AddRefs(blockNode));
+      targetNode = blockNode;
+    }
+#endif
+
     nsCOMPtr<nsIAccessible> accessible;
     if (NS_FAILED(mAccService->GetAccessibleFor(targetNode, getter_AddRefs(accessible))))
       return NS_OK;
@@ -502,40 +501,47 @@ NS_IMETHODIMP nsRootAccessible::HandleEvent(nsIDOMEvent* aEvent)
     // tree event
     if (treeItemAccessible && 
         (eventType.EqualsIgnoreCase("DOMMenuItemActive") || eventType.EqualsIgnoreCase("select"))) {
-      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, treeItemAccessible, nsnull);
+      HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, treeItemAccessible, nsnull);
       return NS_OK;
     }
 
     if (eventType.EqualsIgnoreCase("focus") || eventType.EqualsIgnoreCase("DOMMenuItemActive")) { 
       if (optionTargetNode &&
           NS_SUCCEEDED(mAccService->GetAccessibleFor(optionTargetNode, getter_AddRefs(accessible)))) {
+        if (eventType.EqualsIgnoreCase("focus")) {
+          nsCOMPtr<nsIAccessible> selectAccessible;
+          mAccService->GetAccessibleFor(targetNode, getter_AddRefs(selectAccessible));
+          if (selectAccessible) {
+            FireAccessibleFocusEvent(selectAccessible, targetNode);
+          }
+        }
         FireAccessibleFocusEvent(accessible, optionTargetNode);
       }
       else
         FireAccessibleFocusEvent(accessible, targetNode);
     }
     else if (eventType.EqualsIgnoreCase("ListitemStateChange")) {
-      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, nsnull);
+      HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, nsnull);
       FireAccessibleFocusEvent(accessible, optionTargetNode);
     }
     else if (eventType.EqualsIgnoreCase("CheckboxStateChange")) { 
-      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, nsnull);
+      HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, nsnull);
     }
     else if (eventType.EqualsIgnoreCase("RadioStateChange") ) {
       // first the XUL radio buttons
       if (targetNode &&
           NS_SUCCEEDED(mAccService->GetAccessibleFor(targetNode, getter_AddRefs(accessible)))) {
-        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, nsnull);
+        HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, nsnull);
         FireAccessibleFocusEvent(accessible, targetNode);
       }
       else { // for the html radio buttons -- apparently the focus code just works. :-)
-        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, nsnull);
+        HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, nsnull);
       }
     }
     else if (eventType.EqualsIgnoreCase("DOMMenuBarActive")) 
-      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_MENUSTART, accessible, nsnull);
+      HandleEvent(nsIAccessibleEventListener::EVENT_MENUSTART, accessible, nsnull);
     else if (eventType.EqualsIgnoreCase("DOMMenuBarInactive")) {
-      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_MENUEND, accessible, nsnull);
+      HandleEvent(nsIAccessibleEventListener::EVENT_MENUEND, accessible, nsnull);
       GetAccFocused(getter_AddRefs(accessible));
       if (accessible) {
         accessible->AccGetDOMNode(getter_AddRefs(targetNode));
@@ -553,58 +559,44 @@ NS_IMETHODIMP nsRootAccessible::HandleEvent(nsIDOMEvent* aEvent)
         PRUint32 role = ROLE_NOTHING;
         accessible->GetAccRole(&role);
         if (role == ROLE_MENUPOPUP)
-          mListener->HandleEvent(menuEvent, accessible, nsnull);
+          HandleEvent(menuEvent, accessible, nsnull);
       }
     }
 #else
     AtkStateChange stateData;
-    AtkTextChange textData;
     if (eventType.EqualsIgnoreCase("focus") || eventType.EqualsIgnoreCase("DOMMenuItemActive")) {
       if (treeItemAccessible) // use focused treeitem
-        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, treeItemAccessible, nsnull);
+        HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, treeItemAccessible, nsnull);
+      else if (anchorElement) {
+        nsCOMPtr<nsIAccessibleHyperText> hyperText(do_QueryInterface(accessible));
+        if (hyperText) {
+          PRInt32 selectedLink;
+          hyperText->GetSelectedLinkIndex(&selectedLink);
+          HandleEvent(nsIAccessibleEventListener::EVENT_ATK_LINK_SELECTED, accessible, &selectedLink);
+        }
+      }
       else if (optionTargetNode && // use focused option
           NS_SUCCEEDED(mAccService->GetAccessibleFor(optionTargetNode, getter_AddRefs(accessible))))
         FireAccessibleFocusEvent(accessible, optionTargetNode);
       else
         FireAccessibleFocusEvent(accessible, targetNode);
     }
-    else if (eventType.EqualsIgnoreCase("change")) {
-      if (selectElement) // it's a HTML <select>
-        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_ATK_SELECTION_CHANGE, accessible, nsnull);
-      else {
-        nsCOMPtr<nsIDOMHTMLInputElement> inputElement(do_QueryInterface(targetNode));
-        if (inputElement) { // it's a HTML <input>
-          accessible->GetAccState(&stateData.state);
-          stateData.enable = (stateData.state & STATE_CHECKED) != 0;
-          stateData.state = STATE_CHECKED;
-          mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, (AccessibleEventData*)&stateData);
-        }
-      }
-    }
     else if (eventType.EqualsIgnoreCase("select")) {
-      if (selectControl) // it's a XUL <listbox>
-        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_ATK_SELECTION_CHANGE, accessible, nsnull);
-      else if (treeBox && treeIndex >= 0) // it's a XUL <tree>
+      if (treeBox && treeIndex >= 0) // it's a XUL <tree>
         // use EVENT_FOCUS instead of EVENT_ATK_SELECTION_CHANGE
-        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, treeItemAccessible, nsnull);
-    }
-    else if (eventType.EqualsIgnoreCase("input")) {
-      // XXX kyle.yuan@sun.com future work, put correct values for text change data
-      textData.start = 0;
-      textData.length = 0;
-      textData.add = PR_TRUE;
-      nsAutoString accName;
-      accessible->GetAccValue(accName);
-      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_ATK_TEXT_CHANGE, accessible, (AccessibleEventData*)&textData);
+        HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, treeItemAccessible, nsnull);
     }
     else if (eventType.EqualsIgnoreCase("ListitemStateChange")) // it's a XUL <listbox>
-      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, accessible, nsnull);
+      HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, accessible, nsnull);
     else if (eventType.EqualsIgnoreCase("CheckboxStateChange") || // it's a XUL <checkbox>
              eventType.EqualsIgnoreCase("RadioStateChange")) { // it's a XUL <radio>
       accessible->GetAccState(&stateData.state);
       stateData.enable = (stateData.state & STATE_CHECKED) != 0;
       stateData.state = STATE_CHECKED;
-      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, (AccessibleEventData*)&stateData);
+      HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, &stateData);
+      if (eventType.EqualsIgnoreCase("RadioStateChange")) {
+        FireAccessibleFocusEvent(accessible, targetNode);
+      }
     }
     else if (eventType.EqualsIgnoreCase("popupshowing")) 
       FireAccessibleFocusEvent(accessible, targetNode);
@@ -630,6 +622,13 @@ NS_IMETHODIMP nsRootAccessible::GetTargetNode(nsIDOMEvent *aEvent, nsCOMPtr<nsID
   return rv;
 }
 
+NS_IMETHODIMP nsRootAccessible::HandleEvent(PRUint32 aEvent, nsIAccessible *aTarget, void * aData)
+{
+  if (mListener)
+    mListener->HandleEvent(aEvent, aTarget, aData);
+  return NS_OK;
+}
+
 // ------- nsIDOMFocusListener Methods (1) -------------
 
 NS_IMETHODIMP nsRootAccessible::Focus(nsIDOMEvent* aEvent) 
@@ -661,11 +660,7 @@ NS_IMETHODIMP nsRootAccessible::Change(nsIDOMEvent* aEvent)
   // this may be the event that we have the individual Accessible objects
   //  handle themselves -- have list/combos figure out the change in selection
   //  have textareas and inputs fire a change of state etc...
-#ifdef MOZ_ACCESSIBILITY_ATK
-  return HandleEvent(aEvent);
-#else
   return NS_OK;   // Ignore form change events in MSAA
-#endif
 }
 
 // gets Select events when text is selected in a textarea or input
@@ -677,11 +672,7 @@ NS_IMETHODIMP nsRootAccessible::Select(nsIDOMEvent* aEvent)
 // gets Input events when text is entered or deleted in a textarea or input
 NS_IMETHODIMP nsRootAccessible::Input(nsIDOMEvent* aEvent) 
 { 
-#ifndef MOZ_ACCESSIBILITY_ATK
   return NS_OK; 
-#else
-  return HandleEvent(aEvent);
-#endif
 }
 
 // ------- nsIDOMXULListener Methods (8) ---------------
@@ -825,8 +816,7 @@ void nsRootAccessible::FireDocLoadFinished()
     if (mBusy != eBusyStateDone) {
       mBusy = eBusyStateDone;
 #ifndef MOZ_ACCESSIBILITY_ATK
-      if (mListener)
-        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, this, nsnull);
+      HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, this, nsnull);
 #endif
     }
   }
@@ -864,16 +854,13 @@ NS_IMETHODIMP nsRootAccessible::OnLocationChange(nsIWebProgress *aWebProgress,
     mBusy = eBusyStateLoading; 
     // Fire a "new doc has started to load" event
 #ifndef MOZ_ACCESSIBILITY_ATK
-    if (mListener)
-      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, this, nsnull);
+    HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, this, nsnull);
 #else
-    if (mListener) {
-      AtkChildrenChange childrenData;
-      childrenData.index = -1;
-      childrenData.child = 0;
-      childrenData.add = PR_FALSE;
-      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_REORDER , this, (AccessibleEventData*)&childrenData);
-    }
+    AtkChildrenChange childrenData;
+    childrenData.index = -1;
+    childrenData.child = 0;
+    childrenData.add = PR_FALSE;
+    HandleEvent(nsIAccessibleEventListener::EVENT_REORDER , this, &childrenData);
 #endif
   }
 
@@ -896,6 +883,9 @@ NS_IMETHODIMP nsRootAccessible::OnSecurityChange(nsIWebProgress *aWebProgress,
   return NS_OK;
 }
 
+//-----------------------------------------------------
+// nsDocAccessibleMixin 
+//-----------------------------------------------------
 nsDocAccessibleMixin::nsDocAccessibleMixin(nsIDocument *aDoc):mDocument(aDoc)
 {
 }
@@ -936,9 +926,7 @@ NS_IMETHODIMP nsDocAccessibleMixin::GetURL(nsAString& aURL)
 
 NS_IMETHODIMP nsDocAccessibleMixin::GetTitle(nsAString& aTitle)
 {
-  // This doesn't leak - we don't own the const pointer that's returned
-  aTitle = *(mDocument->GetDocumentTitle());
-  return NS_OK;
+  return mDocument->GetDocumentTitle(aTitle);
 }
 
 NS_IMETHODIMP nsDocAccessibleMixin::GetMimeType(nsAString& aMimeType)
@@ -970,8 +958,9 @@ NS_IMETHODIMP nsDocAccessibleMixin::GetDocType(nsAString& aDocType)
 NS_IMETHODIMP nsDocAccessibleMixin::GetNameSpaceURIForID(PRInt16 aNameSpaceID, nsAString& aNameSpaceURI)
 {
   if (mDocument) {
-    nsCOMPtr<nsINameSpaceManager> nameSpaceManager;
-    if (NS_SUCCEEDED(mDocument->GetNameSpaceManager(*getter_AddRefs(nameSpaceManager)))) 
+    nsCOMPtr<nsINameSpaceManager> nameSpaceManager =
+        do_GetService(NS_NAMESPACEMANAGER_CONTRACTID);
+    if (nameSpaceManager) 
       return nameSpaceManager->GetNameSpaceURI(aNameSpaceID, aNameSpaceURI);
   }
   return NS_ERROR_FAILURE;
