@@ -40,9 +40,7 @@
 // NOTE: alphabetically ordered
 #include "nsAccessible.h"
 #include "nsFormControlAccessible.h"
-#include "nsHTMLAtoms.h"
 #include "nsHTMLFormControlAccessible.h"
-#include "nsIClipboard.h"
 #include "nsIDOMHTMLButtonElement.h"
 #include "nsIDOMHTMLFormElement.h"
 #include "nsIDOMHTMLInputElement.h"
@@ -53,14 +51,7 @@
 #include "nsIDOMXULButtonElement.h"
 #include "nsIDOMXULSelectCntrlItemEl.h"
 #include "nsIDOMXULSelectCntrlEl.h"
-#include "nsIEditor.h"
-#include "nsIEventStateManager.h"
 #include "nsIFrame.h"
-#include "nsITextControlFrame.h"
-#include "nsIPlaintextEditor.h"
-#include "nsISelectionController.h"
-#include "nsReadableUtils.h"
-#include "nsWeakReference.h"
 
 // --- checkbox -----
 
@@ -317,37 +308,30 @@ NS_IMETHODIMP nsHTML4ButtonAccessible::GetAccName(nsAString& _retval)
 nsHTMLTextFieldAccessible::nsHTMLTextFieldAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
 nsFormControlAccessible(aNode, aShell)
 { 
-  // In nsHTMLTextFieldAccessible, mDOMNode is a nsHTMLInputElement. But we need 
-  // a *true* text node(nsTextNode) for the text operation. It's the first child
-  // of our editor's root element
+#ifdef MOZ_ACCESSIBILITY_ATK
+  SetTextNode(aNode);
+
   nsCOMPtr<nsIPresShell> shell(do_QueryReferent(mPresShell));
-  if (!shell)
-    return;
-
-  nsCOMPtr<nsIContent> content(do_QueryInterface(mDOMNode));
-  nsIFrame *frame = nsnull;
-  shell->GetPrimaryFrameFor(content, &frame);
-  nsITextControlFrame *tframe = nsnull;
-  frame->QueryInterface(NS_GET_IID(nsITextControlFrame), (void**)&tframe);
-  if (!tframe)
-    return;
-
-  nsCOMPtr<nsIEditor> editor;
-  tframe->GetEditor(getter_AddRefs(editor));
-  if (!editor)
-    return;
-
-  nsCOMPtr<nsIDOMElement> rootElement;
-  editor->GetRootElement(getter_AddRefs(rootElement));
-  nsCOMPtr<nsIDOMNode> rootNode(do_QueryInterface(rootElement));
-  if (rootNode) {
-    nsCOMPtr<nsIDOMNode> domNode;
-    rootNode->GetFirstChild(getter_AddRefs(domNode));
-    SetTextNode(domNode);
+  if (shell) {
+    nsCOMPtr<nsIContent> content(do_QueryInterface(mDOMNode));
+    nsIFrame *frame = nsnull;
+    shell->GetPrimaryFrameFor(content, &frame);
+    nsITextControlFrame *textFrame;
+    frame->QueryInterface(NS_GET_IID(nsITextControlFrame), (void**)&textFrame);
+    if (textFrame) {
+      nsCOMPtr<nsIEditor> editor;
+      textFrame->GetEditor(getter_AddRefs(editor));
+      SetEditor(editor);
+    }
   }
+#endif //MOZ_ACCESSIBILITY_ATK
 }
 
-NS_IMPL_ISUPPORTS_INHERITED2(nsHTMLTextFieldAccessible, nsFormControlAccessible, nsIAccessibleEditableText, nsAccessibleText)
+#ifndef MOZ_ACCESSIBILITY_ATK
+NS_IMPL_ISUPPORTS_INHERITED0(nsHTMLTextFieldAccessible, nsFormControlAccessible)
+#else
+NS_IMPL_ISUPPORTS_INHERITED2(nsHTMLTextFieldAccessible, nsFormControlAccessible, nsIAccessibleEditableText, nsIAccessibleText)
+#endif
 
 NS_IMETHODIMP nsHTMLTextFieldAccessible::GetAccRole(PRUint32 *_retval)
 {
@@ -439,108 +423,6 @@ NS_IMETHODIMP nsHTMLTextFieldAccessible::GetAccState(PRUint32 *_retval)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsHTMLTextFieldAccessible::SetAttributes(PRInt32 aStartPos, PRInt32 aEndPos, nsISupports *aAttributes)
-{
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP nsHTMLTextFieldAccessible::SetTextContents(const nsAString &aText)
-{
-  nsCOMPtr<nsIDOMHTMLTextAreaElement> textArea(do_QueryInterface(mDOMNode));
-  if (textArea)
-    return textArea->SetValue(aText);
-  
-  nsCOMPtr<nsIDOMHTMLInputElement> inputElement(do_QueryInterface(mDOMNode));
-  if (inputElement)
-    return inputElement->SetValue(aText);
-
-  return NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP nsHTMLTextFieldAccessible::MakeSelection(PRInt32 aStartPos, PRInt32 aEndPos, nsIEditor **aEditor)
-{
-  nsCOMPtr<nsIPresShell> shell(do_QueryReferent(mPresShell));
-  if (!shell)
-    return NS_ERROR_FAILURE;  
-
-  AccTakeFocus();
-
-  nsCOMPtr<nsIContent> content(do_QueryInterface(mDOMNode));
-  nsIFrame *frame = nsnull;
-  shell->GetPrimaryFrameFor(content, &frame);
-  nsITextControlFrame *tframe = nsnull;
-  frame->QueryInterface(NS_GET_IID(nsITextControlFrame), (void**)&tframe);
-  if (!tframe)
-    return NS_ERROR_FAILURE;  
-
-  nsCOMPtr<nsIEditor> editor;
-  tframe->SetSelectionRange(aStartPos, aEndPos);
-  tframe->GetEditor(getter_AddRefs(editor));
-  if (!editor)
-    return NS_ERROR_FAILURE;  
-
-  *aEditor = editor;
-  NS_ADDREF(*aEditor);
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsHTMLTextFieldAccessible::InsertText(const nsAString &aText, PRInt32 aPosition)
-{
-  nsCOMPtr<nsIEditor> editor;
-  if (NS_SUCCEEDED(MakeSelection(aPosition, aPosition, getter_AddRefs(editor)))) {
-    nsCOMPtr<nsIPlaintextEditor> peditor(do_QueryInterface(editor));
-    peditor->InsertText(aText);
-    return NS_OK;
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP nsHTMLTextFieldAccessible::CopyText(PRInt32 aStartPos, PRInt32 aEndPos)
-{
-  nsCOMPtr<nsIEditor> editor;
-  if (NS_SUCCEEDED(MakeSelection(aStartPos, aEndPos, getter_AddRefs(editor)))) {
-    editor->Copy();
-    return NS_OK;
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP nsHTMLTextFieldAccessible::CutText(PRInt32 aStartPos, PRInt32 aEndPos)
-{
-  nsCOMPtr<nsIEditor> editor;
-  if (NS_SUCCEEDED(MakeSelection(aStartPos, aEndPos, getter_AddRefs(editor)))) {
-    editor->Cut();
-    return NS_OK;
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP nsHTMLTextFieldAccessible::DeleteText(PRInt32 aStartPos, PRInt32 aEndPos)
-{
-  nsCOMPtr<nsIEditor> editor;
-  if (NS_SUCCEEDED(MakeSelection(aStartPos, aEndPos, getter_AddRefs(editor)))) {
-    editor->DeleteSelection(nsIEditor::eNone);
-    return NS_OK;
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP nsHTMLTextFieldAccessible::PasteText(PRInt32 aPosition)
-{
-  nsCOMPtr<nsIEditor> editor;
-  if (NS_SUCCEEDED(MakeSelection(aPosition, aPosition, getter_AddRefs(editor)))) {
-    editor->Paste(nsIClipboard::kGlobalClipboard);
-    return NS_OK;
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
 // --- groupbox  -----
 
 /*
