@@ -39,20 +39,12 @@
  * ***** END LICENSE BLOCK ***** */
 
 // NOTE: alphabetically ordered
-#include "nsIDocument.h"
-#include "nsIDOMNodeList.h"
+#include "nsXULFormControlAccessible.h"
 #include "nsIDOMXULButtonElement.h"
 #include "nsIDOMXULCheckboxElement.h"
-#include "nsIDOMXULDocument.h"
-#include "nsIDOMXULLabelElement.h"
 #include "nsIDOMXULMenuListElement.h"
-#include "nsIDOMXULSelectCntrlEl.h"
 #include "nsIDOMXULSelectCntrlItemEl.h"
-#include "nsReadableUtils.h"
-#include "nsString.h"
-#include "nsXULFormControlAccessible.h"
-#include "nsIAccessibilityService.h"
-#include "nsIServiceManager.h"
+#include "nsAccessibleTreeWalker.h"
 
 /**
   * XUL Button: can contain arbitrary HTML content
@@ -64,7 +56,7 @@
 
 // Don't inherit from nsFormControlAccessible - it doesn't allow children and a button can have a dropmarker child
 nsXULButtonAccessible::nsXULButtonAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsAccessible(aNode, aShell)
+nsAccessibleWrap(aNode, aShell)
 { 
 }
 
@@ -167,22 +159,25 @@ NS_IMETHODIMP nsXULButtonAccessible::GetAccState(PRUint32 *_retval)
   */
 NS_IMETHODIMP nsXULButtonAccessible::GetAccFirstChild(nsIAccessible **aResult)
 {
-  *aResult = nsnull;
+  if (!mFirstChild) {
+    nsAccessibleTreeWalker walker(mWeakShell, mDOMNode, PR_TRUE);
+    walker.GetLastChild();
 
-  nsCOMPtr<nsIAccessible> testAccessible;
-  nsAccessible::GetAccLastChild(getter_AddRefs(testAccessible));
+    // If the anonymous tree walker can find accessible children, 
+    // and the last one is a push button, then use it as the only accessible 
+    // child -- because this is the scenario where we have a dropmarker child
 
-  // If the anonymous tree walker can find accessible children, and the last one is a push button, 
-  // then use it as the only accessible child -- because this is the scenario where we have a dropmarker child
-
-  if (testAccessible) {    
-    PRUint32 role;
-    if (NS_SUCCEEDED(testAccessible->GetAccRole(&role)) && role == ROLE_PUSHBUTTON) {
-      *aResult = testAccessible;
-      NS_ADDREF(*aResult);
+    if (walker.mState.accessible) {    
+      PRUint32 role;
+      if (NS_SUCCEEDED(walker.mState.accessible->GetAccRole(&role)) && role == ROLE_PUSHBUTTON) {
+        mFirstChild = walker.mState.accessible;
+        mFirstChild->SetAccNextSibling(nsnull);
+      }
     }
   }
 
+  mAccChildCount = (mFirstChild != nsnull);
+  NS_IF_ADDREF(*aResult = mFirstChild);
   return NS_OK;
 }
 
@@ -193,12 +188,9 @@ NS_IMETHODIMP nsXULButtonAccessible::GetAccLastChild(nsIAccessible **aResult)
 
 NS_IMETHODIMP nsXULButtonAccessible::GetAccChildCount(PRInt32 *aResult)
 {
-  *aResult = 0;
-
   nsCOMPtr<nsIAccessible> accessible;
   GetAccFirstChild(getter_AddRefs(accessible));
-  if (accessible)
-    *aResult = 1;
+  *aResult = mAccChildCount;
 
   return NS_OK;
 }
@@ -352,7 +344,6 @@ NS_IMETHODIMP nsXULCheckboxAccessible::GetAccActionName(PRUint8 index, nsAString
 NS_IMETHODIMP nsXULCheckboxAccessible::AccDoAction(PRUint8 index)
 {
   if (index == eAction_Click) {
-    PRBool checked = PR_FALSE;
     nsCOMPtr<nsIDOMXULCheckboxElement> xulCheckboxElement(do_QueryInterface(mDOMNode));
     if (xulCheckboxElement) {
       xulCheckboxElement->Click();
@@ -393,7 +384,7 @@ NS_IMETHODIMP nsXULCheckboxAccessible::GetAccState(PRUint32 *_retval)
   */
 
 nsXULGroupboxAccessible::nsXULGroupboxAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsAccessible(aNode, aShell)
+nsAccessibleWrap(aNode, aShell)
 { 
 }
 
@@ -435,7 +426,7 @@ NS_IMETHODIMP nsXULGroupboxAccessible::GetAccName(nsAString& _retval)
 /**
   * progressmeter
   */
-NS_IMPL_ISUPPORTS_INHERITED1(nsXULProgressMeterAccessible, nsFormControlAccessible, nsIAccessibleValue)
+NS_IMPL_ISUPPORTS_INHERITED0(nsXULProgressMeterAccessible, nsFormControlAccessible)
 
 nsXULProgressMeterAccessible::nsXULProgressMeterAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
 nsFormControlAccessible(aNode, aShell)
@@ -467,55 +458,6 @@ NS_IMETHODIMP nsXULProgressMeterAccessible::GetAccValue(nsAString& _retval)
   return NS_OK;
 }
 
-/* readonly attribute double maximumValue; */
-NS_IMETHODIMP nsXULProgressMeterAccessible::GetMaximumValue(double *aMaximumValue)
-{
-  *aMaximumValue = 1; // 100% = 1;
-  return NS_OK;
-}
-
-/* readonly attribute double minimumValue; */
-NS_IMETHODIMP nsXULProgressMeterAccessible::GetMinimumValue(double *aMinimumValue)
-{
-  *aMinimumValue = 0;
-  return NS_OK;
-}
-
-/* readonly attribute double currentValue; */
-NS_IMETHODIMP nsXULProgressMeterAccessible::GetCurrentValue(double *aCurrentValue)
-{
-  nsAutoString currentValue;
-  GetAccValue(currentValue);
-  PRInt32 error;
-  *aCurrentValue = currentValue.ToFloat(&error) / 100;
-  return NS_OK;
-}
-
-/* boolean setCurrentValue (in double value); */
-NS_IMETHODIMP nsXULProgressMeterAccessible::SetCurrentValue(double aValue, PRBool *_retval)
-{
-  //Here I do not suppose the min/max are 0/1.00 because I want
-  // these part of code to be more extensible.
-  *_retval = PR_FALSE;
-  double min, max;
-  GetMinimumValue(&min);
-  GetMaximumValue(&max);
-  if (aValue > max || aValue < min)
-    return NS_ERROR_INVALID_ARG;
-
-  nsCOMPtr<nsIDOMElement> element(do_QueryInterface(mDOMNode));
-  NS_ASSERTION(element, "No element for DOM node!");
-  PRUint32 value = PRUint32(aValue * 100.0 + 0.5);
-  nsAutoString valueString;
-  valueString.AppendInt(value);
-  valueString.Append(NS_LITERAL_STRING("%"));
-  if (NS_SUCCEEDED(element->SetAttribute(NS_LITERAL_STRING("value"), valueString))) {
-    *_retval = PR_TRUE;
-    return NS_OK;
-  }
-  return NS_ERROR_INVALID_ARG;
-}
-
 /**
   * XUL Radio Button
   */
@@ -575,7 +517,7 @@ NS_IMETHODIMP nsXULRadioButtonAccessible::GetAccParent(nsIAccessible **  aAccPar
     nsCOMPtr<nsIAccessible> tempParent;
     nsAccessible::GetAccParent(getter_AddRefs(tempParent));
     if (tempParent)
-      tempParent->GetAccParent(getter_AddRefs(mParent));
+      tempParent->GetAccParent(&mParent);  // mParent is a weak pointer
   }
   NS_ASSERTION(mParent,"Whoa! This RadioButtonAcc doesn't have a parent! Better find out why.");
   *aAccParent = mParent;
@@ -594,7 +536,7 @@ NS_IMETHODIMP nsXULRadioButtonAccessible::GetAccParent(nsIAccessible **  aAccPar
 
 /** Constructor */
 nsXULRadioGroupAccessible::nsXULRadioGroupAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsAccessible(aNode, aShell)
+nsAccessibleWrap(aNode, aShell)
 { 
 }
 
@@ -621,7 +563,7 @@ NS_IMETHODIMP nsXULRadioGroupAccessible::GetAccState(PRUint32 *_retval)
   * Default Constructor
   */
 nsXULStatusBarAccessible::nsXULStatusBarAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsAccessible(aNode, aShell)
+nsAccessibleWrap(aNode, aShell)
 { 
 }
 
@@ -645,7 +587,7 @@ NS_IMETHODIMP nsXULStatusBarAccessible::GetAccState(PRUint32 *_retval)
   */
 
 nsXULToolbarAccessible::nsXULToolbarAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsAccessible(aNode, aShell)
+nsAccessibleWrap(aNode, aShell)
 { 
 }
 
