@@ -40,6 +40,106 @@
                              
 icaltimetype ConvertFromPrtime( PRTime indate );
 PRTime ConvertToPrtime ( icaltimetype indate );
+extern "C" {
+extern icalarray *builtin_timezones;
+extern char *gDefaultTzidPrefix;
+}
+//icaltimetype ConvertFromPrtime( PRTime indate );
+void icaltimezone_init_mozilla_zones (void)
+{
+    gDefaultTzidPrefix="/Mozilla.org/";
+
+    char timezonecalstr[] = \
+"BEGIN:VCALENDAR\n\
+PRODID:-//Mozilla.org/NONSGML Mozilla Calendar Timezone Table V1.0//EN\n\
+VERSION:2.0\n\
+END:VCALENDAR\n\
+";
+
+    char *timezoneformatstr = "BEGIN:VTIMEZONE\n\
+TZID:/Mozilla.org/BasicTimezones/NH-GMT%c%02d:%02d\n\
+LOCATION:NH-GMT%c%02d:%02d\n\
+BEGIN:STANDARD\n\
+TZOFFSETFROM:%c%02d%02d\n\
+TZOFFSETTO:%c%02d%02d\n\
+TZNAME:NHS-GMT%c%02d:%02d\n\
+DTSTART:19991031T020000\n\
+RRULE:FREQ=YEARLY;BYMONTH=10;BYDAY=-1SU\n\
+END:STANDARD\n\
+BEGIN:DAYLIGHT\n\
+TZOFFSETFROM:%c%02d%02d\n\
+TZOFFSETTO:%c%02d%02d\n\
+TZNAME:NHD-GMT%c%02d:%02d\n\
+DTSTART:20000402T020000\n\
+RRULE:FREQ=YEARLY;BYMONTH=4;BYMONTHDAY=1,2,3,4,5,6,7;BYDAY=SU\n\
+END:DAYLIGHT\n\
+END:VTIMEZONE\n";
+
+    char timezonestr[1024];
+
+    builtin_timezones = icalarray_new ( 44, 32); //HARDCODED sizeof(icaltimezone)=44
+
+    icalcomponent *vcalendar = icalparser_parse_string(timezonecalstr);
+    icalcomponent *vtimezone;
+
+    //Components should be sorted by location
+    //+00:00 to +12:00
+    int i;
+    for( i=0; i<25; i++ ) {
+        char sign = '+';
+        int hour = i/2;
+        int minute = i%2==0 ? 00 : 30;
+        int hour2 = (i+2)/2;
+        sprintf( timezonestr, timezoneformatstr
+                 , sign, hour, minute
+                 , sign, hour, minute
+                 , sign, hour2, minute
+                 , sign, hour, minute
+                 , sign, hour, minute
+                 , sign, hour, minute
+                 , sign, hour2, minute
+                 , sign, hour, minute );
+        vtimezone = icalcomponent_new_from_string( timezonestr );
+        icalcomponent_add_component( vcalendar, vtimezone );
+    }
+    //-00:30 to -12:00
+    for( i=-1; i>-25; i-- ) {
+        char sign = '-';
+        int hour = -1*i/2;
+        int minute = i%2==0 ? 00 : 30;
+        char sign2 = (i+2)<0 ? '-' : '+';
+        int hour2 = (i+2)<0 ? -1*(i+2)/2 : (i+2)/2;
+        sprintf( timezonestr, timezoneformatstr
+                 , sign, hour, minute
+                 , sign, hour, minute
+                 , sign2, hour2, minute
+                 , sign, hour, minute
+                 , sign, hour, minute
+                 , sign, hour, minute
+                 , sign2, hour2, minute
+                 , sign, hour, minute );
+        vtimezone = icalcomponent_new_from_string( timezonestr );
+        icalcomponent_add_component( vcalendar, vtimezone );
+    }
+
+    for( vtimezone = icalcomponent_get_first_component( vcalendar, ICAL_VTIMEZONE_COMPONENT );
+         vtimezone != NULL;
+         vtimezone = icalcomponent_get_next_component( vcalendar, ICAL_VTIMEZONE_COMPONENT ) ) {
+        icaltimezone *zone=icaltimezone_new();
+        icaltimezone_get_vtimezone_properties  ( zone, vtimezone );
+        icalarray_append (builtin_timezones, zone);
+    }
+
+    PRExplodedTime ext;
+    PR_ExplodeTime( PR_Now(), PR_LocalTimeParameters, &ext);
+    int gmtoffsethour = ext.tm_params.tp_gmt_offset < 0 ? -1*ext.tm_params.tp_gmt_offset / 3600 : ext.tm_params.tp_gmt_offset / 3600;
+    int gmtoffsetminute = ext.tm_params.tp_gmt_offset%3600 ? 30 : 00;
+    char zone_location[20];
+    sprintf( zone_location, "NH-GMT%c%02d:%02d", ext.tm_params.tp_gmt_offset < 0 ? '-' : '+'
+                                                , gmtoffsethour, gmtoffsetminute );
+
+    currenttimezone = icaltimezone_get_builtin_timezone ( zone_location );
+}
 
 oeICalContainerImpl::oeICalContainerImpl()
 {
@@ -73,6 +173,8 @@ oeICalContainerImpl::oeICalContainerImpl()
         m_filter = new oeICalContainerFilter();
         NS_ADDREF( m_filter );
         m_filter->m_calendarArray = m_calendarArray;
+
+        icaltimezone_init_mozilla_zones ();
 }
 
 oeICalContainerImpl::~oeICalContainerImpl()
@@ -81,6 +183,7 @@ oeICalContainerImpl::~oeICalContainerImpl()
     printf( "oeICalContainerImpl::~oeICalContainerImpl()\n" );
 #endif
 
+    m_calendarArray->Clear();
     m_calendarArray = nsnull;
     m_observerArray = nsnull;
     m_todoobserverArray = nsnull;
@@ -148,12 +251,13 @@ oeICalContainerImpl::GetCalendar( const char *server, oeIICal **calendar ) {
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* tmpcalendar;
+        nsCOMPtr<oeIICal> tmpcalendar;
         char *tmpserver;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&tmpcalendar );
+        m_calendarArray->GetElementAt( i, getter_AddRefs( tmpcalendar ) );
         tmpcalendar->GetServer( &tmpserver );
         if( strcmp( tmpserver, server ) == 0 ) {
             *calendar = tmpcalendar;
+            NS_ADDREF( *calendar );
             nsMemory::Free( tmpserver );
             return NS_OK;
         }
@@ -174,9 +278,9 @@ oeICalContainerImpl::RemoveCalendar( const char *server ) {
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* tmpcalendar;
+        nsCOMPtr<oeIICal> tmpcalendar;
         char *tmpserver;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&tmpcalendar );
+        m_calendarArray->GetElementAt( i, getter_AddRefs( tmpcalendar ) );
         tmpcalendar->GetServer( &tmpserver );
         if( strcmp( tmpserver, server ) == 0 ) {
             nsMemory::Free( tmpserver );
@@ -222,8 +326,8 @@ NS_IMETHODIMP oeICalContainerImpl::SetBatchMode(PRBool aBatchMode)
         m_calendarArray->Count( &num );
         for( i=0; i<num; i++ ) 
         {
-            oeIICal* calendar;
-            m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+            nsCOMPtr<oeIICal> calendar;
+            m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
             calendar->SetBatchMode( aBatchMode );
         }
 
@@ -321,6 +425,7 @@ NS_IMETHODIMP oeICalContainerImpl::ModifyEvent( oeIICalEvent *icalevent, char **
         return NS_ERROR_FAILURE;
     }
     nsresult rv = calendar->ModifyEvent( icalevent, retid );
+    NS_RELEASE( calendar );
     return rv;
 }
 
@@ -345,8 +450,8 @@ NS_IMETHODIMP oeICalContainerImpl::FetchEvent( const char *id, oeIICalEvent **ev
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->FetchEvent( id, ev );
         if( *ev )
             break;
@@ -385,6 +490,7 @@ oeICalContainerImpl::DeleteEvent( const char *id )
         return NS_ERROR_FAILURE;
     }
     nsresult rv = calendar->DeleteEvent( id );
+    NS_RELEASE( calendar );
 
 	return rv;
 }
@@ -408,8 +514,8 @@ oeICalContainerImpl::GetAllEvents(nsISimpleEnumerator **resultList )
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->GetAllEvents( (nsISimpleEnumerator **)&eventEnum );
     }
 
@@ -434,8 +540,8 @@ oeICalContainerImpl::GetEventsForMonth( PRTime datems, nsISimpleEnumerator **eve
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->GetEventsForMonth( datems, (nsISimpleEnumerator **)&eventEnum );
     }
 
@@ -460,8 +566,8 @@ oeICalContainerImpl::GetEventsForWeek( PRTime datems, nsISimpleEnumerator **even
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->GetEventsForWeek( datems, (nsISimpleEnumerator **)&eventEnum );
     }
     return NS_OK;
@@ -486,8 +592,8 @@ oeICalContainerImpl::GetEventsForDay( PRTime datems, nsISimpleEnumerator **event
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->GetEventsForDay( datems, (nsISimpleEnumerator **)&eventEnum );
     }
     return NS_OK;
@@ -512,8 +618,8 @@ oeICalContainerImpl::GetEventsForRange( PRTime checkdateinms, PRTime checkenddat
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->GetEventsForRange( checkdateinms, checkenddateinms, (nsISimpleEnumerator **)&eventEnum );
     }
 
@@ -539,8 +645,8 @@ oeICalContainerImpl::GetFirstEventsForRange( PRTime checkdateinms, PRTime checke
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->GetFirstEventsForRange( checkdateinms, checkenddateinms, (nsISimpleEnumerator **)&eventEnum );
     }
 
@@ -559,8 +665,9 @@ icaltimetype oeICalContainerImpl::GetNextEvent( icaltimetype starting ) {
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeICalImpl* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> tmpcalendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(tmpcalendar) );
+        oeICalImpl *calendar = (oeICalImpl *)(&tmpcalendar);
         icaltimetype next = calendar->GetNextEvent( starting );
         if( !icaltime_is_null_time( next ) && ( icaltime_is_null_time( soonest ) || (icaltime_compare( soonest, next ) > 0) ) ) {
             soonest = next;
@@ -597,8 +704,9 @@ oeICalContainerImpl::GetNextNEvents( PRTime datems, PRInt32 maxcount, nsISimpleE
             m_calendarArray->Count( &num );
             for( i=0; i<num && count<maxcount; i++ ) 
             {
-                oeICalImpl* calendar;
-                m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+                nsCOMPtr<oeIICal> tmpcalendar;
+                m_calendarArray->GetElementAt( i, getter_AddRefs(tmpcalendar) );
+                oeICalImpl *calendar = (oeICalImpl *)(&tmpcalendar);
                 EventList *tmplistptr = calendar->GetEventList();
                 while( tmplistptr && count<maxcount ) {
                     if( tmplistptr->event ) {
@@ -637,8 +745,8 @@ oeICalContainerImpl::AddObserver(oeIICalObserver *observer)
         m_calendarArray->Count( &num );
         for( i=0; i<num; i++ ) 
         {
-            oeICalImpl* calendar;
-            m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+            nsCOMPtr<oeIICal> calendar;
+            m_calendarArray->GetElementAt( i, getter_AddRefs( calendar ) );
             calendar->AddObserver( observer );
         }
         observer->OnLoad();
@@ -658,8 +766,8 @@ oeICalContainerImpl::RemoveObserver(oeIICalObserver *observer)
         m_calendarArray->Count( &num );
         for( i=0; i<num; i++ ) 
         {
-            oeICalImpl* calendar;
-            m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+            nsCOMPtr<oeIICal> calendar;
+            m_calendarArray->GetElementAt( i, getter_AddRefs( calendar ) );
             calendar->RemoveObserver( observer );
         }
         m_observerArray->RemoveElement( observer );
@@ -683,8 +791,8 @@ oeICalContainerImpl::AddTodoObserver(oeIICalTodoObserver *observer)
         m_calendarArray->Count( &num );
         for( i=0; i<num; i++ ) 
         {
-            oeICalImpl* calendar;
-            m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+            nsCOMPtr<oeIICal> calendar;
+            m_calendarArray->GetElementAt( i, getter_AddRefs( calendar ) );
             calendar->AddTodoObserver( observer );
         }
         observer->OnLoad();
@@ -704,8 +812,8 @@ oeICalContainerImpl::RemoveTodoObserver(oeIICalTodoObserver *observer)
         m_calendarArray->Count( &num );
         for( i=0; i<num; i++ ) 
         {
-            oeICalImpl* calendar;
-            m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+            nsCOMPtr<oeIICal> calendar;
+            m_calendarArray->GetElementAt( i, getter_AddRefs( calendar ) );
             calendar->RemoveTodoObserver( observer );
         }
         m_todoobserverArray->RemoveElement( observer );
@@ -770,6 +878,7 @@ oeICalContainerImpl::DeleteTodo( const char *id )
         return NS_ERROR_FAILURE;
     }
     nsresult rv = calendar->DeleteTodo( id );
+    NS_RELEASE( calendar );
 
 	return rv;
 }
@@ -795,8 +904,8 @@ NS_IMETHODIMP oeICalContainerImpl::FetchTodo( const char *id, oeIICalTodo **ev)
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->FetchTodo( id, ev );
         if( *ev )
             break;
@@ -819,6 +928,7 @@ NS_IMETHODIMP oeICalContainerImpl::ModifyTodo(oeIICalTodo *icalevent, char **ret
         return NS_ERROR_FAILURE;
     }
     nsresult rv = calendar->ModifyTodo( icalevent, retid );
+    NS_RELEASE( calendar );
     return rv;
 }
 
@@ -841,8 +951,8 @@ oeICalContainerImpl::GetAllTodos(nsISimpleEnumerator **resultList )
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs( calendar ) );
         calendar->GetAllTodos( (nsISimpleEnumerator **)&eventEnum );
     }
     return NS_OK;
@@ -870,8 +980,8 @@ NS_IMETHODIMP oeICalContainerImpl::ResetFilter()
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->ResetFilter();
     }
     oeIDateTime *completed;
@@ -1302,8 +1412,8 @@ void oeICalContainerFilter::UpdateAllFilters( PRInt32 icaltype )
         m_calendarArray->Count( &num );
         for( i=0; i<num; i++ ) 
         {
-            oeIICal* calendar;
-            m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+            nsCOMPtr<oeIICal> calendar;
+            m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
             oeIICalTodo *filter;
             calendar->GetFilter( &filter );
             oeIDateTime *completed;
@@ -1445,3 +1555,14 @@ void oeFilterDateTime::SetFieldType( PRInt32 icaltype )
 {
     m_icaltype = icaltype;
 }
+
+NS_IMETHODIMP oeFilterDateTime::GetTzID(char **aRetVal)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP oeFilterDateTime::SetTimeInTimezone( PRTime ms, const char *tzid )
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
