@@ -122,6 +122,19 @@ PRTime ConvertToPrtime ( icaltimetype indate ) {
     return result;
 }
 
+extern "C" {
+#include "token.h"
+}
+
+void GenerateUUID(char *output) {
+  uuid_state state;
+  uuid_t uuid;
+
+  create_uuid_state(&state);
+  create_token(&state, &uuid);
+  format_token(output, &uuid);
+}
+
 //////////////////////////////////////////////////
 //   UTF-8 functions
 //////////////////////////////////////////////////
@@ -186,8 +199,10 @@ const char* strForceUTF8(const char * str)
 }
 
 
+#ifdef ICAL_DEBUG
 int gEventCount = 0;
 int gEventDisplayCount = 0;
+#endif
 
 //////////////////////////////////////////////////
 //   ICalEvent Factory
@@ -325,6 +340,7 @@ NS_IMETHODIMP oeICalEventImpl::GetParent( oeIICal **calendar )
     printf( "oeICalEventImpl::GetParent()\n" );
 #endif
     *calendar = m_calendar;
+    NS_ADDREF( *calendar );
     return NS_OK;
 }
 
@@ -564,7 +580,7 @@ NS_IMETHODIMP oeICalEventImpl::GetMethod(eventMethodProperty *aRetVal)
 NS_IMETHODIMP oeICalEventImpl::SetMethod(eventMethodProperty aNewVal)
 {
 #ifdef ICAL_DEBUG_ALL
-    printf( "SetMethod( %s )\n", aNewVal );
+    printf( "SetMethod( %d )\n", aNewVal );
 #endif
     m_method= aNewVal;
     return NS_OK;
@@ -691,7 +707,7 @@ NS_IMETHODIMP oeICalEventImpl::GetAlarmLength(PRUint32 *aRetVal)
 NS_IMETHODIMP oeICalEventImpl::SetAlarmLength(PRUint32 aNewVal)
 {
 #ifdef ICAL_DEBUG_ALL
-    printf( "SetAlarmLength(%lu)\n", aNewVal );
+    printf( "SetAlarmLength(%ul)\n", aNewVal );
 #endif
     m_alarmlength = aNewVal;
     return NS_OK;
@@ -715,7 +731,7 @@ NS_IMETHODIMP oeICalEventImpl::GetAlarmEmailAddress(char * *aRetVal)
 NS_IMETHODIMP oeICalEventImpl::SetAlarmEmailAddress(const char * aNewVal)
 {
 #ifdef ICAL_DEBUG_ALL
-    printf( "SetAlarmEmailAddres()\n" );
+    printf( "SetAlarmEmailAddres( %s )\n", aNewVal );
 #endif
     if( m_alarmemail )
         nsMemory::Free( m_alarmemail );
@@ -810,7 +826,7 @@ NS_IMETHODIMP oeICalEventImpl::GetRecurUnits(char **aRetVal)
 NS_IMETHODIMP oeICalEventImpl::SetRecurUnits(const char * aNewVal)
 {
 #ifdef ICAL_DEBUG_ALL
-    printf( "SetRecurUnits()\n" );
+    printf( "SetRecurUnits( %s )\n", aNewVal );
 #endif
     if( m_recurunits )
         nsMemory::Free( m_recurunits );
@@ -972,8 +988,13 @@ icaltimetype oeICalEventImpl::GetNextRecurrence( icaltimetype begin, bool *isbeg
         *isbeginning = true;
     }
 
-    if( icaltime_compare( m_start->m_datetime , begin ) > 0 )
-        return m_start->m_datetime;
+    if( icaltime_compare( m_start->m_datetime , begin ) > 0 ) {
+        if( !m_recur )
+            return m_start->m_datetime;
+        PRTime nextinms = ConvertToPrtime( m_start->m_datetime );
+        if( !IsExcepted( nextinms ) )
+            return m_start->m_datetime;
+    }
 
     //for non recurring events
     if( !m_recur ) {
@@ -1635,6 +1656,7 @@ void oeICalEventImpl::ChopAndAddEventToEnum( struct icaltimetype startdate, nsIS
         }
         eventDisplay->SetDisplayEndDate( enddateinms );
     }
+    NS_RELEASE( eventDisplay );
 }
 
 bool oeICalEventImpl::ParseIcalComponent( icalcomponent *comp )
@@ -1925,13 +1947,30 @@ bool oeICalEventImpl::ParseIcalComponent( icalcomponent *comp )
     prop = icalcomponent_get_first_property( vevent, ICAL_DTSTART_PROPERTY );
     if ( prop != 0) {
         m_start->m_datetime = icalproperty_get_dtstart( prop );
-        m_start->m_datetime.is_utc = false; 
-        if( m_start->m_datetime.is_date == true ) {
+        bool datevalue=m_start->m_datetime.is_date;
+        m_start->m_datetime.is_date = false; //Because currently we depend on m_datetime being a complete datetime value.
+        const char *tzid=nsnull;
+        if( m_start->m_datetime.is_utc && !datevalue )
+            tzid="/Mozilla.org/BasicTimezones/GMT";
+        m_start->m_datetime.is_utc = false;
+        if( datevalue ) {
             m_allday = true;
             m_start->SetHour( 0 );
             m_start->SetMinute( 0 );
             m_start->m_datetime.is_date = false; //Because currently we depend on m_datetime being a complete datetime value.
         }
+        icalparameter *tmppar = icalproperty_get_first_parameter( prop, ICAL_TZID_PARAMETER );
+        if( tmppar )
+            tzid = icalparameter_get_tzid( tmppar );
+        if( tzid ) {
+            if( !datevalue ) {
+                PRTime timeinms;
+                m_start->GetTime( &timeinms );
+                m_start->SetTimeInTimezone( timeinms, tzid );
+            } else {
+                m_start->SetTzID( tzid );
+            }
+        }
     } else {
         m_start->m_datetime = icaltime_null_time();
     }
@@ -1949,6 +1988,28 @@ bool oeICalEventImpl::ParseIcalComponent( icalcomponent *comp )
         prop = icalcomponent_get_first_property( vevent, ICAL_DTEND_PROPERTY );
         if ( prop != 0) {
             m_end->m_datetime = icalproperty_get_dtend( prop );
+            bool datevalue=m_end->m_datetime.is_date;
+            m_end->m_datetime.is_date = false; //Because currently we depend on m_datetime being a complete datetime value.
+            const char *tzid=nsnull;
+            if( m_end->m_datetime.is_utc  && !datevalue )
+                tzid="/Mozilla.org/BasicTimezones/GMT";
+            m_end->m_datetime.is_utc = false;
+            if( datevalue ) {
+                m_end->SetHour( 0 );
+                m_end->SetMinute( 0 );
+            }
+            icalparameter *tmppar = icalproperty_get_first_parameter( prop, ICAL_TZID_PARAMETER );
+            if( tmppar )
+                tzid = icalparameter_get_tzid( tmppar );
+            if( tzid ) {
+                if( !datevalue ) {
+                    PRTime timeinms;
+                    m_end->GetTime( &timeinms );
+                    m_end->SetTimeInTimezone( timeinms, tzid );
+                } else {
+                    m_end->SetTzID( tzid );
+                }
+            }
         } else if( !icaltime_is_null_time( m_start->m_datetime ) ) {
             m_end->m_datetime = m_start->m_datetime;
         } else {
@@ -1979,9 +2040,14 @@ bool oeICalEventImpl::ParseIcalComponent( icalcomponent *comp )
     const char *propvalue;
     const char *x_name;
 
-    prop = icalcomponent_get_first_property( vevent, ICAL_X_PROPERTY );
-    while ( prop ) {
+    for( prop = icalcomponent_get_first_property( vevent, ICAL_X_PROPERTY );
+        prop != 0;
+        prop = icalcomponent_get_next_property( vevent, ICAL_X_PROPERTY ) ) {
+
         x_name = icalproperty_get_x_name ( prop );
+        if( !x_name )
+            continue;
+
         propvalue = icalproperty_get_value_as_string( prop );
         //lastalarmack
         if ( strcmp( x_name, XPROP_LASTALARMACK ) == 0) {
@@ -2007,11 +2073,8 @@ bool oeICalEventImpl::ParseIcalComponent( icalcomponent *comp )
         else if ( strcmp( x_name, XPROP_RECURINTERVAL ) == 0) {
             m_recurinterval= atol( propvalue );
         }
-
-        prop = icalcomponent_get_next_property( vevent, ICAL_X_PROPERTY );
     }
 
-
     //recurend & recurforever & recur & recurweekday & recurweeknumber
     m_recur = false;
     m_recurforever = true;
@@ -2409,27 +2472,89 @@ icalcomponent* oeICalEventImpl::AsIcalComponent()
     }
 
     //startdate
+    char *starttzid=nsnull;
     if( m_start && !icaltime_is_null_time( m_start->m_datetime ) ) {
+        m_start->GetTzID( &starttzid );
         if( m_allday ) {
             m_start->SetHour( 0 );
             m_start->SetMinute( 0 );
             m_start->m_datetime.is_date = true; //This will reflect the event being an all-day event
+            prop = icalproperty_new_dtstart( m_start->m_datetime );
+            if( starttzid ) {
+                icaltimezone *timezone = icaltimezone_get_builtin_timezone_from_tzid (starttzid);
+                icalparameter *tmppar = icalparameter_new_tzid( starttzid );
+                icalproperty_add_parameter( prop, tmppar );
+                icalcomponent_add_component( newcalendar, icalcomponent_new_clone( icaltimezone_get_component ( timezone ) ) );
+            }
+            m_start->m_datetime.is_date = false; //Because currently we depend on m_datetime being a complete datetime value.
+        } else {
+            if( starttzid ) {
+                icaltimezone *timezone = icaltimezone_get_builtin_timezone_from_tzid (starttzid);
+                icaltimetype convertedtime = m_start->m_datetime;
+                icaltimezone_convert_time ( &convertedtime, currenttimezone, timezone );
+                if( strcmp( starttzid, "/Mozilla.org/BasicTimezones/GMT" )==0 ) {
+                    convertedtime.is_utc = true;
+                    prop = icalproperty_new_dtstart( convertedtime );
+                } else {
+                    prop = icalproperty_new_dtstart( convertedtime );
+                    icalparameter *tmppar = icalparameter_new_tzid( starttzid );
+                    icalproperty_add_parameter( prop, tmppar );
+                    icalcomponent_add_component( newcalendar, icalcomponent_new_clone( icaltimezone_get_component ( timezone ) ) );
+                }
+            } else 
+                prop = icalproperty_new_dtstart( m_start->m_datetime );
         }
-        prop = icalproperty_new_dtstart( m_start->m_datetime );
         icalcomponent_add_property( vevent, prop );
-        m_start->m_datetime.is_date = false; //Because currently we depend on m_datetime being a complete datetime value.
     }
 
+    //enddate
     if( m_end && !icaltime_is_null_time( m_end->m_datetime ) ) {
-        //enddate
+        char *tzid=nsnull;
+        m_end->GetTzID( &tzid );
         if( m_allday ) {
-            m_end->SetHour( 23 );
-            m_end->SetMinute( 59 );
+            if( m_end->CompareDate( m_start )==0 ) {
+                m_end->m_datetime = m_start->m_datetime;
+                icaltime_adjust( &(m_end->m_datetime), 1, 0, 0, 0 );
+            } else {
+                m_end->SetHour( 0 );
+                m_end->SetMinute( 0 );
+            }
+            m_end->m_datetime.is_date = true;
+            prop = icalproperty_new_dtend( m_end->m_datetime );
+            if( tzid ) {
+                icaltimezone *timezone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
+                icalparameter *tmppar = icalparameter_new_tzid( tzid );
+                icalproperty_add_parameter( prop, tmppar );
+                if( !starttzid || strcmp( starttzid, tzid ) != 0 ) 
+                    icalcomponent_add_component( newcalendar, icalcomponent_new_clone( icaltimezone_get_component ( timezone ) ) );
+                nsMemory::Free( tzid );
+            }
+            m_end->m_datetime.is_date = false; //Because currently we depend on m_datetime being a complete datetime value.
+        } else {
+            if( tzid ) {
+                icaltimezone *timezone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
+                icaltimetype convertedtime = m_end->m_datetime;
+                icaltimezone_convert_time ( &convertedtime, currenttimezone, timezone );
+                if( strcmp( tzid, "/Mozilla.org/BasicTimezones/GMT" )==0 ) {
+                    convertedtime.is_utc = true;
+                    prop = icalproperty_new_dtend( convertedtime );
+                } else {
+                    prop = icalproperty_new_dtend( convertedtime );
+                    icalparameter *tmppar = icalparameter_new_tzid( tzid );
+                    icalproperty_add_parameter( prop, tmppar );
+                    if( !starttzid || strcmp( starttzid, tzid ) != 0 ) 
+                        icalcomponent_add_component( newcalendar, icalcomponent_new_clone( icaltimezone_get_component ( timezone ) ) );
+                }
+                nsMemory::Free( tzid );
+            } else 
+                prop = icalproperty_new_dtend( m_end->m_datetime );
         }
-        prop = icalproperty_new_dtend( m_end->m_datetime );
         icalcomponent_add_property( vevent, prop );
     }
 
+    if( starttzid )
+        nsMemory::Free( starttzid );
+
     if( m_stamp && !icaltime_is_null_time( m_stamp->m_datetime ) ) {
         //stampdate
         prop = icalproperty_new_dtstamp( m_stamp->m_datetime );
@@ -2438,11 +2563,11 @@ icalcomponent* oeICalEventImpl::AsIcalComponent()
 
     //snoozetimes
     icalcomponent *tmpcomp=NULL;
-    int i;
-    for( i=0; i<m_snoozetimes.Count(); i++ ) {
+    int j;
+    for( j=0; j<m_snoozetimes.Count(); j++ ) {
         if( tmpcomp == NULL )
             tmpcomp = icalcomponent_new( ICAL_X_COMPONENT );
-        icaltimetype snoozetime = ConvertFromPrtime( *(PRTime *)(m_snoozetimes[i]) );
+        icaltimetype snoozetime = ConvertFromPrtime( *(PRTime *)(m_snoozetimes[j]) );
         prop = icalproperty_new_dtstamp( snoozetime );
         icalcomponent_add_property( tmpcomp, prop );
     }
@@ -2452,6 +2577,7 @@ icalcomponent* oeICalEventImpl::AsIcalComponent()
     PRUint32 attachmentCount = 0;
     m_attachments->Count(&attachmentCount);
     nsCOMPtr<nsIMsgAttachment> element;
+    unsigned int i;
     for (i = 0; i < attachmentCount; i ++) {
         m_attachments->QueryElementAt(i, NS_GET_IID(nsIMsgAttachment), getter_AddRefs(element));
         if (element)
@@ -2534,7 +2660,6 @@ oeICalEventDisplayImpl::oeICalEventDisplayImpl( oeIICalEvent* event )
     if( event == nsnull ) {
         mEvent = do_CreateInstance(OE_ICALEVENTDISPLAY_CONTRACTID, &rv);
     } else {
-        NS_ADDREF( event );
         mEvent = event;
     }
 
