@@ -25,6 +25,7 @@
  *                 ArentJan Banck <ajbanck@planet.nl>
  *                 Curtis Jewell <csjewell@mail.freeshell.org>
  *                 Eric Belhaire <eric.belhaire@ief.u-psud.fr>
+ *			          Mark Swaffer <swaff@fudo.org>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -171,6 +172,8 @@ function modifyToDoCommand( event )
    
    if( ThisToDo )
       editToDo( ThisToDo );
+   else
+     newToDoCommand();
 }
 
 /**
@@ -256,7 +259,7 @@ function checkboxClick( ThisToDo, completed )
 
 /*
 This function return the progress state of a ToDo task :
-completed, overdue, duetoday, inprogress
+completed, overdue, duetoday, inprogress, future
  */
 function ToDoProgressAtom( calendarToDo )
 {
@@ -284,7 +287,12 @@ function ToDoProgressAtom( calendarToDo )
                   dueDate.getDate() == now.getDate() )
       {
 	    return("duetoday");
+      } else
+	if( tonightMidnight.getTime() < startDate.getTime() )
+	{
+	    return("future");
       }
+
       else
       {
 	    return("inprogress");
@@ -317,7 +325,7 @@ var toDoTreeView =
       props.AppendElement(aserv.getAtom(ToDoProgressAtom(calendarToDo)));
    },
    getColumnProperties : function(){return false;},
-   // By getCellProperties, the properties defined with 
+   // By getRowProperties, the properties defined with 
    // treechildren:-moz-tree-row in CSS are used.
    // It is used here to color the background of a selected
    // ToDo task with a color
@@ -605,10 +613,20 @@ function changeToolTipTextForToDo( event )
    
       if (toDoItem.description)
       {
-         var DescriptionHtml = document.createElement( "description" );
-         var DescriptionText = document.createTextNode( "Description: "+toDoItem.description );
-         DescriptionHtml.appendChild( DescriptionText );
-         HolderBox.appendChild( DescriptionHtml );
+	var text = "Description: "+toDoItem.description ;
+	var lines = text.split("\n");
+	var nbmaxlines = 5 ;
+	var nblines = lines.length ;
+	if( nblines > nbmaxlines ) {
+	  var nblines = nbmaxlines ;
+	  lines[ nblines - 1 ] = "..." ;
+	}
+	for (var i = 0; i < nblines; i++) {
+	  var DescriptionHtml = document.createElement("description");
+	  var DescriptionText = document.createTextNode(lines[i]);
+	  DescriptionHtml.appendChild(DescriptionText);
+	  HolderBox.appendChild(DescriptionHtml);
+	}
       }
       
       Html.appendChild( HolderBox );
