@@ -442,6 +442,7 @@ WeekView.prototype.createEventBox = function ( calendarEventDisplay, dayIndex )
    
    eventBox.setAttribute( "class", "week-view-event-class" );
    eventBox.setAttribute( "eventbox", "weekview" );
+   eventBox.setAttribute( "dayindex", index+1 );
    eventBox.setAttribute( "onclick", "weekEventItemClick( this, event )" );
    eventBox.setAttribute( "ondblclick", "weekEventItemDoubleClick( this, event )" );
    eventBox.setAttribute( "ondraggesture", "nsDragAndDrop.startDrag(event,calendarViewDNDObserver);" );
@@ -567,15 +568,19 @@ WeekView.prototype.refreshDisplay = function( )
    var firstDayOfWeek = new Date( viewYear, viewMonth, dateOfFirstDayInWeek );
    var lastDayOfWeek = new Date( viewYear, viewMonth, dateOfLastDayInWeek );
     
-   var firstDayMonthName = this.calendarWindow.dateFormater.getMonthName( firstDayOfWeek.getMonth() );
-   var lastDayMonthName =  this.calendarWindow.dateFormater.getMonthName( lastDayOfWeek.getMonth() );
+   var firstDayMonthName = this.calendarWindow.dateFormater.getFormatedDateWithoutYear( firstDayOfWeek );
+   var lastDayMonthName =  this.calendarWindow.dateFormater.getFormatedDateWithoutYear( lastDayOfWeek );
    
-   var weekNumber = this.getWeekNumber();
+   var newoffset = (Offset >= 5) ? 8 -Offset : 1 - Offset ;
+   var mondayDate = new Date( firstDayOfWeek.getFullYear(), 
+			      firstDayOfWeek.getMonth(),
+			      firstDayOfWeek.getDate()+newoffset );
+   
+   var weekNumber = DateUtils.getWeekNumber(mondayDate);
 
    var weekViewStringBundle = srGetStrBundle("chrome://calendar/locale/calendar.properties");
    
-   var dateString = weekViewStringBundle.GetStringFromName( "Week" )+" "+weekNumber+ ": "+firstDayMonthName + " " + firstDayOfWeek.getDate() + " - " +
-                    lastDayMonthName  + " " + lastDayOfWeek.getDate();
+   var dateString = weekViewStringBundle.GetStringFromName( "Week" )+" "+weekNumber+ ": "+firstDayMonthName + " - " + lastDayMonthName ;
    
    var weekTextItem = document.getElementById( "week-title-text" );
    weekTextItem.setAttribute( "value" , dateString ); 
@@ -689,6 +694,50 @@ WeekView.prototype.getVisibleEvent = function( calendarEvent )
 }
 
 
+/** PRIVATE
+*
+*   Mark the selected date, also unmark the old selection if there was one
+*/
+
+WeekView.prototype.hiliteSelectedDate = function multiweekView_hiliteSelectedDate( )
+{
+   // Clear the old selection if there was one
+
+   this.clearSelectedDate();
+
+   this.clearSelectedEvent();
+
+   // get the events for the day and loop through them
+   var FirstDay = new Date( gHeaderDateItemArray[1].getAttribute( "date" ) );
+   var LastDay  = new Date( gHeaderDateItemArray[7].getAttribute( "date" ) );
+   LastDay.setHours( 23 );
+   LastDay.setMinutes( 59 );
+   LastDay.setSeconds( 59 );
+   var selectedDay = this.calendarWindow.getSelectedDate() ;
+
+   if ( selectedDay.getTime() > FirstDay.getTime() && selectedDay.getTime() < LastDay.getTime() ) 
+   {
+      var ThisDate;
+      //today is visible, get the day index for today
+      for ( var dayIndex = 1; dayIndex <= 7; ++dayIndex )
+      {
+         ThisDate = new Date( gHeaderDateItemArray[dayIndex].getAttribute( "date" ) );
+         if ( ThisDate.getFullYear() == selectedDay.getFullYear()
+                && ThisDate.getMonth() == selectedDay.getMonth()
+                && ThisDate.getDay()   == selectedDay.getDay() ) 
+         {
+            break;
+         }
+      }
+      //dayIndex now contains the box numbers that we need.
+      for ( i = 0; i < 24; i++ ) 
+      {
+         document.getElementById( "week-tree-day-"+(dayIndex-1)+"-item-"+i ).setAttribute( "weekselected", "true" );
+      }
+
+   }
+}
+
 /** PRIVATE
 *
 *  Mark today as selected, also unmark the old today if there was one.
@@ -778,31 +827,10 @@ WeekView.prototype.getNewEventDate = function( )
 
 WeekView.prototype.clearSelectedDate = function( )
 {
-   if ( this.selectedBox ) 
+   var SelectedBoxes = document.getElementsByAttribute( "weekselected", "true" );
+   
+   for( var i = 0; i < SelectedBoxes.length; i++ )
    {
-      this.selectedBox.removeAttribute( "eventselected" );
-      this.selectedBox = null;
+      SelectedBoxes[i].removeAttribute( "weekselected" );
    }
 }
-
-
-WeekView.prototype.getWeekNumber = function()
-{
-	var today = new Date( this.calendarWindow.getSelectedDate() );
-	var Year = today.getYear() + 1900;
-	var Month = today.getMonth();
-	var Day = today.getDate();
-   var now = new Date(Year,Month,Day+1);
-	now = now.getTime();
-   var Firstday = new Date( this.calendarWindow.getSelectedDate() );
-	Firstday.setYear(Year);
-	Firstday.setMonth(0);
-	Firstday.setDate(1);
-	var then = new Date(Year,0,1);
-   then = then.getTime();
-	var Compensation = Firstday.getDay();
-	if (Compensation > 3) Compensation -= 4;
-	else Compensation += 3;
-	NumberOfWeek =  Math.round((((now-then)/86400000)+Compensation)/7);
-	return NumberOfWeek;
-}
