@@ -41,14 +41,12 @@
 #include "nsISupports.h"
 #include "nsEvent.h"
 #include "nsAString.h"
-#include "nsString.h"
 #include "nsChangeHint.h"
 #include "nsCOMArray.h"
 
 class nsIAtom;
 class nsIArena;
 class nsIContent;
-class nsIDocumentContainer;
 class nsIDocumentObserver;
 class nsIPresContext;
 class nsIPresShell;
@@ -79,7 +77,6 @@ class nsIBindingManager;
 class nsIObserver;
 class nsISupportsArray;
 class nsIScriptLoader;
-class nsString;
 class nsIContentSink;
 class nsIScriptEventManager;
 
@@ -94,6 +91,11 @@ class nsIScriptEventManager;
 // object it creates
 #define NS_CONTENT_ID_COUNTER_BASE 10000
 
+
+// Flag for AddStyleSheet().
+#define NS_STYLESHEET_FROM_CATALOG                (1 << 0)
+
+
 //----------------------------------------------------------------------
 
 // Document interface
@@ -119,7 +121,7 @@ public:
   /**
    * Return the title of the document. May return null.
    */
-  virtual const nsString* GetDocumentTitle() const = 0;
+  NS_IMETHOD GetDocumentTitle(nsAString& aTitle) const = 0;
 
   /**
    * Return the URL for the document. May return null.
@@ -144,7 +146,8 @@ public:
   NS_IMETHOD GetDocumentLoadGroup(nsILoadGroup** aGroup) const = 0;
 
   /**
-   * Return the base URL for relative URLs in the document. May return null (or the document URL).
+   * Return the base URL for relative URLs in the document. May return
+   * null (or the document URL).
    */
   NS_IMETHOD GetBaseURL(nsIURI*& aURL) const = 0;
   NS_IMETHOD SetBaseURL(nsIURI* aURL) = 0;
@@ -156,8 +159,9 @@ public:
   NS_IMETHOD SetBaseTarget(const nsAString &aBaseTarget)=0;
 
   /**
-   * Return a standard name for the document's character set. This will
-   * trigger a startDocumentLoad if necessary to answer the question.
+   * Return a standard name for the document's character set. This
+   * will trigger a startDocumentLoad if necessary to answer the
+   * question.
    */
   NS_IMETHOD GetDocumentCharacterSet(nsAString& oCharSetID) = 0;
   NS_IMETHOD SetDocumentCharacterSet(const nsAString& aCharSetID) = 0;
@@ -189,8 +193,8 @@ public:
    */
   NS_IMETHOD GetContentLanguage(nsAString& aContentLanguage) const = 0;
 
-  // The state BidiEnabled should persist across multiple views (screen, print)
-  // of the same document.
+  // The state BidiEnabled should persist across multiple views
+  // (screen, print) of the same document.
 
   /**
    * Check if the document contains bidi data.
@@ -215,15 +219,15 @@ public:
   NS_IMETHOD SetWordBreaker(nsIWordBreaker* aWordBreaker) = 0;
 
   /**
-   * Access HTTP header data (this may also get set from other sources, like
-   * HTML META tags).
+   * Access HTTP header data (this may also get set from other
+   * sources, like HTML META tags).
    */
   NS_IMETHOD GetHeaderData(nsIAtom* aHeaderField, nsAString& aData) const = 0;
   NS_IMETHOD SetHeaderData(nsIAtom* aheaderField, const nsAString& aData) = 0;
 
   /**
-   * Create a new presentation shell that will use aContext for
-   * it's presentation context (presentation context's <b>must not</b> be
+   * Create a new presentation shell that will use aContext for it's
+   * presentation context (presentation context's <b>must not</b> be
    * shared among multiple presentation shell's).
    */
   NS_IMETHOD CreateShell(nsIPresContext* aContext,
@@ -432,8 +436,6 @@ public:
                             PRUint32 aFlags,
                             nsEventStatus* aEventStatus) = 0;
 
-  NS_IMETHOD_(PRBool) EventCaptureRegistration(PRInt32 aCapturerIncrement) = 0;
-  
   NS_IMETHOD FlushPendingNotifications(PRBool aFlushReflows=PR_TRUE,
                                        PRBool aUpdateViews=PR_FALSE) = 0;
 
@@ -460,40 +462,47 @@ public:
   NS_IMETHOD GetContainer(nsISupports **aContainer) = 0;
 
   NS_IMETHOD GetScriptEventManager(nsIScriptEventManager **aResult) = 0;
+
+  /**
+   * Set and get XML declaration. Notice that if version is empty,
+   * there can be no XML declaration (it is a required part).
+   */
+  NS_IMETHOD SetXMLDeclaration(const nsAString& aVersion,
+                               const nsAString& aEncoding,
+                               const nsAString& Standalone) = 0;
+  NS_IMETHOD GetXMLDeclaration(nsAString& aVersion,
+                               nsAString& aEncoding,
+                               nsAString& Standalone) = 0;
+
+  NS_IMETHOD_(PRBool) IsCaseSensitive() = 0;
 };
 
 
 // XXX These belong somewhere else
-extern NS_EXPORT nsresult
-   NS_NewHTMLDocument(nsIDocument** aInstancePtrResult);
+nsresult
+NS_NewHTMLDocument(nsIDocument** aInstancePtrResult);
 
-extern NS_EXPORT nsresult
-   NS_NewXMLDocument(nsIDocument** aInstancePtrResult);
+nsresult
+NS_NewXMLDocument(nsIDocument** aInstancePtrResult);
 
 #ifdef MOZ_SVG
-extern NS_EXPORT nsresult
-   NS_NewSVGDocument(nsIDocument** aInstancePtrResult);
+nsresult
+NS_NewSVGDocument(nsIDocument** aInstancePtrResult);
 #endif
 
-extern NS_EXPORT nsresult
-   NS_NewImageDocument(nsIDocument** aInstancePtrResult);
+nsresult
+NS_NewImageDocument(nsIDocument** aInstancePtrResult);
 
-extern NS_EXPORT nsresult
-   NS_NewDocumentFragment(nsIDOMDocumentFragment** aInstancePtrResult,
-                          nsIDocument* aOwnerDocument);
-extern NS_EXPORT nsresult
-   NS_NewDOMDocument(nsIDOMDocument** aInstancePtrResult,
-                     const nsAString& aNamespaceURI, 
-                     const nsAString& aQualifiedName, 
-                     nsIDOMDocumentType* aDoctype,
-                     nsIURI* aBaseURI);
-
-// Note: The buffer passed into NewPostData(...) becomes owned by the IPostData
-//       instance and is freed when the instance is destroyed...
-//
-#if 0
-extern NS_EXPORT nsresult
-   NS_NewPostData(PRBool aIsFile, char *aData, nsIPostData** aInstancePtrResult);
-#endif
+nsresult
+NS_NewDocumentFragment(nsIDOMDocumentFragment** aInstancePtrResult,
+                       nsIDocument* aOwnerDocument);
+nsresult
+NS_NewDOMDocument(nsIDOMDocument** aInstancePtrResult,
+                  const nsAString& aNamespaceURI, 
+                  const nsAString& aQualifiedName, 
+                  nsIDOMDocumentType* aDoctype,
+                  nsIURI* aBaseURI);
+nsresult
+NS_NewPluginDocument(nsIDocument** aInstancePtrResult);
 
 #endif /* nsIDocument_h___ */
