@@ -62,6 +62,9 @@
 #include "plstr.h"
 //#include "nsDependentString.h"
 #include "nsIProperties.h"
+#include "nsUnicharUtils.h"
+#include "nsIURI.h"
+#include "nsNetUtil.h"
 
 //#define DEBUG_BenB
 
@@ -366,9 +369,13 @@ mozSanitizingHTMLSerializer::SetTitle(const nsString& aValue)
 {
   if (IsAllowedTag(eHTMLTag_title))
   {
-    //Write(NS_LITERAL_STRING("<title>"));  -- aValue already contains that
+    // See bug 195020 for a good reason to output the tags.
+    // It will make sure we have a closing tag, and a
+    // missing </title> tag won't result in everything
+    // being eaten up as the title.
+    Write(NS_LITERAL_STRING("<title>"));
     Write(aValue);
-    //Write(NS_LITERAL_STRING("</title>"));
+    Write(NS_LITERAL_STRING("</title>"));
   }
   return NS_OK;
 }
@@ -450,13 +457,6 @@ mozSanitizingHTMLSerializer::CloseFrameset()
   return CloseContainer(eHTMLTag_frameset);
 }
 
-NS_IMETHODIMP 
-mozSanitizingHTMLSerializer::DoFragment(PRBool aFlag)
-{
-  return NS_OK;
-}
-
-
 
 // Here comes the actual code...
 
@@ -633,6 +633,22 @@ mozSanitizingHTMLSerializer::SanitizeAttrValue(nsHTMLTag aTag,
       value.Find("base64") != kNotFound)
     return NS_ERROR_ILLEGAL_VALUE;
 
+  // Check img src scheme
+  if (aTag == eHTMLTag_img && 
+      attr_name.Equals(NS_LITERAL_STRING("src"), nsCaseInsensitiveStringComparator()))
+  {
+    nsresult rv;
+    nsCOMPtr<nsIIOService> ioService;
+    ioService = do_GetIOService(&rv);
+    if (NS_FAILED(rv)) return rv;
+    nsCAutoString scheme;
+    rv = ioService->ExtractScheme(NS_LossyConvertUCS2toASCII(value), scheme);
+    if (NS_FAILED(rv)) return rv;
+
+    if (!scheme.Equals("cid", nsCaseInsensitiveCStringComparator()))
+      return NS_ERROR_ILLEGAL_VALUE;
+  }
+
   return NS_OK;
 }
 
