@@ -38,6 +38,9 @@
 
 #include "jsapi.h"
 #include "nsCOMPtr.h"
+#include "nsAString.h"
+#include "nsPrintfCString.h"
+#include "nsUnicharUtils.h"
 #include "nsIServiceManagerUtils.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIScriptContext.h"
@@ -62,6 +65,14 @@
 #include "nsIParserService.h"
 #include "nsIServiceManager.h"
 #include "nsIAttribute.h"
+#include "nsIContentList.h"
+#include "nsIHTMLDocument.h"
+#include "nsIDOMHTMLDocument.h"
+#include "nsIDOMHTMLCollection.h"
+#include "nsIDOMHTMLFormElement.h"
+#include "nsIForm.h"
+#include "nsIFormControl.h"
+#include "nsHTMLAtoms.h"
 
 static const char *kJSStackContractID = "@mozilla.org/js/xpc/ContextStack;1";
 static NS_DEFINE_IID(kParserServiceCID, NS_PARSERSERVICE_CID);
@@ -510,6 +521,7 @@ nsContentUtils::CheckSameOrigin(nsIDOMNode *aTrustedNode,
 #endif
     nsCOMPtr<nsIDOMDocument> domDoc;
     aTrustedNode->GetOwnerDocument(getter_AddRefs(domDoc));
+
     if (!domDoc) {
       // In theory this should never happen. But since theory and reality are
       // different for XUL elements we'll try to get the principal from the
@@ -530,8 +542,7 @@ nsContentUtils::CheckSameOrigin(nsIDOMNode *aTrustedNode,
 
         return NS_ERROR_UNEXPECTED;
       }
-    }
-    else {
+    } else {
       trustedDoc = do_QueryInterface(domDoc);
       NS_ASSERTION(trustedDoc, "QI to nsIDocument failed");
     }
@@ -560,13 +571,16 @@ nsContentUtils::CheckSameOrigin(nsIDOMNode *aTrustedNode,
    */
 
   // If they are in the same document then everything is just fine
-  if (trustedDoc == unTrustedDoc && trustedDoc)
+  if (trustedDoc == unTrustedDoc && trustedDoc) {
     return NS_OK;
+  }
 
   if (!trustedPrincipal) {
     trustedDoc->GetPrincipal(getter_AddRefs(trustedPrincipal));
+
     if (!trustedPrincipal) {
-      // If the trusted node doesn't have a principal we can't check security against it
+      // If the trusted node doesn't have a principal we can't check
+      // security against it
 
       return NS_ERROR_DOM_SECURITY_ERR;
     }
@@ -772,7 +786,7 @@ nsContentUtils::ReparentContentWrapper(nsIContent *aContent,
       ni->GetDocument(*getter_AddRefs(old_doc));
     }
 
-    if (!aOldDocument) {
+    if (!old_doc) {
       // If we can't find our old document we don't know what our old
       // scope was so there's no way to find the old wrapper
 
@@ -1244,6 +1258,8 @@ IsCharInSet(const char* aSet,
 /**
  * This method strips leading/trailing chars, in given set, from string.
  */
+
+// static
 const nsDependentSubstring
 nsContentUtils::TrimCharsInSet(const char* aSet,
                                const nsAString& aValue)
@@ -1275,6 +1291,244 @@ nsContentUtils::TrimCharsInSet(const char* aSet,
   return Substring(valueCurrent, valueEnd);
 }
 
+/**
+ * This method strips leading and trailing whitespace from a string.
+ */
+
+// static
+const nsDependentSubstring
+nsContentUtils::TrimWhitespace(const nsAString& aStr, PRBool aTrimTrailing)
+{
+  nsAString::const_iterator start, end;
+
+  aStr.BeginReading(start);
+  aStr.EndReading(end);
+
+  // Skip whitespace charaters in the beginning
+  while (start != end && nsString::IsSpace(*start)) {
+    ++start;
+  }
+
+  if (aTrimTrailing) {
+    // Skip whitespace characters in the end.
+    while (end != start) {
+      --end;
+
+      if (!nsString::IsSpace(*end)) {
+        // Step back to the last non-whitespace character.
+        ++end;
+
+        break;
+      }
+    }
+  }
+
+  // Return a substring for the string w/o leading and/or trailing
+  // whitespace
+
+  return Substring(start, end);
+}
+
+static inline void KeyAppendSep(nsACString& aKey)
+{
+  if (!aKey.IsEmpty()) {
+    aKey.Append('>');
+  }
+}
+
+static inline void KeyAppendString(const nsAString& aString, nsACString& aKey)
+{
+  KeyAppendSep(aKey);
+
+  // Could escape separator here if collisions happen.  > is not a legal char
+  // for a name or type attribute, so we should be safe avoiding that extra work.
+
+  aKey.Append(NS_ConvertUCS2toUTF8(aString));
+}
+
+static inline void KeyAppendString(const nsACString& aString, nsACString& aKey)
+{
+  KeyAppendSep(aKey);
+  
+  // Could escape separator here if collisions happen.  > is not a legal char
+  // for a name or type attribute, so we should be safe avoiding that extra work.
+
+  aKey.Append(aString);
+}
+
+static inline void KeyAppendInt(PRInt32 aInt, nsACString& aKey)
+{
+  KeyAppendSep(aKey);
+
+  aKey.Append(nsPrintfCString("%d", aInt));
+}
+
+static inline void KeyAppendAtom(nsIAtom* aAtom, nsACString& aKey)
+{
+  NS_PRECONDITION(aAtom, "KeyAppendAtom: aAtom can not be null!\n");
+
+  const char* atomString = nsnull;
+  aAtom->GetUTF8String(&atomString);
+
+  KeyAppendString(nsDependentCString(atomString), aKey);
+}
+
+static inline PRBool IsAutocompleteOff(nsIDOMElement* aElement)
+{
+  nsAutoString autocomplete;
+  aElement->GetAttribute(NS_LITERAL_STRING("autocomplete"), autocomplete);
+  return autocomplete.Equals(NS_LITERAL_STRING("off"),
+                             nsCaseInsensitiveStringComparator());
+}
+
+/*static*/ nsresult
+nsContentUtils::GenerateStateKey(nsIContent* aContent,
+                                 nsIStatefulFrame::SpecialStateID aID,
+                                 nsACString& aKey)
+{
+  aKey.Truncate();
+
+  // SpecialStateID case - e.g. scrollbars around the content window
+  // The key in this case is the special state id (always < min(contentID))
+  if (nsIStatefulFrame::eNoID != aID) {
+    KeyAppendInt(aID, aKey);
+    return NS_OK;
+  }
+
+  // We must have content if we're not using a special state id
+  NS_ENSURE_TRUE(aContent, NS_ERROR_FAILURE);
+
+  // Don't capture state for anonymous content
+  PRUint32 contentID;
+  aContent->GetContentID(&contentID);
+  if (!contentID) {
+    return NS_OK;
+  }
+
+  nsCOMPtr<nsIDOMElement> element(do_QueryInterface(aContent));
+  if (element && IsAutocompleteOff(element)) {
+    return NS_OK;
+  }
+
+  nsCOMPtr<nsIDocument> doc;
+  aContent->GetDocument(*getter_AddRefs(doc));
+  nsCOMPtr<nsIHTMLDocument> htmlDocument(do_QueryInterface(doc));
+
+  PRBool generatedUniqueKey = PR_FALSE;
+
+  if (htmlDocument) {
+    nsCOMPtr<nsIDOMHTMLDocument> domHtmlDocument(do_QueryInterface(htmlDocument));
+    nsCOMPtr<nsIDOMHTMLCollection> forms;
+    domHtmlDocument->GetForms(getter_AddRefs(forms));
+    nsCOMPtr<nsIContentList> htmlForms(do_QueryInterface(forms));
+
+    nsCOMPtr<nsIDOMNodeList> formControls;
+    htmlDocument->GetFormControlElements(getter_AddRefs(formControls));
+    nsCOMPtr<nsIContentList> htmlFormControls(do_QueryInterface(formControls));
+
+    // If we have a form control and can calculate form information, use
+    // that as the key - it is more reliable than contentID.
+    // Important to have a unique key, and tag/type/name may not be.
+    //
+    // If the control has a form, the format of the key is:
+    // type>IndOfFormInDoc>IndOfControlInForm>FormName>name
+    // else:
+    // type>IndOfControlInDoc>name
+    //
+    // XXX We don't need to use index if name is there
+    //
+    nsCOMPtr<nsIFormControl> control(do_QueryInterface(aContent));
+    if (control && htmlFormControls && htmlForms) {
+
+      // Append the control type
+      KeyAppendInt(control->GetType(), aKey);
+
+      // If in a form, add form name / index of form / index in form
+      PRInt32 index = -1;
+      nsCOMPtr<nsIDOMHTMLFormElement> formElement;
+      control->GetForm(getter_AddRefs(formElement));
+      if (formElement) {
+
+        if (IsAutocompleteOff(formElement)) {
+          aKey.Truncate();
+          return NS_OK;
+        }
+
+        // Append the index of the form in the document
+        nsCOMPtr<nsIContent> formContent(do_QueryInterface(formElement));
+        htmlForms->IndexOf(formContent, index, PR_FALSE);
+        if (index <= -1) {
+          //
+          // XXX HACK this uses some state that was dumped into the document
+          // specifically to fix bug 138892.  What we are trying to do is *guess*
+          // which form this control's state is found in, with the highly likely
+          // guess that the highest form parsed so far is the one.
+          // This code should not be on trunk, only branch.
+          //
+          htmlDocument->GetNumFormsSynchronous(&index);
+          index--;
+        }
+        if (index > -1) {
+          KeyAppendInt(index, aKey);
+
+          // Append the index of the control in the form
+          nsCOMPtr<nsIForm> form(do_QueryInterface(formElement));
+          form->IndexOfControl(control, &index);
+          NS_ASSERTION(index > -1,
+                       "nsContentUtils::GenerateStateKey didn't find form control index!");
+
+          if (index > -1) {
+            KeyAppendInt(index, aKey);
+            generatedUniqueKey = PR_TRUE;
+          }
+        }
+
+        // Append the form name
+        nsAutoString formName;
+        formElement->GetName(formName);
+        KeyAppendString(formName, aKey);
+
+      } else {
+
+        // If not in a form, add index of control in document
+        // Less desirable than indexing by form info. 
+
+        // Hash by index of control in doc (we are not in a form)
+        // These are important as they are unique, and type/name may not be.
+
+        // We don't refresh the form control list here (passing PR_TRUE
+        // for aFlush), although we really should. Forcing a flush
+        // causes a signficant pageload performance hit. See bug
+        // 166636. Doing this wrong means you will see the assertion
+        // below being hit.
+        htmlFormControls->IndexOf(aContent, index, PR_FALSE);
+        NS_ASSERTION(index > -1,
+                     "nsContentUtils::GenerateStateKey didn't find content "
+                     "by type! See bug 139568");
+
+        if (index > -1) {
+          KeyAppendInt(index, aKey);
+          generatedUniqueKey = PR_TRUE;
+        }
+      }
+
+      // Append the control name
+      nsAutoString name;
+      aContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::name, name);
+      KeyAppendString(name, aKey);
+    }
+  }
+
+  if (!generatedUniqueKey) {
+
+    // Either we didn't have a form control or we aren't in an HTML document
+    // so we can't figure out form info, hash by content ID instead :(
+    KeyAppendInt(contentID, aKey);
+  }
+
+  return NS_OK;
+}
+
 void
 nsCxPusher::Push(nsISupports *aCurrentTarget)
 {
