@@ -63,8 +63,7 @@
 #define kGreaterThan NS_LITERAL_STRING(">")
 #define kEndTag NS_LITERAL_STRING("</")
 
-static const PRUnichar kMozStr[] = {'m', 'o', 'z', 0};
-static const PRInt32 kMozStrLength = 3;
+static const char kMozStr[] = "moz";
 
 static const PRInt32 kLongLineLen = 128;
 
@@ -199,14 +198,15 @@ nsHTMLContentSerializer::AppendDocumentStart(nsIDOMDocument *aDocument,
 PRBool
 nsHTMLContentSerializer::IsJavaScript(nsIAtom* aAttrNameAtom, const nsAString& aValueString)
 {
-  if (aAttrNameAtom == nsHTMLAtoms::href
-  || aAttrNameAtom == nsHTMLAtoms::src) {
-    // note that there is a problem in that if this value starts with leading spaces we won't do the right thing
-    // this is covered in bug #59604
+  if (aAttrNameAtom == nsHTMLAtoms::href ||
+      aAttrNameAtom == nsHTMLAtoms::src) {
     static const char kJavaScript[] = "javascript";
     PRInt32 pos = aValueString.FindChar(':');
-    const nsAutoString scheme(Substring(aValueString, 0, pos));
-    if ((pos == (PRInt32)(sizeof kJavaScript - 1)) &&
+    if ( pos < (PRInt32)(sizeof kJavaScript - 1) )
+        return PR_FALSE;
+    nsAutoString scheme(Substring(aValueString, 0, pos));
+    scheme.StripWhitespace();
+    if ((scheme.Length() == (sizeof kJavaScript - 1)) &&
         scheme.EqualsIgnoreCase(kJavaScript))
       return PR_TRUE;
     else
@@ -255,9 +255,7 @@ nsHTMLContentSerializer::EscapeURI(const nsAString& aURI, nsAString& aEscapedURI
 
 
   if (mCharSet && !uri.IsASCII()) {
-    const PRUnichar *charset;
-    mCharSet->GetUnicode(&charset);
-    documentCharset.Adopt(ToNewUTF8String(nsDependentString(charset)));
+    mCharSet->ToUTF8String(documentCharset);
     textToSubURI = do_GetService(NS_ITEXTTOSUBURI_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
   }
@@ -315,6 +313,8 @@ nsHTMLContentSerializer::SerializeAttributes(nsIContent* aContent,
 
   aContent->GetAttrCount(count);
 
+  NS_NAMED_LITERAL_STRING(mozStr1, "_moz");
+  
   for (index = 0; index < count; index++) {
     aContent->GetAttrNameAt(index, 
                             namespaceID,
@@ -322,13 +322,12 @@ nsHTMLContentSerializer::SerializeAttributes(nsIContent* aContent,
                             *getter_AddRefs(attrPrefix));
 
     // Filter out any attribute starting with [-|_]moz
-    const PRUnichar* sharedName;
-    attrName->GetUnicode(&sharedName);
+    const char* sharedName;
+    attrName->GetUTF8String(&sharedName);
     if ((('_' == *sharedName) || ('-' == *sharedName)) &&
-        !nsCRT::strncmp(sharedName+1, kMozStr, kMozStrLength)) {
+        !nsCRT::strncmp(sharedName+1, kMozStr, PRUint32(sizeof(kMozStr)-1))) {
       continue;
     }
-
     aContent->GetAttr(namespaceID, attrName, valueStr);
 
     // 
@@ -337,8 +336,7 @@ nsHTMLContentSerializer::SerializeAttributes(nsIContent* aContent,
     //
     if ((aTagName == nsHTMLAtoms::br) &&
         (attrName.get() == nsHTMLAtoms::type) &&
-        !valueStr.IsEmpty() && ('_' == valueStr[0]) &&
-        !nsCRT::strncmp(valueStr.get()+1, kMozStr, kMozStrLength)) {
+        (mozStr1.Equals(Substring(valueStr, 0, mozStr1.Length())))) {
       continue;
     }
     
@@ -394,7 +392,10 @@ nsHTMLContentSerializer::SerializeAttributes(nsIContent* aContent,
     * then start the attribute from a new line.
     */
 
-    if (mDoFormat && (mColPos >= mMaxColumn || ((mColPos + nameStr.Length() + valueStr.Length() + 4) > mMaxColumn))) {
+    if (mDoFormat
+        && (mColPos >= mMaxColumn
+            || ((PRInt32)(mColPos + nameStr.Length() +
+                          valueStr.Length() + 4) > mMaxColumn))) {
         aStr.Append(mLineBreak);
         mColPos = 0;
     }
@@ -456,9 +457,9 @@ nsHTMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   
   AppendToString(kLessThan, aStr);
 
-  const PRUnichar* sharedName;
-  name->GetUnicode(&sharedName);
-  AppendToString(sharedName, -1, aStr);
+  nsAutoString nameStr;
+  name->ToString(nameStr);
+  AppendToString(nameStr.get(), -1, aStr);
 
   // Need to keep track of OL and LI elements in order to get ordinal number 
   // for the LI.
@@ -553,13 +554,12 @@ nsHTMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
     }
   }
   
-  const PRUnichar* sharedName;
-  name->GetUnicode(&sharedName);
+  nsAutoString nameStr;
+  name->ToString(nameStr);
 
   nsIParserService* parserService = nsContentUtils::GetParserServiceWeakRef();
 
   if (parserService && (name.get() != nsHTMLAtoms::style)) {
-    nsAutoString nameStr(sharedName);
     PRBool isContainer;
     PRInt32 id;
 
@@ -581,7 +581,7 @@ nsHTMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
   EndIndentation(name, hasDirtyAttr, aStr);
 
   AppendToString(kEndTag, aStr);
-  AppendToString(sharedName, -1, aStr);
+  AppendToString(nameStr.get(), -1, aStr);
   AppendToString(kGreaterThan, aStr);
 
   if (LineBreakAfterClose(name, hasDirtyAttr)) {
@@ -659,6 +659,7 @@ nsHTMLContentSerializer::AppendToStringWrapped(const nsASingleFragmentString& aS
       if (pos == end) {
         return;
       }
+      lastSpace = pos;
     }
     segStart = pos;
     lastChar = pos;
