@@ -711,7 +711,7 @@ nsPlainTextSerializer::DoOpenContainer(const nsIParserNode* aNode, PRInt32 aTag)
     // We must make sure that the content of two table cells get a
     // space between them.
 
-    // To make the seperation between cells most obvious and
+    // To make the separation between cells most obvious and
     // importable, we use a TAB.
     if (GetLastBool(mHasWrittenCellsForRow)) {
       // Bypass |Write| so that the TAB isn't compressed away.
@@ -1807,8 +1807,8 @@ nsPlainTextSerializer::GetAttributeValue(const nsIParserNode* aNode,
     }
   }
   else if (aNode) {
-    const PRUnichar *name; 
-    aName->GetUnicode(&name);
+    nsAutoString name; 
+    aName->ToString(name);
 
     PRInt32 count = aNode->GetAttributeCount();
     for (PRInt32 i=0;i<count;i++) {
