@@ -971,7 +971,7 @@ nsresult nsRange::GetStartContainer(nsIDOMNode** aStartParent)
     return NS_ERROR_NOT_INITIALIZED;
   if (!aStartParent)
     return NS_ERROR_NULL_POINTER;
-  //NS_IF_RELEASE(*aStartParent); don't think we should be doing this
+
   *aStartParent = mStartParent;
   NS_IF_ADDREF(*aStartParent);
   return NS_OK;
@@ -2067,18 +2067,20 @@ nsresult nsRange::InsertNode(nsIDOMNode* aN)
   nsCOMPtr<nsIDOMNode> tStartContainer;
   res = this->GetStartContainer(getter_AddRefs(tStartContainer));
   if(NS_FAILED(res)) return res;
-  
-  PRUint16 tNodeType;
-  aN->GetNodeType(&tNodeType);
-  if( (nsIDOMNode::CDATA_SECTION_NODE == tNodeType) ||
-      (nsIDOMNode::TEXT_NODE == tNodeType) )
+
+  nsCOMPtr<nsIDOMText> startTextNode(do_QueryInterface(tStartContainer));
+  if (startTextNode)
   {
     nsCOMPtr<nsIDOMNode> tSCParentNode;
     res = tStartContainer->GetParentNode(getter_AddRefs(tSCParentNode));
     if(NS_FAILED(res)) return res;
 
+    nsCOMPtr<nsIDOMText> secondPart;
+    res = startTextNode->SplitText(tStartOffset, getter_AddRefs(secondPart));
+    if (NS_FAILED(res)) return res;
+    
     nsCOMPtr<nsIDOMNode> tResultNode;
-    return tSCParentNode->InsertBefore(aN, tSCParentNode, getter_AddRefs(tResultNode));
+    return tSCParentNode->InsertBefore(aN, secondPart, getter_AddRefs(tResultNode));
   }  
 
   nsCOMPtr<nsIDOMNodeList>tChildList;
@@ -2089,20 +2091,12 @@ nsresult nsRange::InsertNode(nsIDOMNode* aN)
   if(NS_FAILED(res)) return res;
 
   // find the insertion point in the DOM and insert the Node
-  if(tStartOffset == (PRInt32)tChildListLength)
-  {
-    nsCOMPtr<nsIDOMNode> tNode;
-    return tStartContainer->AppendChild(aN, getter_AddRefs(tNode));
-  }
-  else
-  {
-    nsCOMPtr<nsIDOMNode>tChildNode;
-    res = tChildList->Item(tStartOffset, getter_AddRefs(tChildNode));
-    if(NS_FAILED(res)) return res;
-
-    nsCOMPtr<nsIDOMNode> tResultNode;
-    return tStartContainer->InsertBefore(aN, tChildNode, getter_AddRefs(tResultNode));
-  }
+  nsCOMPtr<nsIDOMNode>tChildNode;
+  res = tChildList->Item(tStartOffset, getter_AddRefs(tChildNode));
+  if(NS_FAILED(res)) return res;
+  
+  nsCOMPtr<nsIDOMNode> tResultNode;
+  return tStartContainer->InsertBefore(aN, tChildNode, getter_AddRefs(tResultNode));
 }
 
 nsresult nsRange::SurroundContents(nsIDOMNode* aN)
@@ -2208,7 +2202,7 @@ nsresult nsRange::SurroundContents(nsIDOMNode* aN)
     if(tFound)
     {
       nsCOMPtr<nsIDOMNode> tChild;
-      tChildList->Item(i-1, getter_AddRefs(tChild));
+      tChildList->Item(i, getter_AddRefs(tChild));
       tAncestorContainer->InsertBefore(aN, tChild, getter_AddRefs(tempNode));
     }
     else // there is an error this may need to be updated later
@@ -2532,15 +2526,12 @@ nsRange::CreateContextualFragment(const nsAString& aFragment,
   nsCOMPtr<nsIParser> parser = do_CreateInstance(kCParserCID, &result);
   NS_ENSURE_SUCCESS(result, result);
 
-  nsCOMPtr<nsIContent> content = do_QueryInterface(mStartParent, &result);
-  NS_ENSURE_SUCCESS(result, result);
-
   nsCOMPtr<nsIDocument> document;
   nsCOMPtr<nsIDOMDocument> domDocument;
 
-  result = content->GetDocument(*getter_AddRefs(document));
-  if (document && NS_SUCCEEDED(result)) {
-    domDocument = do_QueryInterface(document, &result);
+  result = mStartParent->GetOwnerDocument(getter_AddRefs(domDocument));
+  if (domDocument && NS_SUCCEEDED(result)) {
+    document = do_QueryInterface(domDocument, &result);
   }
 
   nsVoidArray tagStack;
@@ -2577,6 +2568,7 @@ nsRange::CreateContextualFragment(const nsAString& aFragment,
 
     result = NS_NewHTMLFragmentContentSink(getter_AddRefs(sink));
     if (NS_SUCCEEDED(result)) {
+      sink->SetTargetDocument(document);
       parser->SetContentSink(sink);
       nsCOMPtr<nsIDOMNSDocument> domnsDocument(do_QueryInterface(document));
       if (domnsDocument) {
@@ -2631,9 +2623,7 @@ nsRange::CreateContextualFragment(const nsAString& aFragment,
       }
 
       nsDTDMode mode = eDTDMode_autodetect;
-      nsCOMPtr<nsIDOMDocument> ownerDoc;
-      mStartParent->GetOwnerDocument(getter_AddRefs(ownerDoc));
-      nsCOMPtr<nsIHTMLDocument> htmlDoc(do_QueryInterface(ownerDoc));
+      nsCOMPtr<nsIHTMLDocument> htmlDoc(do_QueryInterface(domDocument));
       if (htmlDoc) {
         nsCompatibility compatMode;
         htmlDoc->GetCompatibilityMode(compatMode);
