@@ -62,6 +62,7 @@
 #include "nsContentUtils.h"
 #include "nsIDOMKeyEvent.h"
 #include "nsIDOMMutationEvent.h"
+#include "nsIURI.h"
 
 static const char* const mEventNames[] = {
   "mousedown", "mouseup", "click", "dblclick", "mouseover",
@@ -73,7 +74,8 @@ static const char* const mEventNames[] = {
   "scroll","overflow", "underflow", "overflowchanged",
   "DOMSubtreeModified", "DOMNodeInserted", "DOMNodeRemoved", 
   "DOMNodeRemovedFromDocument", "DOMNodeInsertedIntoDocument",
-  "DOMAttrModified", "DOMCharacterDataModified"
+  "DOMAttrModified", "DOMCharacterDataModified",
+  "popupBlocked"
 }; 
 
 /** event pool used as a simple recycler for objects of this class */
@@ -152,8 +154,6 @@ nsDOMEvent::nsDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
   mPresContext = aPresContext;
   mEventIsTrusted = PR_FALSE;
 
-  NS_IF_ADDREF(mPresContext);
-
   if (aEvent) {
     mEvent = aEvent;
     mEventIsTrusted = PR_TRUE;
@@ -187,19 +187,19 @@ nsDOMEvent::nsDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
       mEvent = PR_NEWZAP(nsGUIEvent);
       mEvent->eventStructType = NS_POPUP_EVENT;
     }
+    else if (eventType.EqualsIgnoreCase("PopupBlockedEvents")) {
+      mEvent = PR_NEWZAP(nsPopupBlockedEvent);
+      mEvent->eventStructType = NS_POPUPBLOCKED_EVENT;
+    }
     else {
       mEvent = PR_NEWZAP(nsEvent);
       mEvent->eventStructType = NS_EVENT;
     }
   }
 
-  mTarget = nsnull;
-  mCurrentTarget = nsnull;
-  mOriginalTarget = nsnull;
   // Get the explicit original target (if it's anonymous make it null)
   {
-    GetTargetFromFrame(getter_AddRefs(mExplicitOriginalTarget));
-    mTmp13RealOriginalTarget = mExplicitOriginalTarget;
+    mExplicitOriginalTarget = GetTargetFromFrame();
     nsCOMPtr<nsIContent> content = do_QueryInterface(mExplicitOriginalTarget);
     if (content) {
       if (content->IsNativeAnonymous()) {
@@ -213,7 +213,6 @@ nsDOMEvent::nsDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
     }
   }
   mText = nsnull;
-  mTextRange = nsnull;
   mButton = -1;
   if (aEvent) {
     mScreenPoint.x = aEvent->refPoint.x;
@@ -264,8 +263,6 @@ nsDOMEvent::nsDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
 	  // call GetInputRange and AddRef to the result
 
     mTextRange = new nsPrivateTextRangeList(te->rangeCount ,tempTextRangeList);
-
-		NS_IF_ADDREF(mTextRange);
   }
 }
 
@@ -273,22 +270,15 @@ nsDOMEvent::~nsDOMEvent()
 {
   NS_ASSERT_OWNINGTHREAD(nsDOMEvent);
 
-  nsCOMPtr<nsIPresShell> shell;
-  if (mPresContext)
-  { // we were arena-allocated, prepare to recycle myself    
-    // XXX this does not appear to be needed anymore
-    mPresContext->GetShell(getter_AddRefs(shell));
-  }
-  NS_IF_RELEASE(mPresContext);
-  NS_IF_RELEASE(mTarget);
-  NS_IF_RELEASE(mCurrentTarget);
-  NS_IF_RELEASE(mOriginalTarget);
-  NS_IF_RELEASE(mTextRange);
-
   if (mEventIsInternal) {
     if (mEvent->userType) {
       delete mEvent->userType;
     }
+    if (mEvent->eventStructType == NS_POPUPBLOCKED_EVENT) {
+      nsPopupBlockedEvent* event = NS_STATIC_CAST(nsPopupBlockedEvent*, mEvent);
+      NS_IF_RELEASE(event->mRequestingWindowURI);
+      NS_IF_RELEASE(event->mPopupWindowURI);
+    }
     PR_DELETE(mEvent);
   }
 
@@ -305,6 +295,7 @@ NS_INTERFACE_MAP_BEGIN(nsDOMEvent)
   NS_INTERFACE_MAP_ENTRY(nsIDOMKeyEvent)
   NS_INTERFACE_MAP_ENTRY(nsIDOMNSUIEvent)
   NS_INTERFACE_MAP_ENTRY(nsIDOMNSEvent)
+  NS_INTERFACE_MAP_ENTRY(nsIDOMPopupBlockedEvent)
   NS_INTERFACE_MAP_ENTRY(nsIPrivateDOMEvent)
   NS_INTERFACE_MAP_ENTRY(nsIPrivateTextEvent)
   NS_INTERFACE_MAP_ENTRY(nsIPrivateCompositionEvent)
@@ -335,7 +326,7 @@ NS_METHOD nsDOMEvent::GetTarget(nsIDOMEventTarget** aTarget)
 {
   if (nsnull != mTarget) {
     *aTarget = mTarget;
-    NS_ADDREF(mTarget);
+    NS_ADDREF(*aTarget);
     return NS_OK;
   }
   
@@ -349,7 +340,7 @@ NS_METHOD nsDOMEvent::GetTarget(nsIDOMEventTarget** aTarget)
   }
   
   if (targetContent) {
-    CallQueryInterface(targetContent, &mTarget);
+    mTarget = do_QueryInterface(targetContent);
     if (mTarget) {
       *aTarget = mTarget;
       NS_ADDREF(*aTarget);
@@ -361,7 +352,7 @@ NS_METHOD nsDOMEvent::GetTarget(nsIDOMEventTarget** aTarget)
     nsCOMPtr<nsIPresShell> presShell;
     if (mPresContext && NS_SUCCEEDED(mPresContext->GetShell(getter_AddRefs(presShell))) && presShell) {
       if (NS_SUCCEEDED(presShell->GetDocument(getter_AddRefs(doc))) && doc) {
-        CallQueryInterface(doc, &mTarget);
+        mTarget = do_QueryInterface(doc);
         if (mTarget) {
           *aTarget = mTarget;
           NS_ADDREF(*aTarget);
@@ -384,12 +375,10 @@ nsDOMEvent::GetCurrentTarget(nsIDOMEventTarget** aCurrentTarget)
 //
 // Get the actual event target node (may have been retargeted for mouse events)
 //
-void
-nsDOMEvent::GetTargetFromFrame(nsIDOMEventTarget** aTarget)
+already_AddRefed<nsIDOMEventTarget>
+nsDOMEvent::GetTargetFromFrame()
 {
-  *aTarget = nsnull;
-
-  if (!mPresContext) { return; }
+  if (!mPresContext) { return nsnull; }
 
   // Get the target frame (have to get the ESM first)
   nsCOMPtr<nsIEventStateManager> esm;
@@ -397,15 +386,17 @@ nsDOMEvent::GetTargetFromFrame(nsIDOMEventTarget** aTarget)
 
   nsIFrame* targetFrame = nsnull;
   esm->GetEventTarget(&targetFrame);
-  if (!targetFrame) { return; }
+  if (!targetFrame) { return nsnull; }
 
   // get the real content
   nsCOMPtr<nsIContent> realEventContent;
   targetFrame->GetContentForEvent(mPresContext, mEvent, getter_AddRefs(realEventContent));
-  if (!realEventContent) { return; }
+  if (!realEventContent) { return nsnull; }
 
   // Finally, we have the real content.  QI it and return.
-  CallQueryInterface(realEventContent, aTarget);
+  nsIDOMEventTarget* target = nsnull;
+  CallQueryInterface(realEventContent, &target);
+  return target;
 }
 
 NS_IMETHODIMP
@@ -420,18 +411,6 @@ nsDOMEvent::GetExplicitOriginalTarget(nsIDOMEventTarget** aRealEventTarget)
   return GetTarget(aRealEventTarget);
 }
 
-NS_IMETHODIMP
-nsDOMEvent::GetTmp13RealOriginalTarget(nsIDOMEventTarget** aRealEventTarget)
-{
-  if (mTmp13RealOriginalTarget) {
-    *aRealEventTarget = mTmp13RealOriginalTarget;
-    NS_ADDREF(*aRealEventTarget);
-    return NS_OK;
-  }
-
-  return GetOriginalTarget(aRealEventTarget);
-}
-
 NS_IMETHODIMP
 nsDOMEvent::GetOriginalTarget(nsIDOMEventTarget** aOriginalTarget)
 {
@@ -620,49 +599,52 @@ nsDOMEvent::GetDetail(PRInt32* aDetail)
 
 NS_METHOD nsDOMEvent::GetText(nsString& aText)
 {
-	if (mEvent->message == NS_TEXT_EVENT) {
-		aText = *mText;
-		return NS_OK;
-	}
-
-	return NS_ERROR_FAILURE;
+  if (mEvent->message == NS_TEXT_EVENT) {
+    aText = *mText;
+    return NS_OK;
+  }
+  aText.Assign(NS_LITERAL_STRING(""));
+  return NS_ERROR_FAILURE;
 }
 
 NS_METHOD nsDOMEvent::GetInputRange(nsIPrivateTextRangeList** aInputRange)
 {
-	if (mEvent->message == NS_TEXT_EVENT) {
-		*aInputRange = mTextRange;
-		return NS_OK;
-	}
-
-	return NS_ERROR_FAILURE;
+  NS_ENSURE_ARG_POINTER(aInputRange);
+  if (mEvent->message == NS_TEXT_EVENT) {
+    *aInputRange = mTextRange;
+    return NS_OK;
+  }
+  *aInputRange = 0;
+  return NS_ERROR_FAILURE;
 }
 
 NS_METHOD nsDOMEvent::GetEventReply(nsTextEventReply** aReply)
 {
-	if (mEvent->message==NS_TEXT_EVENT) {
-			*aReply = &(((nsTextEvent*)mEvent)->theReply);
-			return NS_OK;
-	}
-
-	return NS_ERROR_FAILURE;
+  NS_ENSURE_ARG_POINTER(aReply);
+  if (mEvent->message==NS_TEXT_EVENT) {
+    *aReply = &(((nsTextEvent*)mEvent)->theReply);
+    return NS_OK;
+  }
+  *aReply = 0;
+  return NS_ERROR_FAILURE;
 }
 
 NS_METHOD nsDOMEvent::GetCompositionReply(nsTextEventReply** aReply)
 {
+  NS_ENSURE_ARG_POINTER(aReply);
   if((mEvent->message==NS_COMPOSITION_START) ||
      (mEvent->message==NS_COMPOSITION_QUERY)) {
     *aReply = &(((nsCompositionEvent*)mEvent)->theReply);
     return NS_OK;
   }
-
+  *aReply = 0;
   return NS_ERROR_FAILURE;
-
 }
 
 NS_METHOD
 nsDOMEvent::GetReconversionReply(nsReconversionEventReply** aReply)
 {
+  NS_ENSURE_ARG_POINTER(aReply);
   *aReply = &(((nsReconversionEvent*)mEvent)->theReply);
   return NS_OK;
 }
@@ -670,6 +652,7 @@ nsDOMEvent::GetReconversionReply(nsReconversionEventReply** aReply)
 
 NS_METHOD nsDOMEvent::GetScreenX(PRInt32* aScreenX)
 {
+  NS_ENSURE_ARG_POINTER(aScreenX);
   if (!mEvent || 
        (mEvent->eventStructType != NS_MOUSE_EVENT &&
         mEvent->eventStructType != NS_POPUP_EVENT &&
@@ -694,6 +677,7 @@ NS_METHOD nsDOMEvent::GetScreenX(PRInt32* aScreenX)
 
 NS_METHOD nsDOMEvent::GetScreenY(PRInt32* aScreenY)
 {
+  NS_ENSURE_ARG_POINTER(aScreenY);
   if (!mEvent || 
        (mEvent->eventStructType != NS_MOUSE_EVENT &&
         mEvent->eventStructType != NS_POPUP_EVENT &&
@@ -718,6 +702,7 @@ NS_METHOD nsDOMEvent::GetScreenY(PRInt32* aScreenY)
 
 NS_METHOD nsDOMEvent::GetClientX(PRInt32* aClientX)
 {
+  NS_ENSURE_ARG_POINTER(aClientX);
   if (!mEvent || 
        (mEvent->eventStructType != NS_MOUSE_EVENT &&
         mEvent->eventStructType != NS_POPUP_EVENT &&
@@ -771,6 +756,7 @@ NS_METHOD nsDOMEvent::GetClientX(PRInt32* aClientX)
 
 NS_METHOD nsDOMEvent::GetClientY(PRInt32* aClientY)
 {
+  NS_ENSURE_ARG_POINTER(aClientY);
   if (!mEvent || 
        (mEvent->eventStructType != NS_MOUSE_EVENT &&
         mEvent->eventStructType != NS_POPUP_EVENT &&
@@ -824,30 +810,35 @@ NS_METHOD nsDOMEvent::GetClientY(PRInt32* aClientY)
 
 NS_METHOD nsDOMEvent::GetAltKey(PRBool* aIsDown)
 {
+  NS_ENSURE_ARG_POINTER(aIsDown);
   *aIsDown = ((nsInputEvent*)mEvent)->isAlt;
   return NS_OK;
 }
 
 NS_METHOD nsDOMEvent::GetCtrlKey(PRBool* aIsDown)
 {
+  NS_ENSURE_ARG_POINTER(aIsDown);
   *aIsDown = ((nsInputEvent*)mEvent)->isControl;
   return NS_OK;
 }
 
 NS_METHOD nsDOMEvent::GetShiftKey(PRBool* aIsDown)
 {
+  NS_ENSURE_ARG_POINTER(aIsDown);
   *aIsDown = ((nsInputEvent*)mEvent)->isShift;
   return NS_OK;
 }
 
 NS_METHOD nsDOMEvent::GetMetaKey(PRBool* aIsDown)
 {
+  NS_ENSURE_ARG_POINTER(aIsDown);
   *aIsDown = ((nsInputEvent*)mEvent)->isMeta;
   return NS_OK;
 }
 
 NS_METHOD nsDOMEvent::GetCharCode(PRUint32* aCharCode)
 {
+  NS_ENSURE_ARG_POINTER(aCharCode);
   if (!mEvent || mEvent->eventStructType != NS_KEY_EVENT) {
     *aCharCode = 0;
     return NS_OK;
@@ -872,6 +863,7 @@ NS_METHOD nsDOMEvent::GetCharCode(PRUint32* aCharCode)
 
 NS_METHOD nsDOMEvent::GetKeyCode(PRUint32* aKeyCode)
 {
+  NS_ENSURE_ARG_POINTER(aKeyCode);
   if (!mEvent || mEvent->eventStructType != NS_KEY_EVENT) {
     *aKeyCode = 0;
     return NS_OK;
@@ -891,6 +883,7 @@ NS_METHOD nsDOMEvent::GetKeyCode(PRUint32* aKeyCode)
 
 NS_METHOD nsDOMEvent::GetButton(PRUint16* aButton)
 {
+  NS_ENSURE_ARG_POINTER(aButton);
   if (!mEvent || mEvent->eventStructType != NS_MOUSE_EVENT) {
     NS_WARNING("Tried to get mouse button for null or non-mouse event!");
     *aButton = (PRUint16)-1;
@@ -930,6 +923,7 @@ NS_METHOD nsDOMEvent::GetButton(PRUint16* aButton)
 
 NS_METHOD nsDOMEvent::GetRelatedTarget(nsIDOMEventTarget** aRelatedTarget)
 {
+  NS_ENSURE_ARG_POINTER(aRelatedTarget);
   *aRelatedTarget = nsnull;
 
   if (!mPresContext) {
@@ -954,6 +948,7 @@ NS_METHOD nsDOMEvent::GetRelatedTarget(nsIDOMEventTarget** aRelatedTarget)
 // nsINSEventInterface
 NS_METHOD nsDOMEvent::GetLayerX(PRInt32* aLayerX)
 {
+  NS_ENSURE_ARG_POINTER(aLayerX);
   if (!mEvent || (mEvent->eventStructType != NS_MOUSE_EVENT) ||
       !mPresContext) {
     *aLayerX = 0;
@@ -968,6 +963,7 @@ NS_METHOD nsDOMEvent::GetLayerX(PRInt32* aLayerX)
 
 NS_METHOD nsDOMEvent::GetLayerY(PRInt32* aLayerY)
 {
+  NS_ENSURE_ARG_POINTER(aLayerY);
   if (!mEvent || (mEvent->eventStructType != NS_MOUSE_EVENT) ||
       !mPresContext) {
     *aLayerY = 0;
@@ -983,6 +979,9 @@ NS_METHOD nsDOMEvent::GetLayerY(PRInt32* aLayerY)
 nsresult nsDOMEvent::GetScrollInfo(nsIScrollableView** aScrollableView,
    float* aP2T, float* aT2P)
 {
+  NS_ENSURE_ARG_POINTER(aScrollableView);
+  NS_ENSURE_ARG_POINTER(aP2T);
+  NS_ENSURE_ARG_POINTER(aT2P);
   if (!mPresContext) {
     *aScrollableView = nsnull;
     return NS_ERROR_FAILURE;
@@ -1004,6 +1003,7 @@ nsresult nsDOMEvent::GetScrollInfo(nsIScrollableView** aScrollableView,
 
 NS_METHOD nsDOMEvent::GetPageX(PRInt32* aPageX)
 {
+  NS_ENSURE_ARG_POINTER(aPageX);
   nsresult ret = NS_OK;
   PRInt32 scrollX = 0;
   nsIScrollableView* view = nsnull;
@@ -1029,6 +1029,7 @@ NS_METHOD nsDOMEvent::GetPageX(PRInt32* aPageX)
 
 NS_METHOD nsDOMEvent::GetPageY(PRInt32* aPageY)
 {
+  NS_ENSURE_ARG_POINTER(aPageY);
   nsresult ret = NS_OK;
   PRInt32 scrollY = 0;
   nsIScrollableView* view = nsnull;
@@ -1054,6 +1055,7 @@ NS_METHOD nsDOMEvent::GetPageY(PRInt32* aPageY)
 
 NS_METHOD nsDOMEvent::GetWhich(PRUint32* aWhich)
 {
+  NS_ENSURE_ARG_POINTER(aWhich);
   switch (mEvent->eventStructType) {
   case NS_KEY_EVENT:
 	  switch (mEvent->message) {
@@ -1090,6 +1092,7 @@ NS_METHOD nsDOMEvent::GetWhich(PRUint32* aWhich)
 
 NS_METHOD nsDOMEvent::GetRangeParent(nsIDOMNode** aRangeParent)
 {
+  NS_ENSURE_ARG_POINTER(aRangeParent);
   nsIFrame* targetFrame = nsnull;
   nsCOMPtr<nsIEventStateManager> manager;
 
@@ -1121,6 +1124,7 @@ NS_METHOD nsDOMEvent::GetRangeParent(nsIDOMNode** aRangeParent)
 
 NS_METHOD nsDOMEvent::GetRangeOffset(PRInt32* aRangeOffset)
 {
+  NS_ENSURE_ARG_POINTER(aRangeOffset);
   nsIFrame* targetFrame = nsnull;
   nsIEventStateManager* manager;
 
@@ -1150,6 +1154,7 @@ NS_METHOD nsDOMEvent::GetRangeOffset(PRInt32* aRangeOffset)
 
 NS_METHOD nsDOMEvent::GetCancelBubble(PRBool* aCancelBubble)
 {
+  NS_ENSURE_ARG_POINTER(aCancelBubble);
   if (mEvent->flags & NS_EVENT_FLAG_BUBBLE || mEvent->flags & NS_EVENT_FLAG_INIT) {
     *aCancelBubble = (mEvent->flags &= NS_EVENT_FLAG_STOP_DISPATCH) ? PR_TRUE : PR_FALSE;
   }
@@ -1174,6 +1179,7 @@ NS_METHOD nsDOMEvent::SetCancelBubble(PRBool aCancelBubble)
 
 NS_METHOD nsDOMEvent::GetIsChar(PRBool* aIsChar)
 {
+  NS_ENSURE_ARG_POINTER(aIsChar);
   if (!mEvent) {
     *aIsChar = PR_FALSE;
     return NS_OK;
@@ -1193,6 +1199,7 @@ NS_METHOD nsDOMEvent::GetIsChar(PRBool* aIsChar)
 
 NS_METHOD nsDOMEvent::GetPreventDefault(PRBool* aReturn)
 {
+  NS_ENSURE_ARG_POINTER(aReturn);
   if (!mEvent) {
     *aReturn = PR_FALSE; 
   }
@@ -1324,6 +1331,7 @@ nsDOMEvent::InitMouseEvent(const nsAString & aTypeArg, PRBool aCanBubbleArg, PRB
   mEvent->flags |= aCanBubbleArg ? NS_EVENT_FLAG_NONE : NS_EVENT_FLAG_CANT_BUBBLE;
   mEvent->flags |= aCancelableArg ? NS_EVENT_FLAG_NONE : NS_EVENT_FLAG_CANT_CANCEL;
 
+  NS_ASSERTION(mEvent->eventStructType == NS_MOUSE_EVENT || mEvent->eventStructType == NS_MOUSE_SCROLL_EVENT, "event type mismatch");
   if (mEvent->eventStructType == NS_MOUSE_EVENT || mEvent->eventStructType == NS_MOUSE_SCROLL_EVENT) {
     nsInputEvent* inputEvent = NS_STATIC_CAST(nsInputEvent*, mEvent);
     inputEvent->isControl = aCtrlKeyArg;
@@ -1346,10 +1354,11 @@ nsDOMEvent::InitMouseEvent(const nsAString & aTypeArg, PRBool aCanBubbleArg, PRB
       nsMouseEvent* mouseEvent = NS_STATIC_CAST(nsMouseEvent*, mEvent);
       mouseEvent->clickCount = aDetailArg;
     }
+    return NS_OK;
   }
   //include a way to set view once we have more than one
 
-  return NS_OK;
+  return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
@@ -1363,6 +1372,7 @@ nsDOMEvent::InitKeyEvent(const nsAString& aTypeArg, PRBool aCanBubbleArg, PRBool
   mEvent->flags |= aCancelableArg ? NS_EVENT_FLAG_NONE : NS_EVENT_FLAG_CANT_CANCEL;
   mEvent->internalAppFlags |= NS_APP_EVENT_FLAG_NONE;
 
+  NS_ASSERTION(mEvent->eventStructType == NS_KEY_EVENT, "event type mismatch");
   if (mEvent->eventStructType == NS_KEY_EVENT) {
     nsKeyEvent* keyEvent = NS_STATIC_CAST(nsKeyEvent*, mEvent);
     keyEvent->isControl = aCtrlKeyArg;
@@ -1371,10 +1381,62 @@ nsDOMEvent::InitKeyEvent(const nsAString& aTypeArg, PRBool aCanBubbleArg, PRBool
     keyEvent->isMeta = aMetaKeyArg;
     keyEvent->keyCode = aKeyCodeArg;
     keyEvent->charCode = aCharCodeArg;
+    return NS_OK;
   }
   //include a way to set view once we have more than one
 
-  return NS_OK;
+  return NS_ERROR_FAILURE;
+}
+
+NS_IMETHODIMP nsDOMEvent::InitPopupBlockedEvent(const nsAString & aTypeArg,
+                            PRBool aCanBubbleArg, PRBool aCancelableArg,
+                            nsIURI *aRequestingWindowURI,
+                            nsIURI *aPopupWindowURI)
+{
+  NS_ENSURE_SUCCESS(SetEventType(aTypeArg), NS_ERROR_FAILURE);
+  mEvent->flags |= aCanBubbleArg ? NS_EVENT_FLAG_NONE : NS_EVENT_FLAG_CANT_BUBBLE;
+  mEvent->flags |= aCancelableArg ? NS_EVENT_FLAG_NONE : NS_EVENT_FLAG_CANT_CANCEL;
+  mEvent->internalAppFlags |= NS_APP_EVENT_FLAG_NONE;
+
+  NS_ASSERTION(mEvent->eventStructType == NS_POPUPBLOCKED_EVENT, "event type mismatch");
+  if (mEvent->eventStructType == NS_POPUPBLOCKED_EVENT) {
+    nsPopupBlockedEvent* event = NS_STATIC_CAST(nsPopupBlockedEvent*, mEvent);
+    event->mRequestingWindowURI = aRequestingWindowURI;
+    event->mPopupWindowURI = aPopupWindowURI;
+    NS_IF_ADDREF(event->mRequestingWindowURI);
+    NS_IF_ADDREF(event->mPopupWindowURI);
+    return NS_OK;
+  }
+
+  return NS_ERROR_FAILURE;
+}
+
+/* readonly attribute nsIURI requestingWindowURI; */
+NS_IMETHODIMP nsDOMEvent::GetRequestingWindowURI(nsIURI **aRequestingWindowURI)
+{
+  NS_ENSURE_ARG_POINTER(aRequestingWindowURI);
+  if (mEvent->eventStructType == NS_POPUPBLOCKED_EVENT) {
+    nsPopupBlockedEvent* event = NS_STATIC_CAST(nsPopupBlockedEvent*, mEvent);
+    *aRequestingWindowURI = event->mRequestingWindowURI;
+    NS_IF_ADDREF(*aRequestingWindowURI);
+    return NS_OK;
+  }
+  *aRequestingWindowURI = 0;
+  return NS_ERROR_FAILURE;
+}
+
+/* readonly attribute nsIURI popupWindowURI; */
+NS_IMETHODIMP nsDOMEvent::GetPopupWindowURI(nsIURI **aPopupWindowURI)
+{
+  NS_ENSURE_ARG_POINTER(aPopupWindowURI);
+  if (mEvent->eventStructType == NS_POPUPBLOCKED_EVENT) {
+    nsPopupBlockedEvent* event = NS_STATIC_CAST(nsPopupBlockedEvent*, mEvent);
+    *aPopupWindowURI = event->mPopupWindowURI;
+    NS_IF_ADDREF(*aPopupWindowURI);
+    return NS_OK;
+  }
+  *aPopupWindowURI = 0;
+  return NS_ERROR_FAILURE;
 }
 
 NS_METHOD nsDOMEvent::DuplicatePrivateData()
@@ -1387,31 +1449,19 @@ NS_METHOD nsDOMEvent::DuplicatePrivateData()
 
 NS_METHOD nsDOMEvent::SetTarget(nsIDOMEventTarget* aTarget)
 {
-  if (mTarget != aTarget) {
-    NS_IF_RELEASE(mTarget);
-    NS_IF_ADDREF(aTarget);
-    mTarget = aTarget;
-  }
+  mTarget = aTarget;
   return NS_OK;
 }
 
 NS_METHOD nsDOMEvent::SetCurrentTarget(nsIDOMEventTarget* aCurrentTarget)
 {
-  if (mCurrentTarget != aCurrentTarget) {
-    NS_IF_RELEASE(mCurrentTarget);
-    NS_IF_ADDREF(aCurrentTarget);
-    mCurrentTarget = aCurrentTarget;
-  }
+  mCurrentTarget = aCurrentTarget;
   return NS_OK;
 }
 
 NS_METHOD nsDOMEvent::SetOriginalTarget(nsIDOMEventTarget* aOriginalTarget)
 {
-  if (mOriginalTarget != aOriginalTarget) {
-    NS_IF_RELEASE(mOriginalTarget);
-    NS_IF_ADDREF(aOriginalTarget);
-    mOriginalTarget = aOriginalTarget;
-  }
+  mOriginalTarget = aOriginalTarget;
   return NS_OK;
 }
 
@@ -1451,6 +1501,7 @@ nsDOMEvent::SetHandled(PRBool aHandled)
 NS_IMETHODIMP
 nsDOMEvent::GetInternalNSEvent(nsEvent** aNSEvent)
 {
+  NS_ENSURE_ARG_POINTER(aNSEvent);
   *aNSEvent = mEvent;
   return NS_OK;
 }
@@ -1574,10 +1625,10 @@ const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
   return nsnull;
 }
 
-nsresult NS_NewDOMUIEvent(nsIDOMEvent** aInstancePtrResult,
-                          nsIPresContext* aPresContext,
-                          const nsAString& aEventType,
-                          nsEvent *aEvent) 
+nsresult
+NS_NewDOMUIEvent(nsIDOMEvent** aInstancePtrResult,
+                 nsIPresContext* aPresContext, const nsAString& aEventType,
+                 nsEvent *aEvent)
 {
   nsDOMEvent* it = new nsDOMEvent(aPresContext, aEvent, aEventType);
 
@@ -1588,7 +1639,9 @@ nsresult NS_NewDOMUIEvent(nsIDOMEvent** aInstancePtrResult,
   return CallQueryInterface(it, aInstancePtrResult);
 }
 
-nsresult NS_NewDOMEvent(nsIDOMEvent** aInstancePtrResult, nsIPresContext* aPresContext, nsEvent *aEvent) 
+nsresult
+NS_NewDOMEvent(nsIDOMEvent** aInstancePtrResult, nsIPresContext* aPresContext,
+               nsEvent *aEvent) 
 {
   return NS_ERROR_FAILURE;
 }
