@@ -44,7 +44,9 @@
 #include "nsCOMPtr.h"
 #include "nsIPrincipal.h"
 #include "nsIDOMEventReceiver.h"
+#include "nsIDOM3EventTarget.h"
 #include "nsHashtable.h"
+#include "nsIScriptContext.h"
 
 class nsIDOMEvent;
 class nsIAtom;
@@ -55,6 +57,7 @@ typedef struct {
   PRUint8 mSubType;
   PRUint8 mHandlerIsString;
   PRUint8 mSubTypeCapture;
+  PRUint16 mGroupFlags;
 } nsListenerStruct;
 
 //These define the internal type of the EventListenerManager
@@ -95,7 +98,8 @@ enum EventArrayType {
  */
 
 class nsEventListenerManager : public nsIEventListenerManager,
-                               public nsIDOMEventReceiver
+                               public nsIDOMEventReceiver,
+                               public nsIDOM3EventTarget
 {
 
 public:
@@ -114,11 +118,13 @@ public:
                                       const nsIID& aIID, PRInt32 aFlags);
   NS_IMETHOD AddEventListenerByType(nsIDOMEventListener *aListener,
                                     const nsAString& type,
-                                    PRInt32 aFlags);
+                                    PRInt32 aFlags,
+                                    nsIDOMEventGroup* aEvtGroup);
   NS_IMETHOD RemoveEventListenerByType(nsIDOMEventListener *aListener,
                                        const nsAString& type,
-                                       PRInt32 aFlags) ;
-  NS_IMETHOD AddScriptEventListener(nsIScriptContext*aContext,
+                                       PRInt32 aFlags,
+                                       nsIDOMEventGroup* aEvtGroup);
+  NS_IMETHOD AddScriptEventListener(nsIScriptContext *aContext,
                                     nsISupports *aObject,
                                     nsIAtom *aName,
                                     const nsAString& aFunc,
@@ -157,18 +163,17 @@ public:
     return NS_OK;
   }
 
+  NS_IMETHOD GetSystemEventGroupLM(nsIDOMEventGroup** aGroup);
+
   static nsresult GetIdentifiersForType(nsIAtom* aType,
                                         EventArrayType* aArrayType,
                                         PRInt32* aSubType);
 
-  // nsIDOMEventTarget interface
-  NS_IMETHOD AddEventListener(const nsAString& aType, 
-                              nsIDOMEventListener* aListener, 
-                              PRBool aUseCapture);
-  NS_IMETHOD RemoveEventListener(const nsAString& aType, 
-                                 nsIDOMEventListener* aListener, 
-                                 PRBool aUseCapture);
-  NS_IMETHOD DispatchEvent(nsIDOMEvent* aEvent, PRBool *_retval);
+  // nsIDOMEventTarget
+  NS_DECL_NSIDOMEVENTTARGET
+
+  // nsIDOM3EventTarget
+  NS_DECL_NSIDOM3EVENTTARGET
 
   // nsIDOMEventReceiver interface
   NS_IMETHOD AddEventListenerByIID(nsIDOMEventListener *aListener,
@@ -177,6 +182,7 @@ public:
                                       const nsIID& aIID);
   NS_IMETHOD GetListenerManager(nsIEventListenerManager** aInstancePtrResult);
   NS_IMETHOD HandleEvent(nsIDOMEvent *aEvent);
+  NS_IMETHOD GetSystemEventGroup(nsIDOMEventGroup** aGroup);
 
   static void Shutdown();
 
@@ -199,16 +205,21 @@ protected:
                             EventArrayType aType, 
                             PRInt32 aSubType,
                             nsHashKey* aKey,
-                            PRInt32 aFlags);
+                            PRInt32 aFlags,
+                            nsIDOMEventGroup* aEvtGrp);
   nsresult RemoveEventListener(nsIDOMEventListener *aListener,
                                EventArrayType aType,
                                PRInt32 aSubType,
                                nsHashKey* aKey,
-                               PRInt32 aFlags);
+                               PRInt32 aFlags,
+                               nsIDOMEventGroup* aEvtGrp);
   void ReleaseListeners(nsVoidArray** aListeners, PRBool aScriptOnly);
   nsresult FlipCaptureBit(PRInt32 aEventTypes, PRBool aInitCapture);
   nsVoidArray* GetListenersByType(EventArrayType aType, nsHashKey* aKey, PRBool aCreate);
   EventArrayType GetTypeForIID(const nsIID& aIID);
+  void GetCoordinatesFor(nsIDOMElement *aCurrentEl, nsIPresContext *aPresContext,
+                         nsIPresShell *aPresShell, nsPoint& aTargetPt);
+  nsresult GetDOM2EventGroup(nsIDOMEventGroup** aGroup);
 
   PRUint8 mManagerType;
   EventArrayType mSingleListenerType;
@@ -216,6 +227,7 @@ protected:
   nsVoidArray* mMultiListeners;
   nsHashtable* mGenericListeners;
   PRBool mListenersRemoved;
+  static PRUint32 mInstanceCount;
 
   nsCOMPtr<nsIPrincipal> mPrincipal;
   nsISupports* mTarget;  //WEAK
@@ -321,4 +333,5 @@ protected:
 #define NS_EVENT_BITS_CONTEXT_NONE  0x00
 #define NS_EVENT_BITS_CONTEXT_MENU  0x01
 
+
 #endif // nsEventListenerManager_h__
