@@ -39,22 +39,14 @@
 
 // JBK added for submit move from content frame
 #include "nsIFile.h"
-#include "nsIFileStreams.h"
-#include "nsIFileSpec.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsIFormProcessor.h"
-static NS_DEFINE_CID(kFormProcessorCID, NS_FORMPROCESSOR_CID);
 #include "nsIURI.h"
 #include "nsNetUtil.h"
-static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
 #include "nsIPrefBranch.h"
 #include "nsIPrefService.h"
-#include "nsSpecialSystemDirectory.h"
 #include "nsLinebreakConverter.h"
 #include "nsICharsetConverterManager.h"
-static NS_DEFINE_CID(kCharsetConverterManagerCID,
-                     NS_ICHARSETCONVERTERMANAGER_CID);
-#include "xp_path.h"
 #include "nsICharsetAlias.h"
 #include "nsEscape.h"
 #include "nsUnicharUtils.h"
@@ -68,6 +60,8 @@ static NS_DEFINE_CID(kCharsetConverterManagerCID,
 #include "nsBidiUtils.h"
 //end
 
+static NS_DEFINE_CID(kFormProcessorCID, NS_FORMPROCESSOR_CID);
+static NS_DEFINE_CID(kCharsetAliasCID, NS_CHARSETALIAS_CID);
 
 /**
  * Helper superclass implementation of nsIFormSubmission, providing common
@@ -1459,7 +1453,10 @@ nsFormSubmission::ProcessValue(nsIDOMHTMLElement* aSource,
       return nsnull;
     }
 
-    nsresult rv = mFormProcessor->ProcessValue(aSource, tmpNameStr, *retval);
+#ifdef DEBUG
+    nsresult rv =
+#endif
+    mFormProcessor->ProcessValue(aSource, tmpNameStr, *retval);
     NS_ASSERTION(NS_SUCCEEDED(rv), "Unable to Notify form process observer");
   }
 
