@@ -1,26 +1,46 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim:set tw=80 expandtab softtabstop=2 ts=2 sw=2: */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is Mozilla Communicator client code.
  *
- * The Initial Developer of the Original Code is Netscape Communications
- * Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
- */
+ * Contributor(s):
+ *   Daniel Glazman <glazman@netscape.com>
+ *
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 #include "nsCOMPtr.h"
 #include "nsHTMLUtils.h"
+#include "nsReadableUtils.h"
+#include "nsUnicharUtils.h"
 #include "nsIDOMHTMLAnchorElement.h"
 #include "nsIDOMNSHTMLAnchorElement.h"
 #include "nsIDOMEventReceiver.h"
@@ -29,8 +49,6 @@
 #include "nsGenericHTMLElement.h"
 #include "nsILink.h"
 #include "nsHTMLAtoms.h"
-#include "nsHTMLIIDs.h"
-#include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
 #include "nsIEventStateManager.h"
@@ -38,12 +56,20 @@
 #include "nsIEventStateManager.h"
 #include "nsIDOMEvent.h"
 #include "nsNetUtil.h"
+#include "nsCRT.h"
+
+// For GetText().
+#include "nsIContentIterator.h"
+#include "nsIDOMText.h"
+#include "nsIEnumerator.h"
 
 #include "nsCOMPtr.h"
 #include "nsIFrameManager.h"
 #include "nsIPresShell.h"
 #include "nsIDocument.h"
-#include "nsIHTMLAttributes.h"
+#include "nsHTMLAttributes.h"
+
+nsresult NS_NewContentIterator(nsIContentIterator** aInstancePtrResult);
 
 // XXX suppress
 
@@ -87,16 +113,22 @@ public:
   NS_IMETHOD SetFocus(nsIPresContext* aPresContext);
   NS_IMETHOD RemoveFocus(nsIPresContext* aPresContext);
   NS_IMETHOD StringToAttribute(nsIAtom* aAttribute,
-                               const nsAReadableString& aValue,
+                               const nsAString& aValue,
                                nsHTMLValue& aResult);
   NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
                             nsIDOMEvent** aDOMEvent, PRUint32 aFlags,
                             nsEventStatus* aEventStatus);
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
+
+  NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                     const nsAString& aValue,
+                     PRBool aNotify);
+  NS_IMETHOD SetAttr(nsINodeInfo* aNodeInfo,
+                     const nsAString& aValue,
+                     PRBool aNotify);
+  NS_IMETHOD UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute,
+                       PRBool aNotify);
 
 protected:
-  nsresult RegUnRegAccessKey(PRBool aDoReg);
-
   // The cached visited state
   nsLinkState mLinkState;
 
@@ -123,7 +155,7 @@ NS_NewHTMLAnchorElement(nsIHTMLContent** aInstancePtrResult,
     return rv;
   }
 
-  *aInstancePtrResult = NS_STATIC_CAST(nsIHTMLContent *, it);
+  *aInstancePtrResult = it;
   NS_ADDREF(*aInstancePtrResult);
 
   return NS_OK;
@@ -176,7 +208,7 @@ nsHTMLAnchorElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
 
   CopyInnerTo(this, it, aDeep);
 
-  *aReturn = NS_STATIC_CAST(nsIDOMNode *, it);
+  *aReturn = it;
 
   NS_ADDREF(*aReturn);
 
@@ -184,7 +216,6 @@ nsHTMLAnchorElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
 }
 
 
-NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, AccessKey, accesskey)
 NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, Charset, charset)
 NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, Coords, coords)
 NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, Hreflang, hreflang)
@@ -194,67 +225,23 @@ NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, Rev, rev)
 NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, Shape, shape)
 NS_IMPL_INT_ATTR(nsHTMLAnchorElement, TabIndex, tabindex)
 NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, Type, type)
+NS_IMPL_STRING_ATTR(nsHTMLAnchorElement, AccessKey, accesskey)
 
 
-// This goes and gets the proper PresContext in order
-// for it to get the EventStateManager so it can register 
-// the access key
-nsresult nsHTMLAnchorElement::RegUnRegAccessKey(PRBool aDoReg)
-{
-  // first check to see if it even has an acess key
-  nsAutoString accessKey;
-  PRInt32 nameSpaceID;
-  GetNameSpaceID(nameSpaceID);
-  nsresult rv;
-
-  rv = NS_STATIC_CAST(nsIContent *, this)->GetAttribute(nameSpaceID,
-                                                        nsHTMLAtoms::accesskey,
-                                                        accessKey);
-
-  if (NS_CONTENT_ATTR_NOT_THERE != rv) {
-    nsCOMPtr<nsIPresContext> presContext;
-    GetPresContext(this, getter_AddRefs(presContext));
-
-    // With a valid PresContext we can get the EVM 
-    // and register the access key
-    if (presContext) {
-      nsCOMPtr<nsIEventStateManager> stateManager;
-      presContext->GetEventStateManager(getter_AddRefs(stateManager));
-
-      if (stateManager) {
-        if (aDoReg) {
-          return stateManager->RegisterAccessKey(nsnull, this,
-                                                 (PRUint32)accessKey.First());
-        } else {
-          return stateManager->UnregisterAccessKey(nsnull, this,
-                                                   (PRUint32)accessKey.First());
-        }
-      }
-    }
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
 NS_IMETHODIMP
 nsHTMLAnchorElement::SetDocument(nsIDocument* aDocument, PRBool aDeep,
                                  PRBool aCompileEventHandlers)
 {
-  // The document gets set to null before it is destroyed,
-  // so we unregister the the access key here (if it has one)
-  // before setting it to null
-  if (aDocument == nsnull) {
+  // Unregister the access key for the old document.
+  if (mDocument) {
     RegUnRegAccessKey(PR_FALSE);
   }
 
-  nsresult rv;
-  rv = nsGenericHTMLContainerElement::SetDocument(aDocument,
-                                                  aDeep,
-                                                  aCompileEventHandlers);
+  nsresult rv = nsGenericHTMLContainerElement::SetDocument(aDocument, aDeep,
+                                                        aCompileEventHandlers);
 
-  // Register the access key here (if it has one) 
-  // if the document isn't null
-  if (aDocument != nsnull) {
+  // Register the access key for the new document.
+  if (mDocument) {
     RegUnRegAccessKey(PR_TRUE);
   }
 
@@ -337,17 +324,17 @@ nsHTMLAnchorElement::RemoveFocus(nsIPresContext* aPresContext)
 
 NS_IMETHODIMP
 nsHTMLAnchorElement::StringToAttribute(nsIAtom* aAttribute,
-                                       const nsAReadableString& aValue,
+                                       const nsAString& aValue,
                                        nsHTMLValue& aResult)
 {
   if (aAttribute == nsHTMLAtoms::tabindex) {
-    if (ParseValue(aValue, 0, 32767, aResult, eHTMLUnit_Integer)) {
+    if (aResult.ParseIntWithBounds(aValue, eHTMLUnit_Integer, 0, 32767)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
   else if (aAttribute == nsHTMLAtoms::suppress) {
-    if (nsCRT::strcasecmp(PromiseFlatString(aValue).get(),
-                          NS_LITERAL_STRING("true").get())) {
+    if (!aValue.Equals(NS_LITERAL_STRING("true"),
+                       nsCaseInsensitiveStringComparator())) {
       aResult.SetEmptyValue();  // XXX? shouldn't just leave "true"
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
@@ -368,7 +355,7 @@ nsHTMLAnchorElement::HandleDOMEvent(nsIPresContext* aPresContext,
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::GetHref(nsAWritableString& aValue)
+nsHTMLAnchorElement::GetHref(nsAString& aValue)
 {
   char *buf;
   nsresult rv = GetHrefCString(buf);
@@ -382,26 +369,18 @@ nsHTMLAnchorElement::GetHref(nsAWritableString& aValue)
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::SetHref(const nsAReadableString& aValue)
+nsHTMLAnchorElement::SetHref(const nsAString& aValue)
 {
-  // Clobber our "cache", so we'll recompute it the next time
-  // somebody asks for it.
-  mLinkState = eLinkState_Unknown;
-
-  return NS_STATIC_CAST(nsIContent *, this)->SetAttribute(kNameSpaceID_HTML,
-                                                          nsHTMLAtoms::href,
-                                                          aValue, PR_TRUE);
+  return SetAttr(kNameSpaceID_None, nsHTMLAtoms::href, aValue, PR_TRUE);
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::GetTarget(nsAWritableString& aValue)
+nsHTMLAnchorElement::GetTarget(nsAString& aValue)
 {
   aValue.Truncate();
 
   nsresult rv;
-  rv = NS_STATIC_CAST(nsIContent *, this)->GetAttribute(kNameSpaceID_HTML,
-                                                        nsHTMLAtoms::target,
-                                                        aValue);
+  rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::target, aValue);
   if (rv == NS_CONTENT_ATTR_NOT_THERE && mDocument) {
     rv = mDocument->GetBaseTarget(aValue);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -410,249 +389,259 @@ nsHTMLAnchorElement::GetTarget(nsAWritableString& aValue)
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::SetTarget(const nsAReadableString& aValue)
+nsHTMLAnchorElement::SetTarget(const nsAString& aValue)
 {
-  return NS_STATIC_CAST(nsIContent *, this)->SetAttribute(kNameSpaceID_HTML,
-                                                          nsHTMLAtoms::target,
-                                                          aValue, PR_TRUE);
+  return SetAttr(kNameSpaceID_None, nsHTMLAtoms::target, aValue, PR_TRUE);
+}
+
+NS_IMETHODIMP    
+nsHTMLAnchorElement::GetProtocol(nsAString& aProtocol)
+{
+  nsAutoString href;
+
+  nsresult rv = GetHref(href);
+  if (NS_FAILED(rv))
+    return rv;
+
+  nsCOMPtr<nsIDocument> doc;
+  mNodeInfo->GetDocument(*getter_AddRefs(doc));
+
+  return GetProtocolFromHrefString(href, aProtocol, doc);
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
+nsHTMLAnchorElement::SetProtocol(const nsAString& aProtocol)
 {
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
+  nsAutoString href, new_href;
+  nsresult rv = GetHref(href);
+  if (NS_FAILED(rv))
+    return rv;
 
-  return NS_OK;
-}
-
-
-NS_IMETHODIMP    
-nsHTMLAnchorElement::GetProtocol(nsAWritableString& aProtocol)
-{
-  nsAutoString href;
-  nsCOMPtr<nsIURI> url;
-  nsresult result = NS_OK;
-  
-  result = GetHref(href);
-  if (NS_OK == result) {
-    result = NS_NewURI(getter_AddRefs(url), href);
-    if (NS_FAILED(result))
-      return result;
-
-    char* protocol;
-    result = url->GetScheme(&protocol);
-    if (NS_FAILED(result))
-      return result;
-
-    aProtocol.Assign(NS_ConvertASCIItoUCS2(protocol));
-    aProtocol.Append(PRUnichar(':'));
-
-    nsCRT::free(protocol);
-  }
-
-  return result;
-}
-
-NS_IMETHODIMP    
-nsHTMLAnchorElement::GetHost(nsAWritableString& aHost)
-{
-  nsAutoString href;
-  nsCOMPtr<nsIURI> url;
-  nsresult result = NS_OK;
-  
-  result = GetHref(href);
-  if (NS_OK == result) {
-    result = NS_NewURI(getter_AddRefs(url), href);
-    if (NS_OK == result) {
-      char* host;
-      result = url->GetHost(&host);
-      if (result == NS_OK) {
-        aHost.Assign(NS_ConvertASCIItoUCS2(host));
-        nsCRT::free(host);
-        PRInt32 port;
-        (void)url->GetPort(&port);
-        if (-1 != port) {
-          aHost.Append(NS_LITERAL_STRING(":"));
-          nsAutoString portStr;
-          portStr.AppendInt(port);
-          aHost.Append(portStr);
-        }
-      }
-    }
-  }
-
-  return result;
-}
-
-NS_IMETHODIMP    
-nsHTMLAnchorElement::GetHostname(nsAWritableString& aHostname)
-{
-  nsAutoString href;
-  nsIURI *url;
-  nsresult result = NS_OK;
-  
-  result = GetHref(href);
-  if (NS_OK == result) {
-    result = NS_NewURI(&url, href);
-    if (NS_OK == result) {
-      char* host;
-      result = url->GetHost(&host);
-      if (result == NS_OK) {
-        aHostname.Assign(NS_ConvertASCIItoUCS2(host));
-        nsCRT::free(host);
-      }
-      NS_RELEASE(url);
-    }
-  }
-
-  return result;
-}
-
-NS_IMETHODIMP    
-nsHTMLAnchorElement::GetPathname(nsAWritableString& aPathname)
-{
-  nsAutoString href;
-  nsCOMPtr<nsIURI> uri;
-  nsresult result = NS_OK;
-
-  aPathname.Truncate();
-  
-  result = GetHref(href);
-  if (NS_FAILED(result)) {
-    return result;
-  }
-
-  result = NS_NewURI(getter_AddRefs(uri), href);
-  if (NS_FAILED(result)) {
-    return result;
-  }
-
-  nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
-
-  if (!url) {
+  rv = SetProtocolInHrefString(href, aProtocol, new_href);
+  if (NS_FAILED(rv))
+    // Ignore failures to be compatible with NS4
     return NS_OK;
-  }
 
-  char* file;
-  result = url->GetFilePath(&file);
-  if (NS_FAILED(result)) {
-    return result;
-  }
-
-  aPathname.Assign(NS_ConvertASCIItoUCS2(file));
-  nsCRT::free(file);
-
-  return result;
+  return SetHref(new_href);
 }
 
 NS_IMETHODIMP    
-nsHTMLAnchorElement::GetSearch(nsAWritableString& aSearch)
+nsHTMLAnchorElement::GetHost(nsAString& aHost)
 {
   nsAutoString href;
-  nsCOMPtr<nsIURI> uri;
-  nsresult result = NS_OK;
+  
+  nsresult rv = GetHref(href);
+  if (NS_FAILED(rv))
+    return rv;
 
-  result = GetHref(href);
-  if (NS_OK == result) {
-    result = NS_NewURI(getter_AddRefs(uri), href);
-    if (NS_OK == result) {
-      char *search;
-      nsIURL* url;
-      result = uri->QueryInterface(NS_GET_IID(nsIURL), (void**)&url);
+  return GetHostFromHrefString(href, aHost);
+}
 
-      if (NS_SUCCEEDED(result)) {
-        result = url->GetEscapedQuery(&search);
-        NS_RELEASE(url);
-      }
+NS_IMETHODIMP
+nsHTMLAnchorElement::SetHost(const nsAString& aHost)
+{
+  nsAutoString href, new_href;
+  nsresult rv = GetHref(href);
+  if (NS_FAILED(rv))
+    return rv;
 
-      if (NS_SUCCEEDED(result) && search && (*search)) {
-        aSearch.Assign(PRUnichar('?'));
-        aSearch.Append(NS_ConvertASCIItoUCS2(search));
-        nsCRT::free(search);
-      }
-      else {
-        aSearch.SetLength(0);
-      }
-    }
-  }
+  rv = SetHostInHrefString(href, aHost, new_href);
+  if (NS_FAILED(rv))
+    // Ignore failures to be compatible with NS4
+    return NS_OK;
 
-  return result;
+  return SetHref(new_href);
 }
 
 NS_IMETHODIMP    
-nsHTMLAnchorElement::GetPort(nsAWritableString& aPort)
+nsHTMLAnchorElement::GetHostname(nsAString& aHostname)
 {
   nsAutoString href;
-  nsCOMPtr<nsIURI> url;
-  nsresult result = NS_OK;
+  nsresult rv = GetHref(href);
+  if (NS_FAILED(rv))
+    return rv;
 
-  result = GetHref(href);
-  if (NS_OK == result) {
-    result = NS_NewURI(getter_AddRefs(url), href);
-    if (NS_OK == result) {
-      aPort.Truncate(0);
-      PRInt32 port;
-      (void)url->GetPort(&port);
-      if (-1 != port) {
-        nsAutoString portStr;
-        portStr.AppendInt(port);
-        aPort.Append(portStr);
-      }
-    }
-  }
+  return GetHostnameFromHrefString(href, aHostname);
+}
 
-  return result;
+NS_IMETHODIMP
+nsHTMLAnchorElement::SetHostname(const nsAString& aHostname)
+{
+  nsAutoString href, new_href;
+  nsresult rv = GetHref(href);
+  if (NS_FAILED(rv))
+    return rv;
+
+  rv = SetHostnameInHrefString(href, aHostname, new_href);
+  if (NS_FAILED(rv))
+    // Ignore failures to be compatible with NS4
+    return NS_OK;
+  
+  return SetHref(new_href);
 }
 
 NS_IMETHODIMP    
-nsHTMLAnchorElement::GetHash(nsAWritableString& aHash)
+nsHTMLAnchorElement::GetPathname(nsAString& aPathname)
 {
   nsAutoString href;
-  nsCOMPtr<nsIURI> uri;
-  nsresult result = NS_OK;
+ 
+  nsresult rv = GetHref(href);
+  if (NS_FAILED(rv))
+    return rv;
 
-  result = GetHref(href);
-  if (NS_OK == result) {
-    result = NS_NewURI(getter_AddRefs(uri), href);
+  return GetPathnameFromHrefString(href, aPathname);
+}
 
-    if (NS_OK == result) {
-      nsXPIDLCString ref;
-      nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
-      if (url) {
-        result = url->GetRef(getter_Copies(ref));
-      }
+NS_IMETHODIMP
+nsHTMLAnchorElement::SetPathname(const nsAString& aPathname)
+{
+  nsAutoString href, new_href;
+  nsresult rv = GetHref(href);
+  if (NS_FAILED(rv))
+    return rv;
 
-      if (result == NS_OK && (nsnull != ref.get()) && ('\0' != *ref.get())) {
-        aHash.Assign(PRUnichar('#'));
-        aHash.Append(NS_ConvertASCIItoUCS2(ref));
-      }
-      else {
-        aHash.SetLength(0);
-      }
-    }
-  }
+  rv = SetPathnameInHrefString(href, aPathname, new_href);
+  if (NS_FAILED(rv))
+    // Ignore failures to be compatible with NS4
+    return NS_OK;
 
-  return result;
+  return SetHref(new_href);
 }
 
 NS_IMETHODIMP    
-nsHTMLAnchorElement::GetText(nsAWritableString& aText)
+nsHTMLAnchorElement::GetSearch(nsAString& aSearch)
+{
+  nsAutoString href;
+
+  nsresult rv = GetHref(href);
+  if (NS_FAILED(rv))
+    return rv;
+
+  return GetSearchFromHrefString(href, aSearch);
+}
+
+NS_IMETHODIMP
+nsHTMLAnchorElement::SetSearch(const nsAString& aSearch)
+{
+  nsAutoString href, new_href;
+  nsresult rv = GetHref(href);
+
+  if (NS_FAILED(rv))
+    return rv;
+
+  rv = SetSearchInHrefString(href, aSearch, new_href);
+  if (NS_FAILED(rv))
+    // Ignore failures to be compatible with NS4
+    return NS_OK;
+
+  return SetHref(new_href);
+}
+
+NS_IMETHODIMP    
+nsHTMLAnchorElement::GetPort(nsAString& aPort)
+{
+  nsAutoString href;
+  
+  nsresult rv = GetHref(href);
+  if (NS_FAILED(rv))
+    return rv;
+
+  return GetPortFromHrefString(href, aPort);
+}
+
+NS_IMETHODIMP
+nsHTMLAnchorElement::SetPort(const nsAString& aPort)
+{
+  nsAutoString href, new_href;
+  nsresult rv = GetHref(href);
+
+  if (NS_FAILED(rv))
+    return rv;
+
+  rv = SetPortInHrefString(href, aPort, new_href);
+  if (NS_FAILED(rv))
+    // Ignore failures to be compatible with NS4
+    return NS_OK;
+  
+  return SetHref(new_href);
+}
+
+NS_IMETHODIMP    
+nsHTMLAnchorElement::GetHash(nsAString& aHash)
+{
+  nsAutoString href;
+
+  nsresult rv = GetHref(href);
+  if (NS_FAILED(rv))
+    return rv;
+
+  return GetHashFromHrefString(href, aHash);
+}
+
+NS_IMETHODIMP
+nsHTMLAnchorElement::SetHash(const nsAString& aHash)
+{
+  nsAutoString href, new_href;
+  nsresult rv = GetHref(href);
+
+  if (NS_FAILED(rv))
+    return rv;
+
+  rv = SetHashInHrefString(href, aHash, new_href);
+  if (NS_FAILED(rv))
+    // Ignore failures to be compatible with NS4
+    return NS_OK;
+
+  return SetHref(new_href);
+}
+
+NS_IMETHODIMP    
+nsHTMLAnchorElement::GetText(nsAString& aText)
 {
   aText.Truncate();
 
-  nsCOMPtr<nsIDOMNode> child;
+  // Since this is a Netscape 4 proprietary attribute, we have to implement
+  // the same behavior. Basically it is returning the last text node of
+  // of the anchor. Returns an empty string if there is no text node.
+  // The nsIContentIterator does exactly what we want, if we start the 
+  // iteration from the end.
+  nsCOMPtr<nsIContentIterator> iter;
+  nsresult rv = NS_NewContentIterator(getter_AddRefs(iter));
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  GetFirstChild(getter_AddRefs(child));
+  // Initialize the content iterator with the children of the anchor
+  iter->Init(this);
 
-  if (child) {
-    child->GetNodeValue(aText);
+  nsCOMPtr<nsIContent> curNode;
+
+  // Position the iterator. Last() is the anchor itself, this is not what we 
+  // want. Prev() positions the iterator to the last child of the anchor,
+  // starting at the deepest level of children, just like NS4 does.
+  rv = iter->Last();
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = iter->Prev();
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  iter->CurrentNode(getter_AddRefs(curNode));
+
+  while(curNode && (NS_ENUMERATOR_FALSE == iter->IsDone())) {
+    nsCOMPtr<nsIDOMText> textNode(do_QueryInterface(curNode));
+    if(textNode) {
+      // The current node is a text node. Get its value and break the loop.
+      textNode->GetData(aText);
+      break;
+    }
+
+    rv = iter->Prev();
+    NS_ENSURE_SUCCESS(rv, rv);
+    iter->CurrentNode(getter_AddRefs(curNode));
   }
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHTMLAnchorElement::ToString(nsAWritableString& aSource)
+nsHTMLAnchorElement::ToString(nsAString& aSource)
 {
   return GetHref(aSource);
 }
@@ -678,9 +667,7 @@ nsHTMLAnchorElement::GetHrefCString(char* &aBuf)
   nsAutoString relURLSpec;
 
   if (NS_CONTENT_ATTR_HAS_VALUE ==
-      NS_STATIC_CAST(nsIContent *, this)->GetAttribute(kNameSpaceID_HTML,
-                                                       nsHTMLAtoms::href,
-                                                       relURLSpec)) {
+      GetAttr(kNameSpaceID_None, nsHTMLAtoms::href, relURLSpec)) {
     // Clean up any leading or trailing whitespace
     relURLSpec.Trim(" \t\n\r");
 
@@ -690,13 +677,15 @@ nsHTMLAnchorElement::GetHrefCString(char* &aBuf)
 
     if (baseURL) {
       // Get absolute URL.
-      NS_MakeAbsoluteURIWithCharset(&aBuf, relURLSpec, mDocument, baseURL,
+      nsCAutoString buf;
+      NS_MakeAbsoluteURIWithCharset(buf, relURLSpec, mDocument, baseURL,
                                     nsHTMLUtils::IOService,
                                     nsHTMLUtils::CharsetMgr);
+      aBuf = ToNewCString(buf);
     }
     else {
       // Absolute URL is same as relative URL.
-      aBuf = relURLSpec.ToNewUTF8String();
+      aBuf = ToNewUTF8String(relURLSpec);
     }
   }
   else {
@@ -706,3 +695,54 @@ nsHTMLAnchorElement::GetHrefCString(char* &aBuf)
 
   return NS_OK;
 }
+
+NS_IMETHODIMP
+nsHTMLAnchorElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                             const nsAString& aValue,
+                             PRBool aNotify)
+{
+  if (aName == nsHTMLAtoms::href && kNameSpaceID_None == aNameSpaceID) {
+    nsAutoString val;
+    GetHref(val);
+    if (!val.Equals(aValue)) {
+      SetLinkState(eLinkState_Unknown);
+    }
+  }
+
+  if (aName == nsHTMLAtoms::accesskey && kNameSpaceID_None == aNameSpaceID) {
+    RegUnRegAccessKey(PR_FALSE);
+  }
+
+  nsresult rv =
+      nsGenericHTMLElement::SetAttr(aNameSpaceID, aName, aValue, aNotify);
+
+  if (aName == nsHTMLAtoms::accesskey && kNameSpaceID_None == aNameSpaceID &&
+      !aValue.IsEmpty()) {
+    RegUnRegAccessKey(PR_TRUE);
+  }
+
+  return rv;
+}
+
+NS_IMETHODIMP
+nsHTMLAnchorElement::SetAttr(nsINodeInfo* aNodeInfo,
+                             const nsAString& aValue,
+                             PRBool aNotify)
+{
+  return nsGenericHTMLElement::SetAttr(aNodeInfo, aValue, aNotify);
+}
+
+NS_IMETHODIMP
+nsHTMLAnchorElement::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute, PRBool aNotify)
+{
+  if (aAttribute == nsHTMLAtoms::href && kNameSpaceID_None == aNameSpaceID) {
+    SetLinkState(eLinkState_Unknown);
+  }
+
+  if (aAttribute == nsHTMLAtoms::accesskey &&
+      kNameSpaceID_None == aNameSpaceID) {
+    RegUnRegAccessKey(PR_FALSE);
+  }
+
+  return nsGenericHTMLElement::UnsetAttr(aNameSpaceID, aAttribute, aNotify);
+}
