@@ -51,7 +51,6 @@
 #include "nsIHTMLStyleSheet.h"
 #include "nsIHTMLCSSStyleSheet.h"
 #include "nsICSSStyleRule.h"
-#include "nsIWebShell.h"
 #include "nsIContentViewer.h"
 #include "nsIMarkupDocumentViewer.h"
 #include "nsHTMLAttributes.h"
@@ -62,7 +61,6 @@
 #include "nsIFrameManager.h"
 #include "nsCOMPtr.h"
 #include "nsIStyleSet.h"
-#include "nsISizeOfHandler.h"
 #include "nsIView.h"
 #include "nsLayoutAtoms.h"
 #include "nsRuleWalker.h"
@@ -86,8 +84,6 @@ public:
 
 #ifdef DEBUG
   NS_IMETHOD List(FILE* out = stdout, PRInt32 aIndent = 0) const;
-
-  virtual void SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize);
 #endif
 
   nsHTMLBodyElement*  mPart;  // not ref-counted, cleared by content 
@@ -127,9 +123,6 @@ public:
   NS_IMETHOD WalkContentStyleRules(nsRuleWalker* aRuleWalker);
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
                                       nsChangeHint& aHint) const;
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
 
 protected:
   BodyRule* mContentStyleRule;
@@ -170,7 +163,9 @@ BodyRule::MapRuleInfoInto(nsRuleData* aData)
   PRInt32 bodyMarginWidth  = -1;
   PRInt32 bodyMarginHeight = -1;
   PRInt32 bodyTopMargin = -1;
+  PRInt32 bodyBottomMargin = -1;
   PRInt32 bodyLeftMargin = -1;
+  PRInt32 bodyRightMargin = -1;
 
   // check the mode (fortunately, the ruleData has a presContext for us to use!)
   nsCompatibility mode;
@@ -213,6 +208,16 @@ BodyRule::MapRuleInfoInto(nsRuleData* aData)
           margin->mTop.SetFloatValue((float)bodyTopMargin, eCSSUnit_Pixel);
       }
 
+      // bottommargin (IE-attribute)
+      mPart->GetHTMLAttribute(nsHTMLAtoms::bottommargin, value);
+      if (eHTMLUnit_Pixel == value.GetUnit()) {
+        bodyBottomMargin = value.GetPixelValue();
+        if (bodyBottomMargin < 0) bodyBottomMargin = 0;
+        nsCSSRect* margin = aData->mMarginData->mMargin;
+        if (margin->mBottom.GetUnit() == eCSSUnit_Null)
+          margin->mBottom.SetFloatValue((float)bodyBottomMargin, eCSSUnit_Pixel);
+      }
+
       // leftmargin (IE-attribute)
       mPart->GetHTMLAttribute(nsHTMLAtoms::leftmargin, value);
       if (eHTMLUnit_Pixel == value.GetUnit()) {
@@ -222,6 +227,16 @@ BodyRule::MapRuleInfoInto(nsRuleData* aData)
         if (margin->mLeft.GetUnit() == eCSSUnit_Null)
           margin->mLeft.SetFloatValue((float)bodyLeftMargin, eCSSUnit_Pixel);
       }
+
+      // rightmargin (IE-attribute)
+      mPart->GetHTMLAttribute(nsHTMLAtoms::rightmargin, value);
+      if (eHTMLUnit_Pixel == value.GetUnit()) {
+        bodyRightMargin = value.GetPixelValue();
+        if (bodyRightMargin < 0) bodyRightMargin = 0;
+        nsCSSRect* margin = aData->mMarginData->mMargin;
+        if (margin->mRight.GetUnit() == eCSSUnit_Null)
+          margin->mRight.SetFloatValue((float)bodyRightMargin, eCSSUnit_Pixel);
+      }
     }
 
   }
@@ -278,45 +293,6 @@ BodyRule::List(FILE* out, PRInt32 aIndent) const
 {
   return NS_OK;
 }
-
-/******************************************************************************
-* SizeOf method:
-*
-*  Self (reported as BodyRule's size): 
-*    1) sizeof(*this)
-*
-*  Contained / Aggregated data (not reported as BodyRule's size):
-*    1) delegate to mSheet if it exists
-*
-*  Children / siblings / parents:
-*    none
-*    
-******************************************************************************/
-void BodyRule::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
-{
-  NS_ASSERTION(aSizeOfHandler != nsnull, "SizeOf handler cannot be null");
-
-  // first get the unique items collection
-  UNIQUE_STYLE_ITEMS(uniqueItems);
-  if(! uniqueItems->AddItem((void*)this)){
-    return;
-  }
-
-  PRUint32 localSize=0;
-
-  // create a tag for this instance
-  nsCOMPtr<nsIAtom> tag;
-  tag = do_GetAtom("BodyRule");
-  // get the size of an empty instance and add to the sizeof handler
-  aSize = sizeof(*this);
-  aSizeOfHandler->AddSize(tag, aSize);
-
-  if(mSheet){
-    mSheet->SizeOf(aSizeOfHandler, localSize);
-  }
-
-  return;
-}
 #endif
 
 //----------------------------------------------------------------------
@@ -476,12 +452,7 @@ nsHTMLBodyElement::GetBgColor(nsAString& aBgColor)
       NS_ENSURE_SUCCESS(rv, rv);
 
       if (frame) {
-        const nsStyleBackground* StyleBackground;
-        rv = frame->GetStyleData(eStyleStruct_Background,
-                                   (const nsStyleStruct*&)StyleBackground);
-        NS_ENSURE_SUCCESS(rv, rv);
-
-        bgcolor = StyleBackground->mBackgroundColor;
+        bgcolor = frame->GetStyleBackground()->mBackgroundColor;
         nsHTMLValue(bgcolor).ToString(aBgColor);
       }
     }
@@ -522,7 +493,9 @@ nsHTMLBodyElement::StringToAttribute(nsIAtom* aAttribute,
   else if ((aAttribute == nsHTMLAtoms::marginwidth) ||
            (aAttribute == nsHTMLAtoms::marginheight) ||
            (aAttribute == nsHTMLAtoms::topmargin) ||
-           (aAttribute == nsHTMLAtoms::leftmargin)) {
+           (aAttribute == nsHTMLAtoms::bottommargin) ||
+           (aAttribute == nsHTMLAtoms::leftmargin) ||
+           (aAttribute == nsHTMLAtoms::rightmargin)) {
     if (aResult.ParseIntWithBounds(aValue, eHTMLUnit_Pixel, 0)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
@@ -654,31 +627,23 @@ NS_IMETHODIMP
 nsHTMLBodyElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
                                             nsChangeHint& aHint) const
 {
-  if ((aAttribute == nsHTMLAtoms::link) ||
-      (aAttribute == nsHTMLAtoms::vlink) ||
-      (aAttribute == nsHTMLAtoms::alink) ||
-      (aAttribute == nsHTMLAtoms::text)) {
-    aHint = NS_STYLE_HINT_VISUAL;
-  }
-  else if ((aAttribute == nsHTMLAtoms::marginwidth) ||
-           (aAttribute == nsHTMLAtoms::marginheight)) {
-    aHint = NS_STYLE_HINT_REFLOW;
-  }
-  else if (!GetCommonMappedAttributesImpact(aAttribute, aHint)) {
-    if (!GetBackgroundAttributesImpact(aAttribute, aHint)) {
-      aHint = NS_STYLE_HINT_CONTENT;
-    }
-  }
+  static const AttributeImpactEntry attributes[] = {
+    { &nsHTMLAtoms::link,  NS_STYLE_HINT_VISUAL },
+    { &nsHTMLAtoms::vlink, NS_STYLE_HINT_VISUAL },
+    { &nsHTMLAtoms::alink, NS_STYLE_HINT_VISUAL },
+    { &nsHTMLAtoms::text,  NS_STYLE_HINT_VISUAL },
+    { &nsHTMLAtoms::marginwidth, NS_STYLE_HINT_REFLOW },
+    { &nsHTMLAtoms::marginheight, NS_STYLE_HINT_REFLOW },
+    { nsnull, NS_STYLE_HINT_NONE },
+  };
+
+  static const AttributeImpactEntry* const map[] = {
+    attributes,
+    sCommonAttributeMap,
+    sBackgroundAttributeMap,
+  };
+
+  FindAttributeImpact(aAttribute, aHint, map, NS_ARRAY_LENGTH(map));
 
   return NS_OK;
 }
-
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLBodyElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
