@@ -55,7 +55,6 @@
 #include "nsIEventStateManager.h"
 #include "nsIDOMEvent.h"
 #include "nsIDOMNSEvent.h"
-#include "nsISizeOfHandler.h"
 #include "nsIDocument.h"
 #include "nsGUIEvent.h"
 #include "nsUnicharUtils.h"
@@ -111,9 +110,6 @@ public:
                             nsIDOMEvent** aDOMEvent,
                             PRUint32 aFlags,
                             nsEventStatus* aEventStatus);
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
 
 protected:
   PRInt8 mType;
@@ -524,15 +520,6 @@ nsHTMLButtonElement::HandleDOMEvent(nsIPresContext* aPresContext,
       case NS_MOUSE_LEFT_CLICK:
         {
           if (mForm) {
-            if (mType == NS_FORM_BUTTON_SUBMIT) {
-              // tell the form to forget a possible pending submission.
-              // the reason is that the script returned true (the event was
-              // ignored) so if there is a stored submission, it will miss
-              // the name/value of the submitting element, thus we need
-              // to forget it and the form element will build a new one
-              mForm->ForgetPendingSubmission();
-            }
-
             if (mType == NS_FORM_BUTTON_SUBMIT || mType == NS_FORM_BUTTON_RESET) {
               nsFormEvent event;
               event.eventStructType = NS_FORM_EVENT;
@@ -635,16 +622,6 @@ nsHTMLButtonElement::HandleDOMEvent(nsIPresContext* aPresContext,
   return ret;
 }
 
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLButtonElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
-
 nsresult
 nsHTMLButtonElement::GetDefaultValue(nsAString& aDefaultValue)
 {
