@@ -1,3 +1,4 @@
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -67,7 +68,6 @@
 #include "nsIFormControlFrame.h"
 #include "nsIFrame.h"
 #include "nsIEventStateManager.h"
-#include "nsISizeOfHandler.h"
 #include "nsIServiceManager.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsDOMError.h"
@@ -104,6 +104,10 @@
 #include "nsIFileStreams.h"
 #include "nsNetUtil.h"
 
+// input type=image
+#include "nsImageLoadingContent.h"
+#include "nsIDOMWindowInternal.h"
+
 // XXX align=left, hspace, vspace, border? other nav4 attrs
 
 static NS_DEFINE_CID(kXULControllersCID,  NS_XULCONTROLLERS_CID);
@@ -127,6 +131,7 @@ static NS_DEFINE_CID(kXULControllersCID,  NS_XULCONTROLLERS_CID);
                                         : ((bitfield) &= ~(0x01 << (field))))
 
 class nsHTMLInputElement : public nsGenericHTMLLeafFormElement,
+                           public nsImageLoadingContent,
                            public nsIDOMHTMLInputElement,
                            public nsIDOMNSHTMLInputElement,
                            public nsITextControlElement,
@@ -186,10 +191,6 @@ public:
   NS_IMETHOD SetDocument(nsIDocument* aDocument, PRBool aDeep,
                          PRBool aCompileEventHandlers);
 
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
-
   NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
                      const nsAString& aValue, PRBool aNotify) {
     BeforeSetAttr(aNameSpaceID, aName, &aValue, aNotify);
@@ -241,7 +242,7 @@ protected:
   //Helper method
 #ifdef ACCESSIBILITY
   nsresult FireEventForAccessibility(nsIPresContext* aPresContext,
-			    		                       const nsAString& aEventType);
+                                     const nsAString& aEventType);
 #endif
 
   /**
@@ -316,34 +317,6 @@ protected:
   char*                    mValue;
 };
 
-//
-// Visitor declarations
-//
-
-/**
- * This visitor sets CheckedChanged on all elements it finds.
- *
- * @param aCheckedChanged the value of CheckedChanged to set on all elements
- * @param aVisitor the visitor (out param)
- */
-NS_METHOD NS_GetRadioSetCheckedChangedVisitor(PRBool aCheckedChanged,
-                                              nsIRadioVisitor** aVisitor);
-
-/**
- * This visitor will take the boolean you're pointing at and put
- * aCheckedChanged into it.  If the visitor is never called, aCheckedChanged
- * will of course not change.
- *
- * @param aCheckedChanged the boolean to put CheckedChanged into
- * @param aExcludeElement an element to exclude--i.e. don't get checked changed
- *        on this element
- * @param aVisitor the visitor (out param)
- */
-NS_METHOD NS_GetRadioGetCheckedChangedVisitor(PRBool* aCheckedChanged,
-                                              nsIFormControl* aExcludeElement,
-                                              nsIRadioVisitor** aVisitor);
-
-
 //
 // construction, destruction
 //
@@ -406,6 +379,8 @@ NS_HTML_CONTENT_INTERFACE_MAP_BEGIN(nsHTMLInputElement,
   NS_INTERFACE_MAP_ENTRY(nsITextControlElement)
   NS_INTERFACE_MAP_ENTRY(nsIRadioControlElement)
   NS_INTERFACE_MAP_ENTRY(nsIPhonetic)
+  NS_INTERFACE_MAP_ENTRY(imgIDecoderObserver)
+  NS_INTERFACE_MAP_ENTRY(nsIImageLoadingContent)
   NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(HTMLInputElement)
 NS_HTML_CONTENT_INTERFACE_MAP_END
 
@@ -455,6 +430,10 @@ nsHTMLInputElement::BeforeSetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
       mType == NS_FORM_INPUT_RADIO &&
       (mForm || !(GET_BOOLBIT(mBitField, BF_PARSER_CREATING)))) {
     WillRemoveFromRadioGroup();
+  } else if (aName == nsHTMLAtoms::src && aNameSpaceID == kNameSpaceID_None &&
+             aValue && mType == NS_FORM_INPUT_IMAGE) {
+    // Null value means the attr got unset; don't trigger on that
+    ImageURIChanged(*aValue);
   }
 }
 
@@ -523,6 +502,14 @@ nsHTMLInputElement::AfterSetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
       }
     }
 
+    if (mType == NS_FORM_INPUT_IMAGE && !mCurrentRequest) {
+      // We just got switched to be an image input; we should see
+      // whether we have an image to load;
+      nsAutoString src;
+      GetAttr(kNameSpaceID_None, nsHTMLAtoms::src, src);
+      ImageURIChanged(src);
+    }
+
     // If the type of the input has changed we might need to change the type
     // of the size attribute.
     nsHTMLValue value;
@@ -1068,6 +1055,8 @@ nsHTMLInputElement::SetFocus(nsIPresContext* aPresContext)
   PRBool isActive = PR_FALSE;
   focusController->GetActive(&isActive);
   if (!isActive) {
+    nsCOMPtr<nsIDOMWindowInternal> domWin(do_QueryInterface(globalObj));
+    focusController->SetFocusedWindow(domWin);
     focusController->SetFocusedElement(this);
     return NS_OK;
   }
@@ -1159,6 +1148,8 @@ nsHTMLInputElement::Select()
     PRBool isActive = PR_FALSE;
     focusController->GetActive(&isActive);
     if (!isActive) {
+      nsCOMPtr<nsIDOMWindowInternal> domWin(do_QueryInterface(globalObj));
+      focusController->SetFocusedWindow(domWin);
       focusController->SetFocusedElement(this);
       SelectAll(presContext);
       return NS_OK;
@@ -1309,9 +1300,7 @@ nsHTMLInputElement::HandleDOMEvent(nsIPresContext* aPresContext,
       nsIFrame* formFrame = nsnull;
       CallQueryInterface(formControlFrame, &formFrame);
       if (formFrame) {
-        const nsStyleUserInterface* uiStyle;
-        formFrame->GetStyleData(eStyleStruct_UserInterface,
-                                (const nsStyleStruct *&)uiStyle);
+        const nsStyleUserInterface* uiStyle = formFrame->GetStyleUserInterface();
 
         if (uiStyle->mUserInput == NS_STYLE_USER_INPUT_NONE ||
             uiStyle->mUserInput == NS_STYLE_USER_INPUT_DISABLED) {
@@ -1663,7 +1652,7 @@ nsHTMLInputElement::HandleDOMEvent(nsIPresContext* aPresContext,
                   }
                 }
               }
-	      NS_ENSURE_SUCCESS(rv, rv);
+              NS_ENSURE_SUCCESS(rv, rv);
             
               nsCOMPtr<nsIPresShell> shell;
               aPresContext->GetShell(getter_AddRefs(shell));
@@ -1723,14 +1712,7 @@ nsHTMLInputElement::HandleDOMEvent(nsIPresContext* aPresContext,
         {
           if (mForm && (oldType == NS_FORM_INPUT_SUBMIT ||
                         oldType == NS_FORM_INPUT_IMAGE)) {
-            if (mType == NS_FORM_INPUT_SUBMIT || mType == NS_FORM_INPUT_IMAGE) {
-              // tell the form to forget a possible pending submission.
-              // the reason is that the script returned true (the event was
-              // ignored) so if there is a stored submission, it will miss
-              // the name/value of the submitting element, thus we need
-              // to forget it and the form element will build a new one
-              mForm->ForgetPendingSubmission();
-            } else {
+            if (mType != NS_FORM_INPUT_SUBMIT && mType != NS_FORM_INPUT_IMAGE) {
               // If the type has changed to a non-submit type, then we want to
               // flush the stored submission if there is one (as if the submit()
               // was allowed to succeed)
@@ -1869,12 +1851,12 @@ nsHTMLInputElement::StringToAttribute(nsIAtom* aAttribute,
     return NS_CONTENT_ATTR_HAS_VALUE;
   }
   else if (aAttribute == nsHTMLAtoms::width) {
-    if (aResult.ParseIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE)) {
+    if (aResult.ParseSpecialIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE, PR_FALSE)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
   else if (aAttribute == nsHTMLAtoms::height) {
-    if (aResult.ParseIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE)) {
+    if (aResult.ParseSpecialIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE, PR_FALSE)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
@@ -1988,28 +1970,27 @@ NS_IMETHODIMP
 nsHTMLInputElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
                                              nsChangeHint& aHint) const
 {
-  if (aAttribute == nsHTMLAtoms::value) {
-    if (mType == NS_FORM_INPUT_BUTTON ||
-        mType == NS_FORM_INPUT_RESET ||
-        mType == NS_FORM_INPUT_SUBMIT) {
-      aHint = NS_STYLE_HINT_CONTENT;
-    }
-    else {
-      aHint = NS_STYLE_HINT_ATTRCHANGE;
-    }
-  }
-  else if ((aAttribute == nsHTMLAtoms::align) ||
-           (aAttribute == nsHTMLAtoms::type)) {
-    aHint = NS_STYLE_HINT_FRAMECHANGE;
-  }
-  else if (!GetCommonMappedAttributesImpact(aAttribute, aHint)) {
-    if (!GetImageMappedAttributesImpact(aAttribute, aHint)) {
-      if (!GetImageBorderAttributeImpact(aAttribute, aHint)) {
-        aHint = NS_STYLE_HINT_CONTENT;
-      }
-    }
-  }
+  nsChangeHint valueHint = (mType == NS_FORM_INPUT_BUTTON ||
+                            mType == NS_FORM_INPUT_RESET ||
+                            mType == NS_FORM_INPUT_SUBMIT) ?
+    NS_STYLE_HINT_CONTENT : NS_STYLE_HINT_ATTRCHANGE;
 
+  const AttributeImpactEntry attributes[] = {
+    { &nsHTMLAtoms::value, valueHint },
+    { &nsHTMLAtoms::align, NS_STYLE_HINT_FRAMECHANGE },
+    { &nsHTMLAtoms::type, NS_STYLE_HINT_FRAMECHANGE },
+    { nsnull, NS_STYLE_HINT_NONE },
+  };
+
+  const AttributeImpactEntry* const map[] = {
+    attributes,
+    sCommonAttributeMap,
+    sImageAttributeMap,
+    sImageBorderAttributeMap,
+  };
+
+  FindAttributeImpact(aAttribute, aHint, map, NS_ARRAY_LENGTH(map));
+  
   return NS_OK;
 }
 
@@ -2022,19 +2003,6 @@ nsHTMLInputElement::GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapR
 }
 
 
-// nsIFormControl
-
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLInputElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
-
-
 // Controllers Methods
 
 NS_IMETHODIMP
@@ -2189,43 +2157,20 @@ nsresult
 nsHTMLInputElement::FireEventForAccessibility(nsIPresContext* aPresContext,
                                               const nsAString& aEventType)
 {
-  nsCOMPtr<nsIEventListenerManager> listenerManager;
-
-  nsresult rv = GetListenerManager(getter_AddRefs(listenerManager));
-  if ( !listenerManager )
-    return rv;
-
-  // Create the DOM event
-  nsCOMPtr<nsIDOMEvent> domEvent;
-  rv = listenerManager->CreateEvent(aPresContext,
-                                    nsnull, 
-                                    NS_LITERAL_STRING("MutationEvents"),
-                                    getter_AddRefs(domEvent) );
-  if ( !domEvent )
-    return NS_ERROR_FAILURE;
-
-  // Initialize the mutation event
-  nsCOMPtr<nsIDOMMutationEvent> mutEvent(do_QueryInterface(domEvent));
-  if ( !mutEvent )
-    return NS_ERROR_FAILURE;
-  nsAutoString empty;
-  mutEvent->InitMutationEvent( aEventType, PR_TRUE, PR_TRUE, nsnull, empty, empty, empty, nsIDOMMutationEvent::MODIFICATION);
-
-  // Set the target of the event to this nsHTMLInputElement, which should be checkbox content??
-  nsCOMPtr<nsIPrivateDOMEvent> privEvent(do_QueryInterface(domEvent));
-  if ( ! privEvent )
-    return NS_ERROR_FAILURE;
-  nsCOMPtr<nsIDOMEventTarget> targ(do_QueryInterface(NS_STATIC_CAST(nsIDOMHTMLInputElement *, this)));
-  if ( ! targ )
-    return NS_ERROR_FAILURE;
-  privEvent->SetTarget(targ);
-
-  // Dispatch the event
-  nsCOMPtr<nsIDOMEventReceiver> eventReceiver(do_QueryInterface(listenerManager));
-  if ( ! eventReceiver )
-    return NS_ERROR_FAILURE;
-  PRBool noDefault;
-  eventReceiver->DispatchEvent(domEvent, &noDefault);
+  nsCOMPtr<nsIDOMEvent> event;
+  nsCOMPtr<nsIEventListenerManager> manager;
+  GetListenerManager(getter_AddRefs(manager));
+  if (manager &&
+      NS_SUCCEEDED(manager->CreateEvent(aPresContext, nsnull, NS_LITERAL_STRING("Events"), getter_AddRefs(event)))) {
+    event->InitEvent(aEventType, PR_TRUE, PR_TRUE);
+    PRBool noDefault;
+    nsCOMPtr<nsIEventStateManager> esm;
+    aPresContext->GetEventStateManager(getter_AddRefs(esm));
+    if (esm) {
+      nsCOMPtr<nsISupports> target(do_QueryInterface(NS_STATIC_CAST(nsIDOMHTMLInputElement *, this)));
+      esm->DispatchNewEvent(target, event, &noDefault);
+    }
+  }
 
   return NS_OK;
 }
@@ -2856,7 +2801,7 @@ protected:
   nsIFormControl* mExcludeElement;
 };
 
-NS_METHOD
+nsresult
 NS_GetRadioSetCheckedChangedVisitor(PRBool aCheckedChanged,
                                     nsIRadioVisitor** aVisitor)
 {
@@ -2900,7 +2845,7 @@ NS_GetRadioSetCheckedChangedVisitor(PRBool aCheckedChanged,
   return NS_OK;
 }
 
-NS_METHOD
+nsresult
 NS_GetRadioGetCheckedChangedVisitor(PRBool* aCheckedChanged,
                                     nsIFormControl* aExcludeElement,
                                     nsIRadioVisitor** aVisitor)
