@@ -37,6 +37,7 @@
  * ***** END LICENSE BLOCK ***** */
 #include "nsIDOMHTMLObjectElement.h"
 #include "nsGenericHTMLElement.h"
+#include "nsImageLoadingContent.h"
 #include "nsHTMLAtoms.h"
 #include "nsStyleConsts.h"
 #include "nsDOMError.h"
@@ -48,6 +49,7 @@
 #include "nsIFormControl.h"
 
 class nsHTMLObjectElement : public nsGenericHTMLContainerFormElement,
+                            public nsImageLoadingContent,
                             public nsIDOMHTMLObjectElement
 {
 public:
@@ -86,9 +88,6 @@ public:
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
                                       nsChangeHint& aHint) const;
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
 };
 
 nsresult
@@ -134,6 +133,8 @@ NS_IMPL_RELEASE_INHERITED(nsHTMLObjectElement, nsGenericElement)
 NS_HTML_CONTENT_INTERFACE_MAP_BEGIN(nsHTMLObjectElement,
                                     nsGenericHTMLContainerFormElement)
   NS_INTERFACE_MAP_ENTRY(nsIDOMHTMLObjectElement)
+  NS_INTERFACE_MAP_ENTRY(imgIDecoderObserver)
+  NS_INTERFACE_MAP_ENTRY(nsIImageLoadingContent)
   NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(HTMLObjectElement)
 NS_HTML_CONTENT_INTERFACE_MAP_END
 
@@ -239,12 +240,6 @@ nsHTMLObjectElement::GetContentDocument(nsIDOMDocument** aContentDocument)
   return CallQueryInterface(sub_doc, aContentDocument);
 }
 
-NS_IMETHODIMP
-nsHTMLObjectElement::SetContentDocument(nsIDOMDocument* aContentDocument)
-{
-  return NS_ERROR_DOM_INVALID_MODIFICATION_ERR;
-}
-
 NS_IMETHODIMP
 nsHTMLObjectElement::StringToAttribute(nsIAtom* aAttribute,
                                        const nsAString& aValue,
@@ -324,13 +319,3 @@ nsHTMLObjectElement::GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMap
   aMapRuleFunc = &MapAttributesIntoRule;
   return NS_OK;
 }
-
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLObjectElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
