@@ -37,6 +37,7 @@
  *
  * ***** END LICENSE BLOCK ***** */
 #include "nsIDOMHTMLOptionElement.h"
+#include "nsIDOMNSHTMLOptionElement.h"
 #include "nsIOptionElement.h"
 #include "nsIDOMHTMLOptGroupElement.h"
 #include "nsIDOMHTMLFormElement.h"
@@ -75,6 +76,7 @@
  */
 class nsHTMLOptionElement : public nsGenericHTMLContainerElement,
                             public nsIDOMHTMLOptionElement,
+                            public nsIDOMNSHTMLOptionElement,
                             public nsIJSNativeInitializer,
                             public nsIOptionElement
 {
@@ -97,6 +99,9 @@ public:
   // nsIDOMHTMLOptionElement
   NS_DECL_NSIDOMHTMLOPTIONELEMENT
 
+  // nsIDOMNSHTMLOptionElement
+  NS_IMETHOD SetText(const nsAString & aText); 
+
   // nsIJSNativeInitializer
   NS_IMETHOD Initialize(JSContext* aContext, JSObject *aObj, 
                         PRUint32 argc, jsval *argv);
@@ -104,11 +109,9 @@ public:
   NS_IMETHOD StringToAttribute(nsIAtom* aAttribute,
                                const nsAString& aValue,
                                nsHTMLValue& aResult);
-  NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
+  NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute,
+                                      PRInt32 aModType,
                                       nsChangeHint& aHint) const;
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
 
   // nsIOptionElement
   NS_IMETHOD SetSelectedInternal(PRBool aValue, PRBool aNotify);
@@ -218,6 +221,7 @@ NS_IMPL_RELEASE_INHERITED(nsHTMLOptionElement, nsGenericElement);
 NS_HTML_CONTENT_INTERFACE_MAP_BEGIN(nsHTMLOptionElement,
                                     nsGenericHTMLContainerElement)
   NS_INTERFACE_MAP_ENTRY(nsIDOMHTMLOptionElement)
+  NS_INTERFACE_MAP_ENTRY(nsIDOMNSHTMLOptionElement)
   NS_INTERFACE_MAP_ENTRY(nsIJSNativeInitializer)
   NS_INTERFACE_MAP_ENTRY(nsIOptionElement)
   NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(HTMLOptionElement)
@@ -444,14 +448,12 @@ nsHTMLOptionElement::GetIndex(PRInt32* aIndex)
 
   if (selectElement) {
     // Get the options from the select object.
-    nsCOMPtr<nsIDOMHTMLCollection> options;
-
+    nsCOMPtr<nsIDOMHTMLOptionsCollection> options;
     selectElement->GetOptions(getter_AddRefs(options));
 
     if (options) {
       // Walk the options to find out where we are in the list (ick, O(n))
       PRUint32 length = 0;
-
       options->GetLength(&length);
 
       nsCOMPtr<nsIDOMNode> thisOption;
@@ -495,14 +497,21 @@ nsHTMLOptionElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32
   nsIFormControlFrame* fcFrame = GetSelectFrame();
   
   if (fcFrame) {    
-    if (aAttribute == nsHTMLAtoms::label) {
-      aHint = NS_STYLE_HINT_REFLOW; 
-    } else if (aAttribute == nsHTMLAtoms::text) {
-      aHint = NS_STYLE_HINT_REFLOW; 
-    } else if (!GetCommonMappedAttributesImpact(aAttribute, aHint)) {
-      aHint = NS_STYLE_HINT_CONTENT;
-    }
+    static const AttributeImpactEntry attributes[] = {
+      { &nsHTMLAtoms::label, NS_STYLE_HINT_REFLOW },
+      { &nsHTMLAtoms::text, NS_STYLE_HINT_REFLOW },
+      { nsnull, NS_STYLE_HINT_NONE }
+    };
+
+    static const AttributeImpactEntry* const map[] = {
+      attributes,
+      sCommonAttributeMap,
+    };
+
+    FindAttributeImpact(aAttribute, aHint, map, NS_ARRAY_LENGTH(map));
+    
   } else {
+    // XXX don't we want to try common attributes here?
     if (aAttribute == nsXULAtoms::menuactive) {
       aHint = NS_STYLE_HINT_CONTENT;
     } else {
@@ -800,13 +809,3 @@ nsHTMLOptionElement::Initialize(JSContext* aContext,
 
   return result;
 }
-
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLOptionElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
