@@ -44,7 +44,7 @@
 #include "nsIPresContext.h"
 #include "nsHTMLAttributes.h"
 #include "nsRuleNode.h"
-#include "nsCSSDeclaration.h"
+#include "nsCSSStruct.h"
 
 // XXX wrap, variable, cols, tabstop
 
@@ -78,9 +78,6 @@ public:
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
                                       nsChangeHint& aHint) const;
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
 };
 
 nsresult
@@ -258,16 +255,21 @@ NS_IMETHODIMP
 nsHTMLPreElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
                                            nsChangeHint& aHint) const
 {
-  if ((aAttribute == nsHTMLAtoms::variable) || 
-      (aAttribute == nsHTMLAtoms::wrap) ||
-      (aAttribute == nsHTMLAtoms::cols) ||
-      (aAttribute == nsHTMLAtoms::width) ||
-      (aAttribute == nsHTMLAtoms::tabstop)) {
-    aHint = NS_STYLE_HINT_REFLOW;
-  }
-  else if (!GetCommonMappedAttributesImpact(aAttribute, aHint)) {
-    aHint = NS_STYLE_HINT_CONTENT;
-  }
+  static const AttributeImpactEntry attributes[] = {
+    { &nsHTMLAtoms::variable, NS_STYLE_HINT_REFLOW},
+    { &nsHTMLAtoms::wrap, NS_STYLE_HINT_REFLOW},
+    { &nsHTMLAtoms::cols, NS_STYLE_HINT_REFLOW},
+    { &nsHTMLAtoms::width, NS_STYLE_HINT_REFLOW},
+    { &nsHTMLAtoms::tabstop, NS_STYLE_HINT_REFLOW},
+    { nsnull, NS_STYLE_HINT_NONE },
+  };
+  
+  static const AttributeImpactEntry* const map[] = {
+    attributes,
+    sCommonAttributeMap,
+  };
+
+  FindAttributeImpact(aAttribute, aHint, map, NS_ARRAY_LENGTH(map));
 
   return NS_OK;
 }
@@ -280,13 +282,3 @@ nsHTMLPreElement::GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRul
   aMapRuleFunc = &MapAttributesIntoRule;
   return NS_OK;
 }
-
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLPreElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
