@@ -59,6 +59,7 @@
 #include "nsGenericDOMHTMLCollection.h"
 #include "nsISelectElement.h"
 #include "nsISelectControlFrame.h"
+#include "nsIDOMHTMLOptionsCollection.h"
 #include "nsIDOMNSHTMLOptionCollectn.h"
 #include "nsGUIEvent.h"
 #include "nsIBoxObject.h"
@@ -90,7 +91,8 @@ class nsHTMLSelectElement;
  * The collection of options in the select (what you get back when you do
  * select.options in DOM)
  */
-class nsHTMLOptionCollection: public nsIDOMNSHTMLOptionCollection,
+class nsHTMLOptionCollection: public nsIDOMHTMLOptionsCollection,
+                              public nsIDOMNSHTMLOptionCollection,
                               public nsGenericDOMHTMLCollection
 {
 public:
@@ -99,17 +101,19 @@ public:
 
   NS_DECL_ISUPPORTS_INHERITED
 
+  // nsIDOMHTMLOptionsCollection interface
+  NS_DECL_NSIDOMHTMLOPTIONSCOLLECTION
+
   // nsIDOMNSHTMLOptionCollection interface, can't use the macro
   // NS_DECL_NSIDOMNSHTMLOPTIONCOLLECTION here since GetLength() is
   // defined in more than one interface
-  NS_IMETHOD SetLength(PRUint32 aLength);
   NS_IMETHOD GetSelectedIndex(PRInt32* aSelectedIndex);
   NS_IMETHOD SetSelectedIndex(PRInt32 aSelectedIndex);
   NS_IMETHOD SetOption(PRInt32 aIndex, nsIDOMHTMLOptionElement* aOption);
   NS_IMETHOD Add(nsIDOMHTMLOptionElement* aOption);
 
-  // nsIDOMHTMLCollection interface
-  NS_DECL_NSIDOMHTMLCOLLECTION
+  // nsIDOMHTMLCollection interface, all its methods are defined in
+  // nsIDOMHTMLOptionsCollection
 
   // Helpers for nsHTMLSelectElement
   /**
@@ -235,9 +239,6 @@ public:
                             PRUint32 aFlags,
                             nsEventStatus* aEventStatus);
 
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
   NS_IMETHOD SetFocus(nsIPresContext* aPresContext);
   NS_IMETHOD RemoveFocus(nsIPresContext* aPresContext);   
 
@@ -276,13 +277,15 @@ protected:
   void FindSelectedIndex(PRInt32 aStartIndex);
   /**
    * Select some option if possible (generally the first non-disabled option).
+   * @return true if something was selected, false otherwise
    */
-  void SelectSomething();
+  PRBool SelectSomething();
   /**
    * Call SelectSomething(), but only if nothing is selected
    * @see SelectSomething()
+   * @return true if something was selected, false otherwise
    */
-  void CheckSelectSomething();
+  PRBool CheckSelectSomething();
   /**
    * Called to trigger notifications of frames and fixing selected index
    *
@@ -784,7 +787,8 @@ nsresult
 nsHTMLSelectElement::RemoveOptionsFromListRecurse(nsIContent* aOptions,
                                                   PRInt32 aRemoveIndex,
                                                   PRInt32* aNumRemoved,
-                                                  PRInt32 aDepth) {
+                                                  PRInt32 aDepth)
+{
   // We *assume* here that someone's brain has not gone horribly
   // wrong by putting <option> inside of <option>.  I'm sorry, I'm
   // just not going to look for an option inside of an option.
@@ -1082,7 +1086,7 @@ nsHTMLSelectElement::Remove(PRInt32 aIndex)
 }
 
 NS_IMETHODIMP
-nsHTMLSelectElement::GetOptions(nsIDOMHTMLCollection** aValue)
+nsHTMLSelectElement::GetOptions(nsIDOMHTMLOptionsCollection** aValue)
 {
   *aValue = mOptions;
   NS_IF_ADDREF(*aValue);
@@ -1487,9 +1491,9 @@ nsHTMLSelectElement::SetOptionsSelectedByIndex(PRInt32 aStartIndex,
     }
   }
 
-  // Make sure something is selected
-  if (optionsDeselected) {
-    CheckSelectSomething();
+  // Make sure something is selected unless we were set to -1 (none)
+  if (optionsDeselected && aStartIndex != -1) {
+    optionsSelected = CheckSelectSomething() || optionsSelected;
   }
 
   // Let the caller know whether anything was changed
@@ -1757,7 +1761,7 @@ nsHTMLSelectElement::NamedItem(const nsAString& aName,
   return mOptions->NamedItem(aName, aReturn);
 }
 
-void
+PRBool
 nsHTMLSelectElement::CheckSelectSomething()
 {
   if (mIsDoneAddingChildren) {
@@ -1766,17 +1770,18 @@ nsHTMLSelectElement::CheckSelectSomething()
     PRBool isMultiple;
     GetMultiple(&isMultiple);
     if (mSelectedIndex < 0 && !isMultiple && size <= 1) {
-      SelectSomething();
+      return SelectSomething();
     }
   }
+  return PR_FALSE;
 }
 
-void
+PRBool
 nsHTMLSelectElement::SelectSomething()
 {
   // If we're not done building the select, don't play with this yet.
   if (!mIsDoneAddingChildren) {
-    return;
+    return PR_FALSE;
   }
 
   PRUint32 count;
@@ -1786,10 +1791,13 @@ nsHTMLSelectElement::SelectSomething()
     nsresult rv = IsOptionDisabled(i, &disabled);
 
     if (NS_FAILED(rv) || !disabled) {
-      SetSelectedIndex(i);
-      break;
+      rv = SetSelectedIndex(i);
+      NS_ENSURE_SUCCESS(rv, PR_FALSE);
+      return PR_TRUE;
     }
   }
+
+  return PR_FALSE;
 }
 
 NS_IMETHODIMP
@@ -2226,10 +2234,11 @@ nsHTMLOptionCollection::DropReference()
 
 // QueryInterface implementation for nsHTMLOptionCollection
 NS_INTERFACE_MAP_BEGIN(nsHTMLOptionCollection)
+  NS_INTERFACE_MAP_ENTRY(nsIDOMHTMLOptionsCollection)
   NS_INTERFACE_MAP_ENTRY(nsIDOMNSHTMLOptionCollection)
   NS_INTERFACE_MAP_ENTRY(nsIDOMHTMLCollection)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIDOMNSHTMLOptionCollection)
-  NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(HTMLOptionCollection)
+  NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(HTMLOptionsCollection)
 NS_INTERFACE_MAP_END
 
 
@@ -2416,13 +2425,3 @@ nsHTMLOptionCollection::IndexOf(nsIContent* aOption)
 
   return mElements->IndexOf(aOption);
 }
-
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLSelectElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
