@@ -65,10 +65,6 @@ public:
 
   nsresult GetInnerHTML(nsAString& aInnerHTML);
   nsresult SetInnerHTML(const nsAString& aInnerHTML);
-
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
 };
 
 nsresult
@@ -169,13 +165,3 @@ nsHTMLSpanElement::SetInnerHTML(const nsAString& aInnerHTML)
 
   return nsGenericHTMLContainerElement::SetInnerHTML(aInnerHTML);
 }
-
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLSpanElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
