@@ -50,6 +50,7 @@
 #include "nsIPresContext.h"
 #include "nsHTMLParts.h"
 #include "nsRuleNode.h"
+#include "nsStyleContext.h"
 
 /* for collections */
 #include "nsIDOMElement.h"
@@ -90,9 +91,6 @@ public:
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
                                       nsChangeHint& aHint) const;
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
 
 protected:
   GenericElementCollection *mTBodies;
@@ -118,12 +116,6 @@ public:
 
   NS_IMETHOD    ParentDestroyed();
 
-#ifdef DEBUG
-  void SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const {
-    *aResult = sizeof(*this);
-  }
-#endif
-
 protected:
   nsHTMLTableElement * mParent;
 };
@@ -977,9 +969,9 @@ nsHTMLTableElement::StringToAttribute(nsIAtom* aAttribute,
 {
   /* ignore summary, just a string */
   /* attributes that resolve to pixels, with min=0 */
-  if ((aAttribute == nsHTMLAtoms::cellspacing) ||
-      (aAttribute == nsHTMLAtoms::cellpadding)) {
-    if (aResult.ParseIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE)) {
+  if (aAttribute == nsHTMLAtoms::cellspacing ||
+      aAttribute == nsHTMLAtoms::cellpadding) {
+    if (aResult.ParseSpecialIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE, PR_FALSE)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
@@ -1006,14 +998,14 @@ nsHTMLTableElement::StringToAttribute(nsIAtom* aAttribute,
   else if (aAttribute == nsHTMLAtoms::height) {
     /* attributes that resolve to integers or percents */
 
-    if (aResult.ParseIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE)) {
+    if (aResult.ParseSpecialIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE, PR_FALSE)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
   else if (aAttribute == nsHTMLAtoms::width) {
     /* attributes that resolve to integers or percents or proportions */
 
-    if (aResult.ParseIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE)) {
+    if (aResult.ParseSpecialIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE, PR_FALSE)) {
       // treat 0 width as auto
       nsHTMLUnit unit = aResult.GetUnit();
       if ((eHTMLUnit_Pixel == unit) && (0 == aResult.GetPixelValue())) {
@@ -1035,12 +1027,8 @@ nsHTMLTableElement::StringToAttribute(nsIAtom* aAttribute,
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
-  else if (aAttribute == nsHTMLAtoms::bgcolor) {
-    if (aResult.ParseColor(aValue, mDocument)) {
-      return NS_CONTENT_ATTR_HAS_VALUE;
-    }
-  }
-  else if (aAttribute == nsHTMLAtoms::bordercolor) {
+  else if (aAttribute == nsHTMLAtoms::bgcolor ||
+           aAttribute == nsHTMLAtoms::bordercolor) {
     if (aResult.ParseColor(aValue, mDocument)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
@@ -1060,12 +1048,8 @@ nsHTMLTableElement::StringToAttribute(nsIAtom* aAttribute,
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
-  else if (aAttribute == nsHTMLAtoms::hspace) {
-    if (aResult.ParseIntWithBounds(aValue, eHTMLUnit_Pixel, 0)) {
-      return NS_CONTENT_ATTR_HAS_VALUE;
-    }
-  }
-  else if (aAttribute == nsHTMLAtoms::vspace) {
+  else if (aAttribute == nsHTMLAtoms::hspace ||
+           aAttribute == nsHTMLAtoms::vspace) {
     if (aResult.ParseIntWithBounds(aValue, eHTMLUnit_Pixel, 0)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
@@ -1557,13 +1541,3 @@ nsHTMLTableElement::GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapR
   aMapRuleFunc = &MapAttributesIntoRule;
   return NS_OK;
 }
-
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLTableElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
