@@ -57,7 +57,6 @@
 #include "nsIFormControlFrame.h"
 #include "nsITextControlFrame.h"
 #include "nsIEventStateManager.h"
-#include "nsISizeOfHandler.h"
 #include "nsLinebreakConverter.h"
 #include "nsIDocument.h"
 #include "nsIPresShell.h"
@@ -141,16 +140,14 @@ public:
   nsresult GetInnerHTML(nsAString& aInnerHTML);
   nsresult SetInnerHTML(const nsAString& aInnerHTML);
 
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
-
 protected:
   nsCOMPtr<nsIControllers> mControllers;
   /** The current value.  This is null if the frame owns the value. */
   char*                    mValue;
   /** Whether or not the value has changed since its default value was given. */
   PRPackedBool             mValueChanged;
+  /** Whether or not we are already handling select event. */
+  PRPackedBool             mHandlingSelect;
 
   NS_IMETHOD SelectAll(nsIPresContext* aPresContext);
   /**
@@ -195,13 +192,17 @@ NS_NewHTMLTextAreaElement(nsIHTMLContent** aInstancePtrResult,
 
 
 nsHTMLTextAreaElement::nsHTMLTextAreaElement()
+  : mValue(nsnull),
+    mValueChanged(PR_FALSE),
+    mHandlingSelect(PR_FALSE)
 {
-  mValue = nsnull;
-  mValueChanged = PR_FALSE;
 }
 
 nsHTMLTextAreaElement::~nsHTMLTextAreaElement()
 {
+  if (mValue) {
+    nsMemory::Free(mValue);
+  }
 }
 
 
@@ -637,15 +638,19 @@ nsHTMLTextAreaElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt
   // rows and cols and why the AttributeChanged method in
   // nsTextControlFrame does take care of the entire problem, but
   // it doesn't and this makes things better
-  if (aAttribute == nsHTMLAtoms::align ||
-      aAttribute == nsHTMLAtoms::rows ||
-      aAttribute == nsHTMLAtoms::cols) {
-    aHint = NS_STYLE_HINT_REFLOW;
-  }
-  else if (!GetCommonMappedAttributesImpact(aAttribute, aHint)) {
-    aHint = NS_STYLE_HINT_CONTENT;
-  }
+  static const AttributeImpactEntry attributes[] = {
+    { &nsHTMLAtoms::align, NS_STYLE_HINT_REFLOW },
+    { &nsHTMLAtoms::rows, NS_STYLE_HINT_REFLOW },
+    { &nsHTMLAtoms::cols, NS_STYLE_HINT_REFLOW },
+    { nsnull, NS_STYLE_HINT_NONE }
+  };
 
+  static const AttributeImpactEntry* const map[] = {
+    attributes,
+    sCommonAttributeMap,
+  };
+
+  FindAttributeImpact(aAttribute, aHint, map, NS_ARRAY_LENGTH(map));
   return NS_OK;
 }
 
@@ -677,9 +682,7 @@ nsHTMLTextAreaElement::HandleDOMEvent(nsIPresContext* aPresContext,
   if (formControlFrame &&
       NS_SUCCEEDED(CallQueryInterface(formControlFrame, &formFrame)) &&
       formFrame) {
-    const nsStyleUserInterface* uiStyle;
-    formFrame->GetStyleData(eStyleStruct_UserInterface,
-                            (const nsStyleStruct *&)uiStyle);
+    const nsStyleUserInterface* uiStyle = formFrame->GetStyleUserInterface();
 
     if (uiStyle->mUserInput == NS_STYLE_USER_INPUT_NONE ||
         uiStyle->mUserInput == NS_STYLE_USER_INPUT_DISABLED) {
@@ -687,6 +690,13 @@ nsHTMLTextAreaElement::HandleDOMEvent(nsIPresContext* aPresContext,
     }
   }
 
+  PRBool isSelectEvent = (aEvent->message == NS_FORM_SELECTED);
+  // Don't dispatch a second select event if we are already handling
+  // one.
+  if (isSelectEvent && mHandlingSelect) {
+    return NS_OK;
+  }
+
   // We have anonymous content underneath
   // that we need to hide.  We need to set the event target now
   // to ourselves
@@ -737,10 +747,18 @@ nsHTMLTextAreaElement::HandleDOMEvent(nsIPresContext* aPresContext,
     aEvent->flags &= ~NS_EVENT_FLAG_NO_CONTENT_DISPATCH;
   }
 
+  if (isSelectEvent) {
+    mHandlingSelect = PR_TRUE;
+  }
+
   rv = nsGenericHTMLContainerFormElement::HandleDOMEvent(aPresContext, aEvent,
                                                          aDOMEvent,
                                                          aFlags, aEventStatus);
 
+  if (isSelectEvent) {
+    mHandlingSelect = PR_FALSE;
+  }
+
   // Reset the flag for other content besides this text field
   aEvent->flags |= noContentDispatch ? NS_EVENT_FLAG_NO_CONTENT_DISPATCH : NS_EVENT_FLAG_NONE;
 
@@ -785,19 +803,6 @@ nsHTMLTextAreaElement::SetInnerHTML(const nsAString& aInnerHTML)
   return ReplaceContentsWithText(aInnerHTML, PR_TRUE);
 }
 
-
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLTextAreaElement::SizeOf(nsISizeOfHandler* aSizer,
-                              PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
-
-
 // Controllers Methods
 
 NS_IMETHODIMP
