@@ -41,6 +41,7 @@
 #include "nsRect.h"
 #include "nsHTMLDocument.h"
 #include "nsIImageDocument.h"
+#include "nsIImageLoadingContent.h"
 #include "nsGenericHTMLElement.h"
 #include "nsIDOMHTMLImageElement.h"
 #include "nsIDOMEvent.h"
@@ -51,22 +52,18 @@
 #include "imgILoader.h"
 #include "imgIContainer.h"
 #include "imgIDecoderObserver.h"
-#include "nsIURL.h"
-#include "nsIScrollable.h"
 #include "nsIPresShell.h"
 #include "nsIPresContext.h"
-#include "nsIStyleContext.h"
-#include "nsIViewManager.h"
-#include "nsIStringBundle.h"
+#include "nsStyleContext.h"
 #include "nsIPrefService.h"
-#include "nsITextToSubURI.h"
+#include "nsAutoPtr.h"
+#include "nsMediaDocument.h"
 
-#define NSIMAGEDOCUMENT_PROPERTIES_URI "chrome://communicator/locale/layout/ImageDocument.properties"
 #define AUTOMATIC_IMAGE_RESIZING_PREF "browser.enable_automatic_image_resizing"
 
 class nsImageDocument;
 
-class ImageListener: public nsIStreamListener
+class ImageListener: public nsMediaDocumentStreamListener
 {
 public:
   ImageListener(nsImageDocument* aDocument);
@@ -75,14 +72,9 @@ public:
   NS_DECL_ISUPPORTS
 
   NS_DECL_NSIREQUESTOBSERVER
-
-  NS_DECL_NSISTREAMLISTENER
-
-  nsImageDocument*              mDocument;
-  nsCOMPtr<nsIStreamListener>   mNextStream;
 };
 
-class nsImageDocument : public nsHTMLDocument,
+class nsImageDocument : public nsMediaDocument,
                         public nsIImageDocument,
                         public imgIDecoderObserver,
                         public nsIDOMEventListener
@@ -93,8 +85,7 @@ public:
 
   NS_DECL_ISUPPORTS
 
-  // nsIHTMLDocument
-  nsresult Init();
+  virtual nsresult Init();
 
   NS_IMETHOD StartDocumentLoad(const char*         aCommand,
                                nsIChannel*         aChannel,
@@ -121,108 +112,95 @@ protected:
 
   nsresult CheckOverflowing();
 
-  nsresult StartLayout();
+  void UpdateTitleAndCharset();
 
-  nsresult UpdateTitle();
+  float GetRatio() {
+    return PR_MIN((float)mVisibleWidth / mImageWidth,
+                  (float)mVisibleHeight / mImageHeight);
+  }
 
-  nsCOMPtr<nsIStringBundle>     mStringBundle;
   nsCOMPtr<nsIDOMElement>       mImageElement;
-  nsCOMPtr<imgIRequest>         mImageRequest;
 
-  nscoord                       mVisibleWidth;
-  nscoord                       mVisibleHeight;
-  nscoord                       mImageWidth;
-  nscoord                       mImageHeight;
+  PRInt32                       mVisibleWidth;
+  PRInt32                       mVisibleHeight;
+  PRInt32                       mImageWidth;
+  PRInt32                       mImageHeight;
 
   PRPackedBool                  mImageResizingEnabled;
   PRPackedBool                  mImageIsOverflowing;
   PRPackedBool                  mImageIsResized;
 };
 
+NS_IMPL_ADDREF_INHERITED(ImageListener, nsMediaDocumentStreamListener)
+NS_IMPL_RELEASE_INHERITED(ImageListener, nsMediaDocumentStreamListener)
+
+NS_INTERFACE_MAP_BEGIN(ImageListener)
+NS_INTERFACE_MAP_END_INHERITING(nsMediaDocumentStreamListener)
 
 ImageListener::ImageListener(nsImageDocument* aDocument)
+  : nsMediaDocumentStreamListener(aDocument)
 {
-  NS_ADDREF(mDocument = aDocument);
 }
 
+
 ImageListener::~ImageListener()
 {
-  NS_RELEASE(mDocument);
-}
-
-NS_IMPL_THREADSAFE_ISUPPORTS2(ImageListener,
-                              nsIRequestObserver,
-                              nsIStreamListener)
+};
 
 NS_IMETHODIMP
 ImageListener::OnStartRequest(nsIRequest* request, nsISupports *ctxt)
 {
+  NS_ENSURE_TRUE(mDocument, NS_ERROR_FAILURE);
+
+  nsImageDocument *imgDoc = (nsImageDocument*)mDocument.get();
   nsCOMPtr<nsIChannel> channel = do_QueryInterface(request);
   if (!channel) {
     return NS_ERROR_FAILURE;
   }
 
-  nsCOMPtr<nsIPresShell> shell;
-  nsCOMPtr<nsIPresContext> context;
-  mDocument->GetShellAt(0, getter_AddRefs(shell));
-  if (shell) {
-    shell->GetPresContext(getter_AddRefs(context));
-  }
+  nsCOMPtr<nsIImageLoadingContent> imageLoader = do_QueryInterface(imgDoc->mImageElement);
+  NS_ENSURE_TRUE(imageLoader, NS_ERROR_UNEXPECTED);
 
-  nsCOMPtr<imgILoader> il(do_GetService("@mozilla.org/image/loader;1"));
-  il->LoadImageWithChannel(channel, mDocument, context, getter_AddRefs(mNextStream), 
-                           getter_AddRefs(mDocument->mImageRequest));
+  imageLoader->AddObserver(imgDoc);
+  imageLoader->LoadImageWithChannel(channel, getter_AddRefs(mNextStream));
 
-  mDocument->StartLayout();
-
-  if (mNextStream) {
-    return mNextStream->OnStartRequest(request, ctxt);
-  }
-
-  return NS_OK;
+  return nsMediaDocumentStreamListener::OnStartRequest(request, ctxt);
 }
 
 NS_IMETHODIMP
 ImageListener::OnStopRequest(nsIRequest* request, nsISupports *ctxt,
                              nsresult status)
 {
-  mDocument->UpdateTitle();
+  NS_ENSURE_TRUE(mDocument, NS_ERROR_FAILURE);
+  nsImageDocument *imgDoc = (nsImageDocument*)mDocument.get();
+  imgDoc->UpdateTitleAndCharset();
+  
+  nsCOMPtr<nsIImageLoadingContent> imageLoader = do_QueryInterface(imgDoc->mImageElement);
+  NS_ENSURE_TRUE(imageLoader, NS_ERROR_UNEXPECTED);
 
-  if (mNextStream) {
-    return mNextStream->OnStopRequest(request, ctxt, status);
-  }
+  imageLoader->RemoveObserver(imgDoc);
 
-  return NS_OK;
+  return nsMediaDocumentStreamListener::OnStopRequest(request, ctxt, status);
 }
 
-NS_IMETHODIMP
-ImageListener::OnDataAvailable(nsIRequest* request, nsISupports *ctxt,
-                               nsIInputStream *inStr, PRUint32 sourceOffset, PRUint32 count)
-{
-  if (mNextStream) {
-    return mNextStream->OnDataAvailable(request, ctxt, inStr, sourceOffset, count);
-  }
 
-  return NS_OK;
-}
+  // NOTE! nsDocument::operator new() zeroes out all members, so don't
+  // bother initializing members to 0.
 
 nsImageDocument::nsImageDocument()
-  : mVisibleWidth(0),
-    mVisibleHeight(0),
-    mImageWidth(0),
-    mImageHeight(0),
-    mImageResizingEnabled(PR_FALSE),
-    mImageIsOverflowing(PR_FALSE),
-    mImageIsResized(PR_FALSE)
 {
+
+  // NOTE! nsDocument::operator new() zeroes out all members, so don't
+  // bother initializing members to 0.
+
 }
 
 nsImageDocument::~nsImageDocument()
 {
 }
 
-NS_IMPL_ADDREF_INHERITED(nsImageDocument, nsHTMLDocument)
-NS_IMPL_RELEASE_INHERITED(nsImageDocument, nsHTMLDocument)
+NS_IMPL_ADDREF_INHERITED(nsImageDocument, nsMediaDocument)
+NS_IMPL_RELEASE_INHERITED(nsImageDocument, nsMediaDocument)
 
 NS_INTERFACE_MAP_BEGIN(nsImageDocument)
   NS_INTERFACE_MAP_ENTRY(nsIImageDocument)
@@ -230,13 +208,13 @@ NS_INTERFACE_MAP_BEGIN(nsImageDocument)
   NS_INTERFACE_MAP_ENTRY(imgIContainerObserver)
   NS_INTERFACE_MAP_ENTRY(nsIDOMEventListener)
   NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(ImageDocument)
-NS_INTERFACE_MAP_END_INHERITING(nsHTMLDocument)
+NS_INTERFACE_MAP_END_INHERITING(nsMediaDocument)
 
 
 nsresult
 nsImageDocument::Init()
 {
-  nsresult rv = nsHTMLDocument::Init();
+  nsresult rv = nsMediaDocument::Init();
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
@@ -246,14 +224,6 @@ nsImageDocument::Init()
     mImageResizingEnabled = temp;
   }
 
-  // Create a bundle for the localization
-  nsCOMPtr<nsIStringBundleService> stringService(
-    do_GetService(NS_STRINGBUNDLE_CONTRACTID));
-  if (stringService) {
-    stringService->CreateBundle(NSIMAGEDOCUMENT_PROPERTIES_URI,
-                                getter_AddRefs(mStringBundle));
-  }
-
   return NS_OK;
 }
 
@@ -266,27 +236,10 @@ nsImageDocument::StartDocumentLoad(const char*         aCommand,
                                    PRBool              aReset,
                                    nsIContentSink*     aSink)
 {
-  nsresult rv = nsDocument::StartDocumentLoad(aCommand, aChannel, aLoadGroup,
-                                              aContainer, aDocListener, aReset,
-                                              aSink);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(aChannel);
-  if (httpChannel) {
-    // The misspelled key 'referer' is as per the HTTP spec
-    nsCAutoString referrerHeader;
-    rv = httpChannel->GetRequestHeader(NS_LITERAL_CSTRING("referer"),
-                                       referrerHeader);
-    
-    if (NS_SUCCEEDED(rv)) {
-      SetReferrer(NS_ConvertASCIItoUCS2(referrerHeader));
-    }
-  }
-
-  // Create synthetic document
-  rv = CreateSyntheticDocument();
+  nsresult rv =
+    nsMediaDocument::StartDocumentLoad(aCommand, aChannel, aLoadGroup,
+                                       aContainer, aDocListener, aReset,
+                                       aSink);
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -304,21 +257,34 @@ NS_IMETHODIMP
 nsImageDocument::SetScriptGlobalObject(nsIScriptGlobalObject* aScriptGlobalObject)
 {
   if (!aScriptGlobalObject) {
-    // If the global object is being set to null, then it means we are
-    // going away soon. Drop our ref to imgRequest so that we don't end
-    // up leaking due to cycles through imgLib
-    mImageRequest = nsnull;
-
     if (mImageResizingEnabled) {
       nsCOMPtr<nsIDOMEventTarget> target = do_QueryInterface(mImageElement);
       target->RemoveEventListener(NS_LITERAL_STRING("click"), this, PR_FALSE);
 
       target = do_QueryInterface(mScriptGlobalObject);
       target->RemoveEventListener(NS_LITERAL_STRING("resize"), this, PR_FALSE);
-      target->RemoveEventListener(NS_LITERAL_STRING("keypress"), this, PR_FALSE);
+      target->RemoveEventListener(NS_LITERAL_STRING("keypress"), this,
+                                  PR_FALSE);
     }
+
+    // drop the ref to mImageElement, in case it has a ref to us
+    mImageElement = nsnull;
   }
-  else {
+
+  // Set the script global object on the superclass before doing
+  // anything that might require it....
+  nsresult rv = nsHTMLDocument::SetScriptGlobalObject(aScriptGlobalObject);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  if (aScriptGlobalObject) {
+    // Create synthetic document
+    rv = CreateSyntheticDocument();
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
+
     if (mImageResizingEnabled) {
       nsCOMPtr<nsIDOMEventTarget> target = do_QueryInterface(mImageElement);
       target->AddEventListener(NS_LITERAL_STRING("click"), this, PR_FALSE);
@@ -329,7 +295,7 @@ nsImageDocument::SetScriptGlobalObject(nsIScriptGlobalObject* aScriptGlobalObjec
     }
   }
 
-  return nsHTMLDocument::SetScriptGlobalObject(aScriptGlobalObject);
+  return NS_OK;
 }
 
 
@@ -359,15 +325,14 @@ nsImageDocument::ShrinkToFit()
 {
   if (mImageResizingEnabled) {
     nsCOMPtr<nsIDOMHTMLImageElement> image = do_QueryInterface(mImageElement);
-
-    float ratio = PR_MIN((float)mVisibleWidth / mImageWidth,
-                         (float)mVisibleHeight / mImageHeight);
-    image->SetWidth(NSToCoordFloor(mImageWidth * ratio));
+    image->SetWidth(NSToCoordFloor(GetRatio() * mImageWidth));
 
     mImageElement->SetAttribute(NS_LITERAL_STRING("style"),
-                                NS_LITERAL_STRING("cursor: move"));
+                                NS_LITERAL_STRING("cursor: -moz-zoom-in"));
 
     mImageIsResized = PR_TRUE;
+
+    UpdateTitleAndCharset();
   }
   return NS_OK;
 }
@@ -378,11 +343,17 @@ nsImageDocument::RestoreImage()
   if (mImageResizingEnabled) {
     mImageElement->RemoveAttribute(NS_LITERAL_STRING("width"));
 
-    if (!mImageIsOverflowing) {
+    if (mImageIsOverflowing) {
+      mImageElement->SetAttribute(NS_LITERAL_STRING("style"),
+                                  NS_LITERAL_STRING("cursor: -moz-zoom-out"));
+    }
+    else {
       mImageElement->RemoveAttribute(NS_LITERAL_STRING("style"));
     }
 
     mImageIsResized = PR_FALSE;
+
+    UpdateTitleAndCharset();
   }
 
   return NS_OK;
@@ -403,18 +374,14 @@ nsImageDocument::ToggleImageSize()
   return NS_OK;
 }
 
-
 NS_IMETHODIMP
-nsImageDocument::OnStartDecode(imgIRequest* aRequest,
-                               nsISupports* aCX)
+nsImageDocument::OnStartDecode(imgIRequest* aRequest)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsImageDocument::OnStartContainer(imgIRequest* aRequest,
-                                  nsISupports* aCX,
-                                  imgIContainer* aImage)
+nsImageDocument::OnStartContainer(imgIRequest* aRequest, imgIContainer* aImage)
 {
   aImage->GetWidth(&mImageWidth);
   aImage->GetHeight(&mImageHeight);
@@ -426,16 +393,13 @@ nsImageDocument::OnStartContainer(imgIRequest* aRequest,
 }
 
 NS_IMETHODIMP
-nsImageDocument::OnStartFrame(imgIRequest* aRequest,
-                              nsISupports* aCX,
-                              gfxIImageFrame* aFrame)
+nsImageDocument::OnStartFrame(imgIRequest* aRequest, gfxIImageFrame* aFrame)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsImageDocument::OnDataAvailable(imgIRequest* aRequest,
-                                 nsISupports *aCX,
                                  gfxIImageFrame* aFrame,
                                  const nsRect* aRect)
 {
@@ -444,7 +408,6 @@ nsImageDocument::OnDataAvailable(imgIRequest* aRequest,
 
 NS_IMETHODIMP
 nsImageDocument::OnStopFrame(imgIRequest* aRequest,
-                             nsISupports* aCX,
                              gfxIImageFrame* aFrame)
 {
   return NS_OK;
@@ -452,7 +415,6 @@ nsImageDocument::OnStopFrame(imgIRequest* aRequest,
 
 NS_IMETHODIMP
 nsImageDocument::OnStopContainer(imgIRequest* aRequest,
-                                 nsISupports* aCX,
                                  imgIContainer* aImage)
 {
   return NS_OK;
@@ -460,7 +422,6 @@ nsImageDocument::OnStopContainer(imgIRequest* aRequest,
 
 NS_IMETHODIMP
 nsImageDocument::OnStopDecode(imgIRequest* aRequest,
-                              nsISupports* aCX,
                               nsresult status,
                               const PRUnichar* statusArg)
 {
@@ -469,7 +430,6 @@ nsImageDocument::OnStopDecode(imgIRequest* aRequest,
 
 NS_IMETHODIMP
 nsImageDocument::FrameChanged(imgIContainer* aContainer,
-                              nsISupports *aCX,
                               gfxIImageFrame* aFrame,
                               nsRect* aDirtyRect)
 {
@@ -513,35 +473,16 @@ nsresult
 nsImageDocument::CreateSyntheticDocument()
 {
   // Synthesize an html document that refers to the image
-  nsresult rv;
+  nsresult rv = nsMediaDocument::CreateSyntheticDocument();
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIHTMLContent> body = do_QueryInterface(mBodyContent);
+  if (!body) {
+    NS_WARNING("no body on image document!");
+    return NS_ERROR_FAILURE;
+  }
 
   nsCOMPtr<nsINodeInfo> nodeInfo;
-  rv = mNodeInfoManager->GetNodeInfo(nsHTMLAtoms::html, nsnull,
-                                     kNameSpaceID_None,
-                                     *getter_AddRefs(nodeInfo));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIHTMLContent> root;
-  rv = NS_NewHTMLHtmlElement(getter_AddRefs(root), nodeInfo);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-  root->SetDocument(this, PR_FALSE, PR_TRUE);
-  SetRootContent(root);
-
-  rv = mNodeInfoManager->GetNodeInfo(nsHTMLAtoms::body, nsnull,
-                                     kNameSpaceID_None,
-                                     *getter_AddRefs(nodeInfo));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIHTMLContent> body;
-  rv = NS_NewHTMLBodyElement(getter_AddRefs(body), nodeInfo);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-  body->SetDocument(this, PR_FALSE, PR_TRUE);
-  mBodyContent = do_QueryInterface(body);
-
   rv = mNodeInfoManager->GetNodeInfo(nsHTMLAtoms::img, nsnull,
                                      kNameSpaceID_None,
                                      *getter_AddRefs(nodeInfo));
@@ -554,26 +495,28 @@ nsImageDocument::CreateSyntheticDocument()
   }
   image->SetDocument(this, PR_FALSE, PR_TRUE);
   mImageElement = do_QueryInterface(image);
+  nsCOMPtr<nsIImageLoadingContent> imageLoader = do_QueryInterface(image);
+  NS_ENSURE_TRUE(imageLoader, NS_ERROR_UNEXPECTED);
 
   nsCAutoString src;
   mDocumentURL->GetSpec(src);
 
   NS_ConvertUTF8toUCS2 srcString(src);
-  nsHTMLValue val(srcString);
-
-  image->SetHTMLAttribute(nsHTMLAtoms::src, val, PR_FALSE);
+  // Make sure not to start the image load from here...
+  imageLoader->SetLoadingEnabled(PR_FALSE);
+  image->SetAttr(kNameSpaceID_None, nsHTMLAtoms::src, srcString, PR_FALSE);
+  imageLoader->SetLoadingEnabled(PR_TRUE);
 
   if (mStringBundle) {
     const PRUnichar* formatString[1] = { srcString.get() };
     nsXPIDLString errorMsg;
-    rv = mStringBundle->FormatStringFromName(NS_LITERAL_STRING("InvalidImage").get(),
-      formatString, 1, getter_Copies(errorMsg));
+    NS_NAMED_LITERAL_STRING(str, "InvalidImage");
+    mStringBundle->FormatStringFromName(str.get(), formatString, 1,
+                                        getter_Copies(errorMsg));
 
-    nsHTMLValue errorText(errorMsg);
-    image->SetHTMLAttribute(nsHTMLAtoms::alt, errorText, PR_FALSE);
+    image->SetAttr(kNameSpaceID_None, nsHTMLAtoms::alt, errorMsg, PR_FALSE);
   }
 
-  root->AppendChildTo(body, PR_FALSE, PR_FALSE);
   body->AppendChildTo(image, PR_FALSE, PR_FALSE);
 
   return NS_OK;
@@ -594,9 +537,9 @@ nsImageDocument::CheckOverflowing()
   nsRect visibleArea;
   context->GetVisibleArea(visibleArea);
 
-  nsCOMPtr<nsIStyleContext> styleContext;
   nsCOMPtr<nsIContent> content = do_QueryInterface(mBodyContent);
-  context->ResolveStyleContextFor(content, nsnull, getter_AddRefs(styleContext));
+  nsRefPtr<nsStyleContext> styleContext =
+    context->ResolveStyleContextFor(content, nsnull);
 
   const nsStyleMargin* marginData =
     (const nsStyleMargin*)styleContext->GetStyleData(eStyleStruct_Margin);
@@ -614,7 +557,8 @@ nsImageDocument::CheckOverflowing()
   mVisibleWidth = NSTwipsToIntPixels(visibleArea.width, t2p);
   mVisibleHeight = NSTwipsToIntPixels(visibleArea.height, t2p);
 
-  mImageIsOverflowing = mImageWidth > mVisibleWidth || mImageHeight > mVisibleHeight;
+  mImageIsOverflowing =
+    mImageWidth > mVisibleWidth || mImageHeight > mVisibleHeight;
 
   if (mImageIsOverflowing) {
     ShrinkToFit();
@@ -626,128 +570,67 @@ nsImageDocument::CheckOverflowing()
   return NS_OK;
 }
 
-nsresult
-nsImageDocument::StartLayout()
+void 
+nsImageDocument::UpdateTitleAndCharset()
 {
-  // Reset scrolling to default settings for this shell.
-  // This must happen before the initial reflow, when we create the root frame
-  nsCOMPtr<nsIScrollable> scrollableContainer(do_QueryReferent(mDocumentContainer));
-  if (scrollableContainer) {
-    scrollableContainer->ResetScrollbarPreferences();
+  nsCAutoString typeStr;
+  nsCOMPtr<imgIRequest> imageRequest;
+  nsCOMPtr<nsIImageLoadingContent> imageLoader = do_QueryInterface(mImageElement);
+  if (imageLoader) {
+    imageLoader->GetRequest(nsIImageLoadingContent::CURRENT_REQUEST,
+                            getter_AddRefs(imageRequest));
   }
-
-  PRInt32 numberOfShells = GetNumberOfShells();
-  for (PRInt32 i = 0; i < numberOfShells; i++) {
-    nsCOMPtr<nsIPresShell> shell;
-    GetShellAt(i, getter_AddRefs(shell));
-    if (shell) {
-      // Make shell an observer for next time.
-      shell->BeginObservingDocument();
-
-      // Initial-reflow this time.
-      nsCOMPtr<nsIPresContext> context;
-      shell->GetPresContext(getter_AddRefs(context));
-      nsRect visibleArea;
-      context->GetVisibleArea(visibleArea);
-      shell->InitialReflow(visibleArea.width, visibleArea.height);
-
-      // Now trigger a refresh.
-      nsCOMPtr<nsIViewManager> vm;
-      shell->GetViewManager(getter_AddRefs(vm));
-      if (vm) {
-        vm->EnableRefresh(NS_VMREFRESH_IMMEDIATE);
-      }
-    }
-  }
-
-  return NS_OK;
-}
-
-nsresult nsImageDocument::UpdateTitle()
-{
-  if (mStringBundle) {
-    nsXPIDLString fileStr;
-    nsAutoString widthStr;
-    nsAutoString heightStr;
-    nsAutoString typeStr;
-    nsXPIDLString valUni;
-
-    nsCOMPtr<nsIURL> url = do_QueryInterface(mDocumentURL);
-    if (url) {
-      nsresult rv;
-      nsCAutoString fileName;
-      url->GetFileName(fileName);
-      nsCOMPtr<nsITextToSubURI> textToSubURI = do_GetService(NS_ITEXTTOSUBURI_CONTRACTID, &rv);
-      if (NS_SUCCEEDED(rv))
-      {
-        nsCAutoString originCharset;
-        rv = url->GetOriginCharset(originCharset);
-        if (NS_SUCCEEDED(rv)) 
-          rv = textToSubURI->UnEscapeURIForUI(originCharset, fileName, fileStr);
-      }
-      if (NS_FAILED(rv))
-        fileStr.Assign(NS_ConvertUTF8toUCS2(fileName));
-    }
-
-    widthStr.AppendInt(mImageWidth);
-    heightStr.AppendInt(mImageHeight);
-
-    if (mImageRequest) {
-      nsXPIDLCString mimeType;
-      mImageRequest->GetMimeType(getter_Copies(mimeType));
-      ToUpperCase(mimeType);
-      nsXPIDLCString::const_iterator start, end;
-      mimeType.BeginReading(start);
-      mimeType.EndReading(end);
-      nsXPIDLCString::const_iterator iter = end;
-      if (FindInReadable(NS_LITERAL_CSTRING("IMAGE/"), start, iter) && iter != end) {
-        // strip out "X-" if any
-        if (*iter == 'X') {
+    
+  if (imageRequest) {
+    nsXPIDLCString mimeType;
+    imageRequest->GetMimeType(getter_Copies(mimeType));
+    ToUpperCase(mimeType);
+    nsXPIDLCString::const_iterator start, end;
+    mimeType.BeginReading(start);
+    mimeType.EndReading(end);
+    nsXPIDLCString::const_iterator iter = end;
+    if (FindInReadable(NS_LITERAL_CSTRING("IMAGE/"), start, iter) && 
+        iter != end) {
+      // strip out "X-" if any
+      if (*iter == 'X') {
+        ++iter;
+        if (iter != end && *iter == '-') {
           ++iter;
-          if (iter != end && *iter == '-') {
-            ++iter;
-            if (iter == end) {
-              // looks like "IMAGE/X-" is the type??  Bail out of here.
-              mimeType.BeginReading(iter);
-            }
-          } else {
-            --iter;
+          if (iter == end) {
+            // looks like "IMAGE/X-" is the type??  Bail out of here.
+            mimeType.BeginReading(iter);
           }
+        } else {
+          --iter;
         }
-        CopyASCIItoUCS2(Substring(iter, end), typeStr);
-      } else {
-        CopyASCIItoUCS2(mimeType, typeStr);
-      }
-    }
-
-    // If we got a filename, display it
-    if (!fileStr.IsEmpty()) {
-      // if we got a valid size (sometimes we do not) then display it
-      if (mImageWidth != 0 && mImageHeight != 0){
-        const PRUnichar *formatStrings[4]  = {fileStr.get(), typeStr.get(), widthStr.get(), heightStr.get()};
-        mStringBundle->FormatStringFromName(NS_LITERAL_STRING("ImageTitleWithDimensionsAndFile").get(), formatStrings, 4, getter_Copies(valUni));
-      } else {
-        const PRUnichar *formatStrings[2] = {fileStr.get(), typeStr.get()};
-        mStringBundle->FormatStringFromName(NS_LITERAL_STRING("ImageTitleWithoutDimensions").get(), formatStrings, 2, getter_Copies(valUni));
       }
+      typeStr = Substring(iter, end);
     } else {
-      // if we got a valid size (sometimes we do not) then display it
-      if (mImageWidth != 0 && mImageHeight != 0){
-        const PRUnichar *formatStrings[3]  = {typeStr.get(), widthStr.get(), heightStr.get()};
-        mStringBundle->FormatStringFromName(NS_LITERAL_STRING("ImageTitleWithDimensions").get(), formatStrings, 3, getter_Copies(valUni));
-      } else {
-        const PRUnichar *formatStrings[1]  = {typeStr.get()};
-        mStringBundle->FormatStringFromName(NS_LITERAL_STRING("ImageTitleWithNeitherDimensionsNorFile").get(), formatStrings, 1, getter_Copies(valUni));
-      }
-    }
-
-    if (valUni) {
-      // set it on the document
-      SetTitle(valUni);
+      typeStr = mimeType;
     }
   }
 
-  return NS_OK;
+  nsXPIDLString status;
+  if (mImageIsResized) {
+    nsAutoString ratioStr;
+    ratioStr.AppendInt(NSToCoordFloor(GetRatio() * 100));
+
+    const PRUnichar* formatString[1] = { ratioStr.get() };
+    mStringBundle->FormatStringFromName(NS_LITERAL_STRING("ScaledImage").get(),
+                                        formatString, 1,
+                                        getter_Copies(status));
+  }
+
+  static const char* const formatNames[4] = 
+  {
+    "ImageTitleWithNeitherDimensionsNorFile",
+    "ImageTitleWithoutDimensions",
+    "ImageTitleWithDimesions",
+    "ImageTitleWithDimensionsAndFile",
+  };
+
+  nsMediaDocument::UpdateTitleAndCharset(typeStr, formatNames,
+                                         mImageWidth, mImageHeight, status);
 }
 
 
