@@ -51,7 +51,6 @@
 
 #include "nsCOMPtr.h"
 #include "nsIStyleSet.h"
-#include "nsISizeOfHandler.h"
 #include "nsIDOMCSSStyleSheet.h"
 #include "nsIDOMCSSRule.h"
 #include "nsIDOMCSSImportRule.h"
@@ -180,8 +179,6 @@ public:
   // nsIStyleRule methods
 #ifdef DEBUG
   NS_IMETHOD List(FILE* out = stdout, PRInt32 aIndent = 0) const;
-
-  virtual void SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize);
 #endif
 
   // nsICSSRule methods
@@ -243,42 +240,6 @@ CSSCharsetRuleImpl::List(FILE* out, PRInt32 aIndent) const
 
   return NS_OK;
 }
-
-/******************************************************************************
-* SizeOf method:
-*
-*  Self (reported as CSSCharSetRuleImpl's size): 
-*    1) sizeof(*this) + the size of the mEncoding string
-*
-*  Contained / Aggregated data (not reported as CSSCharsetRuleImpl's size):
-*    none
-*
-*  Children / siblings / parents:
-*    none
-*    
-******************************************************************************/
-void CSSCharsetRuleImpl::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
-{
-  NS_ASSERTION(aSizeOfHandler != nsnull, "SizeOf handler cannot be null");
-
-  // first get the unique items collection
-  UNIQUE_STYLE_ITEMS(uniqueItems);
-  if(! uniqueItems->AddItem((void*)this)){
-    return;
-  }
-
-  PRUint32 localSize=0;
-
-  // create a tag for this instance
-  nsCOMPtr<nsIAtom> tag = do_GetAtom("CSSCharsetRuleImpl");
-  // get the size of an empty instance and add to the sizeof handler
-  aSize = sizeof(*this);
-  // add the string for encoding value
-  mEncoding.SizeOf(aSizeOfHandler, &localSize);
-  aSize += localSize;
-  aSize -= sizeof(mEncoding); // counted in sizeof(*this) and nsString->SizeOf()
-  aSizeOfHandler->AddSize(tag,aSize);
-}
 #endif
 
 NS_IMETHODIMP
@@ -314,7 +275,7 @@ CSSCharsetRuleImpl::SetEncoding(const nsAString& aEncoding)
 }
 
 
-NS_EXPORT nsresult
+nsresult
 NS_NewCSSCharsetRule(nsICSSRule** aInstancePtrResult, const nsAString& aEncoding)
 {
   if (! aInstancePtrResult) {
@@ -395,8 +356,6 @@ public:
   // nsIStyleRule methods
 #ifdef DEBUG
   NS_IMETHOD List(FILE* out = stdout, PRInt32 aIndent = 0) const;
-
-  virtual void SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize);
 #endif
 
   // nsICSSRule methods
@@ -489,44 +448,6 @@ CSSImportRuleImpl::List(FILE* out, PRInt32 aIndent) const
 
   return NS_OK;
 }
-
-/******************************************************************************
-* SizeOf method:
-*
-*  Self (reported as CSSImportRuleImpl's size): 
-*    1) sizeof(*this) + the size of the mURLSpec string
-*
-*  Contained / Aggregated data (not reported as CSSImportRuleImpl's size):
-*    none
-*
-*  Children / siblings / parents:
-*    none
-*    
-******************************************************************************/
-void CSSImportRuleImpl::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
-{
-  NS_ASSERTION(aSizeOfHandler != nsnull, "SizeOf handler cannot be null");
-
-  // first get the unique items collection
-  UNIQUE_STYLE_ITEMS(uniqueItems);
-  if(! uniqueItems->AddItem((void*)this)){
-    return;
-  }
-
-  PRUint32 localSize=0;
-
-  // create a tag for this instance
-  nsCOMPtr<nsIAtom> tag = do_GetAtom("CSSImportRuleImpl");
-  // get the size of an empty instance and add to the sizeof handler
-  aSize = sizeof(*this);
-
-  // add the strings for the URLSpec and the Media
-  mURLSpec.SizeOf(aSizeOfHandler, &localSize);
-  aSize += localSize;
-  aSize -= sizeof(mURLSpec); // counted in sizeof(*this) and nsString->SizeOf()
-  aSizeOfHandler->AddSize(tag,aSize);
-
-}
 #endif
 
 NS_IMETHODIMP
@@ -603,7 +524,7 @@ CSSImportRuleImpl::SetSheet(nsICSSStyleSheet* aSheet)
   return NS_OK;
 }
 
-NS_EXPORT nsresult
+nsresult
 NS_NewCSSImportRule(nsICSSImportRule** aInstancePtrResult, 
                     const nsString& aURLSpec,
                     const nsString& aMedia)
@@ -724,8 +645,6 @@ public:
   // nsIStyleRule methods
 #ifdef DEBUG
   NS_IMETHOD List(FILE* out = stdout, PRInt32 aIndent = 0) const;
-
-  virtual void SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize);
 #endif
 
   // nsICSSRule methods
@@ -903,64 +822,6 @@ CSSMediaRuleImpl::List(FILE* out, PRInt32 aIndent) const
   fputs("}\n", out);
   return NS_OK;
 }
-
-/******************************************************************************
-* SizeOf method:
-*
-*  Self (reported as CSSMediaRuleImpl's size): 
-*    1) sizeof(*this) + the size of each unique atom in the mMedia collection
-*
-*  Contained / Aggregated data (not reported as CSSMediaRuleImpl's size):
-*    1) Delegate to the rules in the mRules collection to report theri own size
-*
-*  Children / siblings / parents:
-*    none
-*    
-******************************************************************************/
-void CSSMediaRuleImpl::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
-{
-  NS_ASSERTION(aSizeOfHandler != nsnull, "SizeOf handler cannot be null");
-
-  // first get the unique items collection
-  UNIQUE_STYLE_ITEMS(uniqueItems);
-  if(! uniqueItems->AddItem((void*)this)){
-    return;
-  }
-
-  PRUint32 localSize=0;
-
-  // create a tag for this instance
-  nsCOMPtr<nsIAtom> tag = do_GetAtom("CSSMediaRuleImpl");
-  // get the size of an empty instance and add to the sizeof handler
-  aSize = sizeof(*this);
-
-  if (mMedia) {
-    // get the sizes of the media atoms (if unique)
-    PRUint32 index = 0;
-    PRUint32 count;
-    mMedia->Count(&count);
-    while (index < count) {
-      nsCOMPtr<nsIAtom> medium = dont_AddRef((nsIAtom*)mMedia->ElementAt(index++));
-      if(medium && uniqueItems->AddItem(medium)){
-        medium->SizeOf(aSizeOfHandler, &localSize);
-        aSize += localSize;
-      }
-    }
-  }
-  // we are done with the size we report for ourself
-  aSizeOfHandler->AddSize(tag,aSize);
-
-  if (mRules) {
-    // delegate to the rules themselves (do not sum into our size)
-    PRUint32 index = 0;
-    PRUint32 count;
-    mRules->Count(&count);
-    while (index < count) {
-      nsCOMPtr<nsICSSRule> rule = dont_AddRef((nsICSSRule*)mRules->ElementAt(index++));
-      rule->SizeOf(aSizeOfHandler, localSize);
-    }
-  }
-}
 #endif
 
 NS_IMETHODIMP
@@ -1267,8 +1128,6 @@ public:
   // nsIStyleRule methods
 #ifdef DEBUG
   NS_IMETHOD List(FILE* out = stdout, PRInt32 aIndent = 0) const;
-
-  virtual void SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize);
 #endif
 
   // nsICSSRule methods
@@ -1346,49 +1205,6 @@ CSSNameSpaceRuleImpl::List(FILE* out, PRInt32 aIndent) const
   fputs(")\n", out);
   return NS_OK;
 }
-
-/******************************************************************************
-* SizeOf method:
-*
-*  Self (reported as CSSNamespaceRuleImpl's size): 
-*    1) sizeof(*this) + the size of the mURLSpec string +
-*       the sizeof the mPrefix atom (if it ieists)
-*
-*  Contained / Aggregated data (not reported as CSSNamespaceRuleImpl's size):
-*    none
-*
-*  Children / siblings / parents:
-*    none
-*    
-******************************************************************************/
-void CSSNameSpaceRuleImpl::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
-{
-  NS_ASSERTION(aSizeOfHandler != nsnull, "SizeOf handler cannot be null");
-
-  // first get the unique items collection
-  UNIQUE_STYLE_ITEMS(uniqueItems);
-  if(! uniqueItems->AddItem((void*)this)){
-    return;
-  }
-
-  PRUint32 localSize=0;
-
-  // create a tag for this instance
-  nsCOMPtr<nsIAtom> tag = do_GetAtom("CSSNameSpaceRuleImpl");
-  // get the size of an empty instance and add to the sizeof handler
-  aSize = sizeof(*this);
-  
-  // get the member data as part of this dump
-  mURLSpec.SizeOf(aSizeOfHandler, &localSize);
-  aSize += localSize;
-  aSize -= sizeof(mURLSpec); // counted in sizeof(*this) and nsString->SizeOf()
-
-  if(mPrefix && uniqueItems->AddItem(mPrefix)){
-    mPrefix->SizeOf(aSizeOfHandler, &localSize);
-    aSize += localSize;
-  }
-  aSizeOfHandler->AddSize(tag, aSize);
-}
 #endif
 
 NS_IMETHODIMP
