@@ -70,9 +70,9 @@
 #include "nsUnicharUtils.h"
 
 #include "nsIStyleSet.h"
-#include "nsISizeOfHandler.h"
 
 #include "nsContentUtils.h"
+#include "nsContentErrors.h"
 
 // #define DEBUG_REFS
  
@@ -308,51 +308,6 @@ PRBool nsAttrSelector::Equals(const nsAttrSelector* aOther) const
   return PR_FALSE;
 }
 
-#ifdef DEBUG
-/******************************************************************************
-* SizeOf method:
-*
-*  Self (reported as nsAttrSelector's size): 
-*    1) sizeof(*this) + the size of mAttr atom (if it exists and is unique)
-*
-*  Contained / Aggregated data (not reported as nsAttrSelector's size):
-*    none
-*
-*  Children / siblings / parents:
-*    1) Recurses to the mMext instance which is reported as a seperate instance
-*    
-******************************************************************************/
-void nsAttrSelector::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
-{
-  NS_ASSERTION(aSizeOfHandler != nsnull, "SizeOf handler cannot be null");
-
-  // first get the unique items collection
-  UNIQUE_STYLE_ITEMS(uniqueItems);
-  if(! uniqueItems->AddItem((void*)this)){
-    return;
-  }
-
-  PRUint32 localSize=0;
-
-  // create a tag for this instance
-  nsCOMPtr<nsIAtom> tag = do_GetAtom("nsAttrSelector");
-  // get the size of an empty instance and add to the sizeof handler
-  aSize = sizeof(*this);
-
-  // add in the mAttr atom
-  if (mAttr && uniqueItems->AddItem(mAttr)){
-    mAttr->SizeOf(aSizeOfHandler, &localSize);
-    aSize += localSize;
-  }
-  aSizeOfHandler->AddSize(tag,aSize);
-
-  // recurse to the next one...
-  if(mNext){
-    mNext->SizeOf(aSizeOfHandler, localSize);
-  }
-}
-#endif
-
 MOZ_DECL_CTOR_COUNTER(nsCSSSelector)
 
 #ifdef DEBUG_REFS
@@ -624,120 +579,13 @@ PRInt32 nsCSSSelector::CalcWeight(void) const
   return weight;
 }
 
-#ifdef DEBUG
-/******************************************************************************
-* SizeOf method:
-*
-*  Self (reported as nsCSSSelector's size): 
-*    1) sizeof(*this) + the size of the mTag 
-*       + the size of the mIDList unique items 
-*       + the size of the mClassList and mPseudoClassList unique items
-*
-*  Contained / Aggregated data (not reported as nsCSSSelector's size):
-*    1) AttributeList is called out to seperately if it exists
-*
-*  Children / siblings / parents:
-*    1) Recurses to mNext which is counted as it's own instance
-*    
-******************************************************************************/
-void nsCSSSelector::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
-{
-  NS_ASSERTION(aSizeOfHandler != nsnull, "SizeOf handler cannot be null");
-
-  // first get the unique items collection
-  UNIQUE_STYLE_ITEMS(uniqueItems);
-  if(! uniqueItems->AddItem((void*)this)){
-    return;
-  }
-
-  PRUint32 localSize=0;
-
-  // create a tag for this instance
-  nsCOMPtr<nsIAtom> tag = do_GetAtom("nsCSSSelector");
-  // get the size of an empty instance and add to the sizeof handler
-  aSize = sizeof(*this);
-  
-  // now get the member-atoms and add them in
-  if(mTag && uniqueItems->AddItem(mTag)){
-    localSize = 0;
-    mTag->SizeOf(aSizeOfHandler, &localSize);
-    aSize += localSize;
-  }
-
-
-  // XXX ????
-
-
-
-  // a couple of simple atom lists
-  if(mIDList && uniqueItems->AddItem(mIDList)){
-    aSize += sizeof(*mIDList);
-    nsAtomList *pNext = nsnull;
-    pNext = mIDList;    
-    while(pNext){
-      if(pNext->mAtom && uniqueItems->AddItem(pNext->mAtom)){
-        localSize = 0;
-        pNext->mAtom->SizeOf(aSizeOfHandler, &localSize);
-        aSize += localSize;
-      }
-      pNext = pNext->mNext;
-    }
-  }
-  if(mClassList && uniqueItems->AddItem(mClassList)){
-    aSize += sizeof(*mClassList);
-    nsAtomList *pNext = nsnull;
-    pNext = mClassList;    
-    while(pNext){
-      if(pNext->mAtom && uniqueItems->AddItem(pNext->mAtom)){
-        localSize = 0;
-        pNext->mAtom->SizeOf(aSizeOfHandler, &localSize);
-        aSize += localSize;
-      }
-      pNext = pNext->mNext;
-    }
-  }
-  if(mPseudoClassList && uniqueItems->AddItem(mPseudoClassList)){
-    nsAtomStringList *pNext = nsnull;
-    pNext = mPseudoClassList;    
-    while(pNext){
-      if(pNext->mAtom && uniqueItems->AddItem(pNext->mAtom)){
-        localSize = 0;
-        pNext->mAtom->SizeOf(aSizeOfHandler, &localSize);
-        aSize += localSize;
-      }
-      pNext = pNext->mNext;
-    }
-  }
-  // done with undelegated sizes 
-  aSizeOfHandler->AddSize(tag, aSize);
-
-  // the AttributeList gets its own delegation-call
-  if(mAttrList){
-    localSize = 0;
-    mAttrList->SizeOf(aSizeOfHandler, localSize);
-  }
-
-  // don't forget the negated selectors
-  if(mNegations) {
-    localSize = 0;
-    mNegations->SizeOf(aSizeOfHandler, localSize);
-  }
-  
-  // finally chain to the next...
-  if(mNext){
-    localSize = 0;
-    mNext->SizeOf(aSizeOfHandler, localSize);
-  }
-}
-#endif
-
 // pseudo-elements are stored in the selectors' chain using fictional elements;
 // these fictional elements have mTag starting with a colon
 static PRBool IsPseudoElement(nsIAtom* aAtom)
 {
   if (aAtom) {
-    const PRUnichar *str;
-    aAtom->GetUnicode(&str);
+    const char* str;
+    aAtom->GetUTF8String(&str);
     return str && (*str == ':');
   }
 
@@ -756,7 +604,7 @@ void nsCSSSelector::AppendNegationToString(nsAString& aString)
 nsresult nsCSSSelector::ToString( nsAString& aString, nsICSSStyleSheet* aSheet, PRBool aIsPseudoElem,
                                   PRInt8 aNegatedIndex) const
 {
-  const PRUnichar* temp;
+  nsAutoString temp;
   PRBool aIsNegated = PRBool(0 < aNegatedIndex);
 
   // selectors are linked from right-to-left, so the next selector in the linked list
@@ -783,8 +631,8 @@ nsresult nsCSSSelector::ToString( nsAString& aString, nsICSSStyleSheet* aSheet,
     // will return null if namespace was the default
     sheetNS->FindNameSpacePrefix(mNameSpace, *getter_AddRefs(prefixAtom));
     if (prefixAtom) {
-      const PRUnichar* prefix;
-      prefixAtom->GetUnicode(&prefix);
+      nsAutoString prefix;
+      prefixAtom->ToString(prefix);
       aString.Append(prefix);
       aString.Append(PRUnichar('|'));
     }
@@ -800,15 +648,16 @@ nsresult nsCSSSelector::ToString( nsAString& aString, nsICSSStyleSheet* aSheet,
   } else {
     // Append the tag name, if there is one
     if (mTag) {
-      mTag->GetUnicode(&temp);
-      aString.Append(temp);
+      nsAutoString prefix;
+      mTag->ToString(prefix);
+      aString.Append(prefix);
       NS_IF_NEGATED_END(aIsNegated, aString)
     }
     // Append the id, if there is one
     if (mIDList) {
       nsAtomList* list = mIDList;
       while (list != nsnull) {
-        list->mAtom->GetUnicode(&temp);
+        list->mAtom->ToString(temp);
         NS_IF_NEGATED_START(aIsNegated, aString)
         aString.Append(PRUnichar('#'));
         aString.Append(temp);
@@ -820,7 +669,7 @@ nsresult nsCSSSelector::ToString( nsAString& aString, nsICSSStyleSheet* aSheet,
     if (mClassList) {
       nsAtomList* list = mClassList;
       while (list != nsnull) {
-        list->mAtom->GetUnicode(&temp);
+        list->mAtom->ToString(temp);
         NS_IF_NEGATED_START(aIsNegated, aString)
         aString.Append(PRUnichar('.'));
         aString.Append(temp);
@@ -844,14 +693,14 @@ nsresult nsCSSSelector::ToString( nsAString& aString, nsICSSStyleSheet* aSheet,
         // will return null if namespace was the default
         sheetNS->FindNameSpacePrefix(list->mNameSpace, *getter_AddRefs(prefixAtom));
         if (prefixAtom) { 
-          const PRUnichar* prefix;
-          prefixAtom->GetUnicode(&prefix);
+          nsAutoString prefix;
+          prefixAtom->ToString(prefix);
           aString.Append(prefix);
           aString.Append(PRUnichar('|'));
         }
       }
       // Append the attribute name
-      list->mAttr->GetUnicode(&temp);
+      list->mAttr->ToString(temp);
       aString.Append(temp);
       // Append the function
       if (list->mFunction == NS_ATTR_FUNC_EQUALS) {
@@ -884,7 +733,7 @@ nsresult nsCSSSelector::ToString( nsAString& aString, nsICSSStyleSheet* aSheet,
   if (mPseudoClassList) {
     nsAtomStringList* list = mPseudoClassList;
     while (list != nsnull) {
-      list->mAtom->GetUnicode(&temp);
+      list->mAtom->ToString(temp);
       NS_IF_NEGATED_START(aIsNegated, aString)
       aString.Append(temp);
       if (nsnull != list->mString) {
@@ -949,8 +798,6 @@ public:
 
 #ifdef DEBUG
   NS_IMETHOD List(FILE* out = stdout, PRInt32 aIndent = 0) const;
-
-  virtual void SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize);
 #endif
 
 protected:
@@ -1039,47 +886,6 @@ CSSImportantRule::List(FILE* out, PRInt32 aIndent) const
 
   return NS_OK;
 }
-
-/******************************************************************************
-* SizeOf method:
-*
-*  Self (reported as CSSImportantRule's size): 
-*    1) sizeof(*this) 
-*
-*  Contained / Aggregated data (not reported as CSSImportantRule's size):
-*    1) mDeclaration is sized seperately
-*    2) mSheet is sized seperately
-*
-*  Children / siblings / parents:
-*    none
-*    
-******************************************************************************/
-void CSSImportantRule::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
-{
-  NS_ASSERTION(aSizeOfHandler != nsnull, "SizeOf handler cannot be null");
-
-  // first get the unique items collection
-  UNIQUE_STYLE_ITEMS(uniqueItems);
-  if(! uniqueItems->AddItem((void*)this)){
-    return;
-  }
-
-  PRUint32 localSize=0;
-
-  // create a tag for this instance
-  nsCOMPtr<nsIAtom> tag = do_GetAtom("CSSImportantRule");
-  // get the size of an empty instance and add to the sizeof handler
-  aSize = sizeof(CSSImportantRule);
-  aSizeOfHandler->AddSize(tag,aSize);
-
-  // now dump the mDeclaration and mSheet
-  if(mDeclaration){
-    mDeclaration->SizeOf(aSizeOfHandler, localSize);
-  }
-  if(mSheet){
-    mSheet->SizeOf(aSizeOfHandler, localSize);
-  }
-}
 #endif
 
 // -- nsDOMStyleRuleDeclaration -------------------------------
@@ -1096,7 +902,6 @@ public:
   virtual void DropReference(void);
   virtual nsresult GetCSSDeclaration(nsCSSDeclaration **aDecl,
                                      PRBool aAllocate);
-  virtual nsresult SetCSSDeclaration(nsCSSDeclaration *aDecl);
   virtual nsresult GetCSSParsingEnvironment(nsICSSStyleRule* aRule,
                                             nsICSSStyleSheet** aSheet,
                                             nsIDocument** aDocument,
@@ -1201,16 +1006,6 @@ DOMCSSDeclarationImpl::GetCSSDeclaration(nsCSSDeclaration **aDecl,
   return NS_OK;
 }
 
-nsresult
-DOMCSSDeclarationImpl::SetCSSDeclaration(nsCSSDeclaration *aDecl)
-{
-  if (mRule) {
-    mRule->SetDeclaration(aDecl);
-  }
-
-  return NS_OK;
-}
-
 /*
  * This is a utility function.  It will only fail if it can't get a
  * parser.  This means it can return NS_OK without all of aSheet,
@@ -1347,10 +1142,7 @@ DOMCSSDeclarationImpl::ParseDeclaration(const nsAString& aDecl,
       result = cssParser->ParseAndAppendDeclaration(aDecl, baseURI, decl,
                                                     aParseOnlyOneDecl, &hint);
 
-      if (result == NS_CSS_PARSER_DROP_DECLARATION) {
-        SetCSSDeclaration(declClone);
-        result = NS_OK;
-      } else if (NS_SUCCEEDED(result)) {
+      if (NS_SUCCEEDED(result)) {
         if (cssSheet) {
           cssSheet->SetModified(PR_TRUE);
         }
@@ -1428,8 +1220,6 @@ public:
 
 #ifdef DEBUG
   NS_IMETHOD List(FILE* out = stdout, PRInt32 aIndent = 0) const;
-
-  virtual void SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize);
 #endif
 
   // nsIDOMCSSRule interface
@@ -2332,7 +2122,7 @@ MapDisplayForDeclaration(nsCSSDeclaration* aDecl, const nsStyleStructID& aID, ns
     }
   }
   else if (aID == eStyleStruct_Visibility) {
-    // opacity: factor, percent, inherit
+    // opacity: factor, inherit
     if (aDisplay.mOpacity.GetUnit() == eCSSUnit_Null && ourDisplay->mOpacity.GetUnit() != eCSSUnit_Null)
       aDisplay.mOpacity = ourDisplay->mOpacity;
 
@@ -2415,58 +2205,6 @@ CSSStyleRuleImpl::List(FILE* out, PRInt32 aIndent) const
 
   return NS_OK;
 }
-
-/******************************************************************************
-* SizeOf method:
-*
-*  Self (reported as CSSStyleRuleImpl's size): 
-*    1) sizeof(*this) 
-*       + sizeof the DOMDeclaration if it exists and is unique
-*
-*  Contained / Aggregated data (not reported as CSSStyleRuleImpl's size):
-*    1) mDeclaration if it exists
-*    2) mImportantRule if it exists
-*
-*  Children / siblings / parents:
-*    none
-*    
-******************************************************************************/
-void CSSStyleRuleImpl::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
-{
-  NS_ASSERTION(aSizeOfHandler != nsnull, "SizeOf handler cannot be null");
-
-  // first get the unique items collection
-  UNIQUE_STYLE_ITEMS(uniqueItems);
-  if(! uniqueItems->AddItem((void*)this)){
-    return;
-  }
-
-  PRUint32 localSize=0;
-
-  // create a tag for this instance
-  nsCOMPtr<nsIAtom> tag = do_GetAtom("CSSStyleRuleImpl");
-  // get the size of an empty instance and add to the sizeof handler
-  aSize = sizeof(*this);
-  // remove the sizeof the mSelector's class since we count it seperately below
-  aSize -= sizeof(mSelector);
-
-  // and add the size of the DOMDeclaration
-  // XXX - investigate the size and quantity of these
-  if(mDOMDeclaration && uniqueItems->AddItem(mDOMDeclaration)){
-    aSize += sizeof(DOMCSSDeclarationImpl);
-  }
-  aSizeOfHandler->AddSize(tag,aSize);
-  
-  // now delegate to the Selector, Declaration, and ImportantRule
-  mSelector.SizeOf(aSizeOfHandler, localSize);
-
-  if(mDeclaration){
-    mDeclaration->SizeOf(aSizeOfHandler, localSize);
-  }
-  if(mImportantRule){
-    mImportantRule->SizeOf(aSizeOfHandler, localSize);
-  }
-}
 #endif
 
 NS_IMETHODIMP    
@@ -2556,8 +2294,9 @@ CSSStyleRuleImpl::GetStyle(nsIDOMCSSStyleDeclaration** aStyle)
   return NS_OK;
 }
 
-NS_EXPORT nsresult
-  NS_NewCSSStyleRule(nsICSSStyleRule** aInstancePtrResult, const nsCSSSelector& aSelector)
+nsresult
+NS_NewCSSStyleRule(nsICSSStyleRule** aInstancePtrResult,
+                   const nsCSSSelector& aSelector)
 {
   if (aInstancePtrResult == nsnull) {
     return NS_ERROR_NULL_POINTER;
