@@ -64,7 +64,7 @@ enum nsStyleStructID {
  * increase by 1.
  */
 
-#define STYLE_STRUCT(name, checkdata_cb) eStyleStruct_##name,
+#define STYLE_STRUCT(name, checkdata_cb, ctor_args) eStyleStruct_##name,
 #include "nsStyleStructList.h"
 #undef STYLE_STRUCT
 
@@ -76,9 +76,13 @@ nsStyleStructID_Length /* one past the end; length of 0-based list */
 #define NS_STYLE_INHERIT_BIT(sid_)        (1 << PRInt32(eStyleStruct_##sid_))
 #define NS_STYLE_INHERIT_MASK             0x00ffffff
 
+// Additional bits for nsStyleContext's mBits:
 // A bit to test whether or not we have any text decorations.
 #define NS_STYLE_HAS_TEXT_DECORATIONS     0x01000000
 
+// Additional bits for nsRuleNode's mDependentBits:
+#define NS_RULE_NODE_GC_MARK              0x02000000
+
 #define NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(the_sid) \
   static const nsStyleStructID GetStyleStructID() {return the_sid;}
 
@@ -92,12 +96,18 @@ struct nsStyleStruct {
 
 struct nsStyleFont : public nsStyleStruct {
   nsStyleFont(void);
+  nsStyleFont(const nsFont& aFont);
+  nsStyleFont(const nsStyleFont& aStyleFont);
+  nsStyleFont(nsIPresContext *aPresContext);
   ~nsStyleFont(void) {};
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_Font)
 
   nsChangeHint CalcDifference(const nsStyleFont& aOther) const;
   static nsChangeHint CalcFontDifference(const nsFont& aFont1, const nsFont& aFont2);
+
+  static nscoord ZoomText(nsIPresContext* aPresContext, nscoord aSize);
+  static nscoord UnZoomText(nsIPresContext* aPresContext, nscoord aSize);
   
   void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW;
   void Destroy(nsIPresContext* aContext);
@@ -108,10 +118,6 @@ struct nsStyleFont : public nsStyleStruct {
                         // which is our "actual size" and is enforced to be >= the user's
                         // preferred min-size. mFont.size should be used for display purposes
                         // while mSize is the value to return in getComputedStyle() for example.
-
-  nsStyleFont(const nsFont& aFont);
-  nsStyleFont(const nsStyleFont& aStyleFont);
-  nsStyleFont(nsIPresContext* aPresContext);
 };
 
 struct nsStyleColor : public nsStyleStruct {
@@ -695,7 +701,7 @@ struct nsStyleVisibility : public nsStyleStruct {
   PRUint8 mDirection;                  // [inherited] see nsStyleConsts.h NS_STYLE_DIRECTION_*
   PRUint8   mVisible;                  // [inherited]
   nsCOMPtr<nsILanguageAtom> mLanguage; // [inherited]
-  float mOpacity;                      // [inherited] percentage
+  float mOpacity;                      // [inherited]
  
   PRBool IsVisible() const {
 		return (mVisible == NS_STYLE_VISIBILITY_VISIBLE);
@@ -1091,7 +1097,6 @@ struct nsStyleUserInterface: public nsStyleStruct {
   nsString mCursorImage;          // [inherited] url string
 };
 
-#ifdef INCLUDE_XUL
 struct nsStyleXUL : public nsStyleStruct {
   nsStyleXUL();
   nsStyleXUL(const nsStyleXUL& aSource);
@@ -1118,7 +1123,6 @@ struct nsStyleXUL : public nsStyleStruct {
   PRUint8       mBoxOrient;             // [reset] see nsStyleConsts.h
   PRUint8       mBoxPack;               // [reset] see nsStyleConsts.h
 };
-#endif
 
 #ifdef MOZ_SVG
 enum nsStyleSVGPaintType {
@@ -1138,6 +1142,8 @@ struct nsStyleSVG : public nsStyleStruct {
   nsStyleSVG(const nsStyleSVG& aSource);
   ~nsStyleSVG();
 
+  NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_SVG)
+
   void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
@@ -1219,7 +1225,7 @@ inline nsBorderEdges::nsBorderEdges()
 
 // typesafe mechanisms for accessing style data, global function
 // templates |GetStyleData(nsIFrame*, const T**)| and
-// |GetStyleData(nsIStyleContext*, const T**)|, where T is derived from
-// nsStyleStruct, are located in nsIStyleContext.h and nsIFrame.h
+// |GetStyleData(nsStyleContext*, const T**)|, where T is derived from
+// nsStyleStruct, are located in nsStyleContext.h and nsIFrame.h
 
 #endif /* nsStyleStruct_h___ */
