@@ -47,9 +47,12 @@
 #include "nsReadableUtils.h"
 #include "nsStaticNameTable.h"
 
+// required to make the symbol external, so that TestCSSPropertyLookup.cpp can link with it
+extern const char* const kCSSRawProperties[];
+
 // define an array of all CSS properties
 #define CSS_PROP(_name, _id, _method, _hint) #_name,
-const char* kCSSRawProperties[] = {
+const char* const kCSSRawProperties[] = {
 #include "nsCSSPropList.h"
 };
 #undef CSS_PROP
@@ -135,6 +138,9 @@ const PRInt32 nsCSSProps::kAppearanceKTable[] = {
   eCSSKeyword_button,                 NS_THEME_BUTTON,
   eCSSKeyword_radio,                  NS_THEME_RADIO,
   eCSSKeyword_checkbox,               NS_THEME_CHECKBOX,
+  eCSSKeyword_radio_small,            NS_THEME_RADIO_SMALL,
+  eCSSKeyword_checkbox_small,         NS_THEME_CHECKBOX_SMALL,
+  eCSSKeyword_button_small,           NS_THEME_BUTTON_SMALL,
   eCSSKeyword_toolbox,                NS_THEME_TOOLBOX,
   eCSSKeyword_toolbar,                NS_THEME_TOOLBAR,
   eCSSKeyword_toolbarbutton,          NS_THEME_TOOLBAR_BUTTON,
