@@ -42,6 +42,7 @@
 #include "nsString.h"
 #include "nsUnitConversion.h"
 #include "nsIPresContext.h"
+#include "nsIDeviceContext.h"
 #include "nsIStyleRule.h"
 #include "nsISupportsArray.h"
 #include "nsCRT.h"
@@ -78,8 +79,7 @@ inline nscoord CalcSideFor(const nsIFrame* aFrame, const nsStyleCoord& aCoord,
       nsIFrame* parentFrame;
       aFrame->GetParent(&parentFrame);  // XXX may not be direct parent...
       if (nsnull != parentFrame) {
-        nsIStyleContext* parentContext;
-        parentFrame->GetStyleContext(&parentContext);
+        nsStyleContext* parentContext = parentFrame->GetStyleContext();
         if (nsnull != parentContext) {
           nsMargin  parentSpacing;
           switch (aSpacing) {
@@ -111,7 +111,6 @@ inline nscoord CalcSideFor(const nsIFrame* aFrame, const nsStyleCoord& aCoord,
             case NS_SIDE_RIGHT:   result = parentSpacing.right;  break;
             case NS_SIDE_BOTTOM:  result = parentSpacing.bottom; break;
           }
-          NS_RELEASE(parentContext);
         }
       }
       break;
@@ -232,6 +231,17 @@ nsStyleFont::nsStyleFont(const nsStyleFont& aSrc)
 {
 }
 
+
+nsStyleFont::nsStyleFont(nsIPresContext* aPresContext)
+  : mFlags(NS_STYLE_FONT_DEFAULT)
+{
+  const nsFont* defaultFont;
+  aPresContext->GetDefaultFont(kPresContext_DefaultVariableFont_ID,
+                               &defaultFont);
+  mFont = *defaultFont;
+  mSize = mFont.size = nsStyleFont::ZoomText(aPresContext, mFont.size);
+}
+
 void* 
 nsStyleFont::operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
   void* result = nsnull;
@@ -255,6 +265,28 @@ nsChangeHint nsStyleFont::CalcDifference(const nsStyleFont& aOther) const
   return NS_STYLE_HINT_REFLOW;
 }
 
+inline float
+TextZoomFor(nsIPresContext* aPresContext)
+{
+  nsCOMPtr<nsIDeviceContext> dc;
+  aPresContext->GetDeviceContext(getter_AddRefs(dc));
+  float textZoom;
+  dc->GetTextZoom(textZoom);
+  return textZoom;
+}
+
+/* static */ nscoord
+nsStyleFont::ZoomText(nsIPresContext *aPresContext, nscoord aSize)
+{
+  return nscoord(float(aSize) * TextZoomFor(aPresContext));
+}
+
+/* static */ nscoord
+nsStyleFont::UnZoomText(nsIPresContext *aPresContext, nscoord aSize)
+{
+  return nscoord(float(aSize) / TextZoomFor(aPresContext));
+}
+
 nsChangeHint nsStyleFont::CalcFontDifference(const nsFont& aFont1, const nsFont& aFont2)
 {
   if ((aFont1.size == aFont2.size) && 
@@ -713,7 +745,6 @@ nsChangeHint nsStyleList::CalcDifference(const nsStyleList& aOther) const
   return NS_STYLE_HINT_REFLOW;
 }
 
-#ifdef INCLUDE_XUL
 // --------------------
 // nsStyleXUL
 //
@@ -750,8 +781,6 @@ nsChangeHint nsStyleXUL::CalcDifference(const nsStyleXUL& aOther) const
   return NS_STYLE_HINT_REFLOW;
 }
 
-#endif // INCLUDE_XUL
-
 #ifdef MOZ_SVG
 // --------------------
 // nsStyleSVG
@@ -1006,6 +1035,8 @@ nsChangeHint nsStyleBackground::CalcDifference(const nsStyleBackground& aOther)
     && ((NS_STYLE_BG_ATTACHMENT_FIXED == mBackgroundAttachment) ||
         (NS_STYLE_BG_ATTACHMENT_FIXED == aOther.mBackgroundAttachment)))
     // this might require creation of a view
+    // XXX This probably doesn't call ApplyRenderingChangeToTree, which
+    // means we might not invalidate the canvas if this is the body.
     return NS_STYLE_HINT_FRAMECHANGE;
 
   if ((mBackgroundAttachment == aOther.mBackgroundAttachment) &&
