@@ -768,9 +768,10 @@ nsSVGAttributes::NormalizeAttrString(const nsAString& aStr,
                                      nsINodeInfo*& aNodeInfo)
 {
   PRInt32 indx, count = Count();
+  NS_ConvertUCS2toUTF8 utf8String(aStr);
   for (indx = 0; indx < count; indx++) {
     nsSVGAttribute* attr = ElementAt(indx);
-    if (attr->GetNodeInfo()->QualifiedNameEquals(aStr)) {
+    if (attr->GetNodeInfo()->QualifiedNameEquals(utf8String)) {
       aNodeInfo = attr->GetNodeInfo();
       NS_ADDREF(aNodeInfo);
       
@@ -926,14 +927,12 @@ nsSVGAttributes::RemoveNamedItem(const nsAString& aName,
                                  nsIDOMNode** aReturn)
 {
   nsCOMPtr<nsIDOMElement> element( do_QueryInterface(mContent) );
+  *aReturn = nsnull;
   if (element) {
+    // XXX should set aReturn to the element we are about to remove
     return element->RemoveAttribute(aName);
-    *aReturn = nsnull; // XXX should be the element we just removed
-    return NS_OK;
-  }
-  else {
-    return NS_ERROR_FAILURE;
   }
+  return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
