@@ -414,14 +414,6 @@ nsSVGElement::DumpContent(FILE* out, PRInt32 aIndent,PRBool aDumpAll) const
   return NS_OK; 
 }
 
-NS_IMETHODIMP
-nsSVGElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
-{
-//  NS_NOTYETIMPLEMENTED("write me!");
-  return NS_ERROR_UNEXPECTED;
-}  
-
-
 //----------------------------------------------------------------------
 // nsIStyledContent methods
 
