@@ -90,6 +90,7 @@
 #include "nsIDOMDragListener.h"
 #include "nsIDOMMutationListener.h"
 #include "nsIDOMContextMenuListener.h"
+#include "nsIDOMEventGroup.h"
 
 #include "nsXBLAtoms.h"
 #include "nsXULAtoms.h"
@@ -111,7 +112,7 @@
 #include "nsXBLLoadHandler.h"
 #include "nsXBLContextMenuHandler.h"
 #include "nsXBLCustomHandler.h"
-
+#include "nsXBLPrototypeBinding.h"
 #include "nsXBLBinding.h"
 
 // Helper classes
@@ -237,7 +238,7 @@ nsXBLBinding::kEventHandlerMap[] = {
 NS_IMPL_ISUPPORTS1(nsXBLBinding, nsIXBLBinding)
 
 // Constructors/Destructors
-nsXBLBinding::nsXBLBinding(nsIXBLPrototypeBinding* aBinding)
+nsXBLBinding::nsXBLBinding(nsXBLPrototypeBinding* aBinding)
 : mFirstHandler(nsnull),
   mInsertionPointTable(nsnull),
   mIsStyleBinding(PR_TRUE),
@@ -345,15 +346,14 @@ nsXBLBinding::SetAnonymousContent(nsIContent* aParent)
 }
 
 NS_IMETHODIMP
-nsXBLBinding::GetPrototypeBinding(nsIXBLPrototypeBinding** aResult)
+nsXBLBinding::GetPrototypeBinding(nsXBLPrototypeBinding** aResult)
 {
   *aResult = mPrototypeBinding;
-  NS_IF_ADDREF(*aResult);
   return NS_OK;
 }
   
 NS_IMETHODIMP
-nsXBLBinding::SetPrototypeBinding(nsIXBLPrototypeBinding* aProtoBinding)
+nsXBLBinding::SetPrototypeBinding(nsXBLPrototypeBinding* aProtoBinding)
 {
   mPrototypeBinding = aProtoBinding;
   return NS_OK;
@@ -362,13 +362,15 @@ nsXBLBinding::SetPrototypeBinding(nsIXBLPrototypeBinding* aProtoBinding)
 NS_IMETHODIMP
 nsXBLBinding::GetBindingElement(nsIContent** aResult)
 {
-  return mPrototypeBinding->GetBindingElement(aResult);
+  *aResult = mPrototypeBinding->GetBindingElement().get();
+  return NS_OK;
 }
 
 NS_IMETHODIMP
 nsXBLBinding::SetBindingElement(nsIContent* aElement)
 {
-  return mPrototypeBinding->SetBindingElement(aElement);
+  mPrototypeBinding->SetBindingElement(aElement);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -393,8 +395,7 @@ nsXBLBinding::GetFirstBindingWithConstructor(nsIXBLBinding** aResult)
 {
   *aResult = nsnull;
 
-  nsCOMPtr<nsIXBLPrototypeHandler> constructor;
-  mPrototypeBinding->GetConstructor(getter_AddRefs(constructor));
+  nsXBLPrototypeHandler* constructor = mPrototypeBinding->GetConstructor();
   if (constructor) {
     *aResult = this;
     NS_ADDREF(*aResult);
@@ -410,9 +411,7 @@ nsXBLBinding::HasStyleSheets(PRBool* aResolveStyle)
 {
   // Find out if we need to re-resolve style.  We'll need to do this
   // if we have additional stylesheets in our binding document.
-  PRBool hasSheets;
-  mPrototypeBinding->HasStyleSheets(&hasSheets);
-  if (hasSheets) {
+  if (mPrototypeBinding->HasStyleSheets()) {
     *aResolveStyle = PR_TRUE;
     return NS_OK;
   }
@@ -624,8 +623,7 @@ nsXBLBinding::GenerateAnonymousContent()
 
   // Plan to build the content by default.
   PRBool hasContent = (contentCount > 0);
-  PRBool hasInsertionPoints;
-  mPrototypeBinding->HasInsertionPoints(&hasInsertionPoints);
+  PRBool hasInsertionPoints = mPrototypeBinding->HasInsertionPoints();
 
 #ifdef DEBUG
   // See if there's an includes attribute.
@@ -834,27 +832,23 @@ nsXBLBinding::InstallEventHandlers()
   if (AllowScripts()) {
     // Fetch the handlers prototypes for this binding.
     nsCOMPtr<nsIXBLDocumentInfo> info;
-    mPrototypeBinding->GetXBLDocumentInfo(mBoundElement, getter_AddRefs(info));
+    info = mPrototypeBinding->GetXBLDocumentInfo(mBoundElement);
     if (!info)
       return NS_ERROR_FAILURE;
 
-    nsCOMPtr<nsIXBLPrototypeHandler> handlerChain;
-    mPrototypeBinding->GetPrototypeHandlers(getter_AddRefs(handlerChain));
-  
+    nsXBLPrototypeHandler* handlerChain = mPrototypeBinding->GetPrototypeHandlers();
     nsXBLEventHandler* currHandler = nsnull;
 
-    for (nsCOMPtr<nsIXBLPrototypeHandler> next, curr = handlerChain; curr; 
-         curr->GetNextHandler(getter_AddRefs(next)), curr = next) {
+    for (nsXBLPrototypeHandler* curr = handlerChain; curr;
+         curr = curr->GetNextHandler()) {
       // Fetch the event type.
-      nsCOMPtr<nsIAtom> eventAtom;
-      curr->GetEventName(getter_AddRefs(eventAtom));
+      nsCOMPtr<nsIAtom> eventAtom = curr->GetEventName();
       if (!eventAtom) 
         continue;
 
       nsCOMPtr<nsIDOMEventReceiver> receiver = do_QueryInterface(mBoundElement);
       // Figure out if we're using capturing or not.
-      PRUint8 phase;
-      curr->GetPhase(&phase);
+      PRUint8 phase = curr->GetPhase();
       PRBool useCapture = (phase == NS_PHASE_CAPTURING);
         
       // Create a new nsXBLEventHandler.
@@ -867,6 +861,14 @@ nsXBLBinding::InstallEventHandlers()
       PRBool found = PR_FALSE;
       GetEventHandlerIID(eventAtom, &iid, &found);
 
+      // If this is a command, add it in the system event group, otherwise 
+      // add it to the standard event group.
+
+      nsCOMPtr<nsIDOM3EventTarget> target = do_QueryInterface(receiver);
+      nsCOMPtr<nsIDOMEventGroup> eventGroup;
+      if (curr->GetType() & NS_HANDLER_TYPE_XBL_COMMAND)
+        receiver->GetSystemEventGroup(getter_AddRefs(eventGroup));
+
       if (found) { 
         /*
         // Disable ATTACHTO capability for Mozilla 1.0
@@ -894,78 +896,101 @@ nsXBLBinding::InstallEventHandlers()
         }
         */
 
-        // Add the event listener.
         if (iid.Equals(NS_GET_IID(nsIDOMMouseListener))) {
           nsXBLMouseHandler* mouseHandler;
           NS_NewXBLMouseHandler(receiver, curr, &mouseHandler);
-          receiver->AddEventListener(type, (nsIDOMMouseListener*)mouseHandler, useCapture);
+          target->AddGroupedEventListener(type,
+                                          (nsIDOMMouseListener*)mouseHandler,
+                                          useCapture, eventGroup);
           handler = mouseHandler;
         }
         else if(iid.Equals(NS_GET_IID(nsIDOMKeyListener))) {
           nsXBLKeyHandler* keyHandler;
           NS_NewXBLKeyHandler(receiver, curr, &keyHandler);
-          receiver->AddEventListener(type, (nsIDOMKeyListener*)keyHandler, useCapture);
+          target->AddGroupedEventListener(type,
+                                          (nsIDOMKeyListener*)keyHandler,
+                                          useCapture, eventGroup);
           handler = keyHandler;
         }
         else if (iid.Equals(NS_GET_IID(nsIDOMMouseMotionListener))) {
           nsXBLMouseMotionHandler* mouseHandler;
           NS_NewXBLMouseMotionHandler(receiver, curr, &mouseHandler);
-          receiver->AddEventListener(type, (nsIDOMMouseListener*)mouseHandler, useCapture);
+          target->AddGroupedEventListener(type,
+                                          (nsIDOMMouseListener*)mouseHandler,
+                                          useCapture, eventGroup);
           handler = mouseHandler;
         }
         else if(iid.Equals(NS_GET_IID(nsIDOMFocusListener))) {
           nsXBLFocusHandler* focusHandler;
           NS_NewXBLFocusHandler(receiver, curr, &focusHandler);
-          receiver->AddEventListener(type, (nsIDOMFocusListener*)focusHandler, useCapture);
+          target->AddGroupedEventListener(type,
+                                          (nsIDOMFocusListener*)focusHandler,
+                                          useCapture, eventGroup);
           handler = focusHandler;
         }
         else if (iid.Equals(NS_GET_IID(nsIDOMXULListener))) {
           nsXBLXULHandler* xulHandler;
           NS_NewXBLXULHandler(receiver, curr, &xulHandler);
-          receiver->AddEventListener(type, (nsIDOMXULListener*)xulHandler, useCapture);
+          target->AddGroupedEventListener(type,
+                                           (nsIDOMXULListener*)xulHandler,
+                                           useCapture, eventGroup);
           handler = xulHandler;
         }
         else if (iid.Equals(NS_GET_IID(nsIDOMScrollListener))) {
           nsXBLScrollHandler* scrollHandler;
           NS_NewXBLScrollHandler(receiver, curr, &scrollHandler);
-          receiver->AddEventListener(type, (nsIDOMScrollListener*)scrollHandler, useCapture);
+          target->AddGroupedEventListener(type,
+                                          (nsIDOMScrollListener*)scrollHandler,
+                                          useCapture, eventGroup);
           handler = scrollHandler;
         }
         else if (iid.Equals(NS_GET_IID(nsIDOMFormListener))) {
           nsXBLFormHandler* formHandler;
           NS_NewXBLFormHandler(receiver, curr, &formHandler);
-          receiver->AddEventListener(type, (nsIDOMFormListener*)formHandler, useCapture);
+          target->AddGroupedEventListener(type,
+                                          (nsIDOMFormListener*)formHandler,
+                                          useCapture, eventGroup);
           handler = formHandler;
         }
         else if(iid.Equals(NS_GET_IID(nsIDOMDragListener))) {
           nsXBLDragHandler* dragHandler;
           NS_NewXBLDragHandler(receiver, curr, &dragHandler);
-          receiver->AddEventListener(type, (nsIDOMDragListener*)dragHandler, useCapture);
+          target->AddGroupedEventListener(type,
+                                          (nsIDOMDragListener*)dragHandler,
+                                          useCapture, eventGroup);
           handler = dragHandler;
         }
         else if(iid.Equals(NS_GET_IID(nsIDOMLoadListener))) {
           nsXBLLoadHandler* loadHandler;
           NS_NewXBLLoadHandler(receiver, curr, &loadHandler);
-          receiver->AddEventListener(type, (nsIDOMLoadListener*)loadHandler, useCapture);
+          target->AddGroupedEventListener(type,
+                                          (nsIDOMLoadListener*)loadHandler,
+                                          useCapture, eventGroup);
           handler = loadHandler;
         }
         else if(iid.Equals(NS_GET_IID(nsIDOMMutationListener))) {
           nsXBLMutationHandler* mutationHandler;
           NS_NewXBLMutationHandler(receiver, curr, &mutationHandler);
-          receiver->AddEventListener(type, (nsIDOMMutationListener*)mutationHandler, useCapture);
+          target->AddGroupedEventListener(type,
+                                          (nsIDOMMutationListener*)mutationHandler,
+                                          useCapture, eventGroup);
           handler = mutationHandler;
         }
         else if(iid.Equals(NS_GET_IID(nsIDOMContextMenuListener))) {
           nsXBLContextMenuHandler* menuHandler;
           NS_NewXBLContextMenuHandler(receiver, curr, &menuHandler);
-          receiver->AddEventListener(type, (nsIDOMContextMenuListener*)menuHandler, useCapture);
+          target->AddGroupedEventListener(type,
+                                          (nsIDOMContextMenuListener*)menuHandler,
+                                          useCapture, eventGroup);
           handler = menuHandler;
         }
       }
       else {
         nsXBLCustomHandler* customHandler;
         NS_NewXBLCustomHandler(receiver, curr, &customHandler);
-        receiver->AddEventListener(type, (nsIDOMEventListener*)customHandler, useCapture);
+        target->AddGroupedEventListener(type,
+                                        (nsIDOMEventListener*)customHandler,
+                                        useCapture, eventGroup);
         handler = customHandler;
       }
 
@@ -1028,9 +1053,9 @@ nsXBLBinding::AttributeChanged(nsIAtom* aAttribute, PRInt32 aNameSpaceID,
     return NS_OK;
   }
 
-  return mPrototypeBinding->AttributeChanged(aAttribute, aNameSpaceID,
-                                             aRemoveFlag, mBoundElement,
-                                             mContent, aNotify);
+  mPrototypeBinding->AttributeChanged(aAttribute, aNameSpaceID, aRemoveFlag,
+                                      mBoundElement, mContent, aNotify);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -1180,8 +1205,10 @@ nsXBLBinding::InheritsStyle(PRBool* aResult)
   // Most derived binding with anonymous content determines style inheritance for now.
 
   // XXX What about bindings with <content> but no kids, e.g., my treecell-text binding?
-  if (mContent)
-    return mPrototypeBinding->InheritsStyle(aResult);
+  if (mContent) {
+    *aResult = mPrototypeBinding->InheritsStyle();
+    return NS_OK;
+  }
   
   if (mNextBinding)
     return mNextBinding->InheritsStyle(aResult);
@@ -1199,39 +1226,13 @@ nsXBLBinding::WalkRules(nsISupportsArrayEnumFunc aFunc, void* aData)
       return rv;
   }
 
-  nsCOMPtr<nsISupportsArray> rules;
-  mPrototypeBinding->GetRuleProcessors(getter_AddRefs(rules));
+  nsCOMPtr<nsISupportsArray> rules = mPrototypeBinding->GetRuleProcessors();
   if (rules)
     rules->EnumerateForwards(aFunc, aData);
   
   return rv;
 }
 
-NS_IMETHODIMP
-nsXBLBinding::AttributeAffectsStyle(nsISupportsArrayEnumFunc aFunc, void* aData, PRBool* aAffects)
-{
-  nsresult rv = NS_OK;
-  if (mNextBinding) {
-    rv = mNextBinding->AttributeAffectsStyle(aFunc, aData, aAffects);
-    if (NS_FAILED(rv))
-      return rv;
-
-    if (*aAffects)
-      return NS_OK;
-  }
-
-  nsCOMPtr<nsISupportsArray> sheets;
-  mPrototypeBinding->GetStyleSheets(getter_AddRefs(sheets));
-  if (sheets) {
-    if (!sheets->EnumerateForwards(aFunc, aData))
-      *aAffects = PR_TRUE;
-    else
-      *aAffects = PR_FALSE;
-  }
-  
-  return rv;
-}
-
 // Internal helper methods ////////////////////////////////////////////////////////////////
 
 // static
@@ -1387,8 +1388,7 @@ nsXBLBinding::InitClass(const nsCString& aClassName,
 void
 nsXBLBinding::GetImmediateChild(nsIAtom* aTag, nsIContent** aResult) 
 {
-  nsCOMPtr<nsIContent> binding;
-  mPrototypeBinding->GetBindingElement(getter_AddRefs(binding));
+  nsCOMPtr<nsIContent> binding = mPrototypeBinding->GetBindingElement();
 
   *aResult = nsnull;
   PRInt32 childCount;
@@ -1553,7 +1553,7 @@ nsXBLBinding::GetInsertionPoint(nsIContent* aChild, nsIContent** aResult, PRUint
   *aResult = nsnull;
   *aDefaultContent = nsnull;
   if (mContent)
-    return mPrototypeBinding->GetInsertionPoint(mBoundElement, mContent, aChild, aResult, aIndex, aDefaultContent);
+    mPrototypeBinding->GetInsertionPoint(mBoundElement, mContent, aChild, aResult, aIndex, aDefaultContent);
   else if (mNextBinding)
     return mNextBinding->GetInsertionPoint(aChild, aResult, aIndex, aDefaultContent);
   return NS_OK;
@@ -1567,8 +1567,8 @@ nsXBLBinding::GetSingleInsertionPoint(nsIContent** aResult, PRUint32* aIndex, PR
   *aDefaultContent = nsnull;
   *aMultipleInsertionPoints = PR_FALSE;
   if (mContent)
-    return mPrototypeBinding->GetSingleInsertionPoint(mBoundElement, mContent, aResult, aIndex, 
-                                                      aMultipleInsertionPoints, aDefaultContent);
+    mPrototypeBinding->GetSingleInsertionPoint(mBoundElement, mContent, aResult, aIndex, 
+                                               aMultipleInsertionPoints, aDefaultContent);
   else if (mNextBinding)
     return mNextBinding->GetSingleInsertionPoint(aResult, aIndex, aMultipleInsertionPoints, aDefaultContent);
   return NS_OK;
@@ -1618,7 +1618,7 @@ nsXBLBinding::MarkedForDeath(PRBool* aResult)
 NS_IMETHODIMP
 nsXBLBinding::ImplementsInterface(REFNSIID aIID, PRBool* aResult)
 {
-  mPrototypeBinding->ImplementsInterface(aIID, aResult);
+  *aResult = mPrototypeBinding->ImplementsInterface(aIID);
   if (!*aResult && mNextBinding)
     return mNextBinding->ImplementsInterface(aIID, aResult);
   return NS_OK;
@@ -1642,7 +1642,7 @@ nsXBLBinding::ShouldBuildChildFrames(PRBool* aResult)
 {
   *aResult = PR_TRUE;
   if (mContent)
-    return mPrototypeBinding->ShouldBuildChildFrames(aResult);
+    *aResult = mPrototypeBinding->ShouldBuildChildFrames();
   else if (mNextBinding) 
     return mNextBinding->ShouldBuildChildFrames(aResult);
 
@@ -1652,7 +1652,7 @@ nsXBLBinding::ShouldBuildChildFrames(PRBool* aResult)
 // Creation Routine ///////////////////////////////////////////////////////////////////////
 
 nsresult
-NS_NewXBLBinding(nsIXBLPrototypeBinding* aBinding, nsIXBLBinding** aResult)
+NS_NewXBLBinding(nsXBLPrototypeBinding* aBinding, nsIXBLBinding** aResult)
 {
   *aResult = new nsXBLBinding(aBinding);
   if (!*aResult)
