@@ -38,11 +38,9 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsCOMPtr.h"
-#include "nsIXBLPrototypeHandler.h"
+#include "nsXBLPrototypeHandler.h"
 #include "nsXBLContextMenuHandler.h"
 #include "nsIContent.h"
-#include "nsIAtom.h"
-#include "nsIDOMMouseEvent.h"
 #include "nsIScriptContext.h"
 #include "nsIScriptObjectOwner.h"
 #include "nsIScriptGlobalObject.h"
@@ -68,8 +66,9 @@
 PRUint32 nsXBLContextMenuHandler::gRefCnt = 0;
 nsIAtom* nsXBLContextMenuHandler::kContextMenuAtom = nsnull;
 
-nsXBLContextMenuHandler::nsXBLContextMenuHandler(nsIDOMEventReceiver* aReceiver, nsIXBLPrototypeHandler* aHandler)
-:nsXBLEventHandler(aReceiver,aHandler)
+nsXBLContextMenuHandler::nsXBLContextMenuHandler(nsIDOMEventReceiver* aReceiver,
+                                                 nsXBLPrototypeHandler* aHandler)
+  : nsXBLEventHandler(aReceiver, aHandler)
 {
   gRefCnt++;
   if (gRefCnt == 1) {
@@ -87,33 +86,17 @@ nsXBLContextMenuHandler::~nsXBLContextMenuHandler()
 
 NS_IMPL_ISUPPORTS_INHERITED1(nsXBLContextMenuHandler, nsXBLEventHandler, nsIDOMContextMenuListener)
 
-static inline nsresult DoContextMenu(nsIAtom* aEventType, nsIXBLPrototypeHandler* aHandler, 
-                                     nsIDOMEvent* aContextMenuEvent,
-                                     nsIDOMEventReceiver* aReceiver)
-{
-  if (!aHandler)
-    return NS_OK;
-
-  PRBool matched = PR_FALSE;
-  nsCOMPtr<nsIDOMMouseEvent> contextMenu(do_QueryInterface(aContextMenuEvent));
-  aHandler->MouseEventMatched(aEventType, contextMenu, &matched);
-
-  if (matched)
-    aHandler->ExecuteHandler(aReceiver, aContextMenuEvent);
-  
-  return NS_OK;
-}
-
 nsresult nsXBLContextMenuHandler::ContextMenu(nsIDOMEvent* aContextMenuEvent)
 {
-  return DoContextMenu(kContextMenuAtom, mProtoHandler, aContextMenuEvent, mEventReceiver);
+  return DoMouse(kContextMenuAtom, aContextMenuEvent);
 }
 
 ///////////////////////////////////////////////////////////////////////////////////
 
 nsresult
-NS_NewXBLContextMenuHandler(nsIDOMEventReceiver* aRec, nsIXBLPrototypeHandler* aHandler,
-                      nsXBLContextMenuHandler** aResult)
+NS_NewXBLContextMenuHandler(nsIDOMEventReceiver* aRec,
+                            nsXBLPrototypeHandler* aHandler,
+                            nsXBLContextMenuHandler** aResult)
 {
   *aResult = new nsXBLContextMenuHandler(aRec, aHandler);
   if (!*aResult)
