@@ -95,26 +95,32 @@ const PRInt32 nsXBLPrototypeHandler::cControl = (1<<3);
 const PRInt32 nsXBLPrototypeHandler::cMeta = (1<<4);
 const PRInt32 nsXBLPrototypeHandler::cAllModifiers = cShift | cAlt | cControl | cMeta;
 
-nsXBLPrototypeHandler::nsXBLPrototypeHandler(const PRUnichar* aEvent, const PRUnichar* aPhase,
-                                             const PRUnichar* aAction, const PRUnichar* aCommand,
-                                             const PRUnichar* aKeyCode, const PRUnichar* aCharCode,
-                                             const PRUnichar* aModifiers, const PRUnichar* aButton,
-                                             const PRUnichar* aClickCount, const PRUnichar* aPreventDefault)
+nsXBLPrototypeHandler::nsXBLPrototypeHandler(const PRUnichar* aEvent,
+                                             const PRUnichar* aPhase,
+                                             const PRUnichar* aAction,
+                                             const PRUnichar* aCommand,
+                                             const PRUnichar* aKeyCode,
+                                             const PRUnichar* aCharCode,
+                                             const PRUnichar* aModifiers,
+                                             const PRUnichar* aButton,
+                                             const PRUnichar* aClickCount,
+                                             const PRUnichar* aPreventDefault)
+  : mHandlerText(nsnull),
+    mNextHandler(nsnull)
 {
   ++gRefCnt;
   if (gRefCnt == 1)
     // Get the primary accelerator key.
     InitAccessKeys();
 
-  mHandlerText = nsnull;
-
-  ConstructPrototype(nsnull, aEvent, aPhase, aAction, aCommand, aKeyCode, aCharCode, aModifiers,
-                     aButton, aClickCount, aPreventDefault);
+  ConstructPrototype(nsnull, aEvent, aPhase, aAction, aCommand, aKeyCode,
+                     aCharCode, aModifiers, aButton, aClickCount,
+                     aPreventDefault);
 }
 
 nsXBLPrototypeHandler::nsXBLPrototypeHandler(nsIContent* aHandlerElement)
+  : mNextHandler(nsnull)
 {
-  
   ++gRefCnt;
   if (gRefCnt == 1)
     // Get the primary accelerator key.
@@ -129,23 +135,26 @@ nsXBLPrototypeHandler::~nsXBLPrototypeHandler()
   --gRefCnt;
   if (!(mType & NS_HANDLER_TYPE_XUL) && mHandlerText)
     nsMemory::Free(mHandlerText);
+
+  // We own the next handler in the chain, so delete it now.
+  delete mNextHandler;
 }
 
-NS_IMPL_ISUPPORTS1(nsXBLPrototypeHandler, nsIXBLPrototypeHandler)
-
-NS_IMETHODIMP
-nsXBLPrototypeHandler::GetHandlerElement(nsIContent** aResult)
+already_AddRefed<nsIContent>
+nsXBLPrototypeHandler::GetHandlerElement()
 {
+  nsIContent* result;
   if (mType & NS_HANDLER_TYPE_XUL) {
-    *aResult = mHandlerElement;
-    NS_IF_ADDREF(*aResult);
+    result = mHandlerElement;
+    NS_IF_ADDREF(result);
   }
   else
-    *aResult = nsnull;
-  return NS_OK;
+    result = nsnull;
+
+  return result;
 }
 
-NS_IMETHODIMP
+void
 nsXBLPrototypeHandler::AppendHandlerText(const nsAString& aText) 
 {
   if (mHandlerText) {
@@ -156,22 +165,6 @@ nsXBLPrototypeHandler::AppendHandlerText(const nsAString& aText)
   }
   else
     mHandlerText = ToNewUnicode(aText);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsXBLPrototypeHandler::GetNextHandler(nsIXBLPrototypeHandler** aResult)
-{
-  *aResult = mNextHandler;
-  NS_IF_ADDREF(*aResult);
-  return NS_OK;
-}
-  
-NS_IMETHODIMP
-nsXBLPrototypeHandler::SetNextHandler(nsIXBLPrototypeHandler* aHandler)
-{
-  mNextHandler = aHandler;
-  return NS_OK;
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -201,7 +194,7 @@ nsXBLPrototypeHandler::InitAccessKeys()
   }
 }
 
-NS_IMETHODIMP
+nsresult
 nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
                                       nsIDOMEvent* aEvent)
 {
@@ -292,7 +285,8 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
     command.AssignWithConversion(mHandlerText);
     if (focusController)
       focusController->GetControllerForCommand(command.get(), getter_AddRefs(controller));
-    else GetController(aReceiver, getter_AddRefs(controller)); // We're attached to the receiver possibly.
+    else
+      controller = GetController(aReceiver); // We're attached to the receiver possibly.
 
     nsAutoString type;
     mEventName->ToString(type);
@@ -455,15 +449,15 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXBLPrototypeHandler::GetEventName(nsIAtom** aResult)
+already_AddRefed<nsIAtom>
+nsXBLPrototypeHandler::GetEventName()
 {
-  *aResult = mEventName;
-  NS_IF_ADDREF(*aResult);
-  return NS_OK;
+  nsIAtom* eventName = mEventName;
+  NS_IF_ADDREF(eventName);
+  return eventName;
 }
 
-NS_IMETHODIMP
+nsresult
 nsXBLPrototypeHandler::BindingAttached(nsIDOMEventReceiver* aReceiver)
 {
   nsresult ret;
@@ -503,7 +497,7 @@ nsXBLPrototypeHandler::BindingAttached(nsIDOMEventReceiver* aReceiver)
   return NS_OK;
 }
 
-NS_IMETHODIMP
+nsresult
 nsXBLPrototypeHandler::BindingDetached(nsIDOMEventReceiver* aReceiver)
 {
   nsresult ret;
@@ -543,8 +537,8 @@ nsXBLPrototypeHandler::BindingDetached(nsIDOMEventReceiver* aReceiver)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXBLPrototypeHandler::GetController(nsIDOMEventReceiver* aReceiver, nsIController** aResult)
+already_AddRefed<nsIController>
+nsXBLPrototypeHandler::GetController(nsIDOMEventReceiver* aReceiver)
 {
   // XXX Fix this so there's a generic interface that describes controllers, 
   // This code should have no special knowledge of what objects might have controllers.
@@ -575,27 +569,24 @@ nsXBLPrototypeHandler::GetController(nsIDOMEventReceiver* aReceiver, nsIControll
   // Return the first controller.
   // XXX This code should be checking the command name and using supportscommand and
   // iscommandenabled.
+  nsIController* controller;
   if (controllers) {
-    controllers->GetControllerAt(0, aResult);
+    controllers->GetControllerAt(0, &controller);  // return reference
   }
-  else *aResult = nsnull;
+  else controller = nsnull;
 
-  return NS_OK;
+  return controller;
 }
 
 
-NS_IMETHODIMP
-nsXBLPrototypeHandler::KeyEventMatched(nsIAtom* aEventType, nsIDOMKeyEvent* aKeyEvent, PRBool* aResult)
+PRBool
+nsXBLPrototypeHandler::KeyEventMatched(nsIAtom* aEventType, nsIDOMKeyEvent* aKeyEvent)
 {
-  *aResult = PR_TRUE;
-
-  if (aEventType != mEventName.get()) {
-    *aResult = PR_FALSE;
-    return NS_OK;
-  }
+  if (aEventType != mEventName.get())
+    return PR_FALSE;
 
   if (mDetail == -1 && mMisc == 0 && mKeyMask == 0)
-    return NS_OK; // No filters set up. It's generic.
+    return PR_TRUE; // No filters set up. It's generic.
 
   // Get the keycode and charcode of the key event.
   PRUint32 keyCode, charCode;
@@ -604,10 +595,8 @@ nsXBLPrototypeHandler::KeyEventMatched(nsIAtom* aEventType, nsIDOMKeyEvent* aKey
 
   PRBool keyMatched = (mDetail == PRInt32(mMisc ? charCode : keyCode));
 
-  if (!keyMatched) {
-    *aResult = PR_FALSE;
-    return NS_OK;
-  }
+  if (!keyMatched)
+    return PR_FALSE;
 
   // Now check modifier keys
   PRInt32 modKeys = cAllModifiers;
@@ -615,41 +604,30 @@ nsXBLPrototypeHandler::KeyEventMatched(nsIAtom* aEventType, nsIDOMKeyEvent* aKey
   // in the handler.
   if (mMisc && !(mKeyMask & cShift))
     modKeys &= ~cShift;
-  PRBool result = ModifiersMatchMask(aKeyEvent, modKeys);
-  *aResult = result;
-  return NS_OK;
+
+  return ModifiersMatchMask(aKeyEvent, modKeys);
 }
 
-NS_IMETHODIMP
-nsXBLPrototypeHandler::MouseEventMatched(nsIAtom* aEventType, nsIDOMMouseEvent* aMouseEvent, PRBool* aResult)
+PRBool
+nsXBLPrototypeHandler::MouseEventMatched(nsIAtom* aEventType, nsIDOMMouseEvent* aMouseEvent)
 {
-  *aResult = PR_TRUE;
-
-  if (aEventType != mEventName.get()) {
-    *aResult = PR_FALSE;
-    return NS_OK;
-  }
+  if (aEventType != mEventName.get())
+    return PR_FALSE;
 
   if (mDetail == -1 && mMisc == 0 && mKeyMask == 0)
-    return NS_OK; // No filters set up. It's generic.
+    return PR_TRUE; // No filters set up. It's generic.
 
   unsigned short button;
   aMouseEvent->GetButton(&button);
-  if (mDetail != -1 && (button != mDetail)) {
-    *aResult = PR_FALSE;
-    return NS_OK;
-  }
+  if (mDetail != -1 && (button != mDetail))
+    return PR_FALSE;
 
   PRInt32 clickcount;
   aMouseEvent->GetDetail(&clickcount);
-  if (mMisc != 0 && (clickcount != mMisc)) {
-    *aResult = PR_FALSE;
-    return NS_OK;
-  }
+  if (mMisc != 0 && (clickcount != mMisc))
+    return PR_FALSE;
   
-  PRBool result = ModifiersMatchMask(aMouseEvent, cAllModifiers);
-  *aResult = result;
-  return NS_OK;
+  return ModifiersMatchMask(aMouseEvent, cAllModifiers);
 }
 
 struct keyCodeData {
@@ -832,11 +810,16 @@ nsXBLPrototypeHandler::GetEventType(nsAString& aEvent)
 
 void
 nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement, 
-                                          const PRUnichar* aEvent, const PRUnichar* aPhase,
-                                          const PRUnichar* aAction, const PRUnichar* aCommand,
-                                          const PRUnichar* aKeyCode, const PRUnichar* aCharCode,
-                                          const PRUnichar* aModifiers, const PRUnichar* aButton,
-                                          const PRUnichar* aClickCount, const PRUnichar* aPreventDefault)
+                                          const PRUnichar* aEvent,
+                                          const PRUnichar* aPhase,
+                                          const PRUnichar* aAction,
+                                          const PRUnichar* aCommand,
+                                          const PRUnichar* aKeyCode,
+                                          const PRUnichar* aCharCode,
+                                          const PRUnichar* aModifiers,
+                                          const PRUnichar* aButton,
+                                          const PRUnichar* aClickCount,
+                                          const PRUnichar* aPreventDefault)
 {
   mType = 0;
 
@@ -1004,33 +987,3 @@ nsXBLPrototypeHandler::GetTextData(nsIContent *aParent, nsString& aResult)
   }
   return NS_OK;
 }
-
-///////////////////////////////////////////////////////////////////////////////////
-
-nsresult
-NS_NewXBLPrototypeHandler(const PRUnichar* aEvent, const PRUnichar* aPhase,
-                          const PRUnichar* aAction, const PRUnichar* aCommand,
-                          const PRUnichar* aKeyCode, const PRUnichar* aCharCode,
-                          const PRUnichar* aModifiers, const PRUnichar* aButton,
-                          const PRUnichar* aClickCount, const PRUnichar* aPreventDefault,
-                          nsIXBLPrototypeHandler** aResult)
-{
-  *aResult = new nsXBLPrototypeHandler(aEvent, aPhase, aAction, aCommand, aKeyCode,
-                                       aCharCode, aModifiers, aButton, aClickCount,
-                                       aPreventDefault);
-  if (!*aResult)
-    return NS_ERROR_OUT_OF_MEMORY;
-  NS_ADDREF(*aResult);
-  return NS_OK;
-}
-
-nsresult
-NS_NewXULKeyHandler(nsIContent* aHandlerElement, nsIXBLPrototypeHandler** aResult)
-{
-  *aResult = new nsXBLPrototypeHandler(aHandlerElement);
-  if (!*aResult)
-    return NS_ERROR_OUT_OF_MEMORY;
-  NS_ADDREF(*aResult);
-  return NS_OK;
-}
-
