@@ -57,11 +57,13 @@
 #include "nsXBLAtoms.h"
 #include "nsINameSpaceManager.h"
 #include "nsIFrameManager.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
+#include "nsXBLPrototypeBinding.h"
 
 NS_IMPL_ISUPPORTS1(nsXBLResourceLoader, nsICSSLoaderObserver)
 
-nsXBLResourceLoader::nsXBLResourceLoader(nsIXBLPrototypeBinding* aBinding, nsXBLPrototypeResources* aResources)
+nsXBLResourceLoader::nsXBLResourceLoader(nsXBLPrototypeBinding* aBinding,
+                                         nsXBLPrototypeResources* aResources)
 :mBinding(aBinding),
  mResources(aResources),
  mResourceList(nsnull),
@@ -95,8 +97,7 @@ nsXBLResourceLoader::LoadResources(PRBool* aResult)
   nsCOMPtr<imgILoader> il;
   nsCOMPtr<nsICSSLoader> cssLoader;
 
-  nsCOMPtr<nsIXBLDocumentInfo> info;
-  mBinding->GetXBLDocumentInfo(nsnull, getter_AddRefs(info));
+  nsCOMPtr<nsIXBLDocumentInfo> info = mBinding->GetXBLDocumentInfo(nsnull);
   if (!info) {
     mInLoadResourcesFunc = PR_FALSE;
     return;
@@ -279,8 +280,7 @@ nsXBLResourceLoader::NotifyBoundElements()
             // Check to see if it's in the undisplayed content map.
             nsCOMPtr<nsIFrameManager> frameManager;
             shell->GetFrameManager(getter_AddRefs(frameManager));
-            nsCOMPtr<nsIStyleContext> sc;
-            frameManager->GetUndisplayedContent(content, getter_AddRefs(sc));
+            nsStyleContext* sc = frameManager->GetUndisplayedContent(content);
             if (!sc) {
               nsCOMPtr<nsIDocumentObserver> obs(do_QueryInterface(shell));
               obs->ContentInserted(doc, parent, content, index);
