@@ -42,7 +42,6 @@
 #include "nsCOMPtr.h"
 #include "nsNetUtil.h"
 #include "nsXBLService.h"
-#include "nsIXBLPrototypeHandler.h"
 #include "nsXBLWindowKeyHandler.h"
 #include "nsXBLWindowDragHandler.h"
 #include "nsIInputStream.h"
@@ -72,7 +71,7 @@
 #include "nsIDOMNodeList.h"
 #include "nsXBLContentSink.h"
 #include "nsIXBLBinding.h"
-#include "nsIXBLPrototypeBinding.h"
+#include "nsXBLPrototypeBinding.h"
 #include "nsIXBLDocumentInfo.h"
 #include "nsXBLAtoms.h"
 #include "nsXULAtoms.h"
@@ -80,18 +79,17 @@
 #include "nsContentUtils.h"
 #include "nsISyncLoadDOMService.h"
 
-#include "nsIXBLPrototypeHandler.h"
-
 #include "nsIPrefBranch.h"
 #include "nsIPrefService.h"
 
 #include "nsIPresShell.h"
 #include "nsIDocumentObserver.h"
 #include "nsIFrameManager.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 
 #include "nsIXULPrototypeCache.h"
 #include "nsIDOMLoadListener.h"
+#include "nsIDOMEventGroup.h"
 
 // Static IIDs/CIDs. Try to minimize these.
 static NS_DEFINE_CID(kXMLDocumentCID,             NS_XMLDOCUMENT_CID);
@@ -173,8 +171,7 @@ public:
         // Check to see if it's in the undisplayed content map.
         nsCOMPtr<nsIFrameManager> frameManager;
         shell->GetFrameManager(getter_AddRefs(frameManager));
-        nsCOMPtr<nsIStyleContext> sc;
-        frameManager->GetUndisplayedContent(mBoundElement, getter_AddRefs(sc));
+        nsStyleContext* sc = frameManager->GetUndisplayedContent(mBoundElement);
         if (!sc) {
           nsCOMPtr<nsIDocumentObserver> obs(do_QueryInterface(shell));
           obs->ContentInserted(doc, parent, mBoundElement, index);
@@ -473,7 +470,7 @@ PRUint32 nsXBLService::gClassLRUListQuota = 64;
 
 // Enabled by default. Must be over-ridden to disable
 PRBool nsXBLService::gDisableChromeCache = PR_FALSE;
-static const char kDisableChromeCachePref[] = "nglayout.debug.disable_xul_cache_ex";
+static const char kDisableChromeCachePref[] = "nglayout.debug.disable_xul_cache";
 
 // Implement our nsISupports methods
 NS_IMPL_ISUPPORTS3(nsXBLService, nsIXBLService, nsIObserver, nsISupportsWeakReference)
@@ -627,7 +624,7 @@ nsXBLService::LoadBindings(nsIContent* aContent, const nsAString& aURL, PRBool a
   return NS_OK; 
 }
 
-NS_IMETHODIMP
+nsresult
 nsXBLService::FlushStyleBindings(nsIContent* aContent)
 {
   nsCOMPtr<nsIDocument> document;
@@ -680,7 +677,7 @@ nsXBLService::ResolveTag(nsIContent* aContent, PRInt32* aNameSpaceID, nsIAtom**
   return NS_OK;
 }
 
-NS_IMETHODIMP
+nsresult
 nsXBLService::GetXBLDocumentInfo(const nsCString& aURLStr, nsIContent* aBoundElement, nsIXBLDocumentInfo** aResult)
 {
   *aResult = nsnull;
@@ -741,9 +738,16 @@ nsXBLService::AttachGlobalKeyHandler(nsIDOMEventReceiver* aReceiver)
     return NS_ERROR_FAILURE;
 
   // listen to these events
-  rec->AddEventListener(NS_LITERAL_STRING("keydown"), handler, PR_FALSE);
-  rec->AddEventListener(NS_LITERAL_STRING("keyup"), handler, PR_FALSE);
-  rec->AddEventListener(NS_LITERAL_STRING("keypress"), handler, PR_FALSE);
+  nsCOMPtr<nsIDOMEventGroup> systemGroup;
+  rec->GetSystemEventGroup(getter_AddRefs(systemGroup));
+  nsCOMPtr<nsIDOM3EventTarget> target = do_QueryInterface(rec);
+
+  target->AddGroupedEventListener(NS_LITERAL_STRING("keydown"), handler,
+                                  PR_FALSE, systemGroup);
+  target->AddGroupedEventListener(NS_LITERAL_STRING("keyup"), handler, 
+                                  PR_FALSE, systemGroup);
+  target->AddGroupedEventListener(NS_LITERAL_STRING("keypress"), handler, 
+                                  PR_FALSE, systemGroup);
 
   // Release.  Do this so that only the event receiver holds onto the key handler.
   NS_RELEASE(handler);
@@ -767,12 +771,21 @@ nsXBLService::AttachGlobalDragHandler(nsIDOMEventReceiver* aReceiver)
   if (!handler)
     return NS_ERROR_FAILURE;
 
+  nsCOMPtr<nsIDOMEventGroup> systemGroup;
+  aReceiver->GetSystemEventGroup(getter_AddRefs(systemGroup));
+  nsCOMPtr<nsIDOM3EventTarget> target = do_QueryInterface(aReceiver);
+
   // listen to these events
-  aReceiver->AddEventListener(NS_LITERAL_STRING("draggesture"), handler, PR_FALSE);
-  aReceiver->AddEventListener(NS_LITERAL_STRING("dragenter"), handler, PR_FALSE);
-  aReceiver->AddEventListener(NS_LITERAL_STRING("dragexit"), handler, PR_FALSE);
-  aReceiver->AddEventListener(NS_LITERAL_STRING("dragover"), handler, PR_FALSE);
-  aReceiver->AddEventListener(NS_LITERAL_STRING("dragdrop"), handler, PR_FALSE);
+  target->AddGroupedEventListener(NS_LITERAL_STRING("draggesture"), handler,
+                                  PR_FALSE, systemGroup);
+  target->AddGroupedEventListener(NS_LITERAL_STRING("dragenter"), handler,
+                                  PR_FALSE, systemGroup);
+  target->AddGroupedEventListener(NS_LITERAL_STRING("dragexit"), handler,
+                                  PR_FALSE, systemGroup);
+  target->AddGroupedEventListener(NS_LITERAL_STRING("dragover"), handler,
+                                  PR_FALSE, systemGroup);
+  target->AddGroupedEventListener(NS_LITERAL_STRING("dragdrop"), handler,
+                                  PR_FALSE, systemGroup);
 
   // Release.  Do this so that only the event receiver holds onto the handler.
   NS_RELEASE(handler);
@@ -807,7 +820,7 @@ nsXBLService::FlushMemory()
 
 // Internal helper methods ////////////////////////////////////////////////////////////////
 
-NS_IMETHODIMP nsXBLService::GetBinding(nsIContent* aBoundElement, 
+nsresult nsXBLService::GetBinding(nsIContent* aBoundElement, 
                                        const nsCString& aURLStr, 
                                        nsIXBLBinding** aResult)
 {
@@ -859,19 +872,17 @@ NS_IMETHODIMP nsXBLService::GetBindingInternal(nsIContent* aBoundElement,
   PRBool allowScripts;
   docInfo->GetScriptAccess(&allowScripts);
 
-  nsCOMPtr<nsIXBLPrototypeBinding> protoBinding;
-  docInfo->GetPrototypeBinding(ref, getter_AddRefs(protoBinding));
+  nsXBLPrototypeBinding* protoBinding;
+  docInfo->GetPrototypeBinding(ref, &protoBinding);
 
   NS_ASSERTION(protoBinding, "Unable to locate an XBL binding.");
   if (!protoBinding)
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIContent> child;
-  protoBinding->GetBindingElement(getter_AddRefs(child));
+  nsCOMPtr<nsIContent> child = protoBinding->GetBindingElement();
 
   // Our prototype binding must have all its resources loaded.
-  PRBool ready;
-  protoBinding->LoadResources(&ready);
+  PRBool ready = protoBinding->LoadResources();
   if (!ready) {
     // Add our bound element to the protos list of elts that should
     // be notified when the stylesheets and scripts finish loading.
@@ -881,10 +892,8 @@ NS_IMETHODIMP nsXBLService::GetBindingInternal(nsIContent* aBoundElement,
 
   // If our prototype already has a base, then don't check for an "extends" attribute.
   nsCOMPtr<nsIXBLBinding> baseBinding;
-  nsCOMPtr<nsIXBLPrototypeBinding> baseProto;
-  PRBool hasBase;
-  protoBinding->HasBasePrototype(&hasBase);
-  protoBinding->GetBasePrototype(getter_AddRefs(baseProto));
+  PRBool hasBase = protoBinding->HasBasePrototype();
+  nsXBLPrototypeBinding* baseProto = protoBinding->GetBasePrototype();
   if (baseProto) {
     nsCAutoString url;
     baseProto->GetBindingURI(url);
@@ -964,7 +973,7 @@ NS_IMETHODIMP nsXBLService::GetBindingInternal(nsIContent* aBoundElement,
           return NS_ERROR_FAILURE; // Binding not yet ready or an error occurred.
         if (!aPeekOnly) {
           // Make sure to set the base prototype.
-          baseBinding->GetPrototypeBinding(getter_AddRefs(baseProto));
+          baseBinding->GetPrototypeBinding(&baseProto);
           protoBinding->SetBasePrototype(baseProto);
           child->UnsetAttr(kNameSpaceID_None, nsXBLAtoms::extends, PR_FALSE);
           child->UnsetAttr(kNameSpaceID_None, nsXBLAtoms::display, PR_FALSE);
@@ -1090,34 +1099,7 @@ nsXBLService::LoadBindingDocumentInfo(nsIContent* aBoundElement, nsIDocument* aB
   return NS_OK;
 }
 
-// Helper method for loading an XML doc.
-NS_IMETHODIMP
-nsXBLService::FetchSyncXMLDocument(nsIURI* aURI, nsIDocument** aResult)
-{
-  *aResult = nsnull;
-  nsresult rv = NS_OK;
-  nsCOMPtr<nsIDOMDocument> domDoc;
-  nsCOMPtr<nsISyncLoadDOMService> loader =
-    do_GetService("@mozilla.org/content/syncload-dom-service;1", &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIChannel> channel;
-  rv = NS_NewChannel(getter_AddRefs(channel), aURI, nsnull, nsnull);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = loader->LoadLocalDocument(channel, nsnull, getter_AddRefs(domDoc));
-  if (rv == NS_ERROR_FILE_NOT_FOUND) {
-      return NS_OK;
-  }
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // The document is parsed. We now have a prototype document.
-  // Everything worked, so we can just hand this back now.
-
-  return CallQueryInterface(domDoc, aResult);
-}
-  
-NS_IMETHODIMP
+nsresult
 nsXBLService::FetchBindingDocument(nsIContent* aBoundElement, nsIDocument* aBoundDocument,
                                    nsIURI* aURI, const nsCString& aRef, 
                                    PRBool aForceSyncLoad, nsIDocument** aResult)
@@ -1216,24 +1198,6 @@ nsXBLService::FetchBindingDocument(nsIContent* aBoundElement, nsIDocument* aBoun
   return CallQueryInterface(domDoc, aResult);
 }
 
-static void GetImmediateChild(nsIAtom* aTag, nsIContent* aParent, nsIContent** aResult) 
-{
-  *aResult = nsnull;
-  PRInt32 childCount;
-  aParent->ChildCount(childCount);
-  for (PRInt32 i = 0; i < childCount; i++) {
-    nsCOMPtr<nsIContent> child;
-    aParent->ChildAt(i, *getter_AddRefs(child));
-    nsCOMPtr<nsIAtom> tag;
-    child->GetTag(*getter_AddRefs(tag));
-    if (aTag == tag.get()) {
-      *aResult = child;
-      NS_ADDREF(*aResult);
-      return;
-    }
-  }
-}
-
 // Creation Routine ///////////////////////////////////////////////////////////////////////
 
 nsresult NS_NewXBLService(nsIXBLService** aResult);
