@@ -85,7 +85,6 @@
 #include "prtime.h"
 #include "prlog.h"
 #include "prmem.h"
-#include "nsXSLContentSink.h"
 #include "nsParserCIID.h"
 #include "nsParserUtils.h"
 #include "nsIScrollable.h"
@@ -94,6 +93,7 @@
 #include "nsIWebNavigation.h"
 #include "nsIScriptElement.h"
 #include "nsStyleLinkElement.h"
+#include "nsIImageLoadingContent.h"
 #include "nsEscape.h"
 #include "nsICharsetConverterManager.h"
 #include "nsICharsetConverterManager2.h"
@@ -127,7 +127,10 @@ public:
     : mInner(do_GetWeakReference(aInner))
   {
   }
-
+  virtual ~nsScriptLoaderObserverProxy()
+  {
+  }
+  
   NS_DECL_ISUPPORTS
   NS_DECL_NSISCRIPTLOADEROBSERVER
 
@@ -325,7 +328,7 @@ nsXMLContentSink::WillBuildModel(void)
 nsresult CharsetConvRef(const nsString& aDocCharset, const nsCString& aRefInDocCharset, nsString& aRefInUnicode);
 
 void
-nsXMLContentSink::ScrollToRef()
+nsXMLContentSink::ScrollToRef(PRBool aReallyScroll)
 {
   // XXX Duplicate code in nsHTMLContentSink.
   // XXX Be sure to change both places if you make changes here.
@@ -349,11 +352,13 @@ nsXMLContentSink::ScrollToRef()
       mDocument->GetShellAt(i, getter_AddRefs(shell));
       if (shell) {
         // Scroll to the anchor
-        shell->FlushPendingNotifications(PR_FALSE);
+        if (aReallyScroll) {
+          shell->FlushPendingNotifications(PR_FALSE);
+        }
 
         // Check an empty string which might be caused by the UTF-8 conversion
         if (!ref.IsEmpty())
-          rv = shell->GoToAnchor(ref, PR_TRUE);
+          rv = shell->GoToAnchor(ref, aReallyScroll);
         else
           rv = NS_ERROR_FAILURE;
 
@@ -368,7 +373,7 @@ nsXMLContentSink::ScrollToRef()
             rv = CharsetConvRef(docCharset, unescapedRef, ref);
 
             if (NS_SUCCEEDED(rv) && !ref.IsEmpty())
-              rv = shell->GoToAnchor(ref, PR_TRUE);
+              rv = shell->GoToAnchor(ref, aReallyScroll);
           }
         }
       }
@@ -406,8 +411,6 @@ nsXMLContentSink::MaybePrettyPrint()
 NS_IMETHODIMP
 nsXMLContentSink::DidBuildModel(PRInt32 aQualityLevel)
 {
-  nsresult rv = NS_OK;
-
   // XXX this is silly; who cares?
   PRInt32 i, ns = mDocument->GetNumberOfShells();
   for (i = 0; i < ns; i++) {
@@ -434,13 +437,12 @@ nsXMLContentSink::DidBuildModel(PRInt32 aQualityLevel)
   // Check if we want to prettyprint
   MaybePrettyPrint();
 
-  if (mXSLTransformMediator) {
+  if (mXSLTProcessor) {
     nsCOMPtr<nsIDOMDocument> currentDOMDoc(do_QueryInterface(mDocument));
-    mXSLTransformMediator->SetSourceContentModel(currentDOMDoc);
-    mXSLTransformMediator->SetTransformObserver(this);
-    // Since the mediator now holds a reference to us we drop our reference
+    mXSLTProcessor->SetSourceContentModel(currentDOMDoc);
+    // Since the processor now holds a reference to us we drop our reference
     // to it to avoid owning cycles
-    mXSLTransformMediator = nsnull;
+    mXSLTProcessor = nsnull;
   }
   else {
     // Kick off layout for non-XSLT transformed documents.
@@ -458,12 +460,10 @@ nsXMLContentSink::DidBuildModel(PRInt32 aQualityLevel)
     if (docShell) {
       PRUint32 documentLoadType = 0;
       docShell->GetLoadType(&documentLoadType);
-      if (!(documentLoadType & nsIDocShell::LOAD_CMD_HISTORY)) {
-        ScrollToRef();
-      }
+      ScrollToRef(!(documentLoadType & nsIDocShell::LOAD_CMD_HISTORY));
     }
 #else
-    ScrollToRef();
+    ScrollToRef(PR_TRUE);
 #endif
 
     mDocument->EndLoad();
@@ -549,11 +549,9 @@ nsXMLContentSink::OnTransformDone(nsresult aResult,
   //  Scroll to Anchor only if the document was *not* loaded through history means. 
   PRUint32 documentLoadType = 0;
   docShell->GetLoadType(&documentLoadType);
-  if (!(documentLoadType & nsIDocShell::LOAD_CMD_HISTORY)) {
-    ScrollToRef();
-  }
+  ScrollToRef(!(documentLoadType & nsIDocShell::LOAD_CMD_HISTORY));
 #else
-  ScrollToRef();
+  ScrollToRef(PR_TRUE);
 #endif
 
   originalDocument->EndLoad();
@@ -682,6 +680,23 @@ nsXMLContentSink::CreateElement(const PRUnichar** aAtts, PRUint32 aAttsCount,
       ssle->InitStyleLinkElement(mParser, PR_FALSE);
       ssle->SetEnableUpdates(PR_FALSE);
     }
+  } else if (tagAtom == nsHTMLAtoms::img ||
+             tagAtom == nsHTMLAtoms::input ||
+             tagAtom == nsHTMLAtoms::object ||
+             tagAtom == nsHTMLAtoms::applet) {
+    nsAutoString cmd;
+    if (mParser) {
+      mParser->GetCommand(cmd);
+    }
+    if (cmd.EqualsWithConversion(kLoadAsData)) {
+      // XXXbz Should this be in HandleStartElement so it applies to all
+      // elements, not just XHTML ones?  We don't have any non-XHTML
+      // image loading things yet, but....
+      nsCOMPtr<nsIImageLoadingContent> imgLoader(do_QueryInterface(htmlContent));
+      if (imgLoader) {
+        imgLoader->SetLoadingEnabled(PR_FALSE);
+      }
+    }
   }
 
   return NS_OK;
@@ -768,59 +783,23 @@ nsXMLContentSink::AddContentAsLeaf(nsIContent *aContent)
 nsresult
 nsXMLContentSink::LoadXSLStyleSheet(nsIURI* aUrl)
 {
-  nsresult rv = NS_OK;
-
-  // Create a transform mediator
-  rv = NS_NewTransformMediator(getter_AddRefs(mXSLTransformMediator),
-                               NS_LITERAL_CSTRING(kXSLType));
-  if (NS_FAILED(rv)) {
+  mXSLTProcessor =
+    do_CreateInstance("@mozilla.org/document-transformer;1?type=text/xslt");
+  if (!mXSLTProcessor) {
     // No XSLT processor available, continue normal document loading
     return NS_OK;
   }
 
-  // Enable the transform mediator. It will start the transform
-  // as soon as it has enough state to do so.  The state needed is
-  // the source content model, the style content model, the current
-  // document, and an observer.  The XML and XSL content sinks provide
-  // this state by calling the various setters on nsITransformMediator.
-  mXSLTransformMediator->SetEnabled(PR_TRUE);
+  mXSLTProcessor->SetTransformObserver(this);
 
-  // Create and set up the channel
   nsCOMPtr<nsILoadGroup> loadGroup;
-  rv = mDocument->GetDocumentLoadGroup(getter_AddRefs(loadGroup));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIChannel> channel;
-  rv = NS_NewChannel(getter_AddRefs(channel), aUrl);
-  if (NS_FAILED(rv)) return rv;
-
-  channel->SetLoadGroup(loadGroup);
-
-  nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(channel));
-  if (httpChannel) {
-    httpChannel->SetRequestHeader(NS_LITERAL_CSTRING("Accept"),
-                                  NS_LITERAL_CSTRING("text/xml,application/xml,application/xhtml+xml,*/*;q=0.1"),
-                                  PR_FALSE);
-
-    httpChannel->SetReferrer(mDocumentURL);
+  nsresult rv = mDocument->GetDocumentLoadGroup(getter_AddRefs(loadGroup));
+  if (NS_FAILED(rv)) {
+    mXSLTProcessor = nsnull;
+    return rv;
   }
 
-
-  // Create the XSL stylesheet document
-  nsCOMPtr<nsIDocument> styleDoc = do_CreateInstance(kXMLDocumentCID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // Create the XSL content sink
-  nsCOMPtr<nsIXMLContentSink> sink;
-  rv = NS_NewXSLContentSink(getter_AddRefs(sink), mXSLTransformMediator, styleDoc, aUrl, mWebShell);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIStreamListener> sl;
-  rv = styleDoc->StartDocumentLoad(kLoadAsData, channel, loadGroup, nsnull,
-                                   getter_AddRefs(sl), PR_TRUE, sink);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  return channel->AsyncOpen(sl, nsnull);
+  return mXSLTProcessor->LoadStyleSheet(aUrl, loadGroup, mDocumentURL);
 }
 
 NS_IMETHODIMP
@@ -854,7 +833,7 @@ nsXMLContentSink::ProcessStyleLink(nsIContent* aElement,
     rv = NS_NewURI(getter_AddRefs(url), aHref, nsnull, mDocumentBaseURL);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCOMPtr<nsIScriptSecurityManager> secMan = 
+    nsCOMPtr<nsIScriptSecurityManager> secMan =
       do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv, NS_OK);
 
@@ -1160,7 +1139,6 @@ nsXMLContentSink::ProcessHeaderData(nsIAtom* aHeader,const nsAString& aValue,nsI
     rv = originalCodebase->GetURI(getter_AddRefs(codebaseURI));
     NS_ENSURE_SUCCESS(rv, rv);
 
-    char *cookie = ToNewUTF8String(aValue);
     nsCOMPtr<nsIScriptGlobalObject> globalObj;
     nsCOMPtr<nsIPrompt> prompt;
     mDocument->GetScriptGlobalObject(getter_AddRefs(globalObj));
@@ -1179,8 +1157,8 @@ nsXMLContentSink::ProcessHeaderData(nsIAtom* aHeader,const nsAString& aValue,nsI
       }
     }
 
-    rv = cookieServ->SetCookieString(codebaseURI, prompt, cookie, httpChannel);
-    nsCRT::free(cookie);
+    rv = cookieServ->SetCookieString(codebaseURI, prompt, NS_ConvertUCS2toUTF8(aValue).get(), httpChannel);
+
     if (NS_FAILED(rv)) return rv;
   } // END set-cookie
   else if (aHeader == nsHTMLAtoms::link) {
@@ -1206,12 +1184,11 @@ nsXMLContentSink::ProcessHeaderData(nsIAtom* aHeader,const nsAString& aValue,nsI
     if (NS_SUCCEEDED(mParser->GetChannel(getter_AddRefs(channel)))) {
       nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(channel));
       if (httpChannel) {
-        const PRUnichar *header = 0;
-        (void)aHeader->GetUnicode(&header);
-        (void)httpChannel->SetResponseHeader(
-                       NS_ConvertUCS2toUTF8(header),
-                       NS_ConvertUCS2toUTF8(aValue),
-                       PR_TRUE);
+        const char *header = 0;
+        (void)aHeader->GetUTF8String(&header);
+        (void)httpChannel->SetResponseHeader(nsDependentCString(header),
+                                             NS_ConvertUCS2toUTF8(aValue),
+                                             PR_TRUE);
       }
     }
   }
@@ -1289,22 +1266,23 @@ nsXMLContentSink::FlushText(PRBool aCreateTextNode, PRBool* aDidFlush)
   PRBool didFlush = PR_FALSE;
   if (0 != mTextLength) {
     if (aCreateTextNode) {
-      nsCOMPtr<nsITextContent> content;
-      rv = NS_NewTextNode(getter_AddRefs(content));
-      if (NS_OK == rv) {
-        // Set the content's document
-        content->SetDocument(mDocument, PR_FALSE, PR_TRUE);
+      nsCOMPtr<nsITextContent> textContent;
+      rv = NS_NewTextNode(getter_AddRefs(textContent));
+      NS_ENSURE_SUCCESS(rv, rv);
 
-        // Set the text in the text node
-        content->SetText(mText, mTextLength, PR_FALSE);
+      // Set the content's document
+      textContent->SetDocument(mDocument, PR_FALSE, PR_TRUE);
 
-        // Add text to its parent
-        AddContentAsLeaf(content);
-      }
+      // Set the text in the text node
+      textContent->SetText(mText, mTextLength, PR_FALSE);
+
+      // Add text to its parent
+      AddContentAsLeaf(textContent);
     }
     mTextLength = 0;
     didFlush = PR_TRUE;
   }
+
   if (nsnull != aDidFlush) {
     *aDidFlush = didFlush;
   }
@@ -1802,7 +1780,7 @@ nsXMLContentSink::HandleStartElement(const PRUnichar *aName,
 
       // For XSLT, we need to wait till after the transform
       // to set the root content object.
-      if (!mXSLTransformMediator) {
+      if (!mXSLTProcessor) {
         mDocument->SetRootContent(mDocElement);
       }
     }
@@ -2086,10 +2064,10 @@ nsXMLContentSink::ReportError(const PRUnichar* aErrorText,
   }
   NS_IF_RELEASE(mDocElement); 
 
-  if (mXSLTransformMediator) {
-    // Get rid of the transform mediator.
-    mXSLTransformMediator->SetEnabled(PR_FALSE);
-    mXSLTransformMediator = nsnull;
+  if (mXSLTProcessor) {
+    // Get rid of the XSLT processor.
+    mXSLTProcessor->CancelLoads();
+    mXSLTProcessor = nsnull;
   }
 
   NS_NAMED_LITERAL_STRING(name, "xmlns");
