@@ -43,13 +43,11 @@
 #include "nsIViewManager.h"
 #include "nsIScrollableView.h"
 #include "nsWeakReference.h"
-#include "nsITransformMediator.h"
 #include "nsIUnicharInputStream.h"
 #include "nsIStreamLoader.h"
 #include "nsISupportsArray.h"
 #include "nsINodeInfo.h"
 #include "nsIDOMHTMLTextAreaElement.h"
-class nsICSSStyleSheet;
 #include "nsICSSLoaderObserver.h"
 #include "nsIHTMLContent.h"
 #include "nsIDOMHTMLScriptElement.h"
@@ -58,13 +56,13 @@ class nsICSSStyleSheet;
 #include "nsSupportsArray.h"
 #include "nsIExpatSink.h"
 #include "nsIDocumentTransformer.h"
+#include "nsIDocShell.h"
 
+class nsICSSStyleSheet;
 class nsIDocument;
 class nsIURI;
-class nsIWebShell;
 class nsIContent;
 class nsAutoVoidArray;
-class nsIXMLDocument;
 class nsIUnicharInputStream;
 class nsIParser;
 class nsINameSpace;
@@ -92,7 +90,7 @@ public:
 
   nsresult Init(nsIDocument* aDoc,
                 nsIURI* aURL,
-                nsIWebShell* aContainer,
+                nsISupports* aContainer,
                 nsIChannel* aChannel);
 
   // nsISupports
@@ -170,14 +168,14 @@ protected:
   static void
   GetElementFactory(PRInt32 aNameSpaceID, nsIElementFactory** aResult);
 
-  void ScrollToRef();
+  void ScrollToRef(PRBool aReallyScroll);
   
   nsresult MaybePrettyPrint();
   
   nsIDocument*     mDocument;
   nsIURI*          mDocumentURL;
   nsIURI*          mDocumentBaseURL; // can be set via HTTP headers
-  nsIWebShell*     mWebShell;
+  nsCOMPtr<nsIDocShell> mDocShell;
   nsIParser*       mParser;
   nsIContent*      mDocElement;
   nsAutoVoidArray* mNameSpaceStack;
@@ -204,7 +202,7 @@ protected:
 
   nsCOMPtr<nsISupportsArray>          mContentStack;
   nsCOMPtr<nsINodeInfoManager>        mNodeInfoManager;
-  nsCOMPtr<nsITransformMediator>      mXSLTransformMediator;
+  nsCOMPtr<nsIDocumentTransformer>    mXSLTProcessor;
 };
 
 #endif // nsXMLContentSink_h__
