@@ -83,6 +83,7 @@
 #include "nsDOMAttribute.h"
 #include "nsGUIEvent.h"
 #include "nsFIXptr.h"
+#include "nsXPointer.h"
 #include "nsCExternalHandlerService.h"
 #include "nsIMIMEService.h"
 #include "nsNetUtil.h"
@@ -94,18 +95,22 @@
 #include "nsIWindowWatcher.h"
 #include "nsIAuthPrompt.h"
 #include "nsIScriptGlobalObjectOwner.h"
+static NS_DEFINE_IID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
 
 // XXX The XML world depends on the html atoms
 #include "nsHTMLAtoms.h"
 
-static const char* kLoadAsData = "loadAsData";
+static const char kLoadAsData[] = "loadAsData";
+
+static NS_DEFINE_CID(kCharsetAliasCID, NS_CHARSETALIAS_CID);
+
 
 // ==================================================================
 // =
 // ==================================================================
 
 
-NS_EXPORT nsresult
+nsresult
 NS_NewDOMDocument(nsIDOMDocument** aInstancePtrResult,
                   const nsAString& aNamespaceURI, 
                   const nsAString& aQualifiedName, 
@@ -158,7 +163,7 @@ NS_NewDOMDocument(nsIDOMDocument** aInstancePtrResult,
 }
 
 
-NS_EXPORT nsresult
+nsresult
 NS_NewXMLDocument(nsIDocument** aInstancePtrResult)
 {
   nsXMLDocument* doc = new nsXMLDocument();
@@ -178,10 +183,15 @@ NS_NewXMLDocument(nsIDocument** aInstancePtrResult)
   return NS_OK;
 }
 
+  // NOTE! nsDocument::operator new() zeroes out all members, so don't
+  // bother initializing members to 0.
+
 nsXMLDocument::nsXMLDocument() 
-  : mCountCatalogSheets(0), mCrossSiteAccessEnabled(PR_FALSE),
-    mLoadedAsData(PR_FALSE), mXMLDeclarationBits(0)
+  : mAsync(PR_TRUE)
 {
+
+  // NOTE! nsDocument::operator new() zeroes out all members, so don't
+  // bother initializing members to 0.
 }
 
 nsXMLDocument::~nsXMLDocument()
@@ -195,10 +205,13 @@ nsXMLDocument::~nsXMLDocument()
   if (mCSSLoader) {
     mCSSLoader->DropDocumentReference();
   }
+
+  // XXX We rather crash than hang
+  mLoopingForSyncLoad = PR_FALSE;
 }
 
 
-// QueryInterface implementation for nsHTMLAnchorElement
+// QueryInterface implementation for nsXMLDocument
 NS_INTERFACE_MAP_BEGIN(nsXMLDocument)
   NS_INTERFACE_MAP_ENTRY(nsIXMLDocument)
   NS_INTERFACE_MAP_ENTRY(nsIHTMLContentContainer)
@@ -213,6 +226,17 @@ NS_IMPL_ADDREF_INHERITED(nsXMLDocument, nsDocument)
 NS_IMPL_RELEASE_INHERITED(nsXMLDocument, nsDocument)
 
 
+nsresult
+nsXMLDocument::Init()
+{
+  nsresult rv = nsDocument::Init();
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  mEventQService = do_GetService(kEventQueueServiceCID, &rv);
+
+  return rv;
+}
+
 NS_IMETHODIMP 
 nsXMLDocument::Reset(nsIChannel* aChannel, nsILoadGroup* aLoadGroup)
 {
@@ -331,8 +355,33 @@ nsXMLDocument::EvaluateFIXptr(const nsAString& aExpression, nsIDOMRange **aRange
 }
 
 NS_IMETHODIMP
-nsXMLDocument::Load(const nsAString& aUrl)
+nsXMLDocument::GetAsync(PRBool *aAsync)
 {
+  NS_ENSURE_ARG_POINTER(aAsync);
+  *aAsync = mAsync;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsXMLDocument::SetAsync(PRBool aAsync)
+{
+  mAsync = aAsync;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsXMLDocument::EvaluateXPointer(const nsAString& aExpression,
+                                nsIXPointerResult **aResult)
+{
+  return nsXPointer::Evaluate(this, aExpression, aResult);
+}
+
+NS_IMETHODIMP
+nsXMLDocument::Load(const nsAString& aUrl, PRBool *aReturn)
+{
+  NS_ENSURE_ARG_POINTER(aReturn);
+  *aReturn = PR_FALSE;
+
   nsCOMPtr<nsIChannel> channel;
   nsCOMPtr<nsIURI> uri;
   nsresult rv;
@@ -345,24 +394,30 @@ nsXMLDocument::Load(const nsAString& aUrl)
   nsCOMPtr<nsIScriptSecurityManager> secMan = 
            do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return rv;
-  if (NS_FAILED(secMan->CheckConnect(nsnull, uri, "XMLDocument", "load")))
-    return NS_ERROR_FAILURE;
+  rv = secMan->CheckConnect(nsnull, uri, "XMLDocument", "load");
+  if (NS_FAILED(rv)) {
+    // We need to return success here so that JS will get a proper exception
+    // thrown later. Native calls should always result in CheckConnect() succeeding,
+    // but in case JS calls C++ which calls this code the exception might be lost.
+    return NS_OK;
+  }
 
   // Partial Reset, need to restore principal for security reasons and
   // event listener manager so that load listeners etc. will remain.
-  nsCOMPtr<nsIPrincipal> principal = mPrincipal;
-  nsCOMPtr<nsIEventListenerManager> elm = mListenerManager;
+
+  nsCOMPtr<nsIPrincipal> principal(mPrincipal);
+  nsCOMPtr<nsIEventListenerManager> elm(mListenerManager);
 
   Reset(nsnull, nsnull);
   
   mPrincipal = principal;
   mListenerManager = elm;
-  NS_IF_ADDREF(mListenerManager);
   
   SetDocumentURL(uri);
   SetBaseURL(uri);
 
-  // Store script context, if any, in case we encounter redirect (because we need it there)
+  // Store script context, if any, in case we encounter redirect
+  // (because we need it there)
   nsCOMPtr<nsIJSContextStack> stack =
     do_GetService("@mozilla.org/js/xpc/ContextStack;1");
   if (stack) {
@@ -375,10 +430,11 @@ nsXMLDocument::Load(const nsAString& aUrl)
     }
   }
 
-  // Find out if UniversalBrowserRead privileges are enabled - we will need this
-  // in case of a redirect
+  // Find out if UniversalBrowserRead privileges are enabled - we will
+  // need this in case of a redirect
   PRBool crossSiteAccessEnabled;
-  rv = secMan->IsCapabilityEnabled("UniversalBrowserRead", &crossSiteAccessEnabled);
+  rv = secMan->IsCapabilityEnabled("UniversalBrowserRead",
+                                   &crossSiteAccessEnabled);
   if (NS_FAILED(rv)) return rv;
 
   mCrossSiteAccessEnabled = crossSiteAccessEnabled;
@@ -388,19 +444,30 @@ nsXMLDocument::Load(const nsAString& aUrl)
   if (NS_FAILED(rv)) return rv;
 
   // Set a principal for this document
-  mPrincipal = nsnull;
   nsCOMPtr<nsISupports> channelOwner;
   rv = channel->GetOwner(getter_AddRefs(channelOwner));
-  if (NS_SUCCEEDED(rv) && channelOwner) {
-    mPrincipal = do_QueryInterface(channelOwner, &rv);
-  }
 
-  if (NS_FAILED(rv) || !channelOwner)
-  {
+  // We don't care if GetOwner() succeeded here, if it failed,
+  // channelOwner will be null, which is what we want in that case.
+
+  mPrincipal = do_QueryInterface(channelOwner);
+
+  if (NS_FAILED(rv) || !mPrincipal) {
     rv = secMan->GetCodebasePrincipal(uri, getter_AddRefs(mPrincipal));
     NS_ENSURE_TRUE(mPrincipal, rv);
   }
 
+  nsCOMPtr<nsIEventQueue> modalEventQueue;
+
+  if(!mAsync) {
+    NS_ENSURE_TRUE(mEventQService, NS_ERROR_FAILURE);
+
+    rv = mEventQService->PushThreadEventQueue(getter_AddRefs(modalEventQueue));
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
+  }
+
   // Prepare for loading the XML document "into oneself"
   nsCOMPtr<nsIStreamListener> listener;
   if (NS_FAILED(rv = StartDocumentLoad(kLoadAsData, channel, 
@@ -408,13 +475,48 @@ nsXMLDocument::Load(const nsAString& aUrl)
                                        getter_AddRefs(listener),
                                        PR_FALSE))) {
     NS_ERROR("nsXMLDocument::Load: Failed to start the document load.");
+    if (modalEventQueue) {
+      mEventQService->PopThreadEventQueue(modalEventQueue);
+    }
     return rv;
   }
 
   // Start an asynchronous read of the XML document
   rv = channel->AsyncOpen(listener, nsnull);
+  if (NS_FAILED(rv)) {
+    if (modalEventQueue) {
+      mEventQService->PopThreadEventQueue(modalEventQueue);
+    }
+    return rv;
+  }
 
-  return rv;
+  if (!mAsync) {
+    mLoopingForSyncLoad = PR_TRUE;
+
+    while (mLoopingForSyncLoad) {
+      modalEventQueue->ProcessPendingEvents();
+    }
+
+    mEventQService->PopThreadEventQueue(modalEventQueue);
+
+    // We set return to true unless there was a parsing error
+    nsCOMPtr<nsIDOMNode> node = do_QueryInterface(mRootContent);
+    if (node) {
+      nsAutoString name, ns;      
+      if (NS_SUCCEEDED(node->GetLocalName(name)) &&
+          name.Equals(NS_LITERAL_STRING("parsererror")) &&
+          NS_SUCCEEDED(node->GetNamespaceURI(ns)) &&
+          ns.Equals(NS_LITERAL_STRING("http://www.mozilla.org/newlayout/xml/parsererror.xml"))) {
+        //return is already false
+      } else {
+        *aReturn = PR_TRUE;
+      }
+    }
+  } else {
+    *aReturn = PR_TRUE;
+  }
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP 
@@ -524,6 +626,8 @@ nsXMLDocument::StartDocumentLoad(const char* aCommand,
 NS_IMETHODIMP 
 nsXMLDocument::EndLoad()
 {
+  mLoopingForSyncLoad = PR_FALSE;
+
   if (mLoadedAsData) {
     // Generate a document load event for the case when an XML document was loaded
     // as pure data without any presentation attached to it.
@@ -573,19 +677,20 @@ nsXMLDocument::GetInlineStyleSheet(nsIHTMLCSSStyleSheet** aResult)
 }
 
 // subclass hook for sheet ordering
-void nsXMLDocument::InternalAddStyleSheet(nsIStyleSheet* aSheet, PRUint32 aFlags)
+void
+nsXMLDocument::InternalAddStyleSheet(nsIStyleSheet* aSheet, PRUint32 aFlags)
 {
   // XXXbz this catalog stuff should be in the UA level in the cascade!
   if (aFlags & NS_STYLESHEET_FROM_CATALOG) {
     // always after other catalog sheets
-    mStyleSheets.InsertObjectAt(aSheet, mCountCatalogSheets);
-    ++mCountCatalogSheets;
+    mStyleSheets.InsertObjectAt(aSheet, mCatalogSheetCount);
+    ++mCatalogSheetCount;
   }
   else if (aSheet == mAttrStyleSheet) {  // always after catalog sheets
     NS_ASSERTION(mStyleSheets.Count() == 0 ||
                  mAttrStyleSheet != mStyleSheets[0],
                  "Adding attr sheet twice!");
-    mStyleSheets.InsertObjectAt(aSheet, mCountCatalogSheets);
+    mStyleSheets.InsertObjectAt(aSheet, mCatalogSheetCount);
   }
   else if (aSheet == mInlineStyleSheet) {  // always last
     NS_ASSERTION(mStyleSheets.Count() == 0 ||
@@ -614,13 +719,13 @@ nsXMLDocument::InternalInsertStyleSheetAt(nsIStyleSheet* aSheet, PRInt32 aIndex)
                           /* Don't count Attribute stylesheet */
                           - 1
                           /* Don't count catalog sheets */
-                          - mCountCatalogSheets
+                          - mCatalogSheetCount
                           /* No insertion allowed after StyleAttr stylesheet */
                           - (mInlineStyleSheet ? 1: 0)
                           ),
                "index out of bounds");
   // offset w.r.t. catalog style sheets and the attr style sheet
-  mStyleSheets.InsertObjectAt(aSheet, aIndex + mCountCatalogSheets + 1);
+  mStyleSheets.InsertObjectAt(aSheet, aIndex + mCatalogSheetCount + 1);
 }
 
 already_AddRefed<nsIStyleSheet>
@@ -629,7 +734,7 @@ nsXMLDocument::InternalGetStyleSheetAt(PRInt32 aIndex)
   PRInt32 count = InternalGetNumberOfStyleSheets();
 
   if (aIndex >= 0 && aIndex < count) {
-    nsIStyleSheet* sheet = mStyleSheets[aIndex + mCountCatalogSheets + 1];
+    nsIStyleSheet* sheet = mStyleSheets[aIndex + mCatalogSheetCount + 1];
     NS_ADDREF(sheet);
     return sheet;
   } else {
@@ -642,10 +747,21 @@ PRInt32
 nsXMLDocument::InternalGetNumberOfStyleSheets()
 {
   PRInt32 count = mStyleSheets.Count();
-  if (count != 0 && mInlineStyleSheet == mStyleSheets[count - 1])
+
+  if (count != 0 && mInlineStyleSheet == mStyleSheets[count - 1]) {
+    // subtract the inline style sheet
     --count;
-  count -= (mCountCatalogSheets + 1); // +1 for the attr sheet
-  NS_ASSERTION(count >= 0, "Why did we end up with a negative count?");
+  }
+
+  if (count != 0 && mAttrStyleSheet == mStyleSheets[mCatalogSheetCount]) {
+    // subtract the attr sheet
+    --count;
+  }
+
+  count -= mCatalogSheetCount;
+
+  NS_ASSERTION(count >= 0, "How did we get a negative count?");
+
   return count;
 }
 
@@ -657,7 +773,8 @@ nsXMLDocument::GetDoctype(nsIDOMDocumentType** aDocumentType)
 }
  
 NS_IMETHODIMP    
-nsXMLDocument::CreateCDATASection(const nsAString& aData, nsIDOMCDATASection** aReturn)
+nsXMLDocument::CreateCDATASection(const nsAString& aData,
+                                  nsIDOMCDATASection** aReturn)
 {
   NS_ENSURE_ARG_POINTER(aReturn);
   *aReturn = nsnull;
@@ -681,7 +798,8 @@ nsXMLDocument::CreateCDATASection(const nsAString& aData, nsIDOMCDATASection** a
 }
  
 NS_IMETHODIMP    
-nsXMLDocument::CreateEntityReference(const nsAString& aName, nsIDOMEntityReference** aReturn)
+nsXMLDocument::CreateEntityReference(const nsAString& aName,
+                                     nsIDOMEntityReference** aReturn)
 {
   NS_ENSURE_ARG_POINTER(aReturn);
 
@@ -858,11 +976,8 @@ MatchId(nsIContent *aContent, const nsAString& aName)
     nsCOMPtr<nsIXMLContent> xmlContent = do_QueryInterface(aContent);    
     nsCOMPtr<nsIAtom> IDValue;
     if (xmlContent && NS_SUCCEEDED(xmlContent->GetID(*getter_AddRefs(IDValue))) && IDValue) {
-      const PRUnichar* IDValStr = nsnull;
-      IDValue->GetUnicode(&IDValStr);       
-      if (aName.Equals(IDValStr)) {
+      if (IDValue->Equals(aName))
         return aContent;
-      }
     }
   }
   
@@ -1000,18 +1115,19 @@ nsXMLDocument::GetBaseTarget(nsAString &aBaseTarget)
 NS_IMETHODIMP
 nsXMLDocument::GetCSSLoader(nsICSSLoader*& aLoader)
 {
-  nsresult result = NS_OK;
-  if (! mCSSLoader) {
-    result = NS_NewCSSLoader(this, getter_AddRefs(mCSSLoader));
-    if (mCSSLoader) {
-      mCSSLoader->SetCaseSensitive(PR_TRUE);
-      // No quirks in XML
-      mCSSLoader->SetCompatibilityMode(eCompatibility_FullStandards);
-    }
+  if (!mCSSLoader) {
+    nsresult rv = NS_NewCSSLoader(this, getter_AddRefs(mCSSLoader));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    mCSSLoader->SetCaseSensitive(PR_TRUE);
+    // no quirks in XML
+    mCSSLoader->SetCompatibilityMode(eCompatibility_FullStandards);
   }
+
   aLoader = mCSSLoader;
   NS_IF_ADDREF(aLoader);
-  return result;
+
+  return NS_OK;
 }
 
 nsresult
