@@ -66,6 +66,7 @@
 #include "nsIFormControl.h"
 #include "nsIHTMLContent.h"
 #include "nsIHTMLContentContainer.h"
+#include "nsIHTMLStyleSheet.h"
 #include "nsINameSpace.h"
 #include "nsINameSpaceManager.h"
 #include "nsINodeInfo.h"
@@ -83,6 +84,7 @@
 #include "nsIXULDocument.h"
 #include "nsIXULPrototypeDocument.h"
 #include "nsIXULPrototypeCache.h"
+#include "nsIScriptSecurityManager.h"
 #include "nsLayoutCID.h"
 #include "nsNetUtil.h"
 #include "nsRDFCID.h"
@@ -104,6 +106,7 @@
 #include "nsIExpatSink.h"
 #include "nsUnicharUtils.h"
 #include "nsXULAtoms.h"
+#include "nsHTMLAtoms.h"
 
 static const char kNameSpaceSeparator = ':';
 static const char kNameSpaceDef[] = "xmlns";
@@ -144,8 +147,6 @@ protected:
     static nsrefcnt               gRefCnt;
     static nsIXULPrototypeCache*  gXULCache;
 
-    static nsIAtom* kScriptAtom;
-
     PRUnichar* mText;
     PRInt32 mTextLength;
     PRInt32 mTextSize;
@@ -247,13 +248,12 @@ protected:
     nsString               mPreferredStyle;
     nsCOMPtr<nsICSSLoader> mCSSLoader;            // [OWNER]
     nsCOMPtr<nsICSSParser> mCSSParser;            // [OWNER]
+    nsCOMPtr<nsIScriptSecurityManager> mSecMan;
 };
 
 nsrefcnt XULContentSinkImpl::gRefCnt;
 nsIXULPrototypeCache* XULContentSinkImpl::gXULCache;
 
-nsIAtom* XULContentSinkImpl::kScriptAtom;
-
 //----------------------------------------------------------------------
 
 XULContentSinkImpl::ContextStack::ContextStack()
@@ -338,8 +338,6 @@ XULContentSinkImpl::XULContentSinkImpl(nsresult& rv)
 {
 
     if (gRefCnt++ == 0) {
-        kScriptAtom = NS_NewAtom("script");
-
         rv = CallGetService(kXULPrototypeCacheCID, &gXULCache);
     }
 
@@ -374,9 +372,7 @@ XULContentSinkImpl::~XULContentSinkImpl()
                 nsAutoString prefix;
                 if (prefixAtom)
                     {
-                        const PRUnichar *unicodeString;
-                        prefixAtom->GetUnicode(&unicodeString);
-                        prefix = unicodeString;
+                        prefixAtom->ToString(prefix);
                     }
                 else
                     {
@@ -427,7 +423,6 @@ XULContentSinkImpl::~XULContentSinkImpl()
     PR_FREEIF(mText);
 
     if (--gRefCnt == 0) {
-        NS_IF_RELEASE(kScriptAtom);
         NS_IF_RELEASE(gXULCache);
     }
 }
@@ -493,14 +488,14 @@ XULContentSinkImpl::DidBuildModel(PRInt32 aQualityLevel)
 NS_IMETHODIMP 
 XULContentSinkImpl::WillInterrupt(void)
 {
-    // XXX Notify the webshell, if necessary
+    // XXX Notify the docshell, if necessary
     return NS_OK;
 }
 
 NS_IMETHODIMP 
 XULContentSinkImpl::WillResume(void)
 {
-    // XXX Notify the webshell, if necessary
+    // XXX Notify the docshell, if necessary
     return NS_OK;
 }
 
@@ -1274,8 +1269,8 @@ XULContentSinkImpl::OpenRoot(const PRUnichar** aAttributes,
 
     nsresult rv;
 
-    if (aNodeInfo->Equals(kScriptAtom, kNameSpaceID_XHTML) || 
-        aNodeInfo->Equals(kScriptAtom, kNameSpaceID_XUL)) {
+    if (aNodeInfo->Equals(nsHTMLAtoms::script, kNameSpaceID_XHTML) || 
+        aNodeInfo->Equals(nsHTMLAtoms::script, kNameSpaceID_XUL)) {
         PR_LOG(gLog, PR_LOG_ALWAYS,
                ("xul: script tag not allowed as root content element"));
 
@@ -1354,8 +1349,8 @@ XULContentSinkImpl::OpenTag(const PRUnichar** aAttributes,
 
     children->AppendElement(element);
 
-    if (aNodeInfo->Equals(kScriptAtom, kNameSpaceID_XHTML) || 
-        aNodeInfo->Equals(kScriptAtom, kNameSpaceID_XUL)) {
+    if (aNodeInfo->Equals(nsHTMLAtoms::script, kNameSpaceID_XHTML) || 
+        aNodeInfo->Equals(nsHTMLAtoms::script, kNameSpaceID_XUL)) {
         // Do scripty things now.  OpenScript will push the
         // nsPrototypeScriptElement onto the stack, so we're done after this.
         return OpenScript(aAttributes, aLineNumber);
@@ -1433,6 +1428,17 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
       if (! src.IsEmpty()) {
           // Use the SRC attribute value to load the URL
           rv = NS_NewURI(getter_AddRefs(script->mSrcURI), src, nsnull, mDocumentURL);
+
+          // Check if this document is allowed to load a script from this source
+          if (NS_SUCCEEDED(rv)) {
+              if (!mSecMan)
+                  mSecMan = do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
+              if (NS_SUCCEEDED(rv)) {
+                  rv = mSecMan->CheckLoadURI(mDocumentURL, script->mSrcURI,
+                                             nsIScriptSecurityManager::ALLOW_CHROME);
+              }
+          }
+
           if (NS_FAILED(rv)) {
               delete script;
               return rv;
@@ -1450,7 +1456,7 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
                   nsCOMPtr<nsIScriptContext> scriptContext;
                   globalObject->GetContext(getter_AddRefs(scriptContext));
                   if (scriptContext)
-                      script->DeserializeOutOfLineScript(nsnull, scriptContext);
+                      script->DeserializeOutOfLine(nsnull, scriptContext);
               }
           }
       }
