@@ -21,7 +21,7 @@
  *
  * Contributor(s):
  *   Chris Waterson <waterson@netscape.com>
- *   L. David Baron <dbaron@fas.harvard.edu>
+ *   L. David Baron <dbaron@dbaron.org>
  *   Ben Goodger <ben@netscape.com>
  *
  *
@@ -47,6 +47,7 @@
 */
 
 #include "nsCOMPtr.h"
+#include "nsAString.h"
 #include "nsIObjectInputStream.h"
 #include "nsIObjectOutputStream.h"
 #include "nsIPrincipal.h"
@@ -86,7 +87,8 @@ public:
     NS_IMETHOD SetContext(nsIScriptContext *aContext);
     NS_IMETHOD GetContext(nsIScriptContext **aContext);
     NS_IMETHOD SetNewDocument(nsIDOMDocument *aDocument,
-                              PRBool removeEventListeners);
+                              PRBool aRemoveEventListeners,
+                              PRBool aClearScope);
     NS_IMETHOD SetDocShell(nsIDocShell *aDocShell);
     NS_IMETHOD GetDocShell(nsIDocShell **aDocShell);
     NS_IMETHOD SetOpenerWindow(nsIDOMWindowInternal *aOpener);
@@ -149,6 +151,8 @@ public:
 
     NS_IMETHOD AwaitLoadDone(nsIXULDocument* aDocument, PRBool* aResult);
     NS_IMETHOD NotifyLoadDone();
+    
+    NS_IMETHOD GetNodeInfoManager(nsINodeInfoManager** aNodeInfoManager);
 
     // nsIScriptGlobalObjectOwner methods
     NS_DECL_NSISCRIPTGLOBALOBJECTOWNER
@@ -167,12 +171,15 @@ protected:
     PRPackedBool mLoaded;
     nsCOMPtr<nsICollection> mPrototypeWaiters;
 
+    nsCOMPtr<nsINodeInfoManager> mNodeInfoManager;
+
     nsXULPrototypeDocument();
     virtual ~nsXULPrototypeDocument();
     nsresult Init();
 
     friend NS_IMETHODIMP
-    NS_NewXULPrototypeDocument(nsISupports* aOuter, REFNSIID aIID, void** aResult);
+    NS_NewXULPrototypeDocument(nsISupports* aOuter, REFNSIID aIID,
+                               void** aResult);
 };
 
 
@@ -222,7 +229,6 @@ nsXULPrototypeDocument::nsXULPrototypeDocument()
       mGlobalObject(nsnull),
       mLoaded(PR_FALSE)
 {
-    NS_INIT_REFCNT();
 }
 
 
@@ -232,10 +238,16 @@ nsXULPrototypeDocument::Init()
     nsresult rv;
 
     rv = NS_NewISupportsArray(getter_AddRefs(mStyleSheetReferences));
-    if (NS_FAILED(rv)) return rv;
+    NS_ENSURE_SUCCESS(rv, rv);
 
     rv = NS_NewISupportsArray(getter_AddRefs(mOverlayReferences));
-    if (NS_FAILED(rv)) return rv;
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    mNodeInfoManager = do_CreateInstance(NS_NODEINFOMANAGER_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = mNodeInfoManager->Init(nsnull);
+    NS_ENSURE_SUCCESS(rv, rv);
 
     return NS_OK;
 }
@@ -327,8 +339,16 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
     if (! securityManager)
         return NS_ERROR_FAILURE;
 
-    rv |= securityManager->GetCodebasePrincipal(mURI, getter_AddRefs(mDocumentPrincipal));
     rv |= NS_ReadOptionalObject(aStream, PR_TRUE, getter_AddRefs(mDocumentPrincipal));
+    if (!mDocumentPrincipal) {
+        // XXX This should be handled by the security manager, see bug 160042
+        PRBool isChrome = PR_FALSE;
+        if (NS_SUCCEEDED(mURI->SchemeIs("chrome", &isChrome)) && isChrome)
+            rv |= securityManager->GetSystemPrincipal(getter_AddRefs(mDocumentPrincipal));
+        else
+            rv |= securityManager->GetCodebasePrincipal(mURI, getter_AddRefs(mDocumentPrincipal));
+    }
+    mNodeInfoManager->SetDocumentPrincipal(mDocumentPrincipal);
 
     // nsIScriptGlobalObject mGlobalObject
     mGlobalObject = new nsXULPDGlobalObject();
@@ -336,7 +356,7 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
         return NS_ERROR_OUT_OF_MEMORY;
     rv |= mGlobalObject->SetGlobalObjectOwner(this); // does not refcount
 
-    mRoot = new nsXULPrototypeElement(-1);
+    mRoot = new nsXULPrototypeElement();
     if (! mRoot)
        return NS_ERROR_OUT_OF_MEMORY;
 
@@ -345,13 +365,30 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
     NS_ASSERTION(scriptContext != nsnull,
                  "no prototype script context!");
 
+    // nsINodeInfo table
+    nsCOMPtr<nsISupportsArray> nodeInfos;
+    rv |= NS_NewISupportsArray(getter_AddRefs(nodeInfos));
+    NS_ENSURE_TRUE(nodeInfos, rv);
+
+    rv |= aStream->Read32(&referenceCount);
+    nsAutoString namespaceURI, qualifiedName;
+    for (i = 0; i < referenceCount; ++i) {
+        rv |= aStream->ReadString(namespaceURI);
+        rv |= aStream->ReadString(qualifiedName);
+
+        nsCOMPtr<nsINodeInfo> nodeInfo;
+        rv |= mNodeInfoManager->GetNodeInfo(qualifiedName, namespaceURI, *getter_AddRefs(nodeInfo));
+        rv |= nodeInfos->AppendElement(nodeInfo);
+    }
+
+    // Document contents
     PRUint32 type;
     rv |= aStream->Read32(&type);
 
     if ((nsXULPrototypeNode::Type)type != nsXULPrototypeNode::eType_Element)
         return NS_ERROR_FAILURE;
 
-    rv |= mRoot->Deserialize(aStream, scriptContext, mURI);
+    rv |= mRoot->Deserialize(aStream, scriptContext, mURI, nodeInfos);
     rv |= NotifyLoadDone();
 
     return rv;
@@ -395,6 +432,27 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
     // nsIPrincipal mDocumentPrincipal
     rv |= NS_WriteOptionalObject(aStream, mDocumentPrincipal, PR_TRUE);
     
+    // nsINodeInfo table
+    nsCOMPtr<nsISupportsArray> nodeInfos;
+    rv |= mNodeInfoManager->GetNodeInfoArray(getter_AddRefs(nodeInfos));
+    NS_ENSURE_SUCCESS(rv, rv);
+    PRUint32 nodeInfoCount;
+    nodeInfos->Count(&nodeInfoCount);
+
+    rv |= aStream->Write32(nodeInfoCount);
+    for (i = 0; i < nodeInfoCount; ++i) {
+        nsCOMPtr<nsINodeInfo> nodeInfo = do_QueryElementAt(nodeInfos, i);
+        NS_ENSURE_TRUE(nodeInfo, NS_ERROR_FAILURE);
+
+        nsAutoString namespaceURI;
+        rv |= nodeInfo->GetNamespaceURI(namespaceURI);
+        rv |= aStream->WriteWStringZ(namespaceURI.get());
+
+        nsAutoString qualifiedName;
+        rv |= nodeInfo->GetQualifiedName(qualifiedName);
+        rv |= aStream->WriteWStringZ(qualifiedName.get());
+    }
+
     // Now serialize the document contents
     nsCOMPtr<nsIScriptGlobalObject> globalObject;
     rv |= GetScriptGlobalObject(getter_AddRefs(globalObject));
@@ -403,7 +461,7 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
     rv |= globalObject->GetContext(getter_AddRefs(scriptContext));
     
     if (mRoot)
-        rv |= mRoot->Serialize(aStream, scriptContext);
+        rv |= mRoot->Serialize(aStream, scriptContext, nodeInfos);
  
     return rv;
 }
@@ -426,7 +484,17 @@ nsXULPrototypeDocument::GetURI(nsIURI** aResult)
 NS_IMETHODIMP
 nsXULPrototypeDocument::SetURI(nsIURI* aURI)
 {
-    mURI = dont_QueryInterface(aURI);
+    NS_ASSERTION(!mURI, "Can't change the uri of a xul prototype document");
+    if (mURI)
+        return NS_ERROR_ALREADY_INITIALIZED;
+
+    mURI = aURI;
+    if (!mDocumentPrincipal) {
+        // If the document doesn't have a principal yet we'll force the creation of one
+        // so that mNodeInfoManager properly gets one.
+        nsCOMPtr<nsIPrincipal> principal;
+        GetDocumentPrincipal(getter_AddRefs(principal));
+    }
     return NS_OK;
 }
 
@@ -511,6 +579,7 @@ nsXULPrototypeDocument::SetHeaderData(nsIAtom* aField, const nsAString& aData)
 NS_IMETHODIMP
 nsXULPrototypeDocument::GetDocumentPrincipal(nsIPrincipal** aResult)
 {
+    NS_PRECONDITION(mNodeInfoManager, "missing nodeInfoManager");
     if (!mDocumentPrincipal) {
         nsresult rv;
         nsCOMPtr<nsIScriptSecurityManager> securityManager = 
@@ -519,10 +588,17 @@ nsXULPrototypeDocument::GetDocumentPrincipal(nsIPrincipal** aResult)
         if (NS_FAILED(rv))
             return NS_ERROR_FAILURE;
 
-        rv = securityManager->GetCodebasePrincipal(mURI, getter_AddRefs(mDocumentPrincipal));
+        // XXX This should be handled by the security manager, see bug 160042
+        PRBool isChrome = PR_FALSE;
+        if (NS_SUCCEEDED(mURI->SchemeIs("chrome", &isChrome)) && isChrome)
+            rv = securityManager->GetSystemPrincipal(getter_AddRefs(mDocumentPrincipal));
+        else
+            rv = securityManager->GetCodebasePrincipal(mURI, getter_AddRefs(mDocumentPrincipal));
 
         if (NS_FAILED(rv))
             return NS_ERROR_FAILURE;
+
+        mNodeInfoManager->SetDocumentPrincipal(mDocumentPrincipal);
     }
 
     *aResult = mDocumentPrincipal;
@@ -534,10 +610,19 @@ nsXULPrototypeDocument::GetDocumentPrincipal(nsIPrincipal** aResult)
 NS_IMETHODIMP
 nsXULPrototypeDocument::SetDocumentPrincipal(nsIPrincipal* aPrincipal)
 {
+    NS_PRECONDITION(mNodeInfoManager, "missing nodeInfoManager");
     mDocumentPrincipal = aPrincipal;
+    mNodeInfoManager->SetDocumentPrincipal(aPrincipal);
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsXULPrototypeDocument::GetNodeInfoManager(nsINodeInfoManager** aNodeInfoManager)
+{
+    *aNodeInfoManager = mNodeInfoManager;
+    NS_IF_ADDREF(*aNodeInfoManager);
+    return NS_OK;
+}
 
 
 NS_IMETHODIMP
@@ -644,7 +729,6 @@ nsXULPDGlobalObject::nsXULPDGlobalObject()
     : mJSObject(nsnull),
       mGlobalObjectOwner(nsnull)
 {
-    NS_INIT_REFCNT();
 }
 
 
@@ -711,7 +795,8 @@ nsXULPDGlobalObject::GetContext(nsIScriptContext **aContext)
 
 NS_IMETHODIMP
 nsXULPDGlobalObject::SetNewDocument(nsIDOMDocument *aDocument,
-                                    PRBool removeEventListeners)
+                                    PRBool aRemoveEventListeners,
+                                    PRBool aClearScope)
 {
     NS_NOTREACHED("waaah!");
     return NS_ERROR_UNEXPECTED;
