@@ -50,7 +50,7 @@
 extern PRLogModuleInfo* gXULTemplateLog;
 #endif
 
-extern PRBool
+PRBool
 IsElementInBuilder(nsIContent *aContent, nsIXULTemplateBuilder *aBuilder);
 
 nsContentTestNode::nsContentTestNode(InnerNode* aParent,
@@ -184,14 +184,14 @@ nsContentTestNode::FilterInstantiations(InstantiationSet& aInstantiations, void*
                 if (tag != mTag) {
                     consistent = PR_FALSE;
 
-                    const PRUnichar *expected, *actual;
-                    mTag->GetUnicode(&expected);
-                    tag->GetUnicode(&actual);
+                    const char *expected, *actual;
+                    mTag->GetUTF8String(&expected);
+                    tag->GetUTF8String(&actual);
 
                     PR_LOG(gXULTemplateLog, PR_LOG_DEBUG,
                            ("    => tag mismatch; expected %s, actual %s",
-                            NS_LossyConvertUCS2toASCII(expected).get(),
-                            NS_LossyConvertUCS2toASCII(actual).get()));
+                            expected,
+                            actual));
                 }
             }
 
