@@ -1,27 +1,41 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 2000 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 2000
+ * the Initial Developer. All Rights Reserved.
  *
- * Original Author(s):
+ * Contributor(s):
  *   Chris Waterson <waterson@netscape.com>
  *
- * Contributor(s): 
- */
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 /*
 
@@ -29,14 +43,14 @@
 
   To Do.
 
-  - Constrain() & Propogate() still feel like they are poorly named.
+  - Constrain() & Propagate() still feel like they are poorly named.
   - As do Instantiation and InstantiationSet.
   - Make InstantiationSet share and do copy-on-write.
   - Make things iterative, instead of recursive.
 
  */
 
-
+#include "nscore.h"
 #include "nsCOMPtr.h"
 #include "nsCRT.h"
 #include "nsIComponentManager.h"
@@ -130,12 +144,9 @@ value_to_isupports(const nsIID& aIID, const Value& aValue)
     // Need to const_cast aValue because QI() & Release() are not const
     nsISupports* isupports = NS_STATIC_CAST(nsISupports*, NS_CONST_CAST(Value&, aValue));
     if (isupports) {
-        nsISupports* dummy;
-        rv = isupports->QueryInterface(aIID, (void**) &dummy);
-        if (NS_SUCCEEDED(rv)) {
-            NS_RELEASE(dummy);
-        }
-        else {
+        nsCOMPtr<nsISupports> dummy;
+        rv = isupports->QueryInterface(aIID, getter_AddRefs(dummy));
+        if (NS_FAILED(rv)) {
             NS_ERROR("value does not support expected interface");
         }
     }
@@ -318,7 +329,7 @@ Value::Hash() const
         break;
 
     case eISupports:
-        temp = PLHashNumber(mISupports) >> 2; // strip alignment bits
+        temp = PLHashNumber(NS_PTR_TO_INT32(mISupports)) >> 2; // strip alignment bits
         break;
 
     case eString:
@@ -359,6 +370,63 @@ Value::operator PRInt32() const
     return (mType == eInteger) ? mInteger : 0;
 }
 
+#ifdef DEBUG
+#include "nsIRDFResource.h"
+#include "nsIRDFLiteral.h"
+#include "nsString.h"
+#include "nsPrintfCString.h"
+
+void
+Value::ToCString(nsACString& aResult)
+{
+    switch (mType) {
+    case eUndefined:
+        aResult = "[(undefined)]";
+        break;
+
+    case eISupports:
+        do {
+            nsCOMPtr<nsIRDFResource> res = do_QueryInterface(mISupports);
+            if (res) {
+                aResult = "[nsIRDFResource ";
+                const char* s;
+                res->GetValueConst(&s);
+                aResult += s;
+                aResult += "]";
+                break;
+            }
+
+            nsCOMPtr<nsIRDFLiteral> lit = do_QueryInterface(mISupports);
+            if (lit) {
+                aResult = "[nsIRDFLiteral \"";
+                const PRUnichar* s;
+                lit->GetValueConst(&s);
+                aResult += NS_ConvertUCS2toUTF8(s);
+                aResult += "\"]";
+                break;
+            }
+
+            aResult = "[nsISupports ";
+            aResult += nsPrintfCString("%p", mISupports);
+            aResult += "]";
+        } while (0);
+        break;
+
+    case eString:
+        aResult = "[string \"";
+        aResult += NS_ConvertUCS2toUTF8(mString);
+        aResult += "\"]";
+        break;
+
+    case eInteger:
+        aResult = "[integer ";
+        aResult += nsPrintfCString("%d", mInteger);
+        aResult += "]";
+        break;
+    }
+}
+#endif
+
 
 //----------------------------------------------------------------------
 //
@@ -679,17 +747,17 @@ InstantiationSet::HasAssignmentFor(PRInt32 aVariable) const
 //
 
 nsresult
-RootNode::Propogate(const InstantiationSet& aInstantiations, void* aClosure)
+RootNode::Propagate(const InstantiationSet& aInstantiations, void* aClosure)
 {
     PR_LOG(gXULTemplateLog, PR_LOG_DEBUG,
-           ("RootNode[%p]: Propogate() begin", this));
+           ("RootNode[%p]: Propagate() begin", this));
 
     ReteNodeSet::Iterator last = mKids.Last();
     for (ReteNodeSet::Iterator kid = mKids.First(); kid != last; ++kid)
-        kid->Propogate(aInstantiations, aClosure);
+        kid->Propagate(aInstantiations, aClosure);
 
     PR_LOG(gXULTemplateLog, PR_LOG_DEBUG,
-           ("RootNode[%p]: Propogate() end", this));
+           ("RootNode[%p]: Propagate() end", this));
 
     return NS_OK;
 }
@@ -738,9 +806,9 @@ JoinNode::JoinNode(InnerNode* aLeftParent,
 }
 
 nsresult
-JoinNode::Propogate(const InstantiationSet& aInstantiations, void* aClosure)
+JoinNode::Propagate(const InstantiationSet& aInstantiations, void* aClosure)
 {
-    // the add will have been propogated down from one of the parent
+    // the add will have been propagated down from one of the parent
     // nodes: either the left or the right. Test the other node for
     // matches.
     nsresult rv;
@@ -777,13 +845,13 @@ JoinNode::Propogate(const InstantiationSet& aInstantiations, void* aClosure)
     }
 
     if (! instantiations.Empty()) {
-        // propogate consistency checking back up the tree
+        // propagate consistency checking back up the tree
         rv = test->Constrain(instantiations, aClosure);
         if (NS_FAILED(rv)) return rv;
 
         ReteNodeSet::Iterator last = mKids.Last();
         for (ReteNodeSet::Iterator kid = mKids.First(); kid != last; ++kid)
-            kid->Propogate(instantiations, aClosure);
+            kid->Propagate(instantiations, aClosure);
     }
 
     return NS_OK;
@@ -962,12 +1030,12 @@ TestNode::TestNode(InnerNode* aParent)
 
 
 nsresult
-TestNode::Propogate(const InstantiationSet& aInstantiations, void* aClosure)
+TestNode::Propagate(const InstantiationSet& aInstantiations, void* aClosure)
 {
     nsresult rv;
 
     PR_LOG(gXULTemplateLog, PR_LOG_DEBUG,
-           ("TestNode[%p]: Propogate() begin", this));
+           ("TestNode[%p]: Propagate() begin", this));
 
     InstantiationSet instantiations = aInstantiations;
     rv = FilterInstantiations(instantiations, aClosure);
@@ -977,14 +1045,14 @@ TestNode::Propogate(const InstantiationSet& aInstantiations, void* aClosure)
         ReteNodeSet::Iterator last = mKids.Last();
         for (ReteNodeSet::Iterator kid = mKids.First(); kid != last; ++kid) {
             PR_LOG(gXULTemplateLog, PR_LOG_DEBUG,
-                   ("TestNode[%p]: Propogate() passing to child %p", this, kid.operator->()));
+                   ("TestNode[%p]: Propagate() passing to child %p", this, kid.operator->()));
 
-            kid->Propogate(instantiations, aClosure);
+            kid->Propagate(instantiations, aClosure);
         }
     }
 
     PR_LOG(gXULTemplateLog, PR_LOG_DEBUG,
-           ("TestNode[%p]: Propogate() end", this));
+           ("TestNode[%p]: Propagate() end", this));
 
     return NS_OK;
 }
