@@ -47,6 +47,7 @@
 #include "nsITreeBoxObject.h"
 #include "nsITreeSelection.h"
 #include "nsITreeView.h"
+#include "nsTreeUtils.h"
 #include "nsIServiceManager.h"
 #include "nsReadableUtils.h"
 
@@ -152,13 +153,6 @@ protected:
     nsresult
     GetTemplateActionCellFor(PRInt32 aRow, const PRUnichar* aColID, nsIContent** aResult);
 
-    /**
-     * Parse a whitespace separated list of properties into an array
-     * of atoms.
-     */
-    nsresult
-    TokenizeProperties(const nsAString& aString, nsISupportsArray* aProprerties);
-
     /**
      * Return the resource corresponding to a row in the tree. The
      * result is *not* addref'd
@@ -331,8 +325,9 @@ nsXULTreeBuilder::Init()
     if (NS_FAILED(rv)) return rv;
 
     if (gRefCnt++ == 0) {
-        gRDFService->GetResource(RDF_NAMESPACE_URI "type", &kRDF_type);
-        gRDFService->GetResource(NC_NAMESPACE_URI "BookmarkSeparator", &kNC_BookmarkSeparator);
+        gRDFService->GetResource(NS_LITERAL_CSTRING(RDF_NAMESPACE_URI "type"), &kRDF_type);
+        gRDFService->GetResource(NS_LITERAL_CSTRING(NC_NAMESPACE_URI "BookmarkSeparator"),
+                                 &kNC_BookmarkSeparator);
     }
 
     // Try to acquire a collation object for sorting
@@ -443,6 +438,7 @@ nsXULTreeBuilder::Sort(nsIDOMElement* aElement)
             mBoxObject->Invalidate();
 
         header->SetAttr(kNameSpaceID_None, nsXULAtoms::sortDirection, dir, PR_TRUE);
+        header->SetAttr(kNameSpaceID_None, nsXULAtoms::sortActive, NS_LITERAL_STRING("true"), PR_TRUE);
 
         // Unset sort attribute(s) on the other columns
         nsCOMPtr<nsIContent> parentContent;
@@ -462,6 +458,8 @@ nsXULTreeBuilder::Sort(nsIDOMElement* aElement)
                         if (childTag == nsXULAtoms::treecol && childContent != header) {
                             childContent->UnsetAttr(kNameSpaceID_None,
                                                     nsXULAtoms::sortDirection, PR_TRUE);
+                            childContent->UnsetAttr(kNameSpaceID_None,
+                                                    nsXULAtoms::sortActive, PR_TRUE);
                         }
                     }
                 }
@@ -515,7 +513,7 @@ nsXULTreeBuilder::GetRowProperties(PRInt32 aIndex, nsISupportsArray* aProperties
             nsAutoString cooked;
             SubstituteText(*(mRows[aIndex]->mMatch), raw, cooked);
 
-            TokenizeProperties(cooked, aProperties);
+            nsTreeUtils::TokenizeProperties(cooked, aProperties);
         }
     }
 
@@ -539,7 +537,7 @@ nsXULTreeBuilder::GetCellProperties(PRInt32 aRow, const PRUnichar* aColID, nsISu
             nsAutoString cooked;
             SubstituteText(*(mRows[aRow]->mMatch), raw, cooked);
 
-            TokenizeProperties(cooked, aProperties);
+            nsTreeUtils::TokenizeProperties(cooked, aProperties);
         }
     }
 
@@ -584,7 +582,19 @@ nsXULTreeBuilder::IsContainerOpen(PRInt32 aIndex, PRBool* aResult)
     if (aIndex < 0 || aIndex >= mRows.Count())
         return NS_ERROR_INVALID_ARG;
 
-    return IsContainerOpen(GetResourceFor(aIndex), aResult);
+    nsTreeRows::iterator iter = mRows[aIndex];
+
+    if (iter->mContainerState == nsTreeRows::eContainerState_Unknown) {
+        PRBool isOpen;
+        IsContainerOpen(GetResourceFor(aIndex), &isOpen);
+
+        iter->mContainerState = isOpen
+            ? nsTreeRows::eContainerState_Open
+            : nsTreeRows::eContainerState_Closed;
+    }
+
+    *aResult = (iter->mContainerState == nsTreeRows::eContainerState_Open);
+    return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -598,16 +608,16 @@ nsXULTreeBuilder::IsContainerEmpty(PRInt32 aIndex, PRBool* aResult)
     NS_ASSERTION(iter->mContainerType == nsTreeRows::eContainerType_Container,
                  "asking for empty state on non-container");
 
-    if (iter->mContainerState == nsTreeRows::eContainerState_Unknown) {
+    if (iter->mContainerFill == nsTreeRows::eContainerFill_Unknown) {
         PRBool isEmpty;
         CheckContainer(GetResourceFor(aIndex), nsnull, &isEmpty);
 
-        iter->mContainerState = isEmpty
-            ? nsTreeRows::eContainerState_Empty
-            : nsTreeRows::eContainerState_Nonempty;
+        iter->mContainerFill = isEmpty
+            ? nsTreeRows::eContainerFill_Empty
+            : nsTreeRows::eContainerFill_Nonempty;
     }
 
-    *aResult = (iter->mContainerState == nsTreeRows::eContainerState_Empty);
+    *aResult = (iter->mContainerFill == nsTreeRows::eContainerFill_Empty);
     return NS_OK;
 }
 
@@ -855,25 +865,32 @@ nsXULTreeBuilder::ToggleOpenState(PRInt32 aIndex)
     }
     
     if (mPersistStateStore) {
+        PRBool isOpen;
+        IsContainerOpen(aIndex, &isOpen);
+
         nsIRDFResource* container = GetResourceFor(aIndex);
         if (! container)
             return NS_ERROR_FAILURE;
 
-        PRBool open;
-        IsContainerOpen(container, &open);
+        PRBool hasProperty;
+        IsContainerOpen(container, &hasProperty);
 
-        if (open) {
-            mPersistStateStore->Unassert(container,
-                                         nsXULContentUtils::NC_open,
-                                         nsXULContentUtils::true_);
+        if (isOpen) {
+            if (hasProperty) {
+                mPersistStateStore->Unassert(container,
+                                             nsXULContentUtils::NC_open,
+                                             nsXULContentUtils::true_);
+            }
 
             CloseContainer(aIndex, container);
         }
         else {
-            mPersistStateStore->Assert(VALUE_TO_IRDFRESOURCE(container),
-                                       nsXULContentUtils::NC_open,
-                                       nsXULContentUtils::true_,
-                                       PR_TRUE);
+            if (! hasProperty) {
+                mPersistStateStore->Assert(VALUE_TO_IRDFRESOURCE(container),
+                                           nsXULContentUtils::NC_open,
+                                           nsXULContentUtils::true_,
+                                           PR_TRUE);
+            }
 
             OpenContainer(aIndex, container);
         }
@@ -1161,13 +1178,7 @@ nsXULTreeBuilder::ReplaceMatch(nsIRDFResource* aMember,
             // Use the persist store to remember if the container
             // is open or closed.
             PRBool open = PR_FALSE;
-
-            if (mPersistStateStore)
-                mPersistStateStore->HasAssertion(container,
-                                                 nsXULContentUtils::NC_open,
-                                                 nsXULContentUtils::true_,
-                                                 PR_TRUE,
-                                                 &open);
+            IsContainerOpen(row, &open);
 
             // If it's open, make sure that we've got a subtree structure ready.
             if (open)
@@ -1177,9 +1188,9 @@ nsXULTreeBuilder::ReplaceMatch(nsIRDFResource* aMember,
             // container, so whether its open or closed, make sure
             // that we've got our tree row's state correct.
             if ((iter->mContainerType != nsTreeRows::eContainerType_Container) ||
-                (iter->mContainerState != nsTreeRows::eContainerState_Nonempty)) {
+                (iter->mContainerFill != nsTreeRows::eContainerFill_Nonempty)) {
                 iter->mContainerType  = nsTreeRows::eContainerType_Container;
-                iter->mContainerState = nsTreeRows::eContainerState_Nonempty;
+                iter->mContainerFill = nsTreeRows::eContainerFill_Nonempty;
                 mBoxObject->InvalidateRow(iter.GetRowIndex());
             }
         }
@@ -1494,48 +1505,6 @@ nsXULTreeBuilder::GetTemplateActionCellFor(PRInt32 aRow,
     return NS_OK;
 }
 
-nsresult
-nsXULTreeBuilder::TokenizeProperties(const nsAString& aString,
-                                         nsISupportsArray* aProperties)
-{
-    NS_PRECONDITION(aProperties != nsnull, "null ptr");
-    if (! aProperties)
-        return NS_ERROR_NULL_POINTER;
-
-    nsAString::const_iterator end;
-    aString.EndReading(end);
-
-    nsAString::const_iterator iter;
-    aString.BeginReading(iter);
-
-    do {
-        // Skip whitespace
-        while (iter != end && nsCRT::IsAsciiSpace(*iter))
-            ++iter;
-
-        // If only whitespace, we're done
-        if (iter == end)
-            break;
-
-        // Note the first non-whitespace character
-        nsAString::const_iterator first = iter;
-
-        // Advance to the next whitespace character
-        while (iter != end && ! nsCRT::IsAsciiSpace(*iter))
-            ++iter;
-
-        // XXX this would be nonsensical
-        NS_ASSERTION(iter != first, "eh? something's wrong here");
-        if (iter == first)
-            break;
-
-        nsCOMPtr<nsIAtom> atom = do_GetAtom(Substring(first, iter));
-        aProperties->AppendElement(atom);
-    } while (iter != end);
-
-    return NS_OK;
-}
-
 nsIRDFResource*
 nsXULTreeBuilder::GetResourceFor(PRInt32 aRow)
 {
@@ -1561,6 +1530,8 @@ nsXULTreeBuilder::OpenContainer(PRInt32 aIndex, nsIRDFResource* aContainer)
         nsTreeRows::iterator iter = mRows[aIndex];
         container = mRows.EnsureSubtreeFor(iter.GetParent(),
                                            iter.GetChildIndex());
+
+        iter->mContainerState = nsTreeRows::eContainerState_Open;
     }
     else
         container = mRows.GetRoot();
@@ -1767,6 +1738,8 @@ nsXULTreeBuilder::CloseContainer(PRInt32 aIndex, nsIRDFResource* aContainer)
         PRInt32 count = mRows.GetSubtreeSizeFor(iter);
         mRows.RemoveSubtreeFor(iter);
 
+        iter->mContainerState = nsTreeRows::eContainerState_Closed;
+
         if (mBoxObject) {
             mBoxObject->InvalidateRow(aIndex);
 
