@@ -40,6 +40,8 @@
 #include "nsLDAPMessage.h"
 #include "nsLDAPServer.h"
 #include "nsLDAPService.h"
+#include "nsLDAPBERValue.h"
+#include "ldappr.h"
 
 #ifdef MOZ_LDAP_XPCOM_EXPERIMENTAL
 #include "nsLDAPProtocolHandler.h"
@@ -54,15 +56,15 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(nsLDAPMessage);
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsLDAPServer);
 NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(nsLDAPURL, Init);
 NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(nsLDAPService, Init);
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsLDAPBERValue);
 
 #ifdef MOZ_LDAP_XPCOM_EXPERIMENTAL
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsLDAPProtocolHandler);
 #endif
 
-
-// a table of the CIDs implemented by this module (in this case, just one)
+// a table of the CIDs implemented by this module
 //
-static nsModuleComponentInfo components[] =
+static const nsModuleComponentInfo components[] =
 {
     { "LDAP Connection", NS_LDAPCONNECTION_CID,
           "@mozilla.org/network/ldap-connection;1", 
@@ -82,12 +84,53 @@ static nsModuleComponentInfo components[] =
           nsLDAPProtocolHandlerConstructor },   
 #endif
     { "LDAP URL", NS_LDAPURL_CID,
-          "@mozilla.org/network/ldap-url;1", nsLDAPURLConstructor }
+          "@mozilla.org/network/ldap-url;1", nsLDAPURLConstructor },
+    { "LDAP BER Value", NS_LDAPBERVALUE_CID,
+          "@mozilla.org/network/ldap-ber-value;1", nsLDAPBERValueConstructor }
 };
 
+PR_STATIC_CALLBACK(nsresult)
+nsLDAPInitialize(nsIModule *aSelf)
+{
+#ifdef PR_LOGGING
+    gLDAPLogModule = PR_NewLogModule("ldap");
+    if (!gLDAPLogModule) {
+        PR_fprintf(PR_STDERR, 
+                   "nsLDAP_Initialize(): PR_NewLogModule() failed\n");
+        return NS_ERROR_NOT_AVAILABLE;
+    }
+#endif
+
+    // use NSPR under the hood for all networking
+    //
+    int rv = prldap_install_routines( NULL, 1 /* shared */ );
+
+    if (rv != LDAP_SUCCESS) {
+        PR_LOG(gLDAPLogModule, PR_LOG_ERROR,
+               ("nsLDAPInitialize(): pr_ldap_install_routines() failed: %s\n",
+               ldap_err2string(rv)));
+        return NS_ERROR_FAILURE;
+    }
+
+    // Never block for more than 10000 milliseconds (ie 10 seconds) doing any 
+    // sort of I/O operation.
+    //
+    rv = prldap_set_session_option(0, 0, PRLDAP_OPT_IO_MAX_TIMEOUT, 
+                                   10000);
+    if (rv != LDAP_SUCCESS) {
+        PR_LOG(gLDAPLogModule, PR_LOG_ERROR,
+               ("nsLDAPInitialize(): error setting PRLDAP_OPT_IO_MAX_TIMEOUT:"
+                " %s\n", ldap_err2string(rv)));
+        return NS_ERROR_FAILURE;
+    }
+
+    return NS_OK;
+}
+
 // implement the NSGetModule() exported function
 //
-NS_IMPL_NSGETMODULE(nsLDAPProtocolModule, components);
+NS_IMPL_NSGETMODULE_WITH_CTOR(nsLDAPProtocolModule, components, 
+                              nsLDAPInitialize)
 
 #ifdef PR_LOGGING
 PRLogModuleInfo *gLDAPLogModule = 0;
