@@ -52,7 +52,6 @@ typedef unsigned long HMTX;
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIWebProgress.h"
 #include "nsIDocumentLoader.h"
-#include "nsIDocumentLoaderFactory.h"
 #include "nsIContentViewer.h"
 #include "nsIDocumentViewer.h"
 #include "nsIMarkupDocumentViewer.h"
@@ -114,11 +113,11 @@ typedef unsigned long HMTX;
 #include "nsIController.h"
 #include "nsIFocusController.h"
 #include "nsGUIEvent.h"
-#include "nsIFileStream.h"
 #include "nsISHistoryInternal.h"
 
 #include "nsIHttpChannel.h"
 #include "nsIUploadChannel.h"
+#include "nsISeekableStream.h"
 
 #include "nsILocaleService.h"
 #include "nsIStringBundle.h"
@@ -129,6 +128,11 @@ typedef unsigned long HMTX;
 #include "nsIDocument.h"
 #include "nsITextToSubURI.h"
 
+#ifdef MOZ_THUNDERBIRD
+#include "nsIExternalProtocolService.h"
+#include "nsCExternalHandlerService.h"
+#endif
+
 #ifdef NS_DEBUG
 /**
  * Note: the log module is created during initialization which
@@ -253,7 +257,10 @@ nsWebShell::EnsureCommandHandler()
     if (!commandUpdater) return NS_ERROR_FAILURE;
     
     nsCOMPtr<nsIDOMWindow> domWindow = do_GetInterface(NS_STATIC_CAST(nsIInterfaceRequestor *, this));
-    nsresult rv = commandUpdater->Init(domWindow);
+#ifdef DEBUG
+    nsresult rv =
+#endif
+    commandUpdater->Init(domWindow);
     NS_ASSERTION(NS_SUCCEEDED(rv), "Initting command manager failed");
   }
   
@@ -574,6 +581,48 @@ nsWebShell::OnLinkClickSync(nsIContent *aContent,
                             nsIDocShell** aDocShell,
                             nsIRequest** aRequest)
 {
+#ifdef MOZ_THUNDERBIRD
+  // XXX ugly thunderbird hack to force all url clicks to go to the system default app
+  // I promise this will be removed once we figure out a better way. 
+  nsCAutoString scheme;
+  aURI->GetScheme(scheme);
+                                          
+  static const char kMailToURI[] = "mailto";                                    
+  static const char kNewsURI[] = "news";                                        
+  static const char kSnewsURI[] = "snews";                                      
+  static const char kNntpURI[] = "nntp";                                        
+  static const char kImapURI[] = "imap"; 
+  if (scheme.EqualsIgnoreCase(kMailToURI)) 
+  {
+    // the scheme is mailto, we can handle it
+  } 
+  else if (scheme.EqualsIgnoreCase(kNewsURI)) 
+  {
+    // the scheme is news, we can handle it
+  } 
+  else if (scheme.EqualsIgnoreCase(kSnewsURI)) 
+  {
+     // the scheme is snews, we can handle it
+  } 
+  else if (scheme.EqualsIgnoreCase(kNntpURI)) 
+  {
+     // the scheme is nntp, we can handle it 
+  } else if (scheme.EqualsIgnoreCase(kImapURI)) 
+  {
+      // the scheme is imap, we can handle it
+  }
+  else 
+  {
+      // we don't handle this type, the the registered handler take it 
+      nsresult rv = NS_OK;
+      nsCOMPtr<nsIExternalProtocolService> extProtService = do_GetService(NS_EXTERNALPROTOCOLSERVICE_CONTRACTID, &rv);
+      NS_ENSURE_SUCCESS(rv,rv);
+      rv = extProtService->LoadUrl(aURI);
+      NS_ENSURE_SUCCESS(rv,rv);
+      return rv;                                                                  
+  }                                                                               
+#endif
+
   nsCOMPtr<nsIDOMNode> node(do_QueryInterface(aContent));
   NS_ENSURE_TRUE(node, NS_ERROR_UNEXPECTED);
 
@@ -624,11 +673,8 @@ nsWebShell::OnLinkClickSync(nsIContent *aContent,
   nsCOMPtr<nsIURI> referer;
   refererDoc->GetDocumentURL(getter_AddRefs(referer));
 
-  if (!referer) {
-    // No referer, no loading links!
-
-    return NS_OK;
-  }
+  // referer could be null here in some odd cases, but that's ok,
+  // we'll just load the link w/o sending a referer in those cases.
 
   nsAutoString target(aTargetSpec);
 
@@ -1282,14 +1328,19 @@ NS_IMETHODIMP nsWebShell::Create()
   WEB_TRACE(WEB_TRACE_CALLS,
             ("nsWebShell::Init: this=%p", this));
 
-  // HACK....force the uri loader to give us a load cookie for this webshell...then get it's
+  // HACK....force the uri loader to give us a load cookie for this webshell...then get its
   // doc loader and store it...as more of the docshell lands, we'll be able to get rid
   // of this hack...
-  nsCOMPtr<nsIURILoader> uriLoader = do_GetService(NS_URI_LOADER_CONTRACTID);
-  uriLoader->GetDocumentLoaderForContext(NS_STATIC_CAST(nsIWebShell*, this),
-                                         getter_AddRefs(mDocLoader));
+  nsresult rv;
+  nsCOMPtr<nsIURILoader> uriLoader = do_GetService(NS_URI_LOADER_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIContentViewerContainer> shellAsContainer = do_QueryInterface(NS_STATIC_CAST(nsIWebShell*, this));
+  rv = uriLoader->GetDocumentLoaderForContext(NS_STATIC_CAST(nsIWebShell*, this),
+                                              getter_AddRefs(mDocLoader));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIContentViewerContainer> shellAsContainer;
+  CallQueryInterface(this, NS_STATIC_CAST(nsIContentViewerContainer**, getter_AddRefs(shellAsContainer)));
   // Set the webshell as the default IContentViewerContainer for the loader...
   mDocLoader->SetContainer(shellAsContainer);
 
