@@ -84,7 +84,7 @@ enum nsDOMClassInfoID {
   // HTML classes
   eDOMClassInfo_HTMLDocument_id,
   eDOMClassInfo_HTMLCollection_id,
-  eDOMClassInfo_HTMLOptionCollection_id,
+  eDOMClassInfo_HTMLOptionsCollection_id,
   eDOMClassInfo_HTMLFormControlCollection_id,
   eDOMClassInfo_HTMLGenericCollection_id,
 
@@ -266,6 +266,9 @@ enum nsDOMClassInfoID {
   
   eDOMClassInfo_ImageDocument_id,
 
+  // XPointer evaluation result
+  eDOMClassInfo_XPointerResult_id,
+
   // This one better be the last one in this list
   eDOMClassInfoIDCount
 };
