@@ -63,6 +63,7 @@ struct nsDOMClassInfoData
     nsDOMClassInfoConstructorFnc mConstructorFptr;
     nsDOMClassInfoExternalConstructorFnc mExternalConstructorFptr;
   } u;
+
   nsIClassInfo *mCachedClassInfo; // low bit is set to 1 if external,
                                   // so be sure to mask if necessary!
   const nsIID *mProtoChainInterface;
@@ -163,7 +164,8 @@ protected:
             id == sScrollX_id      ||
             id == sScrollY_id      ||
             id == sLength_id       ||
-            id == sFrames_id);
+            id == sFrames_id       ||
+            id == sSelf_id);
   }
 
   static inline PRBool IsWritableReplaceable(jsval id)
@@ -250,6 +252,7 @@ protected:
   static jsval sDocument_id;
   static jsval sWindow_id;
   static jsval sFrames_id;
+  static jsval sSelf_id;
 
   static const JSClass *sObjectClass;
 
@@ -806,7 +809,8 @@ public:
 class nsHTMLPluginObjElementSH : public nsHTMLAppletElementSH
 {
 protected:
-  nsHTMLPluginObjElementSH(nsDOMClassInfoData* aData) : nsHTMLAppletElementSH(aData)
+  nsHTMLPluginObjElementSH(nsDOMClassInfoData* aData)
+    : nsHTMLAppletElementSH(aData)
   {
   }
 
@@ -831,16 +835,17 @@ public:
 };
 
 
-// HTMLOptionCollection helper
+// HTMLOptionsCollection helper
 
-class nsHTMLOptionCollectionSH : public nsHTMLCollectionSH
+class nsHTMLOptionsCollectionSH : public nsHTMLCollectionSH
 {
 protected:
-  nsHTMLOptionCollectionSH(nsDOMClassInfoData* aData) : nsHTMLCollectionSH(aData)
+  nsHTMLOptionsCollectionSH(nsDOMClassInfoData* aData)
+    : nsHTMLCollectionSH(aData)
   {
   }
 
-  virtual ~nsHTMLOptionCollectionSH()
+  virtual ~nsHTMLOptionsCollectionSH()
   {
   }
 
@@ -850,7 +855,7 @@ public:
 
   static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsHTMLOptionCollectionSH(aData);
+    return new nsHTMLOptionsCollectionSH(aData);
   }
 };
 
