@@ -84,8 +84,10 @@
 #include "prlog.h"
 #include "prthread.h"
 
+#ifdef OJI
 #include "nsIJVMManager.h"
 #include "nsILiveConnectManager.h"
+#endif
 
 const size_t gStackSize = 8192;
 
@@ -118,7 +120,6 @@ static PRBool sDidShutdown = PR_FALSE;
 
 static PRInt32 sContextCount = 0;
 
-static nsIScriptSecurityManager *sSecurityManager = nsnull;
 
 void JS_DLL_CALLBACK
 NS_ScriptErrorReporter(JSContext *cx,
@@ -435,6 +436,8 @@ nsJSContext::nsJSContext(JSRuntime *aRuntime) : mGCOnDestruction(PR_TRUE)
 
 nsJSContext::~nsJSContext()
 {
+  mSecurityManager = nsnull; // Force release
+
   // Cope with JS_NewContext failure in ctor (XXXbe move NewContext to Init?)
   if (!mContext)
     return;
@@ -466,12 +469,9 @@ nsJSContext::~nsJSContext()
 
   if (!sContextCount && sDidShutdown) {
     // The last context is being deleted, and we're already in the
-    // process of shutting down, release the JS runtime service, and
-    // the security manager.
+    // process of shutting down, release the JS runtime service.
 
     NS_IF_RELEASE(sRuntimeService);
-
-    NS_IF_RELEASE(sSecurityManager);
   }
 }
 
@@ -501,7 +501,10 @@ nsJSContext::EvaluateStringWithValue(const nsAString& aScript,
   // Beware that the result is not rooted! Be very careful not to run
   // the GC before rooting the result somehow!
   if (!mScriptsEnabled) {
-    *aIsUndefined = PR_TRUE;
+    if (aIsUndefined) {
+      *aIsUndefined = PR_TRUE;
+    }
+
     return NS_OK;
   }
 
@@ -592,11 +595,16 @@ nsJSContext::EvaluateStringWithValue(const nsAString& aScript,
 
   // If all went well, convert val to a string (XXXbe unless undefined?).
   if (ok) {
-    if (aIsUndefined) *aIsUndefined = JSVAL_IS_VOID(val);
+    if (aIsUndefined) {
+      *aIsUndefined = JSVAL_IS_VOID(val);
+    }
+
     *NS_STATIC_CAST(jsval*, aRetValue) = val;
   }
   else {
-    if (aIsUndefined) *aIsUndefined = PR_TRUE;
+    if (aIsUndefined) {
+      *aIsUndefined = PR_TRUE;
+    }
   }
 
   // Pop here, after JS_ValueToString and any other possible evaluation.
@@ -710,7 +718,10 @@ nsJSContext::EvaluateString(const nsAString& aScript,
 
   // If all went well, convert val to a string (XXXbe unless undefined?).
   if (ok) {
-    if (aIsUndefined) *aIsUndefined = JSVAL_IS_VOID(val);
+    if (aIsUndefined) {
+      *aIsUndefined = JSVAL_IS_VOID(val);
+    }
+
     JSString* jsstring = ::JS_ValueToString(mContext, val);
     if (jsstring) {
       aRetValue.Assign(NS_REINTERPRET_CAST(const PRUnichar*,
@@ -721,7 +732,10 @@ nsJSContext::EvaluateString(const nsAString& aScript,
     }
   }
   else {
-    if (aIsUndefined) *aIsUndefined = PR_TRUE;
+    if (aIsUndefined) {
+      *aIsUndefined = PR_TRUE;
+    }
+
     aRetValue.Truncate();
   }
 
@@ -812,10 +826,14 @@ nsJSContext::ExecuteScript(void* aScriptObject,
                            PRBool* aIsUndefined)
 {
   if (!mScriptsEnabled) {
-    if (aIsUndefined)
+    if (aIsUndefined) {
       *aIsUndefined = PR_TRUE;
-    if (aRetValue)
+    }
+
+    if (aRetValue) {
       aRetValue->Truncate();
+    }
+
     return NS_OK;
   }
 
@@ -849,8 +867,10 @@ nsJSContext::ExecuteScript(void* aScriptObject,
 
   if (ok) {
     // If all went well, convert val to a string (XXXbe unless undefined?).
-    if (aIsUndefined)
+    if (aIsUndefined) {
       *aIsUndefined = JSVAL_IS_VOID(val);
+    }
+
     if (aRetValue) {
       JSString* jsstring = ::JS_ValueToString(mContext, val);
       if (jsstring) {
@@ -862,10 +882,13 @@ nsJSContext::ExecuteScript(void* aScriptObject,
       }
     }
   } else {
-    if (aIsUndefined)
+    if (aIsUndefined) {
       *aIsUndefined = PR_TRUE;
-    if (aRetValue)
+    }
+
+    if (aRetValue) {
       aRetValue->Truncate();
+    }
   }
 
   ScriptEvaluated(PR_TRUE);
@@ -885,14 +908,14 @@ AtomToEventHandlerName(nsIAtom *aName, char *charName, PRUint32 charNameSize)
   // optimized to avoid ns*Str*.h explicit/implicit copying and malloc'ing
   // even nsCAutoString may call an Append that copy-constructs an nsStr from
   // a const PRUnichar*
-  const PRUnichar *name;
-  aName->GetUnicode(&name);
+  const char *name;
+  aName->GetUTF8String(&name);
   char c;
   PRUint32 i = 0;
 
   do {
     NS_ASSERTION(name[i] < 128, "non-ASCII event handler name");
-    c = char(name[i]);
+    c = name[i];
 
     // The HTML content sink must have folded to lowercase already.
     NS_ASSERTION(c == '\0' || isalpha(c), "non-alphabetic event handler name");
@@ -907,35 +930,25 @@ nsJSContext::CompileEventHandler(void *aTarget, nsIAtom *aName,
                                  const nsAString& aBody,
                                  PRBool aShared, void** aHandler)
 {
-  if (!sSecurityManager) {
-    NS_ERROR("Huh, we need a script security manager to compile "
-             "an event handler!");
-
-    return NS_ERROR_UNEXPECTED;
-  }
-
-  JSObject *target = (JSObject*)aTarget;
-
   JSPrincipals *jsprin = nsnull;
 
-  if (target) {
-    // Get the principal of the event target (the object principal),
-    // don't get the principal of the global object in this context
-    // since that opens up security exploits with delayed event
-    // handler compilation on stale DOM objects (objects that live in
-    // a document that has already been unloaded).
-    nsCOMPtr<nsIPrincipal> prin;
-    nsresult rv = sSecurityManager->GetObjectPrincipal(mContext, target,
-                                                       getter_AddRefs(prin));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    prin->GetJSPrincipals(&jsprin);
-    NS_ENSURE_TRUE(jsprin, NS_ERROR_NOT_AVAILABLE);
+  nsCOMPtr<nsIScriptGlobalObject> global;
+  GetGlobalObject(getter_AddRefs(global));
+  if (global) {
+    // XXXbe why the two-step QI? speed up via a new GetGlobalObjectData func?
+    nsCOMPtr<nsIScriptObjectPrincipal> globalData = do_QueryInterface(global);
+    if (globalData) {
+      nsCOMPtr<nsIPrincipal> prin;
+      if (NS_FAILED(globalData->GetPrincipal(getter_AddRefs(prin))))
+        return NS_ERROR_FAILURE;
+      prin->GetJSPrincipals(&jsprin);
+    }
   }
 
   char charName[64];
   AtomToEventHandlerName(aName, charName, sizeof charName);
 
+  JSObject *target = (JSObject*)aTarget;
   JSFunction* fun =
       ::JS_CompileUCFunctionForPrincipals(mContext, target, jsprin,
                                           charName, 1, gEventArgv,
@@ -944,12 +957,10 @@ nsJSContext::CompileEventHandler(void *aTarget, nsIAtom *aName,
                                           //XXXbe filename, lineno:
                                           nsnull, 0);
 
-  if (jsprin) {
+  if (jsprin)
     JSPRINCIPALS_DROP(mContext, jsprin);
-  }
-  if (!fun) {
+  if (!fun)
     return NS_ERROR_FAILURE;
-  }
 
   JSObject *handler = ::JS_GetFunctionObject(fun);
   if (aHandler)
@@ -1241,6 +1252,7 @@ nsJSContext::InitializeLiveConnectClasses()
 {
   nsresult rv = NS_OK;
 
+#ifdef OJI
   nsCOMPtr<nsIJVMManager> jvmManager =
     do_GetService(nsIJVMManager::GetCID(), &rv);
 
@@ -1258,6 +1270,7 @@ nsJSContext::InitializeLiveConnectClasses()
       }
     }
   }
+#endif /* OJI */
 
   // return all is well until things are stable.
   return NS_OK;
@@ -1509,12 +1522,14 @@ nsJSContext::ScriptEvaluated(PRBool aTerminated)
 NS_IMETHODIMP
 nsJSContext::GetSecurityManager(nsIScriptSecurityManager **aInstancePtr)
 {
-  *aInstancePtr = sSecurityManager;
+  if (!mSecurityManager) {
+    nsresult rv = NS_OK;
 
-  if (!sSecurityManager) {
-    return NS_ERROR_NOT_AVAILABLE;
+    mSecurityManager = do_GetService(kScriptSecurityManagerContractID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
   }
 
+  *aInstancePtr = mSecurityManager;
   NS_ADDREF(*aInstancePtr);
 
   return NS_OK;
@@ -1702,6 +1717,7 @@ nsresult nsJSEnvironment::Init()
     NS_WARNING("Failed to get XPConnect service!");
   }
 
+#ifdef OJI
   // Initialize LiveConnect.  XXXbe use contractid rather than GetCID
   // NOTE: LiveConnect is optional so initialisation will still succeed
   //       even if the service is not present.
@@ -1714,10 +1730,7 @@ nsresult nsJSEnvironment::Init()
     PRBool started = PR_FALSE;
     rv = manager->StartupLiveConnect(sRuntime, started);
   }
-
-  rv = nsServiceManager::GetService(kScriptSecurityManagerContractID,
-                                    NS_GET_IID(nsIScriptSecurityManager),
-                                    (nsISupports**)&sSecurityManager);
+#endif /* OJI */
 
   isInitialized = NS_SUCCEEDED(rv);
 
@@ -1741,10 +1754,9 @@ void nsJSEnvironment::ShutDown()
 
   if (!sContextCount) {
     // We're being shutdown, and there are no more contexts
-    // alive, release the JS runtime service and the security manager.
+    // alive, release the JS runtime service.
 
     NS_IF_RELEASE(sRuntimeService);
-    NS_IF_RELEASE(sSecurityManager);
   }
 
   sDidShutdown = PR_TRUE;
