@@ -43,11 +43,12 @@
 #include "nsIDOMCSSStyleDeclaration.h"
 #include "nsIDOMElementCSSInlineStyle.h"
 #include "nsIContent.h"
-#include "nsIEditProperty.h"
+#include "nsEditProperty.h"
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
 #include "nsCRT.h"
 #include "nsIAtom.h"
+#include "nsIHTMLObjectResizer.h"
 
 #define kNullCh (PRUnichar('\0'))
 
@@ -364,4 +365,3 @@ ChangeCSSInlineStyleTxn::AddValueToMultivalueProperty(nsAString & aValues, const
   }
   return NS_OK;
 }
-
