@@ -211,10 +211,6 @@ class nsDOMIterator
     nsresult Init(nsIDOMRange* aRange);
     nsresult Init(nsIDOMNode* aNode);
     void ForEach(nsDomIterFunctor& functor) const;
-    // XXX probably want to remove MakeList in favor of AppendList at
-    // some point.
-    nsresult MakeList(nsBoolDomIterFunctor& functor,
-                      nsCOMArray<nsIDOMNode>& outArrayOfNodes) const;
     nsresult AppendList(nsBoolDomIterFunctor& functor,
                         nsCOMArray<nsIDOMNode>& arrayOfNodes) const;
   protected:
@@ -270,4 +266,24 @@ class nsEditorUtils
     static PRBool IsLeafNode(nsIDOMNode *aNode);
 };
 
+
+class nsIDragSession;
+class nsITransferable;
+class nsIDOMEvent;
+class nsISimpleEnumerator;
+
+class nsEditorHookUtils
+{
+  public:
+    static PRBool   DoAllowDragHook(nsIDOMDocument *aDoc, nsIDOMEvent *aEvent);
+    static PRBool   DoDragHook(nsIDOMDocument *aDoc, nsITransferable *aTrans);
+    static PRBool   DoAllowDropHook(nsIDOMDocument *aDoc, nsIDOMEvent *aEvent,
+                                    nsIDragSession *aSession);
+    static PRBool   DoInsertionHook(nsIDOMDocument *aDoc, nsIDOMEvent *aEvent,
+                                    nsITransferable *aTrans);
+  private:
+    static nsresult GetHookEnumeratorFromDocument(nsIDOMDocument *aDoc,
+                                                  nsISimpleEnumerator **aEnumerator);
+};
+
 #endif // nsEditorUtils_h__
