@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Daniel Glazman <glazman@netscape.com>
+ *   Kathleen Brade <brade@netscape.com>
  *
  *
  * Alternatively, the contents of this file may be used under the terms of
@@ -48,6 +49,7 @@
 #include "nsITableEditor.h"
 #include "nsIEditorMailSupport.h"
 #include "nsIEditorStyleSheets.h"
+#include "nsIDocumentObserver.h"
 
 #include "nsEditor.h"
 #include "nsIDOMElement.h"
@@ -59,12 +61,16 @@
 
 #include "nsEditRules.h"
 
-#include "nsIEditProperty.h"
+#include "nsEditProperty.h"
 #include "nsHTMLCSSUtils.h"
 #include "nsIParserService.h"
 
 #include "nsVoidArray.h"
 
+#include "nsHTMLObjectResizer.h"
+
+#include "nsPoint.h"
+
 class nsIDOMKeyEvent;
 class nsITransferable;
 class nsIDOMEventReceiver;
@@ -72,6 +78,7 @@ class nsIDOMNSRange;
 class nsIDocumentEncoder;
 class nsIClipboard;
 class TypeInState;
+class nsIContentFilter;
 
 /**
  * The HTML editor implementation.<br>
@@ -79,6 +86,7 @@ class TypeInState;
  */
 class nsHTMLEditor : public nsPlaintextEditor,
                      public nsIHTMLEditor,
+                     public nsIHTMLObjectResizer,
                      public nsITableEditor,
                      public nsIEditorStyleSheets,
                      public nsICSSLoaderObserver
@@ -102,9 +110,17 @@ public:
     kOpSetTextProperty     = 3010,
     kOpRemoveTextProperty  = 3011,
     kOpHTMLPaste           = 3012,
-    kOpLoadHTML            = 3013
+    kOpLoadHTML            = 3013,
+    kOpResetTextProperties = 3014
   };
 
+  enum ResizingRequestID
+  {
+    kX      = 0,
+    kY      = 1,
+    kWidth  = 2,
+    kHeight = 3
+  };
 
   // see nsIHTMLEditor for documentation
 
@@ -123,6 +139,9 @@ public:
   NS_IMETHOD GetIsDocumentEditable(PRBool *aIsDocumentEditable);
   NS_IMETHODIMP BeginningOfDocument();
 
+  /* ------------ nsIHTMLObjectResizer methods -------------- */
+  NS_DECL_NSIHTMLOBJECTRESIZER
+
   /* ------------ nsIHTMLEditor methods -------------- */
   NS_IMETHOD UpdateBaseURL();
 
@@ -132,6 +151,16 @@ public:
   NS_IMETHOD ParseStyleAttrIntoCSSRule(const nsAString& aString,
                                        nsIDOMCSSStyleRule **_retval); 
 
+  NS_IMETHOD AddDefaultProperty(nsIAtom *aProperty, 
+                                const nsAString & aAttribute, 
+                                const nsAString & aValue);
+
+  NS_IMETHOD RemoveDefaultProperty(nsIAtom *aProperty, 
+                                   const nsAString & aAttribute, 
+                                   const nsAString & aValue);
+
+  NS_IMETHOD RemoveAllDefaultProperties();
+
   NS_IMETHOD SetCSSInlineProperty(nsIAtom *aProperty, 
                              const nsAString & aAttribute, 
                              const nsAString & aValue);
@@ -160,8 +189,14 @@ public:
 
   NS_IMETHOD PasteNoFormatting(PRInt32 aSelectionType);
   NS_IMETHOD InsertHTML(const nsAString &aInputString);
-  NS_IMETHOD InsertHTMLWithCharset(const nsAString& aInputString,
-                                   const nsAString& aCharset);
+  NS_IMETHOD InsertHTMLWithCharsetAndContext(const nsAString & aInputString,
+                                             const nsAString & aCharset,
+                                             const nsAString & aContextStr,
+                                             const nsAString & aInfoStr,
+                                             const nsAString & aFlavor,
+                                             nsIDOMNode *aDestinationNode,
+                                             PRInt32 aDestinationOffset,
+                                             PRBool aDeleteSelection);
 
   NS_IMETHOD LoadHTML(const nsAString &aInputString);
   NS_IMETHOD LoadHTMLWithCharset(const nsAString& aInputString,
@@ -206,6 +241,8 @@ public:
 
   NS_IMETHOD SetIsCSSEnabled(PRBool aIsCSSPrefChecked);
   NS_IMETHOD GetIsCSSEnabled(PRBool *aIsCSSEnabled);
+  NS_IMETHOD AddInsertionListener(nsIContentFilter *aFilter);
+  NS_IMETHOD RemoveInsertionListener(nsIContentFilter *aFilter);
 
   /* ------------ nsIEditorIMESupport overrides -------------- */
   
@@ -362,6 +399,8 @@ public:
 
   /* ------------ Overrides of nsEditor interface methods -------------- */
 
+  nsresult EndUpdateViewBatch();
+
   /** prepare the editor for use */
   NS_IMETHOD Init(nsIDOMDocument *aDoc, nsIPresShell *aPresShell,  nsIContent *aRoot, nsISelectionController *aSelCon, PRUint32 aFlags);
   
@@ -618,7 +657,7 @@ protected:
                                           const nsAString   *aValue,
                                           PRBool            &aIsSet,
                                           nsIDOMNode       **aStyleNode,
-                                          nsAString *outValue = nsnull) const;
+                                          nsAString *outValue = nsnull);
 
   void ResetTextSelectionForRange(nsIDOMNode *aParent,
                                   PRInt32     aStartOffset,
@@ -644,32 +683,44 @@ protected:
   // factored methods for handling insertion of data from transferables (drag&drop or clipboard)
   NS_IMETHOD PrepareTransferable(nsITransferable **transferable);
   NS_IMETHOD PrepareHTMLTransferable(nsITransferable **transferable, PRBool havePrivFlavor);
+  nsresult   PutDragDataInTransferable(nsITransferable **aTransferable);
   NS_IMETHOD InsertFromTransferable(nsITransferable *transferable, 
                                     const nsAString & aContextStr,
-                                    const nsAString & aInfoStr);
+                                    const nsAString & aInfoStr,
+                                    nsIDOMNode *aDestinationNode,
+                                    PRInt32 aDestinationOffset,
+                                    PRBool aDoDeleteSelection);
   PRBool HavePrivateHTMLFlavor( nsIClipboard *clipboard );
   nsresult   ParseCFHTML(nsCString & aCfhtml, PRUnichar **aStuffToPaste, PRUnichar **aCfcontext);
-  nsresult   InsertHTMLWithContext(const nsAString & aInputString, 
-                                   const nsAString & aContextStr, 
-                                   const nsAString & aInfoStr);
-  nsresult   InsertHTMLWithCharsetAndContext(const nsAString & aInputString,
-                                             const nsAString & aCharset,
-                                             const nsAString & aContextStr,
-                                             const nsAString & aInfoStr);
+  nsresult   DoContentFilterCallback(const nsAString &aFlavor,
+                                     PRBool aWillDeleteSelection,
+                                     nsIDOMNode **aFragmentAsNode,      
+                                     nsIDOMNode **aFragStartNode,
+                                     PRInt32 *aFragStartOffset,
+                                     nsIDOMNode **aFragEndNode,
+                                     PRInt32 *aFragEndOffset,
+                                     nsIDOMNode **aTargetNode,       
+                                     PRInt32 *aTargetOffset,   
+                                     PRBool *aDoContinue);
   PRBool     IsInLink(nsIDOMNode *aNode, nsCOMPtr<nsIDOMNode> *outLink = nsnull);
   nsresult   StripFormattingNodes(nsIDOMNode *aNode, PRBool aOnlyList = PR_FALSE);
-  nsresult   CreateDOMFragmentFromPaste(nsIDOMNSRange *aNSRange,
-                                        const nsAString & aInputString,
+  nsresult   CreateDOMFragmentFromPaste(const nsAString & aInputString,
                                         const nsAString & aContextStr,
                                         const nsAString & aInfoStr,
                                         nsCOMPtr<nsIDOMNode> *outFragNode,
                                         PRInt32 *outRangeStartHint,
                                         PRInt32 *outRangeEndHint);
-  nsresult   ParseFragment(const nsAString & aFragStr, nsCOMPtr<nsIDOMNode> *outNode);
+  nsresult   ParseFragment(const nsAString & aStr, nsVoidArray &aTagStack,
+                           nsIDocument* aTargetDoc,
+                           nsCOMPtr<nsIDOMNode> *outNode);
   nsresult   CreateListOfNodesToPaste(nsIDOMNode  *aFragmentAsNode,
                                       nsCOMArray<nsIDOMNode>& outNodeList,
-                                      PRInt32 aRangeStartHint,
-                                      PRInt32 aRangeEndHint);
+                                      nsIDOMNode *aStartNode,
+                                      PRInt32 aStartOffset,
+                                      nsIDOMNode *aEndNode,
+                                      PRInt32 aEndOffset);
+  nsresult CreateTagStack(nsVoidArray &aTagStack, nsIDOMNode *aNode);
+  void     FreeTagStackStrings(nsVoidArray &tagStack);
   nsresult GetListAndTableParents( PRBool aEnd, 
                                    nsCOMArray<nsIDOMNode>& aListOfNodes,
                                    nsCOMArray<nsIDOMNode>& outArray);
@@ -692,6 +743,12 @@ protected:
   /* small utility routine to test if a break node is visible to user */
   PRBool   IsVisBreak(nsIDOMNode *aNode);
 
+  /* utility routine to possibly adjust the insertion position when 
+     inserting a block level element */
+  void NormalizeEOLInsertPosition(nsIDOMNode *firstNodeToInsert,
+                                  nsCOMPtr<nsIDOMNode> *insertParentNode,
+                                  PRInt32 *insertOffset);
+
   /* small utility routine to test the eEditorReadonly bit */
   PRBool IsModifiable();
 
@@ -735,6 +792,7 @@ protected:
                                 const nsAString *aAttribute,
                                 nsCOMPtr<nsIDOMNode> *outLeftNode = nsnull,
                                 nsCOMPtr<nsIDOMNode> *outRightNode = nsnull);
+  nsresult ApplyDefaultProperties();
   nsresult RemoveStyleInside(nsIDOMNode *aNode, 
                              nsIAtom *aProperty, 
                              const nsAString *aAttribute, 
@@ -778,13 +836,16 @@ protected:
                              PRBool *aFirst, 
                              PRBool *aAny, 
                              PRBool *aAll,
-                             nsAString *outValue);
+                             nsAString *outValue,
+                             PRBool aCheckDefaults = PR_TRUE);
   nsresult HasStyleOrIdOrClass(nsIDOMElement * aElement, PRBool *aHasStyleOrIdOrClass);
   nsresult RemoveElementIfNoStyleOrIdOrClass(nsIDOMElement * aElement, nsIAtom * aTag);
 
 // Data members
 protected:
 
+  nsCOMArray<nsIContentFilter> mContentFilters;
+
   TypeInState*         mTypeInState;
 
   nsCOMPtr<nsIAtom>    mBoldAtom;
@@ -793,8 +854,6 @@ protected:
   nsCOMPtr<nsIAtom>    mFontAtom;
   nsCOMPtr<nsIAtom>    mLinkAtom;
 
-  nsCOMPtr<nsIEditProperty> mEditProperty;
-
   nsCOMPtr<nsIDOMNode> mCachedNode;
   
   PRBool   mCachedBoldStyle;
@@ -820,12 +879,76 @@ protected:
   nsStringArray mStyleSheetURLs;
   nsCOMArray<nsICSSStyleSheet> mStyleSheets;
   PRInt32 mNumStyleSheets;
+  
+  // an array for holding default style settings
+  nsVoidArray mDefaultStyles;
 
   // Maintain a static parser service ...
   static nsCOMPtr<nsIParserService> sParserService;
   // ... which means that we need an instance count to know when to delete it
   static PRInt32 sInstanceCount;
 
+protected:
+  PRPackedBool mIsImageResizingEnabled;
+  PRPackedBool mIsShowingResizeHandles;
+  PRPackedBool mIsResizing;
+  PRPackedBool mPreserveRatio;
+  PRPackedBool mResizedObjectIsAnImage;
+
+  nsCOMPtr<nsIDOMElement> mTopLeftHandle;
+  nsCOMPtr<nsIDOMElement> mTopHandle;
+  nsCOMPtr<nsIDOMElement> mTopRightHandle;
+  nsCOMPtr<nsIDOMElement> mLeftHandle;
+  nsCOMPtr<nsIDOMElement> mRightHandle;
+  nsCOMPtr<nsIDOMElement> mBottomLeftHandle;
+  nsCOMPtr<nsIDOMElement> mBottomHandle;
+  nsCOMPtr<nsIDOMElement> mBottomRightHandle;
+
+  nsCOMPtr<nsIDOMElement> mResizingShadow;
+  nsCOMPtr<nsIDOMElement> mResizingInfo;
+
+  nsCOMPtr<nsIDOMElement> mResizedObject;
+
+  nsCOMPtr<nsIDOMEventListener>  mMouseMotionListenerP;
+  nsCOMPtr<nsISelectionListener> mSelectionListenerP;
+  nsCOMPtr<nsIDOMEventListener>  mResizeEventListenerP;
+
+  PRInt32 mOriginalX;
+  PRInt32 mOriginalY;
+
+  PRInt32 mResizedObjectX;
+  PRInt32 mResizedObjectY;
+  PRInt32 mResizedObjectWidth;
+  PRInt32 mResizedObjectHeight;
+
+  PRInt32 mXIncrementFactor;
+  PRInt32 mYIncrementFactor;
+  PRInt32 mWidthIncrementFactor;
+  PRInt32 mHeightIncrementFactor;
+
+  nsresult CreateResizer(nsIDOMElement ** aReturn, PRInt16 aLocation, nsISupportsArray * aArray);
+  void     SetResizerPosition(PRInt32 aX, PRInt32 aY, nsIDOMElement *aResizer);
+  nsresult SetAllResizersPosition(nsIDOMElement * aResizedElement, PRInt32 & aX, PRInt32 & aY);
+  nsresult CreateShadow(nsIDOMElement ** aReturn, nsISupportsArray * aArray);
+  nsresult SetShadowPosition(nsIDOMElement *aResizedObject,
+                             PRInt32 aX, PRInt32 aY);
+  nsresult CreateResizingInfo(nsIDOMElement ** aReturn, nsISupportsArray * aArray);
+  nsresult SetResizingInfoPosition(PRInt32 aX, PRInt32 aY,
+                                   PRInt32 aW, PRInt32 aH);
+
+  PRInt32  GetNewResizingIncrement(PRInt32 aX, PRInt32 aY, PRInt32 aID);
+  nsresult StartResizing(nsIDOMElement * aHandle);
+  PRInt32  GetNewResizingX(PRInt32 aX, PRInt32 aY);
+  PRInt32  GetNewResizingY(PRInt32 aX, PRInt32 aY);
+  PRInt32  GetNewResizingWidth(PRInt32 aX, PRInt32 aY);
+  PRInt32  GetNewResizingHeight(PRInt32 aX, PRInt32 aY);
+  void     HideShadowAndInfo();
+  void     SetFinalSize(PRInt32 aX, PRInt32 aY);
+  void     DeleteRefToAnonymousNode(nsIDOMElement* aElement,
+                                    nsIContent * aParentContent,
+                                    nsIDocumentObserver * aDocObserver);
+  void     SetResizeIncrements(PRInt32 aX, PRInt32 aY, PRInt32 aW, PRInt32 aH, PRBool aPreserveRatio);
+  nsresult GetElementOrigin(nsIDOMElement * aElement, PRInt32 & aX, PRInt32 & aY);
 public:
 
 // friends
