@@ -62,6 +62,7 @@
 #include "nsISelectionController.h"
 #include "nsGUIEvent.h"
 #include "nsIDOMEventGroup.h"
+#include "nsCRT.h"
 
 #include "nsIIndependentSelection.h" //domselections answer to frameselection
 
@@ -77,7 +78,6 @@
 #include "nsIDOMNSRange.h"
 #include "nsISupportsArray.h"
 #include "nsVoidArray.h"
-#include "nsFileSpec.h"
 #include "nsIURL.h"
 #include "nsIComponentManager.h"
 #include "nsIServiceManager.h"
@@ -361,6 +361,8 @@ nsPlaintextEditor::InstallEventListeners()
   NS_ASSERTION(mDocWeak, "no document set on this editor");
   if (!mDocWeak) return NS_ERROR_NOT_INITIALIZED;
 
+  if (!mPresShellWeak) return NS_ERROR_NOT_INITIALIZED;
+
   nsresult result;
   // get a key listener
   result = NS_NewEditorKeyListener(getter_AddRefs(mKeyListenerP), this);
@@ -397,7 +399,8 @@ printf("nsTextEditor.cpp: failed to get TextEvent Listener\n");
   }
 
   // get a drag listener
-  result = NS_NewEditorDragListener(getter_AddRefs(mDragListenerP), this);
+  nsCOMPtr<nsIPresShell> presShell = do_QueryReferent(mPresShellWeak);
+  result = NS_NewEditorDragListener(getter_AddRefs(mDragListenerP), presShell, this);
   if (NS_FAILED(result)) {
     HandleEventListenerError();
     return result;
@@ -1958,7 +1961,8 @@ nsPlaintextEditor::SetCompositionString(const nsAString& aCompositionString, nsI
     mIMEBufferLength = aCompositionString.Length();
 
     ps->GetCaret(getter_AddRefs(caretP));
-    caretP->SetCaretDOMSelection(selection);
+    if (caretP)
+      caretP->SetCaretDOMSelection(selection);
 
     // second part of 23558 fix:
     if (aCompositionString.IsEmpty()) 
