@@ -51,7 +51,6 @@ const kDisplayModeSource = 2;
 const kDisplayModePreview = 3;
 const kDisplayModeMenuIDs = ["viewNormalMode", "viewAllTagsMode", "viewSourceMode", "viewPreviewMode"];
 const kDisplayModeTabIDS = ["NormalModeButton", "TagModeButton", "SourceModeButton", "PreviewModeButton"];
-const kBaseEditorStyleSheet = "chrome://editor/content/EditorOverride.css";
 const kNormalStyleSheet = "chrome://editor/content/EditorContent.css";
 const kAllTagsStyleSheet = "chrome://editor/content/EditorAllTags.css";
 const kParagraphMarksStyleSheet = "chrome://editor/content/EditorParagraphMarks.css";
@@ -80,6 +79,7 @@ var gDefaultTextColor = "";
 var gDefaultBackgroundColor = "";
 var gCSSPrefListener;
 var gPrefs;
+var gLocalFonts = null;
 
 var gLastFocusNode = null;
 var gLastFocusNodeWasSelected = false;
@@ -89,6 +89,78 @@ var gFontSizeNames = ["xx-small","x-small","small","medium","large","x-large","x
 
 const nsIFilePicker = Components.interfaces.nsIFilePicker;
 
+const kEditorToolbarPrefs = "editor.toolbars.showbutton.";
+
+function ShowHideToolbarSeparators(toolbar) {
+  var childNodes = toolbar.childNodes;
+  var separator = null;
+  var hideSeparator = true;
+  for (var i = 0; childNodes[i].localName != "spacer"; i++) {
+    if (childNodes[i].localName == "toolbarseparator") {
+      if (separator)
+        separator.hidden = true;
+      separator = childNodes[i];
+    } else if (!childNodes[i].hidden) {
+      if (separator)
+        separator.hidden = hideSeparator;
+      separator = null;
+      hideSeparator = false;
+    }
+  }
+}
+
+function ShowHideToolbarButtons()
+{
+  var array = GetPrefs().getChildList(kEditorToolbarPrefs, {});
+  for (var i in array) {
+    var prefName = array[i];
+    var id = prefName.substr(kEditorToolbarPrefs.length) + "Button";
+    var button = document.getElementById(id);
+    if (button)
+      button.hidden = !gPrefs.getBoolPref(prefName);
+  }
+  ShowHideToolbarSeparators(document.getElementById("EditToolbar"));
+  ShowHideToolbarSeparators(document.getElementById("FormatToolbar"));
+}
+  
+function AddToolbarPrefListener()
+{
+  try {
+    var pbi = GetPrefs().QueryInterface(Components.interfaces.nsIPrefBranchInternal);
+    pbi.addObserver(kEditorToolbarPrefs, gEditorToolbarPrefListener, false);
+  } catch(ex) {
+    dump("Failed to observe prefs: " + ex + "\n");
+  }
+}
+
+function RemoveToolbarPrefListener()
+{
+  try {
+    var pbi = GetPrefs().QueryInterface(Components.interfaces.nsIPrefBranchInternal);
+    pbi.removeObserver(kEditorToolbarPrefs, gEditorToolbarPrefListener);
+  } catch(ex) {
+    dump("Failed to remove pref observer: " + ex + "\n");
+  }
+}
+
+// Pref listener constants
+const gEditorToolbarPrefListener =
+{
+  observe: function(subject, topic, prefName)
+  {
+    // verify that we're changing a button pref
+    if (topic != "nsPref:changed")
+      return;
+
+    var id = prefName.substr(kEditorToolbarPrefs.length) + "Button";
+    var button = document.getElementById(id);
+    if (button) {
+      button.hidden = !gPrefs.getBoolPref(prefName);
+      ShowHideToolbarSeparators(button.parentNode);
+    }
+  }
+};
+
 function nsButtonPrefListener()
 {
   try {
@@ -295,9 +367,6 @@ var gEditorDocumentObserver =
         try {
           editor.QueryInterface(nsIEditorStyleSheets);
 
-          // Add the base sheets for editor cursor etc.
-          editor.addOverrideStyleSheet(kBaseEditorStyleSheet);
-
           //  and extra styles for showing anchors, table borders, smileys, etc
           editor.addOverrideStyleSheet(kNormalStyleSheet);
         } catch (e) {}
@@ -305,14 +374,6 @@ var gEditorDocumentObserver =
         // Things for just the Web Composer application
         if (IsWebComposer())
         {
-          // Get the window command controller created in SetupComposerWindowCommands()
-          if (gComposerWindowControllerID)
-          {
-            try { 
-              var controller = window.controllers.getControllerById(gComposerWindowControllerID);
-              controller.SetCommandRefCon(editor.QueryInterface(Components.interfaces.nsISupports));
-            } catch (e) {}
-          }
           // Call EditorSetDefaultPrefsAndDoctype first so it gets the default author before initing toolbars
           EditorSetDefaultPrefsAndDoctype();
 
@@ -376,7 +437,6 @@ var gEditorDocumentObserver =
           onFontColorChange();
           onBackgroundColorChange();
         }
-
         break;
 
       case "cmd_setDocumentModified":
@@ -435,6 +495,9 @@ function EditorStartup()
   //  such as file-related commands, HTML Source editing, Edit Modes...
   SetupComposerWindowCommands();
 
+  ShowHideToolbarButtons();
+  AddToolbarPrefListener();
+
   gCSSPrefListener = new nsButtonPrefListener();
 
   // hide Highlight button if we are in an HTML editor with CSS mode off
@@ -479,7 +542,7 @@ function EditorSharedStartup()
   gContentWindow = window.content;
 
   // Set up the mime type and register the commands.
-  if (IsHTMLEditor)
+  if (IsHTMLEditor())
     SetupHTMLEditorCommands();
   else
     SetupTextEditorCommands();
@@ -570,6 +633,8 @@ function EditorResetFontAndColorAttributes()
 
 function EditorShutdown()
 {
+  RemoveToolbarPrefListener();
+
   try {
     var commandManager = GetCurrentCommandManager();
     commandManager.removeCommandObserver(gEditorDocumentObserver, "obs_documentCreated");
@@ -920,6 +985,8 @@ function EditorSetFontSize(size)
 
 function initFontFaceMenu(menuPopup)
 {
+  initLocalFontFaceMenu(menuPopup);
+
   if (menuPopup)
   {
     var children = menuPopup.childNodes;
@@ -935,7 +1002,11 @@ function initFontFaceMenu(menuPopup)
     // Fixed width (second menu item) is special case: old TT ("teletype") attribute
     EditorGetTextProperty("tt", "", "", firstHas, anyHas, allHas);
     children[1].setAttribute("checked", allHas.value);
-    var fontWasFound = anyHas.value;
+
+    if (!anyHas.value)
+      EditorGetTextProperty("font", "face", "", firstHas, anyHas, allHas);
+
+    children[0].setAttribute("checked", !anyHas.value);
 
     // Skip over default, TT, and separator
     for (var i = 3; i < children.length; i++)
@@ -947,18 +1018,62 @@ function initFontFaceMenu(menuPopup)
       {
         EditorGetTextProperty("font", "face", faceType, firstHas, anyHas, allHas);
 
-        // Check the item only if all of selection has the face...
-        menuItem.setAttribute("checked", allHas.value);
-        // ...but remember if ANY part of the selection has it
-        fontWasFound |= anyHas.value;
+        // Check the menuitem only if all of selection has the face
+        if (allHas.value)
+        {
+          menuItem.setAttribute("checked", "true");
+          break;
+        }
+
+        // in case none match, make sure we've cleared the checkmark
+        menuItem.removeAttribute("checked");
       }
     }
-    // Check the default item if no other item was checked
-    // note that no item is checked in the case of "mixed" selection
-    children[0].setAttribute("checked", !fontWasFound);
   }
 }
 
+const kFixedFontFaceMenuItems = 7; // number of fixed font face menuitems
+
+function initLocalFontFaceMenu(menuPopup)
+{
+  if (!gLocalFonts)
+  {
+    // Build list of all local fonts once per editor
+    try 
+    {
+      var enumerator = Components.classes["@mozilla.org/gfx/fontenumerator;1"]
+                                 .getService(Components.interfaces.nsIFontEnumerator);
+      var localFontCount = { value: 0 }
+      gLocalFonts = enumerator.EnumerateAllFonts(localFontCount);
+    }
+    catch(e) { }
+  }
+  
+  var useRadioMenuitems = (menuPopup.parentNode.localName == "menu"); // don't do this for menulists  
+  if (menuPopup.childNodes.length == kFixedFontFaceMenuItems) 
+  {
+    if (gLocalFonts.length == 0) {
+      menuPopup.childNodes[kFixedFontFaceMenuItems - 1].hidden = true;
+    }
+    for (var i = 0; i < gLocalFonts.length; ++i)
+    {
+      if (gLocalFonts[i] != "")
+      {
+        var itemNode = document.createElementNS(XUL_NS, "menuitem");
+        itemNode.setAttribute("label", gLocalFonts[i]);
+        itemNode.setAttribute("value", gLocalFonts[i]);
+        if (useRadioMenuitems) {
+          itemNode.setAttribute("type", "radio");
+          itemNode.setAttribute("name", "2");
+          itemNode.setAttribute("observes", "cmd_renderedHTMLEnabler");
+        }
+        menuPopup.appendChild(itemNode);
+      }
+    }
+  }
+}
+ 
+
 function initFontSizeMenu(menuPopup)
 {
   if (menuPopup)
@@ -1023,7 +1138,7 @@ function initFontSizeMenu(menuPopup)
     // note that no item is checked in the case of "mixed" selection
     children[mediumIndex].setAttribute("checked", !sizeWasFound);
   }
- }
+}
 
 function onHighlightColorChange()
 {
@@ -1226,6 +1341,7 @@ function EditorSelectColor(colorType, mouseEvent)
     // Get color from command node state
     commandNode = document.getElementById("cmd_fontColor");
     currentColor = commandNode.getAttribute("state");
+    currentColor = ConvertRGBColorIntoHEXColor(currentColor);
     gColorObj.TextColor = currentColor;
 
     if (useLastColor && gColorObj.LastTextColor )
@@ -1240,6 +1356,7 @@ function EditorSelectColor(colorType, mouseEvent)
     // Get color from command node state
     commandNode = document.getElementById("cmd_highlight");
     currentColor = commandNode.getAttribute("state");
+    currentColor = ConvertRGBColorIntoHEXColor(currentColor);
     gColorObj.HighlightColor = currentColor;
 
     if (useLastColor && gColorObj.LastHighlightColor )
@@ -1758,6 +1875,7 @@ function SetDisplayMode(mode)
     try {
       var editor = GetCurrentEditor();
       editor.QueryInterface(nsIEditorStyleSheets);
+      editor instanceof Components.interfaces.nsIHTMLObjectResizer;
 
       switch (mode)
       {
@@ -1765,17 +1883,25 @@ function SetDisplayMode(mode)
           // Disable all extra "edit mode" style sheets 
           editor.enableStyleSheet(kNormalStyleSheet, false);
           editor.enableStyleSheet(kAllTagsStyleSheet, false);
+          editor.isImageResizingEnabled = true;
           break;
 
         case kDisplayModeNormal:
           editor.addOverrideStyleSheet(kNormalStyleSheet);
           // Disable ShowAllTags mode
           editor.enableStyleSheet(kAllTagsStyleSheet, false);
+          editor.isImageResizingEnabled = true;
           break;
 
         case kDisplayModeAllTags:
           editor.addOverrideStyleSheet(kNormalStyleSheet);
           editor.addOverrideStyleSheet(kAllTagsStyleSheet);
+          // don't allow resizing in AllTags mode because the visible tags
+          // change the computed size of images and tables...
+          if (editor.resizedObject) {
+            editor.hideResizers();
+          }
+          editor.isImageResizingEnabled = false;
           break;
       }
     } catch(e) {}
@@ -2258,7 +2384,7 @@ function EditorSetDefaultPrefsAndDoctype()
   // if it is missing from existing doc
   if (!domdoc.doctype)
   {
-    var newdoctype = domdoc.implementation.createDocumentType("html", "-//W3C//DTD HTML 4.01 Transitional//EN","");
+    var newdoctype = domdoc.implementation.createDocumentType("HTML", "-//W3C//DTD HTML 4.01 Transitional//EN","");
     if (newdoctype)
       domdoc.insertBefore(newdoctype, domdoc.firstChild);
   }
@@ -2414,7 +2540,8 @@ function EditorSetDefaultPrefsAndDoctype()
     // Default image is independent of Custom colors???
     try {
       var background_image = gPrefs.getCharPref("editor.default_background_image");
-      editor.setAttributeOrEquivalent(bodyelement, "background", background_image, true);
+      if (background_image)
+        editor.setAttributeOrEquivalent(bodyelement, "background", background_image, true);
     } catch (e) {dump("BACKGROUND EXCEPTION: "+e+"\n"); }
 
   }
