@@ -18,40 +18,37 @@
  * Rights Reserved.
  *
  * Contributor(s):
+ *   Charley Manske (cmanske@netscape.com)
+ *   Ryan Cassin (rcassin@supernova.org)
  */
 
-var gPublishPanel = 0;
-var gSettingsPanel = 1;
-var gCurrentPanel = gPublishPanel;
 var gPublishSiteData;
 var gReturnData;
 var gDefaultSiteIndex = -1;
 var gDefaultSiteName;
-var gPreviousDefaultSite;
 var gPreviousDefaultDir;
 var gPreviousTitle;
 var gSettingsChanged = false;
 var gInitialSiteName;
 var gInitialSiteIndex = -1;
+var gPasswordManagerOn = true;
 
 // Dialog initialization code
 function Startup()
 {
   window.opener.ok = false;
 
-  if (!InitEditorShell()) return;
-
   // Element to edit is passed in
   gInitialSiteName = window.arguments[1];
   gReturnData = window.arguments[2];
-  if (!gReturnData)
+  if (!gReturnData || !GetCurrentEditor())
   {
-    dump("Publish: Return data object not supplied\n");
+    dump("Publish: No editor or return data object not supplied\n");
     window.close();
     return;
   }
 
-  gDialog.TabPanels           = document.getElementById("TabPanels");
+  gDialog.TabBox              = document.getElementById("TabBox");
   gDialog.PublishTab          = document.getElementById("PublishTab");
   gDialog.SettingsTab         = document.getElementById("SettingsTab");
 
@@ -67,7 +64,6 @@ function Startup()
   gDialog.OtherDirList        = document.getElementById("OtherDirList");
 
   // Settings Panel
-  gDialog.SettingsPanel       = document.getElementById("SettingsPanel");
   gDialog.SiteNameInput       = document.getElementById("SiteNameInput");
   gDialog.PublishUrlInput     = document.getElementById("PublishUrlInput");
   gDialog.BrowseUrlInput      = document.getElementById("BrowseUrlInput");
@@ -80,9 +76,11 @@ function Startup()
   // Change 'Ok' button to 'Publish'
   gDialog.PublishButton.setAttribute("label", GetString("Publish"));
   
+  gPasswordManagerOn = GetBoolPref("signon.rememberSignons");
+  gDialog.SavePassword.disabled = !gPasswordManagerOn;
+
   gPublishSiteData = GetPublishSiteData();
   gDefaultSiteName = GetDefaultPublishSiteName();
-  gPreviousDefaultSite = gDefaultSiteName;
 
   var addNewSite = false;
   if (gPublishSiteData)
@@ -114,7 +112,7 @@ function Startup()
       {
         var dirObj = {};
         var siteIndex = FindSiteIndexAndDocDir(gPublishSiteData, docUrl, dirObj);
-        
+
         // Select this site only if the same as user's intended site, or there wasn't one
         if (siteIndex != -1 && (gInitialSiteIndex == -1 || siteIndex == gInitialSiteIndex))
         {
@@ -155,11 +153,11 @@ function Startup()
     }
   }
   try {
-    gPreviousTitle = editorShell.GetDocumentTitle();
+    gPreviousTitle = GetDocumentTitle();
   } catch (e) {}
 
   gDialog.PageTitleInput.value = gPreviousTitle;
-  gDialog.FilenameInput.value = filename;
+  gDialog.FilenameInput.value = unescape(filename);
   
   if (!addNewSite)
   {
@@ -195,7 +193,7 @@ function Startup()
 
 function FillSiteList()
 {
-  ClearMenulist(gDialog.SiteList);
+  gDialog.SiteList.removeAllItems();
   gDefaultSiteIndex = -1;
 
   // Fill the site lists
@@ -205,11 +203,7 @@ function FillSiteList()
   for (i = 0; i < count; i++)
   {
     var name = gPublishSiteData[i].siteName;
-    // XXX Bug 131481: Using "max-width" style and "crop" attribute 
-    //     don't constrain grid width properly, resulting in cut-off buttons
-    //     We'll truncate label strings to prevent this problem
-    var menuitem = AppendLabelAndValueToMenulist(gDialog.SiteList, 
-                        TruncateStringAtWordEnd(name, 30, true), name);
+    var menuitem = gDialog.SiteList.appendItem(name);
     // Highlight the default site
     if (name == gDefaultSiteName)
     {
@@ -246,8 +240,8 @@ function SelectSiteList()
   var savePassword = false;
   var publishOtherFiles = true;
 
-  ClearMenulist(gDialog.DocDirList);
-  ClearMenulist(gDialog.OtherDirList);
+  gDialog.DocDirList.removeAllItems();
+  gDialog.OtherDirList.removeAllItems();
 
   if (gPublishSiteData && selectedSiteIndex != -1)
   {
@@ -255,7 +249,7 @@ function SelectSiteList()
     publishUrl = gPublishSiteData[selectedSiteIndex].publishUrl;
     browseUrl = gPublishSiteData[selectedSiteIndex].browseUrl;
     username = gPublishSiteData[selectedSiteIndex].username;
-    savePassword = gPublishSiteData[selectedSiteIndex].savePassword;
+    savePassword = gPasswordManagerOn ? gPublishSiteData[selectedSiteIndex].savePassword : false;
     if (savePassword)
       password = gPublishSiteData[selectedSiteIndex].password;
 
@@ -264,13 +258,10 @@ function SelectSiteList()
     {
       for (var i = 0; i < gPublishSiteData[selectedSiteIndex].dirList.length; i++)
       {
-        AppendStringToMenulist(gDialog.DocDirList, gPublishSiteData[selectedSiteIndex].dirList[i]);
-        AppendStringToMenulist(gDialog.OtherDirList, gPublishSiteData[selectedSiteIndex].dirList[i]);
+        gDialog.DocDirList.appendItem(gPublishSiteData[selectedSiteIndex].dirList[i]);
+        gDialog.OtherDirList.appendItem(gPublishSiteData[selectedSiteIndex].dirList[i]);
       }
     }
-    gDialog.DocDirList.value = gPublishSiteData[selectedSiteIndex].docDir;
-    gDialog.OtherDirList.value = gPublishSiteData[selectedSiteIndex].otherDir;
-
     gDialog.DocDirList.value = FormatDirForPublishing(gPublishSiteData[selectedSiteIndex].docDir);
     gDialog.OtherDirList.value = FormatDirForPublishing(gPublishSiteData[selectedSiteIndex].otherDir);
     publishOtherFiles = gPublishSiteData[selectedSiteIndex].publishOtherFiles;
@@ -298,7 +289,7 @@ function AddNewSite()
   // Button in Publish panel allows user
   //  to automatically switch to "Settings" panel
   //  to enter data for new site
-  SwitchPanel(gSettingsPanel);
+  SwitchPanel(gDialog.SettingsTab);
   
   gDialog.SiteList.selectedIndex = -1;
 
@@ -311,46 +302,23 @@ function AddNewSite()
 
 function SelectPublishTab()
 {
-  if (gSettingsChanged)
-  {
-    gCurrentPanel = gPublishPanel;
-    if (!ValidateSettings())
-      return;
+  if (gSettingsChanged && !ValidateSettings())
+    return;
 
-    gCurrentPanel = gSettingsPanel;
-  }
-
-  SwitchPanel(gPublishPanel);
+  SwitchPanel(gDialog.PublishTab);
   SetTextboxFocus(gDialog.PageTitleInput);
 }
 
 function SelectSettingsTab()
 {
-  SwitchPanel(gSettingsPanel);
+  SwitchPanel(gDialog.SettingsTab);
   SetTextboxFocus(gDialog.SiteNameInput);
 }
 
-function SwitchPanel(panel)
+function SwitchPanel(tab)
 {
-  if (gCurrentPanel != panel)
-  {
-    // Set index for starting panel on the <tabpanels> element
-    gDialog.TabPanels.selectedIndex = panel;
-    if (panel == gSettingsPanel)
-    {
-      // Trigger setting of style for the tab widgets
-      gDialog.SettingsTab.selected = "true";
-      gDialog.PublishTab.selected = null;
-    } else {
-      gDialog.PublishTab.selected = "true";
-      gDialog.SettingsTab.selected = null;
-    }
-    gCurrentPanel = panel;
-
-    // XXX Another hack to workaround bug 131481
-    // Resize dialog to be sure buttons are not cut off the right
-    window.sizeToContent();
-  }
+  if (gDialog.TabBox.selectedTab != tab)
+    gDialog.TabBox.selectedTab = tab;
 }
 
 function onInputSettings()
@@ -395,12 +363,12 @@ function ValidateSettings()
   var siteName = TrimString(gDialog.SiteNameInput.value);
   if (!siteName)
   {
-    ShowErrorInPanel(gSettingsPanel, "MissingSiteNameError", gDialog.SiteNameInput);
+    ShowErrorInPanel(gDialog.SettingsTab, "MissingSiteNameError", gDialog.SiteNameInput);
     return false;
   }
   if (PublishSiteNameExists(siteName, gPublishSiteData, gDialog.SiteList.selectedIndex))
   {
-    SwitchPanel(gSettingsPanel);
+    SwitchPanel(gDialog.SettingsTab);
     ShowInputErrorMessage(GetString("DuplicateSiteNameError").replace(/%name%/, siteName));            
     SetTextboxFocus(gDialog.SiteNameInput);
     return false;
@@ -413,14 +381,19 @@ function ValidateSettings()
   if (publishUrl)
   {
     publishUrl = FormatUrlForPublishing(publishUrl);
+
+    // Assume scheme = "ftp://" if missing
+    // This compensates when user enters hostname w/o scheme (as most ISPs provide)
+    if (!GetScheme(publishUrl))
+      publishUrl = "ftp://" + publishUrl; 
+
     gDialog.PublishUrlInput.value = publishUrl;
   }
   else
   {
-    ShowErrorInPanel(gSettingsPanel, "MissingPublishUrlError", gDialog.PublishUrlInput);
+    ShowErrorInPanel(gDialog.SettingsTab, "MissingPublishUrlError", gDialog.PublishUrlInput);
     return false;
   }
-
   var browseUrl = GetBrowseUrlInput();
 
   var username = TrimString(gDialog.UsernameInput.value);
@@ -468,6 +441,7 @@ function ValidateSettings()
     gPublishSiteData[siteIndex].otherDir = "";
     gPublishSiteData[siteIndex].dirList = [""];
     gPublishSiteData[siteIndex].publishOtherFiles = true;
+    gPublishSiteData[siteIndex].previousSiteName = siteName;
     newSite = true;
   }
   gPublishSiteData[siteIndex].siteName = siteName;
@@ -504,16 +478,17 @@ function ValidateSettings()
   {
     // Update selected item if sitename changed 
     var selectedItem = gDialog.SiteList.selectedItem;
-    if (selectedItem && selectedItem.getAttribute("label") != siteName)
+    if (selectedItem)
     {
-      // XXX More hacks to workaround bug 131481
-      // The real sitename
-      selectedItem.setAttribute("value", siteName);
-
-      // Truncate string to show in the menulist
-      var truncatedName = TruncateStringAtWordEnd(siteName, 30, true);
-      selectedItem.setAttribute("label", truncatedName);
-      gDialog.SiteList.setAttribute("label", truncatedName);
+      var oldName = selectedItem.getAttribute("label");
+      if (oldName != siteName)
+      {
+        selectedItem.setAttribute("label", siteName);
+        gDialog.SiteList.setAttribute("label", siteName);
+        gSettingsChanged = true;
+        if (oldName == gDefaultSiteName)
+          gDefaultSiteName = siteName;
+      }
     }
   }
   
@@ -533,6 +508,7 @@ function ValidateSettings()
 
   // Fill return data object
   gReturnData.siteName = siteName;
+  gReturnData.previousSiteName = gPublishSiteData[siteIndex].previousSiteName;
   gReturnData.publishUrl = publishUrl;
   gReturnData.browseUrl = browseUrl;
   gReturnData.username = username;
@@ -559,7 +535,7 @@ function ValidateData()
   var filename = TrimString(gDialog.FilenameInput.value);
   if (!filename)
   {
-    ShowErrorInPanel(gPublishPanel, "MissingPublishFilename", gDialog.FilenameInput);
+    ShowErrorInPanel(gDialog.PublishTab, "MissingPublishFilename", gDialog.FilenameInput);
     return false;
   }
   gReturnData.filename = filename;
@@ -567,9 +543,9 @@ function ValidateData()
   return true;
 }
 
-function ShowErrorInPanel(panelId, errorMsgId, widgetWithError)
+function ShowErrorInPanel(tab, errorMsgId, widgetWithError)
 {
-  SwitchPanel(panelId);
+  SwitchPanel(tab);
   ShowInputErrorMessage(GetString(errorMsgId));
   if (widgetWithError)
     SetTextboxFocus(widgetWithError);
@@ -577,7 +553,7 @@ function ShowErrorInPanel(panelId, errorMsgId, widgetWithError)
 
 function doHelpButton()
 {
-  if (gCurrentPanel == gPublishPanel)
+  if (gDialog.TabBox.selectedTab == gDialog.PublishTab)
     openHelp("comp-doc-publish-publishtab");
   else
     openHelp("comp-doc-publish-settingstab");
@@ -600,11 +576,7 @@ function onAccept()
 
     var title = TrimString(gDialog.PageTitleInput.value);
     if (title != gPreviousTitle)
-    {
-      try {
-        editorShell.SetDocumentTitle(title);
-      } catch (e) {}
-    }
+      SetDocumentTitle(title);
 
     SaveWindowLocation();
     window.opener.ok = true;
