@@ -114,6 +114,7 @@ public:
 #include "nsNetUtil.h"
 
 #include "nsIURI.h"
+#include "nsIDocument.h"
 #include "nsIDOMWindow.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMDocument.h"
@@ -121,11 +122,15 @@ public:
 #include "nsIDOMLocation.h"
 #include "nsIWebNavigation.h"
 #include "nsILinkHandler.h"
+#include "nsIScriptGlobalObject.h"
+#include "nsIScriptContext.h"
+#include "nsIPrincipal.h"
 
 #include "XPConnect.h"
 #include "XPCBrowser.h"
 #include "LegacyPlugin.h"
 
+#include "IHTMLLocationImpl.h"
 
 /*
  * This file contains partial implementations of various IE objects and
@@ -135,10 +140,60 @@ public:
  * might want to initiate a load, or obtain the user agent.
  */
 
+class IELocation :
+    public CComObjectRootEx<CComSingleThreadModel>,
+    public IHTMLLocationImpl<IELocation>
+{
+public:
+BEGIN_COM_MAP(IELocation)
+    COM_INTERFACE_ENTRY(IDispatch)
+    COM_INTERFACE_ENTRY(IHTMLLocation)
+END_COM_MAP()
+
+    PluginInstanceData *mData;
+    nsCOMPtr<nsIDOMLocation> mDOMLocation;
+
+    IELocation() : mData(NULL)
+    {
+    }
+
+    HRESULT Init(PluginInstanceData *pData)
+    {
+	NS_PRECONDITION(pData != nsnull, "null ptr");
+
+        mData = pData;
+
+        // Get the DOM window
+        nsCOMPtr<nsIDOMWindow> domWindow;
+        NPN_GetValue(mData->pPluginInstance, NPNVDOMWindow, 
+                     NS_STATIC_CAST(nsIDOMWindow **, getter_AddRefs(domWindow)));
+        if (!domWindow)
+        {
+            return E_FAIL;
+        }
+        nsCOMPtr<nsIDOMWindowInternal> windowInternal = do_QueryInterface(domWindow);
+        if (windowInternal)
+        {
+            windowInternal->GetLocation(getter_AddRefs(mDOMLocation));
+        }
+        if (!mDOMLocation)
+            return E_FAIL;
+
+        return S_OK;
+    }
+
+    virtual nsresult GetDOMLocation(nsIDOMLocation **aLocation)
+    {
+        *aLocation = mDOMLocation;
+        NS_IF_ADDREF(*aLocation);
+        return NS_OK;
+    }
+};
+
 // Note: corresponds to the window.navigator property in the IE DOM
 class IENavigator :
     public CComObjectRootEx<CComSingleThreadModel>,
-    public IDispatchImpl<IOmNavigator, &IID_IOmNavigator, &LIBID_MSHTML>
+    public IDispatchImpl<IOmNavigator, &__uuidof(IOmNavigator), &LIBID_MSHTML>
 {
 public:
 BEGIN_COM_MAP(IENavigator)
@@ -285,13 +340,14 @@ END_COM_MAP()
 // Note: Corresponds to the window object in the IE DOM
 class IEWindow :
     public CComObjectRootEx<CComSingleThreadModel>,
-    public IDispatchImpl<IHTMLWindow2, &IID_IHTMLWindow2, &LIBID_MSHTML>
+    public IDispatchImpl<IHTMLWindow2, &__uuidof(IHTMLWindow2), &LIBID_MSHTML>
 {
 public:
     PluginInstanceData *mData;
     CComObject<IENavigator> *mNavigator;
+    CComObject<IELocation>  *mLocation;
 
-    IEWindow() : mNavigator(NULL), mData(NULL)
+    IEWindow() : mNavigator(NULL), mLocation(NULL), mData(NULL)
     {
     }
 
@@ -306,6 +362,15 @@ public:
         }
         mNavigator->AddRef();
         mNavigator->Init(mData);
+
+        CComObject<IELocation>::CreateInstance(&mLocation);
+        if (!mLocation)
+        {
+            return E_UNEXPECTED;
+        }
+        mLocation->AddRef();
+        mLocation->Init(mData);
+
         return S_OK;
     }
 
@@ -316,6 +381,10 @@ protected:
         {
             mNavigator->Release();
         }
+        if (mLocation)
+        {
+            mLocation->Release();
+        }
     }
 
 public:
@@ -417,7 +486,9 @@ END_COM_MAP()
     virtual /* [id][propget] */ HRESULT STDMETHODCALLTYPE get_location( 
         /* [out][retval] */ IHTMLLocation __RPC_FAR *__RPC_FAR *p)
     {
-        return E_NOTIMPL;
+        if (mLocation)
+            return mLocation->QueryInterface(__uuidof(IHTMLLocation), (void **) p);
+        return E_FAIL;
     }
     
     virtual /* [id][propget] */ HRESULT STDMETHODCALLTYPE get_history( 
@@ -446,7 +517,9 @@ END_COM_MAP()
     virtual /* [id][propget] */ HRESULT STDMETHODCALLTYPE get_navigator( 
         /* [out][retval] */ IOmNavigator __RPC_FAR *__RPC_FAR *p)
     {
-        return mNavigator->QueryInterface(__uuidof(IOmNavigator), (void **) p);
+        if (mNavigator)
+            return mNavigator->QueryInterface(__uuidof(IOmNavigator), (void **) p);
+        return E_FAIL;
     }
     
     virtual /* [id][propput] */ HRESULT STDMETHODCALLTYPE put_name( 
@@ -464,7 +537,8 @@ END_COM_MAP()
     virtual /* [id][propget] */ HRESULT STDMETHODCALLTYPE get_parent( 
         /* [out][retval] */ IHTMLWindow2 __RPC_FAR *__RPC_FAR *p)
     {
-        return E_NOTIMPL;
+        *p = NULL;
+        return S_OK;
     }
     
     virtual /* [id] */ HRESULT STDMETHODCALLTYPE open( 
@@ -474,7 +548,8 @@ END_COM_MAP()
         /* [in][defaultvalue] */ VARIANT_BOOL replace,
         /* [out][retval] */ IHTMLWindow2 __RPC_FAR *__RPC_FAR *pomWindowResult)
     {
-        return E_NOTIMPL;
+        *pomWindowResult = NULL;
+        return E_FAIL;
     }
     
     virtual /* [id][propget] */ HRESULT STDMETHODCALLTYPE get_self( 
@@ -486,13 +561,15 @@ END_COM_MAP()
     virtual /* [id][propget] */ HRESULT STDMETHODCALLTYPE get_top( 
         /* [out][retval] */ IHTMLWindow2 __RPC_FAR *__RPC_FAR *p)
     {
-        return E_NOTIMPL;
+        *p = NULL;
+        return S_OK;
     }
     
     virtual /* [id][propget] */ HRESULT STDMETHODCALLTYPE get_window( 
         /* [out][retval] */ IHTMLWindow2 __RPC_FAR *__RPC_FAR *p)
     {
-        return E_NOTIMPL;
+        *p = NULL;
+        return S_OK;
     }
     
     virtual /* [id] */ HRESULT STDMETHODCALLTYPE navigate( 
@@ -717,7 +794,65 @@ END_COM_MAP()
         /* [in][defaultvalue] */ BSTR language,
         /* [out][retval] */ VARIANT __RPC_FAR *pvarRet)
     {
-        return E_NOTIMPL;
+        nsresult rv;
+
+        nsCOMPtr<nsIDOMWindow> domWindow;
+        NPN_GetValue(mData->pPluginInstance, NPNVDOMWindow, 
+                     NS_STATIC_CAST(nsIDOMWindow **, getter_AddRefs(domWindow)));
+        if (!domWindow)
+        {
+            return E_UNEXPECTED;
+        }
+
+        // Now get the DOM Document.  Accessing the document will create one
+        // if necessary.  So, basically, this call ensures that a document gets
+        // created -- if necessary.
+        nsCOMPtr<nsIDOMDocument> domDocument;
+        rv = domWindow->GetDocument(getter_AddRefs(domDocument));
+        NS_ASSERTION(domDocument, "No DOMDocument!");
+        if (NS_FAILED(rv)) {
+            return E_UNEXPECTED;
+        }
+
+        nsCOMPtr<nsIScriptGlobalObject> globalObject(do_QueryInterface(domWindow));
+        if (!globalObject)
+            return E_UNEXPECTED;
+
+        nsCOMPtr<nsIScriptContext> scriptContext;
+        if (NS_FAILED(globalObject->GetContext(getter_AddRefs(scriptContext))) ||
+                !scriptContext)
+            return E_UNEXPECTED;
+
+        nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDocument));
+        if (!doc)
+            return E_UNEXPECTED;
+
+        nsCOMPtr<nsIPrincipal> principal;      
+        doc->GetPrincipal(getter_AddRefs(principal));
+        if (!principal)
+            return E_UNEXPECTED;
+
+        // Execute the script.
+        //
+        // Note: The script context takes care of the JS stack and of ensuring
+        //       nothing is executed when JS is disabled.
+        //
+        nsAutoString scriptString(code);
+        NS_NAMED_LITERAL_CSTRING(url, "javascript:axplugin");
+        nsAutoString result;
+        rv = scriptContext->EvaluateString(scriptString,
+                                           nsnull,      // obj
+                                           principal,
+                                           url.get(),   // url
+                                           1,           // line no
+                                           nsnull,
+                                           result,
+                                           nsnull);
+
+        if (NS_FAILED(rv))
+            return NS_ERROR_FAILURE;
+
+        return S_OK;
     }
     
     virtual /* [id] */ HRESULT STDMETHODCALLTYPE toString( 
@@ -780,7 +915,7 @@ END_COM_MAP()
 // Note: Corresponds to the document object in the IE DOM
 class IEDocument :
     public CComObjectRootEx<CComSingleThreadModel>,
-    public IDispatchImpl<IHTMLDocument2, &IID_IHTMLDocument2, &LIBID_MSHTML>,
+    public IDispatchImpl<IHTMLDocument2, &__uuidof(IHTMLDocument2), &LIBID_MSHTML>,
     public IServiceProvider,
     public IOleContainer,
     public IBindHost,
@@ -911,6 +1046,10 @@ END_COM_MAP()
         else if (IsEqualIID(riid, __uuidof(IHTMLWindow2)))
         {
             ATLTRACE(_T("  IHTMLWindow2\n"));
+            if (mWindow)
+            {
+                return mWindow->QueryInterface(riid, ppvObject);
+            }
         }
         else if (IsEqualIID(riid, __uuidof(IHTMLDocument2)))
         {
@@ -1117,7 +1256,9 @@ END_COM_MAP()
     virtual /* [id][propget] */ HRESULT STDMETHODCALLTYPE get_location( 
         /* [out][retval] */ IHTMLLocation **p)
     {
-        return E_NOTIMPL;
+        if (mWindow)
+            return mWindow->get_location(p);
+        return E_FAIL;
     }
     
     virtual /* [id][propget] */ HRESULT STDMETHODCALLTYPE get_lastModified( 
