@@ -51,6 +51,10 @@
 // for profiles
 #include <nsProfileDirServiceProvider.h>
 
+// app component registration
+#include <nsIGenericFactory.h>
+#include <nsIComponentRegistrar.h>
+
 // all of our local includes
 #include "EmbedPrivate.h"
 #include "EmbedWindow.h"
@@ -59,6 +63,9 @@
 #include "EmbedEventListener.h"
 #include "EmbedWindowCreator.h"
 #include "EmbedStream.h"
+#ifdef MOZ_WIDGET_GTK2
+#include "GtkPromptService.h"
+#endif
 
 #ifdef _BUILD_STATIC_BIN
 #include "nsStaticComponent.h"
@@ -80,6 +87,34 @@ GtkWidget   *EmbedPrivate::sOffscreenFixed  = 0;
 nsIDirectoryServiceProvider *EmbedPrivate::sAppFileLocProvider = nsnull;
 nsProfileDirServiceProvider *EmbedPrivate::sProfileDirServiceProvider = nsnull;
 
+#define NS_PROMPTSERVICE_CID \
+ {0x95611356, 0xf583, 0x46f5, {0x81, 0xff, 0x4b, 0x3e, 0x01, 0x62, 0xc6, 0x19}}
+
+#ifdef MOZ_WIDGET_GTK2
+NS_GENERIC_FACTORY_CONSTRUCTOR(GtkPromptService)
+#endif
+
+#ifdef MOZ_WIDGET_GTK2
+
+static const nsModuleComponentInfo defaultAppComps[] = {
+  {
+    "Prompt Service",
+    NS_PROMPTSERVICE_CID,
+    "@mozilla.org/embedcomp/prompt-service;1",
+    GtkPromptServiceConstructor
+  }
+};
+
+const nsModuleComponentInfo *EmbedPrivate::sAppComps = defaultAppComps;
+int   EmbedPrivate::sNumAppComps = sizeof(defaultAppComps) / sizeof(nsModuleComponentInfo);
+
+#else
+
+const nsModuleComponentInfo *EmbedPrivate::sAppComps = nsnull;
+int   EmbedPrivate::sNumAppComps = 0;
+
+#endif
+
 EmbedPrivate::EmbedPrivate(void)
 {
   mOwningWidget     = nsnull;
@@ -326,7 +361,13 @@ EmbedPrivate::Destroy(void)
 void
 EmbedPrivate::SetURI(const char *aURI)
 {
+#ifdef MOZ_WIDGET_GTK
   mURI.AssignWithConversion(aURI);
+#endif
+
+#ifdef MOZ_WIDGET_GTK2
+  mURI.Assign(NS_ConvertUTF8toUCS2(aURI));
+#endif
 }
 
 void
@@ -374,9 +415,10 @@ EmbedPrivate::PushStartup(void)
     }
 
     rv = StartupProfile();
-    if (NS_FAILED(rv))
-      NS_WARNING("Warning: Failed to start up profiles.\n");
-    
+    NS_WARN_IF_FALSE(NS_SUCCEEDED(rv), "Warning: Failed to start up profiles.\n");
+
+    rv = RegisterAppComponents();
+    NS_WARN_IF_FALSE(NS_SUCCEEDED(rv), "Warning: Failed to register app components.\n");
 
     // XXX startup appshell service?
     // XXX create offscreen window for appshell service?
@@ -432,6 +474,15 @@ EmbedPrivate::SetCompPath(char *aPath)
     sCompPath = nsnull;
 }
 
+/* static */
+void
+EmbedPrivate::SetAppComponents(const nsModuleComponentInfo* aComps,
+                               int aNumComponents)
+{
+  sAppComps = aComps;
+  sNumAppComps = aNumComponents;
+}
+
 /* static */
 void
 EmbedPrivate::SetProfilePath(char *aDir, char *aName)
@@ -832,6 +883,31 @@ EmbedPrivate::ShutdownProfile(void)
   }
 }
 
+/* static */
+nsresult
+EmbedPrivate::RegisterAppComponents(void)
+{
+  nsCOMPtr<nsIComponentRegistrar> cr;
+  nsresult rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  for (int i = 0; i < sNumAppComps; ++i) {
+    nsCOMPtr<nsIGenericFactory> componentFactory;
+    rv = NS_NewGenericFactory(getter_AddRefs(componentFactory),
+                              &(sAppComps[i]));
+    if (NS_FAILED(rv)) {
+      NS_WARNING("Unable to create factory for component");
+      continue;  // don't abort registering other components
+    }
+
+    rv = cr->RegisterFactory(sAppComps[i].mCID, sAppComps[i].mDescription,
+                             sAppComps[i].mContractID, componentFactory);
+    NS_ASSERTION(NS_SUCCEEDED(rv), "Unable to register factory for component");
+  }
+
+  return rv;
+}
+			     
 /* static */
 void
 EmbedPrivate::EnsureOffscreenWindow(void)
