@@ -789,6 +789,13 @@ gtk_moz_embed_set_comp_path(char *aPath)
   EmbedPrivate::SetCompPath(aPath);
 }
 
+void
+gtk_moz_embed_set_app_components(const nsModuleComponentInfo* aComps,
+                                 int aNumComponents)
+{
+  EmbedPrivate::SetAppComponents(aComps, aNumComponents);
+}
+
 void
 gtk_moz_embed_set_profile_path(char *aDir, char *aName)
 {
