@@ -47,12 +47,13 @@
 #include "nsIURI.h"
 #include "nsIFile.h"
 
+#include "EmbedPrivate.h"
 #include "PtMozilla.h"
 
 #include <photon/PtWebClient.h>
 
 nsUnknownContentTypeHandler::nsUnknownContentTypeHandler( ) {
-	NS_INIT_REFCNT();
+	NS_INIT_ISUPPORTS();
 	}
 
 nsUnknownContentTypeHandler::~nsUnknownContentTypeHandler( ) { }
@@ -119,7 +120,7 @@ NS_IMETHODIMP nsUnknownContentTypeHandler::Show( nsIHelperAppLauncher *aLauncher
 	moz->EmbedRef->context = aContext;
 
 	strcpy( cb.content_type, mimeType );
-	strcpy( cb.url, url );
+	REMOVE_WHEN_NEW_PT_WEB_strcpy( cb.url, url );
 	cb.content_length = strlen( cb.url );
 	PtInvokeCallbackList( moz->web_unknown_cb, (PtWidget_t *)moz, &cbinfo);
 	return rv;
@@ -179,7 +180,7 @@ NS_IMETHODIMP nsWebProgressListener::OnProgressChange(nsIWebProgress *aProgress,
 
 	return NS_OK;
 	}
-NS_IMETHODIMP nsWebProgressListener::OnStateChange(nsIWebProgress* aWebProgress, nsIRequest *aRequest, PRInt32 progressStateFlags, nsresult aStatus) {
+NS_IMETHODIMP nsWebProgressListener::OnStateChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRUint32 aStateFlags, nsresult aStatus) {
 	return NS_OK;
 	}
 NS_IMETHODIMP nsWebProgressListener::OnLocationChange(nsIWebProgress* aWebProgress, nsIRequest* aRequest, nsIURI *location) {
