@@ -25,11 +25,6 @@
 #ifndef nsWebBrowserFindImpl_h__
 #define nsWebBrowserFindImpl_h__
 
-// We're going to stop using text services.
-// But the code remains here for a little while, for performance testing
-// and so that we can easily switch back and forth.
-#define TEXT_SVCS_TEST 1
-
 #include "nsIWebBrowserFind.h"
 
 #include "nsCOMPtr.h"
@@ -37,10 +32,6 @@
 
 #include "nsIFind.h"
 
-#ifdef TEXT_SVCS_TEST
-#include "nsIFindAndReplace.h"
-#endif /* TEXT_SVCS_TEST */
-
 #include "nsString.h"
 
 #define NS_WEB_BROWSER_FIND_CONTRACTID "@mozilla.org/embedcomp/find;1"
@@ -53,10 +44,6 @@ class nsIDOMWindow;
 
 class nsIDocShell;
 
-#ifdef TEXT_SVCS_TEST
-class nsITextServicesDocument;
-#endif /* TEXT_SVCS_TEST */
-
 //*****************************************************************************
 // class nsWebBrowserFind
 //*****************************************************************************   
@@ -85,7 +72,8 @@ protected:
 
     void        MoveFocusToCaret(nsIDOMWindow *aWindow);
 
-    nsresult    SearchInFrame(nsIDOMWindow* aWindow, PRBool* didFind);
+    nsresult    SearchInFrame(nsIDOMWindow* aWindow, PRBool aWrapping,
+                              PRBool* didFind);
 
     nsresult    OnStartSearchFrame(nsIDOMWindow *aWindow);
     nsresult    OnEndSearchFrame(nsIDOMWindow *aWindow);
@@ -94,10 +82,6 @@ protected:
     
     nsresult    OnFind(nsIDOMWindow *aFoundWindow);
     
-#ifdef TEXT_SVCS_TEST
-    nsresult    MakeTSDocument(nsIDOMWindow* aWindow, nsITextServicesDocument** aDoc);
-#endif /* TEXT_SVCS_TEST */
-
     nsresult    GetDocShellFromWindow(nsIDOMWindow *inWindow, nsIDocShell** outDocShell);
 
     void        SetSelectionAndScroll(nsIDOMRange* aRange,
@@ -130,9 +114,6 @@ protected:
     nsWeakPtr       mRootSearchFrame;       // who knows if windows can go away during our lifetime, hence weak
     nsWeakPtr       mLastFocusedWindow;     // who knows if windows can go away during our lifetime, hence weak
     
-#ifdef TEXT_SVCS_TEST
-    nsCOMPtr<nsIFindAndReplace> mTSFind;
-#endif /* TEXT_SVCS_TEST */
     nsCOMPtr<nsIFind> mFind;
 };
 
