@@ -610,8 +610,14 @@ nsWindowWatcher::OpenWindowJS(nsIDOMWindow *aParent,
         if (popupConditions)
           contextFlags |= nsIWindowCreator2::PARENT_IS_LOADING_OR_RUNNING_TIMEOUT;
 
+        PRBool cancel = PR_FALSE;
         rv = windowCreator2->CreateChromeWindow2(parentChrome, chromeFlags,
-                               contextFlags, getter_AddRefs(newChrome));
+                               contextFlags, uriToLoad, &cancel,
+                               getter_AddRefs(newChrome));
+        if (NS_SUCCEEDED(rv) && cancel) {
+          newChrome = 0; // just in case
+          rv = NS_ERROR_ABORT;
+        }
       }
       else
         rv = mWindowCreator->CreateChromeWindow(parentChrome, chromeFlags,
@@ -666,7 +672,12 @@ nsWindowWatcher::OpenWindowJS(nsIDOMWindow *aParent,
     }
   }
 
-  newDocShellItem->SetName(nameSpecified ? name.get() : nsnull);
+  /* allow an extant window to keep its name (important for cases like
+     _self where the given name is different (and invalid)). also _blank
+     is not a window name. */
+  if (windowIsNew)
+    newDocShellItem->SetName(nameSpecified && !name.EqualsIgnoreCase("_blank") ?
+                             name.get() : nsnull);
 
   nsCOMPtr<nsIDocShell> newDocShell(do_QueryInterface(newDocShellItem));
 
