@@ -46,14 +46,14 @@
 // to avoid cluttering this one and to demonstrate other embedding
 // principles. For example, nsIPrompt is implemented in a separate DLL.
 // 
-//	nsIWebBrowserChrome	- This is a required interface to be implemented
-//						  by embeddors
+//    nsIWebBrowserChrome    - This is a required interface to be implemented
+//                          by embeddors
 //
-//	nsIEmbeddingSiteWindow - This is a required interface to be implemented
-//						 by embedders			
-//					   - SetTitle() gets called after a document
-//						 load giving us the chance to update our
-//						 titlebar
+//    nsIEmbeddingSiteWindow - This is a required interface to be implemented
+//                         by embedders            
+//                       - SetTitle() gets called after a document
+//                         load giving us the chance to update our
+//                         titlebar
 //
 // Some points to note...
 //
@@ -64,8 +64,8 @@
 // to indicate progress when a page is being loaded
 //
 // Next suggested file(s) to look at : 
-//			Any of the BrowserImpl*.cpp files for other interface
-//			implementation details
+//            Any of the BrowserImpl*.cpp files for other interface
+//            implementation details
 //
 
 #ifdef _WINDOWS
@@ -77,8 +77,6 @@
 
 CBrowserImpl::CBrowserImpl()
 {
-  NS_INIT_ISUPPORTS();
-
   m_pBrowserFrameGlue = NULL;
   mWebBrowser = nsnull;
 }
@@ -94,11 +92,11 @@ CBrowserImpl::~CBrowserImpl()
 NS_METHOD CBrowserImpl::Init(PBROWSERFRAMEGLUE pBrowserFrameGlue, 
                              nsIWebBrowser* aWebBrowser)
 {
-	  m_pBrowserFrameGlue = pBrowserFrameGlue;
-	  
-	  SetWebBrowser(aWebBrowser);
+      m_pBrowserFrameGlue = pBrowserFrameGlue;
+      
+      SetWebBrowser(aWebBrowser);
 
-	  return NS_OK;
+      return NS_OK;
 }
 
 //*****************************************************************************
@@ -116,7 +114,8 @@ NS_INTERFACE_MAP_BEGIN(CBrowserImpl)
    NS_INTERFACE_MAP_ENTRY(nsIEmbeddingSiteWindow)
    NS_INTERFACE_MAP_ENTRY(nsIEmbeddingSiteWindow2)
    NS_INTERFACE_MAP_ENTRY(nsIWebProgressListener)
-   NS_INTERFACE_MAP_ENTRY(nsIContextMenuListener)
+   NS_INTERFACE_MAP_ENTRY(nsIContextMenuListener2)
+   NS_INTERFACE_MAP_ENTRY(nsITooltipListener)
    NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
 NS_INTERFACE_MAP_END
 
@@ -126,14 +125,14 @@ NS_INTERFACE_MAP_END
 
 NS_IMETHODIMP CBrowserImpl::GetInterface(const nsIID &aIID, void** aInstancePtr)
 {
-	if(aIID.Equals(NS_GET_IID(nsIDOMWindow)))
-	{
-		if (mWebBrowser)
-			return mWebBrowser->GetContentDOMWindow((nsIDOMWindow **) aInstancePtr);
-		return NS_ERROR_NOT_INITIALIZED;
-	}
+    if(aIID.Equals(NS_GET_IID(nsIDOMWindow)))
+    {
+        if (mWebBrowser)
+            return mWebBrowser->GetContentDOMWindow((nsIDOMWindow **) aInstancePtr);
+        return NS_ERROR_NOT_INITIALIZED;
+    }
 
-	return QueryInterface(aIID, aInstancePtr);
+    return QueryInterface(aIID, aInstancePtr);
 }
 
 //*****************************************************************************
@@ -144,12 +143,12 @@ NS_IMETHODIMP CBrowserImpl::GetInterface(const nsIID &aIID, void** aInstancePtr)
 //
 NS_IMETHODIMP CBrowserImpl::SetStatus(PRUint32 aType, const PRUnichar* aStatus)
 {
-	if(! m_pBrowserFrameGlue)
-		return NS_ERROR_FAILURE;
+    if(! m_pBrowserFrameGlue)
+        return NS_ERROR_FAILURE;
 
-	m_pBrowserFrameGlue->UpdateStatusBarText(aStatus);
+    m_pBrowserFrameGlue->UpdateStatusBarText(aStatus);
 
-	return NS_OK;
+    return NS_OK;
 }
 
 NS_IMETHODIMP CBrowserImpl::GetWebBrowser(nsIWebBrowser** aWebBrowser)
@@ -189,27 +188,27 @@ NS_IMETHODIMP CBrowserImpl::SetChromeFlags(PRUint32 aChromeMask)
 //
 NS_IMETHODIMP CBrowserImpl::DestroyBrowserWindow()
 {
-	if(! m_pBrowserFrameGlue)
-		return NS_ERROR_FAILURE;
+    if(! m_pBrowserFrameGlue)
+        return NS_ERROR_FAILURE;
 
-	m_pBrowserFrameGlue->DestroyBrowserFrame();
+    m_pBrowserFrameGlue->DestroyBrowserFrame();
 
-	return NS_OK;
+    return NS_OK;
 }
 
 // Gets called in response to set the size of a window
 // Ex: In response to a JavaScript Window.Open() call of
 // the form 
 //
-//		window.open("http://www.mozilla.org", "theWin", "width=200, height=400");
+//        window.open("http://www.mozilla.org", "theWin", "width=200, height=400");
 //
 // First the CreateBrowserWindow() call will be made followed by a 
 // call to this method to resize the window
 //
 NS_IMETHODIMP CBrowserImpl::SizeBrowserTo(PRInt32 aCX, PRInt32 aCY)
 {
-	if(! m_pBrowserFrameGlue)
-		return NS_ERROR_FAILURE;
+    if(! m_pBrowserFrameGlue)
+        return NS_ERROR_FAILURE;
 
     HWND w = m_pBrowserFrameGlue->GetBrowserFrameNativeWnd();
 
@@ -226,9 +225,9 @@ NS_IMETHODIMP CBrowserImpl::SizeBrowserTo(PRInt32 aCX, PRInt32 aCY)
     rcNewFrame.right += rcFrame.Width() - rcClient.Width();
     rcNewFrame.bottom += rcFrame.Height() - rcClient.Height();
 
-	m_pBrowserFrameGlue->SetBrowserFrameSize(rcNewFrame.Width(), rcNewFrame.Height());
+    m_pBrowserFrameGlue->SetBrowserFrameSize(rcNewFrame.Width(), rcNewFrame.Height());
 
-	return NS_OK;
+    return NS_OK;
 }
 
 NS_IMETHODIMP CBrowserImpl::ShowAsModal(void)
@@ -269,25 +268,25 @@ NS_IMETHODIMP CBrowserImpl::FocusPrevElement()
 
 NS_IMETHODIMP CBrowserImpl::SetDimensions(PRUint32 aFlags, PRInt32 x, PRInt32 y, PRInt32 cx, PRInt32 cy)
 {
-	if(! m_pBrowserFrameGlue)
-		return NS_ERROR_FAILURE;
+    if(! m_pBrowserFrameGlue)
+        return NS_ERROR_FAILURE;
 
     if (aFlags & nsIEmbeddingSiteWindow::DIM_FLAGS_POSITION &&
         (aFlags & nsIEmbeddingSiteWindow::DIM_FLAGS_SIZE_INNER || 
          aFlags & nsIEmbeddingSiteWindow::DIM_FLAGS_SIZE_OUTER))
     {
-    	m_pBrowserFrameGlue->SetBrowserFramePositionAndSize(x, y, cx, cy, PR_TRUE);
+        m_pBrowserFrameGlue->SetBrowserFramePositionAndSize(x, y, cx, cy, PR_TRUE);
     }
     else
     {
         if (aFlags & nsIEmbeddingSiteWindow::DIM_FLAGS_POSITION)
         {
-    	    m_pBrowserFrameGlue->SetBrowserFramePosition(x, y);
+            m_pBrowserFrameGlue->SetBrowserFramePosition(x, y);
         }
         if (aFlags & nsIEmbeddingSiteWindow::DIM_FLAGS_SIZE_INNER || 
             aFlags & nsIEmbeddingSiteWindow::DIM_FLAGS_SIZE_OUTER)
         {
-    	    m_pBrowserFrameGlue->SetBrowserFrameSize(cx, cy);
+            m_pBrowserFrameGlue->SetBrowserFrameSize(cx, cy);
         }
     }
 
@@ -296,17 +295,17 @@ NS_IMETHODIMP CBrowserImpl::SetDimensions(PRUint32 aFlags, PRInt32 x, PRInt32 y,
 
 NS_IMETHODIMP CBrowserImpl::GetDimensions(PRUint32 aFlags, PRInt32 *x, PRInt32 *y, PRInt32 *cx, PRInt32 *cy)
 {
-	if(! m_pBrowserFrameGlue)
-		return NS_ERROR_FAILURE;
+    if(! m_pBrowserFrameGlue)
+        return NS_ERROR_FAILURE;
     
     if (aFlags & nsIEmbeddingSiteWindow::DIM_FLAGS_POSITION)
     {
-    	m_pBrowserFrameGlue->GetBrowserFramePosition(x, y);
+        m_pBrowserFrameGlue->GetBrowserFramePosition(x, y);
     }
     if (aFlags & nsIEmbeddingSiteWindow::DIM_FLAGS_SIZE_INNER || 
         aFlags & nsIEmbeddingSiteWindow::DIM_FLAGS_SIZE_OUTER)
     {
-    	m_pBrowserFrameGlue->GetBrowserFrameSize(cx, cy);
+        m_pBrowserFrameGlue->GetBrowserFrameSize(cx, cy);
     }
 
     return NS_OK;
@@ -327,42 +326,42 @@ NS_IMETHODIMP CBrowserImpl::GetSiteWindow(void** aSiteWindow)
 
 NS_IMETHODIMP CBrowserImpl::SetFocus()
 {
-	if(! m_pBrowserFrameGlue)
-		return NS_ERROR_FAILURE;
+    if(! m_pBrowserFrameGlue)
+        return NS_ERROR_FAILURE;
 
-	m_pBrowserFrameGlue->SetFocus();
+    m_pBrowserFrameGlue->SetFocus();
 
-	return NS_OK;
+    return NS_OK;
 }
 
 NS_IMETHODIMP CBrowserImpl::GetTitle(PRUnichar** aTitle)
 {
-	if(! m_pBrowserFrameGlue)
-		return NS_ERROR_FAILURE;
+    if(! m_pBrowserFrameGlue)
+        return NS_ERROR_FAILURE;
 
-	m_pBrowserFrameGlue->GetBrowserFrameTitle(aTitle);
-	
-	return NS_OK;
+    m_pBrowserFrameGlue->GetBrowserFrameTitle(aTitle);
+    
+    return NS_OK;
 }
 
 NS_IMETHODIMP CBrowserImpl::SetTitle(const PRUnichar* aTitle)
 {
-	if(! m_pBrowserFrameGlue)
-		return NS_ERROR_FAILURE;
+    if(! m_pBrowserFrameGlue)
+        return NS_ERROR_FAILURE;
 
-	m_pBrowserFrameGlue->SetBrowserFrameTitle(aTitle);
-	
-	return NS_OK;
+    m_pBrowserFrameGlue->SetBrowserFrameTitle(aTitle);
+    
+    return NS_OK;
 }
 
 NS_IMETHODIMP CBrowserImpl::GetVisibility(PRBool *aVisibility)
 {
-	if(! m_pBrowserFrameGlue)
-		return NS_ERROR_FAILURE;
+    if(! m_pBrowserFrameGlue)
+        return NS_ERROR_FAILURE;
 
     m_pBrowserFrameGlue->GetBrowserFrameVisibility(aVisibility);
 
-	return NS_OK;
+    return NS_OK;
 }
 
 NS_IMETHODIMP CBrowserImpl::SetVisibility(PRBool aVisibility)
@@ -382,4 +381,31 @@ NS_IMETHODIMP CBrowserImpl::SetVisibility(PRBool aVisibility)
 NS_IMETHODIMP CBrowserImpl::Blur()
 {
     return NS_OK;
-}
\ No newline at end of file
+}
+
+
+//*****************************************************************************
+// CBrowserImpl::nsITooltipListener
+//*****************************************************************************   
+
+/* void onShowTooltip (in long aXCoords, in long aYCoords, in wstring aTipText); */
+NS_IMETHODIMP CBrowserImpl::OnShowTooltip(PRInt32 aXCoords, PRInt32 aYCoords, const PRUnichar *aTipText)
+{
+    if(! m_pBrowserFrameGlue)
+        return NS_ERROR_FAILURE;
+
+    m_pBrowserFrameGlue->ShowTooltip(aXCoords, aYCoords, aTipText);
+
+    return NS_OK;
+}
+
+/* void onHideTooltip (); */
+NS_IMETHODIMP CBrowserImpl::OnHideTooltip()
+{
+    if(! m_pBrowserFrameGlue)
+        return NS_ERROR_FAILURE;
+
+    m_pBrowserFrameGlue->HideTooltip();
+
+    return NS_OK;
+}
