@@ -1,51 +1,47 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: Mozilla-sample-code 1.0
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * Copyright (c) 2002 Netscape Communications Corporation and
+ * other contributors
  *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this Mozilla sample software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to permit
+ * persons to whom the Software is furnished to do so, subject to the
+ * following conditions:
  *
- * The Original Code is mozilla.org code.
+ * The above copyright notice and this permission notice shall be included
+ * in all copies or substantial portions of the Software.
  *
- * The Initial Developer of the Original Code is 
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  *
  * Contributor(s):
  *  Doug Turner <dougt@netscape.com> 
  *  Adam Lock <adamlock@netscape.com>
  *
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
- *
  * ***** END LICENSE BLOCK ***** */
 
-
 #include <stdio.h>
-#include "stdafx.h"
+
+// C RunTime Header Files
+#include <stdlib.h>
+#include <malloc.h>
+#include <memory.h>
+#include <tchar.h>
 
 // Win32 header files
-#include "windows.h"
-#include "commctrl.h"
-#include "commdlg.h"
+#include <windows.h>
+#include <commctrl.h>
+#include <commdlg.h>
 
 // Mozilla header files
 #include "nsEmbedAPI.h"
@@ -77,6 +73,14 @@
 
 #define MAX_LOADSTRING 100
 
+
+#ifdef _BUILD_STATIC_BIN
+#include "nsStaticComponent.h"
+nsresult PR_CALLBACK
+app_getModuleInfo(nsStaticModuleInfo **info, PRUint32 *count);
+#endif
+
+
 const TCHAR *szWindowClass = _T("WINEMBED");
 
 // Foward declarations of functions included in this code module:
@@ -125,7 +129,7 @@ class ProfileChangeObserver : public nsIObserver,
 
 {
 public:
-	ProfileChangeObserver();
+    ProfileChangeObserver();
 
     NS_DECL_ISUPPORTS
     NS_DECL_NSIOBSERVER
@@ -150,44 +154,49 @@ int main(int argc, char *argv[])
     
     // Sophisticated command-line parsing in action
     char *szFirstURL = "http://www.mozilla.org/projects/embedding";
-	char *szDefaultProfile = nsnull;
-	int argn;
-	for (argn = 1; argn < argc; argn++)
-	{
-		if (stricmp("-P", argv[argn]) == 0)
-		{
-			if (argn + 1 < argc)
-			{
-				szDefaultProfile = argv[++argn];
-			}
-		}
-		else
-		{
-	        szFirstURL = argv[argn];
-		}
+    char *szDefaultProfile = nsnull;
+    int argn;
+    for (argn = 1; argn < argc; argn++)
+    {
+        if (stricmp("-P", argv[argn]) == 0)
+        {
+            if (argn + 1 < argc)
+            {
+                szDefaultProfile = argv[++argn];
+            }
+        }
+        else
+        {
+            szFirstURL = argv[argn];
+        }
     }
-	strncpy(gFirstURL, szFirstURL, sizeof(gFirstURL) - 1);
+    strncpy(gFirstURL, szFirstURL, sizeof(gFirstURL) - 1);
 
     ghInstanceApp = GetModuleHandle(NULL);
     ghInstanceResources = GetModuleHandle(NULL);
 
-	// Initialize global strings
+    // Initialize global strings
     TCHAR szTitle[MAX_LOADSTRING];
-	LoadString(ghInstanceResources, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
-	MyRegisterClass(ghInstanceApp);
+    LoadString(ghInstanceResources, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
+    MyRegisterClass(ghInstanceApp);
+
+#ifdef _BUILD_STATIC_BIN
+    // Initialize XPCOM's module info table
+    NSGetStaticModuleInfo = app_getModuleInfo;
+#endif
 
     // Init Embedding APIs
     NS_InitEmbedding(nsnull, nsnull);
 
-	// Choose the new profile
-	if (!ChooseNewProfile(TRUE, szDefaultProfile))
+    // Choose the new profile
+    if (!ChooseNewProfile(TRUE, szDefaultProfile))
     {
         NS_TermEmbedding();
         return 1;
     }
     WPARAM rv;
     {    
-    	// Now register an observer to watch for profile changes
+        // Now register an observer to watch for profile changes
         nsCOMPtr<nsIObserverService> observerService(do_GetService("@mozilla.org/observer-service;1"));
 
         ProfileChangeObserver *observer = new ProfileChangeObserver;
@@ -201,7 +210,7 @@ int main(int argc, char *argv[])
         // Open the initial browser window
         OpenWebPage(gFirstURL);
 
-	    // Main message loop.
+        // Main message loop.
         // NOTE: We use a fake event and a timeout in order to process idle stuff for
         //       Mozilla every 1/10th of a second.
         PRBool runCondition = PR_TRUE;
@@ -224,7 +233,6 @@ NS_IMPL_THREADSAFE_ISUPPORTS2(ProfileChangeObserver, nsIObserver, nsISupportsWea
 
 ProfileChangeObserver::ProfileChangeObserver()
 {
-	NS_INIT_REFCNT();
 }
 
 // ---------------------------------------------------------------------------
@@ -237,7 +245,7 @@ NS_IMETHODIMP ProfileChangeObserver::Observe(nsISupports *aSubject, const char *
 
     if (nsCRT::strcmp(aTopic, "profile-approve-change") == 0)
     {
-		// The profile is about to change!
+        // The profile is about to change!
 
         // Ask the user if they want to
         int result = ::MessageBox(NULL, "Do you want to close all windows in order to switch the profile?", "Confirm", MB_YESNO | MB_ICONQUESTION);
@@ -250,15 +258,15 @@ NS_IMETHODIMP ProfileChangeObserver::Observe(nsISupports *aSubject, const char *
     }
     else if (nsCRT::strcmp(aTopic, "profile-change-teardown") == 0)
     {
-		// The profile is changing!
+        // The profile is changing!
 
-		// Prevent WM_QUIT by incrementing the dialog count
-		gDialogCount++;
+        // Prevent WM_QUIT by incrementing the dialog count
+        gDialogCount++;
     }
     else if (nsCRT::strcmp(aTopic, "profile-after-change") == 0)
     {
-		// Decrease the dialog count so WM_QUIT can once more happen
-		gDialogCount--;
+        // Decrease the dialog count so WM_QUIT can once more happen
+        gDialogCount--;
         if (gDialogCount == 0)
         {
             // All the dialogs have been torn down so open new page
@@ -266,7 +274,7 @@ NS_IMETHODIMP ProfileChangeObserver::Observe(nsISupports *aSubject, const char *
         }
         else
         {
-		    // The profile has changed, but dialogs are still being
+            // The profile has changed, but dialogs are still being
             // torn down. Set this flag so when the last one goes
             // it can finish the switch.
             gProfileSwitch = TRUE;
@@ -288,7 +296,7 @@ nsresult InitializeWindowCreator()
         nsCOMPtr<nsIWindowCreator> windowCreator(dont_QueryInterface(NS_STATIC_CAST(nsIWindowCreator *, creatorCallback)));
         if (windowCreator)
         {
-            nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
+            nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
             if (wwatch)
             {
                 wwatch->SetWindowCreator(windowCreator);
@@ -346,9 +354,9 @@ HWND GetBrowserFromChrome(nsIWebBrowserChrome *aChrome)
     {
         return NULL;
     }
-	nsCOMPtr<nsIEmbeddingSiteWindow> baseWindow = do_QueryInterface(aChrome);
+    nsCOMPtr<nsIEmbeddingSiteWindow> baseWindow = do_QueryInterface(aChrome);
     HWND hwnd = NULL;
-	baseWindow->GetSiteWindow((void **) & hwnd);
+    baseWindow->GetSiteWindow((void **) & hwnd);
     return hwnd;
 }
 
@@ -374,50 +382,50 @@ void SaveWebPage(nsIWebBrowser *aWebBrowser)
 {
     // Use the browser window title as the initial file name
     nsCOMPtr<nsIBaseWindow> webBrowserAsWin = do_QueryInterface(aWebBrowser);
-	nsXPIDLString windowTitle;
-	webBrowserAsWin->GetTitle(getter_Copies(windowTitle));
+    nsXPIDLString windowTitle;
+    webBrowserAsWin->GetTitle(getter_Copies(windowTitle));
     nsCString fileName; fileName.AssignWithConversion(windowTitle);
 
-	// Sanitize the title of all illegal characters
+    // Sanitize the title of all illegal characters
     fileName.CompressWhitespace();     // Remove whitespace from the ends
     fileName.StripChars("\\*|:\"><?"); // Strip illegal characters
     fileName.ReplaceChar('.', L'_');   // Dots become underscores
     fileName.ReplaceChar('/', L'-');   // Forward slashes become hyphens
 
     // Copy filename to a character buffer
-	char szFile[_MAX_PATH];
+    char szFile[_MAX_PATH];
     memset(szFile, 0, sizeof(szFile));
     PL_strncpyz(szFile, fileName.get(), sizeof(szFile) - 1); // XXXldb probably should be just sizeof(szfile)
 
     // Initialize the file save as information structure
     OPENFILENAME saveFileNameInfo;
     memset(&saveFileNameInfo, 0, sizeof(saveFileNameInfo));
-	saveFileNameInfo.lStructSize = sizeof(saveFileNameInfo);
-	saveFileNameInfo.hwndOwner = NULL;
-	saveFileNameInfo.hInstance = NULL;
-	saveFileNameInfo.lpstrFilter =
+    saveFileNameInfo.lStructSize = sizeof(saveFileNameInfo);
+    saveFileNameInfo.hwndOwner = NULL;
+    saveFileNameInfo.hInstance = NULL;
+    saveFileNameInfo.lpstrFilter =
         "Web Page, HTML Only (*.htm;*.html)\0*.htm;*.html\0"
         "Web Page, Complete (*.htm;*.html)\0*.htm;*.html\0"
         "Text File (*.txt)\0*.txt\0"; 
-	saveFileNameInfo.lpstrCustomFilter = NULL; 
-	saveFileNameInfo.nMaxCustFilter = NULL; 
-	saveFileNameInfo.nFilterIndex = 1; 
-	saveFileNameInfo.lpstrFile = szFile; 
-	saveFileNameInfo.nMaxFile = sizeof(szFile); 
-	saveFileNameInfo.lpstrFileTitle = NULL;
-	saveFileNameInfo.nMaxFileTitle = 0; 
-	saveFileNameInfo.lpstrInitialDir = NULL; 
-	saveFileNameInfo.lpstrTitle = NULL; 
-	saveFileNameInfo.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT; 
-	saveFileNameInfo.nFileOffset = NULL; 
-	saveFileNameInfo.nFileExtension = NULL; 
-	saveFileNameInfo.lpstrDefExt = "htm"; 
-	saveFileNameInfo.lCustData = NULL; 
-	saveFileNameInfo.lpfnHook = NULL; 
-	saveFileNameInfo.lpTemplateName = NULL; 
+    saveFileNameInfo.lpstrCustomFilter = NULL; 
+    saveFileNameInfo.nMaxCustFilter = NULL; 
+    saveFileNameInfo.nFilterIndex = 1; 
+    saveFileNameInfo.lpstrFile = szFile; 
+    saveFileNameInfo.nMaxFile = sizeof(szFile); 
+    saveFileNameInfo.lpstrFileTitle = NULL;
+    saveFileNameInfo.nMaxFileTitle = 0; 
+    saveFileNameInfo.lpstrInitialDir = NULL; 
+    saveFileNameInfo.lpstrTitle = NULL; 
+    saveFileNameInfo.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT; 
+    saveFileNameInfo.nFileOffset = NULL; 
+    saveFileNameInfo.nFileExtension = NULL; 
+    saveFileNameInfo.lpstrDefExt = "htm"; 
+    saveFileNameInfo.lCustData = NULL; 
+    saveFileNameInfo.lpfnHook = NULL; 
+    saveFileNameInfo.lpTemplateName = NULL; 
 
-	if (GetSaveFileName(&saveFileNameInfo))
-	{
+    if (GetSaveFileName(&saveFileNameInfo))
+    {
         // Does the user want to save the complete document including
         // all frames, images, scripts, stylesheets etc. ?
         char *pszDataPath = NULL;
@@ -466,7 +474,7 @@ nsresult ResizeEmbedding(nsIWebBrowserChrome* chrome)
     
     nsCOMPtr<nsIEmbeddingSiteWindow> embeddingSite = do_QueryInterface(chrome);
     HWND hWnd;
-	embeddingSite->GetSiteWindow((void **) & hWnd);
+    embeddingSite->GetSiteWindow((void **) & hWnd);
     
     if (!hWnd)
         return NS_ERROR_NULL_POINTER;
@@ -474,19 +482,19 @@ nsresult ResizeEmbedding(nsIWebBrowserChrome* chrome)
     RECT rect;
     GetClientRect(hWnd, &rect);
     
-	// Make sure the browser is visible and sized
-	nsCOMPtr<nsIWebBrowser> webBrowser;
-	chrome->GetWebBrowser(getter_AddRefs(webBrowser));
+    // Make sure the browser is visible and sized
+    nsCOMPtr<nsIWebBrowser> webBrowser;
+    chrome->GetWebBrowser(getter_AddRefs(webBrowser));
     nsCOMPtr<nsIBaseWindow> webBrowserAsWin = do_QueryInterface(webBrowser);
-	if (webBrowserAsWin)
-	{
+    if (webBrowserAsWin)
+    {
         webBrowserAsWin->SetPositionAndSize(rect.left, 
                                    rect.top, 
                                    rect.right - rect.left, 
                                    rect.bottom - rect.top,
                                    PR_TRUE);
-		webBrowserAsWin->SetVisibility(PR_TRUE);
-	}
+        webBrowserAsWin->SetVisibility(PR_TRUE);
+    }
 
     return NS_OK;
 }
@@ -507,23 +515,23 @@ nsresult ResizeEmbedding(nsIWebBrowserChrome* chrome)
 //
 ATOM MyRegisterClass(HINSTANCE hInstance)
 {
-	WNDCLASSEX wcex;
+    WNDCLASSEX wcex;
 
     memset(&wcex, 0, sizeof(wcex));
-	wcex.cbSize = sizeof(WNDCLASSEX); 
+    wcex.cbSize = sizeof(WNDCLASSEX); 
 
-	wcex.style			= CS_HREDRAW | CS_VREDRAW;
-	wcex.lpfnWndProc	= (WNDPROC) BrowserWndProc;
-	wcex.cbClsExtra		= 0;
-	wcex.cbWndExtra		= 0;
-	wcex.hInstance		= hInstance;
-	wcex.hIcon			= LoadIcon(ghInstanceResources, (LPCTSTR)IDI_WINEMBED);
-	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
-	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
-	wcex.lpszClassName	= szWindowClass;
-	wcex.hIconSm		= LoadIcon(ghInstanceResources, (LPCTSTR)IDI_SMALL);
+    wcex.style            = CS_HREDRAW | CS_VREDRAW;
+    wcex.lpfnWndProc    = (WNDPROC) BrowserWndProc;
+    wcex.cbClsExtra        = 0;
+    wcex.cbWndExtra        = 0;
+    wcex.hInstance        = hInstance;
+    wcex.hIcon            = LoadIcon(ghInstanceResources, (LPCTSTR)IDI_WINEMBED);
+    wcex.hCursor        = LoadCursor(NULL, IDC_ARROW);
+    wcex.hbrBackground    = (HBRUSH)(COLOR_WINDOW+1);
+    wcex.lpszClassName    = szWindowClass;
+    wcex.hIconSm        = LoadIcon(ghInstanceResources, (LPCTSTR)IDI_SMALL);
 
-	return RegisterClassEx(&wcex);
+    return RegisterClassEx(&wcex);
 }
 
 
@@ -597,11 +605,11 @@ void UpdateUI(nsIWebBrowserChrome *aChrome)
 //
 BOOL CALLBACK BrowserDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
-	if (uMsg == WM_COMMAND && LOWORD(wParam) == MOZ_SwitchProfile)
-	{
+    if (uMsg == WM_COMMAND && LOWORD(wParam) == MOZ_SwitchProfile)
+    {
         ChooseNewProfile(FALSE, NULL);
-		return FALSE;
-	}
+        return FALSE;
+    }
 
     // Get the browser and other pointers since they are used a lot below
     HWND hwndBrowser = GetDlgItem(hwndDlg, IDC_BROWSER);
@@ -639,7 +647,7 @@ BOOL CALLBACK BrowserDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
         break;
 
     case WM_DESTROY:
-	    return TRUE;
+        return TRUE;
 
     case WM_COMMAND:
         if (!webBrowser)
@@ -649,7 +657,7 @@ BOOL CALLBACK BrowserDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
 
         // Test which command was selected
         switch (LOWORD(wParam))
-		{
+        {
         case IDC_ADDRESS:
             if (HIWORD(wParam) == CBN_EDITCHANGE || HIWORD(wParam) == CBN_SELCHANGE)
             {
@@ -776,9 +784,9 @@ BOOL CALLBACK BrowserDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
                 MessageBox(NULL, szAbout, szAboutTitle, MB_OK);
             }
             break;
-		}
+        }
 
-	    return TRUE;
+        return TRUE;
 
     case WM_ACTIVATE:
         {
@@ -877,8 +885,8 @@ BOOL CALLBACK BrowserDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
 LRESULT CALLBACK BrowserWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
     nsIWebBrowserChrome *chrome = (nsIWebBrowserChrome *) GetWindowLong(hWnd, GWL_USERDATA);
-	switch (message) 
-	{
+    switch (message) 
+    {
     case WM_SIZE:
         // Resize the embedded browser
         ResizeEmbedding(chrome);
@@ -964,9 +972,9 @@ BOOL ChooseNewProfile(BOOL bShowForMultipleProfilesOnly, const char *szDefaultPr
 LRESULT CALLBACK ChooseProfileDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
 {
     nsresult rv;
-	switch (message)
-	{
-	case WM_INITDIALOG:
+    switch (message)
+    {
+    case WM_INITDIALOG:
         {
             HWND hwndProfileList = GetDlgItem(hDlg, IDC_PROFILELIST);
 
@@ -1007,11 +1015,11 @@ LRESULT CALLBACK ChooseProfileDlgProc(HWND hDlg, UINT message, WPARAM wParam, LP
                 SendMessage(hwndProfileList, LB_SETCURSEL, currentProfileIndex, 0);
             }
         }
-		return TRUE;
+        return TRUE;
 
-	case WM_COMMAND:
-		if (LOWORD(wParam) == IDOK ||
-			(HIWORD(wParam) & LBN_DBLCLK && LOWORD(wParam) == IDC_PROFILELIST))
+    case WM_COMMAND:
+        if (LOWORD(wParam) == IDOK ||
+            (HIWORD(wParam) & LBN_DBLCLK && LOWORD(wParam) == IDC_PROFILELIST))
         {
             HWND hwndProfileList = GetDlgItem(hDlg, IDC_PROFILELIST);
 
@@ -1028,14 +1036,14 @@ LRESULT CALLBACK ChooseProfileDlgProc(HWND hDlg, UINT message, WPARAM wParam, LP
                 nsAutoString newProfile; newProfile.AssignWithConversion(profileName);
                 rv = profileService->SetCurrentProfile(newProfile.get());
             }
-	        EndDialog(hDlg, IDOK);
+            EndDialog(hDlg, IDOK);
+        }
+        else if (LOWORD(wParam) == IDCANCEL)
+        {
+            EndDialog(hDlg, LOWORD(wParam));
         }
-		else if (LOWORD(wParam) == IDCANCEL)
-		{
-	        EndDialog(hDlg, LOWORD(wParam));
-		}
         return TRUE;
-	}
+    }
 
     return FALSE;
 }
@@ -1148,15 +1156,15 @@ void WebBrowserChromeUI::Destroy(nsIWebBrowserChrome* chrome)
 void WebBrowserChromeUI::Destroyed(nsIWebBrowserChrome* chrome)
 {
     HWND hwndDlg = GetBrowserDlgFromChrome(chrome);
-	if (hwndDlg == NULL)
-	{
-		return;
-	}
+    if (hwndDlg == NULL)
+    {
+        return;
+    }
 
     // Clear the window user data
     HWND hwndBrowser = GetDlgItem(hwndDlg, IDC_BROWSER);
     SetWindowLong(hwndBrowser, GWL_USERDATA, nsnull);
-	DestroyWindow(hwndBrowser);
+    DestroyWindow(hwndBrowser);
     DestroyWindow(hwndDlg);
 
     --gDialogCount;
@@ -1182,10 +1190,10 @@ void WebBrowserChromeUI::Destroyed(nsIWebBrowserChrome* chrome)
 void WebBrowserChromeUI::SetFocus(nsIWebBrowserChrome *chrome)
 {
     HWND hwndDlg = GetBrowserDlgFromChrome(chrome);
-	if (hwndDlg == NULL)
-	{
-		return;
-	}
+    if (hwndDlg == NULL)
+    {
+        return;
+    }
     
     HWND hwndBrowser = GetDlgItem(hwndDlg, IDC_BROWSER);
     ::SetFocus(hwndBrowser);
@@ -1311,11 +1319,21 @@ void WebBrowserChromeUI::ShowWindow(nsIWebBrowserChrome *aChrome, PRBool aShow)
 
 void WebBrowserChromeUI::SizeTo(nsIWebBrowserChrome *aChrome, PRInt32 aWidth, PRInt32 aHeight)
 {
-  HWND win = GetBrowserDlgFromChrome(aChrome);
-  RECT winRect;
+  HWND hchrome = GetBrowserDlgFromChrome(aChrome);
+  HWND hbrowser = GetBrowserFromChrome(aChrome);
+  RECT chromeRect, browserRect;
 
-  ::GetWindowRect(win, &winRect);
-  ::MoveWindow(win, winRect.left, winRect.top, aWidth, aHeight, TRUE);
+  ::GetWindowRect(hchrome,  &chromeRect);
+  ::GetWindowRect(hbrowser, &browserRect);
+
+  PRInt32 decoration_x = (browserRect.left - chromeRect.left) + 
+                         (chromeRect.right - browserRect.right);
+  PRInt32 decoration_y = (browserRect.top - chromeRect.top) + 
+                         (chromeRect.bottom - browserRect.bottom);
+
+  ::MoveWindow(hchrome, chromeRect.left, chromeRect.top,
+      aWidth+decoration_x,
+      aHeight+decoration_y, TRUE);
 }
 
 //
