@@ -1,31 +1,46 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/*
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
+ * Contributor(s):
  *   Pierre Phaneuf <pp@ludusdesign.com>
- */
+ *
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
-#define NS_IMPL_IDS
 #include "pratom.h"
 #include "nsIFactory.h"
 #include "nsIServiceManager.h"
-#include "nsRepository.h"
+#include "nsIComponentManager.h"
 //#include "nsIObserver.h"
 #include "nsIURL.h"
 #include "nsIIOService.h"
@@ -34,6 +49,7 @@
 static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
 #include "nsPICSElementObserver.h"
 #include "nsString.h"
+#include "nsReadableUtils.h"
 #include "nsIPICS.h"
 #include "nspics.h"
 #include "nsIWebShellServices.h"
@@ -96,7 +112,6 @@ NS_PICS nsresult NS_NewPICSElementObserver(nsIObserver** anObserver)
 
 nsPICSElementObserver::nsPICSElementObserver()
 {
-    NS_INIT_REFCNT();
 }
 
 nsPICSElementObserver::~nsPICSElementObserver(void)
@@ -138,19 +153,19 @@ NS_IMETHODIMP nsPICSElementObserver::Notify(PRUint32 aDocumentID,
   int status;
   nsIWebShellServices* ws;
 //  nsString theURL(aSpec);
-// char* url = aSpec.ToNewCString();
+// char* url = ToNewCString(aSpec);
   nsIURI* uaURL = nsnull;
 //  rv = NS_NewURL(&uaURL, nsString(aSpec));
  
     if(numOfAttributes >= 2) {
       const nsString& theValue1=valueArray[0];
-      char *val1 = theValue1.ToNewCString();
+      char *val1 = ToNewCString(theValue1);
       if(theValue1.EqualsIgnoreCase("\"PICS-LABEL\"")) {
 #ifdef DEBUG
         printf("\nReceived notification for a PICS-LABEl\n");
 #endif
         const nsString& theValue2=valueArray[1];
-        char *label = theValue2.ToNewCString();
+        char *label = ToNewCString(theValue2);
         if (valueArray[numOfAttributes]) {
           const nsString& theURLValue=valueArray[numOfAttributes];
           nsCOMPtr<nsIIOService> service(do_GetService(kIOServiceCID, &rv));
@@ -182,7 +197,7 @@ NS_IMETHODIMP nsPICSElementObserver::Notify(PRUint32 aDocumentID,
                 char * text = PR_GetEnv("NGLAYOUT_HOME");
                 nsString mtemplateURL(text ? text : "resource:/res/samples/picstest1.html");
                 //  ws->LoadURL(mtemplateURL, nsnull, nsnull);
-                nsCharsetSource s;
+                PRInt32 s;
                 ws->SetRendering(PR_TRUE);
                 ws->StopDocumentLoad();
                 ws->LoadDocument("resource:/res/samples/picstest1.html", nsnull, s);
@@ -205,12 +220,10 @@ NS_IMETHODIMP nsPICSElementObserver::Observe(nsISupports*, const PRUnichar*, con
 ////////////////////////////////////////////////////////////////////////////////
 // nsPICSElementObserverFactory Implementation
 
-static NS_DEFINE_IID(kIFactoryIID, NS_IFACTORY_IID);
-NS_IMPL_ISUPPORTS(nsPICSElementObserverFactory, kIFactoryIID);
+NS_IMPL_ISUPPORTS1(nsPICSElementObserverFactory, nsIFactory)
 
 nsPICSElementObserverFactory::nsPICSElementObserverFactory(void)
 {
-    NS_INIT_REFCNT();
 }
 
 nsPICSElementObserverFactory::~nsPICSElementObserverFactory(void)
