@@ -13,7 +13,7 @@
  * Portions created by ActiveState Tool Corp. are Copyright (C) 2000, 2001
  * ActiveState Tool Corp.  All Rights Reserved.
  *
- * Contributor(s): Mark Hammond <MarkH@ActiveState.com> (original author)
+ * Contributor(s): Mark Hammond <mhammond@skippinet.com.au> (original author)
  *
  */
 
@@ -28,14 +28,17 @@
 // (c) 2000, ActiveState corp.
 
 #include "PyXPCOM_std.h"
-#include "nsIFileSpec.h"
-#include "nsSpecialSystemDirectory.h"
 #include "nsIThread.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIModule.h"
+#include "nsIFile.h"
+#include "nsILocalFile.h"
 
 #ifdef XP_WIN
+#ifndef WIN32_LEAN_AND_MEAN
 #define WIN32_LEAN_AND_MEAN
+#endif
 #include "windows.h"
 #endif
 
@@ -56,14 +59,14 @@ extern void PyXPCOM_InterpreterState_Ensure();
 
 #endif // XP_WIN
 
-PyXPCOM_INTERFACE_DEFINE(Py_nsIComponentManager, nsIComponentManager, PyMethods_IComponentManager)
+PyXPCOM_INTERFACE_DEFINE(Py_nsIComponentManager, nsIComponentManagerObsolete, PyMethods_IComponentManager)
 PyXPCOM_INTERFACE_DEFINE(Py_nsIInterfaceInfoManager, nsIInterfaceInfoManager, PyMethods_IInterfaceInfoManager)
 PyXPCOM_INTERFACE_DEFINE(Py_nsIEnumerator, nsIEnumerator, PyMethods_IEnumerator)
 PyXPCOM_INTERFACE_DEFINE(Py_nsISimpleEnumerator, nsISimpleEnumerator, PyMethods_ISimpleEnumerator)
 PyXPCOM_INTERFACE_DEFINE(Py_nsIInterfaceInfo, nsIInterfaceInfo, PyMethods_IInterfaceInfo)
-PyXPCOM_INTERFACE_DEFINE(Py_nsIServiceManager, nsIServiceManager, PyMethods_IServiceManager)
 PyXPCOM_INTERFACE_DEFINE(Py_nsIInputStream, nsIInputStream, PyMethods_IInputStream)
 PyXPCOM_INTERFACE_DEFINE(Py_nsIClassInfo, nsIClassInfo, PyMethods_IClassInfo)
+PyXPCOM_INTERFACE_DEFINE(Py_nsIVariant, nsIVariant, PyMethods_IVariant)
 
 ////////////////////////////////////////////////////////////
 // This is the main entry point called by the Python component
@@ -124,34 +127,6 @@ done:
 // "boot-strap" methods - interfaces we need to get the base
 // interface support!
 
-static PyObject *
-PyXPCOMMethod_NS_LocateSpecialSystemDirectory(PyObject *self, PyObject *args)
-{
-	int typ;
-	if (!PyArg_ParseTuple(args, "i", &typ))
-		return NULL;
-	nsSpecialSystemDirectory systemDir((nsSpecialSystemDirectory::SystemDirectories)typ);
-	return PyString_FromString(systemDir.GetNativePathCString());
-}
-
-static PyObject *
-PyXPCOMMethod_NS_NewFileSpec(PyObject *self, PyObject *args)
-{
-	char *szspec = NULL;
-	if (!PyArg_ParseTuple(args, "|s", &szspec))
-		return NULL;
-	nsIFileSpec *spec = NULL;
-	nsresult nr;
-	Py_BEGIN_ALLOW_THREADS;
-	nr = NS_NewFileSpec(&spec);
-	if (NS_SUCCEEDED(nr) && spec && szspec)
-		nr = spec->SetNativePath(szspec);
-	Py_END_ALLOW_THREADS;
-	if (NS_FAILED(nr) || spec==nsnull)
-		return PyXPCOM_BuildPyException(nr);
-	return Py_nsISupports::PyObjectFromInterface(spec, NS_GET_IID(nsIFileSpec), PR_TRUE);
-}
-
 static PyObject *
 PyXPCOMMethod_NS_GetGlobalComponentManager(PyObject *self, PyObject *args)
 {
@@ -171,7 +146,7 @@ PyXPCOMMethod_NS_GetGlobalComponentManager(PyObject *self, PyObject *args)
 	// Return a type based on the IID
 	// Can not auto-wrap the interface info manager as it is critical to
 	// building the support we need for autowrap.
-	return Py_nsISupports::PyObjectFromInterface(cm, NS_GET_IID(nsIComponentManager), PR_TRUE, PR_FALSE);
+	return Py_nsISupports::PyObjectFromInterface(cm, NS_GET_IID(nsIComponentManagerObsolete), PR_TRUE, PR_FALSE);
 }
 
 static PyObject *
@@ -182,18 +157,13 @@ PyXPCOMMethod_GetGlobalServiceManager(PyObject *self, PyObject *args)
 	nsIServiceManager* sm;
 	nsresult rv;
 	Py_BEGIN_ALLOW_THREADS;
-	rv = nsServiceManager::GetGlobalServiceManager(&sm);
+	rv = NS_GetServiceManager(&sm);
 	Py_END_ALLOW_THREADS;
 	if ( NS_FAILED(rv) )
 		return PyXPCOM_BuildPyException(rv);
-	// NOTE - GetGlobalServiceManager DOES NOT ADD A REFCOUNT
-	// (naughty, naughty) - we we explicitly ask our converter to
-	// add one, even tho this is not the common pattern.
 
-	// Return a type based on the IID
-	// Can not auto-wrap the interface info manager as it is critical to
-	// building the support we need for autowrap.
-	return Py_nsISupports::PyObjectFromInterface(sm, NS_GET_IID(nsIServiceManager), PR_TRUE, PR_FALSE);
+	// Return a type based on the IID.
+	return Py_nsISupports::PyObjectFromInterface(sm, NS_GET_IID(nsIServiceManager), PR_FALSE);
 }
 
 
@@ -283,6 +253,34 @@ PyXPCOMMethod_WrapObject(PyObject *self, PyObject *args)
 	return Py_nsISupports::PyObjectFromInterface(ret, iid, PR_FALSE, bWrapClient);
 }
 
+static PyObject *
+PyXPCOMMethod_UnwrapObject(PyObject *self, PyObject *args)
+{
+	PyObject *ob;
+	if (!PyArg_ParseTuple(args, "O", &ob))
+		return NULL;
+
+	nsISupports *uob = NULL;
+	nsIInternalPython *iob = NULL;
+	PyObject *ret = NULL;
+	if (!Py_nsISupports::InterfaceFromPyObject(ob, 
+				NS_GET_IID(nsISupports), 
+				&uob, 
+				PR_FALSE))
+		goto done;
+	if (NS_FAILED(uob->QueryInterface(NS_GET_IID(nsIInternalPython), reinterpret_cast<void **>(&iob)))) {
+		PyErr_SetString(PyExc_ValueError, "This XPCOM object is not implemented by Python");
+		goto done;
+	}
+	ret = iob->UnwrapPythonObject();
+done:
+	Py_BEGIN_ALLOW_THREADS;
+	NS_IF_RELEASE(uob);
+	NS_IF_RELEASE(iob);
+	Py_END_ALLOW_THREADS;
+	return ret;
+}
+
 // @pymethod int|pythoncom|_GetInterfaceCount|Retrieves the number of interface objects currently in existance
 static PyObject *
 PyXPCOMMethod_GetInterfaceCount(PyObject *self, PyObject *args)
@@ -410,9 +408,7 @@ extern PyObject *PyXPCOMMethod_IID(PyObject *self, PyObject *args);
 
 static struct PyMethodDef xpcom_methods[]=
 {
-	{"NS_LocateSpecialSystemDirectory", PyXPCOMMethod_NS_LocateSpecialSystemDirectory, 1},
 	{"NS_GetGlobalComponentManager", PyXPCOMMethod_NS_GetGlobalComponentManager, 1},
-	{"NS_NewFileSpec", PyXPCOMMethod_NS_NewFileSpec, 1},
 	{"XPTI_GetInterfaceInfoManager", PyXPCOMMethod_XPTI_GetInterfaceInfoManager, 1},
 	{"XPTC_InvokeByIndex", PyXPCOMMethod_XPTC_InvokeByIndex, 1},
 	{"GetGlobalServiceManager", PyXPCOMMethod_GetGlobalServiceManager, 1},
@@ -420,6 +416,7 @@ static struct PyMethodDef xpcom_methods[]=
 	{"ID", PyXPCOMMethod_IID, 1}, // This is the official name.
 	{"NS_ShutdownXPCOM", PyXPCOMMethod_NS_ShutdownXPCOM, 1},
 	{"WrapObject", PyXPCOMMethod_WrapObject, 1},
+	{"UnwrapObject", PyXPCOMMethod_UnwrapObject, 1},
 	{"_GetInterfaceCount", PyXPCOMMethod_GetInterfaceCount, 1},
 	{"_GetGatewayCount", PyXPCOMMethod_GetGatewayCount, 1},
 	{"getProxyForObject", PyXPCOMMethod_GetProxyForObject, 1},
@@ -463,7 +460,7 @@ PRBool PyXPCOM_Globals_Ensure()
 			// not already initialized.
 
 			// We need to locate the Mozilla bin directory.
-#ifdef XP_WIN			
+#ifdef XP_WIN
 			// On Windows this by using "xpcom.dll"
 			
 			char landmark[MAX_PATH+1];
@@ -479,21 +476,17 @@ PRBool PyXPCOM_Globals_Ensure()
 			if (end > landmark) *end = '\0';
 
 			nsCOMPtr<nsILocalFile> ns_bin_dir;
-			NS_NewLocalFile(landmark, PR_FALSE, getter_AddRefs(ns_bin_dir));
-			nsresult rv = NS_InitXPCOM(nsnull, ns_bin_dir);
+            NS_ConvertASCIItoUCS2 strLandmark(landmark);
+			NS_NewLocalFile(strLandmark, PR_FALSE, getter_AddRefs(ns_bin_dir));
+			nsresult rv = NS_InitXPCOM2(nsnull, ns_bin_dir, nsnull);
 #else
 			// Elsewhere, Mozilla can find it itself (we hope!)
-			nsresult rv = NS_InitXPCOM(nsnull, nsnull);
-#endif // XP_WIN			
+			nsresult rv = NS_InitXPCOM2(nsnull, nsnull, nsnull);
+#endif // XP_WIN
 			if (NS_FAILED(rv)) {
 				PyErr_SetString(PyExc_RuntimeError, "The XPCOM subsystem could not be initialized");
 				return PR_FALSE;
 			}
-			// Also set the "special directory"
-#ifdef XP_WIN			
-			nsFileSpec spec(landmark);
-			nsSpecialSystemDirectory::Set(nsSpecialSystemDirectory::OS_CurrentProcessDirectory, &spec);
-#endif // XP_WIN			
 		}
 		// Even if xpcom was already init, we want to flag it as init!
 		bHaveInitXPCOM = PR_TRUE;
@@ -554,12 +547,13 @@ init_xpcom() {
 	Py_XDECREF(obFuncPtr);
 
 	REGISTER_IID(nsISupports);
-	REGISTER_IID(nsISupportsString);
+	REGISTER_IID(nsISupportsCString);
 	REGISTER_IID(nsIModule);
 	REGISTER_IID(nsIFactory);
 	REGISTER_IID(nsIWeakReference);
 	REGISTER_IID(nsISupportsWeakReference);
 	REGISTER_IID(nsIClassInfo);
+	REGISTER_IID(nsIServiceManager);
 	// Register our custom interfaces.
 
 	Py_nsISupports::InitType();
@@ -568,10 +562,17 @@ init_xpcom() {
 	Py_nsIEnumerator::InitType(dict);
 	Py_nsISimpleEnumerator::InitType(dict);
 	Py_nsIInterfaceInfo::InitType(dict);
-	Py_nsIServiceManager::InitType(dict);
 	Py_nsIInputStream::InitType(dict);
 	Py_nsIClassInfo::InitType(dict);
-    
+	Py_nsIVariant::InitType(dict);
+
+	// yet another 
+	{ // temp scope nsIComponentManagerObsolete hack :(
+	PyObject *iid_ob = Py_nsIID::PyObjectFromIID(NS_GET_IID(nsIComponentManagerObsolete));
+	PyDict_SetItemString(dict, "IID_nsIComponentManager", iid_ob);
+	Py_DECREF(iid_ob);
+	} // end temp scope
+
     // We have special support for proxies - may as well add their constants!
     REGISTER_INT(PROXY_SYNC);
     REGISTER_INT(PROXY_ASYNC);
