@@ -51,4 +51,34 @@
 #define NS_ERROR_XPATH_INVALID_ARG         NS_ERROR_INVALID_ARG
 #define NS_ERROR_XSLT_INVALID_URL          NS_ERROR_INVALID_ARG
 
+#define NS_ERROR_XSLT_PARSE_FAILURE                    \
+    NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_XSLT, 1)
+
+#define NS_ERROR_XPATH_PARSE_FAILURE                   \
+    NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_XSLT, 2)
+
+#define NS_ERROR_XSLT_GET_NEW_HANDLER                  \
+    NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_XSLT, 3)
+
+#define NS_ERROR_XSLT_ALREADY_SET                      \
+    NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_XSLT, 4)
+
+#define NS_ERROR_XSLT_EXECUTION_FAILURE                \
+    NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_XSLT, 5)
+
+#define NS_ERROR_XPATH_UNKNOWN_FUNCTION                \
+    NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_XSLT, 6)
+
+#define NS_ERROR_XSLT_BAD_RECURSION                    \
+    NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_XSLT, 7)
+
+#define NS_ERROR_XSLT_BAD_VALUE                        \
+    NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_XSLT, 8)
+
+#define NS_ERROR_XSLT_NODESET_EXPECTED                 \
+    NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_XSLT, 9)
+
+#define NS_ERROR_XSLT_ABORTED                          \
+    NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_XSLT, 10)
+
 #endif // __TX_ERROR
