@@ -26,9 +26,11 @@
 #ifndef TRANSFRMX_XMLDOMUTILS_H
 #define TRANSFRMX_XMLDOMUTILS_H
 
-#include "dom.h"
 #include "baseutils.h"
 
+class Node;
+class nsAString;
+
 /*
  * A utility class for use with XML DOM implementations
  */
@@ -38,16 +40,7 @@ public:
     /*
      *  Appends the value of the given node to aResult.
      */
-    static void getNodeValue(Node* aNode, String& aResult);
-
-    /*
-     * Looks up the namespace URI associated with the given prefix.
-     *
-     * @return true if the namespace URI was found
-     */
-    static MBool getNamespaceURI(const String& aPrefix,
-                                 Element* aElement,
-                                 String& aResult);
+    static void getNodeValue(Node* aNode, nsAString& aResult);
 };
 
 #endif
