@@ -62,6 +62,15 @@ void BooleanResult::stringValue(nsAString& str)  {
     else str.Append(NS_LITERAL_STRING("false"));
 } //-- toString
 
+nsAString*
+BooleanResult::stringValuePointer()
+{
+    // In theory we could set strings containing "true" and "false" somewhere,
+    // but most stylesheets never get the stringvalue of a bool so that won't
+    // really buy us anything.
+    return nsnull;
+}
+
 MBool BooleanResult::booleanValue() {
    return this->value;
 } //-- toBoolean
