@@ -41,6 +41,7 @@
 #include "nsString.h"
 #include "nsIAtom.h"
 #include "TxObject.h"
+#include "nsAutoPtr.h"
 
 /*
   XPath class definitions.
@@ -343,18 +344,13 @@ public:
      * @param nodeExpr the NodeExpr to use when matching Nodes
      * @param axisIdentifier the Axis Identifier in which to search for nodes
     **/
-    LocationStep(txNodeTest* aNodeTest, LocationStepType aAxisIdentifier);
-
-    /**
-     * Destructor, will delete all predicates and the given NodeExpr
-    **/
-    virtual ~LocationStep();
+    LocationStep(nsAutoPtr<txNodeTest> aNodeTest, LocationStepType aAxisIdentifier);
 
     TX_DECL_EXPR;
 
 private:
 
-    txNodeTest* mNodeTest;
+    nsAutoPtr<txNodeTest> mNodeTest;
     LocationStepType mAxisIdentifier;
 
     void fromDescendants(Node* node, txIMatchContext* aContext,
@@ -521,11 +517,8 @@ private:
 class RelationalExpr : public Expr {
 
 public:
-
-    //-- RelationalExpr Types
-    //-- LF, changed from static const short to enum
-    enum _RelationalExprType {
-        EQUAL = 1,
+    enum RelationalExprType {
+        EQUAL,
         NOT_EQUAL,
         LESS_THAN,
         GREATER_THAN,
@@ -533,18 +526,17 @@ public:
         GREATER_OR_EQUAL
     };
 
-     RelationalExpr(Expr* leftExpr, Expr* rightExpr, short op);
-     ~RelationalExpr();
+    RelationalExpr(Expr* aLeftExpr, Expr* aRightExpr, RelationalExprType aOp);
 
     TX_DECL_EXPR;
 
 private:
-    short op;
-    Expr* leftExpr;
-    Expr* rightExpr;
+    PRBool compareResults(ExprResult* aLeft, ExprResult* aRight);
 
-    MBool compareResults(ExprResult* left, ExprResult* right);
-}; //-- RelationalExpr
+    nsAutoPtr<Expr> mLeftExpr;
+    nsAutoPtr<Expr> mRightExpr;
+    RelationalExprType mOp;
+};
 
 /**
  * VariableRefExpr
