@@ -32,19 +32,15 @@
  */
 
 #include "Expr.h"
-#include "XMLUtils.h"
+#include "NodeSet.h"
 #include "txNodeSetContext.h"
 #include "txSingleNodeContext.h"
+#include "XMLUtils.h"
 
   //------------/
  //- PathExpr -/
 //------------/
 
-const String PathExpr::RTF_INVALID_OP = 
-    "Result tree fragments don't allow location steps";
-const String PathExpr::NODESET_EXPECTED = 
-    "Filter expression must evaluate to a NodeSet";
-
 /**
  * Creates a new PathExpr
 **/
@@ -58,14 +54,12 @@ PathExpr::PathExpr()
 **/
 PathExpr::~PathExpr()
 {
-    ListIterator* iter = expressions.iterator();
-    while (iter->hasNext()) {
-         iter->next();
-         PathExprItem* pxi = (PathExprItem*)iter->remove();
+    txListIterator iter(&expressions);
+    while (iter.hasNext()) {
+         PathExprItem* pxi = (PathExprItem*)iter.next();
          delete pxi->expr;
          delete pxi;
     }
-    delete iter;
 } //-- ~PathExpr
 
 /**
@@ -104,7 +98,7 @@ ExprResult* PathExpr::evaluate(txIEvalContext* aContext)
 {
     if (!aContext || (expressions.getLength() == 0)) {
         NS_ASSERTION(0, "internal error");
-        return new StringResult("error");
+        return new StringResult(NS_LITERAL_STRING("error"));
     }
 
     NodeSet* nodes = new NodeSet(aContext->getContextNode());
@@ -114,7 +108,7 @@ ExprResult* PathExpr::evaluate(txIEvalContext* aContext)
         return 0;
     }
 
-    ListIterator iter(&expressions);
+    txListIterator iter(&expressions);
     PathExprItem* pxi;
 
     while ((pxi = (PathExprItem*)iter.next())) {
@@ -173,13 +167,13 @@ void PathExpr::evalDescendants (Expr* aStep, Node* aNode,
     delete res;
 
     MBool filterWS = aContext->isStripSpaceAllowed(aNode);
-    
+
     Node* child = aNode->getFirstChild();
     while (child) {
         if (!(filterWS &&
               (child->getNodeType() == Node::TEXT_NODE ||
                child->getNodeType() == Node::CDATA_SECTION_NODE) &&
-              XMLUtils::shouldStripTextnode(child->getNodeValue())))
+              XMLUtils::isWhitespace(child)))
             evalDescendants(aStep, child, aContext, resNodes);
         child = child->getNextSibling();
     }
@@ -193,9 +187,9 @@ void PathExpr::evalDescendants (Expr* aStep, Node* aNode,
  * other #toString() methods for Expressions.
  * @return the String representation of this Expr.
 **/
-void PathExpr::toString(String& dest)
+void PathExpr::toString(nsAString& dest)
 {
-    ListIterator iter(&expressions);
+    txListIterator iter(&expressions);
     
     PathExprItem* pxi = (PathExprItem*)iter.next();
     if (pxi) {
@@ -207,10 +201,10 @@ void PathExpr::toString(String& dest)
     while ((pxi = (PathExprItem*)iter.next())) {
         switch (pxi->pathOp) {
             case DESCENDANT_OP:
-                dest.append("//");
+                dest.Append(NS_LITERAL_STRING("//"));
                 break;
             case RELATIVE_OP:
-                dest.append('/');
+                dest.Append(PRUnichar('/'));
                 break;
         }
         pxi->expr->toString(dest);
