@@ -24,6 +24,8 @@
  */
 
 #include "Expr.h"
+#include "NodeSet.h"
+#include "txNodeSetContext.h"
 
 /*
  * Represents an ordered list of Predicates,
@@ -41,9 +43,7 @@ PredicateList::~PredicateList()
 {
     txListIterator iter(&predicates);
     while (iter.hasNext()) {
-        iter.next();
-        Expr* expr = (Expr*)iter.remove();
-        delete expr;
+        delete (Expr*)iter.next();
     }
 } // ~PredicateList
 
@@ -56,57 +56,48 @@ void PredicateList::add(Expr* expr)
     predicates.add(expr);
 } // add
 
-void PredicateList::evaluatePredicates(NodeSet* nodes, ContextState* cs)
+void PredicateList::evaluatePredicates(NodeSet* nodes,
+                                       txIMatchContext* aContext)
 {
     NS_ASSERTION(nodes, "called evaluatePredicates with NULL NodeSet");
     if (!nodes)
         return;
 
-    cs->getNodeSetStack()->push(nodes);
     NodeSet newNodes;
-    // optimize; set DuplicateChecking to MB_FALSE,
-    // restore original state later
-    // we only work with |Node|s already in the NodeSet, so they
-    // have been checked already, no need to check again in here.
-    MBool ndsCheckDupl = nodes->getDuplicateChecking();
-    nodes->setDuplicateChecking(MB_FALSE);
-    newNodes.setDuplicateChecking(MB_FALSE);
     txListIterator iter(&predicates);
-    while (iter.hasNext()) {
+    while (iter.hasNext() && !nodes->isEmpty()) {
         Expr* expr = (Expr*)iter.next();
+        txNodeSetContext predContext(nodes, aContext);
         /*
          * add nodes to newNodes that match the expression
          * or, if the result is a number, add the node with the right
          * position
          */
         newNodes.clear();
-        PRUint32 nIdx;
-        for (nIdx = 0; nIdx < nodes->size(); nIdx++) {
-            Node* node = nodes->get(nIdx);
-            ExprResult* exprResult = expr->evaluate(node, cs);
+        while (predContext.hasNext()) {
+            predContext.next();
+            ExprResult* exprResult = expr->evaluate(&predContext);
             if (!exprResult)
                 break;
             switch(exprResult->getResultType()) {
                 case ExprResult::NUMBER :
                     // handle default, [position() == numberValue()]
-                    if ((double)(nIdx+1) == exprResult->numberValue())
-                        newNodes.add(node);
+                    if ((double)predContext.position() ==
+                        exprResult->numberValue())
+                        newNodes.append(predContext.getContextNode());
                     break;
                 default:
                     if (exprResult->booleanValue())
-                        newNodes.add(node);
+                        newNodes.append(predContext.getContextNode());
                     break;
             }
             delete exprResult;
         }
         // Move new NodeSet to the current one
         nodes->clear();
-        newNodes.copyInto(*nodes);
+        nodes->append(&newNodes);
     }
-    cs->getNodeSetStack()->pop();
-    // restore DuplicateChecking of NodeSet
-    nodes->setDuplicateChecking(ndsCheckDupl);
-} // evaluatePredicates
+}
 
 /*
  * returns true if this predicate list is empty
@@ -116,14 +107,14 @@ MBool PredicateList::isEmpty()
     return (MBool)(predicates.getLength() == 0);
 } // isEmpty
 
-void PredicateList::toString(String& dest)
+void PredicateList::toString(nsAString& dest)
 {
     txListIterator iter(&predicates);
     while (iter.hasNext()) {
         Expr* expr = (Expr*) iter.next();
-        dest.append("[");
+        dest.Append(PRUnichar('['));
         expr->toString(dest);
-        dest.append("]");
+        dest.Append(PRUnichar(']'));
     }
 } // toString
 
