@@ -24,6 +24,7 @@
  */
 
 #include "Expr.h"
+#include "ExprResult.h"
 
 /**
  * StringExpr
@@ -32,9 +33,9 @@
 /**
  * Creates a new StringExpr
 **/
-StringExpr::StringExpr(const String& value) {
+StringExpr::StringExpr(const nsAString& value) {
     //-- copy value
-    this->value.append(value);
+    this->value.Append(value);
 } //-- StringExpr
 
 /**
@@ -57,12 +58,12 @@ ExprResult* StringExpr::evaluate(txIEvalContext* aContext)
  * other #toString() methods for Expressions.
  * @return the String representation of this Expr.
 **/
-void StringExpr::toString(String& str) {
-    UNICODE_CHAR ch = '\'';
-    if (value.indexOf(ch) != kNotFound)
+void StringExpr::toString(nsAString& str) {
+    PRUnichar ch = '\'';
+    if (value.FindChar(ch) != kNotFound)
         ch = '\"';
-    str.append(ch);
-    str.append(value);
-    str.append(ch);
+    str.Append(ch);
+    str.Append(value);
+    str.Append(ch);
 } //-- toString
 
