@@ -84,7 +84,7 @@ ExprResult* UnionExpr::evaluate(txIEvalContext* aContext)
             exprResult->getResultType() != ExprResult::NODESET) {
             delete exprResult;
             delete nodes;
-            return new StringResult("error");
+            return new StringResult(NS_LITERAL_STRING("error"));
         }
         nodes->add((NodeSet*)exprResult);
         delete exprResult;
@@ -101,14 +101,14 @@ ExprResult* UnionExpr::evaluate(txIEvalContext* aContext)
  * other #toString() methods for Expressions.
  * @return the String representation of this Expr.
 **/
-void UnionExpr::toString(String& dest) {
+void UnionExpr::toString(nsAString& dest) {
     txListIterator iter(&expressions);
 
     short count = 0;
     while (iter.hasNext()) {
         //-- set operator
         if (count > 0)
-            dest.append(" | ");
+            dest.Append(NS_LITERAL_STRING(" | "));
         ((Expr*)iter.next())->toString(dest);
         ++count;
     }
