@@ -82,15 +82,13 @@ txMozillaTextOutput::~txMozillaTextOutput()
 {
 }
 
-NS_IMPL_ISUPPORTS1(txMozillaTextOutput, txIOutputXMLEventHandler);
-
 void txMozillaTextOutput::attribute(const nsAString& aName,
                                     const PRInt32 aNsID,
                                     const nsAString& aValue)
 {
 }
 
-void txMozillaTextOutput::characters(const nsAString& aData)
+void txMozillaTextOutput::characters(const nsAString& aData, PRBool aDOE)
 {
     if (mTextNode)
         mTextNode->AppendData(aData);
