@@ -88,14 +88,18 @@ txMozillaXMLOutput::txMozillaXMLOutput(const nsAString& aRootName,
       mDontAddCurrent(PR_FALSE),
       mHaveTitleElement(PR_FALSE),
       mHaveBaseElement(PR_FALSE),
-      mInTransform(PR_FALSE),
       mCreatingNewDocument(PR_TRUE)
 {
+    if (aObserver) {
+        mNotifier = new txTransformNotifier();
+        if (mNotifier) {
+            mNotifier->Init(aObserver);
+        }
+    }
+
     mOutputFormat.merge(*aFormat);
     mOutputFormat.setFromDefaults();
 
-    mObserver = aObserver;
-
     createResultDocument(aRootName, aRootNsID, aSourceDocument, aResultDocument);
 }
 
@@ -105,7 +109,6 @@ txMozillaXMLOutput::txMozillaXMLOutput(txOutputFormat* aFormat,
       mDontAddCurrent(PR_FALSE),
       mHaveTitleElement(PR_FALSE),
       mHaveBaseElement(PR_FALSE),
-      mInTransform(PR_FALSE),
       mCreatingNewDocument(PR_FALSE)
 {
     mOutputFormat.merge(*aFormat);
@@ -113,6 +116,9 @@ txMozillaXMLOutput::txMozillaXMLOutput(txOutputFormat* aFormat,
 
     aFragment->GetOwnerDocument(getter_AddRefs(mDocument));
 
+    nsCOMPtr<nsIHTMLDocument> htmlDoc = do_QueryInterface(mDocument);
+    mDocumentIsHTML = !!htmlDoc;
+
     mCurrentNode = aFragment;
 }
 
@@ -120,11 +126,6 @@ txMozillaXMLOutput::~txMozillaXMLOutput()
 {
 }
 
-NS_IMPL_ISUPPORTS3(txMozillaXMLOutput,
-                   txIOutputXMLEventHandler,
-                   nsIScriptLoaderObserver,
-                   nsICSSLoaderObserver);
-
 void txMozillaXMLOutput::attribute(const nsAString& aName,
                                    const PRInt32 aNsID,
                                    const nsAString& aValue)
@@ -157,7 +158,7 @@ void txMozillaXMLOutput::attribute(const nsAString& aName,
     }
 }
 
-void txMozillaXMLOutput::characters(const nsAString& aData)
+void txMozillaXMLOutput::characters(const nsAString& aData, PRBool aDOE)
 {
     closePrevious(eCloseElement);
 
@@ -214,8 +215,9 @@ void txMozillaXMLOutput::endDocument()
         }
     }
 
-    mInTransform = PR_FALSE;
-    SignalTransformEnd();
+    if (mNotifier) {
+        mNotifier->OnTransformEnd();
+    }
 }
 
 void txMozillaXMLOutput::endElement(const nsAString& aName, const PRInt32 aNsID)
@@ -313,18 +315,22 @@ void txMozillaXMLOutput::processingInstruction(const nsAString& aTarget, const n
 
     if (ssle) {
         ssle->SetEnableUpdates(PR_TRUE);
-        rv = ssle->UpdateStyleSheet(nsnull, this);
+        rv = ssle->UpdateStyleSheet(nsnull, mNotifier);
         if (rv == NS_ERROR_HTMLPARSER_BLOCK) {
             nsCOMPtr<nsIStyleSheet> stylesheet;
             ssle->GetStyleSheet(*getter_AddRefs(stylesheet));
-            mStylesheets.AppendObject(stylesheet);
+            if (mNotifier) {
+                mNotifier->AddStyleSheet(stylesheet);
+            }
         }
     }
 }
 
 void txMozillaXMLOutput::startDocument()
 {
-    mInTransform = PR_TRUE;
+    if (mNotifier) {
+        mNotifier->OnTransformStart();
+    }
 }
 
 void txMozillaXMLOutput::startElement(const nsAString& aName,
@@ -473,7 +479,7 @@ void txMozillaXMLOutput::closePrevious(PRInt8 aAction)
         NS_ASSERTION(NS_SUCCEEDED(rv), "Can't create text node");
 
         nsCOMPtr<nsIDOMNode> resultNode;
-        mCurrentNode->AppendChild(text, getter_AddRefs(resultNode));
+        rv = mCurrentNode->AppendChild(text, getter_AddRefs(resultNode));
         NS_ASSERTION(NS_SUCCEEDED(rv), "Can't append text node");
 
         mText.Truncate();
@@ -488,7 +494,29 @@ void txMozillaXMLOutput::startHTMLElement(nsIDOMElement* aElement)
 
     mDontAddCurrent = (atom == txHTMLAtoms::script);
 
-    if (mCreatingNewDocument) {
+    if (atom == txHTMLAtoms::head) {
+        // Insert META tag, according to spec, 16.2, like
+        // <META http-equiv="Content-Type" content="text/html; charset=EUC-JP">
+        nsresult rv;
+        nsCOMPtr<nsIDOMNode> foo;
+        nsCOMPtr<nsIDOMElement> meta;
+        rv = mDocument->CreateElement(NS_LITERAL_STRING("meta"),
+                                      getter_AddRefs(meta));
+        NS_ASSERTION(NS_SUCCEEDED(rv), "Can't create meta element");
+        rv = meta->SetAttribute(NS_LITERAL_STRING("http-equiv"),
+                                NS_LITERAL_STRING("Content-Type"));
+        NS_ASSERTION(NS_SUCCEEDED(rv), "Can't set http-equiv on meta");
+        nsAutoString metacontent;
+        metacontent.Append(mOutputFormat.mMediaType);
+        metacontent.Append(NS_LITERAL_STRING("; charset="));
+        metacontent.Append(mOutputFormat.mEncoding);
+        rv = meta->SetAttribute(NS_LITERAL_STRING("content"),
+                                metacontent);
+        NS_ASSERTION(NS_SUCCEEDED(rv), "Can't set content on meta");
+        rv = aElement->AppendChild(meta, getter_AddRefs(foo));
+        NS_ASSERTION(NS_SUCCEEDED(rv), "Can't append meta");
+    }
+    else if (mCreatingNewDocument) {
         nsCOMPtr<nsIStyleSheetLinkingElement> ssle =
             do_QueryInterface(aElement);
         if (ssle) {
@@ -548,13 +576,12 @@ void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
         }
     }
     // Load scripts
-    else if (mCreatingNewDocument &&
-             atom == txHTMLAtoms::script) {
+    else if (mNotifier && atom == txHTMLAtoms::script) {
         // Add this script element to the array of loading script elements.
         nsCOMPtr<nsIDOMHTMLScriptElement> scriptElement =
             do_QueryInterface(mCurrentNode);
         NS_ASSERTION(scriptElement, "Need script element");
-        mScriptElements.AppendObject(scriptElement);
+        mNotifier->AddScriptElement(scriptElement);
     }
     // Set document title
     else if (mCreatingNewDocument &&
@@ -571,8 +598,8 @@ void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
             domDoc->SetTitle(text);
         }
     }
-    else if (mCreatingNewDocument &&
-             atom == txHTMLAtoms::base && !mHaveBaseElement) {
+    else if (mCreatingNewDocument && atom == txHTMLAtoms::base &&
+             !mHaveBaseElement) {
         // The first base wins
         mHaveBaseElement = PR_TRUE;
 
@@ -589,8 +616,7 @@ void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
             return;
         doc->SetBaseURL(baseURI); // The document checks if it is legal to set this base
     }
-    else if (mCreatingNewDocument &&
-             atom == txHTMLAtoms::meta) {
+    else if (mCreatingNewDocument && atom == txHTMLAtoms::meta) {
         // handle HTTP-EQUIV data
         nsAutoString httpEquiv;
         content->GetAttr(kNameSpaceID_None, txHTMLAtoms::httpEquiv, httpEquiv);
@@ -613,11 +639,13 @@ void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
             do_QueryInterface(aElement);
         if (ssle) {
             ssle->SetEnableUpdates(PR_TRUE);
-            rv = ssle->UpdateStyleSheet(nsnull, this);
+            rv = ssle->UpdateStyleSheet(nsnull, mNotifier);
             if (rv == NS_ERROR_HTMLPARSER_BLOCK) {
                 nsCOMPtr<nsIStyleSheet> stylesheet;
                 ssle->GetStyleSheet(*getter_AddRefs(stylesheet));
-                mStylesheets.AppendObject(stylesheet);
+                if (mNotifier) {
+                    mNotifier->AddStyleSheet(stylesheet);
+                }
             }
         }
     }
@@ -685,6 +713,7 @@ txMozillaXMLOutput::createResultDocument(const nsAString& aName, PRInt32 aNsID,
         nsCOMPtr<nsIHTMLDocument> htmlDoc = do_QueryInterface(aResultDocument);
         mDocumentIsHTML = !!htmlDoc;
     }
+
     mCurrentNode = mDocument;
 
     // Reset and set up the document
@@ -722,8 +751,8 @@ txMozillaXMLOutput::createResultDocument(const nsAString& aName, PRInt32 aNsID,
     nsCOMPtr<nsIScriptLoader> loader;
     doc->GetScriptLoader(getter_AddRefs(loader));
     if (loader) {
-        if (mObserver) {
-            loader->AddObserver(this);
+        if (mNotifier) {
+            loader->AddObserver(mNotifier);
         }
         else {
             // Don't load scripts, we can't notify the caller when they're loaded.
@@ -731,9 +760,8 @@ txMozillaXMLOutput::createResultDocument(const nsAString& aName, PRInt32 aNsID,
         }
     }
 
-    // Notify the contentsink that the document is created
-    if (mObserver) {
-        mObserver->OnDocumentCreated(mDocument);
+    if (mNotifier) {
+        mNotifier->SetOutputDocument(mDocument);
     }
 
     // Do this after calling OnDocumentCreated to ensure that the
@@ -768,14 +796,28 @@ txMozillaXMLOutput::createResultDocument(const nsAString& aName, PRInt32 aNsID,
     return NS_OK;
 }
 
+txTransformNotifier::txTransformNotifier()
+    : mInTransform(PR_FALSE)
+      
+{
+}
+
+txTransformNotifier::~txTransformNotifier()
+{
+}
+
+NS_IMPL_ISUPPORTS2(txTransformNotifier,
+                   nsIScriptLoaderObserver,
+                   nsICSSLoaderObserver);
+
 NS_IMETHODIMP
-txMozillaXMLOutput::ScriptAvailable(nsresult aResult, 
-                                    nsIDOMHTMLScriptElement *aElement, 
-                                    PRBool aIsInline,
-                                    PRBool aWasPending,
-                                    nsIURI *aURI, 
-                                    PRInt32 aLineNo,
-                                    const nsAString& aScript)
+txTransformNotifier::ScriptAvailable(nsresult aResult, 
+                                     nsIDOMHTMLScriptElement *aElement, 
+                                     PRBool aIsInline,
+                                     PRBool aWasPending,
+                                     nsIURI *aURI, 
+                                     PRInt32 aLineNo,
+                                     const nsAString& aScript)
 {
     if (NS_FAILED(aResult)) {
         mScriptElements.RemoveObject(aElement);
@@ -786,10 +828,10 @@ txMozillaXMLOutput::ScriptAvailable(nsresult aResult,
 }
 
 NS_IMETHODIMP 
-txMozillaXMLOutput::ScriptEvaluated(nsresult aResult, 
-                                    nsIDOMHTMLScriptElement *aElement,
-                                    PRBool aIsInline,
-                                    PRBool aWasPending)
+txTransformNotifier::ScriptEvaluated(nsresult aResult, 
+                                     nsIDOMHTMLScriptElement *aElement,
+                                     PRBool aIsInline,
+                                     PRBool aWasPending)
 {
     mScriptElements.RemoveObject(aElement);
     SignalTransformEnd();
@@ -797,7 +839,7 @@ txMozillaXMLOutput::ScriptEvaluated(nsresult aResult,
 }
 
 NS_IMETHODIMP 
-txMozillaXMLOutput::StyleSheetLoaded(nsICSSStyleSheet* aSheet, PRBool aNotify)
+txTransformNotifier::StyleSheetLoaded(nsICSSStyleSheet* aSheet, PRBool aNotify)
 {
     // aSheet might not be in our list if the load was done synchronously
     mStylesheets.RemoveObject(aSheet);
@@ -806,17 +848,50 @@ txMozillaXMLOutput::StyleSheetLoaded(nsICSSStyleSheet* aSheet, PRBool aNotify)
 }
 
 void
-txMozillaXMLOutput::SignalTransformEnd()
+txTransformNotifier::Init(nsITransformObserver* aObserver)
 {
-    if (mInTransform) {
-        return;
-    }
+    mObserver = aObserver;
+}
 
-    if (!mObserver) {
-        return;
-    }
+void
+txTransformNotifier::AddScriptElement(nsIDOMHTMLScriptElement* aElement)
+{
+    mScriptElements.AppendObject(aElement);
+}
 
-    if (mScriptElements.Count() > 0 || mStylesheets.Count() > 0) {
+void
+txTransformNotifier::AddStyleSheet(nsIStyleSheet* aStyleSheet)
+{
+    mStylesheets.AppendObject(aStyleSheet);
+}
+
+void
+txTransformNotifier::OnTransformEnd()
+{
+    mInTransform = PR_FALSE;
+    SignalTransformEnd();
+}
+
+void
+txTransformNotifier::OnTransformStart()
+{
+    mInTransform = PR_TRUE;
+}
+
+void
+txTransformNotifier::SetOutputDocument(nsIDOMDocument* aDocument)
+{
+    mDocument = aDocument;
+
+    // Notify the contentsink that the document is created
+    mObserver->OnDocumentCreated(mDocument);
+}
+
+void
+txTransformNotifier::SignalTransformEnd()
+{
+    if (mInTransform || mScriptElements.Count() > 0 ||
+        mStylesheets.Count() > 0) {
         return;
     }
 
