@@ -70,7 +70,9 @@
 #include "nsFrameTraversal.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMXULDocument.h"
+#include "nsIImageDocument.h"
 #include "nsIDOMHTMLDocument.h"
+#include "nsIDOMNSHTMLDocument.h"
 #include "nsIDOMHTMLElement.h"
 #include "nsIEventStateManager.h"
 #include "nsIViewManager.h"
@@ -106,6 +108,8 @@
 #include "nsIPrivateCompositionEvent.h"
 #include "nsGUIEvent.h"
 #include "nsIDOMEventReceiver.h"
+#include "nsIDOM3EventTarget.h"
+#include "nsIDOMEventGroup.h"
 
 // Header for this class
 #include "nsTypeAheadFind.h"
@@ -544,6 +548,40 @@ nsTypeAheadFind::HandleEvent(nsIDOMEvent* aEvent)
   else if (eventType.Equals(NS_LITERAL_STRING("popuphidden"))) {
     mIsMenuPopupActive = PR_FALSE;
   }
+  else if (eventType.Equals(NS_LITERAL_STRING("unload"))) {
+    // When document is unloaded, check to see if it's the 
+    // current typeahead doc. If it is, cancel find
+    // and reset member variables so we don't leak
+    nsCOMPtr<nsIDOMNSEvent> event(do_QueryInterface(aEvent));
+    NS_ENSURE_TRUE(event, NS_ERROR_FAILURE);
+
+    nsCOMPtr<nsIDOMEventTarget> eventTarget;
+    event->GetOriginalTarget(getter_AddRefs(eventTarget));
+    nsCOMPtr<nsIDocument> doc(do_QueryInterface(eventTarget));
+    nsCOMPtr<nsIPresShell> focusedShell(do_QueryReferent(mFocusedWeakShell));
+    NS_ENSURE_TRUE(focusedShell && doc, NS_ERROR_FAILURE);
+
+    PRInt32 numShells = doc->GetNumberOfShells();
+    nsCOMPtr<nsIPresShell> shellToBeDestroyed;
+    PRBool cancelFind = PR_FALSE;
+
+    for (PRInt32 count = 0; count < numShells; count ++) {
+      doc->GetShellAt(count, getter_AddRefs(shellToBeDestroyed));
+      if (shellToBeDestroyed == focusedShell) {
+        cancelFind = PR_TRUE;
+        break;
+      }
+    }
+
+    if (cancelFind) {
+      RemoveDocListeners();
+      mSearchRange = do_CreateInstance(kRangeCID);
+      mStartPointRange = do_CreateInstance(kRangeCID);
+      mEndPointRange = do_CreateInstance(kRangeCID);
+      mFocusedWindow = nsnull;
+      CancelFind();
+    }
+  }
 
   return NS_OK;
 }
@@ -660,7 +698,13 @@ nsTypeAheadFind::KeyPress(nsIDOMEvent* aEvent)
 
   // ----------- Back space -------------------------
   if (keyCode == nsIDOMKeyEvent::DOM_VK_BACK_SPACE) {
-    if (HandleBackspace()) {
+    // The order of seeing keystrokes is:
+    // 1) Chrome, 2) Typeahead, 3) [platform]HTMLBindings.xml
+    // If chrome handles backspace, it needs to do this work
+    // Otherwise, we handle backspace here.
+    PRBool backspaceUsed;
+    BackOneChar(&backspaceUsed);
+    if (backspaceUsed) {
       aEvent->PreventDefault(); // Prevent normal processing of this keystroke
     }
 
@@ -689,13 +733,15 @@ nsTypeAheadFind::KeyPress(nsIDOMEvent* aEvent)
 }
 
 
-PRBool
-nsTypeAheadFind::HandleBackspace()
+NS_IMETHODIMP
+nsTypeAheadFind::BackOneChar(PRBool *aIsBackspaceUsed)
 {
   // In normal type ahead find, remove a printable char from 
   // mTypeAheadBuffer, then search for buffer contents
   // Or, in repeated char find, go backwards
 
+  *aIsBackspaceUsed = PR_TRUE;
+
   // ---------- No chars in string ------------
   if (mTypeAheadBuffer.IsEmpty() || !mStartFindRange) {
     if (!mFindNextBuffer.IsEmpty() &&
@@ -718,9 +764,12 @@ nsTypeAheadFind::HandleBackspace()
           soundInterface->Beep(); // beep to warn
         }
         mIsBackspaceProtectOn = PR_FALSE;
-        return PR_TRUE;
       }
-      return PR_FALSE;
+      else {
+        *aIsBackspaceUsed = PR_FALSE;
+      }
+
+      return NS_OK;
     }
   }
 
@@ -738,7 +787,7 @@ nsTypeAheadFind::HandleBackspace()
     CancelFind();
     mIsBackspaceProtectOn = PR_TRUE;
 
-    return PR_TRUE;
+    return NS_OK;
   }
 
   // ---------- Multiple chars in string ----------
@@ -759,7 +808,7 @@ nsTypeAheadFind::HandleBackspace()
     --mBadKeysSinceMatch;
     DisplayStatus(PR_FALSE, nsnull, PR_FALSE); // Display failure status
     SaveFind();
-    return PR_TRUE;
+    return NS_OK;
   }
 
   mBadKeysSinceMatch = 0;
@@ -781,7 +830,8 @@ nsTypeAheadFind::HandleBackspace()
       }
     }
     if (!presShell) {
-      return PR_FALSE;
+      *aIsBackspaceUsed = PR_FALSE;
+      return NS_ERROR_FAILURE;
     }
     // Set the selection to the where the first character was found
     // so that find starts from there
@@ -804,7 +854,7 @@ nsTypeAheadFind::HandleBackspace()
 
   SaveFind();
 
-  return PR_TRUE;   // Backspace handled
+  return NS_OK;   // Backspace handled
 }
 
 
@@ -908,7 +958,8 @@ nsTypeAheadFind::HandleChar(PRUnichar aChar)
     }
 
 #ifndef NO_LINK_CYCLE_ON_SAME_CHAR
-    if (NS_FAILED(rv) && !mLiteralTextSearchOnly && mAllTheSameChar) {
+    if (NS_FAILED(rv) && !mLiteralTextSearchOnly && mAllTheSameChar && 
+        mTypeAheadBuffer.Length() > 1) {
       mRepeatingMode = eRepeatingChar;
       mDontTryExactMatch = PR_TRUE;  // Repeated character find mode
       rv = FindItNow(nsnull, PR_TRUE, PR_TRUE, mIsFirstVisiblePreferred);
@@ -1328,6 +1379,7 @@ nsTypeAheadFind::FindItNow(nsIPresShell *aPresShell,
     // ======= end-inner-while (go through a single document) ==========
 
     // ---------- Nothing found yet, try next document  -------------
+    PRBool hasTriedFirstDoc = PR_FALSE;
     do {
       // ==== Second inner loop - get another while  ====
       if (NS_SUCCEEDED(docShellEnumerator->HasMoreElements(&hasMoreDocShells))
@@ -1340,11 +1392,15 @@ nsTypeAheadFind::FindItNow(nsIPresShell *aPresShell,
           break;
         }
       }
+      else if (hasTriedFirstDoc) {  // Avoid potential infinite loop
+        return NS_ERROR_FAILURE;  // No content doc shells
+      }
 
       // Reached last doc shell, loop around back to first doc shell
       rootContentDocShell->GetDocShellEnumerator(nsIDocShellTreeItem::typeContent,
                                                  nsIDocShell::ENUMERATE_FORWARDS,
                                                  getter_AddRefs(docShellEnumerator));
+      hasTriedFirstDoc = PR_TRUE;
     } while (docShellEnumerator);  // ==== end second inner while  ===
 
     PRBool continueLoop = PR_FALSE;
@@ -1921,7 +1977,8 @@ nsTypeAheadFind::GetAutoStart(nsIDOMWindow *aDOMWin, PRBool *aIsAutoStartOn)
   NS_ENSURE_TRUE(doc, NS_OK);
 
   nsCOMPtr<nsIDOMXULDocument> xulDoc(do_QueryInterface(doc));
-  if (xulDoc) {
+  nsCOMPtr<nsIImageDocument> imageDoc(do_QueryInterface(doc));
+  if (xulDoc || imageDoc) {
     return NS_OK; // Avoid any xul docs, whether in chrome or content
   }
 
@@ -2240,9 +2297,14 @@ nsTypeAheadFind::RemoveWindowListeners(nsIDOMWindow *aDOMWin)
   }
 
   // Use capturing, otherwise the normal find next will get activated when ours should
-  chromeEventHandler->RemoveEventListener(NS_LITERAL_STRING("keypress"),
-                                          NS_STATIC_CAST(nsIDOMKeyListener*, this),
-                                          PR_FALSE);
+  nsCOMPtr<nsIDOMEventReceiver> receiver(do_QueryInterface(chromeEventHandler));
+  nsCOMPtr<nsIDOMEventGroup> systemGroup;
+  receiver->GetSystemEventGroup(getter_AddRefs(systemGroup));
+  nsCOMPtr<nsIDOM3EventTarget> target(do_QueryInterface(receiver));
+
+  target->RemoveGroupedEventListener(NS_LITERAL_STRING("keypress"),
+                                     NS_STATIC_CAST(nsIDOMKeyListener*, this),
+                                     PR_FALSE, systemGroup);
 
   if (aDOMWin == mFocusedWindow) {
     mFocusedWindow = nsnull;
@@ -2268,6 +2330,10 @@ nsTypeAheadFind::RemoveWindowListeners(nsIDOMWindow *aDOMWin)
                                           genericEventListener, 
                                           PR_TRUE);
 
+  chromeEventHandler->RemoveEventListener(NS_LITERAL_STRING("unload"), 
+                                          genericEventListener, 
+                                          PR_TRUE);
+
   // Remove DOM Text listener for IME text events
   nsCOMPtr<nsIDOMEventReceiver> chromeEventReceiver = 
     do_QueryInterface(chromeEventHandler);
@@ -2288,9 +2354,14 @@ nsTypeAheadFind::AttachWindowListeners(nsIDOMWindow *aDOMWin)
   }
 
   // Use capturing, otherwise the normal find next will get activated when ours should
-  chromeEventHandler->AddEventListener(NS_LITERAL_STRING("keypress"),
-                                       NS_STATIC_CAST(nsIDOMKeyListener*, this),
-                                       PR_FALSE);
+  nsCOMPtr<nsIDOMEventReceiver> receiver(do_QueryInterface(chromeEventHandler));
+  nsCOMPtr<nsIDOMEventGroup> systemGroup;
+  receiver->GetSystemEventGroup(getter_AddRefs(systemGroup));
+  nsCOMPtr<nsIDOM3EventTarget> target(do_QueryInterface(receiver));
+
+  target->AddGroupedEventListener(NS_LITERAL_STRING("keypress"),
+                                  NS_STATIC_CAST(nsIDOMKeyListener*, this),
+                                  PR_FALSE, systemGroup);
 
   // Attach menu listeners, this will help us ignore keystrokes meant for menus
   nsIDOMEventListener *genericEventListener = 
@@ -2312,6 +2383,10 @@ nsTypeAheadFind::AttachWindowListeners(nsIDOMWindow *aDOMWin)
                                        genericEventListener, 
                                        PR_TRUE);
 
+  chromeEventHandler->AddEventListener(NS_LITERAL_STRING("unload"), 
+                                       genericEventListener, 
+                                       PR_TRUE);
+
   // Add DOM Text listener for IME text events
   nsCOMPtr<nsIDOMEventReceiver> chromeEventReceiver =
     do_QueryInterface(chromeEventHandler);
@@ -2338,6 +2413,45 @@ nsTypeAheadFind::GetChromeEventHandler(nsIDOMWindow *aDOMWin,
   NS_IF_ADDREF(*aChromeTarget);
 }
 
+PRBool
+nsTypeAheadFind::IsTargetContentOkay(nsIContent *aContent)
+{
+  if (!aContent) {
+    return PR_FALSE;
+  }
+
+  if (aContent->IsContentOfType(nsIContent::eHTML_FORM_CONTROL)) {
+    nsCOMPtr<nsIFormControl> formControl(do_QueryInterface(aContent));
+    PRInt32 controlType = formControl->GetType();
+    if (controlType == NS_FORM_SELECT || 
+        controlType == NS_FORM_TEXTAREA ||
+        controlType == NS_FORM_INPUT_TEXT ||
+        controlType == NS_FORM_INPUT_PASSWORD ||
+        controlType == NS_FORM_INPUT_FILE) {
+      // Don't steal keys from these form controls 
+      // - selects have their own incremental find for options
+      // - text fields need to allow typing
+      return PR_FALSE;
+    }
+  }
+  else if (aContent->IsContentOfType(nsIContent::eHTML)) {
+    // Test for isindex, a deprecated kind of text field. We're using a string 
+    // compare because <isindex> is not considered a form control, so it does 
+    // not support nsIFormControl or eHTML_FORM_CONTROL, and it's not worth 
+    // having a table of atoms just for it. Instead, we're paying for 1 extra 
+    // string compare per keystroke, which isn't too bad.
+    nsCOMPtr<nsIAtom> targetTagAtom;
+    aContent->GetTag(*getter_AddRefs(targetTagAtom));
+    nsAutoString targetTagString;
+    targetTagAtom->ToString(targetTagString);
+    if (targetTagString.Equals(NS_LITERAL_STRING("isindex"))) {
+      return PR_FALSE;
+    }
+  }
+
+  return PR_TRUE;
+}
+
 
 nsresult
 nsTypeAheadFind::GetTargetIfTypeAheadOkay(nsIDOMEvent *aEvent, 
@@ -2363,45 +2477,13 @@ nsTypeAheadFind::GetTargetIfTypeAheadOkay(nsIDOMEvent *aEvent,
 
   // ---- Exit early if in form controls that can be typed in ---------
 
-  if (!targetContent) {
+  if (!IsTargetContentOkay(targetContent)) {
+    if (!mTypeAheadBuffer.IsEmpty()) {
+      CancelFind();
+    }
     return NS_OK;
   }
 
-  if (targetContent->IsContentOfType(nsIContent::eHTML_FORM_CONTROL)) {
-    nsCOMPtr<nsIFormControl> formControl(do_QueryInterface(targetContent));
-    PRInt32 controlType = formControl->GetType();
-    if (controlType == NS_FORM_SELECT || 
-        controlType == NS_FORM_TEXTAREA ||
-        controlType == NS_FORM_INPUT_TEXT ||
-        controlType == NS_FORM_INPUT_PASSWORD ||
-        controlType == NS_FORM_INPUT_FILE) {
-      // Don't steal keys from these form controls 
-      // - selects have their own incremental find for options
-      // - text fields need to allow typing
-      if (!mTypeAheadBuffer.IsEmpty()) {
-        CancelFind();
-      }
-      return NS_OK;
-    }
-  }
-  else if (targetContent->IsContentOfType(nsIContent::eHTML)) {
-    // Test for isindex, a deprecated kind of text field. We're using a string 
-    // compare because <isindex> is not considered a form control, so it does 
-    // not support nsIFormControl or eHTML_FORM_CONTROL, and it's not worth 
-    // having a table of atoms just for it. Instead, we're paying for 1 extra 
-    // string compare per keystroke, which isn't too bad.
-    nsCOMPtr<nsIAtom> targetTagAtom;
-    targetContent->GetTag(*getter_AddRefs(targetTagAtom));
-    nsAutoString targetTagString;
-    targetTagAtom->ToString(targetTagString);
-    if (targetTagString.Equals(NS_LITERAL_STRING("isindex"))) {
-      if (!mTypeAheadBuffer.IsEmpty())  {
-        CancelFind();
-      }
-      return NS_OK;
-    }
-  }
-
   NS_ADDREF(*aTargetContent = targetContent);
 
   // ---------- Is the keystroke in a new window? -------------------
@@ -2432,6 +2514,10 @@ nsTypeAheadFind::GetTargetIfTypeAheadOkay(nsIDOMEvent *aEvent,
     if (mIsFindAllowedInWindow) {
       UseInWindow(topContentWin);
     }
+    else {
+      CancelFind();
+      mFocusedWindow = nsnull;
+    }
   }
   if (!mIsFindAllowedInWindow) {
     return NS_OK;
@@ -2513,9 +2599,7 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
     return PR_FALSE;
   }
 
-  const nsStyleVisibility* vis = nsnull;
-  ::GetStyleData(frame, &vis);
-  if (!vis || !vis->IsVisible()) {
+  if (!frame->GetStyleVisibility()->IsVisible()) {
     return PR_FALSE;
   }
 
@@ -2791,7 +2875,47 @@ nsTypeAheadController::nsTypeAheadController(nsIFocusController *aFocusControlle
 NS_IMETHODIMP
 nsTypeAheadController::IsCommandEnabled(const char *aCommand, PRBool *aResult)
 {
-  return SupportsCommand(aCommand, aResult);
+  NS_ENSURE_ARG_POINTER(aResult);
+
+  *aResult = PR_FALSE;
+
+  NS_ENSURE_TRUE(mFocusController, NS_ERROR_FAILURE);
+
+  nsCOMPtr<nsIDOMElement> focusedElement;
+  mFocusController->GetFocusedElement(getter_AddRefs(focusedElement));
+
+  nsCOMPtr<nsIContent> focusedContent(do_QueryInterface(focusedElement));
+
+  // Make sure we're not focused on a text field, listbox
+  // or other form control that needs typeahead keystrokes
+  if (focusedContent) {
+    *aResult = nsTypeAheadFind::IsTargetContentOkay(focusedContent);
+    return NS_OK;
+  }
+
+  // We're focused on a document
+  nsCOMPtr<nsIDOMWindowInternal> winInternal;
+  mFocusController->GetFocusedWindow(getter_AddRefs(winInternal));
+  nsCOMPtr<nsIDOMWindow> domWin(do_QueryInterface(winInternal));
+  if (!domWin) {
+    return NS_OK;
+  }
+
+  *aResult = PR_TRUE;
+
+  // Make sure we're not in Midas editing mode
+  nsCOMPtr<nsIDOMDocument> domDoc;
+  domWin->GetDocument(getter_AddRefs(domDoc));
+  nsCOMPtr<nsIDOMNSHTMLDocument> htmlDoc(do_QueryInterface(domDoc));
+  if (htmlDoc) {
+    nsAutoString designMode;
+    htmlDoc->GetDesignMode(designMode);
+    if (designMode.Equals(NS_LITERAL_STRING("on"))) {
+      *aResult = PR_FALSE;
+    }
+  }
+  
+  return NS_OK;
 }
 
 NS_IMETHODIMP
