@@ -58,21 +58,9 @@
 #include "nsIDOMClassInfo.h"
 #include "nsReadableUtils.h"
 #include "nsCRT.h"
-
-#ifdef IMPLEMENT_SYNC_LOAD
-#include "nsIScriptContext.h"
-#include "nsIScriptGlobalObject.h"
-#include "nsIDocShell.h"
-#include "nsIDocShellTreeItem.h"
-#include "nsIDocShellTreeOwner.h"
-#include "nsIEventQueueService.h"
-#include "nsIInterfaceRequestor.h"
-#include "nsIInterfaceRequestorUtils.h"
 #include "nsIDOMEventReceiver.h"
-#include "jsapi.h"
 #include "nsLoadListenerProxy.h"
 static NS_DEFINE_IID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
-#endif
 
 static const char* kLoadAsData = "loadAsData";
 
@@ -104,7 +92,6 @@ protected:
 
 nsDOMParserChannel::nsDOMParserChannel(nsIURI* aURI, const char* aContentType)
 {
-  NS_INIT_ISUPPORTS();
   mURI = aURI;
   mContentType.Assign(aContentType);
   mStatus = NS_OK;
@@ -294,20 +281,6 @@ NS_IMETHODIMP nsDOMParserChannel::AsyncOpen(nsIStreamListener *listener, nsISupp
 //
 /////////////////////////////////////////////
 
-#ifdef IMPLEMENT_SYNC_LOAD
-// See if we have a modal event loop
-static inline PRBool IsModal(nsIWebBrowserChrome *aWindow)
-{
-  if (aWindow) {
-    PRBool isModal;
-    if (NS_SUCCEEDED(aWindow->IsWindowModal(&isModal))) {
-      return isModal;
-    }
-  }
-
-  return PR_FALSE;
-}
-
 // nsIDOMEventListener
 nsresult
 nsDOMParser::HandleEvent(nsIDOMEvent* aEvent)
@@ -319,11 +292,7 @@ nsDOMParser::HandleEvent(nsIDOMEvent* aEvent)
 nsresult
 nsDOMParser::Load(nsIDOMEvent* aEvent)
 {
-  if (IsModal(mChromeWindow)) {
-    mChromeWindow->ExitModalEventLoop(NS_OK);
-  }
-
-  mChromeWindow = nsnull;
+  mLoopingForSyncLoad = PR_FALSE;
 
   return NS_OK;
 }
@@ -337,11 +306,7 @@ nsDOMParser::Unload(nsIDOMEvent* aEvent)
 nsresult
 nsDOMParser::Abort(nsIDOMEvent* aEvent)
 {
-  if (IsModal(mChromeWindow)) {
-    mChromeWindow->ExitModalEventLoop(NS_OK);
-  }
-
-  mChromeWindow = nsnull;
+  mLoopingForSyncLoad = PR_FALSE;
 
   return NS_OK;
 }
@@ -349,28 +314,21 @@ nsDOMParser::Abort(nsIDOMEvent* aEvent)
 nsresult
 nsDOMParser::Error(nsIDOMEvent* aEvent)
 {
-  if (IsModal(mChromeWindow)) {
-    mChromeWindow->ExitModalEventLoop(NS_OK);
-  }
-
-  mChromeWindow = nsnull;
+  mLoopingForSyncLoad = PR_FALSE;
 
   return NS_OK;
 }
-#endif
 
 nsDOMParser::nsDOMParser()
+  : mLoopingForSyncLoad(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
+  mEventQService = do_GetService(kEventQueueServiceCID);
 }
 
 nsDOMParser::~nsDOMParser()
 {
-#ifdef IMPLEMENT_SYNC_LOAD
-  if (IsModal(mChromeWindow)) {
-    mChromeWindow->ExitModalEventLoop(NS_OK);
-  }
-#endif
+  NS_ABORT_IF_FALSE(!mLoopingForSyncLoad, "we rather crash than hang");
+  mLoopingForSyncLoad = PR_FALSE;
 }
 
 
@@ -378,10 +336,8 @@ nsDOMParser::~nsDOMParser()
 NS_INTERFACE_MAP_BEGIN(nsDOMParser)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIDOMParser)
   NS_INTERFACE_MAP_ENTRY(nsIDOMParser)
-#ifdef IMPLEMENT_SYNC_LOAD
   NS_INTERFACE_MAP_ENTRY(nsIDOMLoadListener)
   NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
-#endif
   NS_INTERFACE_MAP_ENTRY_EXTERNAL_DOM_CLASSINFO(DOMParser)
 NS_INTERFACE_MAP_END
 
@@ -551,7 +507,6 @@ nsDOMParser::ParseFromStream(nsIInputStream *stream,
                                       getter_AddRefs(domDocument));
   if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-#ifdef IMPLEMENT_SYNC_LOAD
   // Register as a load listener on the document
   nsCOMPtr<nsIDOMEventReceiver> target(do_QueryInterface(domDocument));
   if (target) {
@@ -565,7 +520,6 @@ nsDOMParser::ParseFromStream(nsIInputStream *stream,
                                        NS_GET_IID(nsIDOMLoadListener));
     if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
   }
-#endif
 
   // Create a fake channel 
   nsDOMParserChannel* parserChannel = new nsDOMParserChannel(baseURI, contentType);
@@ -583,67 +537,30 @@ nsDOMParser::ParseFromStream(nsIInputStream *stream,
   nsCOMPtr<nsIDocument> document(do_QueryInterface(domDocument));
   if (!document) return NS_ERROR_FAILURE;
 
-#ifdef IMPLEMENT_SYNC_LOAD
   nsCOMPtr<nsIEventQueue> modalEventQueue;
-  nsCOMPtr<nsIEventQueueService> eventQService;
-  
-  if (cc) {
-    JSContext* cx;
-    rv = cc->GetJSContext(&cx);
-    if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-    // We can only do this if we're called from a DOM script context
-    nsIScriptContext* scriptCX = (nsIScriptContext*)JS_GetContextPrivate(cx);
-    if (!scriptCX) return NS_OK;
+  if(!mEventQService) {
+    return NS_ERROR_FAILURE;
+  }
 
-    // Get the nsIDocShellTreeOwner associated with the window
-    // containing this script context
-    // XXX Need to find a better way to do this rather than
-    // chaining through a bunch of getters and QIs
-    nsCOMPtr<nsIScriptGlobalObject> global;
-    scriptCX->GetGlobalObject(getter_AddRefs(global));
-    if (!global) return NS_ERROR_FAILURE;
+  mLoopingForSyncLoad = PR_TRUE;
 
-    nsCOMPtr<nsIDocShell> docshell;
-    rv = global->GetDocShell(getter_AddRefs(docshell));
-    if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-    
-    nsCOMPtr<nsIDocShellTreeItem> item = do_QueryInterface(docshell);
-    if (!item) return NS_ERROR_FAILURE;
-
-    nsCOMPtr<nsIDocShellTreeOwner> treeOwner;
-    rv = item->GetTreeOwner(getter_AddRefs(treeOwner));
-    if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-
-    nsCOMPtr<nsIInterfaceRequestor> treeRequestor(do_GetInterface(treeOwner));
-    if (!treeRequestor) return NS_ERROR_FAILURE;
-
-    treeRequestor->GetInterface(NS_GET_IID(nsIWebBrowserChrome), getter_AddRefs(mChromeWindow));
-    if (!mChromeWindow) return NS_ERROR_FAILURE;
-
-    eventQService = do_GetService(kEventQueueServiceCID);
-    if(!eventQService || 
-       NS_FAILED(eventQService->PushThreadEventQueue(getter_AddRefs(modalEventQueue)))) {
-      return NS_ERROR_FAILURE;
-    }
+  rv = mEventQService->PushThreadEventQueue(getter_AddRefs(modalEventQueue));
+  if (NS_FAILED(rv)) {
+    return rv;
   }
-#endif
 
   rv = document->StartDocumentLoad(kLoadAsData, channel, 
                                    nsnull, nsnull, 
                                    getter_AddRefs(listener),
                                    PR_FALSE);
 
-#ifdef IMPLEMENT_SYNC_LOAD
   if (NS_FAILED(rv) || !listener) {
     if (modalEventQueue) {
-      eventQService->PopThreadEventQueue(modalEventQueue);
+      mEventQService->PopThreadEventQueue(modalEventQueue);
     }
     return NS_ERROR_FAILURE;
   }
-#else
-  if (NS_FAILED(rv) || !listener) return NS_ERROR_FAILURE;
-#endif
 
   // Now start pumping data to the listener
   nsresult status;
@@ -657,29 +574,19 @@ nsDOMParser::ParseFromStream(nsIInputStream *stream,
   }
 
   rv = listener->OnStopRequest(request, nsnull, status);
-#ifdef IMPLEMENT_SYNC_LOAD
+
   if (NS_FAILED(rv)) {
     if (modalEventQueue) {
-      eventQService->PopThreadEventQueue(modalEventQueue);
+      mEventQService->PopThreadEventQueue(modalEventQueue);
     }
     return NS_ERROR_FAILURE;
   }
-#else
-  if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-#endif
 
-#ifdef IMPLEMENT_SYNC_LOAD
-  // Spin an event loop here and wait
-  if (mChromeWindow) {
-    rv = mChromeWindow->ShowAsModal();
-    
-    eventQService->PopThreadEventQueue(modalEventQueue);
-    
-    if (NS_FAILED(rv)) return NS_ERROR_FAILURE;      
-  } else if (modalEventQueue) {
-    eventQService->PopThreadEventQueue(modalEventQueue);
+  while (mLoopingForSyncLoad) {
+    modalEventQueue->ProcessPendingEvents();
   }
-#endif
+
+  mEventQService->PopThreadEventQueue(modalEventQueue);
 
   *_retval = domDocument;
   NS_ADDREF(*_retval);
