@@ -184,6 +184,7 @@ NS_IMETHODIMP nsRenderingContextGTK::Init(nsIDeviceContext* aContext,
 
   mSurface = (nsDrawingSurfaceGTK *) aSurface;
   NS_ADDREF(mSurface);
+  mOffscreenSurface = mSurface;
 
   return (CommonInit());
 }
@@ -225,7 +226,7 @@ NS_IMETHODIMP nsRenderingContextGTK::LockDrawingSurface(PRInt32 aX, PRInt32 aY,
   PushState();
 
   return mSurface->Lock(aX, aY, aWidth, aHeight,
-  			aBits, aStride, aWidthBytes, aFlags);
+                        aBits, aStride, aWidthBytes, aFlags);
 }
 
 NS_IMETHODIMP nsRenderingContextGTK::UnlockDrawingSurface(void)
@@ -415,22 +416,13 @@ NS_IMETHODIMP nsRenderingContextGTK::GetClipRect(nsRect &aRect, PRBool &aClipVal
 }
 
 #ifdef DEBUG
-#undef TRACE_SET_CLIP
+// #define TRACE_SET_CLIP
 #endif
 
 #ifdef TRACE_SET_CLIP
 static char *
 nsClipCombine_to_string(nsClipCombine aCombine)
 {
-#ifdef TRACE_SET_CLIP
-  printf("nsRenderingContextGTK::SetClipRect(x=%d,y=%d,w=%d,h=%d,%s)\n",
-         trect.x,
-         trect.y,
-         trect.width,
-         trect.height,
-         nsClipCombine_to_string(aCombine));
-#endif // TRACE_SET_CLIP
-
   switch(aCombine)
     {
       case nsClipCombine_kIntersect:
@@ -458,9 +450,17 @@ NS_IMETHODIMP nsRenderingContextGTK::SetClipRect(const nsRect& aRect,
                                                  nsClipCombine aCombine,
                                                  PRBool &aClipEmpty)
 {
+  nsRect trect = aRect;
+  mTranMatrix->TransformCoord(&trect.x, &trect.y,
+                              &trect.width, &trect.height);
+  SetClipRectInPixels(trect, aCombine, aClipEmpty);
+  return NS_OK;
+}
 
-
-
+void nsRenderingContextGTK::SetClipRectInPixels(const nsRect& aRect,
+                                                nsClipCombine aCombine,
+                                                PRBool &aClipEmpty)
+{
   PRUint32 cnt = mStateCache.Count();
   nsGraphicsState *state = nsnull;
 
@@ -480,29 +480,24 @@ NS_IMETHODIMP nsRenderingContextGTK::SetClipRect(const nsRect& aRect,
 
   CreateClipRegion();
 
-  nsRect trect = aRect;
-
 #ifdef TRACE_SET_CLIP
   printf("nsRenderingContextGTK::SetClipRect(%s)\n",
          nsClipCombine_to_string(aCombine));
 #endif // TRACE_SET_CLIP
 
-  mTranMatrix->TransformCoord(&trect.x, &trect.y,
-                           &trect.width, &trect.height);
-
   switch(aCombine)
   {
     case nsClipCombine_kIntersect:
-      mClipRegion->Intersect(trect.x,trect.y,trect.width,trect.height);
+      mClipRegion->Intersect(aRect.x,aRect.y,aRect.width,aRect.height);
       break;
     case nsClipCombine_kUnion:
-      mClipRegion->Union(trect.x,trect.y,trect.width,trect.height);
+      mClipRegion->Union(aRect.x,aRect.y,aRect.width,aRect.height);
       break;
     case nsClipCombine_kSubtract:
-      mClipRegion->Subtract(trect.x,trect.y,trect.width,trect.height);
+      mClipRegion->Subtract(aRect.x,aRect.y,aRect.width,aRect.height);
       break;
     case nsClipCombine_kReplace:
-      mClipRegion->SetTo(trect.x,trect.y,trect.width,trect.height);
+      mClipRegion->SetTo(aRect.x,aRect.y,aRect.width,aRect.height);
       break;
   }
 #if 0
@@ -512,8 +507,6 @@ NS_IMETHODIMP nsRenderingContextGTK::SetClipRect(const nsRect& aRect,
   SetColor(color);
 #endif
   aClipEmpty = mClipRegion->IsEmpty();
-
-  return NS_OK;
 }
 
 void nsRenderingContextGTK::UpdateGC()
@@ -792,8 +785,12 @@ NS_IMETHODIMP nsRenderingContextGTK::CreateDrawingSurface(const nsRect &aBounds,
   if (surf)
   {
     NS_ADDREF(surf);
+    PushState();
+    mClipRegion = nsnull;
     UpdateGC();
-    rv = surf->Init(mGC, aBounds.width, aBounds.height, aSurfFlags);    
+    rv = surf->Init(mGC, aBounds.width, aBounds.height, aSurfFlags);
+    PRBool empty;
+    PopState(empty);
   } else {
     rv = NS_ERROR_FAILURE;
   }
@@ -876,7 +873,7 @@ NS_IMETHODIMP nsRenderingContextGTK::DrawPolyline(const nsPoint aPoints[], PRInt
   g_return_val_if_fail(mSurface != NULL, NS_ERROR_FAILURE);
 
   GdkPoint *pts = new GdkPoint[aNumPoints];
-	for (i = 0; i < aNumPoints; i++)
+  for (i = 0; i < aNumPoints; i++)
   {
     nsPoint p = aPoints[i];
     mTranMatrix->TransformCoord(&p.x,&p.y);
@@ -1022,13 +1019,13 @@ NS_IMETHODIMP nsRenderingContextGTK::DrawPolygon(const nsPoint aPoints[], PRInt3
   g_return_val_if_fail(mSurface != NULL, NS_ERROR_FAILURE);
 
   GdkPoint *pts = new GdkPoint[aNumPoints];
-	for (PRInt32 i = 0; i < aNumPoints; i++)
+  for (PRInt32 i = 0; i < aNumPoints; i++)
   {
     nsPoint p = aPoints[i];
-		mTranMatrix->TransformCoord(&p.x,&p.y);
-		pts[i].x = p.x;
+    mTranMatrix->TransformCoord(&p.x,&p.y);
+    pts[i].x = p.x;
     pts[i].y = p.y;
-	}
+  }
 
   UpdateGC();
 
@@ -1045,13 +1042,13 @@ NS_IMETHODIMP nsRenderingContextGTK::FillPolygon(const nsPoint aPoints[], PRInt3
   g_return_val_if_fail(mSurface != NULL, NS_ERROR_FAILURE);
 
   GdkPoint *pts = new GdkPoint[aNumPoints];
-	for (PRInt32 i = 0; i < aNumPoints; i++)
+  for (PRInt32 i = 0; i < aNumPoints; i++)
   {
     nsPoint p = aPoints[i];
-		mTranMatrix->TransformCoord(&p.x,&p.y);
-		pts[i].x = p.x;
+    mTranMatrix->TransformCoord(&p.x,&p.y);
+    pts[i].x = p.x;
     pts[i].y = p.y;
-	}
+  }
 
   UpdateGC();
 
