@@ -48,8 +48,6 @@ NS_IMPL_ISUPPORTS1(nsGraphicsImpl, nsIGraphics)
 nsGraphicsImpl::nsGraphicsImpl(nsIRenderingContext* aRenderer)
 	:	mRenderer(aRenderer)
 {
-	NS_INIT_ISUPPORTS();
-	
 	// hack:  back out the coordinate transformation to use pixels.
 	nsCOMPtr<nsIDeviceContext> dc;
 	mRenderer->GetDeviceContext(*getter_AddRefs(dc));
@@ -152,7 +150,7 @@ NS_IMETHODIMP nsGraphicsImpl::SetFont(const PRUnichar *name, nscoord size)
 	size = NS_STATIC_CAST(nscoord, size * mDev2App);
 	nsAutoString nameString(name);
 	nsFont font(nameString, NS_FONT_STYLE_NORMAL, NS_FONT_VARIANT_NORMAL, NS_FONT_WEIGHT_NORMAL, NS_FONT_DECORATION_NONE, size);
-	return mRenderer->SetFont(font);
+	return mRenderer->SetFont(font, nsnull);
 }
 
 NS_IMETHODIMP nsGraphicsImpl::Gsave(void)
