@@ -40,6 +40,7 @@
 #include "nsCoord.h"
 #include "nsUnitConversion.h"
 #include "nsReadableUtils.h"
+#include "nsIPrintSession.h"
 
 NS_IMPL_ISUPPORTS1(nsPrintSettings, nsIPrintSettings)
 
@@ -73,9 +74,10 @@ nsPrintSettings::nsPrintSettings() :
   mPrintInColor(PR_TRUE),
   mOrientation(kPortraitOrientation),
   mNumCopies(1),
-  mPrintToFile(PR_FALSE)
+  mPrintToFile(PR_FALSE),
+  mIsInitedFromPrinter(PR_FALSE),
+  mIsInitedFromPrefs(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
 
   /* member initializers and constructor code */
   nscoord halfInch = NS_INCHES_TO_TWIPS(0.5);
@@ -95,40 +97,9 @@ nsPrintSettings::nsPrintSettings() :
  *  See documentation in nsPrintSettingsImpl.h
  *	@update 6/21/00 dwc
  */
-nsPrintSettings::nsPrintSettings(const nsPrintSettings* aPS) :
-  mPrintOptions(aPS->mPrintOptions),
-  mPrintRange(aPS->mPrintRange),
-  mStartPageNum(aPS->mStartPageNum),
-  mEndPageNum(aPS->mEndPageNum),
-  mScaling(aPS->mScaling),
-  mPrintBGColors(aPS->mPrintBGColors),
-  mPrintBGImages(aPS->mPrintBGImages),
-  mPrintFrameTypeUsage(aPS->mPrintFrameTypeUsage),
-  mPrintFrameType(aPS->mPrintFrameType),
-  mHowToEnableFrameUI(aPS->mHowToEnableFrameUI),
-  mIsCancelled(aPS->mIsCancelled),
-  mPrintSilent(aPS->mPrintSilent),
-	mPrintPreview(aPS->mPrintPreview),
-  mShrinkToFit(aPS->mShrinkToFit),
-  mPrintPageDelay(aPS->mPrintPageDelay),
-  mPaperData(aPS->mPaperData),
-  mPaperSizeType(aPS->mPaperSizeType),
-  mPaperWidth(aPS->mPaperWidth),
-  mPaperHeight(aPS->mPaperHeight),
-  mPaperSizeUnit(aPS->mPaperSizeUnit),
-  mPrintReversed(aPS->mPrintReversed),
-  mPrintInColor(aPS->mPrintInColor),
-  mOrientation(aPS->mOrientation),
-  mNumCopies(aPS->mNumCopies),
-  mPrintToFile(aPS->mPrintToFile),
-  mMargin(aPS->mMargin)
+nsPrintSettings::nsPrintSettings(const nsPrintSettings& aPS)
 {
-  NS_INIT_ISUPPORTS();
-
-  for (PRInt32 i=0;i<3;i++) {
-    mHeaderStrs[i] = aPS->mHeaderStrs[i];
-    mFooterStrs[i] = aPS->mFooterStrs[i];
-  }
+  *this = aPS;
 }
 
 /** ---------------------------------------------------
@@ -139,6 +110,34 @@ nsPrintSettings::~nsPrintSettings()
 {
 }
 
+/* [noscript] attribute nsIPrintSession printSession; */
+NS_IMETHODIMP nsPrintSettings::GetPrintSession(nsIPrintSession **aPrintSession)
+{
+  NS_ENSURE_ARG_POINTER(aPrintSession);
+  *aPrintSession = nsnull;
+  
+  nsCOMPtr<nsIPrintSession> session = do_QueryReferent(mSession);
+  if (!session)
+    return NS_ERROR_NOT_INITIALIZED;
+  *aPrintSession = session;
+  NS_ADDREF(*aPrintSession);
+  return NS_OK;
+}
+NS_IMETHODIMP nsPrintSettings::SetPrintSession(nsIPrintSession *aPrintSession)
+{
+  // Clearing it by passing NULL is not allowed. That's why we
+  // use a weak ref so that it doesn't have to be cleared.
+  NS_ENSURE_ARG(aPrintSession);
+  
+  mSession = getter_AddRefs(NS_GetWeakReference(aPrintSession));
+  if (!mSession) {
+    // This may happen if the implementation of this object does
+    // not support weak references - programmer error.
+    NS_ERROR("Could not get a weak reference from aPrintSession");
+    return NS_ERROR_FAILURE;
+  }
+  return NS_OK;
+}
 
 /* attribute long startPageRange; */
 NS_IMETHODIMP nsPrintSettings::GetStartPageRange(PRInt32 *aStartPageRange)
@@ -208,7 +207,7 @@ NS_IMETHODIMP nsPrintSettings::SetPaperSize(PRInt32 aPaperSize)
 /* attribute short orientation; */
 NS_IMETHODIMP nsPrintSettings::GetOrientation(PRInt32 *aOrientation)
 {
-  //NS_ENSURE_ARG_POINTER(aOrientation);
+  NS_ENSURE_ARG_POINTER(aOrientation);
   *aOrientation = mOrientation;
   return NS_OK;
 }
@@ -221,20 +220,25 @@ NS_IMETHODIMP nsPrintSettings::SetOrientation(PRInt32 aOrientation)
 /* attribute wstring printer; */
 NS_IMETHODIMP nsPrintSettings::GetPrinterName(PRUnichar * *aPrinter)
 {
-   //NS_ENSURE_ARG_POINTER(aPrinter);
+   NS_ENSURE_ARG_POINTER(aPrinter);
    *aPrinter = ToNewUnicode(mPrinter);
    return NS_OK;
 }
 NS_IMETHODIMP nsPrintSettings::SetPrinterName(const PRUnichar * aPrinter)
 {
-   mPrinter = aPrinter;
-   return NS_OK;
+  if (!aPrinter || !mPrinter.Equals(aPrinter)) {
+    mIsInitedFromPrinter = PR_FALSE;
+    mIsInitedFromPrefs   = PR_FALSE;
+  }
+
+  mPrinter.Assign(aPrinter);
+  return NS_OK;
 }
 
 /* attribute long numCopies; */
 NS_IMETHODIMP nsPrintSettings::GetNumCopies(PRInt32 *aNumCopies)
 {
-  //NS_ENSURE_ARG_POINTER(aNumCopies);
+  NS_ENSURE_ARG_POINTER(aNumCopies);
   *aNumCopies = mNumCopies;
   return NS_OK;
 }
@@ -253,7 +257,11 @@ NS_IMETHODIMP nsPrintSettings::GetPrintCommand(PRUnichar * *aPrintCommand)
 }
 NS_IMETHODIMP nsPrintSettings::SetPrintCommand(const PRUnichar * aPrintCommand)
 {
-  mPrintCommand = aPrintCommand;
+  if (aPrintCommand) {
+    mPrintCommand = aPrintCommand;
+  } else {
+    mPrintCommand.SetLength(0);
+  }
   return NS_OK;
 }
 
@@ -279,7 +287,11 @@ NS_IMETHODIMP nsPrintSettings::GetToFileName(PRUnichar * *aToFileName)
 }
 NS_IMETHODIMP nsPrintSettings::SetToFileName(const PRUnichar * aToFileName)
 {
-  mToFileName = aToFileName;
+  if (aToFileName) {
+    mToFileName = aToFileName;
+  } else {
+    mToFileName.SetLength(0);
+  }
   return NS_OK;
 }
 
@@ -295,6 +307,32 @@ NS_IMETHODIMP nsPrintSettings::SetPrintPageDelay(PRInt32 aPrintPageDelay)
   return NS_OK;
 }
 
+/* attribute boolean isInitializedFromPrinter; */
+NS_IMETHODIMP nsPrintSettings::GetIsInitializedFromPrinter(PRBool *aIsInitializedFromPrinter)
+{
+  NS_ENSURE_ARG_POINTER(aIsInitializedFromPrinter);
+  *aIsInitializedFromPrinter = (PRBool)mIsInitedFromPrinter;
+  return NS_OK;
+}
+NS_IMETHODIMP nsPrintSettings::SetIsInitializedFromPrinter(PRBool aIsInitializedFromPrinter)
+{
+  mIsInitedFromPrinter = (PRPackedBool)aIsInitializedFromPrinter;
+  return NS_OK;
+}
+
+/* attribute boolean isInitializedFromPrefs; */
+NS_IMETHODIMP nsPrintSettings::GetIsInitializedFromPrefs(PRBool *aInitializedFromPrefs)
+{
+  NS_ENSURE_ARG_POINTER(aInitializedFromPrefs);
+  *aInitializedFromPrefs = (PRBool)mIsInitedFromPrefs;
+  return NS_OK;
+}
+NS_IMETHODIMP nsPrintSettings::SetIsInitializedFromPrefs(PRBool aInitializedFromPrefs)
+{
+  mIsInitedFromPrefs = (PRPackedBool)aInitializedFromPrefs;
+  return NS_OK;
+}
+
 /* attribute double marginTop; */
 NS_IMETHODIMP nsPrintSettings::GetMarginTop(double *aMarginTop)
 {
@@ -413,8 +451,11 @@ NS_IMETHODIMP nsPrintSettings::GetTitle(PRUnichar * *aTitle)
 }
 NS_IMETHODIMP nsPrintSettings::SetTitle(const PRUnichar * aTitle)
 {
-  NS_ENSURE_ARG_POINTER(aTitle);
-  mTitle = aTitle;
+  if (aTitle) {
+    mTitle = aTitle;
+  } else {
+    mTitle.SetLength(0);
+  }
   return NS_OK;
 }
 
@@ -431,8 +472,11 @@ NS_IMETHODIMP nsPrintSettings::GetDocURL(PRUnichar * *aDocURL)
 }
 NS_IMETHODIMP nsPrintSettings::SetDocURL(const PRUnichar * aDocURL)
 {
-  NS_ENSURE_ARG_POINTER(aDocURL);
-  mURL = aDocURL;
+  if (aDocURL) {
+    mURL = aDocURL;
+  } else {
+    mURL.SetLength(0);
+  }
   return NS_OK;
 }
 
@@ -659,8 +703,11 @@ NS_IMETHODIMP nsPrintSettings::GetPaperName(PRUnichar * *aPaperName)
 }
 NS_IMETHODIMP nsPrintSettings::SetPaperName(const PRUnichar * aPaperName)
 {
-  NS_ENSURE_ARG_POINTER(aPaperName);
-  mPaperName = aPaperName;
+  if (aPaperName) {
+    mPaperName = aPaperName;
+  } else {
+    mPaperName.SetLength(0);
+  }
   return NS_OK;
 }
 
@@ -796,9 +843,9 @@ nsPrintSettings::GetPageSizeInTwips(PRInt32 *aWidth, PRInt32 *aHeight)
 }
 
 nsresult 
-nsPrintSettings::CloneObj(nsIPrintSettings **_retval)
+nsPrintSettings::_Clone(nsIPrintSettings **_retval)
 {
-  nsPrintSettings* printSettings = new nsPrintSettings(this);
+  nsPrintSettings* printSettings = new nsPrintSettings(*this);
   return printSettings->QueryInterface(NS_GET_IID(nsIPrintSettings), (void**)_retval); // ref counts
 }
 
@@ -806,5 +853,72 @@ nsPrintSettings::CloneObj(nsIPrintSettings **_retval)
 NS_IMETHODIMP 
 nsPrintSettings::Clone(nsIPrintSettings **_retval)
 {
-  return CloneObj(_retval);
+  NS_ENSURE_ARG_POINTER(_retval);
+  return _Clone(_retval);
 }
+
+/* void assign (in nsIPrintSettings aPS); */
+nsresult 
+nsPrintSettings::_Assign(nsIPrintSettings *aPS)
+{
+  nsPrintSettings *ps = NS_STATIC_CAST(nsPrintSettings*, aPS);
+  *this = *ps;
+  return NS_OK;
+}
+
+/* void assign (in nsIPrintSettings aPS); */
+NS_IMETHODIMP 
+nsPrintSettings::Assign(nsIPrintSettings *aPS)
+{
+  NS_ENSURE_ARG(aPS);
+  return _Assign(aPS);
+}
+
+//-------------------------------------------
+nsPrintSettings& nsPrintSettings::operator=(const nsPrintSettings& rhs)
+{
+  if (this == &rhs) {
+    return *this;
+  }
+
+  mStartPageNum        = rhs.mStartPageNum;
+  mEndPageNum          = rhs.mEndPageNum;
+  mMargin              = rhs.mMargin;
+  mScaling             = rhs.mScaling;
+  mPrintBGColors       = rhs.mPrintBGColors;
+  mPrintBGImages       = rhs.mPrintBGImages;
+  mPrintRange          = rhs.mPrintRange;
+  mTitle               = rhs.mTitle;
+  mURL                 = rhs.mURL;
+  mHowToEnableFrameUI  = rhs.mHowToEnableFrameUI;
+  mIsCancelled         = rhs.mIsCancelled;
+  mPrintFrameTypeUsage = rhs.mPrintFrameTypeUsage;
+  mPrintFrameType      = rhs.mPrintFrameType;
+  mPrintSilent         = rhs.mPrintSilent;
+  mShrinkToFit         = rhs.mShrinkToFit;
+  mShowPrintProgress   = rhs.mShowPrintProgress;
+  mPaperName           = rhs.mPaperName;
+  mPaperSizeType       = rhs.mPaperSizeType;
+  mPaperData           = rhs.mPaperData;
+  mPaperWidth          = rhs.mPaperWidth;
+  mPaperHeight         = rhs.mPaperHeight;
+  mPaperSizeUnit       = rhs.mPaperSizeUnit;
+  mPrintReversed       = rhs.mPrintReversed;
+  mPrintInColor        = rhs.mPrintInColor;
+  mPaperSize           = rhs.mPaperSize;
+  mOrientation         = rhs.mOrientation;
+  mPrintCommand        = rhs.mPrintCommand;
+  mNumCopies           = rhs.mNumCopies;
+  mPrinter             = rhs.mPrinter;
+  mPrintToFile         = rhs.mPrintToFile;
+  mToFileName          = rhs.mToFileName;
+  mPrintPageDelay      = rhs.mPrintPageDelay;
+
+  for (PRInt32 i=0;i<NUM_HEAD_FOOT;i++) {
+    mHeaderStrs[i] = rhs.mHeaderStrs[i];
+    mFooterStrs[i] = rhs.mFooterStrs[i];
+  }
+
+  return *this;
+}
+
