@@ -599,14 +599,13 @@ nsFontMetricsOS2::SetFontHandle( HPS aPS, nsFontOS2* aFont )
    // Encoding:
    //  There doesn't seem to be any encoding stuff yet, so guess.
    //  (XXX unicode hack; use same codepage as converter!)
-  const PRUnichar* langGroup = nsnull;
-  mLangGroup->GetUnicode(&langGroup);
-  nsCAutoString name(NS_LossyConvertUCS2toASCII(langGroup).get());
+  const char* langGroup;
+  mLangGroup->GetUTF8String(&langGroup);
   for (int j=0; j < eCharset_COUNT; j++ )
   {
-    if (name.get()[0] == gCharsetInfo[j].mLangGroup[0])
+    if (langGroup[0] == gCharsetInfo[j].mLangGroup[0])
     {
-      if (!strcmp(name.get(), gCharsetInfo[j].mLangGroup))
+      if (!strcmp(langGroup, gCharsetInfo[j].mLangGroup))
       {
         mConvertCodePage = gCharsetInfo[j].mCodePage;
         break;
@@ -812,6 +811,7 @@ nsFontMetricsOS2::FindUserDefinedFont( HPS aPS )
 {
   if (mIsUserDefined) {
     nsFontOS2* font = LoadFont( aPS, &mUserDefined );
+    mIsUserDefined = PR_FALSE;
     if( font )
       return font;
   }
@@ -1156,7 +1156,7 @@ nsFontMetricsOS2::GetVectorSubstitute( HPS aPS, const char* aFamilyname,
   PRBool isBold = mFont.weight > NS_FONT_WEIGHT_NORMAL;
   PRBool isItalic = (mFont.style & (NS_FONT_STYLE_ITALIC | NS_FONT_STYLE_OBLIQUE));
 
-  if( strcmpi( aFamilyname, "Tms Rmn" ) == 0 )
+  if( stricmp( aFamilyname, "Tms Rmn" ) == 0 )
   {
     if( !isBold && !isItalic )
       strcpy( alias, "Times New Roman" );
@@ -1171,7 +1171,7 @@ nsFontMetricsOS2::GetVectorSubstitute( HPS aPS, const char* aFamilyname,
     return PR_TRUE;
   }
 
-  if( strcmpi( aFamilyname, "Helv" ) == 0 )
+  if( stricmp( aFamilyname, "Helv" ) == 0 )
   {
     if( !isBold && !isItalic )
       strcpy( alias, "Helvetica" );
@@ -1189,7 +1189,7 @@ nsFontMetricsOS2::GetVectorSubstitute( HPS aPS, const char* aFamilyname,
    // When printing, substitute vector fonts for these common bitmap fonts
   if( !mDeviceContext->SupportsRasterFonts() )
   {
-    if( strcmpi( aFamilyname, "System Proportional" ) == 0 )
+    if( stricmp( aFamilyname, "System Proportional" ) == 0 )
     {
       if( !isBold && !isItalic )
         strcpy( alias, "Helvetica" );
@@ -1204,8 +1204,8 @@ nsFontMetricsOS2::GetVectorSubstitute( HPS aPS, const char* aFamilyname,
       return PR_TRUE;
     }
 
-    if( strcmpi( aFamilyname, "System Monospaced" ) == 0 ||
-        strcmpi( aFamilyname, "System VIO" ) == 0 )
+    if( stricmp( aFamilyname, "System Monospaced" ) == 0 ||
+        stricmp( aFamilyname, "System VIO" ) == 0 )
     {
       if( !isBold && !isItalic )
         strcpy( alias, "Courier" );
@@ -1844,12 +1844,12 @@ nsFontMetricsOS2::InitializeGlobalFonts()
   {
      // The discrepencies between the Courier bitmap and outline fonts are
      // too much to deal with, so we only use the outline font
-    if( strcmpi(pFontMetrics[i].szFamilyname, "Courier") == 0 &&
+    if( stricmp(pFontMetrics[i].szFamilyname, "Courier") == 0 &&
         !(pFontMetrics[i].fsDefn & FM_DEFN_OUTLINE))
       continue;
 
-    if (strcmpi(pFontMetrics[i].szFamilyname, "Roman") == 0 ||
-        strcmpi(pFontMetrics[i].szFamilyname, "Swiss") == 0)
+    if (stricmp(pFontMetrics[i].szFamilyname, "Roman") == 0 ||
+        stricmp(pFontMetrics[i].szFamilyname, "Swiss") == 0)
       continue;
 
     nsGlobalFont* font = new nsGlobalFont;
@@ -2092,6 +2092,19 @@ nsFontEnumeratorOS2::HaveFontFor(const char* aLangGroup, PRBool* aResult)
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+NS_IMETHODIMP
+nsFontEnumeratorOS2::GetDefaultFont(const char *aLangGroup, 
+  const char *aGeneric, PRUnichar **aResult)
+{
+  // aLangGroup=null or ""  means any (i.e., don't care)
+  // aGeneric=null or ""  means any (i.e, don't care)
+
+  NS_ENSURE_ARG_POINTER(aResult);
+  *aResult = nsnull;
+
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsFontEnumeratorOS2::UpdateFontList(PRBool *updateFontList)
 {
