@@ -163,18 +163,12 @@ NS_IMETHODIMP nsFontMetricsPh::Init ( const nsFont& aFont, nsIAtom* aLangGroup,
 		str = strdup("serif");
 	}
 	
-	const PRUnichar *uc;
-	aLangGroup->GetUnicode( &uc );
-	nsString language( uc );
-	char *cstring = ToNewCString(language);
+	const char *cstring;
+	aLangGroup->GetUTF8String( &uc );
 	
 	char prop[256];
 	sprintf( prop, "font.name.%s.%s", str, cstring );
 
-	/* Free cstring.  */
-	if (cstring)
-		free (cstring);
-
 	char *font_default = NULL;
 	nsIPref* prefs = nsnull;
 	nsServiceManager::GetService(kPrefCID, NS_GET_IID(nsIPref), (nsISupports**) &prefs);
