@@ -37,6 +37,7 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "gfx-config.h"
 #include "nsFontMetricsPS.h"
 #include "nsDeviceContextPS.h"
 #include "nsRenderingContextPS.h"
@@ -615,7 +616,7 @@ nsFontPSAFM::RealizeFont(nsFontMetricsPS* aFontMetrics, float dev2app)
   aFontMetrics->SetXHeight(xHeight);
   aFontMetrics->SetSuperscriptOffset(xHeight);
   aFontMetrics->SetSubscriptOffset(xHeight);
-  aFontMetrics->SetStrikeout((nscoord)(xHeight / TWIPS_PER_POINT_FLOAT), onePixel);
+  aFontMetrics->SetStrikeout((nscoord)(xHeight / 2), onePixel);
 
   offset = NSFloatPointsToTwips(fontSize * mAFMInfo->mPSFontInfo->mUnderlinePosition) / 1000.0f;
   aFontMetrics->SetUnderline(NSToCoordRound(offset), onePixel);
@@ -866,13 +867,13 @@ nsFontPSFreeType::FindFont(PRUnichar aChar, const nsFont& aFont,
       aFontMetrics->GetLangGroup(getter_AddRefs(lang));
       if (!lang)
         lang = NS_NewAtom("x-western");
-      const PRUnichar *langStr;
-      lang->GetUnicode(&langStr);
+      const char *langStr;
+      lang->GetUTF8String(&langStr);
       if (langStr)
-        fpi.lang.AppendWithConversion(langStr);
-      gUsersLocale->GetUnicode(&langStr);
+        fpi.lang.Append(langStr);
+      gUsersLocale->GetUTF8String(&langStr);
       if (langStr)
-        locale.AppendWithConversion(langStr);
+        locale.Append(langStr);
       if (NS_IS_BOLD(fpi.nsfont->weight))
         fpi.weight = nsIFontCatalogService::kFCWeightBold;
       else
