@@ -108,23 +108,6 @@ public:
                       PRInt32 aSXOffset, PRInt32 aSYOffset,
                       const nsRect &aTileRect);
 
-  /** 
-   * Progressivly double the bitmap size as we blit.. very fast way to tile
-   * @param aSurface  the surface to blit to
-   * @param aDestBufferWidth   Width of buffer
-   * @param aDestBufferHeight  Height of buffer
-   * @param aScaledTileWidth   Width of tile
-   * @param aScaledTileHeight  Height of tile
-   * @param aX0,aY0,aX1,aY1    Coordinates of screen to blit to
-   * @return if TRUE, no errors
-  */
-
-  PRBool ProgressiveDoubleBlit(nsDrawingSurface aSurface,
-                              PRInt32 aDestBufferWidth, PRInt32 aDestBufferHeight,
-                              PRInt32 aScaledTileWidth,PRInt32 aScaledTileHeight,
-                              PRInt32 aX0,PRInt32 aY0,
-                              PRInt32 aX1,PRInt32 aY1);
-
    /** 
    * Return the header size of the Device Independent Bitmap(DIB).
    * @return size of header in bytes
@@ -179,8 +162,6 @@ private:
    */
   void CleanUpDDB();
 
-  void CleanUpDIBSection();
-
   void CreateImageWithAlphaBits(HDC TheHDC);
 
   /** 
@@ -215,19 +196,37 @@ private:
   nsresult PrintDDB(nsDrawingSurface aSurface,PRInt32 aX,PRInt32 aY,PRInt32 aWidth,PRInt32 aHeight,PRUint32  aROP);
 
 
-  
-  /** ---------------------------------------------------
-   *  build a tile area by doubling the image until it reaches its limits
-   *  @param aTheHDC -- HDC to render to
-   *  @param aSrcRect -- Rectangle we are build with the image
-   *  @param aHeight -- height of the tile
-   *  @param aWidth -- width of the tile
+  /** 
+   * Progressively double the bitmap size as we blit.. very fast way to tile
+   * @return if TRUE, no errors
    */
-  void  BuildTile(HDC aTheHDC, nsRect &aSrcRect,
-                  PRInt32 aWidth, PRInt32 aHeight, PRInt32 aCopyMode);
-
-
+  PRBool ProgressiveDoubleBlit(nsDrawingSurface aSurface,
+                               PRInt32 aSXOffset, PRInt32 aSYOffset,
+                               nsRect aDestRect);
 
+  /**
+   * Draw Image and Mask (if one exists) to another set of DCs
+   * @param aDstDC     Where aSrcDC will be drawn to
+   * @param aDstMaskDC Where aMaskDC will be drawn to.  If same as aDstDC and
+                       there's a aSrcMaskDC, blending occurs. 
+   * @param aDstX      x-pos of where to start drawing to
+   * @param aDstY      y-pos of where to start drawing to
+   * @param aWidth     Width to copy from src to dst
+   * @param aHeight    Height to copy from src to dst
+   * @param aSrcDC     Source Image
+   * @param aSrcMaskDC Source Mask. If nsnull, aDstMaskDC will be ignored
+   * @param aSrcX      copy src starting at this x position
+   * @param aSrcY      copy src starting at this y position
+   * @param aUseAlphaBlend  When True, aDstSrc is a 32-bit DC storing alpha
+                            information in the 4th byte, so use AlphaBlend API
+                            instead of BitBlt.
+                            When False, BitBlt is used.
+   */
+  void BlitImage(HDC aDstDC, HDC aDstMaskDC, PRInt32 aDstX, PRInt32 aDstY,
+                 PRInt32 aWidth, PRInt32 aHeight,
+                 HDC aSrcDC, HDC aSrcMaskDC, PRInt32 aSrcX, PRInt32 aSrcY,
+                 PRBool aUseAlphaBlend);
+  
   /** 
    * Get an index in the palette that matches as closly as possible the passed in RGB colors
    * @update dc - 4/20/2000
@@ -239,20 +238,20 @@ private:
   PRUint8 PaletteMatch(PRUint8 r, PRUint8 g, PRUint8 b);
 
   PRPackedBool        mInitialized;
+  PRPackedBool        mIsOptimized;       // Did we convert our DIB to a HBITMAP
+  PRPackedBool        mIsLocked;          // variable to keep track of the locking
+  PRPackedBool        mDIBTemp;           // boolean to let us know if DIB was created as temp
   PRInt8              mNumBytesPixel;     // number of bytes per pixel
   PRInt16             mNumPaletteColors;  // Number of colors in the pallete 256 
   PRInt32             mSizeImage;         // number of bytes
   PRInt32             mRowBytes;          // number of bytes per row
   PRUint8*            mImageBits;         // starting address of DIB bits
-  PRBool              mIsOptimized;       // Did we convert our DIB to a HBITMAP
   nsColorMap*         mColorMap;          // Redundant with mColorTable, but necessary
 
-  PRInt32             mDecodedX1;			    //Keeps track of what part of image
-  PRInt32             mDecodedY1;			    // has been decoded.
+  PRInt32             mDecodedX1;         //Keeps track of what part of image
+  PRInt32             mDecodedY1;         // has been decoded.
   PRInt32             mDecodedX2; 
   PRInt32             mDecodedY2; 
-  PRBool              mIsLocked;			    // variable to keep track of the locking
-  PRBool              mDIBTemp;           // boolean to let us know if DIB was created as temp
 
   // alpha layer members
   PRUint8             *mAlphaBits;        // alpha layer if we made one
@@ -260,7 +259,6 @@ private:
   PRInt16             mARowBytes;         // number of bytes per row in the image for tha alpha
   PRInt8              mImageCache;        // place to save off the old image for fast animation
   HBITMAP             mHBitmap;           // the GDI bitmaps
-  HBITMAP             mDIBSection;
   LPBITMAPINFOHEADER  mBHead;             // BITMAPINFOHEADER
 
   static ALPHABLENDPROC gAlphaBlend;      // AlphaBlend function pointer
