@@ -1814,7 +1814,8 @@ do_BreakGetTextDimensions(const nsFontSwitch* aFontSwitch,
       // Find the nearest place to break that is less than or equal to
       // the estimated break offset
       breakIndex = data->mPrevBreakState_BreakIndex;
-      while (data->mBreaks[breakIndex + 1] <= estimatedBreakOffset) {
+      while (breakIndex + 1 < data->mNumBreaks &&
+             data->mBreaks[breakIndex + 1] <= estimatedBreakOffset) {
         ++breakIndex;
       }
 
@@ -2979,7 +2980,11 @@ nsRenderingContextWin::InitBidiInfo()
     const PRUnichar one     = 0x0031;
 
     int distanceArray[2];
+#ifdef __MINGW32__
+    UINT glyphArray[2];
+#else
     PRUnichar glyphArray[2];
+#endif
     PRUnichar outStr[] = {0, 0};
 
     GCP_RESULTSW gcpResult;
