@@ -82,8 +82,8 @@ nsImageXlib::nsImageXlib()
 , mIsSpacer(PR_TRUE)
 , mGC(nsnull)
 , mPendingUpdate(PR_FALSE)
-, mDecodedX1(0)
-, mDecodedY1(0)
+, mDecodedX1(PR_INT32_MAX)
+, mDecodedY1(PR_INT32_MAX)
 , mDecodedX2(0)
 , mDecodedY2(0)
 {
@@ -257,6 +257,9 @@ void nsImageXlib::ImageUpdated(nsIDeviceContext *aContext,
   mPendingUpdate = PR_TRUE;
   mUpdateRegion.Or(mUpdateRegion, *aUpdateRect);
 
+  mDecodedX1 = PR_MIN(mDecodedX1, aUpdateRect->x);
+  mDecodedY1 = PR_MIN(mDecodedY1, aUpdateRect->y);
+
   if (aUpdateRect->YMost() > mDecodedY2)
     mDecodedY2 = aUpdateRect->YMost();
   if (aUpdateRect->XMost() > mDecodedX2)
@@ -374,7 +377,7 @@ void nsImageXlib::UpdateCachedImage()
     }
   }
 
-  mUpdateRegion.Empty();
+  mUpdateRegion.SetEmpty();
   mPendingUpdate = PR_FALSE;
   mFlags = nsImageUpdateFlags_kBitsChanged; // this should be 0'd out by Draw()
 }
@@ -397,6 +400,9 @@ nsImageXlib::DrawScaled(nsIRenderingContext &aContext,
   if (0 == aSWidth || 0 == aDWidth || 0 == aSHeight || 0 == aDHeight)
     return NS_OK;
 
+  if (mDecodedX2 < mDecodedX1 || mDecodedY2 < mDecodedY1)
+    return NS_OK;
+
   // limit the size of the blit to the amount of the image read in
   if (aSX + aSWidth > mDecodedX2) {
     aDWidth -= ((aSX + aSWidth - mDecodedX2)*origDWidth)/origSWidth;
@@ -567,6 +573,9 @@ nsImageXlib::Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
   if ((mAlphaDepth == 1) && mIsSpacer)
     return NS_OK;
 
+  if (mDecodedX2 < mDecodedX1 || mDecodedY2 < mDecodedY1)
+    return NS_OK;
+
   if (aSWidth != aDWidth || aSHeight != aDHeight) {
     return DrawScaled(aContext, aSurface, aSX, aSY, aSWidth, aSHeight,
                       aDX, aDY, aDWidth, aDHeight);
@@ -1211,6 +1220,9 @@ nsImageXlib::Draw(nsIRenderingContext &aContext,
   if (aSurface == nsnull)
     return NS_ERROR_FAILURE;
 
+  if (mDecodedX2 < mDecodedX1 || mDecodedY2 < mDecodedY1)
+    return NS_OK;
+
   if ((mAlphaDepth == 8) && mAlphaValid) {
     DrawComposited(aContext, aSurface,
         0, 0, aWidth, aHeight,
@@ -1331,7 +1343,10 @@ NS_IMETHODIMP nsImageXlib::DrawTile(nsIRenderingContext &aContext,
   if (aTileRect.width <= 0 || aTileRect.height <= 0) {
     return NS_OK;
   }
-  
+
+  if (mDecodedX2 < mDecodedX1 || mDecodedY2 < mDecodedY1)
+    return NS_OK;
+
   nsIDrawingSurfaceXlib *drawing = NS_STATIC_CAST(nsIDrawingSurfaceXlib *, aSurface);
 
   PRBool partial = PR_FALSE;
@@ -1376,9 +1391,9 @@ NS_IMETHODIMP nsImageXlib::DrawTile(nsIRenderingContext &aContext,
     // Set up clipping and call Draw().
     PRBool clipState;
     aContext.PushState();
-    aContext.SetClipRect(aTileRect, nsClipCombine_kIntersect,
-                         clipState);
-
+    ((nsRenderingContextXlib&)aContext).SetClipRectInPixels(
+      aTileRect, nsClipCombine_kIntersect, clipState);
+    ((nsRenderingContextXlib&)aContext).UpdateGC();
     for (PRInt32 y = aY0; y < aY1; y += mHeight)
       for (PRInt32 x = aX0; x < aX1; x += mWidth)
         Draw(aContext,aSurface, x, y,
