@@ -47,7 +47,7 @@
 #define strtok_r(s1, s2, x) PL_strtok_r((s1), (s2), (x))
 #endif /* USE_MOZILLA_TYPES */
 
-/* List of tokens which can be used to seperate entries in the 
+/* List of tokens which can be used to separate entries in the 
  * $XPSERVERLIST env var */
 static const char XPServerListSeparators[] = " \t\v\n\r\f";
 
@@ -662,8 +662,9 @@ Bool XpuParseMediumSourceSize( const char *value,
     s++;  
   }
   while(*s);
+  *d = '\0';
     
-  /* seperate medium name from string */
+  /* separate medium name from string */
   d = (char *)search_next_space(name);
   if( !d )
   {
@@ -682,10 +683,16 @@ Bool XpuParseMediumSourceSize( const char *value,
    * the original locale.
    * XXX: This may affect all threads and not only the calling one...
    */
-  cur_locale = setlocale(LC_NUMERIC, NULL);
-  setlocale(LC_NUMERIC, "C"); 
-  num_input_items = sscanf(d, "%s %f %f %f %f", boolbuf, ma1, ma2, ma3, ma4);
-  setlocale(LC_NUMERIC, cur_locale);
+  {
+#define CUR_LOCALE_SIZE 256
+    char cur_locale[CUR_LOCALE_SIZE+1];
+    strncpy(cur_locale, setlocale(LC_NUMERIC, NULL), CUR_LOCALE_SIZE);
+    cur_locale[CUR_LOCALE_SIZE]='\0';
+    setlocale(LC_NUMERIC, "C"); 
+    num_input_items = sscanf(d, "%s %f %f %f %f", boolbuf, ma1, ma2, ma3, ma4);
+    setlocale(LC_NUMERIC, cur_locale);
+#undef CUR_LOCALE_SIZE
+  }
 
   if( num_input_items != 5 )
   {
