@@ -128,9 +128,7 @@ class CStartToken: public CHTMLToken {
 
     virtual PRBool        IsEmpty(void);
     virtual void          SetEmpty(PRBool aValue);
-#ifdef DEBUG
-    virtual void          DebugDumpSource(nsOutputStream& out);
-#endif
+
     virtual const nsAString& GetStringValue();
     virtual void          GetSource(nsString& anOutputString);
     virtual void          AppendSourceTo(nsAString& anOutputString);
@@ -180,9 +178,7 @@ class CEndToken: public CHTMLToken {
     virtual PRInt32     GetTypeID(void);
     virtual const char* GetClassName(void);
     virtual PRInt32     GetTokenType(void);
-#ifdef DEBUG
-    virtual void        DebugDumpSource(nsOutputStream& out);
-#endif
+
     virtual const nsAString& GetStringValue();
     virtual void        GetSource(nsString& anOutputString);
     virtual void        AppendSourceTo(nsAString& anOutputString);
@@ -240,9 +236,7 @@ class CEntityToken : public CHTMLToken {
     virtual nsresult    Consume(PRUnichar aChar,nsScanner& aScanner,PRInt32 aMode);
     static  nsresult    ConsumeEntity(PRUnichar aChar,nsString& aString,nsScanner& aScanner);
     static  PRInt32     TranslateToUnicodeStr(PRInt32 aValue,nsString& aString);
-#ifdef DEBUG
-    virtual  void       DebugDumpSource(nsOutputStream& out);
-#endif
+
     virtual const nsAString& GetStringValue(void);
     virtual void        GetSource(nsString& anOutputString);
     virtual void        AppendSourceTo(nsAString& anOutputString);
@@ -372,15 +366,9 @@ class CAttributeToken: public CHTMLToken {
     virtual void          BindKey(nsScanner* aScanner, nsReadingIterator<PRUnichar>& aStart, nsReadingIterator<PRUnichar>& aEnd);
     virtual const nsAString&     GetValue(void) {return mTextValue;}
     virtual void          SanitizeKey();
-#ifdef DEBUG
-    virtual void          DebugDumpToken(nsOutputStream& out);
-#endif
     virtual const nsAString& GetStringValue(void);
     virtual void          GetSource(nsString& anOutputString);
     virtual void          AppendSourceTo(nsAString& anOutputString);
-#ifdef DEBUG
-    virtual void          DebugDumpSource(nsOutputStream& out);
-#endif
     
     PRPackedBool       mHasEqualWithoutValue;
   protected:
