@@ -38,8 +38,6 @@
  * ***** END LICENSE BLOCK ***** */
        
 //#define ENABLE_CRC              
-//#define RICKG_DEBUG           
-     
           
 #include "nsDebug.h"      
 #include "nsIAtom.h"
@@ -176,14 +174,6 @@ COtherDTD::COtherDTD() : nsIDTD() {
   if(!gElementTable) {
     gElementTable = new CElementTable();
   }
-
-#ifdef  RICKG_DEBUG
-  //DebugDumpContainmentRules2(*this,"c:/temp/DTDRules.new","New COtherDTD Containment Rules");
-  nsHTMLElement::DebugDumpContainment("c:/temp/contain.new","ElementTable Rules");
-  nsHTMLElement::DebugDumpMembership("c:/temp/membership.out");
-  nsHTMLElement::DebugDumpContainType("c:/temp/ctnrules.out");
-#endif
-
 }
 
 /**
@@ -564,7 +554,7 @@ nsresult COtherDTD::HandleToken(CToken* aToken,nsIParser* aParser){
  * This gets called after we've handled a given start tag.
  * It's a generic hook to let us to post processing.
  * @param   aToken contains the tag in question
- * @param   aChildTag is the tag itself.
+ * @param   aTag is the tag itself.
  * @return  status
  */
 nsresult COtherDTD::DidHandleStartTag(nsIParserNode& aNode,eHTMLTags aChildTag){
@@ -628,16 +618,6 @@ nsresult COtherDTD::DidHandleStartTag(nsIParserNode& aNode,eHTMLTags aChildTag){
   return result;
 } 
  
-
-#ifdef  RICKG_DEBUG
-void WriteTokenToLog(CToken* aToken) {
-
-  static nsFileSpec fileSpec("c:\\temp\\tokenlog.html");
-  static nsOutputFileStream outputStream(fileSpec);
-  aToken->DebugDumpSource(outputStream); //write token without close bracket...
-}
-#endif
- 
 /**
  * This gets called before we've handled a given start tag.
  * It's a generic hook to let us do pre processing.
@@ -694,9 +674,6 @@ nsresult COtherDTD::WillHandleStartTag(CToken* aToken,eHTMLTags aTag,nsIParserNo
  *  @return  PR_TRUE if all went well; PR_FALSE if error occured
  */      
 nsresult COtherDTD::HandleStartToken(CToken* aToken) {  
-  #ifdef  RICKG_DEBUG
-    WriteTokenToLog(aToken);
-  #endif
 
   //Begin by gathering up attributes...  
  
@@ -767,10 +744,6 @@ nsresult COtherDTD::HandleEndToken(CToken* aToken) {
   nsresult    result=NS_OK;
   eHTMLTags   theChildTag=(eHTMLTags)aToken->GetTypeID();
  
-  #ifdef  RICKG_DEBUG    
-    WriteTokenToLog(aToken); 
-  #endif  
-  
   switch(theChildTag) {    
  
     case eHTMLTag_body: //we intentionally don't let the user close HTML or BODY
@@ -826,10 +799,6 @@ nsresult COtherDTD::CollectAttributes(nsIParserNode& aNode,eHTMLTags aTag,PRInt3
         // a legitimate "SELECTED" key.
         ((CAttributeToken*)theToken)->SanitizeKey();
  
-  #ifdef  RICKG_DEBUG
-    WriteTokenToLog(theToken);
-  #endif
- 
         aNode.AddAttribute(theToken); 
       }
     }
@@ -941,15 +910,6 @@ COtherDTD::IntTagToStringTag(PRInt32 aIntTag) const
   return str_ptr;
 }  
 
-NS_IMETHODIMP
-COtherDTD::ConvertEntityToUnicode(const nsAString& aEntity,
-                                  PRInt32* aUnicode) const
-{
-  *aUnicode = nsHTMLEntities::EntityToUnicode(aEntity);
-
-  return NS_OK;
-}
- 
 /**
  *  This method is called to determine whether or not
  *  the given childtag is a block element.
