@@ -1,24 +1,40 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/*
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
- */
+ * Contributor(s):
+ *
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 
 /**
@@ -107,9 +123,11 @@ inline PRBool TestBits(int aBitset,int aTest) {
  */
 struct nsHTMLElement {
 
+#ifdef DEBUG
   static  void    DebugDumpMembership(const char* aFilename);
   static  void    DebugDumpContainment(const char* aFilename,const char* aTitle);
   static  void    DebugDumpContainType(const char* aFilename);
+#endif
 
   static  PRBool  IsInlineEntity(eHTMLTags aTag);
   static  PRBool  IsFlowEntity(eHTMLTags aTag);
@@ -177,15 +195,15 @@ struct nsHTMLElement {
 
   static  PRInt32 GetIndexOfChildOrSynonym(nsDTDContext& aContext,eHTMLTags aChildTag);
 
-  TagList*        GetSynonymousTags(void) const {return mSynonymousTags;}
-  TagList*        GetRootTags(void) const {return mRootNodes;}
-  TagList*        GetEndRootTags(void) const {return mEndRootNodes;}
-  TagList*        GetAutoCloseStartTags(void) const {return mAutocloseStart;}
-  TagList*        GetAutoCloseEndTags(void) const {return mAutocloseEnd;}
+  const TagList*  GetSynonymousTags(void) const {return mSynonymousTags;}
+  const TagList*  GetRootTags(void) const {return mRootNodes;}
+  const TagList*  GetEndRootTags(void) const {return mEndRootNodes;}
+  const TagList*  GetAutoCloseStartTags(void) const {return mAutocloseStart;}
+  const TagList*  GetAutoCloseEndTags(void) const {return mAutocloseEnd;}
   eHTMLTags       GetCloseTargetForEndTag(nsDTDContext& aContext,PRInt32 anIndex) const;
 
-  TagList*        GetSpecialChildren(void) const {return mSpecialKids;}
-  TagList*        GetSpecialParents(void) const {return mSpecialParents;}
+  const TagList*        GetSpecialChildren(void) const {return mSpecialKids;}
+  const TagList*        GetSpecialParents(void) const {return mSpecialParents;}
 
   PRBool          IsMemberOf(PRInt32 aType) const;
   PRBool          ContainsSet(PRInt32 aType) const;
@@ -221,19 +239,19 @@ struct nsHTMLElement {
   eHTMLTags       mTagID;
   eHTMLTags       mRequiredAncestor;
   eHTMLTags       mExcludingAncestor; //If set, the presence of the excl-ancestor prevents this from opening.
-  TagList*        mRootNodes;         //These are the tags above which you many not autoclose a START tag
-  TagList*        mEndRootNodes;      //These are the tags above which you many not autoclose an END tag
-  TagList*        mAutocloseStart;    //these are the start tags that you can automatically close with this START tag
-  TagList*        mAutocloseEnd;      //these are the start tags that you can automatically close with this END tag
-  TagList*        mSynonymousTags;    //These are morally equivalent; an end tag for one can close a start tag for another (like <Hn>)
-  TagList*        mExcludableParents; //These are the TAGS that cannot contain you
+  const TagList*  mRootNodes;         //These are the tags above which you many not autoclose a START tag
+  const TagList*        mEndRootNodes;      //These are the tags above which you many not autoclose an END tag
+  const TagList*        mAutocloseStart;    //these are the start tags that you can automatically close with this START tag
+  const TagList*        mAutocloseEnd;      //these are the start tags that you can automatically close with this END tag
+  const TagList*        mSynonymousTags;    //These are morally equivalent; an end tag for one can close a start tag for another (like <Hn>)
+  const TagList*        mExcludableParents; //These are the TAGS that cannot contain you
   int             mParentBits;        //defines groups that can contain this element
   int             mInclusionBits;     //defines parental and containment rules
   int             mExclusionBits;     //defines things you CANNOT contain
   int             mSpecialProperties; //used for various special purposes...
   PRUint32        mPropagateRange;    //tells us how far a parent is willing to prop. badly formed children
-  TagList*        mSpecialParents;    //These are the special tags that contain this tag (directly)
-  TagList*        mSpecialKids;       //These are the extra things you can contain
+  const TagList*  mSpecialParents;    //These are the special tags that contain this tag (directly)
+  const TagList*  mSpecialKids;       //These are the extra things you can contain
   eHTMLTags       mSkipTarget;        //If set, then we skip all content until this tag is seen
   ContainFunc     mCanBeContained;
 }; 
