@@ -1,25 +1,41 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
+ * Contributor(s):
  *   Pierre Phaneuf <pp@ludusdesign.com>
- */
+ *
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #include <locale.h>
 #include "plstr.h"
@@ -31,15 +47,9 @@
 #include "nsIPlatformCharset.h"
 #include "nsIPosixLocale.h"
 #include "nsCRT.h"
+#include "nsUnicharUtils.h"
 
-static NS_DEFINE_IID(kIDateTimeFormatIID, NS_IDATETIMEFORMAT_IID);
-static NS_DEFINE_CID(kPosixLocaleFactoryCID, NS_POSIXLOCALEFACTORY_CID);
-static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
-static NS_DEFINE_IID(kICharsetConverterManagerIID, NS_ICHARSETCONVERTERMANAGER_IID);
-static NS_DEFINE_CID(kLocaleServiceCID, NS_LOCALESERVICE_CID); 
-static NS_DEFINE_CID(kPlatformCharsetCID, NS_PLATFORMCHARSET_CID);
-
-NS_IMPL_THREADSAFE_ISUPPORTS(nsDateTimeFormatUnix, kIDateTimeFormatIID);
+NS_IMPL_THREADSAFE_ISUPPORTS1(nsDateTimeFormatUnix, nsIDateTimeFormat);
 
 // init this interface to a specified locale
 nsresult nsDateTimeFormatUnix::Initialize(nsILocale* locale)
@@ -51,14 +61,17 @@ nsresult nsDateTimeFormatUnix::Initialize(nsILocale* locale)
 
   // use cached info if match with stored locale
   if (NULL == locale) {
-    if (mLocale.Length() && mLocale.EqualsIgnoreCase(mAppLocale)) {
+    if (mLocale.Length() &&
+        mLocale.Equals(mAppLocale, nsCaseInsensitiveStringComparator())) {
       return NS_OK;
     }
   }
   else {
     res = locale->GetCategory(aCategory.get(), &aLocaleUnichar);
     if (NS_SUCCEEDED(res) && NULL != aLocaleUnichar) {
-      if (mLocale.Length() && mLocale.EqualsIgnoreCase(nsAutoString(aLocaleUnichar))) {
+      if (!mLocale.IsEmpty() &&
+          mLocale.Equals(aLocaleUnichar,
+                         nsCaseInsensitiveStringComparator())) {
         nsMemory::Free(aLocaleUnichar);
         return NS_OK;
       }
@@ -66,13 +79,13 @@ nsresult nsDateTimeFormatUnix::Initialize(nsILocale* locale)
     }
   }
 
-  mCharset.AssignWithConversion("ISO-8859-1");
+  mCharset.Assign(NS_LITERAL_STRING("ISO-8859-1"));
   PL_strncpy(mPlatformLocale, "en_US", kPlatformLocaleLength+1);
 
   // get locale name string, use app default if no locale specified
   if (NULL == locale) {
     nsCOMPtr<nsILocaleService> localeService = 
-             do_GetService(kLocaleServiceCID, &res);
+             do_GetService(NS_LOCALESERVICE_CONTRACTID, &res);
     if (NS_SUCCEEDED(res)) {
       nsILocale *appLocale;
       res = localeService->GetApplicationLocale(&appLocale);
@@ -95,7 +108,7 @@ nsresult nsDateTimeFormatUnix::Initialize(nsILocale* locale)
     mLocale = aLocaleUnichar; // cache locale name
     nsMemory::Free(aLocaleUnichar);
 
-    nsCOMPtr <nsIPosixLocale> posixLocale = do_GetService(kPosixLocaleFactoryCID, &res);
+    nsCOMPtr <nsIPosixLocale> posixLocale = do_GetService(NS_POSIXLOCALE_CONTRACTID, &res);
     if (NS_SUCCEEDED(res)) {
       res = posixLocale->GetPlatformLocale(&mLocale, mPlatformLocale, kPlatformLocaleLength+1);
     }
@@ -122,12 +135,12 @@ nsresult nsDateTimeFormatUnix::Initialize(nsILocale* locale)
     }
   }
 
-  mLocalePreferred24hour = LocalePreferred24hour();
+  LocalePreferred24hour();
 
   return res;
 }
 
-PRBool nsDateTimeFormatUnix::LocalePreferred24hour()
+void nsDateTimeFormatUnix::LocalePreferred24hour()
 {
   char str[100];
   time_t tt;
@@ -143,14 +156,24 @@ PRBool nsDateTimeFormatUnix::LocalePreferred24hour()
 
   char *temp = setlocale(LC_TIME, mPlatformLocale);
   strftime(str, (size_t)99, "%X", (struct tm *)tmc);
+
   (void) setlocale(LC_TIME, temp);
 
-  for (i=0; str[i]; i++)
+  mLocalePreferred24hour = PR_FALSE;
+  for (i=0; str[i]; i++) {
     if (str[i] == '2') {    // if there is any '2', that locale use 0-23 time format
-        return PR_TRUE;
+        mLocalePreferred24hour = PR_TRUE;
+        break;
     }
+  }
 
-  return PR_FALSE;
+  mLocaleAMPMfirst = PR_TRUE;
+  if (mLocalePreferred24hour == PR_FALSE) {
+    if (str[0] && str[0] == '1') { // if the first character is '1' of 10:00,
+			           // AMPM string is located after 10:00
+      mLocaleAMPMfirst = PR_FALSE;
+    }
+  }
 }
 
 nsresult nsDateTimeFormatUnix::FormatTime(nsILocale* locale, 
@@ -160,7 +183,7 @@ nsresult nsDateTimeFormatUnix::FormatTime(nsILocale* locale,
                                       nsString& stringOut) 
 {
   struct tm tmTime;
-  nsCRT::memcpy(&tmTime, localtime(&timetTime), sizeof(struct tm));
+  memcpy(&tmTime, localtime(&timetTime), sizeof(struct tm));
   return FormatTMTime(locale, dateFormatSelector, timeFormatSelector, &tmTime, stringOut);
 }
 
@@ -174,11 +197,12 @@ nsresult nsDateTimeFormatUnix::FormatTMTime(nsILocale* locale,
 #define NSDATETIME_FORMAT_BUFFER_LEN  80
   char strOut[NSDATETIME_FORMAT_BUFFER_LEN*2];  // buffer for date and time
   char fmtD[NSDATETIME_FORMAT_BUFFER_LEN], fmtT[NSDATETIME_FORMAT_BUFFER_LEN];
-  nsresult res;
+  nsresult rv;
 
   
   // set up locale data
   (void) Initialize(locale);
+  NS_ENSURE_TRUE(mDecoder, NS_ERROR_NOT_INITIALIZED);
 
   // set date format
   switch (dateFormatSelector) {
@@ -186,8 +210,6 @@ nsresult nsDateTimeFormatUnix::FormatTMTime(nsILocale* locale,
       PL_strncpy(fmtD, "", NSDATETIME_FORMAT_BUFFER_LEN);
       break; 
     case kDateFormatLong:
-      PL_strncpy(fmtD, "%c", NSDATETIME_FORMAT_BUFFER_LEN);
-      break; 
     case kDateFormatShort:
       PL_strncpy(fmtD, "%x", NSDATETIME_FORMAT_BUFFER_LEN);
       break; 
@@ -208,12 +230,12 @@ nsresult nsDateTimeFormatUnix::FormatTMTime(nsILocale* locale,
       break;
     case kTimeFormatSeconds:
       PL_strncpy(fmtT, 
-                 mLocalePreferred24hour ? "%H:%M:%S" : "%I:%M:%S %p", 
+                 mLocalePreferred24hour ? "%H:%M:%S" : mLocaleAMPMfirst ? "%p %I:%M:%S" : "%I:%M:%S %p", 
                  NSDATETIME_FORMAT_BUFFER_LEN);
       break;
     case kTimeFormatNoSeconds:
       PL_strncpy(fmtT, 
-                 mLocalePreferred24hour ? "%H:%M" : "%I:%M %p", 
+                 mLocalePreferred24hour ? "%H:%M" : mLocaleAMPMfirst ? "%p %I:%M" : "%I:%M %p", 
                  NSDATETIME_FORMAT_BUFFER_LEN);
       break;
     case kTimeFormatSecondsForce24Hour:
@@ -246,18 +268,16 @@ nsresult nsDateTimeFormatUnix::FormatTMTime(nsILocale* locale,
   (void) setlocale(LC_TIME, old_locale);
 
   // convert result to unicode
-  if (mDecoder) {
-    PRInt32 srcLength = (PRInt32) PL_strlen(strOut);
-    PRInt32 unicharLength = NSDATETIME_FORMAT_BUFFER_LEN*2;
-    PRUnichar unichars[NSDATETIME_FORMAT_BUFFER_LEN*2];   // buffer for date and time
+  PRInt32 srcLength = (PRInt32) PL_strlen(strOut);
+  PRInt32 unicharLength = NSDATETIME_FORMAT_BUFFER_LEN*2;
+  PRUnichar unichars[NSDATETIME_FORMAT_BUFFER_LEN*2];   // buffer for date and time
 
-    res = mDecoder->Convert(strOut, &srcLength, unichars, &unicharLength);
-    if (NS_SUCCEEDED(res)) {
-      stringOut.Assign(unichars, unicharLength);
-    }
-  }
-  
-  return res;
+  rv = mDecoder->Convert(strOut, &srcLength, unichars, &unicharLength);
+  if (NS_FAILED(rv))
+    return rv;
+  stringOut.Assign(unichars, unicharLength);
+
+  return rv;
 }
 
 // performs a locale sensitive date formatting operation on the PRTime parameter
@@ -292,7 +312,7 @@ nsresult nsDateTimeFormatUnix::FormatPRExplodedTime(nsILocale* locale,
    * when tmTime is passed to strftime() with garbage bad things may happen. 
    * see bug #10412
    */
-  nsCRT::memset( &tmTime, 0, sizeof(tmTime) );
+  memset( &tmTime, 0, sizeof(tmTime) );
 
   tmTime.tm_yday = explodedTime->tm_yday;
   tmTime.tm_wday = explodedTime->tm_wday;
