@@ -37,34 +37,40 @@
  *
  * ***** END LICENSE BLOCK ***** */
 #include "nsCOMPtr.h"
+#include "nsCRT.h"
 #include "nsIModule.h"
 #include "nsIGenericFactory.h"
 #include "nsIComponentManager.h"
 #include "nsICategoryManager.h"
 #include "nsICharsetConverterManager.h"
 #include "nsICharsetConverterManager2.h"
-#include "nsIUnicodeDecodeHelper.h"
-#include "nsIUnicodeEncodeHelper.h"
 #include "nsIUnicodeDecoder.h"
 #include "nsIUnicodeEncoder.h"
-#include "nsICharsetConverterManager.h"
-#include "nsIPlatformCharset.h"
 #include "nsICharsetAlias.h"
-#include "nsITextToSubURI.h"
 #include "nsIServiceManager.h"
-#include "nsUConvDll.h"
-#include "nsFileSpec.h"
-#include "nsIFile.h"
-#include "nsIScriptableUConv.h"
-#include "nsConverterInputStream.h"
 
-#include "nsUCSupport.h"
+
 #include "nsCharsetConverterManager.h"
-#include "nsUnicodeDecodeHelper.h"
-#include "nsUnicodeEncodeHelper.h"
-#include "nsPlatformCharset.h"
 #include "nsCharsetAlias.h"
 #include "nsTextToSubURI.h"
+#include "nsConverterInputStream.h"
+#include "nsPlatformCharset.h"
+
+#ifndef MOZ_USE_NATIVE_UCONV
+#include "nsIUnicodeDecodeHelper.h"
+#include "nsIUnicodeEncodeHelper.h"
+#include "nsIPlatformCharset.h"
+#include "nsITextToSubURI.h"
+
+#include "nsUConvDll.h"
+#include "nsIFile.h"
+#include "nsIScriptableUConv.h"
+
+#include "nsCRT.h"
+
+#include "nsUCSupport.h"
+#include "nsUnicodeDecodeHelper.h"
+#include "nsUnicodeEncodeHelper.h"
 #include "nsISO88591ToUnicode.h"
 #include "nsCP1252ToUnicode.h"
 #include "nsMacRomanToUnicode.h"
@@ -197,7 +203,11 @@
 #include "nsUnicodeToMacGujarati.h"
 #include "nsUnicodeToMacGurmukhi.h"
 #include "nsUnicodeToMacHebrew.h"
+#include "nsUnicodeToTSCII.h"
+#ifdef MOZ_EXTRA_X11CONVERTERS
 #include "nsUnicodeToLangBoxArabic8.h"
+#include "nsUnicodeToLangBoxArabic16.h"
+#endif // MOZ_EXTRA_X11CONVERTERS
 
 // ucvibm
 #include "nsUCvIBMCID.h"
@@ -226,8 +236,10 @@
 #include "nsUnicodeToEUCJP.h"
 #include "nsUnicodeToISO2022JP.h"
 #include "nsUnicodeToJISx0201.h"
+#ifdef MOZ_EXTRA_X11CONVERTERS
 #include "nsUnicodeToJISx0208.h"
 #include "nsUnicodeToJISx0212.h"
+#endif
 
 // ucvtw2
 #include "nsUCvTW2CID.h"
@@ -247,7 +259,9 @@
 #include "nsUCvTWDll.h"
 #include "nsBIG5ToUnicode.h"
 #include "nsUnicodeToBIG5.h"
+#ifdef MOZ_EXTRA_X11CONVERTERS
 #include "nsUnicodeToBIG5NoAscii.h"
+#endif
 #include "nsBIG5HKSCSToUnicode.h"
 #include "nsUnicodeToBIG5HKSCS.h"
 #include "nsUnicodeToHKSCS.h"
@@ -257,14 +271,17 @@
 #include "nsUCvKODll.h"
 #include "nsEUCKRToUnicode.h"
 #include "nsUnicodeToEUCKR.h"
-#include "nsUnicodeToKSC5601.h"
-#include "nsUnicodeToX11Johab.h"
 #include "nsJohabToUnicode.h"
 #include "nsUnicodeToJohab.h"
+#ifdef MOZ_EXTRA_X11CONVERTERS
+#include "nsUnicodeToKSC5601.h"
+#include "nsUnicodeToX11Johab.h"
 #include "nsUnicodeToJohabNoAscii.h"
+#endif
 #include "nsCP949ToUnicode.h"
 #include "nsUnicodeToCP949.h"
 #include "nsISO2022KRToUnicode.h"
+#include "nsUnicodeToJamoTTF.h"
 
 // ucvcn
 #include "nsUCvCnCID.h"
@@ -273,7 +290,9 @@
 #include "nsUnicodeToHZ.h"
 #include "nsGBKToUnicode.h"
 #include "nsUnicodeToGBK.h"
+#ifdef MOZ_EXTRA_X11CONVERTERS
 #include "nsUnicodeToGBKNoAscii.h"
+#endif
 #include "nsCP936ToUnicode.h"
 #include "nsUnicodeToCP936.h"
 #include "nsGB2312ToUnicodeV2.h"
@@ -356,7 +375,12 @@ NS_UCONV_REG_UNREG("x-mac-hebrew" , NS_MACHEBREWTOUNICODE_CID, NS_UNICODETOMACHE
 NS_UCONV_REG_UNREG_ENCODER("UTF-16" , NS_UNICODETOUTF16_CID)
 NS_UCONV_REG_UNREG_ENCODER("Adobe-Symbol-Encoding" , NS_UNICODETOSYMBOL_CID)
 NS_UCONV_REG_UNREG_ENCODER("x-zapf-dingbats" , NS_UNICODETOZAPFDINGBATS_CID)
+NS_UCONV_REG_UNREG_ENCODER("x-tscii",  NS_UNICODETOTSCII_CID)
+NS_UCONV_REG_UNREG_ENCODER("x-tamilttf-0",  NS_UNICODETOTAMILTTF_CID)
+#ifdef MOZ_EXTRA_X11CONVERTERS
 NS_UCONV_REG_UNREG_ENCODER("x-iso-8859-6-8-x" , NS_UNICODETOLANGBOXARABIC_CID)
+NS_UCONV_REG_UNREG_ENCODER("x-iso-8859-6-16" , NS_UNICODETOLANGBOXARABIC16_CID)
+#endif // MOZ_EXTRA_X11CONVERTERS
 
   // ucvibm
 NS_UCONV_REG_UNREG("IBM850", NS_CP850TOUNICODE_CID, NS_UNICODETOCP850_CID)
@@ -373,8 +397,10 @@ NS_UCONV_REG_UNREG("ISO-2022-JP", NS_ISO2022JPTOUNICODE_CID, NS_UNICODETOISO2022
 NS_UCONV_REG_UNREG("EUC-JP", NS_EUCJPTOUNICODE_CID, NS_UNICODETOEUCJP_CID)
   
 NS_UCONV_REG_UNREG_ENCODER("jis_0201" , NS_UNICODETOJISX0201_CID)
+#ifdef MOZ_EXTRA_X11CONVERTERS
 NS_UCONV_REG_UNREG_ENCODER("jis_0208-1983" , NS_UNICODETOJISX0208_CID)
 NS_UCONV_REG_UNREG_ENCODER("jis_0212-1990" , NS_UNICODETOJISX0212_CID)
+#endif
 
     // ucvtw2
 NS_UCONV_REG_UNREG("x-euc-tw", NS_EUCTWTOUNICODE_CID, NS_UNICODETOEUCTW_CID)
@@ -391,23 +417,30 @@ NS_UCONV_REG_UNREG("Big5", NS_BIG5TOUNICODE_CID, NS_UNICODETOBIG5_CID)
 NS_UCONV_REG_UNREG("Big5-HKSCS", NS_BIG5HKSCSTOUNICODE_CID, NS_UNICODETOBIG5HKSCS_CID)
   
 NS_UCONV_REG_UNREG_ENCODER("hkscs-1" , NS_UNICODETOHKSCS_CID)
+#ifdef MOZ_EXTRA_X11CONVERTERS
 NS_UCONV_REG_UNREG_ENCODER("x-x-big5",  NS_UNICODETOBIG5NOASCII_CID)
+#endif
 
     // ucvko
 NS_UCONV_REG_UNREG("EUC-KR", NS_EUCKRTOUNICODE_CID, NS_UNICODETOEUCKR_CID)
 NS_UCONV_REG_UNREG("x-johab", NS_JOHABTOUNICODE_CID, NS_UNICODETOJOHAB_CID)
 NS_UCONV_REG_UNREG("x-windows-949", NS_CP949TOUNICODE_CID, NS_UNICODETOCP949_CID)
 NS_UCONV_REG_UNREG_DECODER("ISO-2022-KR", NS_ISO2022KRTOUNICODE_CID)
+NS_UCONV_REG_UNREG_ENCODER("x-koreanjamo-0",  NS_UNICODETOJAMOTTF_CID)
 
+#ifdef MOZ_EXTRA_X11CONVERTERS
 NS_UCONV_REG_UNREG_ENCODER("ks_c_5601-1987",  NS_UNICODETOKSC5601_CID)
 NS_UCONV_REG_UNREG_ENCODER("x-x11johab",  NS_UNICODETOX11JOHAB_CID)
 NS_UCONV_REG_UNREG_ENCODER("x-johab-noascii",  NS_UNICODETOJOHABNOASCII_CID)
+#endif
 
 // ucvcn
 NS_UCONV_REG_UNREG("GB2312", NS_GB2312TOUNICODE_CID, NS_UNICODETOGB2312_CID)
 NS_UCONV_REG_UNREG("windows-936", NS_CP936TOUNICODE_CID, NS_UNICODETOCP936_CID)
 NS_UCONV_REG_UNREG("x-gbk", NS_GBKTOUNICODE_CID, NS_UNICODETOGBK_CID)
+#ifdef MOZ_EXTRA_X11CONVERTERS
 NS_UCONV_REG_UNREG_ENCODER("x-gbk-noascii",  NS_UNICODETOGBKNOASCII_CID)
+#endif
 NS_UCONV_REG_UNREG("HZ-GB-2312", NS_HZTOUNICODE_CID, NS_UNICODETOHZ_CID)
 NS_UCONV_REG_UNREG_ENCODER("gb_2312-80",  NS_UNICODETOGB2312GL_CID)
 NS_UCONV_REG_UNREG("gb18030", NS_GB18030TOUNICODE_CID, NS_UNICODETOGB18030_CID)
@@ -419,14 +452,9 @@ NS_CONVERTER_REGISTRY_END
 
 NS_IMPL_NSUCONVERTERREGSELF
 
-NS_GENERIC_FACTORY_CONSTRUCTOR(nsCharsetConverterManager)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeDecodeHelper)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeEncodeHelper)
-NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(nsPlatformCharset, Init)
-NS_GENERIC_FACTORY_CONSTRUCTOR(nsCharsetAlias2)
-NS_GENERIC_FACTORY_CONSTRUCTOR(nsTextToSubURI)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsScriptableUnicodeConverter)
-NS_GENERIC_FACTORY_CONSTRUCTOR(nsConverterInputStream)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToUTF8)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUTF8ToUnicode)
 
@@ -446,7 +474,12 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToUTF16LE);
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToUTF16);
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToUTF32BE);
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToUTF32LE);
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToTSCII);
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToTamilTTF);
+#ifdef MOZ_EXTRA_X11CONVERTERS
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToLangBoxArabic8);
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToLangBoxArabic16);
+#endif // MOZ_EXTRA_X11CONVERTERS
 
 // ucvibm
 
@@ -459,11 +492,16 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToISO2022JP);
 // ucvtw2
 
 // ucvtw
+#ifdef MOZ_EXTRA_X11CONVERTERS
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToBIG5NoAscii);
+#endif
 
 // ucvko
+#ifdef MOZ_EXTRA_X11CONVERTERS
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToX11Johab);
+#endif
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsISO2022KRToUnicode);
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToJamoTTF);
 
 // ucvcn
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsGB2312ToUnicodeV2);
@@ -472,7 +510,9 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(nsCP936ToUnicode);
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToCP936);
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsGBKToUnicode);
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToGBK);
+#ifdef MOZ_EXTRA_X11CONVERTERS
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToGBKNoAscii);
+#endif
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsHZToUnicode);
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToHZ);
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToGB2312GL);
@@ -492,126 +532,139 @@ static NS_DEFINE_CID(kComponentManagerCID, NS_COMPONENTMANAGER_CID);
 #define ENCODER_NAME_BASE "Unicode Encoder-"
 
 // ucvja
-PRUint16 g_uf0201Mapping[] = {
+const PRUint16 g_uf0201Mapping[] = {
 #include "jis0201.uf"
 };
-PRUint16 g_uf0201GLMapping[] = {
+
+const PRUint16 g_uf0201GLMapping[] = {
 #include "jis0201gl.uf"
 };
 
-PRUint16 g_uf0208Mapping[] = {
+const PRUint16 g_uf0208Mapping[] = {
 #include "jis0208.uf"
 };
 
-PRUint16 g_uf0208extMapping[] = {
+const PRUint16 g_uf0208extMapping[] = {
 #include "jis0208ext.uf"
 };
 
-PRUint16 g_uf0212Mapping[] = {
+const PRUint16 g_uf0212Mapping[] = {
 #include "jis0212.uf"
 };
 
 // ucvtw2
-PRUint16 g_ufCNS1MappingTable[] = {
+const PRUint16 g_ufCNS1MappingTable[] = {
 #include "cns_1.uf"
 };
 
-PRUint16 g_ufCNS2MappingTable[] = {
+const PRUint16 g_ufCNS2MappingTable[] = {
 #include "cns_2.uf"
 };
 
-PRUint16 g_ufCNS3MappingTable[] = {
+const PRUint16 g_ufCNS3MappingTable[] = {
 #include "cns3.uf"
 };
 
-PRUint16 g_ufCNS4MappingTable[] = {
+const PRUint16 g_ufCNS4MappingTable[] = {
 #include "cns4.uf"
 };
 
-PRUint16 g_ufCNS5MappingTable[] = {
+const PRUint16 g_ufCNS5MappingTable[] = {
 #include "cns5.uf"
 };
 
-PRUint16 g_ufCNS6MappingTable[] = {
+const PRUint16 g_ufCNS6MappingTable[] = {
 #include "cns6.uf"
 };
 
-PRUint16 g_ufCNS7MappingTable[] = {
+const PRUint16 g_ufCNS7MappingTable[] = {
 #include "cns7.uf"
 };
 
-PRUint16 g_utCNS1MappingTable[] = {
+const PRUint16 g_utCNS1MappingTable[] = {
 #include "cns_1.ut"
 };
 
-PRUint16 g_utCNS2MappingTable[] = {
+const PRUint16 g_utCNS2MappingTable[] = {
 #include "cns_2.ut"
 };
 
-PRUint16 g_utCNS3MappingTable[] = {
+const PRUint16 g_utCNS3MappingTable[] = {
 #include "cns3.ut"
 };
 
-PRUint16 g_utCNS4MappingTable[] = {
+const PRUint16 g_utCNS4MappingTable[] = {
 #include "cns4.ut"
 };
 
-PRUint16 g_utCNS5MappingTable[] = {
+const PRUint16 g_utCNS5MappingTable[] = {
 #include "cns5.ut"
 };
 
-PRUint16 g_utCNS6MappingTable[] = {
+const PRUint16 g_utCNS6MappingTable[] = {
 #include "cns6.ut"
 };
 
-PRUint16 g_utCNS7MappingTable[] = {
+const PRUint16 g_utCNS7MappingTable[] = {
 #include "cns7.ut"
 };
 
-PRUint16 g_ASCIIMappingTable[] = {
+const PRUint16 g_ASCIIMappingTable[] = {
   0x0001, 0x0004, 0x0005, 0x0008, 0x0000, 0x0000, 0x007F, 0x0000
 };
 
 // ucvtw
-PRUint16 g_ufBig5Mapping[] = {
+const PRUint16 g_ufBig5Mapping[] = {
 #include "big5.uf"
 };
 
-PRUint16 g_utBIG5Mapping[] = {
+const PRUint16 g_utBIG5Mapping[] = {
 #include "big5.ut"
 };
 
-PRUint16 g_ufBig5HKSCSMapping[] = {
+const PRUint16 g_ufBig5HKSCSMapping[] = {
 #include "hkscs.uf"
 };
 
-PRUint16 g_ASCIIMapping[] = {
+const PRUint16 g_ASCIIMapping[] = {
   0x0001, 0x0004, 0x0005, 0x0008, 0x0000, 0x0000, 0x007F, 0x0000
 };
 
-PRUint16 g_utBig5HKSCSMapping[] = {
+const PRUint16 g_utBig5HKSCSMapping[] = {
 #include "hkscs.ut"
 };
 
 // ucvko
-PRUint16 g_utKSC5601Mapping[] = {
+const PRUint16 g_utKSC5601Mapping[] = {
 #include "u20kscgl.ut"
 };
 
-PRUint16 g_ufKSC5601Mapping[] = {
+const PRUint16 g_ufKSC5601Mapping[] = {
 #include "u20kscgl.uf"
 };
 
-PRUint16 g_ucvko_AsciiMapping[] = {
+const PRUint16 g_ucvko_AsciiMapping[] = {
   0x0001, 0x0004, 0x0005, 0x0008, 0x0000, 0x0000, 0x007F, 0x0000
 };
-PRUint16 g_HangulNullMapping[] ={
+
+const PRUint16 g_HangulNullMapping[] ={
   0x0001, 0x0004, 0x0005, 0x0008, 0x0000, 0xAC00, 0xD7A3, 0xAC00
 };
-PRUint16 g_ufJohabJamoMapping[] ={   
+
+const PRUint16 g_ufJohabJamoMapping[] ={   
 #include "johabjamo.uf"
 };
 
+#else // MOZ_USE_NATIVE_UCONV
+
+#include "nsINativeUConvService.h"
+#include "nsNativeUConvService.h"
+
+NS_GENERIC_FACTORY_CONSTRUCTOR(NativeUConvService)
+
+#endif // #ifndef MOZ_USE_NATIVE_UCONV
+
+
 NS_IMETHODIMP
 nsConverterManagerDataRegister(nsIComponentManager* aCompMgr,
                                 nsIFile* aPath,
@@ -622,6 +675,12 @@ nsConverterManagerDataRegister(nsIComponentManager* aCompMgr,
   return nsCharsetConverterManager::RegisterConverterManagerData();
 }
 
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsCharsetConverterManager)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsTextToSubURI)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsCharsetAlias2)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsConverterInputStream)
+NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(nsPlatformCharset, Init)
+
 static const nsModuleComponentInfo components[] = 
 {
   { 
@@ -630,21 +689,6 @@ static const nsModuleComponentInfo components[] =
     nsCharsetConverterManagerConstructor,
     nsConverterManagerDataRegister,
   },
-  { 
-    "Unicode Decode Helper", NS_UNICODEDECODEHELPER_CID,
-    NS_UNICODEDECODEHELPER_CONTRACTID, 
-    nsUnicodeDecodeHelperConstructor 
-  },
-  { 
-    "Unicode Encode Helper", NS_UNICODEENCODEHELPER_CID,
-    NS_UNICODEENCODEHELPER_CONTRACTID, 
-    nsUnicodeEncodeHelperConstructor 
-  },
-  { 
-    "Platform Charset Information", NS_PLATFORMCHARSET_CID,
-    NS_PLATFORMCHARSET_CONTRACTID, 
-    nsPlatformCharsetConstructor
-  },
   { 
     "Charset Alias Information",  NS_CHARSETALIAS_CID,
     NS_CHARSETALIAS_CONTRACTID, 
@@ -656,14 +700,37 @@ static const nsModuleComponentInfo components[] =
     nsTextToSubURIConstructor
   },
   { 
-    "Unicode Encoder / Decoder for Script", NS_ISCRIPTABLEUNICODECONVERTER_CID,
-    NS_ISCRIPTABLEUNICODECONVERTER_CONTRACTID, 
-    nsScriptableUnicodeConverterConstructor
+    "Platform Charset Information", NS_PLATFORMCHARSET_CID,
+    NS_PLATFORMCHARSET_CONTRACTID, 
+    nsPlatformCharsetConstructor
   },
   { "Unicode converter input stream", NS_CONVERTERINPUTSTREAM_CID,              
     NS_CONVERTERINPUTSTREAM_CONTRACTID, 
     nsConverterInputStreamConstructor 
-  },    
+  },   
+#ifdef MOZ_USE_NATIVE_UCONV
+  { 
+    "Native UConv Service", 
+    NS_NATIVE_UCONV_SERVICE_CID,
+    NS_NATIVE_UCONV_SERVICE_CONTRACT_ID, 
+    NativeUConvServiceConstructor,
+  },
+#else
+  { 
+    "Unicode Decode Helper", NS_UNICODEDECODEHELPER_CID,
+    NS_UNICODEDECODEHELPER_CONTRACTID, 
+    nsUnicodeDecodeHelperConstructor 
+  },
+  { 
+    "Unicode Encode Helper", NS_UNICODEENCODEHELPER_CID,
+    NS_UNICODEENCODEHELPER_CONTRACTID, 
+    nsUnicodeEncodeHelperConstructor 
+  },
+  { 
+    "Unicode Encoder / Decoder for Script", NS_ISCRIPTABLEUNICODECONVERTER_CID,
+    NS_ISCRIPTABLEUNICODECONVERTER_CONTRACTID, 
+    nsScriptableUnicodeConverterConstructor
+  },
   { 
     "ISO-8859-1 To Unicode Converter", NS_ISO88591TOUNICODE_CID, 
     NS_ISO88591TOUNICODE_CONTRACTID,
@@ -1318,11 +1385,28 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEENCODER_CONTRACTID_BASE "x-mac-hebrew",
     nsUnicodeToMacHebrewConstructor, 
   },
+  { 
+    ENCODER_NAME_BASE "x-tscii" , NS_UNICODETOTSCII_CID,
+    NS_UNICODEENCODER_CONTRACTID_BASE "x-tscii",
+    nsUnicodeToTSCIIConstructor, 
+  },
+  { 
+    ENCODER_NAME_BASE "x-tamilttf-0" , NS_UNICODETOTAMILTTF_CID,
+    NS_UNICODEENCODER_CONTRACTID_BASE "x-tamilttf-0",
+    nsUnicodeToTamilTTFConstructor, 
+  },
+#ifdef MOZ_EXTRA_X11CONVERTERS
   { 
     ENCODER_NAME_BASE "x-iso-8859-6-8-x" , NS_UNICODETOLANGBOXARABIC_CID, 
     NS_UNICODEENCODER_CONTRACTID_BASE "x-iso-8859-6-8-x",
     nsUnicodeToLangBoxArabic8Constructor, 
   },
+  { 
+    ENCODER_NAME_BASE "x-iso-8859-6-16" , NS_UNICODETOLANGBOXARABIC16_CID, 
+    NS_UNICODEENCODER_CONTRACTID_BASE "x-iso-8859-6-16",
+    nsUnicodeToLangBoxArabic16Constructor, 
+  },
+#endif // MOZ_EXTRA_X11CONVERTERS
   // ucvibm
   { 
     DECODER_NAME_BASE "IBM850" , NS_CP850TOUNICODE_CID, 
@@ -1430,6 +1514,7 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEENCODER_CONTRACTID_BASE "jis_0201",
     nsUnicodeToJISx0201Constructor, 
   },
+#ifdef MOZ_EXTRA_X11CONVERTERS
   { 
     ENCODER_NAME_BASE "jis_0208-1983" , NS_UNICODETOJISX0208_CID, 
     NS_UNICODEENCODER_CONTRACTID_BASE "jis_0208-1983",
@@ -1440,6 +1525,7 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEENCODER_CONTRACTID_BASE "jis_0212-1990",
     nsUnicodeToJISx0212Constructor, 
   },
+#endif
 
   // ucvtw2
   { 
@@ -1494,11 +1580,13 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEENCODER_CONTRACTID_BASE "Big5",
     nsUnicodeToBIG5Constructor,
   },
+#ifdef MOZ_EXTRA_X11CONVERTERS
   { 
     ENCODER_NAME_BASE "x-x-big5" , NS_UNICODETOBIG5NOASCII_CID, 
     NS_UNICODEENCODER_CONTRACTID_BASE "x-x-big5",
     nsUnicodeToBIG5NoAsciiConstructor,
   },
+#endif
   { 
     DECODER_NAME_BASE "Big5" , NS_BIG5TOUNICODE_CID, 
     NS_UNICODEDECODER_CONTRACTID_BASE "Big5",
@@ -1531,6 +1619,17 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEENCODER_CONTRACTID_BASE "EUC-KR",
     nsUnicodeToEUCKRConstructor, 
   },
+  { 
+    DECODER_NAME_BASE "x-johab" , NS_JOHABTOUNICODE_CID, 
+    NS_UNICODEDECODER_CONTRACTID_BASE "x-johab",
+    nsJohabToUnicodeConstructor ,
+  },
+  { 
+    ENCODER_NAME_BASE "x-johab" , NS_UNICODETOJOHAB_CID, 
+    NS_UNICODEENCODER_CONTRACTID_BASE "x-johab",
+    nsUnicodeToJohabConstructor,
+  },
+#ifdef MOZ_EXTRA_X11CONVERTERS
   { 
     ENCODER_NAME_BASE "ks_c_5601-1987" , NS_UNICODETOKSC5601_CID, 
     NS_UNICODEENCODER_CONTRACTID_BASE "ks_c_5601-1987",
@@ -1541,21 +1640,12 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEENCODER_CONTRACTID_BASE "x-x11johab",
     nsUnicodeToX11JohabConstructor,
   },
-  { 
-    DECODER_NAME_BASE "x-johab" , NS_JOHABTOUNICODE_CID, 
-    NS_UNICODEDECODER_CONTRACTID_BASE "x-johab",
-    nsJohabToUnicodeConstructor ,
-  },
-  { 
-    ENCODER_NAME_BASE "x-johab" , NS_UNICODETOJOHAB_CID, 
-    NS_UNICODEENCODER_CONTRACTID_BASE "x-johab",
-    nsUnicodeToJohabConstructor,
-  },
   { 
     ENCODER_NAME_BASE "x-johab-noascii", NS_UNICODETOJOHABNOASCII_CID, 
     NS_UNICODEENCODER_CONTRACTID_BASE "x-johab-noascii",
     nsUnicodeToJohabNoAsciiConstructor,
   },
+#endif
   { 
     DECODER_NAME_BASE "x-windows-949" , NS_CP949TOUNICODE_CID, 
     NS_UNICODEDECODER_CONTRACTID_BASE "x-windows-949",
@@ -1571,6 +1661,11 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEDECODER_CONTRACTID_BASE "ISO-2022-KR",
     nsISO2022KRToUnicodeConstructor ,
   },
+  { 
+    ENCODER_NAME_BASE "x-koreanjamo-0" , NS_UNICODETOJAMOTTF_CID, 
+    NS_UNICODEENCODER_CONTRACTID_BASE "x-koreanjamo-0",
+    nsUnicodeToJamoTTFConstructor,
+  },
   // ucvcn
   { 
     DECODER_NAME_BASE "GB2312" , NS_GB2312TOUNICODE_CID, 
@@ -1602,11 +1697,13 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEENCODER_CONTRACTID_BASE "x-gbk",
     nsUnicodeToGBKConstructor, 
   },  
+#ifdef MOZ_EXTRA_X11CONVERTERS
   { 
     ENCODER_NAME_BASE "x-gbk-noascii" , NS_UNICODETOGBKNOASCII_CID, 
     NS_UNICODEENCODER_CONTRACTID_BASE "x-gbk-noascii",
     nsUnicodeToGBKNoAsciiConstructor, 
   },  
+#endif
   { 
     DECODER_NAME_BASE "HZ-GB-2312" , NS_HZTOUNICODE_CID, 
     NS_UNICODEDECODER_CONTRACTID_BASE "HZ-GB-2312",
@@ -1647,6 +1744,7 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEDECODER_CONTRACTID_BASE "ISO-2022-CN",
     nsISO2022CNToUnicodeConstructor,
   },
+#endif // MOZ_USE_NATIVE_UCONV
 };
 
 NS_IMPL_NSGETMODULE(nsUConvModule, components);
