@@ -1,24 +1,40 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is Mozilla Communicator client code.
  *
- * The Initial Developer of the Original Code is Netscape Communications
- * Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
- */
+ * Contributor(s):
+ *
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #include "nsUnicodeToISO2022JP.h"
 #include "nsIComponentManager.h"
@@ -71,8 +87,11 @@ static const PRInt16 * g_ufShiftTables[SIZE_OF_TABLES] = {
 //----------------------------------------------------------------------
 // Class nsUnicodeToISO2022JP [implementation]
 
+// worst case max length: 
+//  1  2 3  4  5  6  7 8
+// ESC $ B XX XX ESC ( B
 nsUnicodeToISO2022JP::nsUnicodeToISO2022JP() 
-: nsEncoderSupport()
+: nsEncoderSupport(8)
 {
   mHelper = NULL;
   Reset();
@@ -83,16 +102,6 @@ nsUnicodeToISO2022JP::~nsUnicodeToISO2022JP()
   NS_IF_RELEASE(mHelper);
 }
 
-nsresult nsUnicodeToISO2022JP::CreateInstance(nsISupports ** aResult) 
-{
-  nsIUnicodeEncoder *p = new nsUnicodeToISO2022JP();
-  if(p) {
-   *aResult = p;
-   return NS_OK;
-  }
-  return NS_ERROR_OUT_OF_MEMORY;
-}
-
 nsresult nsUnicodeToISO2022JP::ChangeCharset(PRInt32 aCharset,
                                              char * aDest, 
                                              PRInt32 * aDestLength)
@@ -233,17 +242,6 @@ NS_IMETHODIMP nsUnicodeToISO2022JP::FinishNoBuff(char * aDest,
   return NS_OK;
 }
 
-NS_IMETHODIMP nsUnicodeToISO2022JP::GetMaxLength(const PRUnichar * aSrc, 
-                                                 PRInt32 aSrcLength,
-                                                 PRInt32 * aDestLength)
-{
-  // worst case
-  //  1  2 3  4  5  6  7 8
-  // ESC $ B XX XX ESC ( B
-  *aDestLength = 8*aSrcLength;
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsUnicodeToISO2022JP::Reset()
 {
   mCharset = 0;
