@@ -39,7 +39,7 @@
 #ifndef nsUnicodeToX11Johab_h___
 #define nsUnicodeToX11Johab_h___
 
-#include "nsUCvKOSupport.h"
+#include "nsUCSupport.h"
 
 //----------------------------------------------------------------------
 // Class nsUnicodeToX11Johab [declaration]
@@ -55,7 +55,7 @@ public:
    * Class constructor.
    */
   nsUnicodeToX11Johab();
-  ~nsUnicodeToX11Johab();
+  virtual ~nsUnicodeToX11Johab();
 
   NS_IMETHOD Convert(
       const PRUnichar * aSrc, PRInt32 * aSrcLength,
