@@ -74,7 +74,7 @@ nsBIG5ToUnicodeConstructor(nsISupports *aOuter, REFNSIID aIID,
                            void **aResult)
 {
   return CreateMultiTableDecoder(2, 
-                                 (uRange* ) &g_BIG5Ranges,
+                                 (const uRange* ) &g_BIG5Ranges,
                                  (uShiftTable**) &g_BIG5ShiftTableSet, 
                                  (uMappingTable**) &g_BIG5MappingTableSet, 1,
                                  aOuter, aIID, aResult);
