@@ -38,31 +38,24 @@
 
 #include "nsUnicodeToCNS11643p2.h"
 #include "nsUCvTW2Dll.h"
+#include "nsUCConstructors.h"
+
 
 //----------------------------------------------------------------------
 // Global functions and data [declaration]
 
-static PRInt16 g_ShiftTable[] =  {
+static const PRInt16 g_ShiftTable[] =  {
   0, u2BytesCharset,
   ShiftCell(0, 0, 0, 0, 0, 0, 0, 0),
 };
 
-//----------------------------------------------------------------------
-// Class nsUnicodeToCNS11643p2 [implementation]
-
-nsUnicodeToCNS11643p2::nsUnicodeToCNS11643p2() 
-: nsTableEncoderSupport((uShiftTable*) &g_ShiftTable, 
-                        (uMappingTable*) &g_ufCNS2MappingTable)
+NS_METHOD
+nsUnicodeToCNS11643p2Constructor(nsISupports* aOuter, REFNSIID aIID,
+                                 void **aResult)
 {
+  return CreateTableEncoder((uShiftTable*) &g_ShiftTable, 
+                            (uMappingTable*) &g_ufCNS2MappingTable,
+                            2 /* max length = src * 2 */,
+                            aOuter, aIID, aResult);
 }
 
-//----------------------------------------------------------------------
-// Subclassing of nsTableEncoderSupport class [implementation]
-
-NS_IMETHODIMP nsUnicodeToCNS11643p2::GetMaxLength(const PRUnichar * aSrc, 
-                                              PRInt32 aSrcLength,
-                                              PRInt32 * aDestLength)
-{
-  *aDestLength = 2 * aSrcLength;
-  return NS_OK_UENC_EXACTLENGTH;
-}
