@@ -95,6 +95,9 @@
     {0xad, 0x26, 0x11, 0x3f, 0x2c, 0x02, 0xd0, 0xfe} \
 }
 
+#define JSDS_MAJOR_VERSION 1
+#define JSDS_MINOR_VERSION 1
+
 #define NS_CATMAN_CTRID   "@mozilla.org/categorymanager;1"
 #define NS_JSRT_CTRID     "@mozilla.org/js/xpc/RuntimeService;1"
 
@@ -107,8 +110,9 @@ jsds_GCCallbackProc (JSContext *cx, JSGCStatus status);
 
 /*******************************************************************************
  * global vars
- *******************************************************************************/
+ ******************************************************************************/
 
+const char implementationString[] = "Mozilla JavaScript Debugger Service";
 static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
 static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
 
@@ -1861,6 +1865,10 @@ jsdStackFrame::Eval (const nsAString &bytes, const char *fileName,
                      PRUint32 line, jsdIValue **result, PRBool *_rval)
 {
     ASSERT_VALID_EPHEMERAL;
+
+    if (bytes.IsEmpty())
+        return NS_ERROR_INVALID_ARG;
+
     const nsSharedBufferHandle<PRUnichar> *h = bytes.GetSharedBufferHandle();
     const jschar *char_bytes = NS_REINTERPRET_CAST(const jschar *,
                                                    h->DataStart());
@@ -2370,6 +2378,29 @@ jsdService::SetFlags (PRUint32 flags)
     return NS_OK;
 }
 
+NS_IMETHODIMP
+jsdService::GetImplementationString(char **_rval)
+{
+    *_rval = PL_strdup(implementationString);
+    if (!*_rval)
+        return NS_ERROR_OUT_OF_MEMORY;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+jsdService::GetImplementationMajor(PRUint32 *_rval)
+{
+    *_rval = JSDS_MAJOR_VERSION;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+jsdService::GetImplementationMinor(PRUint32 *_rval)
+{
+    *_rval = JSDS_MINOR_VERSION;
+    return NS_OK;
+}
+
 NS_IMETHODIMP
 jsdService::GetIsOn (PRBool *_rval)
 {
