@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -191,7 +191,6 @@ js_alloc_atom(void *priv, const void *key)
     atom->entry.key = key;
     atom->entry.value = NULL;
     atom->flags = 0;
-    atom->kwindex = -1;
     atom->number = state->number++;
     return &atom->entry;
 }
@@ -217,7 +216,6 @@ static JSHashAllocOps atom_alloc_ops = {
 JSBool
 js_InitAtomState(JSContext *cx, JSAtomState *state)
 {
-    state->runtime = cx->runtime;
     state->table = JS_NewHashTable(JS_ATOM_HASH_SIZE, js_hash_atom_key,
                                    js_compare_atom_keys, js_compare_stub,
                                    &atom_alloc_ops, state);
@@ -225,6 +223,8 @@ js_InitAtomState(JSContext *cx, JSAtomState *state)
         JS_ReportOutOfMemory(cx);
         return JS_FALSE;
     }
+
+    state->runtime = cx->runtime;
 #ifdef JS_THREADSAFE
     js_InitLock(&state->lock);
     state->tablegen = 0;
@@ -303,7 +303,8 @@ js_InitPinnedAtoms(JSContext *cx, JSAtomState *state)
 void
 js_FreeAtomState(JSContext *cx, JSAtomState *state)
 {
-    JS_HashTableDestroy(state->table);
+    if (state->table)
+        JS_HashTableDestroy(state->table);
 #ifdef JS_THREADSAFE
     js_FinishLock(&state->lock);
 #endif
@@ -335,7 +336,7 @@ js_FinishAtomState(JSAtomState *state)
 {
     UninternArgs args;
 
-    if (!state->runtime)
+    if (!state->table)
         return;
     args.rt = state->runtime;
     args.leaks = 0;
@@ -383,6 +384,8 @@ js_MarkAtomState(JSAtomState *state, uintN gcflags, JSGCThingMarker mark,
 {
     MarkArgs args;
 
+    if (!state->table)
+        return;
     args.gcflags = gcflags;
     args.mark = mark;
     args.data = data;
@@ -412,7 +415,8 @@ void
 js_SweepAtomState(JSAtomState *state)
 {
     state->liveAtoms = 0;
-    JS_HashTableEnumerateEntries(state->table, js_atom_sweeper, state);
+    if (state->table)
+        JS_HashTableEnumerateEntries(state->table, js_atom_sweeper, state);
 }
 
 JS_STATIC_DLL_CALLBACK(intN)
@@ -428,7 +432,8 @@ js_atom_unpinner(JSHashEntry *he, intN i, void *arg)
 void
 js_UnpinPinnedAtoms(JSAtomState *state)
 {
-    JS_HashTableEnumerateEntries(state->table, js_atom_unpinner, NULL);
+    if (state->table)
+        JS_HashTableEnumerateEntries(state->table, js_atom_unpinner, NULL);
 }
 
 static JSAtom *
@@ -454,6 +459,7 @@ js_AtomizeHashedKey(JSContext *cx, jsval key, JSHashNumber keyHash, uintN flags)
 
     atom = (JSAtom *)he;
     atom->flags |= flags;
+    cx->lastAtom = atom;
 out:
     JS_UNLOCK(&state->lock,cx);
     return atom;
@@ -544,6 +550,7 @@ js_AtomizeDouble(JSContext *cx, jsdouble d, uintN flags)
 
     atom = (JSAtom *)he;
     atom->flags |= flags;
+    cx->lastAtom = atom;
 out:
     JS_UNLOCK(&state->lock,cx);
     return atom;
@@ -566,30 +573,36 @@ js_AtomizeString(JSContext *cx, JSString *str, uintN flags)
     table = state->table;
     hep = JS_HashTableRawLookup(table, keyHash, (void *)key);
     if ((he = *hep) == NULL) {
-        if (flags & ATOM_TMPSTR) {
 #ifdef JS_THREADSAFE
-            uint32 gen = state->tablegen;
+        uint32 gen = state->tablegen;
+        JS_UNLOCK(&state->lock, cx);
 #endif
-            JS_UNLOCK(&state->lock, cx);
+
+        if (flags & ATOM_TMPSTR) {
             str = (flags & ATOM_NOCOPY)
                   ? js_NewString(cx, str->chars, str->length, 0)
                   : js_NewStringCopyN(cx, str->chars, str->length, 0);
             if (!str)
                 return NULL;
             key = STRING_TO_JSVAL(str);
-            JS_LOCK(&state->lock, cx);
-#ifdef JS_THREADSAFE
-            if (state->tablegen != gen) {
-                hep = JS_HashTableRawLookup(table, keyHash, (void *)key);
-                if ((he = *hep) != NULL) {
-                    atom = (JSAtom *)he;
-                    if (flags & ATOM_NOCOPY)
-                        str->chars = NULL;
-                    goto out;
-                }
-            }
-#endif
+        } else {
+            if (!JS_MakeStringImmutable(cx, str))
+                return NULL;
         }
+
+#ifdef JS_THREADSAFE
+        JS_LOCK(&state->lock, cx);
+        if (state->tablegen != gen) {
+            hep = JS_HashTableRawLookup(table, keyHash, (void *)key);
+            if ((he = *hep) != NULL) {
+                atom = (JSAtom *)he;
+                if (flags & ATOM_NOCOPY)
+                    str->chars = NULL;
+                goto out;
+            }
+        }
+#endif
+
         he = JS_HashTableRawAdd(table, hep, keyHash, (void *)key, NULL);
         if (!he) {
             JS_ReportOutOfMemory(cx);
@@ -600,6 +613,7 @@ js_AtomizeString(JSContext *cx, JSString *str, uintN flags)
 
     atom = (JSAtom *)he;
     atom->flags |= flags & (ATOM_PINNED | ATOM_INTERNED);
+    cx->lastAtom = atom;
 out:
     JS_UNLOCK(&state->lock,cx);
     return atom;
