@@ -701,8 +701,9 @@ call_enumerate(JSContext *cx, JSObject *obj)
     JSStackFrame *fp;
     JSObject *funobj;
     JSScope *scope;
-    JSScopeProperty *sprop;
+    JSScopeProperty *sprop, *cprop;
     JSPropertyOp getter;
+    jsval *vec;
     JSProperty *prop;
 
     fp = (JSStackFrame *) JS_GetPrivate(cx, obj);
@@ -733,13 +734,19 @@ call_enumerate(JSContext *cx, JSObject *obj)
     scope = OBJ_SCOPE(funobj);
     for (sprop = SCOPE_LAST_PROP(scope); sprop; sprop = sprop->parent) {
         getter = sprop->getter;
-        if (getter != js_GetArgument && getter != js_GetLocalVariable)
+        if (getter == js_GetArgument)
+            vec = fp->argv;
+        else if (getter == js_GetLocalVariable)
+            vec = fp->vars;
+        else
             continue;
 
         /* Trigger reflection in call_resolve by doing a lookup. */
         if (!js_LookupProperty(cx, obj, sprop->id, &obj, &prop))
             return JS_FALSE;
         JS_ASSERT(obj && prop);
+        cprop = (JSScopeProperty *)prop;
+        LOCKED_OBJ_SET_SLOT(obj, cprop->slot, vec[sprop->shortid]);
         OBJ_DROP_PROPERTY(cx, obj, prop);
     }
 
@@ -865,7 +872,7 @@ fun_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
     jsint slot;
     JSFunction *fun;
     JSStackFrame *fp;
-#if defined XP_PC && defined _MSC_VER &&_MSC_VER <= 800
+#if defined _MSC_VER &&_MSC_VER <= 800
     /* MSVC1.5 coredumps */
     jsval bogus = *vp;
 #endif
@@ -937,7 +944,7 @@ fun_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
       default:
         /* XXX fun[0] and fun.arguments[0] are equivalent. */
         if (fp && fp->fun && (uintN)slot < fp->fun->nargs)
-#if defined XP_PC && defined _MSC_VER &&_MSC_VER <= 800
+#if defined _MSC_VER &&_MSC_VER <= 800
           /* MSVC1.5 coredumps */
           if (bogus == *vp)
 #endif
@@ -1074,7 +1081,7 @@ fun_xdrObject(JSXDRState *xdr, JSObject **objp)
     JSString *atomstr;
     char *propname;
     JSScopeProperty *sprop;
-    jsid userid;
+    uint32 userid;              /* NB: holds a signed int-tagged jsval */
     JSAtom *atom;
     uintN i, n, dupflag;
     uint32 type;
@@ -1150,7 +1157,7 @@ fun_xdrObject(JSXDRState *xdr, JSObject **objp)
                 userid = INT_TO_JSVAL(sprop->shortid);
                 propname = ATOM_BYTES((JSAtom *)sprop->id);
                 if (!JS_XDRUint32(xdr, &type) ||
-                    !JS_XDRUint32(xdr, (uint32 *)&userid) ||
+                    !JS_XDRUint32(xdr, &userid) ||
                     !JS_XDRCString(xdr, &propname)) {
                     if (mark)
                         JS_ARENA_RELEASE(&cx->tempPool, mark);
@@ -1166,7 +1173,7 @@ fun_xdrObject(JSXDRState *xdr, JSObject **objp)
                 uintN attrs = JSPROP_ENUMERATE | JSPROP_PERMANENT;
 
                 if (!JS_XDRUint32(xdr, &type) ||
-                    !JS_XDRUint32(xdr, (uint32 *)&userid) ||
+                    !JS_XDRUint32(xdr, &userid) ||
                     !JS_XDRCString(xdr, &propname)) {
                     return JS_FALSE;
                 }
