@@ -759,7 +759,7 @@ gc_dump_thing(JSGCThing *thing, uint8 flags, GCMarkNode *prev, FILE *fp)
                                   ? NULL
                                   : JSVAL_TO_PRIVATE(privateValue);
         const char *className = gc_object_class_name(thing);
-        fprintf(fp, "object %08p %s", privateThing, className);
+        fprintf(fp, "object %8p %s", privateThing, className);
         break;
       }
       case GCX_DOUBLE:
@@ -989,6 +989,7 @@ js_ForceGC(JSContext *cx, uintN gcflags)
 
     for (i = 0; i < GCX_NTYPES; i++)
         cx->newborn[i] = NULL;
+    cx->lastAtom = NULL;
     cx->runtime->gcPoke = JS_TRUE;
     js_GC(cx, gcflags);
     JS_ArenaFinish();
@@ -1261,6 +1262,8 @@ restart:
                 NULL);
         for (i = GCX_EXTERNAL_STRING; i < GCX_NTYPES; i++)
             GC_MARK(cx, acx->newborn[i], "newborn external string", NULL);
+        if (acx->lastAtom)
+            GC_MARK_ATOM(cx, acx->lastAtom, NULL);
 #if JS_HAS_EXCEPTIONS
         if (acx->throwing && JSVAL_IS_GCTHING(acx->exception))
             GC_MARK(cx, JSVAL_TO_GCTHING(acx->exception), "exception", NULL);
@@ -1277,16 +1280,8 @@ restart:
         }
     }
 
-#define GC_CALLBACK(status)                                                   \
-    if (rt->gcCallback) {                                                     \
-        if (status == JSGC_END && gcflags & GC_ALREADY_LOCKED)                \
-            JS_UNLOCK_GC(rt);                                                 \
-        (void) rt->gcCallback(cx, status);                                    \
-        if (status == JSGC_END && gcflags & GC_ALREADY_LOCKED)                \
-            JS_LOCK_GC(rt);                                                   \
-    }
-
-    GC_CALLBACK(JSGC_MARK_END);
+    if (rt->gcCallback)
+        (void) rt->gcCallback(cx, JSGC_MARK_END);
 
     /*
      * Sweep phase.
@@ -1378,7 +1373,8 @@ restart:
     /* Terminate the new freelist. */
     *flp = NULL;
 
-    GC_CALLBACK(JSGC_FINALIZE_END);
+    if (rt->gcCallback)
+        (void) rt->gcCallback(cx, JSGC_FINALIZE_END);
 
 out:
     JS_LOCK_GC(rt);
@@ -1402,7 +1398,11 @@ out:
         JS_UNLOCK_GC(rt);
 #endif
 
-    GC_CALLBACK(JSGC_END);
-
-#undef GC_CALLBACK
+    if (rt->gcCallback) {
+        if (gcflags & GC_ALREADY_LOCKED)
+            JS_UNLOCK_GC(rt);
+        (void) rt->gcCallback(cx, JSGC_END);
+        if (gcflags & GC_ALREADY_LOCKED)
+            JS_LOCK_GC(rt);
+    }
 }
