@@ -66,7 +66,8 @@
  * attributes, tiny or short id, and a field telling for..in order.  Note that
  * labels are not unique in the tree, but they are unique among a node's kids
  * (barring rare and benign multi-threaded race condition outcomes, see below)
- * and along any ancestor line from the tree root to a given leaf node.
+ * and along any ancestor line from the tree root to a given leaf node (except
+ * for the hard case of duplicate formal parameters to a function).
  *
  * Thus the root of the tree represents all empty scopes, and the first ply
  * of the tree represents all scopes containing one property, etc.  Each node
@@ -106,10 +107,8 @@
  * What if we add Y again?  X->Y->Z->Y is wrong and we'll enumerate Y twice.
  * Therefore we must fork in such a case, if not earlier.  Because delete is
  * "bursty", we should not fork eagerly.  Delaying a fork till we are at risk
- * of adding Y after it was deleted already requires a flag in the JSScope, at
- * least.  So we tag the scope->lastProp pointer with SCOPE_MIDDLE_DELETE_TAG,
- * to avoid expanding the size of JSScope, and to penalize only the code that
- * already must care about forking (js_{Add,Remove}ScopeProperty).
+ * of adding Y after it was deleted already requires a flag in the JSScope, to
+ * wit, SCOPE_MIDDLE_DELETE.
  *
  * What about thread safety?  If the property tree operations done by requests
  * are find-node and insert-node, then the only hazard is duplicate insertion.
@@ -198,8 +197,8 @@
 struct JSScope {
     JSObjectMap     map;                /* base class state */
     JSObject        *object;            /* object that owns this scope */
+    uint16          flags;              /* flags, see below */
     int16           hashShift;          /* multiplicative hash shift */
-    int16           sizeLog2;           /* log2(table size) */
     uint32          entryCount;         /* number of entries in table */
     uint32          removedCount;       /* removed entry sentinels in table */
     JSScopeProperty **table;            /* table of ptrs to shared tree nodes */
@@ -220,28 +219,28 @@ struct JSScope {
 
 #define OBJ_SCOPE(obj)                  ((JSScope *)(obj)->map)
 
-#define SCOPE_MIDDLE_DELETE_TAG         ((jsuword)1) 
-#define SCOPE_LAST_PROP_WORD(scope)     ((jsuword)(scope)->lastProp)
+/* By definition, hashShift = JS_DHASH_BITS - log2(capacity). */
+#define SCOPE_CAPACITY(scope)           JS_BIT(JS_DHASH_BITS-(scope)->hashShift)
 
-#define SCOPE_LAST_PROP(scope)                                                \
-    ((JSScopeProperty *) (SCOPE_LAST_PROP_WORD(scope) &                       \
-                          ~SCOPE_MIDDLE_DELETE_TAG))
+/* Scope flags and some macros to hide them from other files than jsscope.c. */
+#define SCOPE_MIDDLE_DELETE             0x0001
+#define SCOPE_SEALED                    0x0002
 
-#define SCOPE_HAD_MIDDLE_DELETE(scope)                                        \
-    (SCOPE_LAST_PROP_WORD(scope) & SCOPE_MIDDLE_DELETE_TAG)
+#define SCOPE_HAD_MIDDLE_DELETE(scope)  ((scope)->flags & SCOPE_MIDDLE_DELETE)
+#define SCOPE_SET_MIDDLE_DELETE(scope)  ((scope)->flags |= SCOPE_MIDDLE_DELETE)
+#define SCOPE_CLR_MIDDLE_DELETE(scope)  ((scope)->flags &= ~SCOPE_MIDDLE_DELETE)
 
-#define SCOPE_SET_LAST_PROP(scope, sprop)                                     \
-    ((scope)->lastProp = (JSScopeProperty *)                                  \
-                         ((jsuword) (sprop) |                                 \
-                          SCOPE_HAD_MIDDLE_DELETE(scope)))
+#define SCOPE_IS_SEALED(scope)          ((scope)->flags & SCOPE_SEALED)
+#define SCOPE_SET_SEALED(scope)         ((scope)->flags |= SCOPE_SEALED)
+#define SCOPE_CLR_SEALED(scope)         ((scope)->flags &= ~SCOPE_SEALED)
 
-#define SCOPE_REMOVE_LAST_PROP(scope)                                         \
-    SCOPE_SET_LAST_PROP(scope, SCOPE_LAST_PROP(scope)->parent)
-
-#define SCOPE_SET_MIDDLE_DELETE(scope)                                        \
-    ((scope)->lastProp = (JSScopeProperty *)                                  \
-                         (SCOPE_LAST_PROP_WORD(scope) |                       \
-                          SCOPE_MIDDLE_DELETE_TAG))
+/*
+ * A little information hiding for scope->lastProp, in case it ever becomes
+ * a tagged pointer again.
+ */
+#define SCOPE_LAST_PROP(scope)          ((scope)->lastProp)
+#define SCOPE_REMOVE_LAST_PROP(scope)   ((scope)->lastProp =                  \
+                                         (scope)->lastProp->parent)
 
 struct JSScopeProperty {
     jsid            id;                 /* int-tagged jsval/untagged JSAtom* */
@@ -304,12 +303,16 @@ struct JSScopeProperty {
 
 #define SPROP_GET(cx,sprop,obj,obj2,vp)                                       \
     (((sprop)->attrs & JSPROP_GETTER)                                         \
-     ? js_InternalCall(cx, obj, OBJECT_TO_JSVAL((sprop)->getter), 0, 0, vp)   \
+     ? js_InternalGetOrSet(cx, obj, (sprop)->id,                              \
+                           OBJECT_TO_JSVAL((sprop)->getter), JSACC_READ,      \
+                           0, 0, vp)                                          \
      : SPROP_CALL_GETTER(cx, sprop, (sprop)->getter, obj, obj2, vp))
 
 #define SPROP_SET(cx,sprop,obj,obj2,vp)                                       \
     (((sprop)->attrs & JSPROP_SETTER)                                         \
-     ? js_InternalCall(cx, obj, OBJECT_TO_JSVAL((sprop)->setter), 1, vp, vp)  \
+     ? js_InternalGetOrSet(cx, obj, (sprop)->id,                              \
+                           OBJECT_TO_JSVAL((sprop)->setter), JSACC_WRITE,     \
+                           1, vp, vp)                                         \
      : ((sprop)->attrs & JSPROP_GETTER)                                       \
      ? (JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL,                    \
                              JSMSG_GETTER_ONLY, NULL), JS_FALSE)              \
