@@ -58,7 +58,7 @@
 // all this crap is needed to do the interactive shell stuff
 #include <stdlib.h>
 #include <errno.h>
-#ifdef XP_PC
+#if defined(XP_WIN) || defined(XP_OS2)
 #include <io.h>     /* for isatty() */
 #elif defined(XP_UNIX) || defined(XP_BEOS)
 #include <unistd.h>     /* for isatty() */
@@ -73,8 +73,6 @@
 #include "jsarena.h"
 #include "jscntxt.h"
 
-#include "nsSpecialSystemDirectory.h"	// For exe dir
-
 #include "nsIJSContextStack.h"
 
 /***************************************************************************/
@@ -196,7 +194,7 @@ Dump(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
         return JS_FALSE;
 
     char *bytes = JS_GetStringBytes(str);
-    bytes = nsCRT::strdup(bytes);
+    bytes = strdup(bytes);
 
 #ifdef XP_MAC
     for (char *c = bytes; *c; c++)
@@ -423,6 +421,7 @@ GetLine(JSContext *cx, char *bufp, FILE *fh, const char *prompt) {
     {
         char line[256];
         fprintf(gOutFile, prompt);
+        fflush(gOutFile);
         if (fgets(line, 256, fh) == NULL)
             return JS_FALSE;
         strcpy(bufp, line);
