@@ -137,10 +137,6 @@ STDMETHODIMP_(ULONG) _class::Release(void)                                    \
   return mRefCnt;                                                             \
 }
 
-#ifndef NS_MT_SUPPORTED
-#error "We need MT support
-#endif
-
 extern const nsID NSID_IDISPATCH;
 
 /**
@@ -775,6 +771,16 @@ public:
          * @return true if this is a property
          */
         PRBool IsProperty() const;
+        /**
+         * Is this a parameterized setter
+         * @return true if this is a parameterized property
+         */
+        PRBool IsParameterizedSetter() const;
+        /**
+         * Is this a parameterized getter
+         * @return true if this is a parameterized property
+         */
+        PRBool IsParameterizedGetter() const;
         /**
          * Is this a parameterized property
          * @return true if this is a parameterized property
@@ -806,15 +812,17 @@ public:
         PRUint32 GetDispID() const;
         /**
          * returns the number of parameters of the method
+         * @param Ask from getter instead of setter version of the function
          * @return the number of parameters of the method
          */
-        PRUint32 GetParamCount() const;
+        PRUint32 GetParamCount(PRBool getter = PR_FALSE) const;
         /**
          * Returns parameter information for the specified parameter
          * @param index the index of the parameter
+         * @param Ask from getter instead of setter version of the function
          * @return Parameter information
          */
-        ParamInfo GetParamInfo(PRUint32 index);
+        ParamInfo GetParamInfo(PRUint32 index, PRBool getter = PR_FALSE) const;
         // === Setup functions ===
         /**
          * Sets the name of the method
@@ -847,6 +855,12 @@ public:
          */ 
         void SetTypeInfo(DISPID dispID, ITypeInfo* pTypeInfo,
                          FUNCDESC* funcdesc);
+        /**
+         * Sets the function description for the getter version of the function.
+         * @param funcdesc function description
+         */
+        void SetGetterFuncDesc(FUNCDESC* funcdesc);
+
     private:
         /**
          * Our internal flags identify the type of member
@@ -864,7 +878,9 @@ public:
         jsval mVal;     // Mutable
         jsval mName;    // Mutable
         CComPtr<ITypeInfo> mTypeInfo;
-        FUNCDESC* mFuncDesc; // We keep hold on this so we don't have 
+        FUNCDESC* mFuncDesc;         // We own this, and must release it
+        FUNCDESC* mGetterFuncDesc;   // We own this, and must release it
+                                     // This may be the same as mFuncDesc
         /**
          * Helper function to return the parameter type
          * @param index index of the parameter to return the type of
@@ -1003,6 +1019,7 @@ public:
     JSBool Invoke(XPCCallContext & ccx, CallMode mode);
     /**
      * Instantiates a COM object given a class ID or a prog ID
+     * @param ccx an XPConnect call context
      * @param className a prog ID or a class ID in the form of 
      * {00000000-0000-0000-0000-000000000000}
      * @param enforceSecurity if true, will apply checks to ensure
@@ -1011,8 +1028,8 @@ public:
      * @param result pointer to the pointer to receive the interface pointer
      */
     static
-    HRESULT COMCreateInstance(BSTR className, 
-                               PRBool enforceSecurity, IDispatch ** result);
+    HRESULT COMCreateInstance(XPCCallContext & ccx, BSTR className,
+                              PRBool enforceSecurity, IDispatch ** result);
     /**
      * Wraps an IDispatch interface, returning the object as a jsval
      * @param pDispatch IDispatch pointer
