@@ -138,6 +138,20 @@ XPCDispatchTearOff::~XPCDispatchTearOff()
 NS_COM_IMPL_ADDREF(XPCDispatchTearOff)
 NS_COM_IMPL_RELEASE(XPCDispatchTearOff)
 
+// See bug 127982:
+//
+// Microsoft's InlineIsEqualGUID global function is multiply defined
+// in ATL and/or SDKs with varying namespace requirements. To save the control
+// from future grief, this method is used instead. 
+static inline BOOL _IsEqualGUID(REFGUID rguid1, REFGUID rguid2)
+{
+   return (
+	  ((PLONG) &rguid1)[0] == ((PLONG) &rguid2)[0] &&
+	  ((PLONG) &rguid1)[1] == ((PLONG) &rguid2)[1] &&
+	  ((PLONG) &rguid1)[2] == ((PLONG) &rguid2)[2] &&
+	  ((PLONG) &rguid1)[3] == ((PLONG) &rguid2)[3]);
+}
+
 STDMETHODIMP XPCDispatchTearOff::InterfaceSupportsErrorInfo(REFIID riid)
 {
     static const IID* arr[] = 
@@ -147,7 +161,7 @@ STDMETHODIMP XPCDispatchTearOff::InterfaceSupportsErrorInfo(REFIID riid)
 
     for(int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
     {
-        if(InlineIsEqualGUID(*arr[i],riid))
+        if(_IsEqualGUID(*arr[i],riid))
             return S_OK;
     }
     return S_FALSE;
@@ -310,30 +324,24 @@ STDMETHODIMP XPCDispatchTearOff::Invoke(DISPID dispIdMember, REFIID riid,
         uint8 paramCount=0;
         nsresult retval = NS_ERROR_FAILURE;
         nsresult pending_result = NS_OK;
-        JSErrorReporter older;
         JSBool success;
         JSBool readyToDoTheCall = JS_FALSE;
         uint8 outConversionFailedIndex;
         JSObject* obj;
         jsval fval;
         nsCOMPtr<nsIException> xpc_exception;
-        jsval js_exception;
         void* mark;
         JSBool foundDependentParam;
         JSObject* thisObj;
-        JSExceptionState* saved_exception = nsnull;
+        AutoScriptEvaluate scriptEval(ccx);
         XPCJSRuntime* rt = ccx.GetRuntime();
 
         thisObj = obj = GetJSObject();;
 
-        if(cx)
-            older = JS_SetErrorReporter(cx, xpcWrappedJSErrorReporter);
-
-        // dispID's for us are 1 based not zero
         if(!cx || !xpcc)
             goto pre_call_clean_up;
 
-        saved_exception = xpc_DoPreScriptEvaluated(cx);
+        scriptEval.StartEvaluating(xpcWrappedJSErrorReporter);
 
         xpcc->SetPendingResult(pending_result);
         xpcc->SetException(nsnull);
@@ -478,122 +486,11 @@ pre_call_clean_up:
                 JS_smprintf_free(sz);
         }
 
-        /* this one would be set by our error reporter */
-        xpcc->GetException(getter_AddRefs(xpc_exception));
-        if(xpc_exception)
-            xpcc->SetException(nsnull);
-
-        // get this right away in case we do something below to cause JS code
-        // to run on this JSContext
-        pending_result = xpcc->GetPendingResult();
-
-        /* JS might throw an expection whether the reporter was called or not */
-        if(JS_GetPendingException(cx, &js_exception))
+        if (!success)
         {
-            if(!xpc_exception)
-                XPCConvert::JSValToXPCException(ccx, js_exception, "IDispatch",
-                                                name.get(), getter_AddRefs(xpc_exception));
-
-            /* cleanup and set failed even if we can't build an exception */
-            if(!xpc_exception)
-            {
-                ccx.GetThreadData()->SetException(nsnull); // XXX necessary?
-                success = JS_FALSE;
-            }
-            JS_ClearPendingException(cx);
-        }
-        // TODO: This exception logic needs to be refactored and shared with wrapped JS
-        if(xpc_exception)
-        {
-            nsresult e_result;
-            if(NS_SUCCEEDED(xpc_exception->GetResult(&e_result)))
-            {
-                if(xpc_IsReportableErrorCode(e_result))
-                {
-
-                    // Log the exception to the JS Console, so that users can do
-                    // something with it.
-                    nsCOMPtr<nsIConsoleService> consoleService
-                        (do_GetService(XPC_CONSOLE_CONTRACTID));
-                    if(nsnull != consoleService)
-                    {
-                        nsresult rv;
-                        nsCOMPtr<nsIScriptError> scriptError;
-                        nsCOMPtr<nsISupports> errorData;
-                        rv = xpc_exception->GetData(getter_AddRefs(errorData));
-                        if(NS_SUCCEEDED(rv))
-                            scriptError = do_QueryInterface(errorData);
-
-                        if(nsnull == scriptError)
-                        {
-                            // No luck getting one from the exception, so
-                            // try to cook one up.
-                            scriptError = do_CreateInstance(XPC_SCRIPT_ERROR_CONTRACTID);
-                            if(nsnull != scriptError)
-                            {
-                                char* exn_string;
-                                rv = xpc_exception->ToString(&exn_string);
-                                if(NS_SUCCEEDED(rv))
-                                {
-                                    // use toString on the exception as the message
-                                    nsAutoString newMessage;
-                                    newMessage.AssignWithConversion(exn_string);
-                                    nsMemory::Free((void *) exn_string);
-
-                                    // try to get filename, lineno from the first
-                                    // stack frame location.
-                                    PRUnichar* sourceNameUni = nsnull;
-                                    PRInt32 lineNumber = 0;
-                                    nsXPIDLCString sourceName;
-
-                                    nsCOMPtr<nsIStackFrame> location;
-                                    xpc_exception->
-                                        GetLocation(getter_AddRefs(location));
-                                    if(location)
-                                    {
-                                        // Get line number w/o checking; 0 is ok.
-                                        location->GetLineNumber(&lineNumber);
-
-                                        // get a filename.
-                                        rv = location->GetFilename(getter_Copies(sourceName));
-                                    }
-
-                                    rv = scriptError->Init(newMessage.get(),
-                                                           NS_ConvertASCIItoUCS2(sourceName).get(),
-                                                           nsnull,
-                                                           lineNumber, 0, 0,
-                                                           "XPConnect JavaScript");
-                                    if(NS_FAILED(rv))
-                                        scriptError = nsnull;
-                                }
-                            }
-                        }
-                        if(nsnull != scriptError)
-                            consoleService->LogMessage(scriptError);
-                    }
-                }
-                // Whether or not it passes the 'reportable' test, it might
-                // still be an error and we have to do the right thing here...
-                if(NS_FAILED(e_result))
-                {
-                    ccx.GetThreadData()->SetException(xpc_exception);
-                    SetCOMError(xpc_exception);
-                    retval = e_result;
-                }
-            }
-            success = JS_FALSE;
-        }
-        else
-        {
-            // see if JS code signaled failure result without throwing exception
-            if(NS_FAILED(pending_result))
-            {
-                retval = pending_result;
-                success = JS_FALSE;
-            }
-        }
-        if(!success)
+            retval = nsXPCWrappedJSClass::CheckForException(ccx, name.get(), "IDispatch");
             goto done;
+        }
 
         ccx.GetThreadData()->SetException(nsnull); // XXX necessary?
 
@@ -646,11 +543,6 @@ done:
         if(sp)
             js_FreeStack(cx, mark);
 
-        if(cx)
-        {
-            JS_SetErrorReporter(cx, older);
-            xpc_DoPostScriptEvaluated(cx, saved_exception);
-        }
         // TODO: I think we may need to translate this error, 
         // for now we'll pass through
         return retval;
