@@ -62,8 +62,6 @@ nsXPConnect::nsXPConnect()
         mDefaultSecurityManagerFlags(0),
         mShuttingDown(JS_FALSE)
 {
-    NS_INIT_ISUPPORTS();
-
     // Ignore the result. If the runtime service is not ready to rumble
     // then we'll set this up later as needed.
     CreateRuntime();
@@ -78,13 +76,12 @@ nsXPConnect::nsXPConnect()
 
 #ifdef XPC_TOOLS_SUPPORT
   {
-    nsDependentCString filename(PR_GetEnv("MOZILLA_JS_PROFILER_OUTPUT"));
-    if(!filename.IsEmpty())
+    char* filename = PR_GetEnv("MOZILLA_JS_PROFILER_OUTPUT");
+    if(filename && *filename)
     {
-
         mProfilerOutputFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
         if(mProfilerOutputFile &&
-           NS_SUCCEEDED(mProfilerOutputFile->InitWithNativePath(filename)))
+           NS_SUCCEEDED(mProfilerOutputFile->InitWithNativePath(nsDependentCString(filename))))
         {
             mProfiler = do_GetService(XPCTOOLS_PROFILER_CONTRACTID);
             if(mProfiler)
@@ -93,7 +90,7 @@ nsXPConnect::nsXPConnect()
                 {
 #ifdef DEBUG
                     printf("***** profiling JavaScript. Output to: %s\n",
-                           filename.get());
+                           filename);
 #endif
                 }
             }
@@ -441,6 +438,10 @@ nsXPConnect::InitClasses(JSContext * aJSContext, JSObject * aGlobalJSObj)
     if(!nsXPCComponents::AttachNewComponentsObject(ccx, scope, aGlobalJSObj))
         return UnexpectedFailure(NS_ERROR_FAILURE);
 
+#ifdef XPC_IDISPATCH_SUPPORT
+    // Initialize any properties IDispatch needs on the global object
+    XPCIDispatchExtension::Initialize(ccx, aGlobalJSObj);
+#endif
     return NS_OK;
 }
 
@@ -1062,6 +1063,7 @@ nsXPConnect::ReleaseJSContext(JSContext * aJSContext, PRBool noGC)
                    aJSContext);
 #endif
             ccx->SetDestroyJSContextInDestructor(JS_TRUE);
+            JS_ClearNewbornRoots(aJSContext);
             return NS_OK;
         }
         // else continue on and synchronously destroy the JSContext ...
@@ -1086,7 +1088,7 @@ nsXPConnect::DebugDump(PRInt16 depth)
 {
 #ifdef DEBUG
     depth-- ;
-    XPC_LOG_ALWAYS(("nsXPConnect @ %x with mRefCnt = %d", this, mRefCnt));
+    XPC_LOG_ALWAYS(("nsXPConnect @ %x with mRefCnt = %d", this, mRefCnt.get()));
     XPC_LOG_INDENT();
         XPC_LOG_ALWAYS(("gSelf @ %x", gSelf));
         XPC_LOG_ALWAYS(("gOnceAliveNowDead is %d", (int)gOnceAliveNowDead));
