@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -70,39 +70,30 @@ JSValIsInterfaceOfType(JSContext *cx, jsval v, REFNSIID iid)
 }
 
 #ifdef XPC_USE_SECURITY_CHECKED_COMPONENT
-static char* CloneAllAccess()
+char* xpc_CloneAllAccess()
 {
     static const char allAccess[] = "AllAccess";
     return (char*)nsMemory::Clone(allAccess, sizeof(allAccess));
 }
+
+char * xpc_CheckAccessList(const PRUnichar* wideName, const char* list[])
+{
+    nsCAutoString asciiName;   
+    CopyUCS2toASCII(nsDependentString(wideName), asciiName);
+
+    for(const char** p = list; *p; p++)
+        if(!strcmp(*p, asciiName.get()))
+            return xpc_CloneAllAccess();
+    
+    return nsnull;
+}
 #endif
 
 /***************************************************************************/
 
-class nsXPCComponents_Interfaces :
-            public nsIXPCComponents_Interfaces,
-            public nsIXPCScriptable
-#ifdef XPC_USE_SECURITY_CHECKED_COMPONENT
-          , public nsISecurityCheckedComponent
-#endif
-{
-public:
-    // all the interface method declarations...
-    NS_DECL_ISUPPORTS
-    NS_DECL_NSIXPCCOMPONENTS_INTERFACES
-    NS_DECL_NSIXPCSCRIPTABLE
-#ifdef XPC_USE_SECURITY_CHECKED_COMPONENT
-    NS_DECL_NSISECURITYCHECKEDCOMPONENT
-#endif
-
-public:
-    nsXPCComponents_Interfaces();
-    virtual ~nsXPCComponents_Interfaces();
-};
-
 nsXPCComponents_Interfaces::nsXPCComponents_Interfaces()
 {
-    NS_INIT_ISUPPORTS();
+    mManager = dont_AddRef(XPTI_GetInterfaceInfoManager());
 }
 
 nsXPCComponents_Interfaces::~nsXPCComponents_Interfaces()
@@ -111,13 +102,26 @@ nsXPCComponents_Interfaces::~nsXPCComponents_Interfaces()
 }
 
 
+/* [noscript] attribute nsIInterfaceInfoManager manager; */
+NS_IMETHODIMP nsXPCComponents_Interfaces::GetManager(nsIInterfaceInfoManager * *aManager)
+{
+    *aManager = mManager;
+    NS_IF_ADDREF(*aManager);
+    return NS_OK;
+}
+NS_IMETHODIMP nsXPCComponents_Interfaces::SetManager(nsIInterfaceInfoManager * aManager)
+{
+    mManager = aManager;
+    return NS_OK;
+}
+
 NS_INTERFACE_MAP_BEGIN(nsXPCComponents_Interfaces)
-  NS_INTERFACE_MAP_ENTRY(nsIXPCComponents_Interfaces)
+  NS_INTERFACE_MAP_ENTRY(nsIScriptableInterfaces)
   NS_INTERFACE_MAP_ENTRY(nsIXPCScriptable)
 #ifdef XPC_USE_SECURITY_CHECKED_COMPONENT
   NS_INTERFACE_MAP_ENTRY(nsISecurityCheckedComponent)
 #endif
-  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIXPCComponents_Interfaces)
+  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIScriptableInterfaces)
 NS_INTERFACE_MAP_END_THREADSAFE
 
 NS_IMPL_THREADSAFE_ADDREF(nsXPCComponents_Interfaces)
@@ -146,10 +150,8 @@ nsXPCComponents_Interfaces::NewEnumerate(nsIXPConnectWrappedNative *wrapper,
     {
         case JSENUMERATE_INIT:
         {
-            nsCOMPtr<nsIInterfaceInfoManager>
-                iim(dont_AddRef(XPTI_GetInterfaceInfoManager()));
-
-            if(!iim || NS_FAILED(iim->EnumerateInterfaces(&e)) || !e ||
+            if(!mManager || 
+               NS_FAILED(mManager->EnumerateInterfaces(&e)) || !e ||
                NS_FAILED(e->First()))
 
             {
@@ -170,7 +172,7 @@ nsXPCComponents_Interfaces::NewEnumerate(nsIXPConnectWrappedNative *wrapper,
             
             while(1)
             {
-                if(NS_COMFALSE == e->IsDone() &&
+                if(NS_ENUMERATOR_FALSE == e->IsDone() &&
                    NS_SUCCEEDED(e->CurrentItem(getter_AddRefs(isup))) && isup)
                 {
                     e->Next();
@@ -219,12 +221,19 @@ nsXPCComponents_Interfaces::NewResolve(nsIXPConnectWrappedNative *wrapper,
 {
     const char* name = nsnull;
 
-    if(JSVAL_IS_STRING(id) &&
+    if(mManager &&
+       JSVAL_IS_STRING(id) &&
        nsnull != (name = JS_GetStringBytes(JSVAL_TO_STRING(id))) &&
        name[0] != '{') // we only allow interfaces by name here
     {
-        nsCOMPtr<nsIJSIID> nsid =
-            dont_AddRef(NS_STATIC_CAST(nsIJSIID*,nsJSIID::NewID(name)));
+        nsCOMPtr<nsIInterfaceInfo> info;
+        mManager->GetInfoForName(name, getter_AddRefs(info));
+        if(!info)
+            return NS_OK;
+
+        nsCOMPtr<nsIJSIID> nsid = 
+            dont_AddRef(NS_STATIC_CAST(nsIJSIID*, nsJSIID::NewID(info)));
+
         if(nsid)
         {
             nsCOMPtr<nsIXPConnect> xpc;
@@ -265,7 +274,7 @@ NS_IMETHODIMP
 nsXPCComponents_Interfaces::CanCreateWrapper(const nsIID * iid, char **_retval)
 {
     // We let anyone do this...
-    *_retval = CloneAllAccess();
+    *_retval = xpc_CloneAllAccess();
     return NS_OK;
 }
 
@@ -317,7 +326,6 @@ public:
 
 nsXPCComponents_Classes::nsXPCComponents_Classes()
 {
-    NS_INIT_ISUPPORTS();
 }
 
 nsXPCComponents_Classes::~nsXPCComponents_Classes()
@@ -351,17 +359,15 @@ nsXPCComponents_Classes::NewEnumerate(nsIXPConnectWrappedNative *wrapper,
                                       PRUint32 enum_op, jsval * statep,
                                       jsid * idp, PRBool *_retval)
 {
-    nsIEnumerator* e;
+    nsISimpleEnumerator* e;
 
     switch(enum_op)
     {
         case JSENUMERATE_INIT:
         {
-            nsIComponentManager* compMgr;
-            if(NS_FAILED(NS_GetGlobalComponentManager(&compMgr)) ||
-               !compMgr || NS_FAILED(compMgr->EnumerateContractIDs(&e)) || !e ||
-               NS_FAILED(e->First()))
-
+            nsCOMPtr<nsIComponentRegistrar> compMgr;            
+            if(NS_FAILED(NS_GetComponentRegistrar(getter_AddRefs(compMgr))) || !compMgr || 
+               NS_FAILED(compMgr->EnumerateContractIDs(&e)) || !e )
             {
                 *statep = JSVAL_NULL;
                 return NS_ERROR_UNEXPECTED;
@@ -375,20 +381,19 @@ nsXPCComponents_Classes::NewEnumerate(nsIXPConnectWrappedNative *wrapper,
         case JSENUMERATE_NEXT:
         {
             nsCOMPtr<nsISupports> isup;
-
-            e = (nsIEnumerator*) JSVAL_TO_PRIVATE(*statep);
-            if(NS_COMFALSE == e->IsDone() &&
-               NS_SUCCEEDED(e->CurrentItem(getter_AddRefs(isup))) && isup)
+            PRBool hasMore;
+            e = (nsISimpleEnumerator*) JSVAL_TO_PRIVATE(*statep);
+ 
+            if(NS_SUCCEEDED(e->HasMoreElements(&hasMore)) && hasMore &&
+               NS_SUCCEEDED(e->GetNext(getter_AddRefs(isup))) && isup)
             {
-                e->Next();
-                nsCOMPtr<nsISupportsString> holder(do_QueryInterface(isup));
+                nsCOMPtr<nsISupportsCString> holder(do_QueryInterface(isup));
                 if(holder)
                 {
-                    char* name;
-                    if(NS_SUCCEEDED(holder->GetData(&name)) && name)
+                    nsCAutoString name;
+                    if(NS_SUCCEEDED(holder->GetData(name)))
                     {
-                        JSString* idstr = JS_NewStringCopyZ(cx, name);
-                        nsMemory::Free(name);
+                        JSString* idstr = JS_NewStringCopyN(cx, name.get(), name.Length());
                         if(idstr &&
                            JS_ValueToId(cx, STRING_TO_JSVAL(idstr), idp))
                         {
@@ -402,7 +407,7 @@ nsXPCComponents_Classes::NewEnumerate(nsIXPConnectWrappedNative *wrapper,
 
         case JSENUMERATE_DESTROY:
         default:
-            e = (nsIEnumerator*) JSVAL_TO_PRIVATE(*statep);
+            e = (nsISimpleEnumerator*) JSVAL_TO_PRIVATE(*statep);
             NS_IF_RELEASE(e);
             *statep = JSVAL_NULL;
             return NS_OK;
@@ -478,7 +483,6 @@ public:
 
 nsXPCComponents_ClassesByID::nsXPCComponents_ClassesByID()
 {
-    NS_INIT_ISUPPORTS();
 }
 
 nsXPCComponents_ClassesByID::~nsXPCComponents_ClassesByID()
@@ -511,17 +515,15 @@ nsXPCComponents_ClassesByID::NewEnumerate(nsIXPConnectWrappedNative *wrapper,
                                           PRUint32 enum_op, jsval * statep,
                                           jsid * idp, PRBool *_retval)
 {
-    nsIEnumerator* e;
+    nsISimpleEnumerator* e;
 
     switch(enum_op)
     {
         case JSENUMERATE_INIT:
         {
-            nsIComponentManager* compMgr;
-            if(NS_FAILED(NS_GetGlobalComponentManager(&compMgr)) ||
-               !compMgr || NS_FAILED(compMgr->EnumerateCLSIDs(&e)) || !e ||
-               NS_FAILED(e->First()))
-
+            nsCOMPtr<nsIComponentRegistrar> compMgr;            
+            if(NS_FAILED(NS_GetComponentRegistrar(getter_AddRefs(compMgr))) || !compMgr || 
+               NS_FAILED(compMgr->EnumerateCIDs(&e)) || !e )
             {
                 *statep = JSVAL_NULL;
                 return NS_ERROR_UNEXPECTED;
@@ -535,12 +537,12 @@ nsXPCComponents_ClassesByID::NewEnumerate(nsIXPConnectWrappedNative *wrapper,
         case JSENUMERATE_NEXT:
         {
             nsCOMPtr<nsISupports> isup;
+            PRBool hasMore;
+            e = (nsISimpleEnumerator*) JSVAL_TO_PRIVATE(*statep);
 
-            e = (nsIEnumerator*) JSVAL_TO_PRIVATE(*statep);
-            if(NS_COMFALSE == e->IsDone() &&
-               NS_SUCCEEDED(e->CurrentItem(getter_AddRefs(isup))) && isup)
+            if(NS_SUCCEEDED(e->HasMoreElements(&hasMore)) && hasMore &&
+               NS_SUCCEEDED(e->GetNext(getter_AddRefs(isup))) && isup)
             {
-                e->Next();
                 nsCOMPtr<nsISupportsID> holder(do_QueryInterface(isup));
                 if(holder)
                 {
@@ -562,7 +564,7 @@ nsXPCComponents_ClassesByID::NewEnumerate(nsIXPConnectWrappedNative *wrapper,
 
         case JSENUMERATE_DESTROY:
         default:
-            e = (nsIEnumerator*) JSVAL_TO_PRIVATE(*statep);
+            e = (nsISimpleEnumerator*) JSVAL_TO_PRIVATE(*statep);
             NS_IF_RELEASE(e);
             *statep = JSVAL_NULL;
             return NS_OK;
@@ -578,9 +580,9 @@ IsRegisteredCLSID(const char* str)
     if(!id.Parse(str))
         return PR_FALSE;
 
-    nsIComponentManager* compMgr;
-    if(NS_FAILED(NS_GetGlobalComponentManager(&compMgr)) ||
-       !compMgr || NS_FAILED(compMgr->IsRegistered(id, &registered)))
+    nsCOMPtr<nsIComponentRegistrar> compMgr;            
+    if(NS_FAILED(NS_GetComponentRegistrar(getter_AddRefs(compMgr))) || !compMgr || 
+       NS_FAILED(compMgr->IsCIDRegistered(id, &registered)))
         return PR_FALSE;
 
     return registered;
@@ -657,7 +659,6 @@ public:
 
 nsXPCComponents_Results::nsXPCComponents_Results()
 {
-    NS_INIT_ISUPPORTS();
 }
 
 nsXPCComponents_Results::~nsXPCComponents_Results()
@@ -792,7 +793,6 @@ private:
 
 nsXPCComponents_ID::nsXPCComponents_ID()
 {
-    NS_INIT_ISUPPORTS();
 }
 
 nsXPCComponents_ID::~nsXPCComponents_ID()
@@ -922,7 +922,6 @@ private:
 
 nsXPCComponents_Exception::nsXPCComponents_Exception()
 {
-    NS_INIT_ISUPPORTS();
 }
 
 nsXPCComponents_Exception::~nsXPCComponents_Exception()
@@ -1072,7 +1071,7 @@ nsXPCComponents_Exception::HasInstance(nsIXPConnectWrappedNative *wrapper,
                                        PRBool *_retval)
 {
     if(bp)
-        *bp = JSValIsInterfaceOfType(cx, val, NS_GET_IID(nsIXPCException));
+        *bp = JSValIsInterfaceOfType(cx, val, NS_GET_IID(nsIException));
     return NS_OK;
 }
 
@@ -1120,7 +1119,6 @@ nsXPCConstructor::nsXPCConstructor(nsIJSCID* aClassID,
                                    nsIJSIID* aInterfaceID,
                                    const char* aInitializer)
 {
-    NS_INIT_ISUPPORTS();
     NS_IF_ADDREF(mClassID = aClassID);
     NS_IF_ADDREF(mInterfaceID = aInterfaceID);
     mInitializer = aInitializer ?
@@ -1286,7 +1284,6 @@ private:
 
 nsXPCComponents_Constructor::nsXPCComponents_Constructor()
 {
-    NS_INIT_ISUPPORTS();
 }
 
 nsXPCComponents_Constructor::~nsXPCComponents_Constructor()
@@ -1384,7 +1381,7 @@ nsXPCComponents_Constructor::CallOrConstruct(nsIXPConnectWrappedNative *wrapper,
         // argv[1] is an iid name string
         // XXXjband support passing "Components.interfaces.foo"?
 
-        nsCOMPtr<nsIXPCComponents_Interfaces> ifaces;
+        nsCOMPtr<nsIScriptableInterfaces> ifaces;
         nsCOMPtr<nsIXPConnectJSObjectHolder> holder;
         JSObject* ifacesObj = nsnull;
 
@@ -1394,7 +1391,7 @@ nsXPCComponents_Constructor::CallOrConstruct(nsIXPConnectWrappedNative *wrapper,
 
         if(NS_FAILED(comp->GetInterfaces(getter_AddRefs(ifaces))) ||
            NS_FAILED(xpc->WrapNative(cx, obj, ifaces,
-                                     NS_GET_IID(nsIXPCComponents_Interfaces),
+                                     NS_GET_IID(nsIScriptableInterfaces),
                                      getter_AddRefs(holder))) || !holder ||
            NS_FAILED(holder->GetJSObject(&ifacesObj)) || !ifacesObj)
         {
@@ -1424,9 +1421,15 @@ nsXPCComponents_Constructor::CallOrConstruct(nsIXPConnectWrappedNative *wrapper,
     }
     else
     {
-        cInterfaceID =
-            dont_AddRef(
-                NS_STATIC_CAST(nsIJSIID*, nsJSIID::NewID("nsISupports")));
+        nsCOMPtr<nsIInterfaceInfo> info;
+        xpc->GetInfoForIID(&NS_GET_IID(nsISupports), getter_AddRefs(info));
+        
+        if(info)
+        {
+            cInterfaceID =
+                dont_AddRef(
+                    NS_STATIC_CAST(nsIJSIID*, nsJSIID::NewID(info)));
+        }
         if(!cInterfaceID)
             return ThrowAndFail(NS_ERROR_XPC_UNEXPECTED, cx, _retval);
     }
@@ -1537,10 +1540,15 @@ nsXPCComponents::nsXPCComponents()
         mException(nsnull),
         mConstructor(nsnull)
 {
-    NS_INIT_ISUPPORTS();
 }
 
 nsXPCComponents::~nsXPCComponents()
+{
+    ClearMembers();
+}
+
+void 
+nsXPCComponents::ClearMembers()
 {
     NS_IF_RELEASE(mInterfaces);
     NS_IF_RELEASE(mClasses);
@@ -1552,8 +1560,8 @@ nsXPCComponents::~nsXPCComponents()
 }
 
 /*******************************************/
-#define XPC_IMPL_GET_OBJ_METHOD(_n) \
-NS_IMETHODIMP nsXPCComponents::Get##_n(nsIXPCComponents_##_n * *a##_n) { \
+#define XPC_IMPL_GET_OBJ_METHOD(_b, _n) \
+NS_IMETHODIMP nsXPCComponents::Get##_n(_b##_n * *a##_n) { \
     NS_ENSURE_ARG_POINTER(a##_n); \
     if(!m##_n) { \
         if(!(m##_n = new nsXPCComponents_##_n())) { \
@@ -1567,13 +1575,13 @@ NS_IMETHODIMP nsXPCComponents::Get##_n(nsIXPCComponents_##_n * *a##_n) { \
     return NS_OK; \
 }
 
-XPC_IMPL_GET_OBJ_METHOD(Interfaces);
-XPC_IMPL_GET_OBJ_METHOD(Classes);
-XPC_IMPL_GET_OBJ_METHOD(ClassesByID);
-XPC_IMPL_GET_OBJ_METHOD(Results);
-XPC_IMPL_GET_OBJ_METHOD(ID);
-XPC_IMPL_GET_OBJ_METHOD(Exception);
-XPC_IMPL_GET_OBJ_METHOD(Constructor);
+XPC_IMPL_GET_OBJ_METHOD(nsIScriptable,     Interfaces);
+XPC_IMPL_GET_OBJ_METHOD(nsIXPCComponents_, Classes);
+XPC_IMPL_GET_OBJ_METHOD(nsIXPCComponents_, ClassesByID);
+XPC_IMPL_GET_OBJ_METHOD(nsIXPCComponents_, Results);
+XPC_IMPL_GET_OBJ_METHOD(nsIXPCComponents_, ID);
+XPC_IMPL_GET_OBJ_METHOD(nsIXPCComponents_, Exception);
+XPC_IMPL_GET_OBJ_METHOD(nsIXPCComponents_, Constructor);
 
 #undef XPC_IMPL_GET_OBJ_METHOD
 /*******************************************/
@@ -1600,17 +1608,7 @@ NS_IMETHODIMP
 nsXPCComponents::GetManager(nsIComponentManager * *aManager)
 {
     NS_ASSERTION(aManager, "bad param");
-
-    nsIComponentManager* cm;
-    if(NS_FAILED(NS_GetGlobalComponentManager(&cm)))
-    {
-        *aManager = nsnull;
-        return NS_ERROR_FAILURE;
-    }
-    NS_IF_ADDREF(cm);
-    *aManager = cm;
-
-    return NS_OK;
+    return NS_GetComponentManager(aManager);
 }
 
 /**********************************************/
@@ -1636,9 +1634,13 @@ nsXPCComponents::NewResolve(nsIXPConnectWrappedNative *wrapper,
         return NS_ERROR_FAILURE;
 
     jsid idid;
+    uintN attrs = 0;
 
     if(id == rt->GetStringJSVal(XPCJSRuntime::IDX_LAST_RESULT))
+    {
         idid = rt->GetStringID(XPCJSRuntime::IDX_LAST_RESULT);
+        attrs = JSPROP_READONLY;
+    }
     else if(id == rt->GetStringJSVal(XPCJSRuntime::IDX_RETURN_CODE))
         idid = rt->GetStringID(XPCJSRuntime::IDX_RETURN_CODE);
     else
@@ -1647,9 +1649,7 @@ nsXPCComponents::NewResolve(nsIXPConnectWrappedNative *wrapper,
     *objp = obj;
     *_retval = OBJ_DEFINE_PROPERTY(cx, obj, idid, JSVAL_VOID,
                                    nsnull, nsnull,
-                                   JSPROP_ENUMERATE |
-                                   JSPROP_READONLY |
-                                   JSPROP_PERMANENT,
+                                   JSPROP_ENUMERATE | JSPROP_PERMANENT | attrs,
                                    nsnull);
     return NS_OK;
 }
@@ -1756,13 +1756,117 @@ nsXPCComponents::AttachNewComponentsObject(XPCCallContext& ccx,
                                       nsnull);
 }
 
+/* void lookupMethod (); */
+NS_IMETHODIMP nsXPCComponents::LookupMethod()
+{
+    nsresult rv;
+
+    nsCOMPtr<nsIXPConnect> xpc(do_GetService(nsIXPConnect::GetCID(), &rv));
+    if(NS_FAILED(rv))
+        return NS_ERROR_FAILURE;
+
+    // get the xpconnect native call context
+    nsCOMPtr<nsIXPCNativeCallContext> cc;
+    xpc->GetCurrentNativeCallContext(getter_AddRefs(cc));
+    if(!cc)
+        return NS_ERROR_FAILURE;
+
+    // verify that we are being called from JS (i.e. the current call is
+    // to this object - though we don't verify that it is to this exact method)
+    nsCOMPtr<nsISupports> callee;
+    cc->GetCallee(getter_AddRefs(callee));
+    if(!callee || callee.get() != 
+                  NS_STATIC_CAST(const nsISupports*,
+                                 NS_STATIC_CAST(const nsIXPCComponents*,this)))
+        return NS_ERROR_FAILURE;
+
+    // Get JSContext of current call
+    JSContext* cx;
+    rv = cc->GetJSContext(&cx);
+    if(NS_FAILED(rv) || !cx)
+        return NS_ERROR_FAILURE;
+
+    // get place for return value
+    jsval *retval = nsnull;
+    rv = cc->GetRetValPtr(&retval);
+    if(NS_FAILED(rv) || !retval)
+        return NS_ERROR_FAILURE;
+
+    // get argc and argv and verify arg count
+    PRUint32 argc;
+    rv = cc->GetArgc(&argc);
+    if(NS_FAILED(rv))
+        return NS_ERROR_FAILURE;
+
+    if(argc < 2)
+        return NS_ERROR_XPC_NOT_ENOUGH_ARGS;
+
+    jsval* argv;
+    rv = cc->GetArgvPtr(&argv);
+    if(NS_FAILED(rv) || !argv)
+        return NS_ERROR_FAILURE;
+
+    // first param must be a JSObject
+    if(JSVAL_IS_PRIMITIVE(argv[0]))
+        return NS_ERROR_XPC_BAD_CONVERT_JS;
+
+    JSObject* obj = JSVAL_TO_OBJECT(argv[0]);
+
+    // second param must be a string
+    if(!JSVAL_IS_STRING(argv[1]))
+        return NS_ERROR_XPC_BAD_CONVERT_JS;
+
+    // Make sure the name (argv[1]) that we use for looking up the
+    // method/property is atomized.
+
+    jsid name_id;
+    if(!JS_ValueToId(cx, argv[1], &name_id) ||
+       !JS_IdToValue(cx, name_id, &argv[1]))
+        return NS_ERROR_XPC_BAD_CONVERT_JS;
+
+    // this will do verification and the method lookup for us
+    XPCCallContext inner_cc(JS_CALLER, cx, obj, nsnull, argv[1]);
+
+    // was our jsobject really a wrapped native at all?
+    XPCWrappedNative* wrapper = inner_cc.GetWrapper();
+    if(!wrapper || !wrapper->IsValid())
+        return NS_ERROR_XPC_BAD_CONVERT_JS;
+
+    // did we find a method/attribute by that name?
+    XPCNativeMember* member = inner_cc.GetMember();
+    if(!member || member->IsConstant())
+        return NS_ERROR_XPC_BAD_CONVERT_JS;
+    
+    // it would a be a big surprise if there is a member without an interface :)
+    XPCNativeInterface* iface = inner_cc.GetInterface();
+    if(!iface)
+        return NS_ERROR_XPC_BAD_CONVERT_JS;
+
+    // get (and perhaps lazily create) the member's cloneable function
+    jsval funval;
+    if(!member->GetValue(inner_cc, iface, &funval))
+        return NS_ERROR_XPC_BAD_CONVERT_JS;
+
+    // clone a function we can use for this object 
+    JSObject* funobj = JS_CloneFunctionObject(cx, JSVAL_TO_OBJECT(funval), 
+                                              wrapper->GetFlatJSObject());
+    if(!funobj)
+        return NS_ERROR_XPC_BAD_CONVERT_JS;
+
+    // return the function and let xpconnect know we did so
+    *retval = OBJECT_TO_JSVAL(funobj);
+    cc->SetReturnValueWasSet(PR_TRUE);
+
+    return NS_OK;
+}
+
 #ifdef XPC_USE_SECURITY_CHECKED_COMPONENT
 /* string canCreateWrapper (in nsIIDPtr iid); */
 NS_IMETHODIMP
 nsXPCComponents::CanCreateWrapper(const nsIID * iid, char **_retval)
 {
     // We let anyone do this...
-    *_retval = CloneAllAccess();
+    *_retval = xpc_CloneAllAccess();
     return NS_OK;
 }
 
@@ -1779,15 +1883,8 @@ nsXPCComponents::CanCallMethod(const nsIID * iid, const PRUnichar *methodName, c
 NS_IMETHODIMP
 nsXPCComponents::CanGetProperty(const nsIID * iid, const PRUnichar *propertyName, char **_retval)
 {
-    static const NS_NAMED_LITERAL_STRING(s_interfaces, "interfaces");
-    static const NS_NAMED_LITERAL_STRING(s_results, "results");
-
-    const nsDependentString name(propertyName);
-
-    if(name.Equals(s_interfaces) || name.Equals(s_results))
-        *_retval = CloneAllAccess();
-    else
-        *_retval = nsnull;
+    static const char* allowed[] = { "interfaces", "results", nsnull};
+    *_retval = xpc_CheckAccessList(propertyName, allowed);
     return NS_OK;
 }
 
