@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -186,13 +186,9 @@ XPCNativeInterface::GetNewOrUsed(XPCCallContext& ccx, const nsIID* iid)
     if(iface)
         return iface;
 
-    nsCOMPtr<nsIInterfaceInfoManager> iimgr;
-    nsXPConnect::GetInterfaceInfoManager(getter_AddRefs(iimgr));
-    if(!iimgr)
-        return nsnull;
-
     nsCOMPtr<nsIInterfaceInfo> info;
-    if(NS_FAILED(iimgr->GetInfoForIID(iid, getter_AddRefs(info))) ||!info)
+    ccx.GetXPConnect()->GetInfoForIID(iid, getter_AddRefs(info));
+    if(!info)
         return nsnull;
 
     iface = NewInstance(ccx, info);
@@ -269,16 +265,9 @@ XPCNativeInterface::GetNewOrUsed(XPCCallContext& ccx, nsIInterfaceInfo* info)
 XPCNativeInterface*
 XPCNativeInterface::GetNewOrUsed(XPCCallContext& ccx, const char* name)
 {
-    nsCOMPtr<nsIInterfaceInfoManager> iimgr;
-    nsXPConnect::GetInterfaceInfoManager(getter_AddRefs(iimgr));
-    if(!iimgr)
-        return nsnull;
-
     nsCOMPtr<nsIInterfaceInfo> info;
-    if(NS_FAILED(iimgr->GetInfoForName(name, getter_AddRefs(info))) || !info)
-        return nsnull;
-
-    return GetNewOrUsed(ccx, info);
+    ccx.GetXPConnect()->GetInfoForName(name, getter_AddRefs(info));
+    return info ? GetNewOrUsed(ccx, info) : nsnull;
 }
 
 // static
@@ -666,6 +655,21 @@ out:
     return set;
 }
 
+// static 
+void 
+XPCNativeSet::ClearCacheEntryForClassInfo(nsIClassInfo* classInfo)
+{
+    XPCJSRuntime* rt;
+    ClassInfo2NativeSetMap* map;
+    
+    if(nsnull != (rt = nsXPConnect::GetRuntime()) && 
+       nsnull != (map = rt->GetClassInfo2NativeSetMap()))
+    {   // scoped lock
+        XPCAutoLock lock(rt->GetMapLock());
+        map->Remove(classInfo);
+    }
+}
+
 // static
 XPCNativeSet*
 XPCNativeSet::GetNewOrUsed(XPCCallContext& ccx,
@@ -736,11 +740,13 @@ XPCNativeSet::NewInstance(XPCCallContext& ccx,
     PRUint16 slots = count+1;
 
     PRUint16 i;
-    XPCNativeInterface* cur;
+    XPCNativeInterface** pcur;
 
-    for(i = 0, cur = *array; i < count; i++, cur++)
-        if(cur == isup)
+    for(i = 0, pcur = array; i < count; i++, pcur++)
+    {
+        if(*pcur == isup)
             slots--;
+    }
 
     // Use placement new to create an object with the right amount of space
     // to hold the members array
@@ -762,6 +768,8 @@ XPCNativeSet::NewInstance(XPCCallContext& ccx,
 
         for(i = 0; i < count; i++)
         {
+            XPCNativeInterface* cur;
+
             if(isup == (cur = *(inp++)))
                 continue;
             *(outp++) = cur;
