@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -112,6 +112,14 @@ XPCWrappedNativeScope::GetNewOrUsed(XPCCallContext& ccx, JSObject* aGlobal)
     XPCWrappedNativeScope* scope = FindInJSObjectScope(ccx, aGlobal, JS_TRUE);
     if(!scope)
         scope = new XPCWrappedNativeScope(ccx, aGlobal);
+    else
+    {
+        // We need to call SetGlobal in order to refresh our cached 
+        // mPrototypeJSObject in the case where the global object is being 
+        // reused (JS_ClearScope has been called).
+        // NOTE: We are only called by nsXPConnect::InitClasses. 
+        scope->SetGlobal(ccx, aGlobal);
+    }
     return scope;
 }
 
@@ -213,6 +221,19 @@ XPCWrappedNativeScope::~XPCWrappedNativeScope()
 }
 
 
+JS_STATIC_DLL_CALLBACK(JSDHashOperator)
+WrappedNativeJSGCThingMarker(JSDHashTable *table, JSDHashEntryHdr *hdr,
+                             uint32 number, void *arg)
+{
+    XPCWrappedNative* wrapper = ((Native2WrappedNativeMap::Entry*)hdr)->value;
+    if(wrapper->HasExternalReference())
+    {
+        JS_MarkGCThing((JSContext*)arg, wrapper->GetFlatJSObject(), 
+                       "XPCWrappedNative::mFlatJSObject", nsnull);
+    }
+    return JS_DHASH_NEXT;
+}
+
 // static
 void
 XPCWrappedNativeScope::FinishedMarkPhaseOfGC(JSContext* cx, XPCJSRuntime* rt)
@@ -220,12 +241,21 @@ XPCWrappedNativeScope::FinishedMarkPhaseOfGC(JSContext* cx, XPCJSRuntime* rt)
     // Hold the lock until return...
     XPCAutoLock lock(rt->GetMapLock());
 
+    XPCWrappedNativeScope* cur;
+    
+    // Do JS_MarkGCThing for all wrapperednatives with external references.
+    for(cur = gScopes; cur; cur = cur->mNext)
+    {
+        cur->mWrappedNativeMap->Enumerate(WrappedNativeJSGCThingMarker, cx);
+    }
+
     // Since the JSGC_END call happens outside of a lock,
     // it is possible for us to get called here twice before the FinshedGC
     // call happens. So, we allow for gDyingScopes not being null.
 
-    XPCWrappedNativeScope* cur = gScopes;
     XPCWrappedNativeScope* prev = nsnull;
+    cur = gScopes;
+
     while(cur)
     {
         XPCWrappedNativeScope* next = cur->mNext;
@@ -431,6 +461,10 @@ XPCWrappedNativeScope::SystemIsBeingShutDown(XPCCallContext& ccx)
 
     for(cur = gDyingScopes; cur; cur = cur->mNext)
     {
+        // Give the Components object a chance to try to clean up.
+        if(cur->mComponents)
+            cur->mComponents->SystemIsBeingShutDown();
+
         // Walk the protos first. Wrapper shutdown can leave dangling
         // proto pointers in the proto map.
         cur->mWrappedNativeProtoMap->
