@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -51,8 +51,7 @@ public:
     NS_DECL_NSIECHO
 
 #ifdef IMPLEMENT_TIMER_STUFF
-    // not very xpcom compilant method from nsITimerCallback
-    NS_IMETHOD_(void) Notify(nsITimer *timer);
+    NS_DECL_NSITIMERCALLBACK
 #endif // IMPLEMENT_TIMER_STUFF
 
     xpctestEcho();
@@ -74,7 +73,6 @@ NS_IMPL_ISUPPORTS1(xpctestEcho, nsIEcho);
 xpctestEcho::xpctestEcho()
     : mReceiver(nsnull), mString(nsnull), mSomeValue(0)
 {
-    NS_INIT_REFCNT();
     NS_ADDREF_THIS();
 }
 
@@ -107,21 +105,73 @@ NS_IMETHODIMP xpctestEcho::In2OutOneInt(int input, int* output)
 }
 
 /* DOMString In2OutOneDOMString (in DOMString input); */
-NS_IMETHODIMP xpctestEcho::In2OutOneDOMString(const nsAReadableString & input, 
-                                              nsAWritableString & _retval)
+NS_IMETHODIMP xpctestEcho::In2OutOneDOMString(const nsAString & input, 
+                                              nsAString & _retval)
 {
     _retval.Assign(input);
     return NS_OK;
 }
 
 /* DOMString EchoIn2OutOneDOMString (in DOMString input); */
-NS_IMETHODIMP xpctestEcho::EchoIn2OutOneDOMString(const nsAReadableString & input, nsAWritableString & _retval)
+NS_IMETHODIMP xpctestEcho::EchoIn2OutOneDOMString(const nsAString & input, nsAString & _retval)
 {
     if(mReceiver)
         return mReceiver->EchoIn2OutOneDOMString(input, _retval);
     return NS_OK;
 }
 
+/* AString In2OutOneAString (in AString input); */
+NS_IMETHODIMP xpctestEcho::In2OutOneAString(const nsAString & input, 
+                                              nsAString & _retval)
+{
+    _retval.Assign(input);
+    return NS_OK;
+}
+
+/* AString EchoIn2OutOneAString (in AString input); */
+NS_IMETHODIMP xpctestEcho::EchoIn2OutOneAString(const nsAString & input, nsAString & _retval)
+{
+    if(mReceiver)
+        return mReceiver->EchoIn2OutOneAString(input, _retval);
+    return NS_OK;
+}
+
+
+/* UTF8String In2OutOneUTF8String (in UTF8String input); */
+NS_IMETHODIMP xpctestEcho::In2OutOneUTF8String(const nsACString & input, 
+                                              nsACString & _retval)
+{
+    _retval.Assign(input);
+    return NS_OK;
+}
+
+/* UTF8String EchoIn2OutOneUTF8String (in UTF8String input); */
+NS_IMETHODIMP xpctestEcho::EchoIn2OutOneUTF8String(const nsACString & input, 
+                                                   nsACString & _retval)
+{
+    if(mReceiver)
+        return mReceiver->EchoIn2OutOneUTF8String(input, _retval);
+    return NS_OK;
+}
+
+/* CString In2OutOneCString (in CString input); */
+NS_IMETHODIMP xpctestEcho::In2OutOneCString(const nsACString & input, 
+                                            nsACString & _retval)
+{
+    _retval.Assign(input);
+    return NS_OK;
+}
+
+/* CString EchoIn2OutOneCString (in CString input); */
+NS_IMETHODIMP xpctestEcho::EchoIn2OutOneCString(const nsACString & input, 
+                                                nsACString & _retval)
+{
+    if(mReceiver)
+        return mReceiver->EchoIn2OutOneCString(input, _retval);
+    return NS_OK;
+}
+
+
 NS_IMETHODIMP xpctestEcho::In2OutAddTwoInts(int input1,
                                        int input2,
                                        int* output1,
@@ -480,7 +530,8 @@ xpctestEcho::CallReceiverSometimeLater(void)
     timer = do_CreateInstance("@mozilla.org/timer;1", &rv);
     if(NS_FAILED(rv))
         return NS_ERROR_FAILURE;
-    timer->Init(NS_STATIC_CAST(nsITimerCallback*,this), 2000);
+    timer->InitWithCallback(NS_STATIC_CAST(nsITimerCallback*,this), 2000,
+                            nsITimer::TYPE_ONE_SHOT);
     return NS_OK;
 #else
     return NS_ERROR_NOT_IMPLEMENTED;
@@ -488,12 +539,13 @@ xpctestEcho::CallReceiverSometimeLater(void)
 }
 
 #ifdef IMPLEMENT_TIMER_STUFF
-NS_IMETHODIMP_(void)
+NS_IMETHODIMP
 xpctestEcho::Notify(nsITimer *timer)
 {
     if(mReceiver)
         mReceiver->CallReceiverSometimeLater();
     NS_RELEASE(timer);
+    return NS_OK;
 }        
 #endif // IMPLEMENT_TIMER_STUFF
 
@@ -533,9 +585,6 @@ NS_IMETHODIMP xpctestEcho::SetSomeValue(PRInt32 aSomeValue)
     return NS_OK;
 }
 
-
-/***************************************************/
-
 /***************************************************************************/
 
 // static
