@@ -1,42 +1,54 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
- */
+ * Contributor(s):
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
+
 #ifndef nsIFrame_h___
 #define nsIFrame_h___
 
+/* nsIFrame is in the process of being deCOMtaminated, i.e., this file is eventually
+   going to be eliminated, and all callers will use nsFrame instead.  At the moment
+   we're midway through this process, so you will see inlined functions and member
+   variables in this file.  -dwh */
+
 #include <stdio.h>
-#include "nslayout.h"
 #include "nsISupports.h"
 #include "nsEvent.h"
 #include "nsStyleStruct.h"
-#include "nsSize.h"
-#include "nsMargin.h"
-#include "nsRect.h"
-#ifdef MOZ_MATHML
-#include "nsIRenderingContext.h" //to get struct nsBoundingMetrics
-#endif
-
-struct nsGUIEvent;
-struct nsHTMLReflowState;
-class nsIReflowCommand;
+#include "nsStyleContext.h"
+#include "nsIContent.h"
 
 /**
  * New rules of reflow:
@@ -55,32 +67,34 @@ class nsIReflowCommand;
  * 5. the view system handles moving of widgets, i.e., it's not our problem
  */
 
+struct nsHTMLReflowState;
+class nsHTMLReflowCommand;
+struct nsHTMLReflowMetrics;
+
 class nsIAtom;
-class nsIContent;
 class nsIPresContext;
 class nsIPresShell;
 class nsIRenderingContext;
-class nsISizeOfHandler;
-class nsIStyleContext;
 class nsIView;
 class nsIWidget;
-class nsAutoString;
-class nsString;
-class nsIFocusTracker; 
-class nsStyleChangeList;
-class nsBlockFrame;
-class nsLineLayout;
+class nsIDOMRange;
+class nsISelectionController;
+#ifdef ACCESSIBILITY
+class nsIAccessible;
+#endif
 
 struct nsPeekOffsetStruct;
 struct nsPoint;
 struct nsRect;
-struct nsStyleStruct;
-class  nsIDOMRange;
-class  nsICaret;
-class  nsISelectionController;
-struct PRLogModuleInfo;
+struct nsSize;
+struct nsMargin;
 
-class nsIAccessible;
+// Calback function used to destroy the value associated with a property.
+typedef void 
+(*NSFramePropertyDtorFunc)(nsIPresContext* aPresContext,
+                           nsIFrame*       aFrame,
+                           nsIAtom*        aPropertyName,
+                           void*           aPropertyValue);
 
 // IID for the nsIFrame interface 
 // a6cf9050-15b3-11d2-932e-00805f8add32
@@ -135,7 +149,7 @@ typedef PRUint32 nsFrameState;
 
 // If this bit is set, then there is a child frame in the frame that
 // extends outside this frame's bounding box. The implication is that
-// the frames rect does not completely cover its children and
+// the frame's rect does not completely cover its children and
 // therefore operations like rendering and hit testing (for example)
 // must operate differently.
 #define NS_FRAME_OUTSIDE_CHILDREN                     0x00000008
@@ -157,7 +171,7 @@ typedef PRUint32 nsFrameState;
 // its lifetime.
 #define NS_FRAME_HAS_LOADED_IMAGES                    0x00000080
 
-// If this bit is set, then the frame is has been moved out of the flow,
+// If this bit is set, then the frame has been moved out of the flow,
 // e.g., it is absolutely positioned or floated
 #define NS_FRAME_OUT_OF_FLOW                          0x00000100
 
@@ -201,6 +215,10 @@ typedef PRUint32 nsFrameState;
 // If this bit is set the frame has descendant with a view
 #define NS_FRAME_HAS_CHILD_WITH_VIEW                  0x00040000
 
+// If this bit is set, then reflow may be dispatched from the current
+// frame instead of the root frame.
+#define NS_FRAME_REFLOW_ROOT                          0x00080000
+
 // The lower 20 bits of the frame state word are reserved by this API.
 #define NS_FRAME_RESERVED                             0x000FFFFF
 
@@ -238,118 +256,6 @@ enum nsSpread {
   eSpreadDown   = 2
 };
 
-enum nsContextProviderRelationship {
-  eContextProvider_Ancestor = 0,
-  eContextProvider_Descendant = 1
-};
-
-//----------------------------------------------------------------------
-
-// Option flags
-#define NS_REFLOW_CALC_MAX_WIDTH         0x0001
-#ifdef MOZ_MATHML
-#define NS_REFLOW_CALC_BOUNDING_METRICS  0x0002
-#endif
-
-/**
- * Reflow metrics used to return the frame's desired size and alignment
- * information.
- *
- * @see #Reflow()
- */
-struct nsHTMLReflowMetrics {
-  nscoord width, height;        // [OUT] desired width and height
-  nscoord ascent, descent;      // [OUT] ascent and descent information
-
-  // Set this to null if you don't need to compute the max element size
-  nsSize* maxElementSize;       // [OUT]
-
-  // Used for incremental reflow. If the NS_REFLOW_CALC_MAX_WIDTH flag is set,
-  // then the caller is requesting that you update and return your maximum width
-  nscoord mMaximumWidth;        // [OUT]
-
-#ifdef MOZ_MATHML
-  // Metrics that _exactly_ enclose the text to allow precise MathML placements.
-  // If the NS_REFLOW_CALC_BOUNDING_METRICS flag is set, then the caller is 
-  // requesting that you also compute additional details about your inner
-  // bounding box and italic correction. For example, the bounding box of
-  // msup is the smallest rectangle that _exactly_ encloses both the text
-  // of the base and the text of the superscript.
-  nsBoundingMetrics mBoundingMetrics;  // [OUT]
-#endif
-
-  // Carried out bottom margin values. This is the collapsed
-  // (generational) bottom margin value.
-  nscoord mCarriedOutBottomMargin;
-  
-  // For frames that have content that overflow their content area
-  // (NS_FRAME_OUTSIDE_CHILDREN) this rectangle represents the total area
-  // of the frame including visible overflow, i.e., don't include overflowing
-  // content that is hidden.
-  // The rect is in the local coordinate space of the frame, and should be at
-  // least as big as the desired size. If there is no content that overflows,
-  // then the overflow area is identical to the desired size and should be
-  // {0, 0, mWidth, mHeight}.
-  nsRect mOverflowArea;
-
-  PRUint32 mFlags;
- 
-  // used by tables to optimize common cases
-  PRBool mNothingChanged;
-
-  nsHTMLReflowMetrics(nsSize* aMaxElementSize, PRUint32 aFlags = 0) {
-    maxElementSize = aMaxElementSize;
-    mMaximumWidth = 0;
-    mFlags = aFlags;
-    mCarriedOutBottomMargin = 0;
-    mOverflowArea.x = 0;
-    mOverflowArea.y = 0;
-    mOverflowArea.width = 0;
-    mOverflowArea.height = 0;
-    mNothingChanged = PR_FALSE;
-#ifdef MOZ_MATHML
-    mBoundingMetrics.Clear();
-#endif
-
-    // XXX These are OUT parameters and so they shouldn't have to be
-    // initialized, but there are some bad frame classes that aren't
-    // properly setting them when returning from Reflow()...
-    width = height = 0;
-    ascent = descent = 0;
-  }
-  
-  void AddBorderPaddingToMaxElementSize(const nsMargin& aBorderPadding) {
-    maxElementSize->width += aBorderPadding.left + aBorderPadding.right;
-    maxElementSize->height += aBorderPadding.top + aBorderPadding.bottom;
-  }
-
-  nsHTMLReflowMetrics& operator=(const nsHTMLReflowMetrics& aOther)
-  {
-    if (maxElementSize && aOther.maxElementSize) {
-      maxElementSize->width = aOther.maxElementSize->width;
-      maxElementSize->height = aOther.maxElementSize->height;
-    }
-    mMaximumWidth = aOther.mMaximumWidth;
-    mFlags = aOther.mFlags;
-    mCarriedOutBottomMargin = aOther.mCarriedOutBottomMargin;
-    mOverflowArea.x = aOther.mOverflowArea.x;
-    mOverflowArea.y = aOther.mOverflowArea.y;
-    mOverflowArea.width = aOther.mOverflowArea.width;
-    mOverflowArea.height = aOther.mOverflowArea.height;
-    mNothingChanged = aOther.mNothingChanged;
-#ifdef MOZ_MATHML
-    mBoundingMetrics = aOther.mBoundingMetrics;
-#endif
-
-    width = aOther.width;
-    height = aOther.height;
-    ascent = aOther.ascent;
-    descent = aOther.descent;
-    return *this;
-  }
-
-};
-
 // Carried out margin flags
 #define NS_CARRIED_TOP_MARGIN_IS_AUTO    0x1
 #define NS_CARRIED_BOTTOM_MARGIN_IS_AUTO 0x2
@@ -447,6 +353,19 @@ typedef PRUint32 nsReflowStatus;
   ((_completionStatus) | NS_INLINE_BREAK | NS_INLINE_BREAK_AFTER |      \
    NS_INLINE_MAKE_BREAK_TYPE(NS_STYLE_CLEAR_LINE))
 
+// The frame (not counting a continuation) did not fit in the available height and 
+// wasn't at the top of a page. If it was at the top of a page, then it is not 
+// possible to reflow it again with more height, so we don't set it in that case.
+#define NS_FRAME_TRUNCATED  0x0010
+#define NS_FRAME_IS_TRUNCATED(status) \
+  (0 != ((status) & NS_FRAME_TRUNCATED))
+#define NS_FRAME_SET_TRUNCATION(status, aReflowState, aMetrics) \
+  if (!aReflowState.mFlags.mIsTopOfPage &&                      \
+      aReflowState.availableHeight < aMetrics.height)           \
+    status |= NS_FRAME_TRUNCATED;                               \
+  else                                                          \
+    status &= ~NS_FRAME_TRUNCATED;
+
 //----------------------------------------------------------------------
 
 /**
@@ -498,7 +417,7 @@ public:
   NS_IMETHOD  Init(nsIPresContext*  aPresContext,
                    nsIContent*      aContent,
                    nsIFrame*        aParent,
-                   nsIStyleContext* aContext,
+                   nsStyleContext*  aContext,
                    nsIFrame*        aPrevInFlow) = 0;
 
   /**
@@ -507,6 +426,11 @@ public:
    */
   NS_IMETHOD  Destroy(nsIPresContext* aPresContext) = 0;
 
+  /*
+   * Notify the frame that it has been removed as the primary frame for its content
+   */
+  virtual void RemovedAsPrimaryFrame(nsIPresContext* aPresContext) {}
+
   /**
    * Called to set the initial list of frames. This happens after the frame
    * has been initialized.
@@ -619,7 +543,7 @@ public:
    *
    * @see nsISupports#Release()
    */
-  NS_IMETHOD  GetContent(nsIContent** aContent) const = 0;
+  nsresult GetContent(nsIContent** aContent) const {  *aContent = mContent; NS_IF_ADDREF(*aContent); return NS_OK; }
 
   /**
    * Get the offsets of the frame. most will be 0,0
@@ -628,22 +552,57 @@ public:
   NS_IMETHOD GetOffsets(PRInt32 &start, PRInt32 &end) const = 0;
 
   /**
-   * Get the style context associated with this frame. Note that GetStyleContext()
-   * adds a reference to the style context so the caller must do a release.
+   * Reset the offsets when splitting frames during Bidi reordering
    *
-   * @see nsISupports#Release()
    */
-  NS_IMETHOD  GetStyleContext(nsIStyleContext** aStyleContext) const = 0;
-  NS_IMETHOD  SetStyleContext(nsIPresContext*  aPresContext,
-                              nsIStyleContext* aContext) = 0;
+  virtual void AdjustOffsetsForBidi(PRInt32 aStart, PRInt32 aEnd) {}
 
   /**
-   * Get the style data associated with this frame.
+   * Get the style context associated with this frame.
+   *
    */
-  NS_IMETHOD  GetStyleData(nsStyleStructID       aSID,
-                           const nsStyleStruct*& aStyleStruct) const = 0;
-  // Fill a style struct with data
-  NS_IMETHOD  GetStyle(nsStyleStructID aSID, const nsStyleStruct** aStruct) const = 0;
+  nsStyleContext* GetStyleContext() const { return mStyleContext; }
+  void SetStyleContext(nsIPresContext* aPresContext, nsStyleContext* aContext)
+  { 
+    if (aContext != mStyleContext) {
+      if (mStyleContext)
+        mStyleContext->Release();
+      mStyleContext = aContext;
+      if (aContext) {
+        aContext->AddRef();
+        DidSetStyleContext(aPresContext);
+      }
+    }
+  }
+
+  // Style post processing hook
+  NS_IMETHOD DidSetStyleContext(nsIPresContext* aPresContext) = 0;
+
+  /**
+   * Get the style data associated with this frame.  This fills in a
+   * const style struct pointer that should never be modified.  See
+   * |nsIStyleContext::GetStyleData| for more information.
+   *
+   * The use of the typesafe global helper function |GetStyleData|,
+   * below, is preferred to direct use of this function.
+   */
+  NS_IMETHOD GetStyleDataExternal(nsStyleStructID aSID,
+                                  const nsStyleStruct*& aStyleStruct) const = 0;
+
+#ifdef _IMPL_NS_LAYOUT
+  nsresult GetStyleData(nsStyleStructID       aSID,
+                        const nsStyleStruct*& aStyleStruct) const {
+    NS_ASSERTION(mStyleContext, "No style context found!");
+    aStyleStruct = mStyleContext->GetStyleData(aSID);
+    return NS_OK;
+  }
+#else
+  nsresult GetStyleData(nsStyleStructID       aSID,
+                        const nsStyleStruct*& aStyleStruct) const {
+    return GetStyleDataExternal(aSID, aStyleStruct);
+  }
+#endif
+
   // Utility function: more convenient than 2 calls to GetStyleData to get border and padding
   NS_IMETHOD  CalcBorderPadding(nsMargin& aBorderPadding) const = 0;
 
@@ -657,16 +616,16 @@ public:
    * The indicies must be consecutive and implementations MUST return an 
    * NS_ERROR_INVALID_ARG if asked for an index that is out of range.
    */
-  NS_IMETHOD  GetAdditionalStyleContext(PRInt32 aIndex, 
-                                        nsIStyleContext** aStyleContext) const = 0;
-  NS_IMETHOD  SetAdditionalStyleContext(PRInt32 aIndex, 
-                                        nsIStyleContext* aStyleContext) = 0;
+  virtual nsStyleContext* GetAdditionalStyleContext(PRInt32 aIndex) const = 0;
+
+  virtual void SetAdditionalStyleContext(PRInt32 aIndex,
+                                         nsStyleContext* aStyleContext) = 0;
 
   /**
    * Accessor functions for geometric parent
    */
-  NS_IMETHOD  GetParent(nsIFrame** aParent) const = 0;
-  NS_IMETHOD  SetParent(const nsIFrame* aParent) = 0;
+  nsresult GetParent(nsIFrame** aParent) const { *aParent = mParent; return NS_OK; }
+  NS_IMETHOD SetParent(const nsIFrame* aParent) { mParent = (nsIFrame*)aParent; return NS_OK; }
 
   /**
    * Bounding rect of the frame. The values are in twips, and the origin is
@@ -676,17 +635,45 @@ public:
    * Note: moving or sizing the frame does not affect the view's size or
    * position.
    */
-  NS_IMETHOD  GetRect(nsRect& aRect) const = 0;
-  NS_IMETHOD  GetOrigin(nsPoint& aPoint) const = 0;
-  NS_IMETHOD  GetSize(nsSize& aSize) const = 0;
-  NS_IMETHOD  SetRect(nsIPresContext* aPresContext,
-                      const nsRect&   aRect) = 0;
-  NS_IMETHOD  MoveTo(nsIPresContext* aPresContext,
-                     nscoord         aX,
-                     nscoord         aY) = 0;
-  NS_IMETHOD  SizeTo(nsIPresContext* aPresContext,
-                     nscoord         aWidth,
-                     nscoord         aHeight) = 0;
+  nsresult GetRect(nsRect& aRect) const {
+    aRect = mRect;
+    return NS_OK;
+  }
+
+  nsresult GetOrigin(nsPoint& aPoint) const {
+    aPoint.x = mRect.x;
+    aPoint.y = mRect.y;
+    return NS_OK;
+  }
+
+  nsresult GetSize(nsSize& aSize) const {
+    aSize.width = mRect.width;
+    aSize.height = mRect.height;
+    return NS_OK;
+  }
+
+  nsresult SetRect(nsIPresContext* aPresContext,
+               const nsRect&   aRect) {
+    MoveTo(aPresContext, aRect.x, aRect.y);
+    SizeTo(aPresContext, aRect.width, aRect.height);
+    return NS_OK;
+  }
+
+  nsresult MoveTo(nsIPresContext* aPresContext,
+                  nscoord         aX,
+                  nscoord         aY) {
+    mRect.x = aX;
+    mRect.y = aY;
+    return NS_OK;
+  }
+
+  nsresult SizeTo(nsIPresContext* aPresContext,
+                  nscoord         aWidth,
+                  nscoord         aHeight) {
+    mRect.width = aWidth;
+    mRect.height = aHeight;
+    return NS_OK;
+  }
 
   /**
    * Used to iterate the list of additional child list names. Returns the atom
@@ -714,20 +701,34 @@ public:
                          nsIFrame**      aFirstChild) const = 0;
 
   /**
-   * Child frames are linked together in a singly-linked
+   * Child frames are linked together in a singly-linked list
    */
-  NS_IMETHOD  GetNextSibling(nsIFrame** aNextSibling) const = 0;
-  NS_IMETHOD  SetNextSibling(nsIFrame* aNextSibling) = 0;
+  nsresult GetNextSibling(nsIFrame** aNextSibling) const {
+    *aNextSibling = mNextSibling;
+    return NS_OK;
+  }
+
+  nsresult SetNextSibling(nsIFrame* aNextSibling) {
+    mNextSibling = aNextSibling;
+    return NS_OK;
+  }
 
   /**
-   * Paint is responsible for painting the a frame. The aWhichLayer
+   * Paint is responsible for painting the frame. The aWhichLayer
    * argument indicates which layer of painting should be done during
    * the call.
    */
   NS_IMETHOD  Paint(nsIPresContext*      aPresContext,
                     nsIRenderingContext& aRenderingContext,
                     const nsRect&        aDirtyRect,
-                    nsFramePaintLayer    aWhichLayer) = 0;
+                    nsFramePaintLayer    aWhichLayer,
+                    PRUint32             aFlags = 0) = 0;
+
+  /* 
+   * Does the frame paint its background? If not, then all or part of it will be
+   * painted by ancestors.
+   */
+  virtual PRBool CanPaintBackground() { return PR_TRUE; }
 
   /**
    * Event handling of GUI events.
@@ -805,16 +806,20 @@ public:
 
  /**
    * Get the current frame-state value for this frame. aResult is
-   * filled in with the state bits. The return value has no
-   * meaning.
+   * filled in with the state bits. 
    */
-  NS_IMETHOD  GetFrameState(nsFrameState* aResult) = 0;
+  nsresult GetFrameState(nsFrameState* aResult) {
+    *aResult = mState;
+    return NS_OK;
+  }
 
   /**
-   * Set the current frame-state value for this frame. The return
-   * value has no meaning.
+   * Set the current frame-state value for this frame. 
    */
-  NS_IMETHOD  SetFrameState(nsFrameState aNewState) = 0;
+  nsresult SetFrameState(nsFrameState aNewState) {
+    mState = aNewState;
+    return NS_OK;
+  }
 
   /**
    * This call is invoked when content is changed in the content tree.
@@ -843,6 +848,7 @@ public:
                                nsIContent*     aChild,
                                PRInt32         aNameSpaceID,
                                nsIAtom*        aAttribute,
+                               PRInt32         aModType, 
                                PRInt32         aHint) = 0;
 
   /**
@@ -871,6 +877,20 @@ public:
   NS_IMETHOD  GetNextInFlow(nsIFrame** aNextInFlow) const = 0;
   NS_IMETHOD  SetNextInFlow(nsIFrame*) = 0;
 
+  /**
+   * Return the first frame in our current flow. 
+   */
+  virtual nsIFrame* GetFirstInFlow() const {
+    return NS_CONST_CAST(nsIFrame*, this);
+  }
+
+  /**
+   * Return the last frame in our current flow.
+   */
+  virtual nsIFrame* GetLastInFlow() const {
+    return NS_CONST_CAST(nsIFrame*, this);
+  }
+
   /**
    * Pre-reflow hook. Before a frame is reflowed this method will be called.
    * This call will always be invoked at least once before a subsequent Reflow
@@ -942,8 +962,9 @@ public:
    * XXX Don't we want the semantics to dictate that we only call this once for
    * a given reflow?
    */
-  NS_IMETHOD  DidReflow(nsIPresContext*   aPresContext,
-                        nsDidReflowStatus aStatus) = 0;
+  NS_IMETHOD  DidReflow(nsIPresContext*           aPresContext,
+                        const nsHTMLReflowState*  aReflowState,
+                        nsDidReflowStatus         aStatus) = 0;
 
   // XXX Maybe these three should be a separate interface?
 
@@ -993,6 +1014,18 @@ public:
                                 nsPoint&        aOffset,
                                 nsIView**       aView) const = 0;
 
+  /**
+   * Returns the offset from this frame's upper left corner to the upper
+   * left corner of the view returned by a call to GetView(). aOffset
+   * will contain the offset to the view or (0,0) if the frame has no
+   * view. aView will contain a pointer to the view returned by GetView().
+   * aView is optional, that is, you may pass null if you are not interested
+   * in getting a pointer to the view.
+   */
+  NS_IMETHOD  GetOriginToViewOffset(nsIPresContext* aPresContext,
+                                    nsPoint&        aOffset,
+                                    nsIView**       aView) const = 0;
+
   /**
    * Returns the window that contains this frame. If this frame has a
    * view and the view has a window, then this frames window is
@@ -1027,8 +1060,8 @@ public:
    *  to redraw event if aSelected == the frame selection with the last parameter.
    *  data in struct may be changed when passed in.
    *  @param aRange is the range that will dictate if the frames need to be redrawn null means the whole content needs to be redrawn
-   *  @param aSelected is it selected
-   *  @param aSpread should is spread selection to flow elements around it? or go down to its children?
+   *  @param aSelected is it selected?
+   *  @param aSpread should it spread the selection to flow elements around it? or go down to its children?
    */
   NS_IMETHOD  SetSelected(nsIPresContext* aPresContext,
                           nsIDOMRange*    aRange,
@@ -1091,7 +1124,7 @@ public:
   /**
    *  Called by a child frame on a parent frame to tell the parent frame that the child needs
    *  to be reflowed.  The parent should either propagate the request to its parent frame or 
-   *  handle the request by generating a nsIReflowCommand::ReflowDirtyChildren reflow command.
+   *  handle the request by generating a eReflowType_ReflowDirtyChildren reflow command.
    */
 
   NS_IMETHOD ReflowDirtyChild(nsIPresShell* aPresShell, nsIFrame* aChild) = 0;
@@ -1103,41 +1136,34 @@ public:
    * Note: nsAccessible must be refcountable. Do not implement directly on your frame
    * Use a mediatior of some kind.
    */
+#ifdef ACCESSIBILITY
   NS_IMETHOD GetAccessible(nsIAccessible** aAccessible) = 0;
+#endif
 
   /**
-   *  Called during appending or cancelling a reflow command to give frames notice
-   *  of reflow commands that will be targeted below them. 
-   */
-  NS_IMETHOD ReflowCommandNotify(nsIPresShell*     aShell,
-                                 nsIReflowCommand* aRC,
-                                 PRBool            aCommandAdded) = 0;
-
-  /**
-   * Called in style ReResolution to get the frame that contains the style context that is the
-   * parent style context for this frame. 
-   * @param aProviderFrame: set to the frame who's style context is to be used as the parent context
-   *                        in reresolving style for this frame. Note that this may or may not be the
-   *                        parent frame of this frame.
-   * @param aRelatioship : indicates how the provider frame relates to this frame: 
-   *                       i.e. if the parent context provider frame is a child of this frame, 
-   *                       then aRelationship must be set to eContextProvider_Descendant, 
-   *                       if a parent, grandparent etc. set to eContextProvider_Ancestor 
-   *                       (this is used to control recursion in style reresolution)
+   * Get the frame whose style context should be the parent of this
+   * frame's style context (i.e., provide the parent style context).
+   * This frame must either be an ancestor of this frame or a child.  If
+   * this frame returns a child frame, then the child frame must be sure
+   * to return a grandparent or higher!
    *
-   * if return value is NS_OK then aProvdierFrame must be set, otherwise it is ignored.
-   *
-   * PRECONDITIONS: aProviderFrame cannot be null
-   * POSTCONDITIONS: *aProfiderFrame will be a valid frame if this routine succeeds
-   *                 aRelationship will be set to a legal enum value indicating the correct relationship
+   * @param aPresContext:   PresContext
+   * @param aProviderFrame: The frame whose style context should be the
+   *                        parent of this frame's style context.  Null
+   *                        is permitted, and means that this frame's
+   *                        style context should be the root of the
+   *                        style context tree.
+   * @param aIsChild:       True if |aProviderFrame| is set to a child
+   *                        of this frame; false if it is an ancestor or
+   *                        null.
    */
-  NS_IMETHOD GetParentStyleContextProvider(nsIPresContext* aPresContext,
-                                           nsIFrame** aProviderFrame, 
-                                           nsContextProviderRelationship& aRelationship) = 0;
+  NS_IMETHOD GetParentStyleContextFrame(nsIPresContext* aPresContext,
+                                        nsIFrame**      aProviderFrame,
+                                        PRBool*         aIsChild) = 0;
 
   /**
    * Determines whether a frame is visible for painting
-   * this takes into acount whether it is painting selection for printin
+   * this takes into account whether it is painting a selection or printing.
    * @param aPresContext PresContext
    * @param aRenderingContext PresContext
    * @param aCheckVis indicates whether it should check for CSS visibility, 
@@ -1148,6 +1174,53 @@ public:
                                   nsIRenderingContext& aRenderingContext,
                                   PRBool               aCheckVis,
                                   PRBool*              aIsVisible) = 0;
+
+  /**
+   * Determine whether the frame is logically empty, i.e., whether the
+   * layout would be the same whether or not the frame is present.
+   * Placeholder frames should return true.  Block frames should be
+   * considered empty whenever margins collapse through them, even
+   * though those margins are relevant.
+   *
+   * aIsPre should be ignored by frames to which the 'white-space'
+   * property applies.
+   */
+  NS_IMETHOD IsEmpty(nsCompatibility aCompatMode,
+                     PRBool aIsPre,
+                     PRBool* aResult) = 0;
+
+  /**
+   * IsGeneratedContentFrame returns whether a frame corresponds to
+   * generated content
+   *
+   * @return whether the frame correspods to generated content
+   */
+  PRBool IsGeneratedContentFrame() {
+    return (mState & NS_FRAME_GENERATED_CONTENT) != 0;
+  }
+
+  /**
+   * IsPseudoFrame returns whether a frame is a pseudo frame (eg an
+   * anonymous table-row frame created for a CSS table-cell without an
+   * enclosing table-row.
+   *
+   * @param aParentContent the content node corresponding to the parent frame
+   * @return whether the frame is a pseudo frame
+   */   
+  PRBool IsPseudoFrame(nsIContent* aParentContent) {
+    return mContent == aParentContent;
+  }
+
+
+  
+  virtual void* GetProperty(nsIPresContext* aPresContext,
+                            nsIAtom*        aPropertyName,
+                            PRBool          aRemoveProperty) const = 0;
+
+  virtual nsresult SetProperty(nsIPresContext*         aPresContext,
+                               nsIAtom*                aPropertyName,
+                               void*                   aPropertyValue,
+                               NSFramePropertyDtorFunc aPropDtorFunc) = 0;
 #ifdef IBMBIDI
   /**
    *  retrieve and set Bidi property of this frame
@@ -1162,9 +1235,35 @@ public:
                              void*           aPropertyValue) = 0;
 #endif // IBMBIDI
 
+  /**
+   * Return PR_TRUE if and only if this frame obeys visibility:hidden.
+   * if it does not, then nsContainerFrame will hide its view even though
+   * this means children can't be made visible again.
+   */
+  virtual PRBool SupportsVisibilityHidden() { return PR_TRUE; }
+
+protected:
+  // Members
+  nsRect           mRect;
+  nsIContent*      mContent;
+  nsStyleContext*  mStyleContext;
+  nsIFrame*        mParent;
+  nsIFrame*        mNextSibling;  // singly-linked list of frames
+  nsFrameState     mState;
+
 private:
   NS_IMETHOD_(nsrefcnt) AddRef(void) = 0;
   NS_IMETHOD_(nsrefcnt) Release(void) = 0;
 };
 
+// typesafe way to access style data.  See comment in nsStyleStruct.h
+// and also overloaded function in nsStyleContext.h
+template <class T>
+inline void
+GetStyleData(nsIFrame* aFrame, const T** aStyleStruct)
+{
+    aFrame->GetStyleData(NS_GET_STYLESTRUCTID(T),
+                    *NS_REINTERPRET_CAST(const nsStyleStruct**, aStyleStruct));
+}
+
 #endif /* nsIFrame_h___ */
