@@ -43,6 +43,7 @@
 #include "nsAString.h"
 #include "nsIRequest.h"
 #include "nsCompatibility.h"
+#include "nsCOMPtr.h"
 #ifdef IBMBIDI
 class nsBidiPresUtils;
 #endif // IBMBIDI
@@ -61,7 +62,7 @@ class nsIImage;
 class nsILinkHandler;
 class nsIPresShell;
 class nsIPref;
-class nsIStyleContext;
+class nsStyleContext;
 class nsIAtom;
 class nsString;
 class nsIEventStateManager;
@@ -192,9 +193,9 @@ public:
    * Resolve style for the given piece of content that will be a child
    * of the aParentContext. Don't use this for pseudo frames.
    */
-  NS_IMETHOD ResolveStyleContextFor(nsIContent* aContent,
-                                    nsIStyleContext* aParentContext,
-                                    nsIStyleContext** aResult) = 0;
+  virtual already_AddRefed<nsStyleContext>
+  ResolveStyleContextFor(nsIContent* aContent,
+                         nsStyleContext* aParentContext) = 0;
 
   /**
    * Resolve style for a non-element content node (i.e., one that is
@@ -208,19 +209,19 @@ public:
    * represents everything except the first letter.)
    *
    */
-  NS_IMETHOD ResolveStyleContextForNonElement(
-                                    nsIStyleContext* aParentContext,
-                                    nsIStyleContext** aResult) = 0;
+  virtual already_AddRefed<nsStyleContext>
+  ResolveStyleContextForNonElement(nsStyleContext* aParentContext) = 0;
 
   /**
    * Resolve style for a pseudo frame within the given aParentContent & aParentContext.
    * The tag should be lowercase and inclue the colon.
    * ie: NS_NewAtom(":first-line");
    */
-  NS_IMETHOD ResolvePseudoStyleContextFor(nsIContent* aParentContent,
-                                          nsIAtom* aPseudoTag,
-                                          nsIStyleContext* aParentContext,
-                                          nsIStyleContext** aResult) = 0;
+
+  virtual already_AddRefed<nsStyleContext>
+  ResolvePseudoStyleContextFor(nsIContent* aParentContent,
+                               nsIAtom* aPseudoTag,
+                               nsStyleContext* aParentContext) = 0;
 
   /**
    * Resolve style for a pseudo frame within the given aParentContent & aParentContext.
@@ -230,11 +231,11 @@ public:
    * Instead of matching solely on aPseudoTag, a comparator function can be
    * passed in to test.
    */
-  NS_IMETHOD ResolvePseudoStyleWithComparator(nsIContent* aParentContent,
-                                              nsIAtom* aPseudoTag,
-                                              nsIStyleContext* aParentContext,
-                                              nsICSSPseudoComparator* aComparator,
-                                              nsIStyleContext** aResult) = 0;
+  virtual already_AddRefed<nsStyleContext>
+  ResolvePseudoStyleWithComparator(nsIContent* aParentContent,
+                                   nsIAtom* aPseudoTag,
+                                   nsStyleContext* aParentContext,
+                                   nsICSSPseudoComparator* aComparator) = 0;
 
   /**
    * Probe style for a pseudo frame within the given aParentContent & aParentContext.
@@ -242,24 +243,24 @@ public:
    * The tag should be lowercase and inclue the colon.
    * ie: NS_NewAtom(":first-line");
    */
-  NS_IMETHOD ProbePseudoStyleContextFor(nsIContent* aParentContent,
-                                        nsIAtom* aPseudoTag,
-                                        nsIStyleContext* aParentContext,
-                                        nsIStyleContext** aResult) = 0;
-
-  /**
-   * Resolve a new style context for a content node and return the URL
-   * for its XBL binding, or the empty string if it has no binding
-   * specified in CSS.
-   */
-  NS_IMETHOD GetXBLBindingURL(nsIContent* aContent, nsAString& aResult) = 0;
+  virtual already_AddRefed<nsStyleContext>
+  ProbePseudoStyleContextFor(nsIContent* aParentContent,
+                             nsIAtom* aPseudoTag,
+                             nsStyleContext* aParentContext) = 0;
+ 
+   /**
+    * Resolve a new style context for a content node and return the URL
+    * for its XBL binding, or the empty string if it has no binding
+    * specified in CSS.
+    */
+   NS_IMETHOD GetXBLBindingURL(nsIContent* aContent, nsAString& aResult) = 0;
 
   /** 
    * For a given frame tree, get a new style context that is the equivalent
    * but within a new parent
    */
   NS_IMETHOD ReParentStyleContext(nsIFrame* aFrame, 
-                                  nsIStyleContext* aNewParentContext) = 0;
+                                  nsStyleContext* aNewParentContext) = 0;
 
 
   NS_IMETHOD AllocateFromShell(size_t aSize, void** aResult) = 0;
@@ -566,8 +567,8 @@ public:
 
 // Factory method to create a "galley" presentation context (galley is
 // a kind of view that has no limit to the size of a page)
-extern NS_EXPORT nsresult
-  NS_NewGalleyContext(nsIPresContext** aInstancePtrResult);
+nsresult
+NS_NewGalleyContext(nsIPresContext** aInstancePtrResult);
 
 #ifdef MOZ_REFLOW_PERF
 
