@@ -1,23 +1,40 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- */
+ * Contributor(s):
+ *   Kathleen Brade <brade@netscape.com>
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #include "nsCopySupport.h"
 #include "nsIDocumentEncoder.h"
@@ -32,8 +49,15 @@
 #include "nsIPresContext.h"
 #include "nsIDOMNSHTMLInputElement.h"
 #include "nsIDOMNSHTMLTextAreaElement.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIDOMRange.h"
+#include "nsIFrame.h"
+
+#include "nsIDocShell.h"
+#include "nsIClipboardDragDropHooks.h"
+#include "nsIClipboardDragDropHookList.h"
+#include "nsIInterfaceRequestorUtils.h"
 
 // for IBMBIDI
 #include "nsIDocument.h"
@@ -43,14 +67,24 @@
 #include "nsIHTMLDocument.h"
 #include "nsHTMLAtoms.h"
 #ifdef IBMBIDI
-#include "nsIUBidiUtils.h"
-static NS_DEFINE_CID(kUBidiUtilCID, NS_UNICHARBIDIUTIL_CID);
+#include "nsBidiUtils.h"
+//static NS_DEFINE_CID(kUBidiUtilCID, NS_UNICHARBIDIUTIL_CID);
 #endif
 
+// image copy stuff
+#include "nsIDOMHTMLImageElement.h"
+#include "nsLayoutAtoms.h"
+#include "nsIImageLoadingContent.h"
+#include "imgIContainer.h"
+#include "imgIRequest.h"
+#include "nsIInterfaceRequestor.h"
+#include "nsIInterfaceRequestorUtils.h"
+#include "gfxIImageFrame.h"
+#include "nsIImage.h"
+
 static NS_DEFINE_CID(kCClipboardCID,           NS_CLIPBOARD_CID);
 static NS_DEFINE_CID(kCTransferableCID,        NS_TRANSFERABLE_CID);
 static NS_DEFINE_CID(kHTMLConverterCID,        NS_HTMLFORMATCONVERTER_CID);
-static NS_DEFINE_CID(kTextEncoderCID,          NS_TEXT_ENCODER_CID);
 
 // private clipboard data flavors for html copy, used by editor when pasting
 #define kHTMLContext   "text/_moz_htmlcontext"
@@ -119,11 +153,10 @@ nsresult nsCopySupport::HTMLCopy(nsISelection *aSel, nsIDocument *aDoc, PRInt16
       if (context) {
         context->IsArabicEncoding(arabicCharset);
         if (arabicCharset) {
-          nsCOMPtr<nsIUBidiUtils> bidiUtils = do_GetService("@mozilla.org/intl/unicharbidiutil;1");
           PRUint32 bidiOptions;
           PRBool isVisual;
           PRBool isBidiSystem;
-    
+
           context->GetBidi(&bidiOptions);
           context->IsVisualMode(isVisual);
           context->GetIsBidiSystem(isBidiSystem);
@@ -132,14 +165,14 @@ nsresult nsCopySupport::HTMLCopy(nsISelection *aSel, nsIDocument *aDoc, PRInt16
             nsAutoString newBuffer;
             if (isBidiSystem) { 
               if (GET_BIDI_OPTION_DIRECTION(bidiOptions) == IBMBIDI_TEXTDIRECTION_RTL) {
-                bidiUtils->Conv_FE_06(buffer, newBuffer);
+                Conv_FE_06(buffer, newBuffer);
               }
               else {
-                bidiUtils->Conv_FE_06_WithReverse(buffer, newBuffer);
+                Conv_FE_06_WithReverse(buffer, newBuffer);
               }
             }
             else { //nonbidisystem
-              bidiUtils->HandleNumbers(buffer, newBuffer);//ahmed
+              HandleNumbers(buffer, newBuffer);//ahmed
             }
             buffer = newBuffer;
           }
@@ -150,8 +183,8 @@ nsresult nsCopySupport::HTMLCopy(nsISelection *aSel, nsIDocument *aDoc, PRInt16
             if (bidiCharset.EqualsIgnoreCase("UTF-8") || (!isVisual)) {
               if ( (GET_BIDI_OPTION_CLIPBOARDTEXTMODE(bidiOptions) == IBMBIDI_CLIPBOARDTEXTMODE_VISUAL) || (!isBidiSystem) ) {
                 nsAutoString newBuffer;
-                bidiUtils->Conv_06_FE_WithReverse(buffer, newBuffer, GET_BIDI_OPTION_DIRECTION(bidiOptions));
-                bidiUtils->HandleNumbers(newBuffer, buffer);
+                Conv_06_FE_WithReverse(buffer, newBuffer, GET_BIDI_OPTION_DIRECTION(bidiOptions));
+                HandleNumbers(newBuffer, buffer);
               }
             }
           }
@@ -181,23 +214,27 @@ nsresult nsCopySupport::HTMLCopy(nsISelection *aSel, nsIDocument *aDoc, PRInt16
       }
       
       // get wStrings to hold clip data
-      nsCOMPtr<nsISupportsWString> dataWrapper, contextWrapper, infoWrapper;
-      dataWrapper = do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID);
+      nsCOMPtr<nsISupportsString> dataWrapper, contextWrapper, infoWrapper;
+      dataWrapper = do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID);
       NS_ENSURE_TRUE(dataWrapper, NS_ERROR_FAILURE);
       if (bIsHTMLCopy)
       {
-        contextWrapper = do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID);
+        contextWrapper = do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID);
         NS_ENSURE_TRUE(contextWrapper, NS_ERROR_FAILURE);
-        infoWrapper = do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID);
+        infoWrapper = do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID);
         NS_ENSURE_TRUE(infoWrapper, NS_ERROR_FAILURE);
       }
       
       // populate the strings
-      dataWrapper->SetData ( NS_CONST_CAST(PRUnichar*,buffer.get()) );
+      nsresult data_rv = NS_OK, context_rv = NS_OK, info_rv = NS_OK;
+      data_rv =
+        dataWrapper->SetData(buffer);
       if (bIsHTMLCopy)
       {
-        contextWrapper->SetData ( NS_CONST_CAST(PRUnichar*,parents.get()) );
-        infoWrapper->SetData ( NS_CONST_CAST(PRUnichar*,info.get()) );
+        context_rv =
+          contextWrapper->SetData(parents);
+        info_rv =
+          infoWrapper->SetData(info);
       }
       
       // QI the data object an |nsISupports| so that when the transferable holds
@@ -205,20 +242,20 @@ nsresult nsCopySupport::HTMLCopy(nsISelection *aSel, nsIDocument *aDoc, PRInt16
       nsCOMPtr<nsISupports> genericDataObj ( do_QueryInterface(dataWrapper) );
       if (bIsHTMLCopy)
       {
-        if (buffer.Length())
+        if (!buffer.IsEmpty() && NS_SUCCEEDED(data_rv))
         {
           // Add the html DataFlavor to the transferable
           trans->AddDataFlavor(kHTMLMime);
           trans->SetTransferData(kHTMLMime, genericDataObj, buffer.Length()*2);
         }
-        if (parents.Length())
+        if (!parents.IsEmpty() && NS_SUCCEEDED(context_rv))
         {
           // Add the htmlcontext DataFlavor to the transferable
           trans->AddDataFlavor(kHTMLContext);
           genericDataObj = do_QueryInterface(contextWrapper);
           trans->SetTransferData(kHTMLContext, genericDataObj, parents.Length()*2);
         }
-        if (info.Length())
+        if (!info.IsEmpty() && NS_SUCCEEDED(info_rv))
         {
           // Add the htmlinfo DataFlavor to the transferable
           trans->AddDataFlavor(kHTMLInfo);
@@ -228,20 +265,65 @@ nsresult nsCopySupport::HTMLCopy(nsISelection *aSel, nsIDocument *aDoc, PRInt16
       }
       else
       {
-        if (buffer.Length())
+        if (!buffer.IsEmpty() && NS_SUCCEEDED(data_rv))
         {
          // Add the unicode DataFlavor to the transferable
           trans->AddDataFlavor(kUnicodeMime);
           trans->SetTransferData(kUnicodeMime, genericDataObj, buffer.Length()*2);
         }
       }
+
+      PRBool doPutOnClipboard = PR_TRUE;
+      DoHooks(aDoc, trans, &doPutOnClipboard);
+
       // put the transferable on the clipboard
-      clipboard->SetData(trans, nsnull, aClipboardID);
+      if (doPutOnClipboard)
+        clipboard->SetData(trans, nsnull, aClipboardID);
     }
   }
   return rv;
 }
 
+nsresult nsCopySupport::DoHooks(nsIDocument *aDoc, nsITransferable *aTrans,
+                                PRBool *aDoPutOnClipboard)
+{
+  NS_ENSURE_ARG(aDoc);
+
+  *aDoPutOnClipboard = PR_TRUE;
+
+  nsCOMPtr<nsISupports> isupp;
+  nsresult rv = aDoc->GetContainer(getter_AddRefs(isupp));
+  nsCOMPtr<nsIDocShell> docShell = do_QueryInterface(isupp);
+  nsCOMPtr<nsIClipboardDragDropHookList> hookObj = do_GetInterface(docShell);
+  if (!hookObj) return NS_ERROR_FAILURE;
+
+  nsCOMPtr<nsISimpleEnumerator> enumerator;
+  hookObj->GetHookEnumerator(getter_AddRefs(enumerator));
+  if (!enumerator) return NS_ERROR_FAILURE;
+
+  // the logic here should follow the behavior specified in
+  // nsIClipboardDragDropHooks.h
+
+  nsCOMPtr<nsIClipboardDragDropHooks> override;
+  PRBool hasMoreHooks = PR_FALSE;
+  while (NS_SUCCEEDED(enumerator->HasMoreElements(&hasMoreHooks))
+         && hasMoreHooks)
+  {
+    rv = enumerator->GetNext(getter_AddRefs(isupp));
+    if (NS_FAILED(rv)) break;
+    override = do_QueryInterface(isupp);
+    if (override)
+    {
+      nsresult hookResult = override->OnCopyOrDrag(aTrans, aDoPutOnClipboard);
+      NS_ASSERTION(NS_SUCCEEDED(hookResult), "OnCopyOrDrag hook failed");
+      if (!*aDoPutOnClipboard)
+        break;
+    }
+  }
+
+  return rv;
+}
+
 nsresult nsCopySupport::IsPlainTextContext(nsISelection *aSel, nsIDocument *aDoc, PRBool *aIsPlainTextContext)
 {
   nsresult rv;
@@ -293,7 +375,7 @@ nsresult nsCopySupport::IsPlainTextContext(nsISelection *aSel, nsIDocument *aDoc
       nsCOMPtr<nsIDOMElement> bodyElem = do_QueryInterface(selContent);
       nsAutoString wsVal;
       rv = bodyElem->GetAttribute(NS_LITERAL_STRING("style"), wsVal);
-      if (NS_SUCCEEDED(rv) && (kNotFound != wsVal.Find(NS_LITERAL_STRING("-moz-pre-wrap").get())))
+      if (NS_SUCCEEDED(rv) && (kNotFound != wsVal.Find(NS_LITERAL_STRING("-moz-pre-wrap"))))
       {
         *aIsPlainTextContext = PR_TRUE;
         break;
@@ -309,3 +391,121 @@ nsresult nsCopySupport::IsPlainTextContext(nsISelection *aSel, nsIDocument *aDoc
 
   return NS_OK;
 }
+
+nsresult
+nsCopySupport::GetContents(const nsACString& aMimeType, PRUint32 aFlags, nsISelection *aSel, nsIDocument *aDoc, nsAString& outdata)
+{
+  nsresult rv = NS_OK;
+  
+  nsCOMPtr<nsIDocumentEncoder> docEncoder;
+
+  nsCAutoString encoderContractID(NS_DOC_ENCODER_CONTRACTID_BASE);
+  encoderContractID.Append(aMimeType);
+    
+  docEncoder = do_CreateInstance(encoderContractID.get());
+  NS_ENSURE_TRUE(docEncoder, NS_ERROR_FAILURE);
+
+  PRUint32 flags = aFlags;
+  
+  if (aMimeType.Equals("text/plain"))
+    flags |= nsIDocumentEncoder::OutputPreformatted;
+
+  NS_ConvertASCIItoUCS2 unicodeMimeType(aMimeType);
+  rv = docEncoder->Init(aDoc, unicodeMimeType, flags);
+  if (NS_FAILED(rv)) return rv;
+  
+  if (aSel)
+  {
+    rv = docEncoder->SetSelection(aSel);
+    if (NS_FAILED(rv)) return rv;
+  } 
+  
+  // encode the selection
+  return docEncoder->EncodeToString(outdata);
+}
+
+
+nsresult
+nsCopySupport::ImageCopy(nsIDOMHTMLImageElement* imageElement, PRInt16 aClipboardID)
+{
+  nsresult rv;
+  
+  nsCOMPtr<nsIDOMNode> imageNode = do_QueryInterface(imageElement, &rv);
+  if (NS_FAILED(rv)) return rv;
+  if (!imageNode) return NS_ERROR_FAILURE;
+  
+  nsCOMPtr<nsIImage> image;
+  rv = GetImageFromDOMNode(imageNode, getter_AddRefs(image));
+  if (NS_FAILED(rv)) return rv;
+  if (!image) return NS_ERROR_FAILURE;
+
+  // Get the Clipboard
+  nsCOMPtr<nsIClipboard> clipboard(do_GetService(kCClipboardCID, &rv));
+  if (NS_FAILED(rv)) return rv;
+  if (!clipboard) return NS_ERROR_FAILURE;
+
+  // Create a transferable for putting data on the Clipboard
+  nsCOMPtr<nsITransferable> trans = do_CreateInstance(kCTransferableCID, &rv);
+  if (NS_FAILED(rv)) return rv;
+  if (!trans) return NS_ERROR_FAILURE;
+
+  nsCOMPtr<nsISupportsInterfacePointer> ptrPrimitive(do_CreateInstance(NS_SUPPORTS_INTERFACE_POINTER_CONTRACTID, &rv));
+  if (NS_FAILED(rv)) return rv;
+  if (!ptrPrimitive)  return NS_ERROR_FAILURE;
+  ptrPrimitive->SetData(image);
+
+  trans->SetTransferData(kNativeImageMime, ptrPrimitive, sizeof(nsISupports*));
+
+  // put the transferable on the clipboard
+  return clipboard->SetData(trans, nsnull, aClipboardID);
+}
+
+//
+// GetImage
+//
+// Given a dom node that's an image, finds the nsIImage associated with it.
+//
+// XXX see also nsContentAreaDragDrop, and factor!
+nsresult
+nsCopySupport::GetImageFromDOMNode(nsIDOMNode* inNode, nsIImage**outImage)
+{
+  NS_ENSURE_ARG_POINTER(outImage);
+  *outImage = nsnull;
+
+  nsCOMPtr<nsIImageLoadingContent> content(do_QueryInterface(inNode));
+  if (!content) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  nsCOMPtr<imgIRequest> imgRequest;
+  content->GetRequest(nsIImageLoadingContent::CURRENT_REQUEST,
+                      getter_AddRefs(imgRequest));
+  if (!imgRequest) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+  
+  nsCOMPtr<imgIContainer> imgContainer;
+  imgRequest->GetImage(getter_AddRefs(imgContainer));
+
+  if (!imgContainer) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+    
+  nsCOMPtr<gfxIImageFrame> imgFrame;
+  imgContainer->GetFrameAt(0, getter_AddRefs(imgFrame));
+
+  if (!imgFrame) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+  
+  nsCOMPtr<nsIInterfaceRequestor> ir = do_QueryInterface(imgFrame);
+
+  if (!ir) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+  
+  return CallGetInterface(ir.get(), outImage);
+}
+
+
+
