@@ -108,7 +108,7 @@ GalleyContext::SetPageDim(nsRect* aPageDim)
   return NS_ERROR_FAILURE;
 }
 
-NS_EXPORT nsresult
+nsresult
 NS_NewGalleyContext(nsIPresContext** aInstancePtrResult)
 {
   if (aInstancePtrResult == nsnull) {
