@@ -48,7 +48,7 @@
 #include "nsIRenderingContext.h"
 #include "nsIURL.h"
 #include "nsIDocument.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsLayoutAtoms.h"
 #include "nsILookAndFeel.h"
 #include "nsWidgetsCID.h"
@@ -69,14 +69,14 @@
 #include "nsContentPolicyUtils.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIDOMDocument.h"
+#include "nsAutoPtr.h"
 #ifdef IBMBIDI
 #include "nsBidiPresUtils.h"
 #endif // IBMBIDI
 
 // Needed for Start/Stop of Image Animation
 #include "imgIContainer.h"
-#include "nsIDOMHTMLImageElement.h"
-#include "nsIImageFrame.h"
+#include "nsIImageLoadingContent.h"
 
 //needed for resetting of image service color
 #include "nsLayoutCID.h"
@@ -209,15 +209,8 @@ nsPresContext::~nsPresContext()
 {
   mImageLoaders.Enumerate(destroy_loads);
 
-  if (mShell) {
-    nsCOMPtr<nsIDocument> doc;
-    mShell->GetDocument(getter_AddRefs(doc));
-    if (doc) {
-      doc->RemoveCharSetObserver(this);
-    }
-  }
-
-  mShell = nsnull;
+  NS_PRECONDITION(!mShell, "Presshell forgot to clear our mShell pointer");
+  SetShell(nsnull);
 
   if (mEventManager)
     mEventManager->SetPresContext(nsnull);   // unclear if this is needed, but can't hurt
@@ -588,7 +581,7 @@ nsPresContext::ClearStyleDataAndReflow()
     // Clear out all our style data.
     nsCOMPtr<nsIStyleSet> set;
     mShell->GetStyleSet(getter_AddRefs(set));
-    set->ClearStyleData(this, nsnull, nsnull);
+    set->ClearStyleData(this, nsnull);
 
     // Force a reflow of the root frame
     // XXX We really should only do a reflow if a preference that affects
@@ -663,8 +656,19 @@ nsPresContext::Init(nsIDeviceContext* aDeviceContext)
 NS_IMETHODIMP
 nsPresContext::SetShell(nsIPresShell* aShell)
 {
+  if (mShell) {
+    // Remove ourselves as the charset observer from the shell's doc, because
+    // this shell may be going away for good.
+    nsCOMPtr<nsIDocument> doc;
+    mShell->GetDocument(getter_AddRefs(doc));
+    if (doc) {
+      doc->RemoveCharSetObserver(this);
+    }
+  }    
+
   mShell = aShell;
-  if (nsnull != mShell) {
+
+  if (mShell) {
     nsCOMPtr<nsIDocument> doc;
     if (NS_SUCCEEDED(mShell->GetDocument(getter_AddRefs(doc)))) {
       NS_ASSERTION(doc, "expect document here");
@@ -807,7 +811,7 @@ nsPresContext::GetImageAnimationMode(PRUint16* aModeResult)
 // Helper function for setting Anim Mode on image
 static void SetImgAnimModeOnImgReq(imgIRequest* aImgReq, PRUint16 aMode)
 {
-  if (aImgReq != nsnull) {
+  if (aImgReq) {
     nsCOMPtr<imgIContainer> imgCon;
     aImgReq->GetImage(getter_AddRefs(imgCon));
     if (imgCon) {
@@ -834,20 +838,14 @@ PR_STATIC_CALLBACK(PRBool) set_animation_mode(nsHashKey *aKey, void *aData, void
 // this is a way to turn on/off image animations
 void nsPresContext::SetImgAnimations(nsCOMPtr<nsIContent>& aParent, PRUint16 aMode)
 {
-  nsCOMPtr<nsIDOMHTMLImageElement> imgContent(do_QueryInterface(aParent));
+  nsCOMPtr<nsIImageLoadingContent> imgContent(do_QueryInterface(aParent));
   if (imgContent) {
-    nsIFrame* frame;
-    mShell->GetPrimaryFrameFor(aParent, &frame);
-    if (frame != nsnull) {
-      nsIImageFrame* imgFrame = nsnull;
-      CallQueryInterface(frame, &imgFrame);
-      if (imgFrame != nsnull) {
-        nsCOMPtr<imgIRequest> imgReq;
-        imgFrame->GetImageRequest(getter_AddRefs(imgReq));
-        SetImgAnimModeOnImgReq(imgReq, aMode);
-      }
-    }
+    nsCOMPtr<imgIRequest> imgReq;
+    imgContent->GetRequest(nsIImageLoadingContent::CURRENT_REQUEST,
+                           getter_AddRefs(imgReq));
+    SetImgAnimModeOnImgReq(imgReq, aMode);
   }
+  
   PRInt32 count;
   aParent->ChildCount(count);
   for (PRInt32 i=0;i<count;i++) {
@@ -934,109 +932,81 @@ nsPresContext::GetBaseURL(nsIURI** aResult)
   return NS_OK;
 }
 
-NS_IMETHODIMP
+already_AddRefed<nsStyleContext>
 nsPresContext::ResolveStyleContextFor(nsIContent* aContent,
-                                      nsIStyleContext* aParentContext,
-                                      nsIStyleContext** aResult)
+                                      nsStyleContext* aParentContext)
 {
-  NS_PRECONDITION(aResult, "null out param");
-
-  nsIStyleContext* result = nsnull;
-  nsCOMPtr<nsIStyleSet> set;
-  nsresult rv = mShell->GetStyleSet(getter_AddRefs(set));
-  if (NS_SUCCEEDED(rv)) {
-    if (set) {
-      result = set->ResolveStyleFor(this, aContent, aParentContext);
-      if (nsnull == result) {
-        rv = NS_ERROR_OUT_OF_MEMORY;
-      }
-    }
-  }
-  *aResult = result;
-  return rv;
-}
-
-NS_IMETHODIMP
-nsPresContext::ResolveStyleContextForNonElement(
-                                      nsIStyleContext* aParentContext,
-                                      nsIStyleContext** aResult)
-{
-  NS_PRECONDITION(aResult, "null out param");
-
-  nsIStyleContext* result = nsnull;
   nsCOMPtr<nsIStyleSet> set;
   nsresult rv = mShell->GetStyleSet(getter_AddRefs(set));
   if (NS_SUCCEEDED(rv) && set) {
-    result = set->ResolveStyleForNonElement(this, aParentContext);
-    if (!result)
-      rv = NS_ERROR_OUT_OF_MEMORY;
+    // return the addref'd style context
+    return set->ResolveStyleFor(this, aContent, aParentContext);
   }
-  *aResult = result;
-  return rv;
+
+  return nsnull;
 }
 
-NS_IMETHODIMP
+already_AddRefed<nsStyleContext>
+nsPresContext::ResolveStyleContextForNonElement(nsStyleContext* aParentContext)
+{
+  nsCOMPtr<nsIStyleSet> set;
+  nsresult rv = mShell->GetStyleSet(getter_AddRefs(set));
+  if (NS_SUCCEEDED(rv) && set) {
+    // return addrefed style context
+    return set->ResolveStyleForNonElement(this, aParentContext);
+  }
+
+  return nsnull;
+}
+
+already_AddRefed<nsStyleContext>
 nsPresContext::ResolvePseudoStyleContextFor(nsIContent* aParentContent,
                                             nsIAtom* aPseudoTag,
-                                            nsIStyleContext* aParentContext,
-                                            nsIStyleContext** aResult)
+                                            nsStyleContext* aParentContext)
 {
-  return ResolvePseudoStyleWithComparator(aParentContent, aPseudoTag, aParentContext,
-                                          nsnull, aResult);
+  return ResolvePseudoStyleWithComparator(aParentContent, aPseudoTag,
+                                          aParentContext, nsnull);
 }
 
-NS_IMETHODIMP
+already_AddRefed<nsStyleContext>
 nsPresContext::ResolvePseudoStyleWithComparator(nsIContent* aParentContent,
                                                 nsIAtom* aPseudoTag,
-                                                nsIStyleContext* aParentContext,
-                                                nsICSSPseudoComparator* aComparator,
-                                                nsIStyleContext** aResult)
+                                                nsStyleContext* aParentContext,
+                                                nsICSSPseudoComparator* aComparator)
 {
-  NS_PRECONDITION(aResult, "null out param");
-
-  nsIStyleContext* result = nsnull;
   nsCOMPtr<nsIStyleSet> set;
   nsresult rv = mShell->GetStyleSet(getter_AddRefs(set));
-  if (NS_SUCCEEDED(rv)) {
-    if (set) {
-      result = set->ResolvePseudoStyleFor(this, aParentContent, aPseudoTag,
-                                          aParentContext, aComparator);
-      if (nsnull == result) {
-        rv = NS_ERROR_OUT_OF_MEMORY;
-      }
-    }
+  if (NS_SUCCEEDED(rv) && set) {
+    // return addrefed style context
+    return set->ResolvePseudoStyleFor(this, aParentContent, aPseudoTag,
+                                      aParentContext, aComparator);
   }
-  *aResult = result;
-  return rv;
+
+  return nsnull;
 }
 
-NS_IMETHODIMP
+already_AddRefed<nsStyleContext>
 nsPresContext::ProbePseudoStyleContextFor(nsIContent* aParentContent,
                                           nsIAtom* aPseudoTag,
-                                          nsIStyleContext* aParentContext,
-                                          nsIStyleContext** aResult)
+                                          nsStyleContext* aParentContext)
 {
-  NS_PRECONDITION(aResult, "null out param");
-
-  nsIStyleContext* result = nsnull;
   nsCOMPtr<nsIStyleSet> set;
   nsresult rv = mShell->GetStyleSet(getter_AddRefs(set));
-  if (NS_SUCCEEDED(rv)) {
-    if (set) {
-      result = set->ProbePseudoStyleFor(this, aParentContent, aPseudoTag,
-                                        aParentContext);
-    }
+  if (NS_SUCCEEDED(rv) && set) {
+    // return addrefed style context
+    return set->ProbePseudoStyleFor(this, aParentContent, aPseudoTag,
+                                    aParentContext);
   }
-  *aResult = result;
-  return rv;
+
+  return nsnull;
 }
 
 NS_IMETHODIMP
 nsPresContext::GetXBLBindingURL(nsIContent* aContent, nsAString& aResult)
 {
-  nsCOMPtr<nsIStyleContext> sc;
-  nsresult rv = ResolveStyleContextFor(aContent, nsnull, getter_AddRefs(sc));
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsRefPtr<nsStyleContext> sc;
+  sc = ResolveStyleContextFor(aContent, nsnull);
+  NS_ENSURE_TRUE(sc, NS_ERROR_FAILURE);
 
   const nsStyleDisplay* display;
   ::GetStyleData(sc.get(), &display);
@@ -1048,7 +1018,7 @@ nsPresContext::GetXBLBindingURL(nsIContent* aContent, nsAString& aResult)
 
 NS_IMETHODIMP
 nsPresContext::ReParentStyleContext(nsIFrame* aFrame, 
-                                    nsIStyleContext* aNewParentContext)
+                                    nsStyleContext* aNewParentContext)
 {
   NS_PRECONDITION(aFrame, "null ptr");
   if (! aFrame) {
