@@ -107,6 +107,7 @@ MOZ_DECL_CTOR_COUNTER(nsSpaceManager)
 nsSpaceManager::nsSpaceManager(nsIPresShell* aPresShell, nsIFrame* aFrame)
   : mFrame(aFrame),
     mXMost(0),
+    mLowestTop(NSCOORD_MIN),
     mFloatDamage(PSArenaAllocCB, PSArenaFreeCB, aPresShell)
 {
   MOZ_COUNT_CTOR(nsSpaceManager);
@@ -817,6 +818,9 @@ nsSpaceManager::AddRectRegion(nsIFrame* aFrame, const nsRect& aUnavailableSpace)
   if (xmost > mXMost)
     mXMost = xmost;
 
+  if (rect.y > mLowestTop)
+    mLowestTop = rect.y;
+
   // Create a frame info structure
   frameInfo = CreateFrameInfo(aFrame, rect);
   if (nsnull == frameInfo) {
@@ -840,58 +844,6 @@ nsSpaceManager::AddRectRegion(nsIFrame* aFrame, const nsRect& aUnavailableSpace)
   return NS_OK;
 }
 
-nsresult
-nsSpaceManager::ResizeRectRegion(nsIFrame*    aFrame,
-                                 nscoord      aDeltaWidth,
-                                 nscoord      aDeltaHeight,
-                                 AffectedEdge aEdge)
-{
-  // Get the frame info associated with with aFrame
-  FrameInfo*  frameInfo = GetFrameInfoFor(aFrame);
-
-  if (nsnull == frameInfo) {
-    NS_WARNING("no region associated with aFrame");
-    return NS_ERROR_INVALID_ARG;
-  }
-
-  // Compute new rect
-  nsRect  rect(frameInfo->mRect);
-  rect.SizeBy(aDeltaWidth, aDeltaHeight);
-  if (aEdge == LeftEdge) {
-    rect.x += aDeltaWidth;
-  }
-
-  // For the time being just remove it and add it back in. Because
-  // AddRectRegion() operates relative to the local coordinate space,
-  // translate from world coordinates to the local coordinate space
-  rect.MoveBy(-mX, -mY);
-  RemoveRegion(aFrame);
-  return AddRectRegion(aFrame, rect);
-}
-
-nsresult
-nsSpaceManager::OffsetRegion(nsIFrame* aFrame, nscoord aDx, nscoord aDy)
-{
-  // Get the frame info associated with with aFrame
-  FrameInfo*  frameInfo = GetFrameInfoFor(aFrame);
-
-  if (nsnull == frameInfo) {
-    NS_WARNING("no region associated with aFrame");
-    return NS_ERROR_INVALID_ARG;
-  }
-
-  // Compute new rect
-  nsRect  rect(frameInfo->mRect);
-  rect.MoveBy(aDx, aDy);
-
-  // For the time being just remove it and add it back in. Because
-  // AddRectRegion() operates relative to the local coordinate space,
-  // translate from world coordinates to the local coordinate space
-  rect.MoveBy(-mX, -mY);
-  RemoveRegion(aFrame);
-  return AddRectRegion(aFrame, rect);
-}
-
 nsresult
 nsSpaceManager::RemoveRegion(nsIFrame* aFrame)
 {
@@ -1001,6 +953,7 @@ nsSpaceManager::ClearRegions()
 {
   ClearFrameInfo();
   mBandList.Clear();
+  mLowestTop = NSCOORD_MIN;
 }
 
 void
@@ -1038,6 +991,7 @@ nsSpaceManager::PushState()
   state->mX = mX;
   state->mY = mY;
   state->mXMost = mXMost;
+  state->mLowestTop = mLowestTop;
 
   if (mFrameInfoMap) {
     state->mLastFrame = mFrameInfoMap->mFrame;
@@ -1082,6 +1036,7 @@ nsSpaceManager::PopState()
   mX = mSavedStates->mX;
   mY = mSavedStates->mY;
   mXMost = mSavedStates->mXMost;
+  mLowestTop = mSavedStates->mLowestTop;
 
   // Now that we've restored our state, pop the topmost
   // state and delete it.
@@ -1091,6 +1046,14 @@ nsSpaceManager::PopState()
   delete state;
 }
 
+nscoord
+nsSpaceManager::GetLowestRegionTop()
+{
+  if (mLowestTop == NSCOORD_MIN)
+    return mLowestTop;
+  return mLowestTop - mY;
+}
+
 #ifdef DEBUG
 void
 DebugListSpaceManager(nsSpaceManager *aSpaceManager)
@@ -1414,34 +1377,3 @@ nsSpaceManager::BandRect::Length() const
 
   return len;
 }
-
-#ifdef DEBUG
-void
-nsSpaceManager::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const
-{
-  NS_PRECONDITION(aResult, "null OUT parameter pointer");
-  *aResult = sizeof(*this);
-
-  // Add in the size of the band data. Don't count the header which has
-  // already been taken into account
-  if (!mBandList.IsEmpty()) {
-    const BandRect* bandRect = mBandList.Head();
-    do {
-      *aResult += sizeof(*bandRect);
-      if (bandRect->mNumFrames > 1) {
-        PRUint32  voidArraySize;
-
-        bandRect->mFrames->SizeOf(aHandler, &voidArraySize);
-        *aResult += voidArraySize;
-      }
-
-      bandRect = bandRect->Next();
-    } while (bandRect != &mBandList);
-  }
-
-  // Add in the size of the frame info map
-  for (FrameInfo* info = mFrameInfoMap; info; info = info->mNext) {
-    *aResult += sizeof(*info);
-  }
-}
-#endif
