@@ -47,7 +47,6 @@
 class nsIPresShell;
 class nsIFrame;
 class nsVoidArray;
-class nsISizeOfHandler;
 struct nsSize;
 
 #define NS_SPACE_MANAGER_CACHE_SIZE 4
@@ -250,36 +249,16 @@ public:
   nsresult AddRectRegion(nsIFrame*     aFrame,
                          const nsRect& aUnavailableSpace);
 
-  /**
-   * Resize the rectangular region associated with aFrame by the specified
-   * deltas. The height change always applies to the bottom edge or the existing
-   * rect. You specify whether the width change applies to the left or right edge
-   *
-   * Returns NS_OK if successful, NS_ERROR_INVALID_ARG if there is no region
-   * tagged with aFrame
-   */
-  enum AffectedEdge {LeftEdge, RightEdge};
-  nsresult ResizeRectRegion(nsIFrame*    aFrame,
-                            nscoord      aDeltaWidth,
-                            nscoord      aDeltaHeight,
-                            AffectedEdge aEdge = RightEdge);
-
-  /**
-   * Offset the region associated with aFrame by the specified amount.
-   *
-   * Returns NS_OK if successful, NS_ERROR_INVALID_ARG if there is no region
-   * tagged with aFrame
-   */
-  nsresult OffsetRegion(nsIFrame* aFrame, nscoord dx, nscoord dy);
-
   /**
    * Remove the region associated with aFrane.
    *
    * Returns NS_OK if successful and NS_ERROR_INVALID_ARG if there is no region
    * tagged with aFrame
    */
+protected: /* doesn't work in the general case */
   nsresult RemoveRegion(nsIFrame* aFrame);
 
+public:
   /**
    * Clears the list of regions representing the unavailable space.
    */
@@ -315,13 +294,18 @@ public:
    */
   void PopState();
 
+  /**
+   * Get the top of the last region placed into the space manager, to
+   * enforce the rule that a float can't be above an earlier float.
+   * Returns the minimum nscoord value if there are no regions.
+   */
+  nscoord GetLowestRegionTop();
+
 #ifdef DEBUG
   /**
    * Dump the state of the spacemanager out to a file
    */
   nsresult List(FILE* out);
-
-  void SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const;
 #endif
 
 protected:
@@ -344,6 +328,7 @@ protected:
     nscoord mX, mY;
     nsIFrame *mLastFrame;
     nscoord mXMost;
+    nscoord mLowestTop;
     SpaceManagerState *mNext;
 
     SpaceManagerState() : mX(0), mY(0), mLastFrame(nsnull), mNext(nsnull) {}
@@ -355,7 +340,7 @@ public:
   struct BandRect : PRCListStr {
     nscoord   mLeft, mTop;
     nscoord   mRight, mBottom;
-    PRIntn    mNumFrames;    // number of frames occupying this rect
+    PRInt32   mNumFrames;    // number of frames occupying this rect
     union {
       nsIFrame*    mFrame;   // single frame occupying the space
       nsVoidArray* mFrames;  // list of frames occupying the space
@@ -419,6 +404,7 @@ protected:
   nscoord         mX, mY;     // translation from local to global coordinate space
   BandList        mBandList;  // header/sentinel for circular linked list of band rects
   nscoord         mXMost;
+  nscoord         mLowestTop;  // the lowest *top*
   FrameInfo*      mFrameInfoMap;
   nsIntervalSet   mFloatDamage;
 
