@@ -37,9 +37,10 @@
  * ***** END LICENSE BLOCK ***** */
 #include "nsContainerFrame.h"
 #include "nsIContent.h"
+#include "nsIDocument.h"
 #include "nsIPresContext.h"
 #include "nsIRenderingContext.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsRect.h"
 #include "nsPoint.h"
 #include "nsGUIEvent.h"
@@ -47,7 +48,6 @@
 #include "nsIView.h"
 #include "nsIScrollableView.h"
 #include "nsVoidArray.h"
-#include "nsISizeOfHandler.h"
 #include "nsHTMLReflowCommand.h"
 #include "nsHTMLContainerFrame.h"
 #include "nsIFrameManager.h"
@@ -84,7 +84,7 @@ NS_IMETHODIMP
 nsContainerFrame::Init(nsIPresContext*  aPresContext,
                        nsIContent*      aContent,
                        nsIFrame*        aParent,
-                       nsIStyleContext* aContext,
+                       nsStyleContext*  aContext,
                        nsIFrame*        aPrevInFlow)
 {
   nsresult rv;
@@ -110,9 +110,7 @@ nsContainerFrame::SetInitialChildList(nsIPresContext* aPresContext,
   if (!mFrames.IsEmpty()) {
     // We already have child frames which means we've already been
     // initialized
-#ifdef DEBUG_dbaron_off // XXX Fix asserts and remove this ifdef.
     NS_NOTREACHED("unexpected second call to SetInitialChildList");
-#endif
     result = NS_ERROR_UNEXPECTED;
   } else if (aListName) {
     // All we know about is the unnamed principal child list
@@ -140,6 +138,10 @@ nsContainerFrame::Destroy(nsIPresContext* aPresContext)
 
   // Delete the primary child list
   mFrames.DestroyFrames(aPresContext);
+  
+  // Destroy overflow frames now
+  nsFrameList overflowFrames(GetOverflowFrames(aPresContext, PR_TRUE));
+  overflowFrames.DestroyFrames(aPresContext);
 
   // Destroy the frame and remove the flow pointers
   return nsSplittableFrame::Destroy(aPresContext);
@@ -168,6 +170,23 @@ nsContainerFrame::FirstChild(nsIPresContext* aPresContext,
   }
 }
 
+NS_IMETHODIMP
+nsContainerFrame::GetAdditionalChildListName(PRInt32   aIndex,
+                                             nsIAtom** aListName) const
+{
+  NS_PRECONDITION(nsnull != aListName, "null OUT parameter pointer");
+  if (aIndex < 0) {
+    return NS_ERROR_INVALID_ARG;
+  }
+  if (aIndex == 0) {
+    *aListName = nsLayoutAtoms::overflowList;
+    NS_ADDREF(*aListName);
+  } else {
+    *aListName = nsnull;
+  }
+  return NS_OK;
+}
+
 /////////////////////////////////////////////////////////////////////////////
 // Painting/Events
 
@@ -344,9 +363,7 @@ nsContainerFrame::GetFrameForPointUsing(nsIPresContext* aPresContext,
   }
 
   if ( inThisFrame && aConsiderSelf ) {
-    const nsStyleVisibility* vis = 
-      (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
-    if (vis->IsVisible()) {
+    if (GetStyleVisibility()->IsVisible()) {
       *aFrame = this;
       return NS_OK;
     }
@@ -444,6 +461,10 @@ nsContainerFrame::PositionFrameView(nsIPresContext* aPresContext,
     // the frame tree of some enclosing document. We do nothing in that case,
     // but we have to check that containingView is nonnull or we will crash.
     if (nsnull != containingView && containingView != parentView) {
+      NS_ERROR("This hack should not be needed now!!! See bug 126263.");
+
+      // XXX roc this is no longer needed; remove it!!
+
       // it is possible for parent view not to have a frame attached to it
       // kind of an anonymous view. This happens with native scrollbars and
       // the clip view. To fix this we need to go up and parentView chain
@@ -514,9 +535,8 @@ NonZeroStyleCoord(const nsStyleCoord& aCoord) {
 }
 
 static PRBool
-HasNonZeroBorderRadius(nsIStyleContext* aStyleContext) {
-  const nsStyleBorder* border;
-  ::GetStyleData(aStyleContext, &border);
+HasNonZeroBorderRadius(nsStyleContext* aStyleContext) {
+  const nsStyleBorder* border = aStyleContext->GetStyleBorder();
 
   nsStyleCoord coord;
   border->mBorderRadius.GetTop(coord);
@@ -534,7 +554,7 @@ HasNonZeroBorderRadius(nsIStyleContext* aStyleContext) {
 static void
 SyncFrameViewGeometryDependentProperties(nsIPresContext*  aPresContext,
                                          nsIFrame*        aFrame,
-                                         nsIStyleContext* aStyleContext,
+                                         nsStyleContext*  aStyleContext,
                                          nsIView*         aView,
                                          PRUint32         aFlags)
 {
@@ -559,18 +579,44 @@ SyncFrameViewGeometryDependentProperties(nsIPresContext*  aPresContext,
     (bg->mBackgroundFlags & NS_STYLE_BG_COLOR_TRANSPARENT) ||
     !aFrame->CanPaintBackground() ||
     HasNonZeroBorderRadius(aStyleContext);
-  if (isCanvas && viewHasTransparentContent) {
+
+  if (isCanvas) {
     nsIView* rootView;
     vm->GetRootView(rootView);
     nsIView* rootParent;
     rootView->GetParent(rootParent);
-    if (nsnull == rootParent) {
+    if (!rootParent) {
+      // We're the root of a view manager hierarchy. We will have to
+      // paint something. NOTE: this can be overridden below.
       viewHasTransparentContent = PR_FALSE;
     }
-  }
 
-  const nsStyleDisplay* display;
-  ::GetStyleData(aStyleContext, &display);
+    nsCOMPtr<nsIPresShell> shell;
+    aPresContext->GetShell(getter_AddRefs(shell));
+    nsCOMPtr<nsIDocument> doc;
+    shell->GetDocument(getter_AddRefs(doc));
+    if (doc) {
+      nsCOMPtr<nsIDocument> parentDoc;
+      doc->GetParentDocument(getter_AddRefs(parentDoc));
+      nsCOMPtr<nsIContent> rootElem;
+      doc->GetRootContent(getter_AddRefs(rootElem));
+      if (!parentDoc && rootElem && rootElem->IsContentOfType(nsIContent::eXUL)) {
+        // we're XUL at the root of the document hierarchy. Try to make our
+        // window translucent.
+        nsCOMPtr<nsIWidget> widget;
+        aView->GetWidget(*getter_AddRefs(widget));
+        // don't proceed unless this is the root view
+        // (sometimes the non-root-view is a canvas)
+        if (widget && aView == rootView) {
+          viewHasTransparentContent = hasBG && (bg->mBackgroundFlags & NS_STYLE_BG_COLOR_TRANSPARENT);
+          widget->SetWindowTranslucency(viewHasTransparentContent);
+        }
+      }
+    }
+  }
+  // XXX we should also set widget transparency for XUL popups
+
+  const nsStyleDisplay* display = aStyleContext->GetStyleDisplay();
   nsFrameState kidState;
   aFrame->GetFrameState(&kidState);
   
@@ -578,8 +624,7 @@ SyncFrameViewGeometryDependentProperties(nsIPresContext*  aPresContext,
     // If we're showing the view but the frame is hidden, then the view is transparent
     nsViewVisibility visibility;
     aView->GetVisibility(visibility);
-    const nsStyleVisibility* vis;
-    ::GetStyleData(aStyleContext, &vis);
+    const nsStyleVisibility* vis = aStyleContext->GetStyleVisibility();
     if ((nsViewVisibility_kShow == visibility
          && NS_STYLE_VISIBILITY_HIDDEN == vis->mVisible)
         || (NS_STYLE_OVERFLOW_VISIBLE == display->mOverflow
@@ -630,10 +675,8 @@ SyncFrameViewGeometryDependentProperties(nsIPresContext*  aPresContext,
     }
 
     if (hasOverflowClip) {
-      const nsStyleBorder* borderStyle;
-      ::GetStyleData(aStyleContext, &borderStyle);
-      const nsStylePadding* paddingStyle;
-      ::GetStyleData(aStyleContext, &paddingStyle);
+      const nsStyleBorder* borderStyle = aStyleContext->GetStyleBorder();
+      const nsStylePadding* paddingStyle = aStyleContext->GetStylePadding();
 
       nsMargin border, padding;
       // XXX We don't support the 'overflow-clip' property yet so just use the
@@ -735,8 +778,7 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
     // detect whether it has or not. Likewise, whether the view size
     // has changed or not, we may need to change the transparency
     // state even if there is no clip.
-    nsCOMPtr<nsIStyleContext> savedStyleContext;
-    aFrame->GetStyleContext(getter_AddRefs(savedStyleContext));
+    nsStyleContext* savedStyleContext = aFrame->GetStyleContext();
     SyncFrameViewGeometryDependentProperties(aPresContext, aFrame, savedStyleContext, aView, aFlags);
   }
 }
@@ -744,7 +786,7 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
 void
 nsContainerFrame::SyncFrameViewAfterSizeChange(nsIPresContext*  aPresContext,
                                                nsIFrame*        aFrame,
-                                               nsIStyleContext* aStyleContext,
+                                               nsStyleContext*  aStyleContext,
                                                nsIView*         aView,
                                                PRUint32         aFlags)
 {
@@ -752,10 +794,8 @@ nsContainerFrame::SyncFrameViewAfterSizeChange(nsIPresContext*  aPresContext,
     return;
   }
   
-  nsCOMPtr<nsIStyleContext> savedStyleContext;
   if (nsnull == aStyleContext) {
-    aFrame->GetStyleContext(getter_AddRefs(savedStyleContext));
-    aStyleContext = savedStyleContext;
+    aStyleContext = aFrame->GetStyleContext();
   }
 
   SyncFrameViewGeometryDependentProperties(aPresContext, aFrame, aStyleContext, aView, aFlags);
@@ -764,7 +804,7 @@ nsContainerFrame::SyncFrameViewAfterSizeChange(nsIPresContext*  aPresContext,
 void
 nsContainerFrame::SyncFrameViewProperties(nsIPresContext*  aPresContext,
                                           nsIFrame*        aFrame,
-                                          nsIStyleContext* aStyleContext,
+                                          nsStyleContext*  aStyleContext,
                                           nsIView*         aView,
                                           PRUint32         aFlags)
 {
@@ -775,14 +815,11 @@ nsContainerFrame::SyncFrameViewProperties(nsIPresContext*  aPresContext,
   nsCOMPtr<nsIViewManager> vm;
   aView->GetViewManager(*getter_AddRefs(vm));
 
-  nsCOMPtr<nsIStyleContext> savedStyleContext;
   if (nsnull == aStyleContext) {
-    aFrame->GetStyleContext(getter_AddRefs(savedStyleContext));
-    aStyleContext = savedStyleContext;
+    aStyleContext = aFrame->GetStyleContext();
   }
     
-  const nsStyleVisibility* vis;
-  ::GetStyleData(aStyleContext, &vis);
+  const nsStyleVisibility* vis = aStyleContext->GetStyleVisibility();
 
   // Set the view's opacity
   vm->SetViewOpacity(aView, vis->mOpacity);
@@ -795,26 +832,13 @@ nsContainerFrame::SyncFrameViewProperties(nsIPresContext*  aPresContext,
     if (NS_STYLE_VISIBILITY_COLLAPSE == vis->mVisible) {
       viewIsVisible = PR_FALSE;
     }
-    else if (NS_STYLE_VISIBILITY_HIDDEN == vis->mVisible) {
-      // If it has a widget, hide the view because the widget can't deal with it
-      nsCOMPtr<nsIWidget> widget;
-      aView->GetWidget(*getter_AddRefs(widget));
-      if (widget) {
-        viewIsVisible = PR_FALSE;
-      }
-      else {
-        // If it's a scroll frame or a list control frame which is derived from the scrollframe, 
-        // then hide the view. This means that
-        // child elements can't override their parent's visibility, but
-        // it's not practical to leave it visible in all cases because
-        // the scrollbars will be showing
-        nsCOMPtr<nsIAtom> frameType;
-        aFrame->GetFrameType(getter_AddRefs(frameType));
-
-        if (frameType == nsLayoutAtoms::scrollFrame || frameType == nsLayoutAtoms::listControlFrame) {
-          viewIsVisible = PR_FALSE;
-        }
-      }
+    else if (NS_STYLE_VISIBILITY_HIDDEN == vis->mVisible &&
+             !aFrame->SupportsVisibilityHidden()) {
+      // If it's a scrollable frame that can't hide its scrollbars,
+      // hide the view. This means that child elements can't override
+      // their parent's visibility, but it's not practical to leave it
+      // visible in all cases because the scrollbars will be showing
+      viewIsVisible = PR_FALSE;
     } else {
       // if the view is for a popup, don't show the view if the popup is closed
       nsCOMPtr<nsIWidget> widget;
@@ -832,15 +856,13 @@ nsContainerFrame::SyncFrameViewProperties(nsIPresContext*  aPresContext,
                           nsViewVisibility_kHide);
   }
 
-  const nsStyleDisplay* display;
-  ::GetStyleData(aStyleContext, &display);
+  const nsStyleDisplay* display = aStyleContext->GetStyleDisplay();
   // See if the frame is being relatively positioned or absolutely
   // positioned
   PRBool isTopMostView = display->IsPositioned();
 
   // Make sure z-index is correct
-  const nsStylePosition* position;
-  ::GetStyleData(aStyleContext, &position);
+  const nsStylePosition* position = aStyleContext->GetStylePosition();
 
   PRInt32 zIndex = 0;
   PRBool  autoZIndex = PR_FALSE;
@@ -859,10 +881,9 @@ nsContainerFrame::SyncFrameViewProperties(nsIPresContext*  aPresContext,
 PRBool
 nsContainerFrame::FrameNeedsView(nsIPresContext* aPresContext,
                                  nsIFrame* aFrame,
-                                 nsIStyleContext* aStyleContext)
+                                 nsStyleContext* aStyleContext)
 {
-  const nsStyleVisibility* vis;
-  ::GetStyleData(aStyleContext, &vis);
+  const nsStyleVisibility* vis = aStyleContext->GetStyleVisibility();
     
   if (vis->mOpacity != 1.0f) {
     return PR_TRUE;
@@ -878,8 +899,7 @@ nsContainerFrame::FrameNeedsView(nsIPresContext* aPresContext,
     return PR_TRUE;
   }
     
-  const nsStyleDisplay* display;
-  ::GetStyleData(aStyleContext, &display);
+  const nsStyleDisplay* display = aStyleContext->GetStyleDisplay();
 
   if (NS_STYLE_POSITION_RELATIVE == display->mPosition) {
     return PR_TRUE;
@@ -887,8 +907,7 @@ nsContainerFrame::FrameNeedsView(nsIPresContext* aPresContext,
     return PR_TRUE;
   } 
 
-  nsCOMPtr<nsIAtom>  pseudoTag;
-  aStyleContext->GetPseudoType(*getter_AddRefs(pseudoTag));
+  nsCOMPtr<nsIAtom>  pseudoTag = aStyleContext->GetPseudoType();
   if (pseudoTag == nsCSSAnonBoxes::scrolledContent) {
     return PR_TRUE;
   }
@@ -1367,14 +1386,4 @@ nsContainerFrame::List(nsIPresContext* aPresContext, FILE* out, PRInt32 aIndent)
 
   return NS_OK;
 }
-
-NS_IMETHODIMP
-nsContainerFrame::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const
-{
-  if (!aResult) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  *aResult = sizeof(*this);
-  return NS_OK;
-}
 #endif
