@@ -39,7 +39,7 @@
 #include "nsBlockFrame.h"
 #include "nsHTMLAtoms.h"
 #include "nsHTMLParts.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsIPresShell.h"
 #include "nsIPresContext.h"
 #include "nsIRenderingContext.h"
@@ -48,6 +48,7 @@
 #include "nsLayoutAtoms.h"
 #include "nsCSSAnonBoxes.h"
 #include "nsReflowPath.h"
+#include "nsAutoPtr.h"
 #ifdef ACCESSIBILITY
 #include "nsIServiceManager.h"
 #include "nsIAccessibilityService.h"
@@ -57,10 +58,8 @@
 #undef NOISY_PUSHING
 #endif
 
-nsIID nsInlineFrame::kInlineFrameCID = NS_INLINE_FRAME_CID;
-
-
 
+NS_DEFINE_IID(kInlineFrameCID, NS_INLINE_FRAME_CID);
 
 
 //////////////////////////////////////////////////////////////////////
@@ -920,7 +919,8 @@ NS_IMETHODIMP nsInlineFrame::GetAccessible(nsIAccessible** aAccessible)
   *aAccessible = nsnull;
   nsCOMPtr<nsIAtom> tagAtom;
   mContent->GetTag(*getter_AddRefs(tagAtom));
-  if (tagAtom == nsHTMLAtoms::img || tagAtom == nsHTMLAtoms::input || tagAtom == nsHTMLAtoms::hr) {
+  if (tagAtom == nsHTMLAtoms::img || tagAtom == nsHTMLAtoms::input || 
+     tagAtom == nsHTMLAtoms::label || tagAtom == nsHTMLAtoms::hr) {
     // Only get accessibility service if we're going to use it
     nsCOMPtr<nsIAccessibilityService> accService(do_GetService("@mozilla.org/accessibilityService;1"));
     if (!accService)
@@ -929,6 +929,8 @@ NS_IMETHODIMP nsInlineFrame::GetAccessible(nsIAccessible** aAccessible)
       return accService->CreateHTML4ButtonAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
     else if (tagAtom == nsHTMLAtoms::img)  // Create accessible for broken <img>
       return accService->CreateHTMLImageAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
+    else if (tagAtom == nsHTMLAtoms::label)  // Creat accessible for <label>
+      return accService->CreateHTMLLabelAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
     // Create accessible for <hr>
     return accService->CreateHTMLHRAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
   }
@@ -943,7 +945,7 @@ NS_IMETHODIMP nsInlineFrame::GetAccessible(nsIAccessible** aAccessible)
 
 static void
 ReParentChildListStyle(nsIPresContext* aPresContext,
-                       nsIStyleContext* aParentStyleContext,
+                       nsStyleContext* aParentStyleContext,
                        nsFrameList& aFrameList)
 {
   nsIFrame* kid = aFrameList.FirstChild();
@@ -1090,26 +1092,22 @@ nsFirstLineFrame::Reflow(nsIPresContext* aPresContext,
       // proper parent context.
       nsIFrame* parentFrame;
       first->GetParent(&parentFrame);
-      nsIStyleContext* parentContext;
-      parentFrame->GetStyleContext(&parentContext);
+      nsStyleContext* parentContext = parentFrame->GetStyleContext();
       if (parentContext) {
         // Create a new style context that is a child of the parent
         // style context thus removing the :first-line style. This way
         // we behave as if an anonymous (unstyled) span was the child
         // of the parent frame.
-        nsIStyleContext* newSC;
-        aPresContext->ResolvePseudoStyleContextFor(nsnull,
-                      nsCSSAnonBoxes::mozLineFrame, parentContext, &newSC);
+        nsRefPtr<nsStyleContext> newSC = aPresContext->ResolvePseudoStyleContextFor(nsnull,
+                                                                                    nsCSSAnonBoxes::mozLineFrame,
+                                                                                    parentContext);
         if (newSC) {
           // Switch to the new style context.
           SetStyleContext(aPresContext, newSC);
 
           // Re-resolve all children
           ReParentChildListStyle(aPresContext, mStyleContext, mFrames);
-
-          NS_RELEASE(newSC);
         }
-        NS_RELEASE(parentContext);
       }
     }
   }
@@ -1369,15 +1367,3 @@ nsPositionedInlineFrame::Reflow(nsIPresContext*          aPresContext,
 
   return rv;
 }
-
-#ifdef DEBUG
-NS_IMETHODIMP
-nsPositionedInlineFrame::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const
-{
-  if (!aResult) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  *aResult = sizeof(*this);
-  return NS_OK;
-}
-#endif
