@@ -120,8 +120,6 @@ public:
 
   NS_IMETHOD CanContinueTextRun(PRBool& aContinueTextRun) const;
 
-  static nsIID kInlineFrameCID;
-
   // Take all of the frames away from this frame. The caller is
   // presumed to keep them alive.
   void StealAllFrames() {
@@ -214,9 +212,6 @@ public:
   virtual ~nsPositionedInlineFrame() { } // useful for debugging
 
   NS_IMETHOD Destroy(nsIPresContext* aPresContext);
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const;
-#endif
 
   NS_IMETHOD SetInitialChildList(nsIPresContext* aPresContext,
                                  nsIAtom*        aListName,
