@@ -38,7 +38,7 @@
 #include "nsHTMLParts.h"
 #include "nsIContent.h"
 #include "nsIPresContext.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsIRenderingContext.h"
 #include "nsHTMLAtoms.h"
 #include "nsLayoutAtoms.h"
@@ -524,7 +524,7 @@ nsPageFrame::DrawHeaderFooter(nsIPresContext*      aPresContext,
     if (len == 0) {
       return; // bail is empty string
     }
-    // find how much text fits, the "position" is the size of the avilable area
+    // find how much text fits, the "position" is the size of the available area
     if (BinarySearchForPosition(&aRenderingContext, text, 0, 0, 0, len,
                                 PRInt32(contentWidth), indx, textWidth)) {
       if (indx < len-1 && len > 3) {
