@@ -14,7 +14,7 @@
  *
  * The Original Code is Mozilla Communicator client code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
@@ -203,23 +203,42 @@ nsresult
 nsTextTransformer::Init(nsIFrame* aFrame,
                         nsIContent* aContent,
                         PRInt32 aStartingOffset,
+                        PRBool aForceArabicShaping,
                         PRBool aLeaveAsAscii)
 {
-#ifdef IBMBIDI
+  /*
+   * If the document has Bidi content, check whether we need to do
+   * Arabic shaping.
+   *
+   *  Does the frame contains Arabic characters
+   *   (mCharType == eCharType_RightToLeftArabic)?
+   *  Are we rendering character by character (aForceArabicShaping ==
+   *   PR_TRUE)? If so, we always do our own Arabic shaping, even if
+   *   the platform has native shaping support. Otherwise, we only do
+   *   shaping if the platform has no shaping support.
+   *
+   *  We do numeric shaping in all Bidi documents.
+   */
   PRBool bidiEnabled;
 
   mPresContext->GetBidiEnabled(&bidiEnabled);
   if (bidiEnabled) {
-    PRBool isBidiSystem;
     aFrame->GetBidiProperty(mPresContext, nsLayoutAtoms::charType,
                             (void**)&mCharType, sizeof(mCharType));
-    mPresContext->GetIsBidiSystem(isBidiSystem);
-    if (mCharType == eCharType_RightToLeftArabic && !isBidiSystem) {
-      SetNeedsArabicShaping(PR_TRUE);
+    if (mCharType == eCharType_RightToLeftArabic) {
+      if (aForceArabicShaping) {
+        SetNeedsArabicShaping(PR_TRUE);
+      }
+      else {
+        PRBool isBidiSystem;
+        mPresContext->GetIsBidiSystem(isBidiSystem);
+        if (!isBidiSystem) {
+          SetNeedsArabicShaping(PR_TRUE);
+        }
+      }
     }
     SetNeedsNumericShaping(PR_TRUE);
   }
-#endif
 
   // Get the contents text content
   nsresult rv;
@@ -1049,14 +1068,20 @@ nsTextTransformer::GetNextWord(PRBool aInWord,
             eLanguageSpecificTransformType_None) {
           LanguageSpecificTransform(result, wordLen, aWasTransformed);
         }
-#ifdef IBMBIDI
         if (NeedsArabicShaping()) {
           DoArabicShaping(result, wordLen, aWasTransformed);
         }
+        else {
+          // We need to strip zero-width joiners and non-joiners even when not
+          // doing Arabic shaping, in order to catch cases where these
+          // characters appear in preshaped text (Bug 192088)
+          // We can't strip them any earlier, because they affect the results
+          // of the Arabic shaping algorithm.
+          StripZeroWidthJoinControls(result, result, wordLen, aWasTransformed);
+        }
         if (NeedsNumericShaping()) {
           DoNumericShaping(result, wordLen, aWasTransformed);
         }
-#endif
       }
     }
 
@@ -1075,9 +1100,12 @@ nsTextTransformer::GetNextWord(PRBool aInWord,
   if ((mTextTransform != NS_STYLE_TEXT_TRANSFORM_NONE) ||
       (*aWordLenResult != *aContentLenResult)) {
     *aWasTransformed = PR_TRUE;
+    mBufferPos = prevBufferPos + *aWordLenResult;
   }
 
   mOffset = offset;
+
+  NS_ASSERTION(mBufferPos == prevBufferPos + *aWordLenResult, "internal error");
   return result;
 }
 
@@ -1452,7 +1480,6 @@ nsTextTransformer::GetPrevWord(PRBool aInWord,
   return result;
 }
 
-#ifdef IBMBIDI
 void
 nsTextTransformer::DoArabicShaping(PRUnichar* aText, 
                                    PRInt32& aTextLength, 
@@ -1471,19 +1498,10 @@ nsTextTransformer::DoArabicShaping(PRUnichar* aText,
   
   ArabicShaping(aText, buf.Length(), buffer, (PRUint32 *)&newLen, !isVisual, !isVisual);
 
-  PRUnichar *source = buffer;
-  PRUnichar *target = aText;
-  for (PRInt32 i = 0; i < newLen; i++) {
-    if (*source == CH_ZWNJ || *source == CH_ZWJ) {
-      source++;
-    }
-    else {
-      *target++ = *source++;
-    }
-  }
-
-  aTextLength = target - aText;
+  aTextLength = newLen;
   *aWasTransformed = PR_TRUE;
+
+  StripZeroWidthJoinControls(buffer, aText, aTextLength, aWasTransformed);
 }
 
 void
@@ -1537,7 +1555,29 @@ nsTextTransformer::DoNumericShaping(PRUnichar* aText,
       break;
   }
 }
-#endif
+
+void
+nsTextTransformer::StripZeroWidthJoinControls(PRUnichar* aSource,
+                                              PRUnichar* aTarget,
+                                              PRInt32& aTextLength, 
+                                              PRBool* aWasTransformed)
+{
+  PRUnichar *src, *dest;
+  PRInt32 stripped = 0;
+
+  src = aSource;
+  dest = aTarget;
+
+  for (PRInt32 i = 0; i < aTextLength; ++i) {
+    while (*src == CH_ZWNJ || *src == CH_ZWJ) {
+      ++stripped;
+      ++src;
+      *aWasTransformed = PR_TRUE;
+    }
+    *dest++ = *src++;
+  }
+  aTextLength -= stripped;
+}
 
 //----------------------------------------------------------------------
 // Self test logic for this class. This will (hopefully) make sure
