@@ -50,6 +50,7 @@
 #include "nsIDOMNode.h"
 #include "nsLayoutAtoms.h"
 #include "nsReflowPath.h"
+#include "nsAutoPtr.h"
 // MouseEvent suppression in PP
 #include "nsGUIEvent.h"
 
@@ -196,9 +197,7 @@ nsGfxButtonControlFrame::CreateAnonymousContent(nsIPresContext* aPresContext,
   }
 
   // Compress whitespace out of label if needed.
-  const nsStyleText* textStyle;
-  GetStyleData(eStyleStruct_Text,  (const nsStyleStruct *&)textStyle);
-  if (!textStyle->WhiteSpaceIsSignificant()) {
+  if (!GetStyleText()->WhiteSpaceIsSignificant()) {
     value.CompressWhitespace();
   } else if (value.Length() > 2 && value[0] == ' ' &&
              value[value.Length() - 1] == ' '){
@@ -253,17 +252,13 @@ nsGfxButtonControlFrame::CreateFrameFor(nsIPresContext*   aPresContext,
     aPresContext->GetShell(getter_AddRefs(shell));
 
     nsIFrame * parentFrame = mFrames.FirstChild();
-    nsCOMPtr<nsIStyleContext> styleContext;
-    parentFrame->GetStyleContext(getter_AddRefs(styleContext));
+    nsStyleContext* styleContext = parentFrame->GetStyleContext();
 
     rv = NS_NewTextFrame(shell, &newFrame);
     if (NS_FAILED(rv)) { return rv; }
     if (!newFrame)   { return NS_ERROR_NULL_POINTER; }
-    nsCOMPtr<nsIStyleContext> textStyleContext;
-    rv = aPresContext->ResolveStyleContextForNonElement(
-                                             styleContext,
-                                             getter_AddRefs(textStyleContext));
-    if (NS_FAILED(rv))     { return rv; }
+    nsRefPtr<nsStyleContext> textStyleContext;
+    textStyleContext = aPresContext->ResolveStyleContextForNonElement(styleContext);
     if (!textStyleContext) { return NS_ERROR_NULL_POINTER; }
 
     if (styleContext) {
@@ -425,8 +420,7 @@ nsGfxButtonControlFrame::HandleEvent(nsIPresContext* aPresContext,
   // takes cares of calling MouseClicked for us.
 
   // do we have user-input style?
-  const nsStyleUserInterface* uiStyle;
-  GetStyleData(eStyleStruct_UserInterface,  (const nsStyleStruct *&)uiStyle);
+  const nsStyleUserInterface* uiStyle = GetStyleUserInterface();
   if (uiStyle->mUserInput == NS_STYLE_USER_INPUT_NONE || uiStyle->mUserInput == NS_STYLE_USER_INPUT_DISABLED)
     return nsFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
   
