@@ -47,7 +47,7 @@
 #include "nsIRenderingContext.h"
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsLeafFrame.h"
 #include "nsCSSRendering.h"
 #include "nsISupports.h"
@@ -123,15 +123,13 @@ NS_IMETHODIMP
 nsHTMLButtonControlFrame::Init(nsIPresContext*  aPresContext,
               nsIContent*      aContent,
               nsIFrame*        aParent,
-              nsIStyleContext* aContext,
+              nsStyleContext*  aContext,
               nsIFrame*        aPrevInFlow)
 {
   nsresult  rv = nsHTMLContainerFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
   mRenderer.SetFrame(this,aPresContext);
    // cache our display type
-  const nsStyleDisplay* styleDisplay;
-  GetStyleData(eStyleStruct_Display, (const nsStyleStruct*&) styleDisplay);
-  mInline = (NS_STYLE_DISPLAY_BLOCK != styleDisplay->mDisplay);
+  mInline = (NS_STYLE_DISPLAY_BLOCK != GetStyleDisplay()->mDisplay);
 
   PRUint32 flags = NS_BLOCK_SPACE_MGR;
   if (mInline) {
@@ -145,13 +143,11 @@ nsHTMLButtonControlFrame::Init(nsIPresContext*  aPresContext,
   mFrames.SetFrames(areaFrame);
 
   // Resolve style and initialize the frame
-  nsIStyleContext* styleContext;
-  aPresContext->ResolvePseudoStyleContextFor(mContent,
+  nsRefPtr<nsStyleContext> styleContext;
+  styleContext = aPresContext->ResolvePseudoStyleContextFor(mContent,
                                             nsCSSAnonBoxes::buttonContent,
-                                            mStyleContext,
-                                            &styleContext);
+                                            mStyleContext);
   mFrames.FirstChild()->Init(aPresContext, mContent, this, styleContext, nsnull);
-  NS_RELEASE(styleContext);                                           
 
   return rv;
 }
@@ -311,10 +307,7 @@ nsHTMLButtonControlFrame::GetFrameForPoint(nsIPresContext* aPresContext,
 {
   if (aWhichLayer == NS_FRAME_PAINT_LAYER_FOREGROUND &&
       mRect.Contains(aPoint)) {
-    const nsStyleVisibility* vis = 
-      (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
-      
-    if (vis->IsVisible()) {
+    if (GetStyleVisibility()->IsVisible()) {
       *aFrame = this;
       return NS_OK;
     }
@@ -333,14 +326,14 @@ nsHTMLButtonControlFrame::SetInitialChildList(nsIPresContext* aPresContext,
   //
   nsCOMPtr<nsIPresShell> shell;
   nsCOMPtr<nsIFrameManager> frameManager;
-  nsCOMPtr<nsIStyleContext> newParentContext;
+  nsStyleContext* newParentContext;
   aPresContext->GetShell(getter_AddRefs(shell));
   if (shell) {
     shell->GetFrameManager(getter_AddRefs(frameManager));
   }
   // get the new parent context from the first child: that is the frame that the
   // subsequent children will be made children of
-  mFrames.FirstChild()->GetStyleContext(getter_AddRefs(newParentContext));
+  newParentContext = mFrames.FirstChild()->GetStyleContext();
 
   // Set the parent for each of the child frames
   for (nsIFrame* frame = aChildList; nsnull != frame; frame->GetNextSibling(&frame)) {
@@ -382,8 +375,7 @@ nsHTMLButtonControlFrame::Paint(nsIPresContext*      aPresContext,
   // but the real problem is the FirstChild (the AreaFrame)
   // isn't being constrained properly
   // Bug #17474
-  const nsStyleBorder* borderStyle;
-  GetStyleData(eStyleStruct_Border,  (const nsStyleStruct *&)borderStyle);
+  const nsStyleBorder* borderStyle = GetStyleBorder();
   nsMargin border;
   border.SizeTo(0, 0, 0, 0);
   borderStyle->CalcBorderFor(this, border);
@@ -452,35 +444,34 @@ nsHTMLButtonControlFrame::Reflow(nsIPresContext* aPresContext,
     GetView(&view);
     if (!view) {
       nsresult result = nsComponentManager::CreateInstance(kViewCID, nsnull, NS_GET_IID(nsIView), (void **)&view);
-	    nsCOMPtr<nsIPresShell> presShell;
+      nsCOMPtr<nsIPresShell> presShell;
       aPresContext->GetShell(getter_AddRefs(presShell));
-	    nsCOMPtr<nsIViewManager> viewMan;
+      nsCOMPtr<nsIViewManager> viewMan;
       presShell->GetViewManager(getter_AddRefs(viewMan));
 
       nsIFrame* parWithView;
-	    nsIView *parView;
+      nsIView *parView;
       GetParentWithView(&parWithView);
-	    parWithView->GetView(&parView);
+      parWithView->GetView(&parView);
       // the view's size is not know yet, but its size will be kept in synch with our frame.
       nsRect boundBox(0, 0, 500, 500); 
       result = view->Init(viewMan, boundBox, parView, nsnull);
       viewMan->InsertChild(parView, view, 0);
       SetView(view);
 
-      const nsStyleColor* color = (const nsStyleColor*) mStyleContext->GetStyleData(eStyleStruct_Color);
       // set the opacity
-      viewMan->SetViewOpacity(view, color->mOpacity);
+      viewMan->SetViewOpacity(view, GetStyleColor()->mOpacity);
 
     }
     mDidInit = PR_TRUE;
   }
 #endif
 
-  // reflow the child
+  // Reflow the child
   nsIFrame* firstKid = mFrames.FirstChild();
   nsSize availSize(aReflowState.mComputedWidth, NS_INTRINSICSIZE);
 
-  // indent the child inside us by the the focus border. We must do this separate from the
+  // Indent the child inside us by the the focus border. We must do this separate from the
   // regular border.
   nsMargin focusPadding = mRenderer.GetAddedButtonBorderAndPadding();
 
@@ -513,60 +504,22 @@ nsHTMLButtonControlFrame::Reflow(nsIPresContext* aPresContext,
     }
   }
 
-  nsHTMLReflowState reflowState(aPresContext, aReflowState, firstKid, availSize, reason);
-  //reflowState.computedWidth = availSize;
+  // Reflow the contents of the button a first time.
+  ReflowButtonContents(aPresContext, aDesiredSize, aReflowState, firstKid,
+                       availSize, reason, focusPadding, aStatus);
 
-  ReflowChild(firstKid, aPresContext, aDesiredSize, reflowState,
-              focusPadding.left + aReflowState.mComputedBorderPadding.left,
-              focusPadding.top + aReflowState.mComputedBorderPadding.top,
-              0, aStatus);
-  
-  // calculate the min internal size so the contents gets centered correctly
-  // minInternalWidth is not being used at all and causes a warning--commenting
-  // out until someone wants it.
-  //  nscoord minInternalWidth  = aReflowState.mComputedMinWidth  == 0?0:aReflowState.mComputedMinWidth - 
-  //    (aReflowState.mComputedBorderPadding.left + aReflowState.mComputedBorderPadding.right);
-  nscoord minInternalHeight = aReflowState.mComputedMinHeight == 0?0:aReflowState.mComputedMinHeight - 
-    (aReflowState.mComputedBorderPadding.top + aReflowState.mComputedBorderPadding.bottom);
+  // If we just performed the first pass of a shrink-wrap reflow (which will have
+  // found the requested size of the children, but not placed them), perform
+  // the second pass of a shrink-wrap reflow (which places the children
+  // within this parent button which has now shrink-wrapped around them).
+  if (availSize.width == NS_SHRINKWRAPWIDTH) {
+    nsSize newAvailSize(aDesiredSize.width, NS_INTRINSICSIZE);
 
-  // center child vertically
-  nscoord yoff = 0;
-  if (aReflowState.mComputedHeight != NS_INTRINSICSIZE) {
-    yoff = (aReflowState.mComputedHeight - aDesiredSize.height)/2;
-    if (yoff < 0) {
-      yoff = 0;
-    }
-  } else if (aDesiredSize.height < minInternalHeight) {
-    yoff = (minInternalHeight - aDesiredSize.height) / 2;
+    ReflowButtonContents(aPresContext, aDesiredSize, aReflowState, firstKid,
+                         newAvailSize, eReflowReason_Resize, focusPadding, aStatus);
   }
 
-  // Place the child.  If we have a non-intrinsic width, we want to
-  // reduce the left padding as needed to try and fit the text in the
-  // button
-  nscoord xoffset = focusPadding.left + aReflowState.mComputedBorderPadding.left;
-  if (aReflowState.mComputedWidth != NS_INTRINSICSIZE) {
-    // First, how much did we "overflow"?  This is the width of our
-    // kid plus our special focus stuff (which did not get accounted
-    // for in calculating aReflowState.mComputedWidth minus the width
-    // we're forced to be.
-    nscoord extrawidth =
-      aDesiredSize.width + focusPadding.left + focusPadding.right
-      - aReflowState.mComputedWidth;
-    if (extrawidth > 0) {
-      // Split it evenly between right and left
-      extrawidth /= 2;
-      // But do not shoot out the left side of the button, please
-      extrawidth = PR_MIN(extrawidth, aReflowState.mComputedPadding.left);
-      xoffset -= extrawidth;
-    }
-  }
-  
-  // Place the child
-  FinishReflowChild(firstKid, aPresContext, &reflowState, aDesiredSize,
-                    xoffset,
-                    yoff + focusPadding.top + aReflowState.mComputedBorderPadding.top, 0);
-
-  // if computed use the computed values.
+  // If computed use the computed values.
   if (aReflowState.mComputedWidth != NS_INTRINSICSIZE) 
     aDesiredSize.width = aReflowState.mComputedWidth;
   else 
@@ -612,6 +565,69 @@ nsHTMLButtonControlFrame::Reflow(nsIPresContext* aPresContext,
   return NS_OK;
 }
 
+void
+nsHTMLButtonControlFrame::ReflowButtonContents(nsIPresContext* aPresContext,
+                                               nsHTMLReflowMetrics& aDesiredSize,
+                                               const nsHTMLReflowState& aReflowState,
+                                               nsIFrame* aFirstKid,
+                                               const nsSize& aAvailSize,
+                                               nsReflowReason aReason,
+                                               nsMargin aFocusPadding,
+                                               nsReflowStatus& aStatus)
+{
+  nsHTMLReflowState reflowState(aPresContext, aReflowState, aFirstKid, aAvailSize, aReason);
+
+  ReflowChild(aFirstKid, aPresContext, aDesiredSize, reflowState,
+              aFocusPadding.left + aReflowState.mComputedBorderPadding.left,
+              aFocusPadding.top + aReflowState.mComputedBorderPadding.top,
+              0, aStatus);
+  
+  // calculate the min internal size so the contents gets centered correctly
+  // minInternalWidth is not being used at all and causes a warning--commenting
+  // out until someone wants it.
+  //  nscoord minInternalWidth  = aReflowState.mComputedMinWidth  == 0?0:aReflowState.mComputedMinWidth - 
+  //    (aReflowState.mComputedBorderPadding.left + aReflowState.mComputedBorderPadding.right);
+  nscoord minInternalHeight = aReflowState.mComputedMinHeight == 0?0:aReflowState.mComputedMinHeight - 
+    (aReflowState.mComputedBorderPadding.top + aReflowState.mComputedBorderPadding.bottom);
+
+  // center child vertically
+  nscoord yoff = 0;
+  if (aReflowState.mComputedHeight != NS_INTRINSICSIZE) {
+    yoff = (aReflowState.mComputedHeight - aDesiredSize.height)/2;
+    if (yoff < 0) {
+      yoff = 0;
+    }
+  } else if (aDesiredSize.height < minInternalHeight) {
+    yoff = (minInternalHeight - aDesiredSize.height) / 2;
+  }
+
+  // Place the child.  If we have a non-intrinsic width, we want to
+  // reduce the left padding as needed to try and fit the text in the
+  // button
+  nscoord xoffset = aFocusPadding.left + aReflowState.mComputedBorderPadding.left;
+  if (aReflowState.mComputedWidth != NS_INTRINSICSIZE) {
+    // First, how much did we "overflow"?  This is the width of our
+    // kid plus our special focus stuff (which did not get accounted
+    // for in calculating aReflowState.mComputedWidth minus the width
+    // we're forced to be.
+    nscoord extrawidth =
+      aDesiredSize.width + aFocusPadding.left + aFocusPadding.right
+      - aReflowState.mComputedWidth;
+    if (extrawidth > 0) {
+      // Split it evenly between right and left
+      extrawidth /= 2;
+      // But do not shoot out the left side of the button, please
+      extrawidth = PR_MIN(extrawidth, aReflowState.mComputedPadding.left);
+      xoffset -= extrawidth;
+    }
+  }
+  
+  // Place the child
+  FinishReflowChild(aFirstKid, aPresContext, &reflowState, aDesiredSize,
+                    xoffset,
+                    yoff + aFocusPadding.top + aReflowState.mComputedBorderPadding.top, 0);
+}
+
 NS_IMETHODIMP
 nsHTMLButtonControlFrame::IsPercentageBase(PRBool& aBase) const
 {
@@ -691,18 +707,17 @@ NS_IMETHODIMP nsHTMLButtonControlFrame::GetProperty(nsIAtom* aName, nsAString& a
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsHTMLButtonControlFrame::GetAdditionalStyleContext(PRInt32 aIndex, 
-                                                    nsIStyleContext** aStyleContext) const
+nsStyleContext*
+nsHTMLButtonControlFrame::GetAdditionalStyleContext(PRInt32 aIndex) const
 {
-  return mRenderer.GetStyleContext(aIndex, aStyleContext);
+  return mRenderer.GetStyleContext(aIndex);
 }
 
-NS_IMETHODIMP
+void
 nsHTMLButtonControlFrame::SetAdditionalStyleContext(PRInt32 aIndex, 
-                                                    nsIStyleContext* aStyleContext)
+                                                    nsStyleContext* aStyleContext)
 {
-  return mRenderer.SetStyleContext(aIndex, aStyleContext);
+  mRenderer.SetStyleContext(aIndex, aStyleContext);
 }
 
 
