@@ -1,25 +1,41 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
- */
+ * Contributor(s):
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 #include "nsCOMPtr.h"
+#include "nsIImageControlFrame.h"
 #include "nsImageFrame.h"
 #include "nsFormControlHelper.h"
 #include "nsIFormControlFrame.h"
@@ -28,10 +44,9 @@
 #include "nsIRenderingContext.h"
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsLeafFrame.h"
 #include "nsCSSRendering.h"
-#include "nsHTMLIIDs.h"
 #include "nsIViewManager.h"
 #include "nsISupports.h"
 #include "nsHTMLAtoms.h"
@@ -40,12 +55,15 @@
 #include "nsIDeviceContext.h"
 #include "nsIFontMetrics.h"
 #include "nsIImage.h"
-#include "nsStyleUtil.h"
 #include "nsStyleConsts.h"
-#include "nsFormFrame.h"
 #include "nsFormControlFrame.h"
 #include "nsGUIEvent.h"
+#include "nsLayoutAtoms.h"
+#include "nsIServiceManager.h"
+#include "nsContainerFrame.h"
+#ifdef ACCESSIBILITY
 #include "nsIAccessibilityService.h"
+#endif
 
 //Enumeration of possible mouse states used to detect mouse clicks
 /*enum nsMouseState {
@@ -60,7 +78,8 @@ static NS_DEFINE_IID(kViewCID, NS_VIEW_CID);
 
 #define nsImageControlFrameSuper nsImageFrame
 class nsImageControlFrame : public nsImageControlFrameSuper,
-                            public nsIFormControlFrame 
+                            public nsIFormControlFrame,
+                            public nsIImageControlFrame
 {
 public:
   nsImageControlFrame();
@@ -72,7 +91,7 @@ public:
   NS_IMETHOD Init(nsIPresContext*  aPresContext,
                   nsIContent*      aContent,
                   nsIFrame*        aParent,
-                  nsIStyleContext* aContext,
+                  nsStyleContext*  aContext,
                   nsIFrame*        aPrevInFlow);
 
   NS_IMETHOD Reflow(nsIPresContext*          aPresContext,
@@ -84,11 +103,15 @@ public:
                          nsGUIEvent* aEvent,
                          nsEventStatus* aEventStatus);
 
+  NS_IMETHOD GetFrameType(nsIAtom** aType) const;
+
+#ifdef ACCESSIBILITY
   NS_IMETHOD GetAccessible(nsIAccessible** aAccessible);
+#endif
 
 #ifdef DEBUG
-  NS_IMETHOD GetFrameName(nsString& aResult) const {
-    return MakeFrameName("ImageControl", aResult);
+  NS_IMETHOD GetFrameName(nsAString& aResult) const {
+    return MakeFrameName(NS_LITERAL_STRING("ImageControl"), aResult);
   }
 #endif
 
@@ -98,20 +121,9 @@ public:
 
   virtual void MouseClicked(nsIPresContext* aPresContext);
 
-  virtual void SetFormFrame(nsFormFrame* aFormFrame) { mFormFrame = aFormFrame; }
+  NS_IMETHOD_(PRInt32) GetType() const;
 
-  PRBool IsSuccessful(nsIFormControlFrame* aSubmitter);
-
-  virtual PRInt32 GetMaxNumValues();
-
-  virtual PRBool GetNamesValues(PRInt32 aMaxNumValues, PRInt32& aNumValues,
-                                nsString* aValues, nsString* aNames);
-
-  NS_IMETHOD GetType(PRInt32* aType) const;
-
-  NS_IMETHOD GetName(nsString* aName);
-
-  virtual void Reset(nsIPresContext*) {};
+  NS_IMETHOD GetName(nsAString* aName);
 
   void SetFocus(PRBool aOn, PRBool aRepaint);
   void ScrollIntoView(nsIPresContext* aPresContext);
@@ -131,16 +143,20 @@ public:
 
 
         // nsIFormControlFrame
-  NS_IMETHOD SetProperty(nsIPresContext* aPresContext, nsIAtom* aName, const nsAReadableString& aValue);
-  NS_IMETHOD GetProperty(nsIAtom* aName, nsAWritableString& aValue); 
+  NS_IMETHOD SetProperty(nsIPresContext* aPresContext, nsIAtom* aName, const nsAString& aValue);
+  NS_IMETHOD GetProperty(nsIAtom* aName, nsAString& aValue); 
   NS_IMETHOD SetSuggestedSize(nscoord aWidth, nscoord aHeight);
+  NS_IMETHOD OnContentReset();
+
+  // nsIImageControlFrame
+  NS_IMETHOD GetClickedX(PRInt32* aX);
+  NS_IMETHOD GetClickedY(PRInt32* aY);
 
 protected:
   void GetTranslatedRect(nsIPresContext* aPresContext, nsRect& aRect); // XXX this implementation is a copy of nsHTMLButtonControlFrame
   NS_IMETHOD_(nsrefcnt) AddRef(void);
   NS_IMETHOD_(nsrefcnt) Release(void);
 
-  nsFormFrame* mFormFrame;
   nsMouseState mLastMouseState;
   nsPoint mLastClickPoint; 
   nsCursor mPreviousCursor;
@@ -157,7 +173,6 @@ nsImageControlFrame::nsImageControlFrame()
   mPreviousCursor = eCursor_standard;
   mTranslatedRect = nsRect(0,0,0,0);
   mGotFocus       = PR_FALSE;
-  mFormFrame      = nsnull;
 }
 
 nsImageControlFrame::~nsImageControlFrame()
@@ -169,10 +184,6 @@ nsImageControlFrame::Destroy(nsIPresContext *aPresContext)
 {
   nsFormControlFrame::RegUnRegAccessKey(aPresContext, NS_STATIC_CAST(nsIFrame*, this), PR_FALSE);
 
-  if (mFormFrame) {
-    mFormFrame->RemoveFormControlFrame(*this);
-    mFormFrame = nsnull;
-  }
   return nsImageControlFrameSuper::Destroy(aPresContext);
 }
 
@@ -203,21 +214,26 @@ nsImageControlFrame::QueryInterface(const nsIID& aIID, void** aInstancePtr)
     *aInstancePtr = (void*) ((nsIFormControlFrame*) this);
     return NS_OK;
   } 
+  if (aIID.Equals(NS_GET_IID(nsIImageControlFrame))) {
+    *aInstancePtr = (void*) ((nsIImageControlFrame*) this);
+    return NS_OK;
+  }
 
   return nsImageControlFrameSuper::QueryInterface(aIID, aInstancePtr);
 }
 
+#ifdef ACCESSIBILITY
 NS_IMETHODIMP nsImageControlFrame::GetAccessible(nsIAccessible** aAccessible)
 {
   nsCOMPtr<nsIAccessibilityService> accService = do_GetService("@mozilla.org/accessibilityService;1");
 
   if (accService) {
-    nsIAccessible* acc = nsnull;
     return accService->CreateHTML4ButtonAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
   }
 
   return NS_ERROR_FAILURE;
 }
+#endif
 
 nsrefcnt nsImageControlFrame::AddRef(void)
 {
@@ -231,11 +247,20 @@ nsrefcnt nsImageControlFrame::Release(void)
   return 1;
 }
 
+NS_IMETHODIMP
+nsImageControlFrame::GetFrameType(nsIAtom** aType) const
+{
+  NS_PRECONDITION(nsnull != aType, "null OUT parameter pointer");
+  *aType = nsLayoutAtoms::imageControlFrame; 
+  NS_ADDREF(*aType);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsImageControlFrame::Init(nsIPresContext*  aPresContext,
                           nsIContent*      aContent,
                           nsIFrame*        aParent,
-                          nsIStyleContext* aContext,
+                          nsStyleContext*  aContext,
                           nsIFrame*        aPrevInFlow)
 {
   // call our base class
@@ -259,13 +284,15 @@ nsImageControlFrame::Init(nsIPresContext*  aPresContext,
     // the view's size is not know yet, but its size will be kept in synch with our frame.
     nsRect boundBox(0, 0, 0, 0); 
     result = view->Init(viewMan, boundBox, parView);
-    view->SetContentTransparency(PR_TRUE);
-    viewMan->InsertChild(parView, view, 0);
-    SetView(aPresContext, view);
 
-    const nsStyleVisibility* vis = (const nsStyleVisibility*) mStyleContext->GetStyleData(eStyleStruct_Visibility);
-    // set the opacity
-    viewMan->SetViewOpacity(view, vis->mOpacity);
+    nsContainerFrame::SyncFrameViewProperties(aPresContext, this, aContext, view);
+
+    // this gets reset during reflow anyway
+    // viewMan->SetViewContentTransparency(view, PR_TRUE);
+
+    // XXX put the view last in document order until we know how to do better
+    viewMan->InsertChild(parView, view, nsnull, PR_TRUE);
+    SetView(aPresContext, view);
   }
 
   return rv;
@@ -278,10 +305,9 @@ nsImageControlFrame::Reflow(nsIPresContext*         aPresContext,
                            nsReflowStatus&          aStatus)
 {
   DO_GLOBAL_REFLOW_COUNT("nsImageControlFrame", aReflowState.reason);
-  if (!mFormFrame && (eReflowReason_Initial == aReflowState.reason)) {
+  DISPLAY_REFLOW(aPresContext, this, aReflowState, aDesiredSize, aStatus);
+  if (aReflowState.reason == eReflowReason_Initial) {
     nsFormControlFrame::RegUnRegAccessKey(aPresContext, NS_STATIC_CAST(nsIFrame*, this), PR_TRUE);
-    // add ourself as an nsIFormControlFrame
-    nsFormFrame::AddFormControlFrame(aPresContext, *NS_STATIC_CAST(nsIFrame*, this));
   }
   return nsImageControlFrameSuper::Reflow(aPresContext, aDesiredSize, aReflowState, aStatus);
 }
@@ -293,13 +319,17 @@ nsImageControlFrame::HandleEvent(nsIPresContext* aPresContext,
 {
   NS_ENSURE_ARG_POINTER(aEventStatus);
 
+  // Don't do anything if the event has already been handled by someone
+  if (nsEventStatus_eConsumeNoDefault == *aEventStatus) {
+    return NS_OK;
+  }
+
   // do we have user-input style?
-  const nsStyleUserInterface* uiStyle;
-  GetStyleData(eStyleStruct_UserInterface,  (const nsStyleStruct *&)uiStyle);
+  const nsStyleUserInterface* uiStyle = GetStyleUserInterface();
   if (uiStyle->mUserInput == NS_STYLE_USER_INPUT_NONE || uiStyle->mUserInput == NS_STYLE_USER_INPUT_DISABLED)
     return nsFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
 
-  if (nsFormFrame::GetDisabled(this)) { // XXX cache disabled
+  if (nsFormControlHelper::GetDisabled(mContent)) { // XXX cache disabled
     return NS_OK;
   }
 
@@ -361,70 +391,16 @@ nsImageControlFrame::GetTranslatedRect(nsIPresContext* aPresContext, nsRect& aRe
   aRect = nsRect(viewOffset.x, viewOffset.y, mRect.width, mRect.height);
 }
 
-NS_IMETHODIMP
-nsImageControlFrame::GetType(PRInt32* aType) const
+NS_IMETHODIMP_(PRInt32)
+nsImageControlFrame::GetType() const
 {
-  *aType = NS_FORM_INPUT_IMAGE;
-  return NS_OK;
+  return NS_FORM_INPUT_IMAGE;
 }
 
 NS_IMETHODIMP
-nsImageControlFrame::GetName(nsString* aResult)
+nsImageControlFrame::GetName(nsAString* aResult)
 {
-  if (nsnull == aResult) {
-    return NS_OK;
-  } else {
-    return nsFormFrame::GetName(this, *aResult);
-  }
-}
-
-PRBool
-nsImageControlFrame::IsSuccessful(nsIFormControlFrame* aSubmitter)
-{
-  // Image control will only add it's value if it is clicked on.
-  // XXX Is this right?
-  return (this == (aSubmitter));
-}
-
-PRInt32
-nsImageControlFrame::GetMaxNumValues() 
-{
-  return 2;
-}
-
-
-PRBool
-nsImageControlFrame::GetNamesValues(PRInt32 aMaxNumValues, PRInt32& aNumValues,
-                                     nsString* aValues, nsString* aNames)
-{
-  if (aMaxNumValues <= 0) {
-    return PR_FALSE;
-  }
-
-  char buf[20];
-  aNumValues = 2;
-
-  sprintf(&buf[0], "%d", mLastClickPoint.x);
-  aValues[0].AssignWithConversion(&buf[0]);
-
-  sprintf(&buf[0], "%d", mLastClickPoint.y);
-  aValues[1].AssignWithConversion(&buf[0]);
-
-  nsAutoString name;
-  nsresult result = GetName(&name);
-  if (NS_CONTENT_ATTR_HAS_VALUE == result && (name.Length() > 0)) {
-    aNames[0] = name;
-    aNames[0].AppendWithConversion(".x");
-    aNames[1] = name;
-    aNames[1].AppendWithConversion(".y");
-  } else {
-    // If the Image Element has no name, simply return x and y
-    // to Nav and IE compatability.
-    aNames[0].AssignWithConversion("x");
-    aNames[1].AssignWithConversion("y");
-  }
-
-  return PR_TRUE;
+  return nsFormControlHelper::GetName(mContent, aResult);
 }
 
 NS_IMETHODIMP
@@ -434,13 +410,8 @@ nsImageControlFrame::GetCursor(nsIPresContext* aPresContext,
 {
   // Use style defined cursor if one is provided, otherwise when
   // the cursor style is "auto" we use the pointer cursor.
-  const nsStyleUserInterface* ui = (const nsStyleUserInterface*) mStyleContext->GetStyleData(eStyleStruct_UserInterface);
-  if (ui) {
-    aCursor = ui->mCursor;
-    if (NS_STYLE_CURSOR_AUTO == aCursor) {
-      aCursor = NS_STYLE_CURSOR_POINTER;
-    }
-  } else {
+  aCursor = GetStyleUserInterface()->mCursor;
+  if (NS_STYLE_CURSOR_AUTO == aCursor) {
     aCursor = NS_STYLE_CURSOR_POINTER;
   }
   return NS_OK;
@@ -449,11 +420,8 @@ nsImageControlFrame::GetCursor(nsIPresContext* aPresContext,
 void
 nsImageControlFrame::MouseClicked(nsIPresContext* aPresContext) 
 {
-  if (nsnull != mFormFrame && !nsFormFrame::GetDisabled(this)) {
-    // Do Submit & DOM Processing
-    nsFormControlHelper::DoManualSubmitOrReset(aPresContext, nsnull, 
-                                               mFormFrame, this, PR_TRUE, PR_TRUE); 
-  } 
+// This is no longer called; click events are handled in
+// nsHTMLInputElement::HandleDOMEvent().
 }
 
 NS_IMETHODIMP
@@ -500,13 +468,13 @@ nsresult nsImageControlFrame::RequiresWidget(PRBool& aRequiresWidget)
 
 NS_IMETHODIMP nsImageControlFrame::SetProperty(nsIPresContext* aPresContext,
                                                nsIAtom* aName,
-                                               const nsAReadableString& aValue)
+                                               const nsAString& aValue)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP nsImageControlFrame::GetProperty(nsIAtom* aName,
-                                               nsAWritableString& aValue)
+                                               nsAString& aValue)
 {
   aValue.Truncate();
   return NS_OK;
@@ -519,3 +487,22 @@ NS_IMETHODIMP nsImageControlFrame::SetSuggestedSize(nscoord aWidth, nscoord aHei
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsImageControlFrame::OnContentReset()
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsImageControlFrame::GetClickedX(PRInt32* aX)
+{
+  *aX = mLastClickPoint.x;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsImageControlFrame::GetClickedY(PRInt32* aY)
+{
+  *aY = mLastClickPoint.y;
+  return NS_OK;
+}
