@@ -48,9 +48,8 @@
 #include "nsIHTMLContent.h"
 #include "nsHTMLParts.h"
 #include "nsHTMLAtoms.h"
-#include "nsIStyleContext.h"
+#include "nsLayoutAtoms.h"
 #include "nsStyleConsts.h"
-#include "nsStyleUtil.h"
 #include "nsFont.h"
 #include "nsFormControlFrame.h"
 
@@ -81,6 +80,15 @@ nsLegendFrame::~nsLegendFrame()
 {
 }
 
+NS_IMETHODIMP
+nsLegendFrame::GetFrameType(nsIAtom** aType) const
+{
+  NS_PRECONDITION(nsnull != aType, "null OUT parameter pointer");
+  *aType = nsLayoutAtoms::legendFrame; 
+  NS_ADDREF(*aType);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsLegendFrame::Destroy(nsIPresContext *aPresContext)
 {
