@@ -60,7 +60,7 @@
 #include "nsIContent.h"
 
 class nsIDOMHTMLSelectElement;
-class nsIDOMHTMLCollection;
+class nsIDOMHTMLOptionsCollection;
 class nsIDOMHTMLOptionElement;
 class nsIComboboxControlFrame;
 class nsIViewManager;
@@ -201,7 +201,7 @@ public:
   NS_IMETHOD Init(nsIPresContext*  aPresContext,
                    nsIContent*      aContent,
                    nsIFrame*        aParent,
-                   nsIStyleContext* aContext,
+                   nsStyleContext*  aContext,
                    nsIFrame*        aPrevInFlow);
 
   NS_IMETHOD DidReflow(nsIPresContext*           aPresContext, 
@@ -223,6 +223,8 @@ public:
    */
   NS_IMETHOD GetFrameType(nsIAtom** aType) const;
 
+  virtual PRBool SupportsVisibilityHidden() { return PR_FALSE; }
+
 #ifdef DEBUG
     // nsIFrameDebug
   NS_IMETHOD GetFrameName(nsAString& aResult) const;
@@ -308,9 +310,9 @@ public:
 
   // Static Methods
   static nsIDOMHTMLSelectElement* GetSelect(nsIContent * aContent);
-  static nsIDOMHTMLCollection*    GetOptions(nsIContent * aContent, nsIDOMHTMLSelectElement* aSelect = nsnull);
-  static nsIDOMHTMLOptionElement* GetOption(nsIDOMHTMLCollection& aOptions, PRInt32 aIndex);
-  static nsIContent* GetOptionAsContent(nsIDOMHTMLCollection* aCollection,PRInt32 aIndex);
+  static nsIDOMHTMLOptionsCollection* GetOptions(nsIContent * aContent, nsIDOMHTMLSelectElement* aSelect = nsnull);
+  static nsIDOMHTMLOptionElement* GetOption(nsIDOMHTMLOptionsCollection* aOptions, PRInt32 aIndex);
+  static nsIContent* GetOptionAsContent(nsIDOMHTMLOptionsCollection* aCollection,PRInt32 aIndex);
 
   static void ComboboxFocusSet();
 
