@@ -38,7 +38,7 @@
 #include "nsTableOuterFrame.h"
 #include "nsTableFrame.h"
 #include "nsHTMLReflowCommand.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
 #include "nsIRenderingContext.h"
@@ -166,7 +166,7 @@ NS_IMETHODIMP
 nsTableOuterFrame::Init(nsIPresContext*  aPresContext,
                    nsIContent*           aContent,
                    nsIFrame*             aParent,
-                   nsIStyleContext*      aContext,
+                   nsStyleContext*       aContext,
                    nsIFrame*             aPrevInFlow)
 {
   nsresult rv = nsHTMLContainerFrame::Init(aPresContext, aContent, aParent, 
@@ -930,7 +930,7 @@ nsTableOuterFrame::BalanceLeftRightCaption(nsIPresContext* aPresContext,
   *     aCaptionWidth = capPercent/(1-capPercent) * 
   *                      (aCaptionMargin.left + aCaptionMargin.right + aInnerwidth + aInnerMargin.right)
   *     this result will cause problems for capPercent >= 1, in these cases the algorithm will now bail out
-  *     a similiar expression can be found for the right case
+  *     a similar expression can be found for the right case
   *  d) both frames have percent widths in this case the caption width will be the inner width multiplied 
   *     by the weight capPercent/innerPercent
   */
@@ -1231,8 +1231,7 @@ nsTableOuterFrame::IsAutoWidth(nsIFrame& aTableOrCaption,
     *aIsPctWidth = PR_FALSE;
   }
 
-  nsCOMPtr<nsIStyleContext> styleContext;
-  aTableOrCaption.GetStyleContext(getter_AddRefs(styleContext)); 
+  nsStyleContext* styleContext = aTableOrCaption.GetStyleContext();
 
   nsStylePosition* position = (nsStylePosition*)styleContext->GetStyleData(eStyleStruct_Position);
 
@@ -1374,30 +1373,56 @@ nsTableOuterFrame::UpdateReflowMetrics(nsIPresContext*      aPresContext,
   if (aMet.mFlags & NS_REFLOW_CALC_MAX_WIDTH) {
     aMet.mMaximumWidth = GetMaxWidth(aCaptionSide, aInnerMarginNoAuto, aCaptionMarginNoAuto);
   }
-  // see if the caption leaks out
-  if (NS_SIDE_LEFT == aCaptionSide) {
-    if (aCaptionMarginNoAuto.left < 0) {
-      aMet.mOverflowArea.x = aCaptionMarginNoAuto.left;
-      aMet.mOverflowArea.y = 0;
-      aMet.mOverflowArea.width = aMet.width - aCaptionMarginNoAuto.left;
-      aMet.mOverflowArea.height = aMet.height;
-      nsFrameState  frameState;
-      GetFrameState(&frameState);
-      frameState |= NS_FRAME_OUTSIDE_CHILDREN;
-      SetFrameState(frameState);
-    }
+  // see if the caption or the inner table frame leaks out
+  // the overflow area of the inner table and caption frame needs to be factored in once 
+  // one can retrieve them from the frames (see bug 197581)
+  switch (aCaptionSide) {
+    case NS_SIDE_LEFT:
+      aMet.mOverflowArea.x      = PR_MIN(0, aCaptionMarginNoAuto.left);
+      aMet.mOverflowArea.width  = aMet.width - aMet.mOverflowArea.x - 
+                                  PR_MIN(0, aInnerMarginNoAuto.right);
+      aMet.mOverflowArea.y      = PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.top), aInnerMarginNoAuto.top);
+      aMet.mOverflowArea.height = aMet.height - aMet.mOverflowArea.y -
+                                  PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.bottom), aInnerMarginNoAuto.bottom); 
+      break;
+    case NS_SIDE_RIGHT:
+      aMet.mOverflowArea.x      = PR_MIN(0, aInnerMarginNoAuto.left);
+      aMet.mOverflowArea.width  = aMet.width - aMet.mOverflowArea.x - 
+                                  PR_MIN(0, aCaptionMarginNoAuto.right);
+      aMet.mOverflowArea.y      = PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.top), aInnerMarginNoAuto.top);
+      aMet.mOverflowArea.height = aMet.height - aMet.mOverflowArea.y -
+                                  PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.bottom), aInnerMarginNoAuto.bottom);
+      break;
+    case NS_SIDE_BOTTOM:
+      aMet.mOverflowArea.x      = PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.left), aInnerMarginNoAuto.left);
+      aMet.mOverflowArea.width  = aMet.width - aMet.mOverflowArea.x -
+                                  PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.right), aInnerMarginNoAuto.right);
+      aMet.mOverflowArea.y      = PR_MIN(0, aInnerMarginNoAuto.top);
+      aMet.mOverflowArea.height = aMet.height - aMet.mOverflowArea.y - 
+                                  PR_MIN(0, aCaptionMarginNoAuto.bottom);
+      break;
+    case NS_SIDE_TOP:
+      aMet.mOverflowArea.x      = PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.left), aInnerMarginNoAuto.left);
+      aMet.mOverflowArea.width  = aMet.width - aMet.mOverflowArea.x -
+                                  PR_MIN(PR_MIN(0, aCaptionMarginNoAuto.right), aInnerMarginNoAuto.right);
+      aMet.mOverflowArea.y      = PR_MIN(0, aCaptionMarginNoAuto.top);
+      aMet.mOverflowArea.height = aMet.height - aMet.mOverflowArea.y - 
+                                  PR_MIN(0, aInnerMarginNoAuto.bottom);
+      break;
+    default: // no caption
+      aMet.mOverflowArea.x      = PR_MIN(0, aInnerMarginNoAuto.left);
+      aMet.mOverflowArea.width  = aMet.width - aMet.mOverflowArea.x - PR_MIN(0, aInnerMarginNoAuto.right);
+      aMet.mOverflowArea.y      = PR_MIN(0, aInnerMarginNoAuto.top);
+      aMet.mOverflowArea.height = aMet.height - aMet.mOverflowArea.y - PR_MIN(0, aInnerMarginNoAuto.bottom);
   }
-  else if (NS_SIDE_RIGHT == aCaptionSide) {
-    if (aCaptionMarginNoAuto.right <0) {
-      aMet.mOverflowArea.x = 0;
-      aMet.mOverflowArea.y = 0;
-      aMet.mOverflowArea.width = aMet.width - aCaptionMarginNoAuto.right;
-      aMet.mOverflowArea.height = aMet.height;
-      nsFrameState  frameState;
-      GetFrameState(&frameState);
-      frameState |= NS_FRAME_OUTSIDE_CHILDREN;
-      SetFrameState(frameState);
-    }
+  if ((aMet.mOverflowArea.x < 0) ||
+      (aMet.mOverflowArea.y < 0) ||
+      (aMet.mOverflowArea.XMost() > aMet.width) ||
+      (aMet.mOverflowArea.YMost() > aMet.height)) {
+    mState |= NS_FRAME_OUTSIDE_CHILDREN;
+  } 
+  else {
+    mState &= ~NS_FRAME_OUTSIDE_CHILDREN;
   } 
 }
 
@@ -1407,6 +1432,13 @@ nsTableOuterFrame::IncrementalReflow(nsIPresContext*          aPresContext,
                                      const nsHTMLReflowState& aReflowState,
                                      nsReflowStatus&          aStatus)
 {
+  // At this point, we need an inner table frame, and we might have a caption.  
+  // Due to the logic in nsCSSFrameConstructor::ConstructTableFrame, we can 
+  // end here without an inner table frame.
+  if (mFrames.IsEmpty() || !mInnerTableFrame) {
+    NS_ASSERTION(PR_FALSE, "incomplete children");
+    return NS_ERROR_FAILURE;
+  }
   // the outer table is a target if its path has a reflow command
   nsHTMLReflowCommand* command = aReflowState.path->mReflowCommand;
   if (command)
@@ -1877,8 +1909,7 @@ static PRBool
 IsPctHeight(nsIFrame* aFrame)
 {
   if (aFrame) {
-    nsCOMPtr<nsIStyleContext> styleContext;
-    aFrame->GetStyleContext(getter_AddRefs(styleContext)); 
+    nsStyleContext* styleContext = aFrame->GetStyleContext();
     nsStylePosition* position = (nsStylePosition*)styleContext->GetStyleData(eStyleStruct_Position);
     if (eStyleUnit_Percent == position->mHeight.GetUnit()) {
       float percent = position->mHeight.GetPercentValue();
@@ -1970,8 +2001,14 @@ NS_METHOD nsTableOuterFrame::Reflow(nsIPresContext*          aPresContext,
         mMinCaptionWidth = captionMet.mMaxElementWidth;
       }
     }
-    // At this point, we must have an inner table frame, and we might have a caption
-    NS_ASSERTION(mFrames.NotEmpty() && mInnerTableFrame, "incomplete children");
+    // At this point, we need an inner table frame, and we might have a 
+    // caption. Due to the logic in 
+    // nsCSSFrameConstructor::ConstructTableFrame, we can end here 
+    // without an inner table frame.
+    if (mFrames.IsEmpty() || !mInnerTableFrame) {
+      NS_ASSERTION(PR_FALSE, "incomplete children");
+      return NS_ERROR_FAILURE;
+    }
     nsSize   innerSize;
     nsMargin innerMargin, innerMarginNoAuto, innerPadding;
 
@@ -2228,15 +2265,4 @@ nsTableOuterFrame::GetFrameName(nsAString& aResult) const
 {
   return MakeFrameName(NS_LITERAL_STRING("TableOuter"), aResult);
 }
-
-NS_IMETHODIMP
-nsTableOuterFrame::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const
-{
-  if (!aResult) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  PRUint32 sum = sizeof(*this);
-  *aResult = sum;
-  return NS_OK;
-}
 #endif
