@@ -39,7 +39,7 @@
 #include "nsIRenderingContext.h"
 #include "nsIPresShell.h"
 #include "nsIPresContext.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIHTMLContent.h"
 #include "nsHTMLAtoms.h"
@@ -190,7 +190,7 @@ NS_IMETHODIMP
 nsTableRowFrame::Init(nsIPresContext*  aPresContext,
                       nsIContent*      aContent,
                       nsIFrame*        aParent,
-                      nsIStyleContext* aContext,
+                      nsStyleContext*  aContext,
                       nsIFrame*        aPrevInFlow)
 {
   nsresult  rv;
@@ -525,15 +525,14 @@ nscoord
 nsTableRowFrame::CalcHeight(const nsHTMLReflowState& aReflowState)
 {
   nsTableFrame* tableFrame = nsnull;
-  nsresult rv = nsTableFrame::GetTableFrame(this, tableFrame);
+  nsTableFrame::GetTableFrame(this, tableFrame);
   if (!tableFrame) return 0;
 
   nscoord computedHeight = (NS_UNCONSTRAINEDSIZE == aReflowState.mComputedHeight)
                             ? 0 : aReflowState.mComputedHeight;
   ResetHeight(computedHeight);
 
-  const nsStylePosition* position;
-  GetStyleData(eStyleStruct_Position, (const nsStyleStruct*&)position);
+  const nsStylePosition* position = GetStylePosition();
   if (eStyleUnit_Coord == position->mHeight.GetUnit()) {
     SetFixedHeight(position->mHeight.GetCoordValue());
   }
@@ -598,8 +597,7 @@ NS_METHOD nsTableRowFrame::Paint(nsIPresContext*      aPresContext,
 #endif
   // Standards mode background painting removed.  See bug 4510
 
-  const nsStyleDisplay* disp = (const nsStyleDisplay*)
-  mStyleContext->GetStyleData(eStyleStruct_Display);
+  const nsStyleDisplay* disp = GetStyleDisplay();
   if (disp && (NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow)) {
     aRenderingContext.PushState();
     SetOverflowClipRect(aRenderingContext);
@@ -727,8 +725,7 @@ nsTableRowFrame::CalculateCellActualSize(nsIFrame* aCellFrame,
   nscoord specifiedHeight = 0;
   
   // Get the height specified in the style information
-  const nsStylePosition* position;
-  aCellFrame->GetStyleData(eStyleStruct_Position, (const nsStyleStruct*&)position);
+  const nsStylePosition* position = aCellFrame->GetStylePosition();
 
   nsTableFrame* tableFrame = nsnull;
   nsTableFrame::GetTableFrame(this, tableFrame);
@@ -770,6 +767,7 @@ nsTableRowFrame::CalculateCellActualSize(nsIFrame* aCellFrame,
 static void 
 CalcAvailWidth(nsTableFrame&     aTableFrame,
                nscoord           aTableComputedWidth,
+               float             aPixelToTwips,
                nsTableCellFrame& aCellFrame,
                nscoord           aCellSpacingX,
                nscoord&          aColAvailWidth,
@@ -801,15 +799,15 @@ CalcAvailWidth(nsTableFrame&     aTableFrame,
   // if this is its initial reflow
   if ((frameState & NS_FRAME_FIRST_REFLOW) && (aTableFrame.GetEffectiveColSpan(aCellFrame) > 1)) {
     // see if the cell has a style width specified
-    const nsStylePosition* cellPosition;
-    aCellFrame.GetStyleData(eStyleStruct_Position, (const nsStyleStruct *&)cellPosition);
+    const nsStylePosition* cellPosition = aCellFrame.GetStylePosition();
     if (eStyleUnit_Coord == cellPosition->mWidth.GetUnit()) {
       // need to add padding into fixed width
-      nsMargin padding(0,0,0,0);
+      nsMargin borderPadding(0,0,0,0);
       if (NS_UNCONSTRAINEDSIZE != aTableComputedWidth) {
-        padding = nsTableFrame::GetPadding(nsSize(aTableComputedWidth, 0), &aCellFrame);
+        borderPadding = nsTableFrame::GetBorderPadding(nsSize(aTableComputedWidth, 0), 
+                                                       aPixelToTwips,  &aCellFrame);
       }
-      nscoord fixWidth = cellPosition->mWidth.GetCoordValue() + padding.left + padding.right;
+      nscoord fixWidth = cellPosition->mWidth.GetCoordValue() + borderPadding.left + borderPadding.right;
       aCellAvailWidth = PR_MIN(aColAvailWidth, fixWidth);
     }
   }
@@ -913,7 +911,6 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
   PRInt32 prevColIndex  = firstPrevColIndex;
   nscoord x = 0; // running total of children x offset
 
-  nsTableFrame* tableFirstInFlow = (nsTableFrame*)aTableFrame.GetFirstInFlow();
   PRBool isAutoLayout = aTableFrame.IsAutoLayout();
   PRBool needToNotifyTable = PR_TRUE;
   nscoord paginatedHeight = 0;
@@ -980,7 +977,7 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
         }
         // Calculate the available width for the table cell using the known column widths
         nscoord availColWidth, availCellWidth;
-        CalcAvailWidth(aTableFrame, GetComputedWidth(aReflowState, aTableFrame),
+        CalcAvailWidth(aTableFrame, GetComputedWidth(aReflowState, aTableFrame), p2t,
                        *cellFrame, cellSpacingX, availColWidth, availCellWidth);
         if (0 == availColWidth)  availColWidth  = NS_UNCONSTRAINEDSIZE;
         if (0 == availCellWidth) availCellWidth = NS_UNCONSTRAINEDSIZE;
@@ -1082,9 +1079,7 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
 
           // if we are in a floated table, our position is not yet established, so we cannot reposition our views
           // the containing glock will do this for us after positioning the table
-          const nsStyleDisplay *display;
-          aTableFrame.GetStyleData(eStyleStruct_Display, ((const nsStyleStruct *&)display));
-          if (display && !display->IsFloating()) {
+          if (!aTableFrame.GetStyleDisplay()->IsFloating()) {
             // Because we may have moved the frame we need to make sure any views are
             // positioned properly. We have to do this, because any one of our parent
             // frames could have moved and we have no way of knowing...
@@ -1261,9 +1256,9 @@ nsTableRowFrame::IR_TargetIsChild(nsIPresContext*          aPresContext,
   if (!aNextFrame) return NS_ERROR_NULL_POINTER;
   nsresult rv = NS_OK;
 
+  GET_PIXELS_TO_TWIPS(aPresContext, p2t);
   PRBool isAutoLayout = aTableFrame.IsAutoLayout();
-  const nsStyleDisplay *childDisplay;
-  aNextFrame->GetStyleData(eStyleStruct_Display, ((const nsStyleStruct *&)childDisplay));
+  const nsStyleDisplay* childDisplay = aNextFrame->GetStyleDisplay();
   if (NS_STYLE_DISPLAY_TABLE_CELL == childDisplay->mDisplay) {
     nsTableCellFrame* cellFrame = (nsTableCellFrame*)aNextFrame;
     // Get the x coord of the cell
@@ -1273,11 +1268,10 @@ nsTableRowFrame::IR_TargetIsChild(nsIPresContext*          aPresContext,
     // At this point, we know the column widths. Compute the cell available width
     PRInt32 cellColIndex;
     cellFrame->GetColIndex(cellColIndex);
-    PRInt32 cellColSpan = aTableFrame.GetEffectiveColSpan(*cellFrame);
     nscoord cellSpacingX = aTableFrame.GetCellSpacingX();
 
     nscoord colAvailWidth, cellAvailWidth;
-    CalcAvailWidth(aTableFrame, GetComputedWidth(aReflowState, aTableFrame),
+    CalcAvailWidth(aTableFrame, GetComputedWidth(aReflowState, aTableFrame), p2t,
                    *cellFrame, cellSpacingX, colAvailWidth, cellAvailWidth);
 
     // Always let the cell be as high as it wants. We ignore the height that's
@@ -1677,15 +1671,4 @@ nsTableRowFrame::GetFrameName(nsAString& aResult) const
 {
   return MakeFrameName(NS_LITERAL_STRING("TableRow"), aResult);
 }
-
-NS_IMETHODIMP
-nsTableRowFrame::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const
-{
-  if (!aResult) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  PRUint32 sum = sizeof(*this);
-  *aResult = sum;
-  return NS_OK;
-}
 #endif
