@@ -49,7 +49,7 @@
 #include "nsIPresContext.h"
 #include "nsInlineFrame.h"
 #include "nsIAtom.h"
-
+#include "nsAutoPtr.h"
 
 ///////////////////////////////////////////////////////////////////////////////
 //
@@ -291,7 +291,7 @@ TestReflowUnmapped(nsIPresContext* presContext)
   // Create an inline frame for the HTML container and set its
   // style context
   InlineFrame*      f = new InlineFrame(b, 0, nsnull);
-  nsIStyleContext*  styleContext = presContext->ResolveStyleContextFor(b, nsnull);
+  nsRefPtr<nsStyleContext> styleContext = presContext->ResolveStyleContextFor(b, nsnull);
 
   f->SetStyleContext(presContext,styleContext);
 
@@ -384,7 +384,7 @@ TestChildrenThatDontFit(nsIPresContext* presContext)
   // Create an inline frame for the HTML container and set its
   // style context
   InlineFrame*      f = new InlineFrame(b, 0, nsnull);
-  nsIStyleContext*  styleContext = presContext->ResolveStyleContextFor(b, nsnull);
+  nsRefPtr<nsStyleContext> styleContext = presContext->ResolveStyleContextFor(b, nsnull);
 
   f->SetStyleContext(presContext,styleContext);
 
@@ -556,7 +556,7 @@ TestOverflow(nsIPresContext* presContext)
   // Create an inline frame for the HTML container and set its
   // style context
   InlineFrame*      f = new InlineFrame(b, 0, nsnull);
-  nsIStyleContext*  styleContext = presContext->ResolveStyleContextFor(b, nsnull);
+  nsRefPtr<nsStyleContext> styleContext = presContext->ResolveStyleContextFor(b, nsnull);
 
   f->SetStyleContext(presContext,styleContext);
 
@@ -668,7 +668,7 @@ TestPushingPulling(nsIPresContext* presContext)
   // Create an inline frame for the HTML container and set its
   // style context
   InlineFrame*      f = new InlineFrame(b, 0, nsnull);
-  nsIStyleContext*  styleContext = presContext->ResolveStyleContextFor(b, nsnull);
+  nsRefPtr<nsStyleContext> styleContext = presContext->ResolveStyleContextFor(b, nsnull);
 
   f->SetStyleContext(presContext,styleContext);
 
@@ -1089,7 +1089,7 @@ TestSplittableChildren(nsIPresContext* presContext)
   // Create an inline frame for the HTML container and set its
   // style context
   InlineFrame*      f = new InlineFrame(b, 0, nsnull);
-  nsIStyleContext*  styleContext = presContext->ResolveStyleContextFor(b, nsnull);
+  nsRefPtr<nsStyleContext> styleContext = presContext->ResolveStyleContextFor(b, nsnull);
 
   f->SetStyleContext(presContext,styleContext);
 
@@ -1448,7 +1448,7 @@ TestMaxElementSize(nsIPresContext* presContext)
   // Create an inline frame for the HTML container and set its
   // style context
   InlineFrame*      f = new InlineFrame(b, 0, nsnull);
-  nsIStyleContext*  styleContext = presContext->ResolveStyleContextFor(b, nsnull);
+  nsRefPtr<nsStyleContext> styleContext = presContext->ResolveStyleContextFor(b, nsnull);
 
   f->SetStyleContext(presContext,styleContext);
 
