@@ -14,7 +14,19 @@ nodots() {
   echo $NODOTS_OUT
 }
 
-dirs="base bugs ../table/core ../table/viewer_tests ../table/bugs ../table/marvin ../table/other ../table/dom"
+dirs="	base			\
+	bugs			\
+	printing			\
+	../table/core		\
+	../table/viewer_tests	\
+	../table/bugs		\
+	../table/marvin		\
+	../table/other		\
+	../table/dom		\
+	../table/printing	\
+        ../formctls/base        \
+        ../formctls/bugs        \
+	../xbl"
 
 # This doesn't appear to work on Linux right now; needs support for a
 # null driver, perhaps?
@@ -39,7 +51,15 @@ else
   MOZ_OBJ=$MOZ_TEST_BASE/mozilla/
 fi
 
-MOZ_TEST_VIEWER="${MOZ_OBJ}dist/bin/mozilla-viewer.sh -- -d 500"
+w1=`uname | grep WIN`
+if [ "$w1" = "" ]; then
+ MOZ_TEST_VIEWER="${MOZ_OBJ}dist/bin/mozilla-viewer.sh -- -d 500"
+else
+ MOZ_TEST_VIEWER="${MOZ_OBJ}dist/bin/viewer.exe"
+ if test ! -f $MOZ_TEST_VIEWER; then
+  MOZ_TEST_VIEWER="${MOZ_OBJ}dist/win32_d.obj/bin/viewer.exe"
+ fi
+fi
 # These are needed by runtests.sh
 MOZ_TEST_VIEWER=`nodots $MOZ_TEST_VIEWER`
 MOZ_TEST_BASE=`nodots $MOZ_TEST_BASE`
