@@ -63,6 +63,11 @@ public:
 
   ~nsMathMLChar() { // not a virtual destructor: this class is not intended to be subclassed
     MOZ_COUNT_DTOR(nsMathMLChar);
+    // there is only one style context owned by the "root" char
+    // and it may be used by child chars as well
+    if (!mParent && mStyleContext) { // only the "root" need to release it
+      mStyleContext->Release();
+    }
     if (mSibling) {
       delete mSibling;
     }
@@ -73,7 +78,8 @@ public:
         nsIRenderingContext& aRenderingContext,
         const nsRect&        aDirtyRect,
         nsFramePaintLayer    aWhichLayer,
-        nsIFrame*            aForFrame);
+        nsIFrame*            aForFrame,
+        const nsRect*        aSelectedRect = nsnull);
 
   // This is the method called to ask the char to stretch itself.
   // @param aContainerSize - IN - suggested size for the stretched char
@@ -141,15 +147,18 @@ public:
     aBoundingMetrics = mBoundingMetrics;
   }
 
+  void
+  SetBoundingMetrics(nsBoundingMetrics& aBoundingMetrics) {
+    mBoundingMetrics = aBoundingMetrics;
+  }
+
   // Hooks to access the extra leaf style contexts given to the MathMLChars.
   // They provide an interface to make them acessible to the Style System via
   // the Get/Set AdditionalStyleContext() APIs. Owners of MathMLChars
   // should honor these APIs.
-  nsresult
-  GetStyleContext(nsIStyleContext** aStyleContext) const;
+  nsStyleContext* GetStyleContext() const;
 
-  nsresult
-  SetStyleContext(nsIStyleContext* aStyleContext);
+  void SetStyleContext(nsStyleContext* aStyleContext);
 
 protected:
   friend class nsGlyphTable;
@@ -161,9 +170,10 @@ protected:
 
 private:
   nsRect             mRect;
+  PRInt32            mOperator;
   nsStretchDirection mDirection;
   nsBoundingMetrics  mBoundingMetrics;
-  nsIStyleContext*   mStyleContext;
+  nsStyleContext*    mStyleContext;
   nsGlyphTable*      mGlyphTable;
   nsGlyphCode        mGlyph;
 
@@ -180,8 +190,7 @@ private:
   PaintVertically(nsIPresContext*      aPresContext,
                   nsIRenderingContext& aRenderingContext,
                   nsFont&              aFont,
-                  nscoord              aFontAscent,
-                  nsIStyleContext*     aStyleContext,
+                  nsStyleContext*      aStyleContext,
                   nsGlyphTable*        aGlyphTable,
                   nsMathMLChar*        aChar,
                   nsRect&              aRect);
@@ -190,8 +199,7 @@ private:
   PaintHorizontally(nsIPresContext*      aPresContext,
                     nsIRenderingContext& aRenderingContext,
                     nsFont&              aFont,
-                    nscoord              aFontAscent,
-                    nsIStyleContext*     aStyleContext,
+                    nsStyleContext*      aStyleContext,
                     nsGlyphTable*        aGlyphTable,
                     nsMathMLChar*        aChar,
                     nsRect&              aRect);
