@@ -23,19 +23,13 @@
 
 
 #include "nsCOMPtr.h"
-#include "nsHTMLParts.h"
-#include "nsIHTMLContent.h"
 #include "nsFrame.h"
-#include "nsLineLayout.h"
 #include "nsIPresContext.h"
-#include "nsHTMLAtoms.h"
 #include "nsUnitConversion.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsStyleConsts.h"
-#include "nsINameSpaceManager.h"
 #include "nsIRenderingContext.h"
 #include "nsIFontMetrics.h"
-#include "nsStyleUtil.h"
 
 #include "nsMathMLmfencedFrame.h"
 #include "nsMathMLmfracFrame.h"
@@ -105,7 +99,7 @@ NS_IMETHODIMP
 nsMathMLmfracFrame::Init(nsIPresContext*  aPresContext,
                          nsIContent*      aContent,
                          nsIFrame*        aParent,
-                         nsIStyleContext* aContext,
+                         nsStyleContext*  aContext,
                          nsIFrame*        aPrevInFlow)
 {
   nsresult rv = nsMathMLContainerFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
@@ -158,7 +152,7 @@ nsMathMLmfracFrame::TransmitAutomaticData(nsIPresContext* aPresContext)
 
 nscoord 
 nsMathMLmfracFrame::CalcLineThickness(nsIPresContext*  aPresContext,
-                                      nsIStyleContext* aStyleContext,
+                                      nsStyleContext*  aStyleContext,
                                       nsString&        aThicknessAttribute,
                                       nscoord          onePixel,
                                       nscoord          aDefaultRuleThickness)
@@ -554,36 +548,31 @@ nsMathMLmfracFrame::UpdatePresentationDataFromChildAt(nsIPresContext* aPresConte
 
 // ----------------------
 // the Style System will use these to pass the proper style context to our MathMLChar
-NS_IMETHODIMP
-nsMathMLmfracFrame::GetAdditionalStyleContext(PRInt32           aIndex, 
-                                              nsIStyleContext** aStyleContext) const
+nsStyleContext*
+nsMathMLmfracFrame::GetAdditionalStyleContext(PRInt32 aIndex) const
 {
-  NS_PRECONDITION(aStyleContext, "null OUT ptr");
-  if (!mSlashChar || aIndex < 0) {
-    return NS_ERROR_INVALID_ARG;
+  if (!mSlashChar) {
+    return nsnull;
   }
-  *aStyleContext = nsnull;
   switch (aIndex) {
   case NS_SLASH_CHAR_STYLE_CONTEXT_INDEX:
-    mSlashChar->GetStyleContext(aStyleContext);
+    return mSlashChar->GetStyleContext();
     break;
   default:
-    return NS_ERROR_INVALID_ARG;
+    return nsnull;
   }
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsMathMLmfracFrame::SetAdditionalStyleContext(PRInt32          aIndex, 
-                                              nsIStyleContext* aStyleContext)
+                                              nsStyleContext*  aStyleContext)
 {
-  if (!mSlashChar || aIndex < 0) {
-    return NS_ERROR_INVALID_ARG;
+  if (!mSlashChar) {
+    return;
   }
   switch (aIndex) {
   case NS_SLASH_CHAR_STYLE_CONTEXT_INDEX:
     mSlashChar->SetStyleContext(aStyleContext);
     break;
   }
-  return NS_OK;
 }
