@@ -23,19 +23,13 @@
  */
 
 #include "nsCOMPtr.h"
-#include "nsHTMLParts.h"
-#include "nsIHTMLContent.h"
 #include "nsFrame.h"
-#include "nsLineLayout.h"
 #include "nsIPresContext.h"
-#include "nsHTMLAtoms.h"
 #include "nsUnitConversion.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsStyleConsts.h"
-#include "nsINameSpaceManager.h"
 #include "nsIRenderingContext.h"
 #include "nsIFontMetrics.h"
-#include "nsStyleUtil.h"
 
 #include "nsIDOMText.h"
 
@@ -71,6 +65,38 @@ nsMathMLmoFrame::~nsMathMLmoFrame()
 {
 }
 
+static const PRUnichar kInvisibleComma = PRUnichar(0x200B); // a.k.a. ZERO WIDTH SPACE
+static const PRUnichar kApplyFunction  = PRUnichar(0x2061);
+static const PRUnichar kInvisibleTimes = PRUnichar(0x2062);
+static const PRUnichar kNullCh         = PRUnichar('\0');
+
+NS_IMETHODIMP
+nsMathMLmoFrame::GetFrameType(nsIAtom** aType) const
+{
+  if (mFrames.GetLength() > 1) {
+    *aType = nsMathMLAtoms::operatorVisibleMathMLFrame;
+    NS_ADDREF(*aType);
+    return NS_OK;
+  }
+
+  nsAutoString data;
+  ((nsMathMLChar&)mMathMLChar).GetData(data); // VC++ insists for this cast :-(
+  PRInt32 length = data.Length();
+  PRUnichar ch = (length == 0) ? kNullCh : data[0];
+  if (length > 1)
+    *aType = nsMathMLAtoms::operatorVisibleMathMLFrame;
+  else if (ch == kInvisibleComma || 
+           ch == kApplyFunction  ||
+           ch == kInvisibleTimes ||
+           ch == kNullCh)
+    *aType = nsMathMLAtoms::operatorInvisibleMathMLFrame;
+  else
+    *aType = nsMathMLAtoms::operatorVisibleMathMLFrame;
+
+  NS_ADDREF(*aType);
+  return NS_OK;
+}
+
 // since a mouse click implies selection, we cannot just rely on the
 // frame's state bit in our child text frame. So we will first check
 // its selected state bit, and use this little helper to double check.
@@ -130,8 +156,8 @@ nsMathMLmoFrame::Paint(nsIPresContext*      aPresContext,
 
   if (!useMathMLChar || NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer) {
     // let the base class paint the background, border, outline
-    rv = nsMathMLContainerFrame::Paint(aPresContext, aRenderingContext,
-                                       aDirtyRect, aWhichLayer);
+    rv = nsMathMLTokenFrame::Paint(aPresContext, aRenderingContext,
+                                   aDirtyRect, aWhichLayer);
   }
   if (useMathMLChar) {
     // make our char selected if our inner child text frame is selected
@@ -230,6 +256,8 @@ nsMathMLmoFrame::ProcessTextData(nsIPresContext* aPresContext)
   if (1 == length) {
     PRUnichar ch = data[0];
     if ((ch == '+') || (ch == '=') || (ch == '*') ||
+        (ch == 0x2264) || // &le;
+        (ch == 0x2265) || // &ge;
         (ch == 0x00D7)) { // &times;
       mFlags |= NS_MATHML_OPERATOR_CENTERED;
     }
@@ -400,10 +428,8 @@ nsMathMLmoFrame::ProcessOperatorData(nsIPresContext* aPresContext)
       // cache the default values of lspace & rspace that we get from the dictionary.
       // since these values are relative to the 'em' unit, convert to twips now
       nscoord em;
-      const nsStyleFont* font;
-      GetStyleData(eStyleStruct_Font, (const nsStyleStruct *&)font);
       nsCOMPtr<nsIFontMetrics> fm;
-      aPresContext->GetMetricsFor(font->mFont, getter_AddRefs(fm));
+      aPresContext->GetMetricsFor(GetStyleFont()->mFont, getter_AddRefs(fm));
       GetEmHeight(fm, em);
 
       mEmbellishData.leftSpace = NSToCoordRound(lspace * em);
@@ -601,15 +627,21 @@ nsMathMLmoFrame::Stretch(nsIPresContext*      aPresContext,
   }
   mPresentationData.flags |= NS_MATHML_STRETCH_DONE;
 
+  nsIFrame* firstChild = mFrames.FirstChild();
+
   // get the axis height;
-  const nsStyleFont *font = NS_STATIC_CAST(const nsStyleFont*,
-    mStyleContext->GetStyleData(eStyleStruct_Font));
   nsCOMPtr<nsIFontMetrics> fm;
-  aRenderingContext.SetFont(font->mFont, nsnull);
+  aRenderingContext.SetFont(GetStyleFont()->mFont, nsnull);
   aRenderingContext.GetFontMetrics(*getter_AddRefs(fm));
-  nscoord leading = 0, axisHeight, height;
+  nscoord axisHeight, height;
   GetAxisHeight(aRenderingContext, fm, axisHeight);
 
+  // get the leading to be left at the top and the bottom of the stretched char
+  // this seems more reliable than using fm->GetLeading() on suspicious fonts               
+  nscoord em;
+  GetEmHeight(fm, em);
+  nscoord leading = NSToCoordRound(0.2f * em);
+
   // Operators that are stretchy, or those that are to be centered
   // to cater for fonts that are not math-aware, are handled by the MathMLChar
   // ('form' is reset if stretch fails -- i.e., we don't bother to stretch next time)
@@ -779,12 +811,6 @@ nsMathMLmoFrame::Stretch(nsIPresContext*      aPresContext,
           }
           mBoundingMetrics.ascent = height - mBoundingMetrics.descent;
         }
-
-        // get the leading to be left at the top and the bottom of the stretched char
-        // this seems more reliable than using fm->GetLeading() on suspicious fonts               
-        nscoord em;
-        GetEmHeight(fm, em);
-        leading = NSToCoordRound(0.2f * em); // so, leading remains 0 if we don't get here
       }
     }
   }
@@ -817,11 +843,17 @@ nsMathMLmoFrame::Stretch(nsIPresContext*      aPresContext,
         NS_MATHML_EMBELLISH_IS_ACCENTUNDER(parentData.flags)) &&
        parentData.coreFrame != this;
   }
-  if (isAccent) {
+  if (isAccent && firstChild) {
+    // see bug 188467 for what is going on here
+    nscoord dy = aDesiredStretchSize.ascent - (mBoundingMetrics.ascent + leading);
     aDesiredStretchSize.ascent = mBoundingMetrics.ascent + leading;
-    aDesiredStretchSize.descent = mBoundingMetrics.descent + leading;
+    aDesiredStretchSize.descent = mBoundingMetrics.descent;
+
+    nsPoint origin;
+    firstChild->GetOrigin(origin);
+    firstChild->MoveTo(aPresContext, origin.x, origin.y - dy);
   }
-  else {
+  else if (useMathMLChar) {
     nscoord ascent, descent;
     fm->GetMaxAscent(ascent);
     fm->GetMaxDescent(descent);
@@ -873,7 +905,7 @@ nsMathMLmoFrame::Stretch(nsIPresContext*      aPresContext,
         mMathMLChar.SetRect(nsRect(rect.x + leftSpace, rect.y, rect.width, rect.height));
       }
       else {
-        nsIFrame* childFrame = mFrames.FirstChild();
+        nsIFrame* childFrame = firstChild;
         while (childFrame) {
           childFrame->GetRect(rect);
           childFrame->MoveTo(aPresContext, rect.x + leftSpace, rect.y);
@@ -887,7 +919,6 @@ nsMathMLmoFrame::Stretch(nsIPresContext*      aPresContext,
     return NS_OK;
 
   nsRect rect;
-  nsIFrame* firstChild = mFrames.FirstChild();
   firstChild->GetRect(rect);
   if (useMathMLChar) {
     // even though our child text frame is not doing the rendering, we make it play
@@ -901,29 +932,13 @@ nsMathMLmoFrame::Stretch(nsIPresContext*      aPresContext,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsMathMLmoFrame::SetInitialChildList(nsIPresContext* aPresContext,
-                                     nsIAtom*        aListName,
-                                     nsIFrame*       aChildList)
-{
-  // First, let the base class do its work
-  nsresult rv = nsMathMLContainerFrame::SetInitialChildList(aPresContext, aListName, aChildList);
-  if (NS_FAILED(rv)) return rv;
-
-  // No need to tract the style context given to our MathML char. 
-  // The Style System will use Get/SetAdditionalStyleContext() to keep it
-  // up-to-date if dynamic changes arise.
-  ProcessTextData(aPresContext);
-  return rv;
-}
-
 NS_IMETHODIMP
 nsMathMLmoFrame::InheritAutomaticData(nsIPresContext* aPresContext,
                                       nsIFrame*       aParent)
 {
   // retain our native direction, it only changes if our text content changes
   nsStretchDirection direction = mEmbellishData.direction;
-  nsMathMLContainerFrame::InheritAutomaticData(aPresContext, aParent);
+  nsMathMLTokenFrame::InheritAutomaticData(aPresContext, aParent);
   mEmbellishData.direction = direction;
   return NS_OK;
 }
@@ -948,18 +963,8 @@ nsMathMLmoFrame::Reflow(nsIPresContext*          aPresContext,
   // certain values use units that depend on our style context, so
   // it is safer to just process the whole lot here
   ProcessOperatorData(aPresContext);
-  return ReflowTokenFor(this, aPresContext, aDesiredSize,
-                        aReflowState, aStatus);
-}
-
-NS_IMETHODIMP
-nsMathMLmoFrame::Place(nsIPresContext*      aPresContext,
-                       nsIRenderingContext& aRenderingContext,
-                       PRBool               aPlaceOrigin,
-                       nsHTMLReflowMetrics& aDesiredSize)
-{
-  return PlaceTokenFor(this, aPresContext, aRenderingContext,
-                       aPlaceOrigin, aDesiredSize);
+  return nsMathMLTokenFrame::Reflow(aPresContext, aDesiredSize,
+                                    aReflowState, aStatus);
 }
 
 NS_IMETHODIMP
@@ -1013,43 +1018,32 @@ nsMathMLmoFrame::AttributeChanged(nsIPresContext* aPresContext,
     return ReLayoutChildren(aPresContext, target);
   }
 
-  return nsMathMLContainerFrame::
+  return nsMathMLTokenFrame::
          AttributeChanged(aPresContext, aContent, aNameSpaceID,
                           aAttribute, aModType, aHint);
 }
 
 // ----------------------
+// No need to tract the style context given to our MathML char. 
 // the Style System will use these to pass the proper style context to our MathMLChar
-NS_IMETHODIMP
-nsMathMLmoFrame::GetAdditionalStyleContext(PRInt32           aIndex,
-                                           nsIStyleContext** aStyleContext) const
+nsStyleContext*
+nsMathMLmoFrame::GetAdditionalStyleContext(PRInt32 aIndex) const
 {
-  NS_PRECONDITION(aStyleContext, "null OUT ptr");
-  if (aIndex < 0) {
-    return NS_ERROR_INVALID_ARG;
-  }
-  *aStyleContext = nsnull;
   switch (aIndex) {
   case NS_MATHML_CHAR_STYLE_CONTEXT_INDEX:
-    mMathMLChar.GetStyleContext(aStyleContext);
-    break;
+    return mMathMLChar.GetStyleContext();
   default:
-    return NS_ERROR_INVALID_ARG;
+    return nsnull;
   }
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsMathMLmoFrame::SetAdditionalStyleContext(PRInt32          aIndex,
-                                           nsIStyleContext* aStyleContext)
+                                           nsStyleContext*  aStyleContext)
 {
-  if (aIndex < 0) {
-    return NS_ERROR_INVALID_ARG;
-  }
   switch (aIndex) {
   case NS_MATHML_CHAR_STYLE_CONTEXT_INDEX:
     mMathMLChar.SetStyleContext(aStyleContext);
     break;
   }
-  return NS_OK;
 }
