@@ -10,7 +10,9 @@ close(UA);
 
 if (!($css =~ m|\@import.*mathml\.css|)) 
 {
-  $css =~ s|(\@import[^\@]+\;)\n\n|$1\n\@import url\(resource:/res/mathml\.css\);\n\n|;
+  # since bad end of lines cause troubles on some platforms
+  # do a little perl magic for dos2{unix or mac} here
+  $css =~ s#(\@import[^\@]+\;)(\cM?)(\n\cM?\n)#$1$2\n\@import url\(resource:/res/mathml\.css\);$2$3#;
   open(UA, ">$ua");
   print UA $css;
   close(UA);
