@@ -40,7 +40,6 @@
 #include "nsBoxLayoutState.h"
 #include "nsBox.h"
 #include "nsBoxFrame.h"
-#include "nsIStyleContext.h"
 #include "nsIPresContext.h"
 #include "nsCOMPtr.h"
 #include "nsIContent.h"
@@ -668,9 +667,7 @@ nsBox::GetBorder(nsMargin& aMargin)
 
   aMargin.SizeTo(0,0,0,0);
     
-  const nsStyleDisplay* disp;
-  frame->GetStyleData(eStyleStruct_Display,
-                      (const nsStyleStruct*&) disp);
+  const nsStyleDisplay* disp = frame->GetStyleDisplay();
   if (disp->mAppearance && gTheme) {
     // Go to the theme for the border.
     nsCOMPtr<nsIContent> content;
@@ -701,16 +698,9 @@ nsBox::GetBorder(nsMargin& aMargin)
     }
   }
 
-  const nsStyleBorder* border;
-  nsresult rv = frame->GetStyleData(eStyleStruct_Border,
-        (const nsStyleStruct*&) border);
+  frame->GetStyleBorder()->GetBorder(aMargin);
 
-  if (NS_FAILED(rv))
-    return rv;
-
-  border->GetBorder(aMargin);
-
-  return rv;
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -719,17 +709,10 @@ nsBox::GetPadding(nsMargin& aMargin)
   nsIFrame* frame = nsnull;
   GetFrame(&frame);
 
-  const nsStylePadding* padding;
-  nsresult rv = frame->GetStyleData(eStyleStruct_Padding,
-        (const nsStyleStruct*&) padding);
-
- if (NS_FAILED(rv))
-    return rv;
-
   aMargin.SizeTo(0,0,0,0);
-  padding->GetPadding(aMargin);
+  frame->GetStylePadding()->GetPadding(aMargin);
 
-  return rv;
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -738,17 +721,10 @@ nsBox::GetMargin(nsMargin& aMargin)
   nsIFrame* frame = nsnull;
   GetFrame(&frame);
 
-  const nsStyleMargin* margin;
-        nsresult rv = frame->GetStyleData(eStyleStruct_Margin,
-        (const nsStyleStruct*&) margin);
-  
-  if (NS_FAILED(rv))
-     return rv;
-
   aMargin.SizeTo(0,0,0,0);
-  margin->GetMargin(aMargin);
+  frame->GetStyleMargin()->GetMargin(aMargin);
 
-  return rv;
+  return NS_OK;
 }
 
 NS_IMETHODIMP 
@@ -1199,10 +1175,8 @@ nsBox::Redraw(nsBoxLayoutState& aState,
 
   // Checks to see if the damaged rect should be infalted 
   // to include the outline
-  const nsStyleOutline* outline;
-  frame->GetStyleData(eStyleStruct_Outline, (const nsStyleStruct*&)outline);
   nscoord width;
-  outline->GetOutlineWidth(width);
+  frame->GetStyleOutline()->GetOutlineWidth(width);
   if (width > 0) {
     damageRect.Inflate(width, width);
   }
@@ -1238,9 +1212,7 @@ nsIBox::AddCSSPrefSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
     aBox->GetFrame(&frame);
 
     // add in the css min, max, pref
-    const nsStylePosition* position;
-    frame->GetStyleData(eStyleStruct_Position,
-                  (const nsStyleStruct*&) position);
+    const nsStylePosition* position = frame->GetStylePosition();
 
     // see if the width or height was specifically set
     if (position->mWidth.GetUnit() == eStyleUnit_Coord)  {
@@ -1256,7 +1228,10 @@ nsIBox::AddCSSPrefSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
     nsCOMPtr<nsIContent> content;
     frame->GetContent(getter_AddRefs(content));
 
-    if (content) {
+    // ignore 'height' and 'width' attributes if the actual element is not XUL
+    // For example, we might be magic XUL frames whose primary content is an HTML
+    // <select>
+    if (content && content->IsContentOfType(nsIContent::eXUL)) {
         nsIPresContext* presContext = aState.GetPresContext();
 
         nsAutoString value;
@@ -1301,9 +1276,7 @@ nsIBox::AddCSSMinSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
     aBox->GetFrame(&frame);
 
     // See if a native theme wants to supply a minimum size.
-    const nsStyleDisplay* display;
-    frame->GetStyleData(eStyleStruct_Display,
-                  (const nsStyleStruct*&) display);
+    const nsStyleDisplay* display = frame->GetStyleDisplay();
     if (display->mAppearance) {
       nsCOMPtr<nsITheme> theme;
       aState.GetPresContext()->GetTheme(getter_AddRefs(theme));
@@ -1315,20 +1288,20 @@ nsIBox::AddCSSMinSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
                                       display->mAppearance, &size, &canOverride);
           float p2t;
           aState.GetPresContext()->GetScaledPixelsToTwips(&p2t);
-          aSize.width = NSIntPixelsToTwips(size.width, p2t);
-          aSize.height = NSIntPixelsToTwips(size.height, p2t);
-          if (aSize.width)
+          if (size.width) {
+            aSize.width = NSIntPixelsToTwips(size.width, p2t);
             widthSet = PR_TRUE;
-          if (aSize.height)
+          }
+          if (size.height) {
+            aSize.height = NSIntPixelsToTwips(size.height, p2t);
             heightSet = PR_TRUE;
+          }
         }
       }
     }
 
     // add in the css min, max, pref
-    const nsStylePosition* position;
-    frame->GetStyleData(eStyleStruct_Position,
-                  (const nsStyleStruct*&) position);
+    const nsStylePosition* position = frame->GetStylePosition();
 
     // same for min size. Unfortunately min size is always set to 0. So for now
     // we will assume 0 means not set.
@@ -1399,9 +1372,7 @@ nsIBox::AddCSSMaxSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
     aBox->GetFrame(&frame);
 
     // add in the css min, max, pref
-    const nsStylePosition* position;
-    frame->GetStyleData(eStyleStruct_Position,
-                  (const nsStyleStruct*&) position);
+    const nsStylePosition* position = frame->GetStylePosition();
 
     // and max
     if (position->mMaxWidth.GetUnit() == eStyleUnit_Coord) {
@@ -1478,8 +1449,7 @@ nsIBox::AddCSSFlex(nsBoxLayoutState& aState, nsIBox* aBox, nscoord& aFlex)
         }
         else {
           // No attribute value.  Check CSS.
-          const nsStyleXUL* boxInfo;
-          frame->GetStyleData(eStyleStruct_XUL, (const nsStyleStruct*&)boxInfo);
+          const nsStyleXUL* boxInfo = frame->GetStyleXUL();
           if (boxInfo->mBoxFlex > 0.0f) {
             // The flex was defined in CSS.
             aFlex = (nscoord)boxInfo->mBoxFlex;
@@ -1496,9 +1466,8 @@ nsIBox::AddCSSCollapsed(nsBoxLayoutState& aState, nsIBox* aBox, PRBool& aCollaps
 {
   nsIFrame* frame = nsnull;
   aBox->GetFrame(&frame);
-  const nsStyleVisibility* vis;
-  frame->GetStyleData(eStyleStruct_Visibility, ((const nsStyleStruct *&)vis));
-  aCollapsed = vis->mVisible == NS_STYLE_VISIBILITY_COLLAPSE;
+  aCollapsed = frame->GetStyleVisibility()->mVisible ==
+               NS_STYLE_VISIBILITY_COLLAPSE;
   return PR_TRUE;
 }
 
@@ -1524,8 +1493,7 @@ nsIBox::AddCSSOrdinal(nsBoxLayoutState& aState, nsIBox* aBox, PRUint32& aOrdinal
     }
     else {
       // No attribute value.  Check CSS.
-      const nsStyleXUL* boxInfo;
-      frame->GetStyleData(eStyleStruct_XUL, (const nsStyleStruct*&)boxInfo);
+      const nsStyleXUL* boxInfo = frame->GetStyleXUL();
       if (boxInfo->mBoxOrdinal > 1) {
         // The ordinal group was defined in CSS.
         aOrdinal = (nscoord)boxInfo->mBoxOrdinal;
