@@ -46,6 +46,8 @@ class nsBox : public nsIBox {
 
 public:
 
+  static void Shutdown();
+
   NS_DECL_ISUPPORTS_INHERITED
 
   NS_IMETHOD GetChildBox(nsIBox** aBox);
@@ -99,7 +101,9 @@ public:
 
   NS_IMETHOD MarkChildrenStyleChange();
   NS_IMETHOD MarkStyleChange(nsBoxLayoutState& aState);
+#ifdef DEBUG_LAYOUT
   NS_IMETHOD DumpBox(FILE* out);
+#endif
   NS_IMETHOD ChildrenMustHaveWidgets(PRBool& aMust);
   NS_IMETHOD GetIndexOf(nsIBox* aChild, PRInt32* aIndex);
 
@@ -130,10 +134,12 @@ public:
 
 protected:
 
+#ifdef DEBUG_LAYOUT
   virtual void AppendAttribute(const nsAutoString& aAttribute, const nsAutoString& aValue, nsAutoString& aResult);
 
   virtual void ListBox(nsAutoString& aResult);
-
+#endif
+  
   virtual PRBool HasStyleChange();
   virtual void SetStyleChangeFlag(PRBool aDirty);
 
@@ -146,9 +152,11 @@ protected:
   NS_IMETHOD DoLayout(nsBoxLayoutState& aBoxLayoutState);
   NS_IMETHOD EndLayout(nsBoxLayoutState& aState);
 
+#ifdef DEBUG_LAYOUT
   virtual void GetBoxName(nsAutoString& aName);
+#endif
 
-  static PRUint32 gRefCnt;
+  static PRBool gGotTheme;
   static nsITheme* gTheme;
 
   enum eMouseThrough {
@@ -166,7 +174,7 @@ private:
   //nscoord mY;
 };
 
-#ifdef DEBUG
+#ifdef DEBUG_LAYOUT
 #define NS_BOX_ASSERTION(box,expr,str) \
   if (!(expr)) { \
        box->DumpBox(stdout); \
