@@ -74,7 +74,7 @@ public:
   NS_IMETHOD  Init(nsIPresContext*  aPresContext,
                nsIContent*      aContent,
                nsIFrame*        aParent,
-               nsIStyleContext* aContext,
+               nsStyleContext*  aContext,
                nsIFrame*        asPrevInFlow);
 
   NS_IMETHOD GetFrameForPoint(nsIPresContext* aPresContext,
