@@ -49,7 +49,7 @@
 #include "nsIAtom.h"
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsCSSRendering.h"
 #include "nsINameSpaceManager.h"
 #include "nsLayoutAtoms.h"
@@ -60,7 +60,6 @@
 #include "nsIDocument.h"
 #include "nsIDOMNSDocument.h"
 #include "nsIDOMDocument.h"
-#include "nsIDOMXULDocument.h"
 #include "nsIDOMElement.h"
 #include "nsISupportsArray.h"
 #include "nsIDOMText.h"
@@ -88,7 +87,7 @@
 #include "nsITimerInternal.h"
 #define NS_MENU_POPUP_LIST_INDEX   0
 
-#ifdef XP_PC
+#if defined(XP_WIN) || defined(XP_OS2)
 #define NSCONTEXTMENUISMOUSEUP 1
 #endif
 
@@ -183,7 +182,7 @@ NS_IMETHODIMP
 nsMenuFrame::Init(nsIPresContext*  aPresContext,
                      nsIContent*      aContent,
                      nsIFrame*        aParent,
-                     nsIStyleContext* aContext,
+                     nsStyleContext*  aContext,
                      nsIFrame*        aPrevInFlow)
 {
   mPresContext = aPresContext; // Don't addref it.  Our lifetime is shorter.
@@ -546,8 +545,12 @@ nsMenuFrame::ToggleMenuState()
       // a menubar item to toggle its submenu closed.
       OpenMenu(PR_FALSE);
       SelectMenu(PR_TRUE);
+      mMenuParent->SetActive(PR_FALSE);
     }
     else {
+      if (mMenuParent) {
+        mMenuParent->SetActive(PR_TRUE);
+      }
       OpenMenu(PR_TRUE);
     }
   }
@@ -555,8 +558,6 @@ nsMenuFrame::ToggleMenuState()
   if (mMenuParent) {
     // Make sure the current menu which is being toggled on
     // the menubar is highlighted
-    mMenuParent->SetActive(PR_FALSE);
-    mMenuParent->SetActive(PR_TRUE);
     mMenuParent->SetCurrentMenuItem(this);
     // We've successfully prevented the same click from both
     // dismissing and reopening this menu. 
@@ -1521,13 +1522,13 @@ nsMenuFrame::BuildAcceleratorText()
   nsCOMPtr<nsIDocument> document;
   mContent->GetDocument(*getter_AddRefs(document));
 
-  // Turn the document into a XUL document so we can use getElementById
-  nsCOMPtr<nsIDOMXULDocument> xulDocument(do_QueryInterface(document));
-  if (!xulDocument)
+  // Turn the document into a DOM document so we can use getElementById
+  nsCOMPtr<nsIDOMDocument> domDocument(do_QueryInterface(document));
+  if (!domDocument)
     return;
 
   nsCOMPtr<nsIDOMElement> keyDOMElement;
-  xulDocument->GetElementById(keyValue, getter_AddRefs(keyDOMElement));
+  domDocument->GetElementById(keyValue, getter_AddRefs(keyDOMElement));
   if (!keyDOMElement)
     return;
 
@@ -1711,25 +1712,7 @@ nsMenuFrame::Execute(nsGUIEvent *aEvent)
   nsIFrame* me = this;
   if (NS_SUCCEEDED(result) && shell) {
     shell->GetViewManager(getter_AddRefs(kungFuDeathGrip));
-
-    // See if we have a command elt.  If so, we execute on the command instead
-    // of on our content element.
-    nsAutoString command;
-    mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::command, command);
-    if (!command.IsEmpty()) {
-      nsCOMPtr<nsIDocument> doc;
-      mContent->GetDocument(*getter_AddRefs(doc));
-      nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(doc));
-      nsCOMPtr<nsIDOMElement> commandElt;
-      domDoc->GetElementById(command, getter_AddRefs(commandElt));
-      nsCOMPtr<nsIContent> commandContent(do_QueryInterface(commandElt));
-      if (commandContent)
-        shell->HandleDOMEventWithTarget(commandContent, &event, &status);
-      else
-        NS_ASSERTION(PR_FALSE, "A XUL <menuitem> is attached to a command that doesn't exist! Unable to execute menu item!\n");
-    }
-    else
-      shell->HandleDOMEventWithTarget(mContent, &event, &status);
+    shell->HandleDOMEventWithTarget(mContent, &event, &status);
   }
 
   // XXX HACK. Just gracefully exit if the node has been removed, e.g., window.close()
@@ -2152,7 +2135,7 @@ nsMenuFrame::SetActiveChild(nsIDOMElement* aChild)
 }
 
 NS_IMETHODIMP
-nsMenuFrame::GetScrollableView(nsIScrollableView** aView)
+nsMenuFrame::GetScrollableView(nsIPresContext* aPresContext, nsIScrollableView** aView)
 {
   *aView = nsnull;
   if (!mPopupFrames.FirstChild())
