@@ -1,24 +1,39 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is Mozilla Communicator client code.
  *
- * The Initial Developer of the Original Code is Netscape Communications
- * Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
- */
+ * Contributor(s):
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 //
 // Eric Vaughan
@@ -28,16 +43,13 @@
 //
 
 #include "nsStackFrame.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsIPresContext.h"
 #include "nsIContent.h"
 #include "nsCOMPtr.h"
-#include "nsHTMLIIDs.h"
 #include "nsUnitConversion.h"
-#include "nsINameSpaceManager.h"
 #include "nsXULAtoms.h"
 #include "nsHTMLAtoms.h"
-#include "nsIReflowCommand.h"
 #include "nsHTMLParts.h"
 #include "nsIPresShell.h"
 #include "nsStyleChangeList.h"
@@ -95,7 +107,9 @@ nsStackFrame::GetFrameForPoint(nsIPresContext* aPresContext,
   GetInset(im);
   nsMargin border(0,0,0,0);
   nsStyleBorderPadding  bPad;
-  aFrame->GetStyle(eStyleStruct_BorderPaddingShortcut, (nsStyleStruct&)bPad);
+  nsStyleContext* styleContext;
+  aFrame->GetStyleContext(&styleContext);
+  styleContext->GetBorderPaddingFor(bPad);
   bPad.GetBorderPadding(borderPadding);
   r.Deflate(im);
   r.Deflate(border);    
@@ -121,7 +135,7 @@ nsStackFrame::GetFrameForPoint(nsIPresContext* aPresContext,
   } else
       rv = NS_ERROR_FAILURE;
 
-  if (!NS_SUCCEEDED(rv)) {
+  if (NS_FAILED(rv)) {
         const nsStyleColor* color =
     (const nsStyleColor*)mStyleContext->GetStyleData(eStyleStruct_Color);
 
@@ -178,9 +192,10 @@ nsStackFrame::GetStackedFrameForPoint(nsIPresContext* aPresContext,
 
 void
 nsStackFrame::PaintChildren(nsIPresContext*      aPresContext,
-                                nsIRenderingContext& aRenderingContext,
-                                const nsRect&        aDirtyRect,
-                                nsFramePaintLayer    aWhichLayer)
+                            nsIRenderingContext& aRenderingContext,
+                            const nsRect&        aDirtyRect,
+                            nsFramePaintLayer    aWhichLayer,
+                            PRUint32             aFlags)
 {
   // we need to make sure we paint background then foreground of each child because they
   // are stacked. Otherwise the foreground of the first child could be on the top of the
@@ -194,10 +209,11 @@ nsStackFrame::PaintChildren(nsIPresContext*      aPresContext,
 // Paint one child frame
 void
 nsStackFrame::PaintChild(nsIPresContext*      aPresContext,
-                             nsIRenderingContext& aRenderingContext,
-                             const nsRect&        aDirtyRect,
-                             nsIFrame*            aFrame,
-                             nsFramePaintLayer    aWhichLayer)
+                         nsIRenderingContext& aRenderingContext,
+                         const nsRect&        aDirtyRect,
+                         nsIFrame*            aFrame,
+                         nsFramePaintLayer    aWhichLayer,
+                         PRUint32             aFlags)
 {
   // we need to make sure we paint background then foreground of each child because they
   // are stacked. Otherwise the foreground of the first child could be on the top of the
