@@ -39,6 +39,7 @@
 
 #include "nsIDOMMouseEvent.h"
 #include "nsIDOMEventTarget.h"
+#include "nsIDOMDocument.h"
 #include "nsIDOMXULDocument.h"
 #include "nsIDOMXULElement.h"
 #include "nsIDocument.h"
@@ -117,8 +118,10 @@ nsXULTooltipListener::MouseOut(nsIDOMEvent* aMouseEvent)
     return NS_OK;
   }
 
+#ifdef DEBUG_crap
   if (mNeedTitletip)
     return NS_OK;
+#endif
 
   // check to see if the mouse left the targetNode, and if so,
   // hide the tooltip
@@ -343,7 +346,6 @@ nsXULTooltipListener::CheckTreeBodyMove(nsIDOMMouseEvent* aMouseEvent)
     // determine if we are going to need a titletip
     // XXX check the disabletitletips attribute on the tree content
     mNeedTitletip = PR_FALSE;
-#ifdef DEBUG_crap
     if (row >= 0 && obj.Equals(NS_LITERAL_STRING("text"))) {
       nsCOMPtr<nsITreeView> view;
       obx->GetView(getter_AddRefs(view));
@@ -351,7 +353,6 @@ nsXULTooltipListener::CheckTreeBodyMove(nsIDOMMouseEvent* aMouseEvent)
       obx->IsCellCropped(row, colId, &isCropped);
       mNeedTitletip = isCropped;
     }
-#endif
 
     if (mCurrentTooltip && 
         (row != mLastTreeRow || !mLastTreeCol.Equals(colId))) {
@@ -453,7 +454,7 @@ SetTitletipLabel(nsITreeBoxObject* aTreeBox, nsIContent* aTooltip,
   nsAutoString label;
   view->GetCellText(aRow, aCol.get(), label);
   
-  aTooltip->SetAttr(nsnull, nsXULAtoms::label, label, PR_FALSE);
+  aTooltip->SetAttr(nsnull, nsXULAtoms::label, label, PR_TRUE);
 }
 
 nsresult
@@ -477,13 +478,15 @@ nsXULTooltipListener::LaunchTooltip(nsIContent* aTarget, PRInt32 aX, PRInt32 aY)
     if (mNeedTitletip) {
       nsCOMPtr<nsITreeBoxObject> obx;
       GetSourceTreeBoxObject(getter_AddRefs(obx));
+#ifdef DEBUG_crap
       GetTreeCellCoords(obx, mSourceNode,
                             mLastTreeRow, mLastTreeCol, &x, &y);
+#endif
 
       SetTitletipLabel(obx, mCurrentTooltip, mLastTreeRow, mLastTreeCol);
-      mCurrentTooltip->SetAttr(nsnull, nsXULAtoms::titletip, NS_LITERAL_STRING("true"), PR_FALSE);
+      mCurrentTooltip->SetAttr(nsnull, nsXULAtoms::titletip, NS_LITERAL_STRING("true"), PR_TRUE);
     } else
-      mCurrentTooltip->UnsetAttr(nsnull, nsXULAtoms::titletip, PR_FALSE);
+      mCurrentTooltip->UnsetAttr(nsnull, nsXULAtoms::titletip, PR_TRUE);
 
     nsCOMPtr<nsIDOMElement> targetEl(do_QueryInterface(aTarget));
     popupBoxObject->ShowPopup(targetEl, xulTooltipEl, x, y,
@@ -577,14 +580,15 @@ nsXULTooltipListener::GetTooltipFor(nsIContent* aTarget, nsIContent** aTooltip)
           } else {
             if (!tooltipId.IsEmpty()) {
               // tooltip must be an id, use getElementById to find it
-              nsCOMPtr<nsIDOMXULDocument> xulDocument = do_QueryInterface(document);
-              if (!xulDocument) {
-                NS_ERROR("tooltip attached to an element that isn't in XUL!");
+              nsCOMPtr<nsIDOMDocument> domDocument =
+                do_QueryInterface(document);
+              if (!domDocument) {
                 return NS_ERROR_FAILURE;
               }
 
               nsCOMPtr<nsIDOMElement> tooltipEl;
-              xulDocument->GetElementById(tooltipId, getter_AddRefs(tooltipEl));
+              domDocument->GetElementById(tooltipId,
+                                          getter_AddRefs(tooltipEl));
 
               if (tooltipEl) {
                 mNeedTitletip = PR_FALSE;
