@@ -62,8 +62,6 @@ public:
   NS_IMETHOD Init(nsIContent* aContent, nsIPresShell* aPresShell);
   NS_IMETHOD SetDocument(nsIDocument* aDocument);
   NS_IMETHOD InvalidatePresentationStuff();
-
-  nsITreeBoxObject* mTreeBody;
 };
 
 /* Implementation file */
@@ -84,8 +82,6 @@ nsTreeBoxObject::SetDocument(nsIDocument* aDocument)
   if (treeView)
     treeView->SetTree(nsnull); // Break the circular ref between the view and us.
 
-  mTreeBody = nsnull;
-
   return nsBoxObject::SetDocument(aDocument);
 }
 
@@ -100,9 +96,7 @@ nsTreeBoxObject::InvalidatePresentationStuff()
 }
   
 nsTreeBoxObject::nsTreeBoxObject()
-:mTreeBody(nsnull)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsTreeBoxObject::~nsTreeBoxObject()
@@ -386,6 +380,14 @@ NS_IMETHODIMP nsTreeBoxObject::InvalidateScrollbar()
   return NS_OK;
 }
 
+NS_IMETHODIMP nsTreeBoxObject::GetRowAt(PRInt32 x, PRInt32 y, PRInt32 *_retval)
+{
+  nsITreeBoxObject* body = GetTreeBody();
+  if (body)
+    return body->GetRowAt(x, y, _retval);
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsTreeBoxObject::GetCellAt(PRInt32 x, PRInt32 y, PRInt32 *row, PRUnichar **colID,
                                              PRUnichar** childElt)
 {
@@ -422,6 +424,22 @@ NS_IMETHODIMP nsTreeBoxObject::RowCountChanged(PRInt32 aIndex, PRInt32 aDelta)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsTreeBoxObject::BeginUpdateBatch()
+{
+  nsITreeBoxObject* body = GetTreeBody();
+  if (body)
+    return body->BeginUpdateBatch();
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsTreeBoxObject::EndUpdateBatch()
+{
+  nsITreeBoxObject* body = GetTreeBody();
+  if (body)
+    return body->EndUpdateBatch();
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsTreeBoxObject::OnDragEnter(nsIDOMEvent* inEvent)
 {
   nsITreeBoxObject* body = GetTreeBody();
