@@ -219,7 +219,7 @@ NS_IMETHODIMP nsAbView::Init(const char *aURI, nsIAbViewListener *abViewListener
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr <nsIRDFResource> resource;
-  rv = rdfService->GetResource(aURI, getter_AddRefs(resource));
+  rv = rdfService->GetResource(nsDependentCString(aURI), getter_AddRefs(resource));
   NS_ENSURE_SUCCESS(rv, rv);
 
   mDirectory = do_QueryInterface(resource, &rv);
@@ -788,6 +788,14 @@ nsresult nsAbView::CreateCollationKey(const PRUnichar *aSource, PRUint8 **aKey,
   NS_ENSURE_ARG_POINTER(aKey);
   NS_ENSURE_ARG_POINTER(aKeyLen);
 
+  if (!*aSource)
+  {
+    // no string, so no key.
+    *aKey = nsnull;
+    *aKeyLen = 0;
+    return NS_OK;
+  }
+
   nsresult rv;
   if (!mCollationKeyGenerator)
   {
