@@ -129,7 +129,8 @@ NS_IMETHODIMP nsAddressBook::NewAddressBook(nsIAbDirectoryProperties *aPropertie
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIRDFResource> parentResource;
-    rv = rdfService->GetResource(kAllDirectoryRoot, getter_AddRefs(parentResource));
+    rv = rdfService->GetResource(NS_LITERAL_CSTRING(kAllDirectoryRoot),
+                                 getter_AddRefs(parentResource));
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIAbDirectory> parentDir = do_QueryInterface(parentResource, &rv);
@@ -139,6 +140,34 @@ NS_IMETHODIMP nsAddressBook::NewAddressBook(nsIAbDirectoryProperties *aPropertie
     return rv;
 }
 
+NS_IMETHODIMP nsAddressBook::ModifyAddressBook
+(nsIRDFDataSource* aDS, nsIAbDirectory *aParentDir, nsIAbDirectory *aDirectory, nsIAbDirectoryProperties *aProperties)
+{
+  NS_ENSURE_ARG_POINTER(aDS);
+  NS_ENSURE_ARG_POINTER(aParentDir);
+  NS_ENSURE_ARG_POINTER(aDirectory);
+  NS_ENSURE_ARG_POINTER(aProperties);
+
+  nsresult rv;
+  nsCOMPtr<nsISupportsArray> parentArray (do_CreateInstance(NS_SUPPORTSARRAY_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsISupportsArray> resourceElement (do_CreateInstance(NS_SUPPORTSARRAY_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsISupportsArray> resourceArray (do_CreateInstance(NS_SUPPORTSARRAY_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  parentArray->AppendElement(aParentDir);
+
+  nsCOMPtr<nsIRDFResource> dirSource(do_QueryInterface(aDirectory, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  resourceElement->AppendElement(dirSource);
+  resourceElement->AppendElement(aProperties);
+  resourceArray->AppendElement(resourceElement);
+
+  return DoCommand(aDS, NS_LITERAL_CSTRING(NC_RDF_MODIFY), parentArray, resourceArray);
+}
+
 NS_IMETHODIMP nsAddressBook::DeleteAddressBooks
 (nsIRDFDataSource* aDS, nsISupportsArray *aParentDir, nsISupportsArray *aResourceArray)
 {
@@ -146,11 +175,11 @@ NS_IMETHODIMP nsAddressBook::DeleteAddressBooks
   NS_ENSURE_ARG_POINTER(aParentDir);
   NS_ENSURE_ARG_POINTER(aResourceArray);
   
-  return DoCommand(aDS, NC_RDF_DELETE, aParentDir, aResourceArray);
+  return DoCommand(aDS, NS_LITERAL_CSTRING(NC_RDF_DELETE), aParentDir, aResourceArray);
 }
 
 nsresult nsAddressBook::DoCommand(nsIRDFDataSource* db,
-                                  const char *command,
+                                  const nsACString& command,
                                   nsISupportsArray *srcArray,
                                   nsISupportsArray *argumentArray)
 {
@@ -385,12 +414,11 @@ nsresult AddressBookParser::ParseFile()
     return ParseLDIFFile();
   }
  
+  // We are migrating 4.x profile
     /* Get database file name */
     char *leafName = nsnull;
-    nsAutoString fileString;
     if (mFileSpec) {
         mFileSpec->GetLeafName(&leafName);
-        fileString.AssignWithConversion(leafName);
 
         PRInt32 i = 0;
         while (leafName[i] != '\0')
@@ -434,27 +462,31 @@ nsresult AddressBookParser::ParseFile()
     nsCOMPtr<nsIRDFService> rdfService = do_GetService (NS_RDF_CONTRACTID "/rdf-service;1", &rv);
     NS_ENSURE_SUCCESS(rv, rv);
     nsCOMPtr<nsIRDFResource> parentResource;
-    char *parentUri = PR_smprintf("%s", kAllDirectoryRoot);
-    rv = rdfService->GetResource(parentUri, getter_AddRefs(parentResource));
+    rv = rdfService->GetResource(NS_LITERAL_CSTRING(kAllDirectoryRoot),
+                                 getter_AddRefs(parentResource));
     nsCOMPtr<nsIAbDirectory> parentDir = do_QueryInterface(parentResource);
     if (!parentDir)
         return NS_ERROR_NULL_POINTER;
-    if (parentUri)
-        PR_smprintf_free(parentUri);
 
-    if (PL_strcmp(fileName, kPersonalAddressbook) == 0)
-    {
-        // This is the personal address book, get name from prefs
-        nsCOMPtr<nsIPref> pPref(do_GetService(NS_PREF_CONTRACTID, &rv)); 
-        if (NS_FAILED(rv) || !pPref) 
-            return nsnull;
-        
-            nsXPIDLString dirName;
+    // Get Pretty name from prefs.
+    nsCOMPtr<nsIPref> pPref(do_GetService(NS_PREF_CONTRACTID, &rv)); 
+    if (NS_FAILED(rv) || !pPref) 
+        return nsnull;
+
+    nsXPIDLString dirName;
+    if (strcmp(fileName, kPersonalAddressbook) == 0)
         rv = pPref->GetLocalizedUnicharPref("ldap_2.servers.pab.description", getter_Copies(dirName));
-        parentDir->CreateDirectoryByURI(dirName, mDbUri, mMigrating);
-    }
     else
-        parentDir->CreateDirectoryByURI(fileString.get(), mDbUri, mMigrating);
+    {
+      nsCAutoString prefName;
+      prefName = NS_LITERAL_CSTRING("ldap_2.servers.") + nsDependentCString(leafName) + NS_LITERAL_CSTRING(".description");
+      rv = pPref->GetLocalizedUnicharPref(prefName.get(), getter_Copies(dirName));
+    }
+
+    // If a name is found then use it, otherwise use the filename as last resort.
+    if (NS_FAILED(rv) || dirName.IsEmpty())
+      dirName = NS_ConvertASCIItoUCS2(leafName);
+    parentDir->CreateDirectoryByURI(dirName, mDbUri, mMigrating);
         
     rv = ParseLDIFFile();
 
@@ -1316,7 +1348,7 @@ NS_IMETHODIMP nsAddressBook::ExportAddressBook(nsIAbDirectory *aDirectory)
     if (extension.Equals(CSV_FILE_EXTENSION))
       rv = ExportDirectoryToDelimitedText(aDirectory, CSV_DELIM, CSV_DELIM_LEN, localFile);
     else if (extension.Equals(TAB_FILE_EXTENSION) || extension.Equals(TXT_FILE_EXTENSION)) {
-      // .tab and .txt are both tab seperated
+      // .tab and .txt are both tab separated
       rv = ExportDirectoryToDelimitedText(aDirectory, TAB_DELIM, TAB_DELIM_LEN, localFile);
     }
     else {
@@ -1468,7 +1500,7 @@ nsAddressBook::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const
             }
           }
 
-          // write out the linebreak that seperates the cards
+          // write out the linebreak that separates the cards
           rv = outputStream->Write(MSG_LINEBREAK, MSG_LINEBREAK_LEN, &writeCount);
           NS_ENSURE_SUCCESS(rv,rv);
           if (MSG_LINEBREAK_LEN != writeCount)
@@ -1586,7 +1618,7 @@ nsAddressBook::ExportDirectoryToLDIF(nsIAbDirectory *aDirectory, nsILocalFile *a
             }
           }
         
-          // write out the linebreak that seperates the cards
+          // write out the linebreak that separates the cards
           rv = outputStream->Write(LDIF_LINEBREAK, LDIF_LINEBREAK_LEN, &writeCount);
           NS_ENSURE_SUCCESS(rv,rv);
           if (LDIF_LINEBREAK_LEN != writeCount)
@@ -1649,7 +1681,7 @@ nsresult nsAddressBook::AppendLDIFForMailList(nsIAbCard *aCard, nsACString &aRes
   NS_ENSURE_SUCCESS(rv,rv);
   
   nsCOMPtr <nsIRDFResource> resource;
-  rv = rdfService->GetResource(mailListURI.get(), getter_AddRefs(resource));
+  rv = rdfService->GetResource(mailListURI, getter_AddRefs(resource));
   NS_ENSURE_SUCCESS(rv,rv);
     
   nsCOMPtr <nsIAbDirectory> mailList = do_QueryInterface(resource, &rv);
