@@ -244,11 +244,14 @@ function onLess(event)
 }
 
 // set scope on all visible searchattribute tags
-function setSearchScope(scope) {
+function setSearchScope(scope) 
+{
     gSearchScope = scope;
     for (var i=0; i<gSearchTerms.length; i++) {
         gSearchTerms[i].obj.searchattribute.searchScope = scope;
         gSearchTerms[i].scope = scope;
+        // act like the user "selected" this, see bug #202848
+        gSearchTerms[i].obj.searchattribute.onSelect(null /* no event */);  
     }
 }
 
@@ -260,10 +263,12 @@ function updateSearchAttributes()
 }
 
 function booleanChanged(event) {
-    // when boolean changes, we have to update all the attributes on the
-    // search terms
-
+    // when boolean changes, we have to update all the attributes on the search terms
     var newBoolValue = (event.target.getAttribute("value") == "and") ? true : false;
+    if (document.getElementById("abPopup")) {
+      var selectedAB = document.getElementById("abPopup").selectedItem.id;
+      setSearchScope(GetScopeForDirectoryURI(selectedAB));
+    }
     for (var i=0; i<gSearchTerms.length; i++) {
         var searchTerm = gSearchTerms[i].obj;
         searchTerm.booleanAnd = newBoolValue;
@@ -277,7 +282,6 @@ function booleanChanged(event) {
     }
 }
 
-
 function createSearchRow(index, scope, searchTerm)
 {
     var searchAttr = document.createElement("searchattribute");
@@ -332,7 +336,10 @@ function createSearchRow(index, scope, searchTerm)
     var listcells = searchrow.childNodes;
     var j=0;
     for (var i=0; i<listcells.length; i+=2) {
-        stringNodes[j++] = listcells[i];
+      stringNodes[j++] = listcells[i];
+
+      // see bug #183994 for why these cells are hidden
+      listcells[i].hidden = true;
     }
     searchTermObj.booleanNodes = stringNodes;
 
