@@ -510,6 +510,7 @@ nsresult nsMsgFilter::ConvertMoveToFolderValue(nsIMsgRuleAction *filterAction, n
           nsCOMPtr <nsIMsgFolder> msgFolder;
           msgFolder = do_QueryInterface(destIFolder);	
           destIFolder->GetURI(getter_Copies(folderUri));
+          filterAction->SetTargetFolderUri(folderUri);
           moveValue.Assign(folderUri);
         }
       }
@@ -552,7 +553,7 @@ nsresult nsMsgFilter::ConvertMoveToFolderValue(nsIMsgRuleAction *filterAction, n
         // need to remove ".sbd" from moveValue, and perhaps escape it.
         moveValue.ReplaceSubstring(".sbd/", "/");
 
-#ifdef XP_MAC
+#if defined(XP_MAC) || defined(XP_MACOSX)
         char *unescapedMoveValue = ToNewCString(moveValue);
         nsUnescape(unescapedMoveValue);
         moveValue.Assign(unescapedMoveValue);
@@ -564,7 +565,7 @@ nsresult nsMsgFilter::ConvertMoveToFolderValue(nsIMsgRuleAction *filterAction, n
           nsAutoString unicodeStr;
           impSvc->SystemStringToUnicode(moveValue.get(), unicodeStr);
           nsXPIDLCString escapedName;
-          rv =NS_MsgEscapeEncodeURLPath(unicodeStr.get(), getter_Copies(escapedName));
+          rv = NS_MsgEscapeEncodeURLPath(unicodeStr.get(), getter_Copies(escapedName));
           if (NS_SUCCEEDED(rv) && escapedName)
             moveValue.Assign(escapedName.get());
         }
