@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Pierre Phaneuf <pp@ludusdesign.com>
+ *   Seth Spitzer <sspitzer@netscape.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or 
@@ -52,8 +53,8 @@
 #include "nsIImportService.h"
 #include "nsMsgBaseCID.h"
 #include "nsIMsgFilterService.h"
-
-static NS_DEFINE_CID(kMsgFilterServiceCID, NS_MSGFILTERSERVICE_CID);
+#include "nsISupportsObsolete.h"
+#include "nsNetUtil.h"
 
 // unicode "%s" format string
 static const PRUnichar unicodeFormatter[] = {
@@ -66,13 +67,14 @@ static const PRUnichar unicodeFormatter[] = {
 nsMsgFilterList::nsMsgFilterList() :
     m_fileVersion(0)
 {
-	// I don't know how we're going to report this error if we failed to create the isupports array...
-	nsresult rv;
-	rv = NS_NewISupportsArray(getter_AddRefs(m_filters));
-	m_loggingEnabled = PR_FALSE;
-	m_curFilter = nsnull;
+  // I don't know how we're going to report this error if we failed to create the isupports array...
+  nsresult rv = NS_NewISupportsArray(getter_AddRefs(m_filters));
+  NS_ASSERTION(NS_SUCCEEDED(rv), "Fixme bug 180312: NS_NewISupportsArray() failed");
+
+  m_loggingEnabled = PR_FALSE;
+  m_startWritingToBuffer = PR_FALSE; 
+  m_curFilter = nsnull;
   m_arbitraryHeaders.SetLength(0);
-	NS_INIT_REFCNT();
 }
 
 NS_IMPL_ADDREF(nsMsgFilterList)
@@ -81,25 +83,20 @@ NS_IMPL_QUERY_INTERFACE1(nsMsgFilterList, nsIMsgFilterList)
 
 NS_IMETHODIMP nsMsgFilterList::CreateFilter(const PRUnichar *name,class nsIMsgFilter **aFilter)
 {
-	NS_ENSURE_ARG_POINTER(aFilter);
+  NS_ENSURE_ARG_POINTER(aFilter);
 
-	nsMsgFilter *filter = new nsMsgFilter;
-    NS_ENSURE_TRUE(filter, NS_ERROR_OUT_OF_MEMORY);
+  nsMsgFilter *filter = new nsMsgFilter;
+  NS_ENSURE_TRUE(filter, NS_ERROR_OUT_OF_MEMORY);
     
-	*aFilter = filter;
-    NS_ADDREF(*aFilter);
+  NS_ADDREF(*aFilter = filter);
     
-    filter->SetFilterName(name);
-    filter->SetFilterList(this);
+  filter->SetFilterName(name);
+  filter->SetFilterList(this);
     
-	return NS_OK;
+  return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgFilterList::SetLoggingEnabled(PRBool enable)
-{
-	m_loggingEnabled = enable;
-	return NS_OK;
-}
+NS_IMPL_GETSET(nsMsgFilterList, LoggingEnabled, PRBool, m_loggingEnabled);
 
 NS_IMETHODIMP nsMsgFilterList::GetFolder(nsIMsgFolder **aFolder)
 {
@@ -115,14 +112,6 @@ NS_IMETHODIMP nsMsgFilterList::SetFolder(nsIMsgFolder *aFolder)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgFilterList::GetLoggingEnabled(PRBool *aResult)
-{
-	if (!aResult)
-		return NS_ERROR_NULL_POINTER;
-	*aResult = m_loggingEnabled;
-	return NS_OK;
-}
-
 NS_IMETHODIMP nsMsgFilterList::SaveToFile(nsIOFileStream *stream)
 {
 	if (!stream)
@@ -130,6 +119,176 @@ NS_IMETHODIMP nsMsgFilterList::SaveToFile(nsIOFileStream *stream)
 	return SaveTextFilters(stream);
 }
 
+NS_IMETHODIMP nsMsgFilterList::EnsureLogFile()
+{
+  nsCOMPtr <nsIFileSpec> file;
+  nsresult rv = GetLogFileSpec(getter_AddRefs(file));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  PRBool exists;
+  rv = file->Exists(&exists);
+  if (NS_SUCCEEDED(rv) && !exists) {
+    rv = file->Touch();
+    NS_ENSURE_SUCCESS(rv,rv);
+  }
+  return NS_OK;
+}
+
+nsresult nsMsgFilterList::TruncateLog()
+{
+  // this will flush and close the steam
+  nsresult rv = SetLogStream(nsnull);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  nsCOMPtr <nsIFileSpec> file;
+  rv = GetLogFileSpec(getter_AddRefs(file));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  rv = file->Truncate(0);
+  NS_ENSURE_SUCCESS(rv,rv);
+  return rv;
+}
+
+NS_IMETHODIMP nsMsgFilterList::ClearLog()
+{
+  PRBool loggingEnabled = m_loggingEnabled;
+  
+  // disable logging while clearing
+  m_loggingEnabled = PR_FALSE;
+
+  nsresult rv = TruncateLog();
+  NS_ASSERTION(NS_SUCCEEDED(rv), "failed to truncate filter log");
+
+  m_loggingEnabled = loggingEnabled;
+  return NS_OK;
+}
+
+nsresult 
+nsMsgFilterList::GetLogFileSpec(nsIFileSpec **aFileSpec)
+{
+  NS_ENSURE_ARG_POINTER(aFileSpec);
+
+  // XXX todo
+  // the path to the log file won't change
+  // should we cache it?
+  nsCOMPtr <nsIMsgFolder> folder;
+  nsresult rv = GetFolder(getter_AddRefs(folder));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  nsCOMPtr <nsIMsgIncomingServer> server;
+  rv = folder->GetServer(getter_AddRefs(server));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  nsXPIDLCString type;
+  rv = server->GetType(getter_Copies(type));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  // for news, the filter file is
+  // mcom.test.dat
+  // where the summary file is 
+  // mcom.test.msf
+  // since the log is an html file we make it
+  // mcom.test.htm
+  if (type.Equals("nntp")) {
+    nsCOMPtr<nsIFileSpec> thisFolder;
+    rv = m_folder->GetPath(getter_AddRefs(thisFolder));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsCOMPtr <nsIFileSpec> filterLogFile = do_CreateInstance(NS_FILESPEC_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+    
+    rv = filterLogFile->FromFileSpec(thisFolder);
+    NS_ENSURE_SUCCESS(rv, rv);
+    
+    // NOTE:
+    // we don't we need to call NS_MsgHashIfNecessary()
+    // it's already been hashed, if necessary
+    nsXPIDLCString filterLogName;
+    rv = filterLogFile->GetLeafName(getter_Copies(filterLogName));
+    NS_ENSURE_SUCCESS(rv,rv);
+    
+    filterLogName.Append(".htm");
+    
+    rv = filterLogFile->SetLeafName(filterLogName.get());
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    NS_IF_ADDREF(*aFileSpec = filterLogFile);
+  }
+  else {
+    rv = server->GetLocalPath(aFileSpec);
+    NS_ENSURE_SUCCESS(rv,rv);
+    
+    rv = (*aFileSpec)->AppendRelativeUnixPath("filterlog.html");
+    NS_ENSURE_SUCCESS(rv,rv);
+  }
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgFilterList::GetLogURL(char **aLogURL)
+{
+  NS_ENSURE_ARG_POINTER(aLogURL);
+
+  nsCOMPtr <nsIFileSpec> file;
+  nsresult rv = GetLogFileSpec(getter_AddRefs(file));
+  NS_ENSURE_SUCCESS(rv,rv);
+  
+  rv = file->GetURLString(aLogURL);
+  NS_ENSURE_SUCCESS(rv,rv);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgFilterList::SetLogStream(nsIOutputStream *aLogStream)
+{
+  // if there is a log stream already, close it
+  if (m_logStream) {
+    // will flush
+    nsresult rv = m_logStream->Close();
+    NS_ENSURE_SUCCESS(rv,rv);
+  }
+
+  m_logStream = aLogStream;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgFilterList::GetLogStream(nsIOutputStream **aLogStream)
+{
+  NS_ENSURE_ARG_POINTER(aLogStream);
+
+  nsresult rv;
+
+  if (!m_logStream) {
+    nsCOMPtr <nsIFileSpec> file;
+    rv = GetLogFileSpec(getter_AddRefs(file));
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    nsXPIDLCString nativePath;
+    rv = file->GetNativePath(getter_Copies(nativePath));
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    nsCOMPtr <nsILocalFile> logFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    rv = logFile->InitWithNativePath(nsDependentCString(nativePath));
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    // append to the end of the log file
+    rv = NS_NewLocalFileOutputStream(getter_AddRefs(m_logStream),
+                                   logFile,
+                                   PR_CREATE_FILE | PR_WRONLY | PR_APPEND,
+                                   0600);
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    if (!m_logStream)
+      return NS_ERROR_FAILURE;
+  }
+ 
+  NS_ADDREF(*aLogStream = m_logStream);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsMsgFilterList::ApplyFiltersToHdr(nsMsgFilterTypeType filterType,
                                    nsIMsgDBHdr *msgHdr,
@@ -142,9 +301,8 @@ nsMsgFilterList::ApplyFiltersToHdr(nsMsgFilterTypeType filterType,
 {
 	nsCOMPtr <nsIMsgFilter>	filter;
 	PRUint32		filterCount = 0;
-	nsresult		ret = NS_OK;
-
-	GetFilterCount(&filterCount);
+	nsresult		rv = GetFilterCount(&filterCount);
+  NS_ENSURE_SUCCESS(rv,rv);
 
 	for (PRUint32 filterIndex = 0; filterIndex < filterCount; filterIndex++)
 	{
@@ -166,29 +324,27 @@ nsMsgFilterList::ApplyFiltersToHdr(nsMsgFilterTypeType filterType,
 				matchTermStatus = filter->MatchHdr(msgHdr, folder, db, headers, headersSize, &result);
 				if (NS_SUCCEEDED(matchTermStatus) && result && listener)
 				{
-					PRBool applyMore;
+					PRBool applyMore = PR_TRUE;
 
-					ret  = listener->ApplyFilterHit(filter, msgWindow, &applyMore);
-					if (NS_FAILED(ret) || !applyMore)
+					rv = listener->ApplyFilterHit(filter, msgWindow, &applyMore);
+					if (NS_FAILED(rv) || !applyMore)
 						break;
 				}
 			}
 		}
 	}
-	return ret;
+	return rv;
 }
 
 NS_IMETHODIMP
 nsMsgFilterList::SetDefaultFile(nsIFileSpec *aFileSpec)
 {
     nsresult rv;
-    m_defaultFile = 
-        do_CreateInstance(NS_FILESPEC_CONTRACTID, &rv);
+    m_defaultFile = do_CreateInstance(NS_FILESPEC_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
     
     rv = m_defaultFile->FromFileSpec(aFileSpec);
     NS_ENSURE_SUCCESS(rv, rv);
-
     return NS_OK;
 }
 
@@ -216,112 +372,12 @@ nsMsgFilterList::SaveToDefaultFile()
 {
     nsresult rv;
     nsCOMPtr<nsIMsgFilterService> filterService =
-        do_GetService(kMsgFilterServiceCID, &rv);
+        do_GetService(NS_MSGFILTERSERVICE_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
 
     return filterService->SaveFilterList(this, m_defaultFile);
 }
 
-#if 0
-nsresult nsMsgFilterList::Open(nsMsgFilterTypeType type, nsIMsgFolder *folder, nsIMsgFilterList **filterList)
-{
-	nsresult	err = NS_OK;
-	nsMsgFilterList	*newFilterList;
-
-	if (type != filterInbox
-		&& type != filterNews)
-		return FilterError_InvalidFilterType;
-
-	if (nsnull == filterList)
-		return NS_ERROR_NULL_POINTER;
-
-	newFilterList = new nsMsgFilterList;
-	if (newFilterList == nsnull)
-		return NS_ERROR_OUT_OF_MEMORY;
-
-	newFilterList->m_master = master;
-
-	// hack up support for news filters by checking the current folder of the pane and massaging input params.
-	if (pane != nsnull && folderInfo == nsnull)
-	{
-		folderInfo = pane->GetFolder();
-		if (folderInfo)
-		{
-			if (folderInfo->IsNews())
-				type = filterNews;
-		}
-	}
-
-	newFilterList->m_folderInfo = folderInfo;
-	newFilterList->m_pane = pane;
-
-	*filterList = newFilterList;
-	const char *upgradeIMAPFiltersDestFileName = 0;
-
-	if (type == filterNews)
-	{
-		MSG_FolderInfoNews *newsFolder = folderInfo->GetNewsFolderInfo();
-		if (newsFolder)
-			newFilterList->m_filterFileName = newsFolder->GetXPRuleFileName();
-		newFilterList->m_fileType = xpNewsSort;
-	}
-	else
-	{
-		MSG_IMAPFolderInfoMail *imapMailFolder = (folderInfo) ? folderInfo->GetIMAPFolderInfoMail() : (MSG_IMAPFolderInfoMail *)nsnull;
-
-		newFilterList->m_filterFileName = "";
-		newFilterList->m_fileType = xpMailSort;
-		if (imapMailFolder)
-		{
-			MSG_IMAPHost *defaultHost = imapMailFolder->GetMaster()->GetIMAPHostTable()->GetDefaultHost();
-			if (imapMailFolder->GetIMAPHost() == defaultHost)
-			{
-				PRBool defaultHostFiltersExist = !XP_Stat(imapMailFolder->GetIMAPHost()->GetHostName(), &outStat, newFilterList->m_fileType);
-				if (!defaultHostFiltersExist)
-					upgradeIMAPFiltersDestFileName = imapMailFolder->GetIMAPHost()->GetHostName();
-			}
-
-			// if it's not the default imap host or there are no filters for the default host, or the old local mail filters 
-			// don't exist, set the filter file name to the filter name for the imap host.
-			if (!upgradeIMAPFiltersDestFileName || XP_Stat(newFilterList->m_filterFileName, &outStat, newFilterList->m_fileType))
-				newFilterList->m_filterFileName = imapMailFolder->GetIMAPHost()->GetHostName();
-		}
-
-	}
-
-	if (XP_Stat(newFilterList->m_filterFileName, &outStat, newFilterList->m_fileType))
-	{
-		// file must not exist - no rules, we're done.
-		return NS_OK;
-	}
-	fid = XP_FileOpen(newFilterList->m_filterFileName, newFilterList->m_fileType, XP_FILE_READ_BIN);
-	if (fid) 
-	{
-		err = newFilterList->LoadTextFilters(fid);
-		XP_FileClose(fid);
-		// if the file version changed, save it out right away.
-		if (newFilterList->GetVersion() != kFileVersion || upgradeIMAPFiltersDestFileName)
-		{
-			if (upgradeIMAPFiltersDestFileName)
-				newFilterList->m_filterFileName = upgradeIMAPFiltersDestFileName;
-			newFilterList->Close();
-		}
-	}
-	else
-	{
-		err = FilterError_FileError;
-	}
-
-	return err;
-}
-
-extern "C" MSG_FolderInfo *MSG_GetFolderInfoForFilterList(nsMsgFilterList *filterList)
-{
-	return filterList ? filterList->GetFolderInfo() : (MSG_FolderInfo *)nsnull;
-}
-#endif
-
-
 typedef struct
 {
 	nsMsgFilterFileAttribValue	attrib;
@@ -347,8 +403,15 @@ static FilterFileAttribEntry FilterFileAttribTable[] =
 char nsMsgFilterList::ReadChar(nsIOFileStream *aStream)
 {
 	char	newChar;
-	*aStream >> newChar; 
-	return (aStream->eof() ? -1 : newChar);
+	*aStream >> newChar;
+	if (aStream->eof())
+    return -1;
+  else
+  {
+    if (m_startWritingToBuffer)
+      m_unparsedFilterBuffer.Append(newChar);
+    return newChar;
+  }
 }
 
 char nsMsgFilterList::SkipWhitespace(nsIOFileStream *aStream)
@@ -370,7 +433,8 @@ char nsMsgFilterList::LoadAttrib(nsMsgFilterFileAttribValue &attrib, nsIOFileStr
 {
 	char	attribStr[100];
 	char	curChar;
-	
+  attrib = nsIMsgFilterList::attribNone;
+
 	curChar = SkipWhitespace(aStream);
 	int i;
 	for (i = 0; i + 1 < (int)(sizeof(attribStr)); )
@@ -449,120 +513,144 @@ nsresult nsMsgFilterList::LoadValue(nsCString &value, nsIOFileStream *aStream)
 nsresult nsMsgFilterList::LoadTextFilters(nsIOFileStream *aStream)
 {
 	nsresult	err = NS_OK;
-	nsMsgFilterFileAttribValue attrib;
-    nsCOMPtr<nsIImportService> impSvc;
-	// We'd really like to move lot's of these into the objects that they refer to.
-	aStream->seek(PR_SEEK_SET, 0);
-	do 
-	{
-		nsCAutoString	value;
-		PRInt32 intToStringResult;
+  nsMsgFilterFileAttribValue attrib;
+  nsCOMPtr<nsIImportService> impSvc;
+  nsCOMPtr<nsIMsgRuleAction> currentFilterAction;
+  // We'd really like to move lot's of these into the objects that they refer to.
+  aStream->seek(PR_SEEK_SET, 0);
+  do 
+  {
+    nsCAutoString	value;
+    PRInt32 intToStringResult;
 
 		char curChar;
-
-        curChar = LoadAttrib(attrib, aStream);
-		if (attrib == nsIMsgFilterList::attribNone)
+    curChar = LoadAttrib(attrib, aStream);
+    if (curChar == (char) -1)  //reached eof
+      break;
+    err = LoadValue(value, aStream);
+    if (err != NS_OK)
+      break;
+    switch(attrib)
+    {
+      case nsIMsgFilterList::attribNone:
+        m_curFilter->SetUnparseable(PR_TRUE);
+        break;
+      case nsIMsgFilterList::attribVersion:
+        m_fileVersion = value.ToInteger(&intToStringResult, 10);
+			  if (intToStringResult != 0)
+        {
+          attrib = nsIMsgFilterList::attribNone;
+          NS_ASSERTION(PR_FALSE, "error parsing filter file version");
+        }
+        if (m_fileVersion == k45Version)
+        {
+          impSvc = do_GetService(NS_IMPORTSERVICE_CONTRACTID);
+          NS_ASSERTION(impSvc, "cannot get importService");
+        }
+        break;
+      case nsIMsgFilterList::attribLogging:
+        m_loggingEnabled = StrToBool(value);
+        m_unparsedFilterBuffer.Truncate(); //we are going to buffer each filter as we read them, make sure no garbage is there
+        m_startWritingToBuffer = PR_TRUE; //filters begin now
+        break;
+		  case nsIMsgFilterList::attribName:  //every filter starts w/ a name
+      {
+        if (m_curFilter)
+        {
+          PRInt32 nextFilterStartPos = m_unparsedFilterBuffer.RFind("name");
+          nsCAutoString nextFilterPart;
+          nextFilterPart = Substring(m_unparsedFilterBuffer, nextFilterStartPos, m_unparsedFilterBuffer.Length());
+          m_unparsedFilterBuffer.Truncate(nextFilterStartPos);
+        
+          PRBool unparseableFilter;
+          m_curFilter->GetUnparseable(&unparseableFilter);
+          if (unparseableFilter)
+          {
+            m_curFilter->SetUnparsedBuffer(m_unparsedFilterBuffer.get());
+            m_curFilter->SetEnabled(PR_FALSE); //disable the filter because we don't know how to apply it
+          }
+          m_unparsedFilterBuffer = nextFilterPart;
+        }
+			  nsMsgFilter *filter = new nsMsgFilter;
+			  if (filter == nsnull)
+        {
+				  err = NS_ERROR_OUT_OF_MEMORY;
+				  break;
+        }
+			  filter->SetFilterList(NS_STATIC_CAST(nsIMsgFilterList*,this));
+        if (m_fileVersion == k45Version && impSvc)
+        {
+          nsAutoString unicodeStr;
+          impSvc->SystemStringToUnicode(value.get(), unicodeStr);
+          filter->SetFilterName(unicodeStr.get());
+        }
+        else
+        {  
+          PRUnichar *unicodeString =
+          nsTextFormatter::smprintf(unicodeFormatter, value.get());
+			    filter->SetFilterName(unicodeString);
+          nsTextFormatter::smprintf_free(unicodeString);
+        }
+			  m_curFilter = filter;
+			  m_filters->AppendElement(NS_STATIC_CAST(nsISupports*,filter));
+      }
 			break;
-		err = LoadValue(value, aStream);
-		if (err != NS_OK)
+		  case nsIMsgFilterList::attribEnabled:
+			  if (m_curFilter)
+				  m_curFilter->SetEnabled(StrToBool(value));
 			break;
-		switch(attrib)
-		{
-		case nsIMsgFilterList::attribNone:
+		  case nsIMsgFilterList::attribDescription:
+			  if (m_curFilter)
+			   	m_curFilter->SetFilterDesc(value.get());
 			break;
-		case nsIMsgFilterList::attribVersion:
-			m_fileVersion = value.ToInteger(&intToStringResult, 10);
-			if (intToStringResult != 0)
-			{
-				attrib = nsIMsgFilterList::attribNone;
-				NS_ASSERTION(PR_FALSE, "error parsing filter file version");
-			}
-            if (m_fileVersion == k45Version)
-            {
-              impSvc = do_GetService(NS_IMPORTSERVICE_CONTRACTID);
-              NS_ASSERTION(impSvc, "cannot get importService");
-            }
- 			break;
-		case nsIMsgFilterList::attribLogging:
-			m_loggingEnabled = StrToBool(value);
+		  case nsIMsgFilterList::attribType:
+			  if (m_curFilter)
+				   m_curFilter->SetType((nsMsgFilterTypeType) value.ToInteger(&intToStringResult, 10));
 			break;
-		case nsIMsgFilterList::attribName:
-		{
-			nsMsgFilter *filter = new nsMsgFilter;
-			if (filter == nsnull)
-			{
-				err = NS_ERROR_OUT_OF_MEMORY;
-				break;
-			}
-			filter->SetFilterList(NS_STATIC_CAST(nsIMsgFilterList*,this));
-            if (m_fileVersion == k45Version && impSvc)
-            {
-                nsAutoString unicodeStr;
-                impSvc->SystemStringToUnicode(value.get(), unicodeStr);
-                filter->SetFilterName(unicodeStr.get());
-            }
+		  case nsIMsgFilterList::attribScriptFile:
+			  if (m_curFilter)
+				  m_curFilter->SetFilterScript(&value);
+			break;
+		  case nsIMsgFilterList::attribAction:
+        if (m_curFilter)
+        {
+          nsMsgRuleActionType actionType = nsMsgFilter::GetActionForFilingStr(value);
+          if (actionType == nsMsgFilterAction::None)
+            m_curFilter->SetUnparseable(PR_TRUE);
+          else
+          {
+            err = m_curFilter->CreateAction(getter_AddRefs(currentFilterAction));
+            NS_ENSURE_SUCCESS(err, err);
+            currentFilterAction->SetType(actionType);
+            m_curFilter->AppendAction(currentFilterAction);
+          }
+        }
+      break;
+      case nsIMsgFilterList::attribActionValue:
+        if (m_curFilter && currentFilterAction)
+        {
+          nsMsgRuleActionType type;
+          currentFilterAction->GetType(&type);
+          if (type == nsMsgFilterAction::MoveToFolder)
+            err = m_curFilter->ConvertMoveToFolderValue(currentFilterAction, value);
+          else if (type == nsMsgFilterAction::ChangePriority)
+          {
+            nsMsgPriorityValue outPriority;
+            nsresult res = NS_MsgGetPriorityFromString(value.get(), &outPriority);
+            if (NS_SUCCEEDED(res))
+              currentFilterAction->SetPriority(outPriority);
             else
-            {  
-              PRUnichar *unicodeString =
-                nsTextFormatter::smprintf(unicodeFormatter, value.get());
-			  filter->SetFilterName(unicodeString);
-              nsTextFormatter::smprintf_free(unicodeString);
-            }
-			m_curFilter = filter;
-			m_filters->AppendElement(NS_STATIC_CAST(nsISupports*,filter));
-		}
-			break;
-		case nsIMsgFilterList::attribEnabled:
-			if (m_curFilter)
-				m_curFilter->SetEnabled(StrToBool(value));
-			break;
-		case nsIMsgFilterList::attribDescription:
-			if (m_curFilter)
-				m_curFilter->SetFilterDesc(value.get());
-			break;
-		case nsIMsgFilterList::attribType:
-			if (m_curFilter)
-			{
-				m_curFilter->SetType((nsMsgFilterTypeType) value.ToInteger(&intToStringResult, 10));
-			}
-			break;
-		case nsIMsgFilterList::attribScriptFile:
-			if (m_curFilter)
-				m_curFilter->SetFilterScript(&value);
-			break;
-		case nsIMsgFilterList::attribAction:
-            if (m_curFilter)
-              m_curFilter->m_action.m_type = nsMsgFilter::GetActionForFilingStr(value);
-            break;
-		case nsIMsgFilterList::attribActionValue:
-            if (m_curFilter)
-            {
-              if (m_curFilter->m_action.m_type == nsMsgFilterAction::MoveToFolder)
-                err = m_curFilter->ConvertMoveToFolderValue(value);
-              else if (m_curFilter->m_action.m_type == nsMsgFilterAction::ChangePriority)
-              {
-                nsMsgPriorityValue outPriority;
-                nsresult res = NS_MsgGetPriorityFromString(value.get(), &outPriority);
-                if (NS_SUCCEEDED(res))
-                {
-                  m_curFilter->SetAction(m_curFilter->m_action.m_type);
-                  m_curFilter->SetActionPriority(outPriority);
-                }
-                else
-                  NS_ASSERTION(PR_FALSE, "invalid priority in filter file");
-              }
-              else if (m_curFilter->m_action.m_type == nsMsgFilterAction::Label)
-              {
-                PRInt32 res;
-                PRInt32 labelInt = value.ToInteger(&res, 10);
-                if (res == 0)
-                {
-                  m_curFilter->SetAction(m_curFilter->m_action.m_type);
-                  m_curFilter->SetActionLabel((nsMsgLabelValue) labelInt);
-                }
-              }
-            }
-            break;
+              NS_ASSERTION(PR_FALSE, "invalid priority in filter file");
+          }
+          else if (type == nsMsgFilterAction::Label)
+          {
+            PRInt32 res;
+            PRInt32 labelInt = value.ToInteger(&res, 10);
+            if (res == 0)
+              currentFilterAction->SetLabel((nsMsgLabelValue) labelInt);
+          }
+        }
+        break;
 		case nsIMsgFilterList::attribCondition:
             if (m_curFilter)
             {
@@ -575,10 +663,25 @@ nsresult nsMsgFilterList::LoadTextFilters(nsIOFileStream *aStream)
                 nsMemory::Free(utf8);
               }
               err = ParseCondition(value);
+              if (err == NS_ERROR_INVALID_ARG)
+                err = m_curFilter->SetUnparseable(PR_TRUE);
+              NS_ENSURE_SUCCESS(err, err);
             }
             break;
 		}
-	} while (attrib != nsIMsgFilterList::attribNone);
+	} while (!aStream->eof());
+
+  if (m_curFilter)
+  {
+    PRBool unparseableFilter;
+    m_curFilter->GetUnparseable(&unparseableFilter);
+    if (unparseableFilter)
+    {
+      m_curFilter->SetUnparsedBuffer(m_unparsedFilterBuffer.get());
+      m_curFilter->SetEnabled(PR_FALSE);  //disable the filter because we don't know how to apply it
+    }
+  }
+
 	return err;
 }
 
@@ -647,9 +750,9 @@ nsresult nsMsgFilterList::ParseCondition(nsCString &value)
                 else {
                     newTerm->m_booleanOp = nsMsgSearchBooleanOp::BooleanOR;
                 }
-                
-                if (newTerm->DeStreamNew(termDup, PL_strlen(termDup)) == NS_OK)
-                    m_curFilter->AppendTerm(newTerm);
+                err = newTerm->DeStreamNew(termDup, PL_strlen(termDup));
+                NS_ENSURE_SUCCESS(err, err);
+                m_curFilter->AppendTerm(newTerm);
             }
 			PR_FREEIF(termDup);
 		}
@@ -747,7 +850,6 @@ nsresult nsMsgFilterList::SaveTextFilters(nsIOFileStream *aStream)
 
 nsMsgFilterList::~nsMsgFilterList()
 {
-
 	// filters should be released for free, because only isupports array
 	// is holding onto them, right?
 //	PRUint32			filterCount;
@@ -762,38 +864,7 @@ nsMsgFilterList::~nsMsgFilterList()
 
 nsresult nsMsgFilterList::Close()
 {
-#ifdef HAVE_PORT
-	nsresult err = FilterError_FileError;
-	XP_File			fid;
-	XP_FileType		retType;
-	const char		*finalName = m_filterFileName;
-	char			*tmpName = (finalName) ? FE_GetTempFileFor(nsnull, finalName, m_fileType, &retType) : (char *)nsnull;
-
-
-	if (!tmpName || !finalName) 
-		return NS_ERROR_OUT_OF_MEMORY;
-	m_fileStream = new nsIOFileStream
-	fid = XP_FileOpen(tmpName, xpTemporary,
-								 XP_FILE_TRUNCATE_BIN);
-	if (fid) 
-	{
-		err = SaveTextFilters(fid);
-		XP_FileClose(fid);
-		if (err == NS_OK)
-		{
-			int status = XP_FileRename(tmpName, xpTemporary, finalName, m_fileType);
-			PR_ASSERT(status >= 0);
-		}
-	}
-
-	PR_FREEIF(tmpName);
-	// tell open DB's that the filter list might have changed.
-	NewsGroupDB::NotifyOpenDBsOfFilterChange(m_folderInfo);
-
-	return err;
-#else
 	return NS_ERROR_NOT_IMPLEMENTED;
-#endif
 }
 
 nsresult nsMsgFilterList::GetFilterCount(PRUint32 *pCount)
@@ -803,7 +874,6 @@ nsresult nsMsgFilterList::GetFilterCount(PRUint32 *pCount)
 
 nsresult nsMsgFilterList::GetMsgFilterAt(PRUint32 filterIndex, nsMsgFilter **filter)
 {
-
 	PRUint32			filterCount;
 	m_filters->Count(&filterCount);
 	if (! (filterCount >= filterIndex))
@@ -945,26 +1015,37 @@ NS_IMETHODIMP nsMsgFilterList::MatchOrChangeFilterTarget(const char *oldFolderUr
   rv = m_filters->Count(&numFilters);
   NS_ENSURE_SUCCESS(rv,rv);
   nsCOMPtr <nsIMsgFilter> filter;
-  nsMsgRuleActionType actionType;
   nsXPIDLCString folderUri;
-  nsCOMPtr <nsISupports> filterSupports;
   for (PRUint32 index = 0; index < numFilters; index++)
   {
-    filterSupports = getter_AddRefs(m_filters->ElementAt(index));
-    filter = do_QueryInterface(filterSupports, &rv);
-    if (NS_SUCCEEDED(rv) && filter)
+    filter = do_QueryElementAt(m_filters, index, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsCOMPtr<nsISupportsArray> filterActionList;
+    rv = filter->GetActionList(getter_AddRefs(filterActionList));
+    PRUint32 numActions;
+    filterActionList->Count(&numActions);
+
+    for (PRUint32 actionIndex =0; actionIndex < numActions; actionIndex++)
     {
-      rv = filter->GetAction(&actionType);
-      if (NS_SUCCEEDED(rv) && actionType == nsMsgFilterAction::MoveToFolder)
+      nsCOMPtr<nsIMsgRuleAction> filterAction =
+          do_QueryElementAt(filterActionList, actionIndex);
+      nsMsgRuleActionType actionType;
+      if (filterAction)
+        filterAction->GetType(&actionType);
+      else
+        continue;
+
+      if (actionType == nsMsgFilterAction::MoveToFolder)
       {
-        rv = filter->GetActionTargetFolderUri(getter_Copies(folderUri));
+        rv = filterAction->GetTargetFolderUri(getter_Copies(folderUri));
         if (NS_SUCCEEDED(rv) && folderUri)
-          if (caseInsensitive)
+           if (caseInsensitive)
           {
             if (PL_strcasecmp(folderUri,oldFolderUri) == 0 ) //local
             {
-              if (newFolderUri)  //in the case where we just want to match the uri's newFolderUri will be null
-                rv = filter->SetActionTargetFolderUri(newFolderUri);
+              if (newFolderUri)  //if we just want to match the uri's, newFolderUri will be null
+                rv = filterAction->SetTargetFolderUri(newFolderUri);
               NS_ENSURE_SUCCESS(rv,rv);
               *found =PR_TRUE;
             }
@@ -973,19 +1054,29 @@ NS_IMETHODIMP nsMsgFilterList::MatchOrChangeFilterTarget(const char *oldFolderUr
           {
             if (PL_strcmp(folderUri,oldFolderUri) == 0 )  //imap
             {
-              if (newFolderUri) //in the case where we just want to match the uri's newFolderUri will be null
-                rv = filter->SetActionTargetFolderUri(newFolderUri);
+              if (newFolderUri) //if we just want to match the uri's, newFolderUri will be null
+                rv = filterAction->SetTargetFolderUri(newFolderUri);
               NS_ENSURE_SUCCESS(rv,rv);
               *found =PR_TRUE;
             }
           }
+        break;  //we allow only one move action per filter
       }
     }
   }
   return rv;
 }
 
-NS_IMETHODIMP nsMsgFilterList::GetShouldDownloadArbitraryHeaders(PRBool *aResult)
+// this would only return true if any filter was on "any header", which we
+// don't support in 6.x
+NS_IMETHODIMP nsMsgFilterList::GetShouldDownloadAllHeaders(PRBool *aResult)
+{
+  *aResult = PR_FALSE;
+  return NS_OK;
+}
+
+// leaves m_arbitraryHeaders filed in with the arbitrary headers.
+nsresult nsMsgFilterList::ComputeArbitraryHeaders()
 {
   nsresult rv = NS_OK;
   if (m_arbitraryHeaders.Length() == 0)
@@ -994,13 +1085,11 @@ NS_IMETHODIMP nsMsgFilterList::GetShouldDownloadArbitraryHeaders(PRBool *aResult
     rv = m_filters->Count(&numFilters);
     NS_ENSURE_SUCCESS(rv,rv);
     nsCOMPtr <nsIMsgFilter> filter;
-    nsCOMPtr <nsISupports> filterSupports;
     nsMsgSearchAttribValue attrib;
     nsXPIDLCString arbitraryHeader;
     for (PRUint32 index = 0; index < numFilters; index++)
     {
-      filterSupports = getter_AddRefs(m_filters->ElementAt(index));
-      filter = do_QueryInterface(filterSupports, &rv);
+      filter = do_QueryElementAt(m_filters, index, &rv);
       if (NS_SUCCEEDED(rv) && filter)
       {
         nsCOMPtr <nsISupportsArray> searchTerms;
@@ -1025,21 +1114,34 @@ NS_IMETHODIMP nsMsgFilterList::GetShouldDownloadArbitraryHeaders(PRBool *aResult
       }
     }
   }
-  if (m_arbitraryHeaders.Length() > 0)  //if any arbitraryHeaders;
-    *aResult=PR_TRUE;
-  return NS_OK;
+  return rv;
 }
 
 NS_IMETHODIMP nsMsgFilterList::GetArbitraryHeaders(char **aResult)
 {
-  PRBool headers=PR_FALSE;
-  GetShouldDownloadArbitraryHeaders(&headers);
-  if (headers)
-    *aResult = ToNewCString(m_arbitraryHeaders);
+  ComputeArbitraryHeaders();
+  *aResult = ToNewCString(m_arbitraryHeaders);
   return NS_OK;
 }
 
+NS_IMETHODIMP nsMsgFilterList::FlushLogIfNecessary()
+{
+  // only flush the log if we are logging
+  PRBool loggingEnabled = PR_FALSE;
+  nsresult rv = GetLoggingEnabled(&loggingEnabled);
+  NS_ENSURE_SUCCESS(rv,rv);
 
+  if (loggingEnabled) 
+  {
+    nsCOMPtr <nsIOutputStream> logStream;
+    rv = GetLogStream(getter_AddRefs(logStream));    
+    if (NS_SUCCEEDED(rv) && logStream) {
+      rv = logStream->Flush();
+      NS_ENSURE_SUCCESS(rv,rv);
+    }
+  }
+  return rv;
+}
 
 #ifdef DEBUG
 void nsMsgFilterList::Dump()
