@@ -55,7 +55,6 @@ NS_IMPL_ISUPPORTS4(nsMsgSearchSession, nsIMsgSearchSession, nsIUrlListener, nsIF
 
 nsMsgSearchSession::nsMsgSearchSession()
 {
-  NS_INIT_ISUPPORTS();
 	m_sortAttribute = nsMsgSearchAttrib::Sender;
 	m_descending = PR_FALSE;
 	m_idxRunningScope = 0; 
@@ -277,8 +276,13 @@ NS_IMETHODIMP nsMsgSearchSession::InterruptSearch()
 {
   if (m_window)
   {
-    ReleaseFolderDBRef();
-    m_idxRunningScope = m_scopeList.Count(); // this'll make us not run another url
+    EnableFolderNotifications(PR_TRUE);
+    while (m_idxRunningScope < m_scopeList.Count())
+    {
+      ReleaseFolderDBRef();
+      m_idxRunningScope++;
+    }
+    //m_idxRunningScope = m_scopeList.Count() so it will make us not run another url
     m_window->StopUrls();
   }
   if (m_backgroundTimer)
@@ -368,6 +372,7 @@ NS_IMETHODIMP nsMsgSearchSession::OnStopRunningUrl(nsIURI *url, nsresult aExitCo
   if (NS_SUCCEEDED(rv) && runningAdapter)
   {
     runningAdapter->CurrentUrlDone(aExitCode);
+    EnableFolderNotifications(PR_TRUE);
     ReleaseFolderDBRef();
   }
   m_idxRunningScope++;
@@ -414,7 +419,6 @@ nsresult nsMsgSearchSession::Initialize()
 		// NS_ASSERTION(scopeTerm->IsValid());
 
 		err = scopeTerm->InitializeAdapter (m_termList);
-
 //		if (scopeTerm->m_folder->GetType() == FOLDER_MAIL)
 //			m_offlineProgressTotal += scopeTerm->m_folder->GetTotalMessages();
 	}
@@ -481,17 +485,18 @@ nsresult nsMsgSearchSession::GetNextUrl()
 
   m_urlQueue.CStringAt(m_idxRunningScope, nextUrl);
   nsMsgSearchScopeTerm *currentTerm = GetRunningScope();
+  EnableFolderNotifications(PR_FALSE);
   nsCOMPtr <nsIMsgFolder> folder = currentTerm->m_folder;
   if (folder)
   {
     nsXPIDLCString folderUri;
     folder->GetURI(getter_Copies(folderUri));
-  nsresult rv = GetMessageServiceFromURI(folderUri.get(), getter_AddRefs(msgService));
+    nsresult rv = GetMessageServiceFromURI(folderUri.get(), getter_AddRefs(msgService));
 
-  if (NS_SUCCEEDED(rv) && msgService && currentTerm)
-    msgService->Search(this, m_window, currentTerm->m_folder, nextUrl.get());
+    if (NS_SUCCEEDED(rv) && msgService && currentTerm)
+      msgService->Search(this, m_window, currentTerm->m_folder, nextUrl.get());
 
-	return rv;
+	  return rv;
   }
   return NS_OK;
 }
@@ -527,13 +532,15 @@ nsresult nsMsgSearchSession::StartTimer()
   PRBool done;
 
   m_backgroundTimer = do_CreateInstance("@mozilla.org/timer;1", &err);
-  m_backgroundTimer->Init(TimerCallback, (void *) this, 0, PR_TRUE, NS_TYPE_REPEATING_SLACK);
+  m_backgroundTimer->InitWithFuncCallback(TimerCallback, (void *) this, 0, 
+                                          nsITimer::TYPE_REPEATING_SLACK);
   // ### start meteors?
   return TimeSlice(&done);
 }
 
 nsresult nsMsgSearchSession::SearchWOUrls ()
 {
+  EnableFolderNotifications(PR_FALSE);
   return StartTimer();
 }
 
@@ -682,8 +689,11 @@ nsresult nsMsgSearchSession::TimeSliceSerial (PRBool *aDone)
       *aDone = PR_TRUE;
     if (*aDone || NS_FAILED(rv))
     {
+      EnableFolderNotifications(PR_TRUE);
       ReleaseFolderDBRef();
       m_idxRunningScope++;
+      EnableFolderNotifications(PR_FALSE);
+
       //			if (m_idxRunningScope < m_scopeList.Count())
       //  			UpdateStatusBar (MK_MSG_SEARCH_STATUS);
     }
@@ -697,24 +707,29 @@ nsresult nsMsgSearchSession::TimeSliceSerial (PRBool *aDone)
   }
 }
 
+void 
+nsMsgSearchSession::EnableFolderNotifications(PRBool aEnable)
+{
+  nsMsgSearchScopeTerm *scope = GetRunningScope();
+  if (scope)
+  {
+    nsCOMPtr<nsIMsgFolder> folder;
+    scope->GetFolder(getter_AddRefs(folder));
+    if (folder)  //enable msg count notifications
+      folder->EnableNotifications(nsIMsgFolder::allMessageCountNotifications, aEnable, PR_FALSE);
+  }
+}
+
 NS_IMETHODIMP
 nsMsgSearchSession::AddFolderListener(nsIFolderListener *listener)
 {
-    nsresult rv = NS_OK;
-    if (!m_folderListenerList)
-        rv = NS_NewISupportsArray(getter_AddRefs(m_folderListenerList));
-
-    if (NS_SUCCEEDED(rv) && m_folderListenerList)
-        m_folderListenerList->AppendElement(listener);
-
-    return rv;
+    return m_folderListenerList.AppendObject(listener) ? NS_OK : NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
 nsMsgSearchSession::RemoveFolderListener(nsIFolderListener *listener)
 {
-    if (m_folderListenerList)
-        m_folderListenerList->RemoveElement(listener);
+    m_folderListenerList.RemoveObject(listener);
 
     return NS_OK;
 }
@@ -724,18 +739,11 @@ NS_IMETHODIMP
 nsMsgSearchSession::OnItemEvent(nsIFolder *aFolder,
                                 nsIAtom *aEvent)
 {
-	nsresult rv;
-	PRUint32 count;
+	PRInt32 count = m_folderListenerList.Count();
 
-	NS_ASSERTION(m_folderListenerList, "no listeners");
-	if (!m_folderListenerList) return NS_ERROR_FAILURE;
-
-	rv = m_folderListenerList->Count(&count);
-	if (NS_FAILED(rv)) return rv;
-
-	for(PRUint32 i = 0; i < count; i++)
+	for(PRInt32 i = 0; i < count; i++)
 	{
-		nsCOMPtr<nsIFolderListener> listener = getter_AddRefs((nsIFolderListener*)m_folderListenerList->ElementAt(i));
+		nsIFolderListener* listener = m_folderListenerList[i];
 		if(listener)
 			listener->OnItemEvent(aFolder, aEvent);
 	}
