@@ -37,6 +37,7 @@
 #include "nsIObserver.h"
 #include "nsWeakReference.h"
 #include "nsIUrlListener.h"
+#include "nsCOMArray.h"
 
 class nsMsgAccountManager
 	: public nsIMsgAccountManager,
@@ -70,7 +71,7 @@ private:
   nsHashtable m_identities;
   nsHashtable m_incomingServers;
   nsCOMPtr<nsIMsgAccount> m_defaultAccount;
-  nsCOMPtr<nsISupportsArray> m_incomingServerListeners;
+  nsCOMArray<nsIIncomingServerListener> m_incomingServerListeners;
   nsCOMPtr<nsIMsgFolder> m_folderDoingEmptyTrash;
   nsCOMPtr<nsIMsgFolder> m_folderDoingCleanupInbox;
   PRBool m_emptyTrashInProgress;
@@ -112,6 +113,7 @@ private:
   nsresult notifyDefaultServerChange(nsIMsgAccount *aOldAccount,
                                      nsIMsgAccount *aNewAccount);
     
+  static void LogoutOfServer(nsIMsgIncomingServer *aServer);
   // hash table enumerators
 
   // add each member of a hash table to an nsISupports array
@@ -126,6 +128,10 @@ private:
   static PRBool PR_CALLBACK hashUnloadServer(nsHashKey *aKey, void *aData,
                                      void *closure);
 
+  // shutdown server and forget cached password
+  static PRBool PR_CALLBACK hashLogoutOfServer(nsHashKey *aKey, void *aData,
+                                     void *closure);
+
   // clean up on exit
   static PRBool PR_CALLBACK cleanupOnExit(nsHashKey *aKey, void *aData,
                                      void *closure);
@@ -168,6 +174,7 @@ private:
 
   // write out the server's cache through the given folder cache
   static PRBool PR_CALLBACK writeFolderCache(nsHashKey *aKey, void *aData, void *closure);
+  static PRBool PR_CALLBACK shutdown(nsHashKey *aKey, void *aData, void *closure);
   static PRBool PR_CALLBACK closeCachedConnections(nsHashKey *aKey, void *aData, void *closure);
 
   static void getUniqueKey(const char* prefix,
@@ -183,6 +190,8 @@ private:
   nsresult getPrefService();
   nsIPref *m_prefs;
 
+  nsresult InternalFindServer(const char* username, const char* hostname, const char* type, PRBool useRealSetting, nsIMsgIncomingServer** aResult);
+
   //
   // root folder listener stuff
   //
