@@ -159,38 +159,55 @@ nsMsgAccountManagerDataSource::nsMsgAccountManagerDataSource()
   
   // do per-class initialization here
   if (gAccountManagerResourceRefCnt++ == 0) {
-      getRDFService()->GetResource(NC_RDF_CHILD, &kNC_Child);
-      getRDFService()->GetResource(NC_RDF_NAME, &kNC_Name);
-      getRDFService()->GetResource(NC_RDF_FOLDERTREENAME, &kNC_FolderTreeName);
-      getRDFService()->GetResource(NC_RDF_FOLDERTREESIMPLENAME, &kNC_FolderTreeSimpleName);
-      getRDFService()->GetResource(NC_RDF_NAME_SORT, &kNC_NameSort);
-      getRDFService()->GetResource(NC_RDF_FOLDERTREENAME_SORT, &kNC_FolderTreeNameSort);
-      getRDFService()->GetResource(NC_RDF_PAGETAG, &kNC_PageTag);
-      getRDFService()->GetResource(NC_RDF_ISDEFAULTSERVER, &kNC_IsDefaultServer);
-      getRDFService()->GetResource(NC_RDF_SUPPORTSFILTERS, &kNC_SupportsFilters);
-      getRDFService()->GetResource(NC_RDF_CANGETMESSAGES, &kNC_CanGetMessages);
-      getRDFService()->GetResource(NC_RDF_CANGETINCOMINGMESSAGES, &kNC_CanGetIncomingMessages);
-      getRDFService()->GetResource(NC_RDF_ACCOUNT, &kNC_Account);
-      getRDFService()->GetResource(NC_RDF_SERVER, &kNC_Server);
-      getRDFService()->GetResource(NC_RDF_IDENTITY, &kNC_Identity);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_MAIN, &kNC_PageTitleMain);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_SERVER, &kNC_PageTitleServer);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_COPIES, &kNC_PageTitleCopies);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_OFFLINEANDDISKSPACE, &kNC_PageTitleOfflineAndDiskSpace);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_DISKSPACE, &kNC_PageTitleDiskSpace);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_ADDRESSING, &kNC_PageTitleAddressing);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_ADVANCED, &kNC_PageTitleAdvanced);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_SMTP, &kNC_PageTitleSMTP);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_FAKEACCOUNT, &kNC_PageTitleFakeAccount);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_CHILD), &kNC_Child);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_NAME), &kNC_Name);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_FOLDERTREENAME),
+                                   &kNC_FolderTreeName);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_FOLDERTREESIMPLENAME),
+                                   &kNC_FolderTreeSimpleName);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_NAME_SORT), &kNC_NameSort);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_FOLDERTREENAME_SORT),
+                                   &kNC_FolderTreeNameSort);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETAG), &kNC_PageTag);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_ISDEFAULTSERVER),
+                                   &kNC_IsDefaultServer);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_SUPPORTSFILTERS),
+                                   &kNC_SupportsFilters);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_CANGETMESSAGES),
+                                   &kNC_CanGetMessages);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_CANGETINCOMINGMESSAGES),
+                                   &kNC_CanGetIncomingMessages);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_ACCOUNT), &kNC_Account);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_SERVER), &kNC_Server);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_IDENTITY), &kNC_Identity);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_MAIN),
+                                   &kNC_PageTitleMain);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_SERVER),
+                                   &kNC_PageTitleServer);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_COPIES),
+                                   &kNC_PageTitleCopies);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_OFFLINEANDDISKSPACE),
+                                   &kNC_PageTitleOfflineAndDiskSpace);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_DISKSPACE),
+                                   &kNC_PageTitleDiskSpace);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_ADDRESSING),
+                                   &kNC_PageTitleAddressing);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_ADVANCED),
+                                   &kNC_PageTitleAdvanced);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_SMTP),
+                                   &kNC_PageTitleSMTP);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_FAKEACCOUNT),
+                                   &kNC_PageTitleFakeAccount);
       
-      getRDFService()->GetResource(NC_RDF_ACCOUNTROOT, &kNC_AccountRoot);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_ACCOUNTROOT),
+                                   &kNC_AccountRoot);
 
       getRDFService()->GetLiteral(NS_LITERAL_STRING("true").get(),
                                   &kTrueLiteral);
       
       // eventually these need to exist in some kind of array
       // that's easily extensible
-      getRDFService()->GetResource(NC_RDF_SETTINGS, &kNC_Settings);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_SETTINGS), &kNC_Settings);
       
       kDefaultServerAtom = NS_NewAtom("DefaultServer");
    }
@@ -758,7 +775,7 @@ nsMsgAccountManagerDataSource::appendGenericSetting(const char *name, nsISupport
   resourceStr = NC_RDF_PAGETITLE_PREFIX;
   resourceStr += name;
 
-  nsresult rv = getRDFService()->GetResource(resourceStr.get(), getter_AddRefs(resource));
+  nsresult rv = getRDFService()->GetResource(resourceStr, getter_AddRefs(resource));
   NS_ENSURE_SUCCESS(rv,rv);
 
   // AppendElement will addref.
