@@ -1,25 +1,40 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998, 1999 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998, 1999
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
+ * Contributor(s):
  *   Pierre Phaneuf <pp@ludusdesign.com>
- */
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #include "nsMsgCopyService.h"
 #include "nsVoidArray.h"
@@ -84,7 +99,6 @@ nsCopyRequest::~nsCopyRequest()
   while(j-- > 0)
   {
       ncs = (nsCopySource*) m_copySourceArray.ElementAt(j);
-      m_copySourceArray.RemoveElementAt(j);
       delete ncs;
   }
 }
@@ -109,6 +123,18 @@ nsCopyRequest::Init(nsCopyRequestType type, nsISupports* aSupport,
     if (m_allowUndo)
 		msgWindow->GetTransactionManager(getter_AddRefs(m_txnMgr));
 	}
+  if (type == nsCopyFoldersType)
+  {
+    // To support multiple copy folder operations to the same destination, we 
+    // need to save the leaf name of the src file spec so that FindRequest() is
+    // able to find the right request when copy finishes.
+    nsCOMPtr<nsIMsgFolder> srcFolder = do_QueryInterface(aSupport, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+    nsXPIDLString folderName;
+    rv = srcFolder->GetName(getter_Copies(folderName));
+    NS_ENSURE_SUCCESS(rv, rv);
+    m_dstFolderName = folderName;
+  }
   
   return rv;
 }
@@ -128,7 +154,6 @@ nsCopyRequest::AddNewCopySource(nsIMsgFolder* srcFolder)
 
 nsMsgCopyService::nsMsgCopyService()
 {
-  NS_INIT_REFCNT();
 }
 
 nsMsgCopyService::~nsMsgCopyService()
@@ -166,18 +191,39 @@ nsMsgCopyService::ClearRequest(nsCopyRequest* aRequest, nsresult rv)
   return rv;
 }
 
+nsresult 
+nsMsgCopyService::QueueRequest(nsCopyRequest* aRequest, PRBool *aCopyImmediately)
+{
+  NS_ENSURE_ARG_POINTER(aRequest);
+  NS_ENSURE_ARG_POINTER(aCopyImmediately);
+  *aCopyImmediately = PR_TRUE;
+  nsCopyRequest* copyRequest;
+
+  PRInt32 cnt, i;
+  cnt = m_copyRequests.Count();
+  for (i=0; i < cnt; i++)
+  {
+    copyRequest = (nsCopyRequest*) m_copyRequests.ElementAt(i);
+    if (copyRequest->m_dstFolder.get() == aRequest->m_dstFolder.get())  //if dst are same and we already have a request, we cannot copy immediately
+    {
+      *aCopyImmediately = PR_FALSE;
+      break;
+    }
+  }
+  return NS_OK;
+}
+  
 nsresult 
 nsMsgCopyService::DoCopy(nsCopyRequest* aRequest)
 {
-  nsresult rv = NS_ERROR_NULL_POINTER;
+  NS_ENSURE_ARG(aRequest);
+  PRBool copyImmediately;
+  QueueRequest(aRequest, &copyImmediately);
+  m_copyRequests.AppendElement((void*) aRequest);
+  if (copyImmediately) // if there wasn't another request for this dest folder then we can copy immediately
+    return DoNextCopy();
 
-  if (aRequest)
-  {
-      m_copyRequests.AppendElement((void*) aRequest);
-      rv = DoNextCopy();
-  }
-
-  return rv;
+  return NS_OK;
 }
 
 nsresult
@@ -203,6 +249,10 @@ nsMsgCopyService::DoNextCopy()
         {
           copySource = (nsCopySource*)
               copyRequest->m_copySourceArray.ElementAt(j);
+
+          if (copySource->m_msgFolder == copyRequest->m_dstFolder)  //don't do move/copy on itself, just skip
+            copySource->m_processed = PR_TRUE;
+
           if (!copySource->m_processed) goto found;
         }
         if (j >= scnt) // all processed set the value
@@ -225,13 +275,13 @@ nsMsgCopyService::DoNextCopy()
                                                               
           }
           else if (copyRequest->m_requestType == nsCopyFoldersType )
-		  {
-			  copySource->m_processed = PR_TRUE;
+          {
+			        copySource->m_processed = PR_TRUE;
               rv = copyRequest->m_dstFolder->CopyFolder
                   (copySource->m_msgFolder,
                    copyRequest->m_isMoveOrDraftOrTemplate,
                    copyRequest->m_msgWindow, copyRequest->m_listener);
-		  }
+          }
           else if (copyRequest->m_requestType == nsCopyFileMessageType)
           {
             nsCOMPtr<nsIFileSpec> aSpec(do_QueryInterface(copyRequest->m_srcSupport, &rv));
@@ -243,10 +293,8 @@ nsMsgCopyService::DoNextCopy()
                 nsCOMPtr<nsIMsgDBHdr> aMessage;
                 if (copySource)
                 {
-                    nsCOMPtr<nsISupports> aSupport;
-                    aSupport =
-                        getter_AddRefs(copySource->m_messageArray->ElementAt(0));
-                    aMessage = do_QueryInterface(aSupport, &rv);
+                    aMessage = do_QueryElementAt(copySource->m_messageArray,
+                                                 0, &rv);
                     copySource->m_processed = PR_TRUE;
                 }
                 copyRequest->m_processed = PR_TRUE;
@@ -274,7 +322,27 @@ nsMsgCopyService::FindRequest(nsISupports* aSupport,
   for (i=0; i < cnt; i++)
   {
     copyRequest = (nsCopyRequest*) m_copyRequests.ElementAt(i);
-    if (copyRequest->m_srcSupport.get() == aSupport &&
+    if (copyRequest->m_requestType == nsCopyFoldersType)
+    {
+        // If the src is different then check next request. 
+        if (copyRequest->m_srcSupport.get() != aSupport)
+          continue;
+
+        // See if the parent of the copied folder is the same as the one when the request was made.
+        nsCOMPtr <nsIMsgFolder> parentMsgFolder;
+        nsresult rv = dstFolder->GetParentMsgFolder(getter_AddRefs(parentMsgFolder));
+        if ((NS_FAILED(rv)) || !parentMsgFolder || (copyRequest->m_dstFolder.get() != parentMsgFolder))
+          continue;
+
+        // Now checks if the folder name is the same.
+        nsXPIDLString folderName;
+        rv = dstFolder->GetName(getter_Copies(folderName));
+        if (NS_FAILED(rv))
+          continue;
+        if (copyRequest->m_dstFolderName == folderName)
+          break;
+    }
+    else if (copyRequest->m_srcSupport.get() == aSupport &&
         copyRequest->m_dstFolder.get() == dstFolder)
         break;
     else
@@ -295,47 +363,96 @@ nsMsgCopyService::CopyMessages(nsIMsgFolder* srcFolder, /* UI src folder */
                                nsIMsgWindow* window,
                                PRBool allowUndo)
 {
-    nsCopyRequest* copyRequest;
-    nsCopySource* copySource = nsnull;
-    nsresult rv = NS_ERROR_NULL_POINTER;
-    nsCOMPtr<nsISupportsArray> msgArray;
-    PRUint32 i, cnt;
-    nsCOMPtr<nsIMsgDBHdr> msg;
-    nsCOMPtr<nsIMsgFolder> curFolder;
-    nsCOMPtr<nsISupports> aSupport;
+  NS_ENSURE_ARG_POINTER(srcFolder);
+  NS_ENSURE_ARG_POINTER(messages);
+  NS_ENSURE_ARG_POINTER(dstFolder);
 
-    if (!srcFolder || !messages || !dstFolder) return rv;
+  nsCopyRequest* copyRequest;
+  nsCopySource* copySource = nsnull;
+  nsCOMPtr<nsISupportsArray> msgArray;
+  PRUint32 cnt;
+  nsCOMPtr<nsIMsgDBHdr> msg;
+  nsCOMPtr<nsIMsgFolder> curFolder;
+  nsCOMPtr<nsISupports> aSupport;
+  nsresult rv;
+    
+  // XXX TODO 
+  // JUNK MAIL RELATED
+  // make sure dest folder exists
+  // and has proper flags, before we start copying?
 
-    copyRequest = new nsCopyRequest();
-    if (!copyRequest) return rv;
-    aSupport = do_QueryInterface(srcFolder, &rv);
+  copyRequest = new nsCopyRequest();
+  if (!copyRequest) 
+    return NS_ERROR_OUT_OF_MEMORY;
 
-    rv = copyRequest->Init(nsCopyMessagesType, aSupport, dstFolder, 
-                           isMove, listener, window, allowUndo);
-    if (NS_FAILED(rv)) goto done;
+  aSupport = do_QueryInterface(srcFolder, &rv);
 
-    rv = NS_NewISupportsArray(getter_AddRefs(msgArray));
-    if (NS_FAILED(rv)) goto done;
+  rv = copyRequest->Init(nsCopyMessagesType, aSupport, dstFolder, 
+                         isMove, listener, window, allowUndo);
+  if (NS_FAILED(rv)) 
+    goto done;
 
-    messages->Count(&cnt);
-    copySource = copyRequest->AddNewCopySource(srcFolder);
+  rv = NS_NewISupportsArray(getter_AddRefs(msgArray));
+  if (NS_FAILED(rv)) 
+    goto done;
 
-    for (i=0; i<cnt; i++)
+  messages->Count(&cnt);
+
+  // duplicate the message array so we could sort the messages by it's
+  // folder easily
+  msgArray->AppendElements(messages);
+
+  rv = msgArray->Count(&cnt);
+  if (NS_FAILED(rv)) 
+    goto done;
+
+  while (cnt-- > 0)
+  {
+    msg = do_QueryElementAt(msgArray, cnt, &rv);
+
+    if (NS_FAILED(rv)) 
+      goto done;
+
+    rv = msg->GetFolder(getter_AddRefs(curFolder));
+
+    if (NS_FAILED(rv)) 
+      goto done;
+    if (!copySource)
     {
-      aSupport = getter_AddRefs(messages->ElementAt(i));
-      msg = do_QueryInterface(aSupport, &rv);
-      copySource->AddMessage(msg);
+      copySource = copyRequest->AddNewCopySource(curFolder);
+      if (!copySource)
+      {
+         rv = NS_ERROR_OUT_OF_MEMORY;
+         goto done;
+      }
     }
-    // undo stuff
-    if (NS_SUCCEEDED(rv) && copyRequest->m_allowUndo && copyRequest->m_copySourceArray.Count() > 1 &&
-        copyRequest->m_txnMgr)
-        copyRequest->m_txnMgr->BeginBatch();
+
+    if (curFolder == copySource->m_msgFolder)
+    {
+      copySource->AddMessage(msg);
+      msgArray->RemoveElementAt(cnt);
+    }
+
+    if (cnt == 0)
+    {
+      rv = msgArray->Count(&cnt);
+      if (cnt > 0)
+        copySource = nsnull; // * force to create a new one and
+                             // * continue grouping the messages
+    }
+  }
+
+  // undo stuff
+  if (NS_SUCCEEDED(rv) && copyRequest->m_allowUndo && copyRequest->m_copySourceArray.Count() > 1 &&
+      copyRequest->m_txnMgr)
+    copyRequest->m_txnMgr->BeginBatch();
+
 done:
     
     if (NS_FAILED(rv))
-        delete copyRequest;
+      delete copyRequest;
     else
-        rv = DoCopy(copyRequest);
+      rv = DoCopy(copyRequest);
     
     msgArray->Clear();
 
@@ -404,19 +521,20 @@ nsMsgCopyService::CopyFileMessage(nsIFileSpec* fileSpec,
     nsresult rv = NS_ERROR_NULL_POINTER;
     nsCopyRequest* copyRequest;
     nsCopySource* copySource = nsnull;
-    nsCOMPtr<nsISupports> aSupport;
+    nsCOMPtr<nsISupports> fileSupport;
 	  nsCOMPtr<nsITransactionManager> txnMgr;
 
-    if (!fileSpec || !dstFolder) return rv;
+    NS_ENSURE_ARG_POINTER(fileSpec);
+    NS_ENSURE_ARG_POINTER(dstFolder);
 
 	if (window)
 		window->GetTransactionManager(getter_AddRefs(txnMgr));
   copyRequest = new nsCopyRequest();
   if (!copyRequest) return rv;
-  aSupport = do_QueryInterface(fileSpec, &rv);
+  fileSupport = do_QueryInterface(fileSpec, &rv);
   if (NS_FAILED(rv)) goto done;
 
-  rv = copyRequest->Init(nsCopyFileMessageType, aSupport, dstFolder,
+  rv = copyRequest->Init(nsCopyFileMessageType, fileSupport, dstFolder,
                          isDraft, listener, window, PR_FALSE);
   if (NS_FAILED(rv)) goto done;
 
