@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -16,10 +16,12 @@
  *
  * The Initial Developer of the Original Code is 
  * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2001
+ * Portions created by the Initial Developer are Copyright (C) 2001-2002
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *  Seth Spitzer <sspitzer@netscape.com>
+ *  Dan Mosedale <dmose@netscape.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or 
@@ -57,11 +59,10 @@
 #include "nsIDOMElement.h"
 #include "nsIAtom.h"
 #include "nsIImapIncomingServer.h"
-#include "nsIMsgSearchNotify.h"
 #include "nsIPref.h"
 #include "nsIWeakReference.h"
 #include "nsIObserver.h"
-
+#include "nsIMsgFilterPlugin.h"
 #include "nsIStringBundle.h"
 
 #define MESSENGER_STRING_URL       "chrome://messenger/locale/messenger.properties"
@@ -88,7 +89,9 @@ enum eFieldType {
 // I think this will be an abstract implementation class.
 // The classes that implement the tree support will probably
 // inherit from this class.
-class nsMsgDBView : public nsIMsgDBView, public nsIDBChangeListener, public nsITreeView, public nsIMsgSearchNotify, public nsIObserver
+class nsMsgDBView : public nsIMsgDBView, public nsIDBChangeListener,
+                    public nsITreeView, public nsIObserver,
+                    public nsIJunkMailClassificationListener
 {
 public:
   nsMsgDBView();
@@ -98,8 +101,8 @@ public:
   NS_DECL_NSIMSGDBVIEW
   NS_DECL_NSIDBCHANGELISTENER
   NS_DECL_NSITREEVIEW
-  NS_DECL_NSIMSGSEARCHNOTIFY
   NS_DECL_NSIOBSERVER
+  NS_DECL_NSIJUNKMAILCLASSIFICATIONLISTENER
 
 protected:
   static nsrefcnt gInstanceCount;
@@ -116,6 +119,7 @@ protected:
   static nsIAtom* kHasUnreadAtom;
   static nsIAtom* kWatchThreadAtom;
   static nsIAtom* kIgnoreThreadAtom;
+  static nsIAtom* kHasImageAtom;
 
 #ifdef SUPPORT_PRIORITY_COLORS
   static nsIAtom* kHighestPriorityAtom;
@@ -133,6 +137,9 @@ protected:
   static nsIAtom* kLabelColorWhiteAtom;
   static nsIAtom* kLabelColorBlackAtom;
 
+  static nsIAtom* kJunkMsgAtom;
+  static nsIAtom* kNotJunkMsgAtom;
+
   static PRUnichar* kReadString;
   static PRUnichar* kRepliedString;
   static PRUnichar* kForwardedString;
@@ -143,7 +150,6 @@ protected:
   PRUint32 mNumSelectedRows; // we cache this to determine when to push command status notifications.
   PRPackedBool   mSuppressMsgDisplay; // set when the message pane is collapsed
   PRPackedBool   mSuppressCommandUpdating;
-  PRPackedBool   mIsSearchView; // tells if the search view is loaded.
   PRPackedBool   mRemovingRow; // set when we're telling the outline a row is being removed. used to suppress msg loading.
                         // during delete/move operations.
   PRPackedBool  mCommandsNeedDisablingBecauseOffline;
@@ -216,6 +222,7 @@ protected:
 	virtual nsMsgViewIndex	FindKey(nsMsgKey key, PRBool expand);
 	virtual nsresult GetDBForViewIndex(nsMsgViewIndex index, nsIMsgDatabase **db);
   virtual nsresult GetFolders(nsISupportsArray **folders);
+  virtual nsresult GetFolderFromMsgURI(const char *aMsgURI, nsIMsgFolder **aFolder);
 
   nsresult ListIdsInThread(nsIMsgThread *threadHdr, nsMsgViewIndex viewIndex, PRUint32 *pNumListed);
   nsresult ListUnreadIdsInThread(nsIMsgThread *threadHdr, nsMsgViewIndex startOfThreadViewIndex, PRUint32 *pNumListed);
@@ -240,6 +247,11 @@ protected:
                     PRInt32 numIndices, nsIMsgFolder *destFolder);
   virtual nsresult CopyMessages(nsIMsgWindow *window, nsMsgViewIndex *indices, PRInt32 numIndices, PRBool isMove, nsIMsgFolder *destFolder);
   virtual nsresult DeleteMessages(nsIMsgWindow *window, nsMsgViewIndex *indices, PRInt32 numIndices, PRBool deleteStorage);
+  nsresult SetStringPropertyByIndex(nsMsgViewIndex index, const char *aProperty, const char *aValue);
+  nsresult SetJunkScoreByIndex(nsIJunkMailPlugin *aJunkPlugin, 
+                               nsMsgViewIndex aIndex,
+                               nsMsgJunkStatus aNewClassification,
+                               PRBool aIsLastInBatch);
   nsresult ToggleReadByIndex(nsMsgViewIndex index);
   nsresult SetReadByIndex(nsMsgViewIndex index, PRBool read);
   nsresult SetThreadOfMsgReadByIndex(nsMsgViewIndex index, nsMsgKeyArray &keysMarkedRead, PRBool read);
@@ -284,7 +296,6 @@ protected:
   nsresult MarkThreadRead(nsIMsgThread *threadHdr, nsMsgViewIndex threadIndex, nsMsgKeyArray &idsMarkedRead, PRBool bRead);
   PRBool IsValidIndex(nsMsgViewIndex index);
   nsresult ToggleIgnored(nsMsgViewIndex * indices, PRInt32 numIndices, PRBool *resultToggleState);
-  nsresult GetKeyForFirstSelectedMessage(nsMsgKey *key);
   PRBool OfflineMsgSelected(nsMsgViewIndex * indices, PRInt32 numIndices);
   PRUnichar * GetString(const PRUnichar *aStringName);
   nsresult AddLabelPrefObservers();
@@ -331,7 +342,7 @@ protected:
 
   nsCOMPtr <nsIMsgDatabase> m_db;
   nsCOMPtr <nsIMsgFolder> m_folder;
-  nsWeakPtr m_searchSession;
+  nsCOMPtr <nsIAtom> mRedirectorTypeAtom;
   nsMsgViewSortTypeValue  m_sortType;
   nsMsgViewSortOrderValue m_sortOrder;
   nsMsgViewFlagsTypeValue m_viewFlags;
@@ -351,6 +362,27 @@ protected:
   nsString mLabelPrefColors[PREF_LABELS_MAX];
   // used to cache the atoms created for each color to be displayed
   static nsIAtom* mLabelPrefColorAtoms[PREF_LABELS_MAX];
+
+  // used to know to finish out the junk mail classification batch when the 
+  // last classification callback happens
+  nsCString mLastJunkUriInBatch;
+  PRUint8 mOutstandingJunkBatches;
+
+protected:
+  static nsresult   InitDisplayFormats();
+
+private:
+  static nsDateFormatSelector  m_dateFormatDefault;
+  static nsDateFormatSelector  m_dateFormatThisWeek;
+  static nsDateFormatSelector  m_dateFormatToday;
+  PRBool ServerSupportsFilterAfterTheFact();
+
+  nsMsgKeyArray	mJunkKeys;
+  nsCOMPtr <nsIMsgFolder> mJunkTargetFolder;
+  nsresult PerformActionOnJunkMsgs();
+  nsresult SaveJunkMsgForAction(nsIMsgIncomingServer *aServer, const char *aMsgURI, nsMsgJunkStatus aClassification);
+
+  nsUInt32Array mIndicesToNoteChange;
 };
 
 #endif
