@@ -49,6 +49,7 @@
 #include "nsRDFCID.h"
 #include "nsIRDFNode.h"
 #include "nsEnumeratorUtils.h"
+#include "nsAdapterEnumerator.h"
 
 #include "nsString.h"
 #include "nsCOMPtr.h"
@@ -135,56 +136,56 @@ nsMsgFolderDataSource::nsMsgFolderDataSource()
   nsIRDFService* rdf = getRDFService();
   
   if (gFolderResourceRefCnt++ == 0) {
-    rdf->GetResource(NC_RDF_CHILD,   &kNC_Child);
-    rdf->GetResource(NC_RDF_FOLDER,  &kNC_Folder);
-    rdf->GetResource(NC_RDF_NAME,    &kNC_Name);
-    rdf->GetResource(NC_RDF_OPEN,    &kNC_Open);
-    rdf->GetResource(NC_RDF_FOLDERTREENAME,    &kNC_FolderTreeName);
-    rdf->GetResource(NC_RDF_FOLDERTREESIMPLENAME,    &kNC_FolderTreeSimpleName);
-    rdf->GetResource(NC_RDF_NAME_SORT,    &kNC_NameSort);
-    rdf->GetResource(NC_RDF_FOLDERTREENAME_SORT,    &kNC_FolderTreeNameSort);
-    rdf->GetResource(NC_RDF_SPECIALFOLDER, &kNC_SpecialFolder);
-    rdf->GetResource(NC_RDF_SERVERTYPE, &kNC_ServerType);
-    rdf->GetResource(NC_RDF_REDIRECTORTYPE, &kNC_RedirectorType);
-    rdf->GetResource(NC_RDF_CANCREATEFOLDERSONSERVER, &kNC_CanCreateFoldersOnServer);
-    rdf->GetResource(NC_RDF_CANFILEMESSAGESONSERVER, &kNC_CanFileMessagesOnServer);
-    rdf->GetResource(NC_RDF_ISSERVER, &kNC_IsServer);
-    rdf->GetResource(NC_RDF_ISSECURE, &kNC_IsSecure);
-    rdf->GetResource(NC_RDF_CANSUBSCRIBE, &kNC_CanSubscribe);
-    rdf->GetResource(NC_RDF_SUPPORTSOFFLINE, &kNC_SupportsOffline);
-    rdf->GetResource(NC_RDF_CANFILEMESSAGES, &kNC_CanFileMessages);
-    rdf->GetResource(NC_RDF_CANCREATESUBFOLDERS, &kNC_CanCreateSubfolders);
-    rdf->GetResource(NC_RDF_CANRENAME, &kNC_CanRename);
-    rdf->GetResource(NC_RDF_CANCOMPACT, &kNC_CanCompact);
-    rdf->GetResource(NC_RDF_TOTALMESSAGES, &kNC_TotalMessages);
-    rdf->GetResource(NC_RDF_TOTALUNREADMESSAGES, &kNC_TotalUnreadMessages);
-    rdf->GetResource(NC_RDF_FOLDERSIZE, &kNC_FolderSize);
-    rdf->GetResource(NC_RDF_CHARSET, &kNC_Charset);
-    rdf->GetResource(NC_RDF_BIFFSTATE, &kNC_BiffState);
-    rdf->GetResource(NC_RDF_HASUNREADMESSAGES, &kNC_HasUnreadMessages);
-    rdf->GetResource(NC_RDF_NEWMESSAGES, &kNC_NewMessages);
-    rdf->GetResource(NC_RDF_SUBFOLDERSHAVEUNREADMESSAGES, &kNC_SubfoldersHaveUnreadMessages);
-    rdf->GetResource(NC_RDF_NOSELECT, &kNC_NoSelect);
-    rdf->GetResource(NC_RDF_IMAPSHARED, &kNC_ImapShared);
-    rdf->GetResource(NC_RDF_SYNCHRONIZE, &kNC_Synchronize);
-    rdf->GetResource(NC_RDF_SYNCDISABLED, &kNC_SyncDisabled);
-    rdf->GetResource(NC_RDF_CANSEARCHMESSAGES, &kNC_CanSearchMessages);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_CHILD),   &kNC_Child);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_FOLDER),  &kNC_Folder);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_NAME),    &kNC_Name);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_OPEN),    &kNC_Open);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_FOLDERTREENAME),    &kNC_FolderTreeName);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_FOLDERTREESIMPLENAME),    &kNC_FolderTreeSimpleName);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_NAME_SORT),    &kNC_NameSort);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_FOLDERTREENAME_SORT),    &kNC_FolderTreeNameSort);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_SPECIALFOLDER), &kNC_SpecialFolder);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_SERVERTYPE), &kNC_ServerType);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_REDIRECTORTYPE), &kNC_RedirectorType);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_CANCREATEFOLDERSONSERVER), &kNC_CanCreateFoldersOnServer);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_CANFILEMESSAGESONSERVER), &kNC_CanFileMessagesOnServer);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_ISSERVER), &kNC_IsServer);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_ISSECURE), &kNC_IsSecure);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_CANSUBSCRIBE), &kNC_CanSubscribe);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_SUPPORTSOFFLINE), &kNC_SupportsOffline);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_CANFILEMESSAGES), &kNC_CanFileMessages);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_CANCREATESUBFOLDERS), &kNC_CanCreateSubfolders);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_CANRENAME), &kNC_CanRename);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_CANCOMPACT), &kNC_CanCompact);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_TOTALMESSAGES), &kNC_TotalMessages);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_TOTALUNREADMESSAGES), &kNC_TotalUnreadMessages);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_FOLDERSIZE), &kNC_FolderSize);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_CHARSET), &kNC_Charset);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_BIFFSTATE), &kNC_BiffState);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_HASUNREADMESSAGES), &kNC_HasUnreadMessages);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_NEWMESSAGES), &kNC_NewMessages);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_SUBFOLDERSHAVEUNREADMESSAGES), &kNC_SubfoldersHaveUnreadMessages);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_NOSELECT), &kNC_NoSelect);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_IMAPSHARED), &kNC_ImapShared);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_SYNCHRONIZE), &kNC_Synchronize);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_SYNCDISABLED), &kNC_SyncDisabled);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_CANSEARCHMESSAGES), &kNC_CanSearchMessages);
     
-    rdf->GetResource(NC_RDF_DELETE, &kNC_Delete);
-    rdf->GetResource(NC_RDF_REALLY_DELETE, &kNC_ReallyDelete);
-    rdf->GetResource(NC_RDF_NEWFOLDER, &kNC_NewFolder);
-    rdf->GetResource(NC_RDF_GETNEWMESSAGES, &kNC_GetNewMessages);
-    rdf->GetResource(NC_RDF_COPY, &kNC_Copy);
-    rdf->GetResource(NC_RDF_MOVE, &kNC_Move);
-    rdf->GetResource(NC_RDF_COPYFOLDER, &kNC_CopyFolder);
-    rdf->GetResource(NC_RDF_MOVEFOLDER, &kNC_MoveFolder);
-    rdf->GetResource(NC_RDF_MARKALLMESSAGESREAD,
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_DELETE), &kNC_Delete);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_REALLY_DELETE), &kNC_ReallyDelete);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_NEWFOLDER), &kNC_NewFolder);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_GETNEWMESSAGES), &kNC_GetNewMessages);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_COPY), &kNC_Copy);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_MOVE), &kNC_Move);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_COPYFOLDER), &kNC_CopyFolder);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_MOVEFOLDER), &kNC_MoveFolder);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_MARKALLMESSAGESREAD),
                              &kNC_MarkAllMessagesRead);
-    rdf->GetResource(NC_RDF_COMPACT, &kNC_Compact);
-    rdf->GetResource(NC_RDF_COMPACTALL, &kNC_CompactAll);
-    rdf->GetResource(NC_RDF_RENAME, &kNC_Rename);
-    rdf->GetResource(NC_RDF_EMPTYTRASH, &kNC_EmptyTrash);
-    rdf->GetResource(NC_RDF_DOWNLOADFLAGGED, &kNC_DownloadFlagged);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_COMPACT), &kNC_Compact);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_COMPACTALL), &kNC_CompactAll);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_RENAME), &kNC_Rename);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_EMPTYTRASH), &kNC_EmptyTrash);
+    rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_DOWNLOADFLAGGED), &kNC_DownloadFlagged);
 
     kTotalMessagesAtom           = NS_NewAtom("TotalMessages");
     kTotalUnreadMessagesAtom     = NS_NewAtom("TotalUnreadMessages");
@@ -879,14 +880,12 @@ nsMsgFolderDataSource::OnItemIntPropertyChanged(nsISupports *item,
 	nsCOMPtr<nsIMsgFolder> folder = do_QueryInterface(item);
 	if(folder)
 	{
-
-		if (kTotalMessagesAtom == property)
-			OnTotalMessagePropertyChanged(folder, oldValue, newValue);
-		else if (kTotalUnreadMessagesAtom == property)
-			OnUnreadMessagePropertyChanged(folder, oldValue, newValue);
-                else if (kFolderSizeAtom == property)
-			OnFolderSizePropertyChanged(folder, oldValue, newValue);
-
+    if (kTotalMessagesAtom == property)
+      OnTotalMessagePropertyChanged(folder, oldValue, newValue);
+    else if (kTotalUnreadMessagesAtom == property)
+      OnUnreadMessagePropertyChanged(folder, oldValue, newValue);
+    else if (kFolderSizeAtom == property)
+      OnFolderSizePropertyChanged(folder, oldValue, newValue);
 	}
 	return NS_OK;
 }
