@@ -92,15 +92,14 @@ nsresult createIntNode(PRInt32 value, nsIRDFNode **node, nsIRDFService *rdfServi
 
 nsresult createBlobNode(PRUint8 *value, PRUint32 &length, nsIRDFNode **node, nsIRDFService *rdfService)
 {
+  NS_ENSURE_ARG_POINTER(node);
+  NS_ENSURE_ARG_POINTER(rdfService);
+
   *node = nsnull;
-  nsresult rv; 
-  if (!rdfService) return NS_ERROR_NULL_POINTER;  
   nsCOMPtr<nsIRDFBlob> blob;
-  rv = rdfService->GetBlobLiteral(value, length, getter_AddRefs(blob));
-  if(NS_SUCCEEDED(rv)) {
-    *node = blob;
-    NS_IF_ADDREF(*node);
-  }
+  nsresult rv = rdfService->GetBlobLiteral(value, length, getter_AddRefs(blob));
+  NS_ENSURE_SUCCESS(rv,rv);
+  NS_IF_ADDREF(*node = blob);
   return rv;
 }
 
