@@ -80,6 +80,7 @@ static PRTime gtimeOfLastPurgeCheck;    //variable to know when to check for pur
 nsIAtom* nsMsgDBFolder::mFolderLoadedAtom=nsnull;
 nsIAtom* nsMsgDBFolder::mDeleteOrMoveMsgCompletedAtom=nsnull;
 nsIAtom* nsMsgDBFolder::mDeleteOrMoveMsgFailedAtom=nsnull;
+nsIAtom* nsMsgDBFolder::mJunkStatusChangedAtom=nsnull;
 nsrefcnt nsMsgDBFolder::mInstanceCount=0;
 
 NS_IMPL_ISUPPORTS_INHERITED2(nsMsgDBFolder, nsMsgFolder,
@@ -94,6 +95,7 @@ nsMsgDBFolder::nsMsgDBFolder(void)
     mFolderLoadedAtom = NS_NewAtom("FolderLoaded");
     mDeleteOrMoveMsgCompletedAtom = NS_NewAtom("DeleteOrMoveMsgCompleted");
     mDeleteOrMoveMsgFailedAtom = NS_NewAtom("DeleteOrMoveMsgFailed");
+    mJunkStatusChangedAtom = NS_NewAtom("JunkStatusChanged");
     LL_I2L(gtimeOfLastPurgeCheck, 0);
   }
 }
@@ -104,6 +106,7 @@ nsMsgDBFolder::~nsMsgDBFolder(void)
     NS_IF_RELEASE(mFolderLoadedAtom);
     NS_IF_RELEASE(mDeleteOrMoveMsgCompletedAtom);
     NS_IF_RELEASE(mDeleteOrMoveMsgFailedAtom);
+    NS_IF_RELEASE(mJunkStatusChangedAtom);
   }
 	//shutdown but don't shutdown children.
 	Shutdown(PR_FALSE);
@@ -411,8 +414,6 @@ nsresult nsMsgDBFolder::ReadDBFolderInfo(PRBool force)
 			}
 		}
 	}
-//	if (m_master->InitFolderFromCache (this))
-//		return err;
 
 	if (force || !mInitializedFromCache)
     {
@@ -635,12 +636,13 @@ NS_IMETHODIMP
 nsMsgDBFolder::GetMsgDatabase(nsIMsgWindow *aMsgWindow,
                               nsIMsgDatabase** aMsgDatabase)
 {
-    GetDatabase(aMsgWindow);
-    if (!aMsgDatabase || !mDatabase)
-        return NS_ERROR_NULL_POINTER;
-    *aMsgDatabase = mDatabase;
-    NS_ADDREF(*aMsgDatabase);
-    return NS_OK;
+  GetDatabase(aMsgWindow);
+  
+  if (!aMsgDatabase || !mDatabase)
+    return NS_ERROR_NULL_POINTER;
+  
+  NS_ADDREF(*aMsgDatabase = mDatabase);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -663,10 +665,18 @@ nsMsgDBFolder::SetMsgDatabase(nsIMsgDatabase *aMsgDatabase)
 NS_IMETHODIMP
 nsMsgDBFolder::OnReadChanged(nsIDBChangeListener * aInstigator)
 {
-    /* do nothing.  if you care about this, over ride it.  see nsNewsFolder.cpp */
-    return NS_OK;
+  /* do nothing.  if you care about this, override it.  see nsNewsFolder.cpp */
+  return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMsgDBFolder::OnJunkScoreChanged(nsIDBChangeListener * aInstigator)
+{
+  NotifyFolderEvent(mJunkStatusChangedAtom);
+  return NS_OK;
+}
+
+
 // 1.  When the status of a message changes.
 NS_IMETHODIMP nsMsgDBFolder::OnKeyChange(nsMsgKey aKeyChanged, PRUint32 aOldFlags, PRUint32 aNewFlags, 
                          nsIDBChangeListener * aInstigator)
@@ -1171,7 +1181,8 @@ nsMsgDBFolder::OnStopRunningUrl(nsIURI *aUrl, nsresult aExitCode)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgDBFolder::GetRetentionSettings(nsIMsgRetentionSettings **settings)
+NS_IMETHODIMP 
+nsMsgDBFolder::GetRetentionSettings(nsIMsgRetentionSettings **settings)
 {
   NS_ENSURE_ARG_POINTER(settings);
   nsresult rv = NS_OK;
@@ -1544,7 +1555,7 @@ nsMsgDBFolder::MatchOrChangeFilterDestination(nsIMsgFolder *newFolder, PRBool ca
     rv = accountMgr->GetAllServers(getter_AddRefs(allServers));
     if (NS_SUCCEEDED(rv) && allServers)
     {
-      PRUint32 numServers,
+      PRUint32 numServers;
       rv = allServers->Count(&numServers);
       for (PRUint32 serverIndex=0; serverIndex < numServers; serverIndex++)
       {
@@ -1673,7 +1684,6 @@ nsMsgDBFolder::SpamFilterClassifyMessages(const char **aURIArray, PRUint32 aURIC
 NS_IMETHODIMP
 nsMsgDBFolder::CallFilterPlugins(nsIMsgWindow *aMsgWindow)
 {
-
     nsCOMPtr<nsIMsgIncomingServer> server;
     nsCOMPtr<nsISpamSettings> spamSettings;
     nsCOMPtr<nsIAbMDBDirectory> whiteListDirectory;
@@ -1681,7 +1691,24 @@ nsMsgDBFolder::CallFilterPlugins(nsIMsgWindow *aMsgWindow)
     PRBool useWhiteList = PR_FALSE;
     PRInt32 spamLevel = 0;
     nsXPIDLCString whiteListAbURI;
-    
+ 
+    // if this is the junk folder, or the trash folder
+    // don't analyze for spam, because we don't care
+    //
+    // if it's the sent, unsent, templates, or drafts, 
+    // don't analyze for spam, because the user
+    // created that message
+    //
+    // if it's a public imap folder, or another users
+    // imap folder, don't analyze for spam, because
+    // it's not ours to analyze
+    if (mFlags & (MSG_FOLDER_FLAG_JUNK | MSG_FOLDER_FLAG_TRASH |
+                 MSG_FOLDER_FLAG_NEWSGROUP |MSG_FOLDER_FLAG_SENTMAIL | 
+                 MSG_FOLDER_FLAG_QUEUE |MSG_FOLDER_FLAG_DRAFTS | 
+                 MSG_FOLDER_FLAG_TEMPLATES |MSG_FOLDER_FLAG_IMAP_PUBLIC | 
+                 MSG_FOLDER_FLAG_IMAP_OTHER_USER))
+      return NS_OK;
+
     nsresult rv = GetServer(getter_AddRefs(server));
     NS_ENSURE_SUCCESS(rv, rv); 
     rv = server->GetSpamSettings(getter_AddRefs(spamSettings));
@@ -1692,7 +1719,7 @@ nsMsgDBFolder::CallFilterPlugins(nsIMsgWindow *aMsgWindow)
 
     NS_ENSURE_SUCCESS(rv, rv); 
     spamSettings->GetLevel(&spamLevel);
-    if (spamLevel == 0)
+    if (!spamLevel)
       return NS_OK;
     nsCOMPtr<nsIMsgMailSession> mailSession = 
         do_GetService(NS_MSGMAILSESSION_CONTRACTID, &rv);
