@@ -184,14 +184,15 @@ nsMsgFolder::~nsMsgFolder(void)
 
   gInstanceCount--;
   if (gInstanceCount <= 0) {
-    NS_IF_RELEASE(kTotalMessagesAtom);
     NS_IF_RELEASE(kBiffStateAtom);
     NS_IF_RELEASE(kNewMessagesAtom);
     NS_IF_RELEASE(kNumNewBiffMessagesAtom);
-    NS_IF_RELEASE(kTotalUnreadMessagesAtom);
-    NS_IF_RELEASE(kFlaggedAtom);
-    NS_IF_RELEASE(kStatusAtom);
     NS_IF_RELEASE(kNameAtom);
+    NS_IF_RELEASE(kTotalUnreadMessagesAtom);
+    NS_IF_RELEASE(kTotalMessagesAtom);
+    NS_IF_RELEASE(kFolderSizeAtom);
+    NS_IF_RELEASE(kStatusAtom);
+    NS_IF_RELEASE(kFlaggedAtom);
     NS_IF_RELEASE(kSynchronizeAtom);
     NS_IF_RELEASE(kOpenAtom);
     NS_IF_RELEASE(kCollationKeyGenerator);
@@ -419,14 +420,14 @@ nsMsgFolder::FindSubFolder(const char *aEscapedSubFolderName, nsIFolder **aFolde
   if (NS_FAILED(rv))
     return rv;
 
-// XXX use necko here
+  // XXX use necko here
   nsCAutoString uri;
   uri.Append(mURI);
   uri.Append('/');
   uri.Append(aEscapedSubFolderName);
 
   nsCOMPtr<nsIRDFResource> res;
-  rv = rdf->GetResource(uri.get(), getter_AddRefs(res));
+  rv = rdf->GetResource(uri, getter_AddRefs(res));
   if (NS_FAILED(rv))
     return rv;
 
