@@ -173,7 +173,7 @@ nsMsgIncomingServer::PerformExpand(nsIMsgWindow *aMsgWindow)
 
   
 NS_IMETHODIMP
-nsMsgIncomingServer::PerformBiff()
+nsMsgIncomingServer::PerformBiff(nsIMsgWindow* aMsgWindow)
 {
   //This had to be implemented in the derived class, but in case someone doesn't implement it
   //just return not implemented.
@@ -757,17 +757,15 @@ NS_IMETHODIMP nsMsgIncomingServer::SetPassword(const char * aPassword)
 
 NS_IMETHODIMP nsMsgIncomingServer::GetPassword(char ** aPassword)
 {
-    NS_ENSURE_ARG_POINTER(aPassword);
-
-	*aPassword = ToNewCString(m_password);
-
-    return NS_OK;
+  NS_ENSURE_ARG_POINTER(aPassword);
+  *aPassword = ToNewCString(m_password);
+  return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgIncomingServer::GetServerRequiresPasswordForBiff(PRBool *_retval)
+NS_IMETHODIMP nsMsgIncomingServer::GetServerRequiresPasswordForBiff(PRBool *aServerRequiresPasswordForBiff)
 {
-    if (!_retval) return NS_ERROR_NULL_POINTER;
-	*_retval = PR_TRUE;
+  NS_ENSURE_ARG_POINTER(aServerRequiresPasswordForBiff);
+	*aServerRequiresPasswordForBiff = PR_TRUE;
 	return NS_OK;
 }
 
@@ -1595,7 +1593,7 @@ nsMsgIncomingServer::GetFilterScope(nsMsgSearchScopeValue *filterScope)
 {
    NS_ENSURE_ARG_POINTER(filterScope);
 
-   *filterScope = nsMsgSearchScope::offlineMail;
+   *filterScope = nsMsgSearchScope::offlineMailFilter;
    return NS_OK;
 }
 
@@ -1612,6 +1610,7 @@ nsMsgIncomingServer::GetSearchScope(nsMsgSearchScopeValue *searchScope)
 NS_IMPL_SERVERPREF_STR(nsMsgIncomingServer, Username, "userName");
 NS_IMPL_SERVERPREF_STR(nsMsgIncomingServer, PrefPassword, "password");
 NS_IMPL_SERVERPREF_BOOL(nsMsgIncomingServer, IsSecure, "isSecure");
+NS_IMPL_SERVERPREF_BOOL(nsMsgIncomingServer, UseSecAuth, "useSecAuth");
 NS_IMPL_SERVERPREF_INT(nsMsgIncomingServer, BiffMinutes, "check_time");
 NS_IMPL_SERVERPREF_STR(nsMsgIncomingServer, Type, "type");
 // in 4.x, this was "mail.pop3_gets_new_mail" for pop and 
@@ -1765,17 +1764,9 @@ nsMsgIncomingServer::ConfigureTemporaryReturnReceiptsFilter(nsIMsgFilterList *fi
       newFilter->SetEnabled(enable);
   else if (enable)
   {
-    nsCOMPtr<nsIMsgFolder> rootFolder;
-    rv = GetRootMsgFolder(getter_AddRefs(rootFolder));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    PRUint32 numFolders;
-    nsCOMPtr<nsIMsgFolder> sentFolder;
-
-    rootFolder->GetFoldersWithFlag(MSG_FOLDER_FLAG_SENTMAIL, 1,
-                                   &numFolders,
-                                   getter_AddRefs(sentFolder));
-    if (sentFolder)
+    nsXPIDLCString actionTargetFolderUri;
+    rv = identity->GetFccFolder(getter_Copies(actionTargetFolderUri));
+    if (!actionTargetFolderUri.IsEmpty())
     {
       filterList->CreateFilter(internalReturnReceiptFilterName.get(),
                                getter_AddRefs(newFilter));
@@ -1829,20 +1820,30 @@ nsMsgIncomingServer::ConfigureTemporaryReturnReceiptsFilter(nsIMsgFilterList *fi
         nsCOMPtr<nsIMsgRuleAction> filterAction;
         newFilter->CreateAction(getter_AddRefs(filterAction));
         filterAction->SetType(nsMsgFilterAction::MoveToFolder);
-        nsXPIDLCString actionTargetFolderUri;
-        rv = sentFolder->GetURI(getter_Copies(actionTargetFolderUri));
-        if (NS_SUCCEEDED(rv))
-        {
           filterAction->SetTargetFolderUri(actionTargetFolderUri);
           newFilter->AppendAction(filterAction);
           filterList->InsertFilterAt(0, newFilter);
         }
       }
     }
-  }
   return rv;
 }
 
+NS_IMETHODIMP
+nsMsgIncomingServer::ClearTemporaryReturnReceiptsFilter()
+{
+  if (mFilterList)
+  {
+   nsCOMPtr<nsIMsgFilter> mdnFilter;
+   nsresult rv = mFilterList->GetFilterNamed(
+     NS_LITERAL_STRING("mozilla-temporary-internal-MDN-receipt-filter").get(),
+     getter_AddRefs(mdnFilter));
+   if (NS_SUCCEEDED(rv) && mdnFilter)
+     return mFilterList->RemoveFilter(mdnFilter);
+  }
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsMsgIncomingServer::GetMsgFolderFromURI(nsIMsgFolder *aFolderResource, const char *aURI, nsIMsgFolder **aFolder)
 {
@@ -1906,16 +1907,20 @@ nsMsgIncomingServer::SetSpamSettings(nsISpamSettings *aSpamSettings)
   NS_ENSURE_SUCCESS(rv,rv);
 
   PRBool moveOnSpam;
-  rv = mSpamSettings->GetMoveOnSpam(&moveOnSpam);
-  NS_ENSURE_SUCCESS(rv,rv);
-  rv = SetBoolValue("moveOnSpam", moveOnSpam);
-  NS_ENSURE_SUCCESS(rv,rv);
+  (void)mSpamSettings->GetMoveOnSpam(&moveOnSpam);
+  (void)SetBoolValue("moveOnSpam", moveOnSpam);
 
   PRInt32 moveTargetMode;
-  rv = mSpamSettings->GetMoveTargetMode(&moveTargetMode);
-  NS_ENSURE_SUCCESS(rv,rv);
-  rv = SetIntValue("moveTargetMode", moveTargetMode);
-  NS_ENSURE_SUCCESS(rv,rv);
+  (void)mSpamSettings->GetMoveTargetMode(&moveTargetMode);
+  (void)SetIntValue("moveTargetMode", moveTargetMode);
+
+  PRBool manualMark;
+  (void)mSpamSettings->GetManualMark(&manualMark);
+  (void)SetBoolValue("manualMark", manualMark);
+
+  PRInt32 manualMarkMode;
+  (void)mSpamSettings->GetManualMarkMode(&manualMarkMode);
+  (void)SetIntValue("manualMarkMode", manualMarkMode);
 
   nsXPIDLCString spamActionTargetAccount;
   rv = mSpamSettings->GetActionTargetAccount(getter_Copies(spamActionTargetAccount));
@@ -1933,13 +1938,17 @@ nsMsgIncomingServer::SetSpamSettings(nsISpamSettings *aSpamSettings)
   rv = mSpamSettings->GetSpamFolderURI(getter_Copies(newJunkFolderURI));
   NS_ENSURE_SUCCESS(rv,rv);
 
-  // as the url listener, the spam settings will set the MSG_FOLDER_FLAG_JUNK folder flag
-  // on the junk mail folder, after it is created
-  nsCOMPtr <nsIUrlListener> listener = do_QueryInterface(mSpamSettings, &rv);
-  NS_ENSURE_SUCCESS(rv,rv);
+  // only try to create the junk folder if we are moving junk
+  // and we have a non-empty uri
+  if (moveOnSpam && !newJunkFolderURI.IsEmpty()) {
+    // as the url listener, the spam settings will set the MSG_FOLDER_FLAG_JUNK folder flag
+    // on the junk mail folder, after it is created
+    nsCOMPtr <nsIUrlListener> listener = do_QueryInterface(mSpamSettings, &rv);
+    NS_ENSURE_SUCCESS(rv,rv);
 
-  rv = GetOrCreateFolder(newJunkFolderURI, listener);
-  NS_ENSURE_SUCCESS(rv,rv);
+    rv = GetOrCreateFolder(newJunkFolderURI, listener);
+    NS_ENSURE_SUCCESS(rv,rv);
+  }
 
   PRBool useWhiteList;
   rv = mSpamSettings->GetUseWhiteList(&useWhiteList);
@@ -2011,6 +2020,14 @@ nsMsgIncomingServer::GetSpamSettings(nsISpamSettings **aSpamSettings)
     rv = mSpamSettings->SetMoveTargetMode(moveTargetMode);
     NS_ENSURE_SUCCESS(rv,rv);
     
+    PRBool manualMark;
+    (void)GetBoolValue("manualMark", &manualMark);
+    (void)mSpamSettings->SetManualMark(manualMark);
+
+    PRInt32 manualMarkMode;
+    (void)GetIntValue("manualMarkMode", &manualMarkMode);
+    (void)mSpamSettings->SetManualMarkMode(manualMarkMode);
+
     nsXPIDLCString spamActionTargetAccount;
     rv = GetCharValue("spamActionTargetAccount", getter_Copies(spamActionTargetAccount));
     NS_ENSURE_SUCCESS(rv,rv);
