@@ -185,8 +185,7 @@ nsMsgProtocol::OpenNetworkSocket(nsIURI * aURL, const char *connectionType,
           nsCAutoString spec;
           rv = aURL->GetSpec(spec);
           if (NS_SUCCEEDED(rv)) {
-              static NS_DEFINE_CID(kSTDURLCID, NS_STANDARDURL_CID);    
-              proxyUri = do_CreateInstance(kSTDURLCID, &rv);
+              proxyUri = do_CreateInstance(NS_STANDARDURL_CONTRACTID, &rv);
           }
           if (NS_SUCCEEDED(rv))
               rv = proxyUri->SetSpec(spec);
@@ -679,10 +678,10 @@ nsMsgProtocol::OnTransportStatus(nsITransport *transport, nsresult status,
     }
     else
     {
-    nsCAutoString host;
-    if (m_url)
-      m_url->GetHost(host);
-    mProgressEventSink->OnStatus(this, nsnull, status, NS_ConvertUTF8toUCS2(host).get()); 
+      nsCAutoString host;
+      if (m_url)
+        m_url->GetHost(host);
+      mProgressEventSink->OnStatus(this, nsnull, status, NS_ConvertUTF8toUCS2(host).get()); 
     }
   } 
   return NS_OK;
