@@ -1,25 +1,40 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
+ * Contributor(s):
  *   Pierre Phaneuf <pp@ludusdesign.com>
- */
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 #include "nsCOMPtr.h"
 #include "nsIURL.h"
 #include "nsIEventQueueService.h"
@@ -49,7 +64,6 @@
 #include "nsMsgCopy.h"
 #include "nsNetUtil.h"
 #include "nsMsgMimeCID.h"
-#include "nsIMsgMailSession.h"
 #include "nsIMsgMailNewsUrl.h"
 #include "nsMsgBaseCID.h"
 
@@ -62,8 +76,6 @@ static NS_DEFINE_CID(kPrefCID,            NS_PREF_CID);
 //
 nsMsgDraft::nsMsgDraft()
 {
-	NS_INIT_REFCNT();
-
   mURI = nsnull;
   mMessageService = nsnull;
   mOutType = nsMimeOutput::nsMimeMessageDraftOrTemplate;
@@ -72,11 +84,7 @@ nsMsgDraft::nsMsgDraft()
 
 nsMsgDraft::~nsMsgDraft()
 {
-  if (mMessageService)
-  {
-    ReleaseMessageServiceFromURI(mURI, mMessageService);
     mMessageService = nsnull;
-  }
 
   PR_FREEIF(mURI);
 }
@@ -84,12 +92,9 @@ nsMsgDraft::~nsMsgDraft()
 /* the following macro actually implement addref, release and query interface for our component. */
 NS_IMPL_ISUPPORTS1(nsMsgDraft, nsIMsgDraft)
 
-// stream converter
-static NS_DEFINE_CID(kStreamConverterCID,    NS_MAILNEWS_MIME_STREAM_CONVERTER_CID);
-
 nsresult    
 nsMsgDraft::ProcessDraftOrTemplateOperation(const char *msgURI, nsMimeOutputType aOutType, 
-                                            nsIMsgIdentity * identity, nsIMsgDBHdr **aMsgToReplace)
+                                            nsIMsgIdentity * identity, const char *originalMsgURI, nsIMsgWindow *aMsgWindow)
 {
   nsresult  rv;
 
@@ -103,7 +108,7 @@ nsMsgDraft::ProcessDraftOrTemplateOperation(const char *msgURI, nsMimeOutputType
   if (!mURI)
     return NS_ERROR_OUT_OF_MEMORY;
 
-  rv = GetMessageServiceFromURI(mURI, &mMessageService);
+  rv = GetMessageServiceFromURI(mURI, getter_AddRefs(mMessageService));
   if (NS_FAILED(rv) && !mMessageService)
   {
     return rv;
@@ -113,13 +118,10 @@ nsMsgDraft::ProcessDraftOrTemplateOperation(const char *msgURI, nsMimeOutputType
 
   // Now, we can create a mime parser (nsIStreamConverter)!
   nsCOMPtr<nsIStreamConverter> mimeParser;
-  rv = nsComponentManager::CreateInstance(kStreamConverterCID, 
-                                          NULL, NS_GET_IID(nsIStreamConverter), 
-                                          (void **) getter_AddRefs(mimeParser)); 
-  if (NS_FAILED(rv) || !mimeParser)
+  mimeParser = do_CreateInstance(NS_MAILNEWS_MIME_STREAM_CONVERTER_CONTRACTID, &rv);
+  if (NS_FAILED(rv))
   {
     Release();
-    ReleaseMessageServiceFromURI(mURI, mMessageService);
     mMessageService = nsnull;
 #ifdef NS_DEBUG
     printf("Failed to create MIME stream converter...\n");
@@ -131,16 +133,16 @@ nsMsgDraft::ProcessDraftOrTemplateOperation(const char *msgURI, nsMimeOutputType
   nsCOMPtr<nsIMimeStreamConverter> mimeConverter = do_QueryInterface(mimeParser);
   if (mimeConverter)
   {
-	  mimeConverter->SetMimeOutputType(mOutType);  // Set the type of output for libmime
-      mimeConverter->SetForwardInline(mAddInlineHeaders);
-      mimeConverter->SetIdentity(identity);
+    mimeConverter->SetMimeOutputType(mOutType);  // Set the type of output for libmime
+    mimeConverter->SetForwardInline(mAddInlineHeaders);
+    mimeConverter->SetIdentity(identity);
+    mimeConverter->SetOriginalMsgURI(originalMsgURI);
   }
 
   nsCOMPtr<nsIStreamListener> convertedListener = do_QueryInterface(mimeParser);
   if (!convertedListener)
   {
     Release();
-    ReleaseMessageServiceFromURI(mURI, mMessageService);
     mMessageService = nsnull;
 #ifdef NS_DEBUG
     printf("Unable to get the nsIStreamListener interface from libmime\n");
@@ -149,36 +151,32 @@ nsMsgDraft::ProcessDraftOrTemplateOperation(const char *msgURI, nsMimeOutputType
   }  
 
   nsCOMPtr<nsIURI> aURL;
-  rv = CreateStartupUrl(mURI, getter_AddRefs(aURL));
+  rv = mMessageService->GetUrlForUri(mURI, getter_AddRefs(aURL), aMsgWindow);
+  if (aURL)
+    aURL->SetSpec(nsDependentCString(mURI));
 
-  // HACK: if we are forwarding a message and that message used a charset over ride
-  // (as speciifed in the top most window (assuming the reply originated from that window)
+  // if we are forwarding a message and that message used a charset over ride
   // then use that over ride charset instead of the charset specified in the message
-	nsCOMPtr <nsIMsgMailSession> mailSession = do_GetService(NS_MSGMAILSESSION_CONTRACTID);          
   nsXPIDLString mailCharset;
-  if (mailSession)
+  if (aMsgWindow)
   {
-    nsCOMPtr<nsIMsgWindow>    msgWindow;
-    mailSession->GetTopmostMsgWindow(getter_AddRefs(msgWindow));
-    if (msgWindow)
+    PRBool charsetOverride;
+    if (NS_SUCCEEDED(aMsgWindow->GetCharsetOverride(&charsetOverride)) && charsetOverride)
     {
-      
-      msgWindow->GetMailCharacterSet(getter_Copies(mailCharset));
-      if (mailCharset)
+      if (NS_SUCCEEDED(aMsgWindow->GetMailCharacterSet(getter_Copies(mailCharset))))
       {
         nsCOMPtr<nsIMsgI18NUrl> i18nUrl(do_QueryInterface(aURL));
         if (i18nUrl)
-          i18nUrl->SetCharsetOverRide(mailCharset);
+          (void) i18nUrl->SetCharsetOverRide(mailCharset);
       }
     }
   }
 
   nsCOMPtr<nsIChannel> dummyChannel;
-  rv = NS_NewInputStreamChannel(getter_AddRefs(dummyChannel), aURL, nsnull, nsnull, -1);
+  rv = NS_NewInputStreamChannel(getter_AddRefs(dummyChannel), aURL, nsnull);
   if (NS_FAILED(mimeParser->AsyncConvertData(nsnull, nsnull, nsnull, dummyChannel)))
   {
     Release();
-    ReleaseMessageServiceFromURI(mURI, mMessageService);
     mMessageService = nsnull;
 #ifdef NS_DEBUG
     printf("Unable to set the output stream for the mime parser...\ncould be failure to create internal libmime data\n");
@@ -187,14 +185,9 @@ nsMsgDraft::ProcessDraftOrTemplateOperation(const char *msgURI, nsMimeOutputType
     return NS_ERROR_UNEXPECTED;
   }
 
-  // Make sure we return this if requested!
-  if (aMsgToReplace)
-     GetMsgDBHdrFromURI(msgURI, aMsgToReplace);
-
   // Now, just plug the two together and get the hell out of the way!
-  rv = mMessageService->DisplayMessage(mURI, convertedListener, nsnull, nsnull, mailCharset, nsnull);
+  rv = mMessageService->DisplayMessage(mURI, convertedListener, aMsgWindow, nsnull, mailCharset, nsnull);
 
-  ReleaseMessageServiceFromURI(mURI, mMessageService);
   mMessageService = nsnull;
   Release();
 
@@ -205,8 +198,8 @@ nsMsgDraft::ProcessDraftOrTemplateOperation(const char *msgURI, nsMimeOutputType
 }
 
 nsresult
-nsMsgDraft::OpenDraftMsg(const char *msgURI, nsIMsgDBHdr **aMsgToReplace,
-                         nsIMsgIdentity * identity, PRBool addInlineHeaders)
+nsMsgDraft::OpenDraftMsg(const char *msgURI, const char *originalMsgURI, nsIMsgIdentity * identity,
+                         PRBool addInlineHeaders, nsIMsgWindow *aMsgWindow)
 {
   // We should really never get here, but if we do, just return 
   // with an error
@@ -215,14 +208,13 @@ nsMsgDraft::OpenDraftMsg(const char *msgURI, nsIMsgDBHdr **aMsgToReplace,
   
   mAddInlineHeaders = addInlineHeaders;
   return ProcessDraftOrTemplateOperation(msgURI, nsMimeOutput::nsMimeMessageDraftOrTemplate, 
-                                         identity, aMsgToReplace);
+                                         identity, originalMsgURI, aMsgWindow);
 }
 
 nsresult
-nsMsgDraft::OpenEditorTemplate(const char *msgURI, nsIMsgDBHdr **aMsgToReplace,
-							   nsIMsgIdentity * identity)
+nsMsgDraft::OpenEditorTemplate(const char *msgURI, nsIMsgIdentity * identity, nsIMsgWindow *aMsgWindow)
 {
   return ProcessDraftOrTemplateOperation(msgURI, nsMimeOutput::nsMimeMessageEditorTemplate, 
-                                         identity, aMsgToReplace);
+                                         identity, nsnull, aMsgWindow);
 }
 
