@@ -71,21 +71,8 @@
 #include "nsMsgSimulateError.h"
 #include "nsNetUtil.h"
 #include "nsIPrefService.h"
-
+#include "nsISignatureVerifier.h"
 #include "nsISSLSocketControl.h"
-/* sigh, cmtcmn.h, included from nsIPSMSocketInfo.h, includes windows.h, which includes winuser.h,
-   which defines PostMessage to be either PostMessageA or PostMessageW... of course it does this
-   without using parameters, so any use of PostMessage now becomes PostMessageA...
-   since this file is XP and doesn't need windows.h, i'm going to just undef this out for now
-
-   when someone comes up with a better solution, please let me know.
-
-   Stuart Parmenter <pavlov@netscape.com>
-*/
-#ifdef PostMessage
-#undef PostMessage
-#endif
-
 
 #ifndef XP_UNIX
 #include <stdarg.h>
@@ -244,9 +231,9 @@ nsSmtpProtocol::nsSmtpProtocol(nsIURI * aURL)
 nsSmtpProtocol::~nsSmtpProtocol()
 {
 	// free our local state
-	PR_FREEIF(m_addressCopy);
-	PR_FREEIF(m_verifyAddress);
-	PR_FREEIF(m_dataBuf);
+	PR_Free(m_addressCopy);
+	PR_Free(m_verifyAddress);
+	PR_Free(m_dataBuf);
 }
 
 void nsSmtpProtocol::Initialize(nsIURI * aURL)
@@ -255,6 +242,7 @@ void nsSmtpProtocol::Initialize(nsIURI * aURL)
     nsresult rv = NS_OK;
 
     m_flags = 0;
+    m_origAuthFlags = 0;
     m_prefAuthMethod = PREF_AUTH_NONE;
     m_usernamePrompted = PR_FALSE;
     m_prefTrySSL = PREF_SSL_TRY;
@@ -365,22 +353,25 @@ const char * nsSmtpProtocol::GetUserDomainName()
 // we suppport the nsIStreamListener interface 
 ////////////////////////////////////////////////////////////////////////////////////////////
 
-// stop binding is a "notification" informing us that the stream associated with aURL is going away. 
-NS_IMETHODIMP nsSmtpProtocol::OnStopRequest(nsIRequest *request, nsISupports *ctxt, nsresult aStatus)
+// stop binding is a "notification" informing us that the stream
+// associated with aURL is going away. 
+NS_IMETHODIMP nsSmtpProtocol::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
+nsresult aStatus)
 {
-  if (aStatus == NS_OK && m_totalAmountRead == 0) {
+  if (aStatus == NS_OK && m_nextState != SMTP_FREE) {
     // if we are getting OnStopRequest() with NS_OK, 
-    // but we haven't read any bytes, that's spells trouble.
-    // it means that the server has dropped us before we could read anything
-    // for example, see bug #158059
-    PR_LOG(SMTPLogModule, PR_LOG_ALWAYS, ("SMTP connection closed, but no data read, so report error"));
-    nsMsgAsyncWriteProtocol::OnStopRequest(nsnull, ctxt, NS_ERROR_CONNECTION_REFUSED);
+    // but we haven't finished clean, that's spells trouble.
+    // it means that the server has dropped us before we could send the whole mail
+    // for example, see bug #200647
+    PR_LOG(SMTPLogModule, PR_LOG_ALWAYS,
+ ("SMTP connection dropped after %ld total bytes read", m_totalAmountRead));
+    nsMsgAsyncWriteProtocol::OnStopRequest(nsnull, ctxt, NS_ERROR_NET_INTERRUPT);
   }
-	else
+  else
     nsMsgAsyncWriteProtocol::OnStopRequest(nsnull, ctxt, aStatus);
 
-	// okay, we've been told that the send is done and the connection is going away. So 
-	// we need to release all of our state
+  // okay, we've been told that the send is done and the connection is going away. So 
+  // we need to release all of our state
   return nsMsgAsyncWriteProtocol::CloseSocket();
 }
 
@@ -556,7 +547,6 @@ PRInt32 nsSmtpProtocol::ExtensionLoginResponse(nsIInputStream * inputStream, PRU
   return(status);
 }
     
-
 PRInt32 nsSmtpProtocol::SendHeloResponse(nsIInputStream * inputStream, PRUint32 length)
 {
 	PRInt32 status = 0;
@@ -663,19 +653,34 @@ PRInt32 nsSmtpProtocol::SendEhloResponse(nsIInputStream * inputStream, PRUint32
 
     if (m_responseCode != 250)
     {
-      /* EHLO must not be implemented by the server so fall back to the HELO case */
-
-        if (m_prefTrySSL == PREF_SSL_ALWAYS)
+        /* EHLO must not be implemented by the server so fall back to the HELO case */
+        if (m_responseCode >= 500 && m_responseCode < 550)
         {
-            m_nextState = SMTP_ERROR_DONE;
-            m_urlErrorState = NS_ERROR_COULD_NOT_LOGIN_TO_SMTP_SERVER;
+            if (m_prefTrySSL == PREF_SSL_ALWAYS)
+            {
+                m_nextState = SMTP_ERROR_DONE;
+                m_urlErrorState = NS_ERROR_COULD_NOT_LOGIN_TO_SMTP_SERVER;
+                return(NS_ERROR_COULD_NOT_LOGIN_TO_SMTP_SERVER);
+            }
+
+            buffer = "HELO ";
+            buffer += GetUserDomainName();
+            buffer += CRLF;
+            status = SendData(url, buffer.get());
+        }
+        // e.g. getting 421 "Server says unauthorized, bye"
+        else
+        {
+#ifdef DEBUG
+            nsresult rv = 
+#endif
+            nsExplainErrorDetails(m_runningURL,
+                          NS_ERROR_SMTP_SERVER_ERROR, m_responseText.get());
+            NS_ASSERTION(NS_SUCCEEDED(rv), "failed to explain SMTP error");
+
+            m_urlErrorState = NS_ERROR_BUT_DONT_SHOW_ALERT;
             return(NS_ERROR_COULD_NOT_LOGIN_TO_SMTP_SERVER);
         }
-        buffer = "HELO ";
-        buffer += GetUserDomainName();
-        buffer += CRLF;
-
-        status = SendData(url, buffer.get());
 
         m_nextState = SMTP_RESPONSE;
         m_nextStateAfterResponse = SMTP_SEND_HELO_RESPONSE;
@@ -710,8 +715,20 @@ PRInt32 nsSmtpProtocol::SendEhloResponse(nsIInputStream * inputStream, PRUint32
             if (m_responseText.Find("LOGIN", PR_TRUE, 5) >= 0)  
                 SetFlag(SMTP_AUTH_LOGIN_ENABLED);
 
-            if (m_responseText.Find("EXTERNAL", PR_TRUE, 5) >= 0)  
+            if (m_responseText.Find("EXTERNAL", PR_TRUE, 8) >= 0)  
                 SetFlag(SMTP_AUTH_EXTERNAL_ENABLED);
+
+            if (m_responseText.Find("CRAM-MD5", PR_TRUE, 8) >= 0)
+            {
+                nsresult rv;
+                nsCOMPtr<nsISignatureVerifier> verifier = do_GetService(SIGNATURE_VERIFIER_CONTRACTID, &rv);
+                // this checks if psm is installed...
+                if (NS_SUCCEEDED(rv))
+                  SetFlag(SMTP_AUTH_CRAM_MD5_ENABLED);
+            }
+
+            // for use after mechs disabled fallbacks when login failed
+            BackupAuthFlags();
         }
         startPos = endPos + 1;
     } while (endPos >= 0);
@@ -788,15 +805,18 @@ PRInt32 nsSmtpProtocol::ProcessAuth()
             m_urlErrorState = NS_ERROR_COULD_NOT_LOGIN_TO_SMTP_SERVER;
             return(NS_ERROR_COULD_NOT_LOGIN_TO_SMTP_SERVER);
         }
+
         if ((TestFlag(SMTP_AUTH_PLAIN_ENABLED) ||
+            TestFlag(SMTP_AUTH_CRAM_MD5_ENABLED) ||
              TestFlag(SMTP_AUTH_LOGIN_ENABLED)) &&
             (m_prefAuthMethod == PREF_AUTH_ANY))
         {
             m_nextState = SMTP_SEND_AUTH_LOGIN_USERNAME;
             m_nextStateAfterResponse = SMTP_AUTH_LOGIN_RESPONSE;
-            return NS_OK;
         }
-        m_nextState = SMTP_SEND_HELO_RESPONSE;
+        else
+            m_nextState = SMTP_SEND_HELO_RESPONSE;
+
         return NS_OK;
     }
     else // TLS enabled
@@ -811,21 +831,33 @@ PRInt32 nsSmtpProtocol::ProcessAuth()
             SetFlag(SMTP_PAUSE_FOR_READ);
             return NS_OK;
         }
-        else if (TestFlag(SMTP_AUTH_LOGIN_ENABLED) ||
-                 TestFlag(SMTP_AUTH_PLAIN_ENABLED))
+        else if ((TestFlag(SMTP_AUTH_LOGIN_ENABLED) ||
+                 TestFlag(SMTP_AUTH_PLAIN_ENABLED) ||
+                 TestFlag(SMTP_AUTH_CRAM_MD5_ENABLED)) &&
+                 m_prefAuthMethod == PREF_AUTH_ANY)
         {
             m_nextState = SMTP_SEND_AUTH_LOGIN_USERNAME;
             m_nextStateAfterResponse = SMTP_AUTH_LOGIN_RESPONSE;
-            return NS_OK;
         }
         else
-        {
             m_nextState = SMTP_SEND_HELO_RESPONSE;
-            return NS_OK;
-        }
+
+        return NS_OK;
     }
 }
 
+
+void nsSmtpProtocol::BackupAuthFlags()
+{
+  m_origAuthFlags = m_flags & SMTP_AUTH_ANY_ENABLED;
+}
+
+void nsSmtpProtocol::RestoreAuthFlags()
+{
+  m_flags |= m_origAuthFlags;
+}
+
+
 PRInt32 nsSmtpProtocol::AuthLoginResponse(nsIInputStream * stream, PRUint32 length)
 {
   PRInt32 status = 0;
@@ -836,7 +868,7 @@ PRInt32 nsSmtpProtocol::AuthLoginResponse(nsIInputStream * stream, PRUint32 leng
   {
   case 2:
       m_nextState = SMTP_SEND_HELO_RESPONSE;
-	break;
+      break;
   case 3:
       m_nextState = SMTP_SEND_AUTH_LOGIN_PASSWORD;
       break;
@@ -844,18 +876,41 @@ PRInt32 nsSmtpProtocol::AuthLoginResponse(nsIInputStream * stream, PRUint32 leng
   default:
       if (smtpServer)
       {
-          // only forget the password if we didn't get here from the redirection.
-          if (mLogonCookie.IsEmpty()) {
-              smtpServer->ForgetPassword();
-              if (m_usernamePrompted)
-                  smtpServer->SetUsername("");
-          }
-          m_nextState = SMTP_SEND_AUTH_LOGIN_USERNAME;
+        // If one authentication failed, we're going to
+        // fall back on a less secure login method.
+        if(TestFlag(SMTP_AUTH_DIGEST_MD5_ENABLED))
+          // if DIGEST-MD5 enabled, clear it if we failed.
+          ClearFlag(SMTP_AUTH_DIGEST_MD5_ENABLED);
+        else
+        if(TestFlag(SMTP_AUTH_CRAM_MD5_ENABLED))
+          // if CRAM-MD5 enabled, clear it if we failed. 
+          ClearFlag(SMTP_AUTH_CRAM_MD5_ENABLED);
+        else
+        if(TestFlag(SMTP_AUTH_PLAIN_ENABLED))
+          // if PLAIN enabled, clear it if we failed. 
+          ClearFlag(SMTP_AUTH_PLAIN_ENABLED);
+        else
+        if(TestFlag(SMTP_AUTH_LOGIN_ENABLED))
+          // if LOGIN enabled, clear it if we failed. 
+          ClearFlag(SMTP_AUTH_LOGIN_ENABLED);
+
+        // Only forget the password if we didn't get here from the redirection
+        // and if we've no mechanism left.
+        if (!TestFlag(SMTP_AUTH_ANY_ENABLED) && mLogonCookie.IsEmpty())
+        {
+            smtpServer->ForgetPassword();
+            if (m_usernamePrompted)
+                smtpServer->SetUsername("");
+
+            // Let's restore the original auth flags from SendEhloResponse so we can
+            // try them again with new password and username
+            RestoreAuthFlags();
+        }
+
+        m_nextState = SMTP_SEND_AUTH_LOGIN_USERNAME;
       }
       else
-      {
           status = NS_ERROR_SMTP_PASSWORD_UNDEFINED;
-      }
       break;
   }
   
@@ -917,7 +972,10 @@ PRInt32 nsSmtpProtocol::AuthLoginUsername()
                           strlen((const char*)username), nsnull);
   } 
   if (base64Str) {
-	  if (TestFlag(SMTP_AUTH_PLAIN_ENABLED))
+		// if cram md5 available, let's use it.
+		if (TestFlag(SMTP_AUTH_CRAM_MD5_ENABLED))
+		  PR_snprintf(buffer, sizeof(buffer), "AUTH CRAM-MD5" CRLF);
+	  else if (TestFlag(SMTP_AUTH_PLAIN_ENABLED))
 		  PR_snprintf(buffer, sizeof(buffer), "AUTH PLAIN %.256s" CRLF, base64Str);
 	  else if (TestFlag(SMTP_AUTH_LOGIN_ENABLED))
 		  PR_snprintf(buffer, sizeof(buffer), "AUTH LOGIN %.256s" CRLF, base64Str);
@@ -962,11 +1020,52 @@ PRInt32 nsSmtpProtocol::AuthLoginPassword()
 
   if (!password.IsEmpty()) 
   {
-    char *base64Str = PL_Base64Encode(password.get(), password.Length(), nsnull);
-
     char buffer[512];
-    PR_snprintf(buffer, sizeof(buffer), "%.256s" CRLF, base64Str);
-    nsCRT::free(base64Str);
+    if (TestFlag(SMTP_AUTH_CRAM_MD5_ENABLED))
+    {
+      unsigned char digest[DIGEST_LENGTH];
+      char * decodedChallenge = PL_Base64Decode(m_responseText.get(), 
+        m_responseText.Length() - 2 /* subtract CRLF */, nsnull);
+      
+      if (decodedChallenge)
+        rv = MSGCramMD5(decodedChallenge, strlen(decodedChallenge), password.get(), password.Length(), digest);
+      else
+        rv = NS_ERROR_FAILURE;
+      
+      PR_Free(decodedChallenge);
+      if (NS_SUCCEEDED(rv) && digest)
+      {
+        nsCAutoString encodedDigest;
+        char hexVal[8];
+        
+        for (PRUint32 j=0; j<16; j++) 
+        {
+          PR_snprintf (hexVal,8, "%.2x", 0x0ff & (unsigned short)digest[j]);
+          encodedDigest.Append(hexVal); 
+        }
+        
+        nsCOMPtr<nsISmtpServer> smtpServer;
+        rv = m_runningURL->GetSmtpServer(getter_AddRefs(smtpServer));
+        if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
+  
+        nsXPIDLCString userName;
+        rv = smtpServer->GetUsername(getter_Copies(userName));
+
+        PR_snprintf(buffer, sizeof(buffer), "%s %s", userName.get(), encodedDigest.get());
+        char *base64Str = PL_Base64Encode(buffer, strlen(buffer), nsnull);
+        PR_snprintf(buffer, sizeof(buffer), "%s" CRLF, base64Str);
+        PR_Free(base64Str);
+      }
+      if (NS_FAILED(rv))
+        ClearFlag(SMTP_AUTH_CRAM_MD5_ENABLED);
+    }
+		else
+		{
+			char *base64Str = PL_Base64Encode(password.get(), password.Length(), nsnull);
+
+			PR_snprintf(buffer, sizeof(buffer), "%.256s" CRLF, base64Str);
+			nsCRT::free(base64Str);
+		}
     nsCOMPtr<nsIURI> url = do_QueryInterface(m_runningURL);
     status = SendData(url, buffer, PR_TRUE);
     m_nextState = SMTP_RESPONSE;
@@ -1073,11 +1172,13 @@ PRInt32 nsSmtpProtocol::SendRecipientResponse()
 	{
 		/* more senders to RCPT to 
 		 */
+        // fake to 250 because SendMailResponse() can't handle 251
+        m_responseCode = 250;
         m_nextState = SMTP_SEND_MAIL_RESPONSE; 
 		return(0);
 	}
 
-    /* else send the RCPT TO: command */
+    /* else send the DATA command */
 	buffer = "DATA";
 	buffer += CRLF;
     nsCOMPtr<nsIURI> url = do_QueryInterface(m_runningURL);  
@@ -1189,7 +1290,9 @@ PRInt32 nsSmtpProtocol::SendMessageInFile()
   nsCOMPtr<nsIURI> url = do_QueryInterface(m_runningURL);
 	m_runningURL->GetPostMessageFile(getter_AddRefs(fileSpec));
 	if (url && fileSpec)
-        PostMessage(url, fileSpec);
+        // need to fully qualify to avoid getting overwritten by a #define
+        // in some windows header file
+        nsMsgAsyncWriteProtocol::PostMessage(url, fileSpec);
 
 	SetFlag(SMTP_PAUSE_FOR_READ);
 
