@@ -41,6 +41,7 @@
 #include "nsReadableUtils.h"
 #include "nsIPref.h"
 #include "nsIIOService.h"
+#include "nsIPipe.h"
 #include "nsNetCID.h"
 #include "nsEscape.h"
 #include "nsNetUtil.h"
@@ -65,6 +66,7 @@
 #define SERVER_DELIMITER ","
 #define APPEND_SERVERS_VERSION_PREF_NAME "append_preconfig_smtpservers.version"
 #define MAIL_ROOT_PREF "mail."
+#define PREF_MAIL_SMTPSERVERS "mail.smtpservers"
 #define PREF_MAIL_SMTPSERVERS_APPEND_SERVERS "mail.smtpservers.appendsmtpservers"
 
 typedef struct _findServerByKeyEntry {
@@ -80,7 +82,6 @@ typedef struct _findServerByHostnameEntry {
 
 static NS_DEFINE_CID(kCSmtpUrlCID, NS_SMTPURL_CID);
 static NS_DEFINE_CID(kCMailtoUrlCID, NS_MAILTOURL_CID);
-static NS_DEFINE_CID(kSmtpServiceCID, NS_SMTPSERVICE_CID); 
 static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID); 
 
 
@@ -159,7 +160,6 @@ nsresult NS_MsgLoadSmtpUrl(nsIURI * aUrl, nsISupports * aConsumer, nsIRequest **
 nsSmtpService::nsSmtpService() :
     mSmtpServersLoaded(PR_FALSE)
 {
-    NS_INIT_REFCNT();
     NS_NewISupportsArray(getter_AddRefs(mSmtpServers));
 }
 
@@ -279,7 +279,7 @@ nsresult NS_MsgBuildSmtpUrl(nsIFileSpec * aFilePath,
             nsCOMPtr<nsIAuthPrompt> smtpAuthPrompt(do_GetInterface(aNotificationCallbacks));
             if (!smtpPrompt || !smtpAuthPrompt)
             {
-                nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
+                nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
                 if (wwatch) {
                     if (!smtpPrompt)
                         wwatch->GetNewPrompter(0, getter_AddRefs(smtpPrompt));
@@ -357,209 +357,6 @@ NS_IMETHODIMP nsSmtpService::GetProtocolFlags(PRUint32 *result)
     return NS_OK; 	
 }
 
-//////////////////////////////////////////////////////////////////////////
-// This is just a little stub channel class for mailto urls. Mailto urls
-// don't really have any data for the stream calls in nsIChannel to make much sense.
-// But we need to have a channel to return for nsSmtpService::NewChannel
-// that can simulate a real channel such that the uri loader can then get the
-// content type for the channel.
-class nsMailtoChannel : public nsIChannel
-{
-public:
-
-	  NS_DECL_ISUPPORTS
-    NS_DECL_NSICHANNEL
-    NS_DECL_NSIREQUEST
-
-    nsMailtoChannel(nsIURI * aURI);
-	  virtual ~nsMailtoChannel();
-
-protected:
-  nsCOMPtr<nsIURI> m_url;
-  nsresult mStatus;
-};
-
-nsMailtoChannel::nsMailtoChannel(nsIURI * aURI)
-    : m_url(aURI), mStatus(NS_OK)
-{
-  NS_INIT_ISUPPORTS();
-}
-
-nsMailtoChannel::~nsMailtoChannel()
-{}
-
-NS_IMPL_ISUPPORTS2(nsMailtoChannel, nsIChannel, nsIRequest);
-
-NS_IMETHODIMP nsMailtoChannel::GetLoadGroup(nsILoadGroup * *aLoadGroup)
-{
-    *aLoadGroup = nsnull;
-    return NS_OK;
-}
-
-NS_IMETHODIMP nsMailtoChannel::SetLoadGroup(nsILoadGroup * aLoadGroup)
-{
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsMailtoChannel::GetNotificationCallbacks(nsIInterfaceRequestor* *aNotificationCallbacks)
-{
-    NS_NOTREACHED("GetNotificationCallbacks");
-	return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP nsMailtoChannel::SetNotificationCallbacks(nsIInterfaceRequestor* aNotificationCallbacks)
-{
-	return NS_OK;       // don't fail when trying to set this
-}
-
-NS_IMETHODIMP nsMailtoChannel::GetOriginalURI(nsIURI* *aURI)
-{
-    *aURI = nsnull;
-    return NS_OK; 
-}
- 
-NS_IMETHODIMP nsMailtoChannel::SetOriginalURI(nsIURI* aURI)
-{
-  return NS_OK;
-}
- 
-NS_IMETHODIMP nsMailtoChannel::GetURI(nsIURI* *aURI)
-{
-  *aURI = m_url;
-  NS_IF_ADDREF(*aURI);
-  return NS_OK; 
-}
- 
-NS_IMETHODIMP nsMailtoChannel::Open(nsIInputStream **_retval)
-{
-  NS_NOTREACHED("Open");
-	return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP nsMailtoChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctxt)
-{
-  mStatus = listener->OnStartRequest(this, ctxt);
-
-  // If OnStartRequest(...) failed, then propagate the error code...
-  if (NS_SUCCEEDED(mStatus)) {
-    // Otherwise, indicate that no content is available...
-    mStatus = NS_ERROR_NO_CONTENT;
-  }
-
-  // Call OnStopRequest(...) for correct-ness.
-  (void) listener->OnStopRequest(this, ctxt, mStatus);
-
-  // Always return NS_ERROR_NO_CONTENT since this channel never provides
-  // data...
-  return NS_ERROR_NO_CONTENT;
-}
-
-NS_IMETHODIMP nsMailtoChannel::GetLoadFlags(nsLoadFlags *aLoadFlags)
-{
-  *aLoadFlags = 0;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsMailtoChannel::SetLoadFlags(nsLoadFlags aLoadFlags)
-{
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsMailtoChannel::GetContentType(nsACString &aContentType)
-{
-	aContentType = NS_LITERAL_CSTRING("x-application-mailto");
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsMailtoChannel::SetContentType(const nsACString &aContentType)
-{
-    // Do not allow the content type to change...
-    return NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP nsMailtoChannel::GetContentCharset(nsACString &aContentCharset)
-{
-	aContentCharset.Truncate();
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsMailtoChannel::SetContentCharset(const nsACString &aContentCharset)
-{
-    // Do not allow the content type to change...
-    return NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP nsMailtoChannel::GetContentLength(PRInt32 * aContentLength)
-{
-  *aContentLength = -1;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMailtoChannel::SetContentLength(PRInt32 aContentLength)
-{
-    NS_NOTREACHED("SetContentLength");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP nsMailtoChannel::GetOwner(nsISupports * *aPrincipal)
-{
-    NS_NOTREACHED("GetOwner");
-	return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP nsMailtoChannel::SetOwner(nsISupports * aPrincipal)
-{
-    NS_NOTREACHED("SetOwner");
-	return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-/* readonly attribute nsISupports securityInfo; */
-NS_IMETHODIMP nsMailtoChannel::GetSecurityInfo(nsISupports * *aSecurityInfo)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-// From nsIRequest
-////////////////////////////////////////////////////////////////////////////////
-
-/* readonly attribute wstring name; */
-NS_IMETHODIMP nsMailtoChannel::GetName(nsACString &aName)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP nsMailtoChannel::IsPending(PRBool *result)
-{
-    *result = PR_TRUE;
-    return NS_OK; 
-}
-
-NS_IMETHODIMP nsMailtoChannel::GetStatus(nsresult *status)
-{
-    *status = mStatus;
-    return NS_OK;
-}
-
-NS_IMETHODIMP nsMailtoChannel::Cancel(nsresult status)
-{
-    mStatus = status;
-    return NS_OK;
-}
-
-NS_IMETHODIMP nsMailtoChannel::Suspend()
-{
-    NS_NOTREACHED("Suspend");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP nsMailtoChannel::Resume()
-{
-    NS_NOTREACHED("Resume");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 // the smtp service is also the protocol handler for mailto urls....
 
 NS_IMETHODIMP nsSmtpService::NewURI(const nsACString &aSpec,
@@ -591,15 +388,17 @@ NS_IMETHODIMP nsSmtpService::NewURI(const nsACString &aSpec,
 
 NS_IMETHODIMP nsSmtpService::NewChannel(nsIURI *aURI, nsIChannel **_retval)
 {
-  nsresult rv = NS_OK;
-  nsMailtoChannel * aMailtoChannel = new nsMailtoChannel(aURI);
-  if (aMailtoChannel)
-  {
-      rv = aMailtoChannel->QueryInterface(NS_GET_IID(nsIChannel), (void **) _retval);
-  }
-  else
-    rv = NS_ERROR_OUT_OF_MEMORY;
-  return rv;
+  // create an empty pipe for use with the input stream channel.
+  nsCOMPtr<nsIInputStream> pipeIn;
+  nsCOMPtr<nsIOutputStream> pipeOut;
+  nsresult rv = NS_NewPipe(getter_AddRefs(pipeIn),
+                           getter_AddRefs(pipeOut));
+  if (NS_FAILED(rv)) return rv;
+
+  pipeOut->Close();
+
+  return NS_NewInputStreamChannel(_retval, aURI, pipeIn,
+                                  NS_LITERAL_CSTRING("x-application-mailto"));
 }
 
 
@@ -631,10 +430,40 @@ nsSmtpService::loadSmtpServers()
     nsresult rv;
     nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
     if (NS_FAILED(rv)) return rv;
-    
-    nsXPIDLCString serverList;
-    rv = prefs->CopyCharPref("mail.smtpservers", getter_Copies(serverList));
 
+    nsXPIDLCString tempServerList;
+    nsXPIDLCString serverList;
+    rv = prefs->CopyCharPref(PREF_MAIL_SMTPSERVERS, getter_Copies(tempServerList));
+
+    //Get the pref in a tempServerList and then parse it to see if it has dupes.
+    //if so remove the dupes and then create the serverList.
+    if (!tempServerList.IsEmpty()) {
+
+      // Tokenize the data and add each smtp server if it is not already there 
+      // in the user's current smtp server list
+      char *tempSmtpServerStr;
+      char *tempSmtpServersStr = nsCRT::strdup(tempServerList.get());
+      char *tempToken = nsCRT::strtok(tempSmtpServersStr, SERVER_DELIMITER, &tempSmtpServerStr);
+
+      nsCAutoString tempSmtpServer;
+      while (tempToken) {
+        if (*tempToken) {
+          if (serverList.IsEmpty() || !strstr(serverList.get(), tempToken)) {
+            tempSmtpServer.Assign(tempToken);
+            tempSmtpServer.StripWhitespace();
+            if (!serverList.IsEmpty())
+              serverList += SERVER_DELIMITER;
+            serverList += tempSmtpServer;
+          }
+        }
+        tempToken = nsCRT::strtok(tempSmtpServerStr, SERVER_DELIMITER, &tempSmtpServerStr);
+      }
+      nsCRT::free(tempSmtpServersStr);
+    }
+    else {
+      serverList = tempServerList;
+    }
+      
     // We need to check if we have any pre-configured smtp servers so that
     // those servers can be appended to the list. 
     nsXPIDLCString appendServerList;
@@ -714,16 +543,22 @@ nsSmtpService::loadSmtpServers()
         }
       }
 
-        char *newStr;
-        char *pref = nsCRT::strtok(NS_CONST_CAST(char*,(const char*)serverList),
-                                   ", ", &newStr);
-        while (pref) {
-            
-            rv = createKeyedServer(pref);
-            
-            pref = nsCRT::strtok(newStr, ", ", &newStr);
-        }
+      char *newStr;
+      char *pref = nsCRT::strtok(NS_CONST_CAST(char*,(const char*)serverList), ", ", &newStr);
 
+      while (pref) {
+        // fix for bug #96207
+        // code above makes sure that no duplicate entries in mail.smtpservers find
+        // their way to the mSmtpServers list.  But it doesn't check, if a server to be
+        // added already is in mSmtpServers.  That can happen in mail has been sent before 
+        // opening the settings (loading the list).
+        // use GetServerByKey to check if the key (pref) is already in
+        // in the list. If not it calls createKeyedServer directly.
+        nsCOMPtr<nsISmtpServer> server;
+        rv = GetServerByKey(pref, getter_AddRefs(server));
+        NS_ASSERTION(NS_SUCCEEDED(rv), "GetServerByKey failed");
+        pref = nsCRT::strtok(newStr, ", ", &newStr);
+      }
     }
 
     saveKeyList();
@@ -740,7 +575,7 @@ nsSmtpService::saveKeyList()
     nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
     if (NS_FAILED(rv)) return rv;
     
-    return prefs->SetCharPref("mail.smtpservers", mServerKeyList.get());
+    return prefs->SetCharPref(PREF_MAIL_SMTPSERVERS, mServerKeyList.get());
 }
 
 nsresult
@@ -817,7 +652,7 @@ nsSmtpService::GetDefaultServer(nsISmtpServer **aServer)
       rv = pref->CopyCharPref("mail.smtp.defaultserver",
                              getter_Copies(defaultServerKey));
       if (NS_SUCCEEDED(rv) &&
-          strlen(defaultServerKey) > 0) {
+          !defaultServerKey.IsEmpty()) {
 
           nsCOMPtr<nsISmtpServer> server;
           rv = GetServerByKey(defaultServerKey,
@@ -862,12 +697,13 @@ nsSmtpService::GetDefaultServer(nsISmtpServer **aServer)
 NS_IMETHODIMP
 nsSmtpService::SetDefaultServer(nsISmtpServer *aServer)
 {
-    nsresult rv;
+    NS_ENSURE_ARG_POINTER(aServer);
+
     mDefaultSmtpServer = aServer;
 
     nsXPIDLCString serverKey;
-    rv = aServer->GetKey(getter_Copies(serverKey));
-    if (NS_FAILED(rv)) return rv;
+    nsresult rv = aServer->GetKey(getter_Copies(serverKey));
+    NS_ENSURE_SUCCESS(rv,rv);
     
     nsCOMPtr<nsIPref> pref(do_GetService(NS_PREF_CONTRACTID, &rv));
     pref->SetCharPref("mail.smtp.defaultserver", serverKey);
@@ -930,6 +766,8 @@ nsSmtpService::CreateSmtpServer(nsISmtpServer **aResult)
 nsresult
 nsSmtpService::GetServerByKey(const char* aKey, nsISmtpServer **aResult)
 {
+    NS_ENSURE_ARG_POINTER(aResult);
+
     findServerByKeyEntry entry;
     entry.key = aKey;
     entry.server = nsnull;
@@ -1048,7 +886,7 @@ NS_IMETHODIMP
 nsSmtpService::GetSmtpServerByIdentity(nsIMsgIdentity *aSenderIdentity, nsISmtpServer **aSmtpServer)
 {
   NS_ENSURE_ARG_POINTER(aSmtpServer);
-  nsresult rv;
+  nsresult rv = NS_ERROR_FAILURE;
 
   // First try the identity's preferred server
   if (aSenderIdentity) {
