@@ -25,8 +25,8 @@
 #include "nsMsgComposeSecure.h"
 #include "nspr.h"
 #include "nsCOMPtr.h"
-#include "nsICMS.h"
 #include "nsIX509Cert.h"
+#include "nsISMimeCert.h"
 #include "nsIMimeConverter.h"
 #include "nsMimeStringResources.h"
 #include "nsMimeTypes.h"
@@ -40,6 +40,7 @@
 #include "nsIMsgCompFields.h"
 #include "nsCRT.h"
 #include "nsReadableUtils.h"
+#include "nsArray.h"
 
 // String bundle for smime. Class static.
 nsCOMPtr<nsIStringBundle> nsMsgComposeSecure::mSMIMEBundle = nsnull;
@@ -70,15 +71,12 @@ static char *mime_make_separator(const char *prefix);
 #include "nsMsgMimeCID.h"
 #include "nsIMimeConverter.h"
 
-/* This is the next generation string retrieval call */
-static NS_DEFINE_CID(kCMimeConverterCID, NS_MIME_CONVERTER_CID);
-
 MimeEncoderData *
 MIME_B64EncoderInit(nsresult (*PR_CALLBACK output_fn) (const char *buf, PRInt32 size, void *closure), void *closure) 
 {
   MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
                                            NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) 
   {
@@ -93,7 +91,7 @@ MIME_QPEncoderInit(nsresult (*PR_CALLBACK output_fn) (const char *buf, PRInt32 s
 {
   MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
                                            NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) 
   {
@@ -108,7 +106,7 @@ MIME_UUEncoderInit(char *filename, nsresult (*PR_CALLBACK output_fn) (const char
 {
   MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
                                            NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) 
   {
@@ -123,7 +121,7 @@ MIME_EncoderDestroy(MimeEncoderData *data, PRBool abort_p)
 {
   //MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
                                            NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) 
   {
@@ -140,7 +138,7 @@ MIME_EncoderWrite(MimeEncoderData *data, const char *buffer, PRInt32 size)
   //  MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
   PRInt32 written = 0;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
     NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) {
     res = converter->EncoderWrite(data, buffer, size, &written);
@@ -195,7 +193,6 @@ NS_IMPL_ISUPPORTS1(nsMsgSMIMEComposeFields, nsIMsgSMIMECompFields)
 nsMsgSMIMEComposeFields::nsMsgSMIMEComposeFields()
 :mSignMessage(PR_FALSE), mAlwaysEncryptMessage(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsMsgSMIMEComposeFields::~nsMsgSMIMEComposeFields()
@@ -234,7 +231,6 @@ NS_IMPL_ISUPPORTS1(nsMsgComposeSecure, nsIMsgComposeSecure)
 
 nsMsgComposeSecure::nsMsgComposeSecure()
 {
-  NS_INIT_ISUPPORTS();
   /* member initializers and constructor code */
   mStream = 0;
   mDataHash = 0;
@@ -485,7 +481,7 @@ NS_IMETHODIMP nsMsgComposeSecure::BeginCryptoEncapsulation(nsOutputFileStream *
     break;
 	case mime_crypto_opaque_signed:
     PR_ASSERT(0);    /* #### no api for this yet */
-    rv = -1;
+    rv = NS_ERROR_NOT_IMPLEMENTED;
 	  break;
 	case mime_crypto_signed_encrypted:
     rv = MimeInitEncryption(PR_TRUE, sendReport);
@@ -510,7 +506,7 @@ FAIL:
 /* void finishCryptoEncapsulation (in boolean aAbort); */
 NS_IMETHODIMP nsMsgComposeSecure::FinishCryptoEncapsulation(PRBool aAbort, nsIMsgSendReport *sendReport)
 {
-  nsresult rv;
+  nsresult rv = NS_OK;
 
   if (!aAbort) {
 	  switch (mCryptoState) {
@@ -615,7 +611,7 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign, nsIMsgSendReport *
 
   if (!mIsDraft) {
     PRUint32 numCerts;
-    mCerts->Count(&numCerts);
+    mCerts->GetLength(&numCerts);
     PR_ASSERT(numCerts > 0);
 	  if (numCerts == 0) return NS_ERROR_FAILURE;
   }
@@ -668,7 +664,6 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter, nsIMsgSen
   unsigned char * sec_item_data;
   nsCOMPtr<nsICMSMessage> cinfo = do_CreateInstance(NS_CMSMESSAGE_CONTRACTID, &rv);
   nsCOMPtr<nsICMSEncoder> encoder = do_CreateInstance(NS_CMSENCODER_CONTRACTID, &rv);
-  PRStatus ds_status = PR_FAILURE;
   char * header = nsnull;
 
   /* Compute the hash...
@@ -858,7 +853,6 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
                                                  PRBool aEncrypt,
                                                  PRBool aSign)
 {
-  int status = 0;
   char *all_mailboxes = 0, *mailboxes = 0, *mailbox_list = 0;
   const char *mailbox = 0;
   PRUint32 count = 0;
@@ -867,16 +861,14 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
   nsCOMPtr<nsIMsgHeaderParser> pHeader = do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID, &res);
   NS_ENSURE_SUCCESS(res,res);
 
-  res = NS_NewISupportsArray(getter_AddRefs(mCerts));
+  res = NS_NewArray(getter_AddRefs(mCerts));
   if (NS_FAILED(res)) {
     return res;
   }
 
-  PRBool no_clearsigning_p = PR_FALSE;
-
   PR_ASSERT(aEncrypt || aSign);
-  certdb->GetEmailEncryptionCert(mEncryptionCertName, getter_AddRefs(mSelfEncryptionCert));
-  certdb->GetEmailSigningCert(mSigningCertName, getter_AddRefs(mSelfSigningCert));
+  certdb->FindEmailEncryptionCert(mEncryptionCertName, getter_AddRefs(mSelfEncryptionCert));
+  certdb->FindEmailSigningCert(mSigningCertName, getter_AddRefs(mSelfSigningCert));
 
   // must have both the signing and encryption certs to sign
 	if ((mSelfSigningCert == nsnull) && aSign) {
@@ -885,12 +877,6 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
 		goto FAIL;
 	}
 
-	if ((mSelfEncryptionCert == nsnull) && aSign) {
-    SetError(sendReport, NS_LITERAL_STRING("SignNoSenderEncryptionCert").get());
-    res = NS_ERROR_FAILURE;
-    goto FAIL;
-  }
-
 	if ((mSelfEncryptionCert == nsnull) && aEncrypt) {
     SetError(sendReport, NS_LITERAL_STRING("NoSenderEncryptionCert").get());
     res = NS_ERROR_FAILURE;
@@ -899,29 +885,55 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
 
   pHeader->ExtractHeaderAddressMailboxes(nsnull,aRecipients, &all_mailboxes);
   pHeader->RemoveDuplicateAddresses(nsnull, all_mailboxes, 0, PR_FALSE /*removeAliasesToMe*/, &mailboxes);
-  PR_FREEIF(all_mailboxes);
+  if (all_mailboxes) {
+    nsMemory::Free(all_mailboxes);
+    all_mailboxes = nsnull;
+  }
 
   if (mailboxes) {
 	  pHeader->ParseHeaderAddresses (nsnull, mailboxes, 0, &mailbox_list, &count);
+    nsMemory::Free(mailboxes);
+    mailboxes = nsnull;
   }
-  PR_FREEIF(mailboxes);
   if (count < 0) return count;
 
+  if (aEncrypt && mSelfEncryptionCert) {
+    // Make sure self's configured cert is prepared for being used
+    // as an email recipient cert.
+    
+    nsCOMPtr<nsISMimeCert> sc = do_QueryInterface(mSelfEncryptionCert);
+    if (sc) {
+      sc->SaveSMimeProfile();
+    }
+  }
+
   /* If the message is to be encrypted, then get the recipient certs */
   if (aEncrypt) {
 	  mailbox = mailbox_list;
-	  nsCString mailbox_lowercase;
-	  ToLowerCase(nsDependentCString(mailbox), mailbox_lowercase);
 
     PRBool already_added_self_cert = PR_FALSE;
 
 	  for (; count > 0; count--) {
+	    nsCString mailbox_lowercase;
+	    ToLowerCase(nsDependentCString(mailbox), mailbox_lowercase);
       nsCOMPtr<nsIX509Cert> cert;
-      certdb->GetCertByEmailAddress(nsnull, mailbox_lowercase.get(), getter_AddRefs(cert));
-		  if (!cert) {
-        // failure to find an encryption cert is
-        // fatal for now. We won't be able to encrypt anyway
-        // ssaux 12/03/2001.
+      certdb->FindCertByEmailAddress(nsnull, mailbox_lowercase.get(), getter_AddRefs(cert));
+      PRBool foundValidCert = PR_FALSE;
+
+		  if (cert) {
+        PRUint32 verification_result;
+
+        if (NS_SUCCEEDED(
+            cert->VerifyForUsage(nsIX509Cert::CERT_USAGE_EmailRecipient, &verification_result))
+            &&
+            nsIX509Cert::VERIFIED_OK == verification_result)
+        {
+          foundValidCert = PR_TRUE;
+        }
+      }
+      
+      if (!foundValidCert) {
+        // Failure to find a valid encryption cert is fatal.
         // here I assume that mailbox contains ascii rather than utf8.
         SetErrorWithParam(sendReport, NS_LITERAL_STRING("MissingRecipientEncryptionCert").get(), mailbox);
         res = NS_ERROR_FAILURE;
@@ -936,21 +948,25 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
 	   */
 
       PRBool isSame;
-      if (NS_SUCCEEDED(cert->IsSameCert(mSelfEncryptionCert, &isSame))
+      if (NS_SUCCEEDED(cert->Equals(mSelfEncryptionCert, &isSame))
           && isSame) {
         already_added_self_cert = PR_TRUE;
       }
 
-      mCerts->AppendElement(cert);
+      mCerts->AppendElement(cert, PR_FALSE);
+      // To understand this loop, especially the "+= strlen +1", look at the documentation
+      // of ParseHeaderAddresses. Basically, it returns a list of zero terminated strings.
 		  mailbox += strlen(mailbox) + 1;
 	  }
     
     if (!already_added_self_cert) {
-      mCerts->AppendElement(mSelfEncryptionCert);
+      mCerts->AppendElement(mSelfEncryptionCert, PR_FALSE);
     }
 	}
 FAIL:
-  PR_FREEIF(mailbox_list);
+  if (mailbox_list) {
+    nsMemory::Free(mailbox_list);
+  }
   return res;
 }
 
