@@ -48,6 +48,7 @@
 #include "nsIMsgLogonRedirector.h"
 #include "nsISubscribableServer.h"
 #include "nsIUrlListener.h"
+#include "nsAdapterEnumerator.h"
 
 /* get some implementation from nsMsgIncomingServer */
 class nsImapIncomingServer : public nsMsgIncomingServer,
@@ -74,7 +75,7 @@ public:
   NS_DECL_NSISUBSCRIBABLESERVER
 	NS_DECL_NSIURLLISTENER
 
-	NS_IMETHOD PerformBiff();
+	NS_IMETHOD PerformBiff(nsIMsgWindow *aMsgWindow);
 	NS_IMETHOD PerformExpand(nsIMsgWindow *aMsgWindow);
   NS_IMETHOD CloseCachedConnections();
   NS_IMETHOD GetConstructedPrettyName(PRUnichar **retval);
@@ -91,7 +92,7 @@ public:
   NS_IMETHOD GetCanFileMessagesOnServer(PRBool *aCanFileMessagesOnServer);
   NS_IMETHOD GetFilterScope(nsMsgSearchScopeValue *filterScope);
   NS_IMETHOD GetSearchScope(nsMsgSearchScopeValue *searchScope);
-  NS_IMETHOD GetServerRequiresPasswordForBiff(PRBool *_retval);
+  NS_IMETHOD GetServerRequiresPasswordForBiff(PRBool *aServerRequiresPasswordForBiff);
   NS_IMETHOD OnUserOrHostNameChanged(const char *oldName, const char *newName);
   NS_IMETHOD GetNumIdleConnections(PRInt32 *aNumIdleConnections);
   NS_IMETHOD ForgetSessionPassword();
