@@ -49,14 +49,13 @@
 #include "nsIRequestObserver.h"
 #include "nsSpecialSystemDirectory.h"
 #include "nsIFileStream.h"
-static NS_DEFINE_CID(kMsgAccountManagerCID, NS_MSGACCOUNTMANAGER_CID);
+#include "nsIMsgCopyService.h"
 static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
 
 NS_IMPL_ISUPPORTS2(nsImapOfflineSync, nsIUrlListener, nsIMsgCopyServiceListener)
 
 nsImapOfflineSync::nsImapOfflineSync(nsIMsgWindow *window, nsIUrlListener *listener, nsIMsgFolder *singleFolderOnly)
 {
-  NS_INIT_REFCNT();
   m_singleFolderToUpdate = singleFolderOnly;
   m_window = window;
   // not the perfect place for this, but I think it will work.
@@ -107,8 +106,12 @@ nsImapOfflineSync::OnStopRunningUrl(nsIURI* url, nsresult exitCode)
   // NS_BINDING_ABORTED is used for the user pressing stop, which
   // should cause us to abort the offline process. Other errors
   // should allow us to continue.
-  if (exitCode != NS_BINDING_ABORTED)
+  if (NS_SUCCEEDED(exitCode))
     rv = ProcessNextOperation();
+  // else if it's a non-stop error, and we're doing multiple folders,
+  // go to the next folder.
+  else if (exitCode != NS_BINDING_ABORTED && !m_singleFolderToUpdate)
+    rv = AdvanceToNextFolder();
   else if (m_listener)  // notify main observer.
     m_listener->OnStopRunningUrl(url, exitCode);
 
@@ -345,11 +348,13 @@ nsImapOfflineSync::ProcessAppendMsgOperation(nsIMsgOfflineImapOperation *current
                   if (NS_SUCCEEDED(rv))
                   {
                     m_curTempFile = tempFileSpec;
-                    rv = destFolder->CopyFileMessage(tempFileSpec,
+                    nsCOMPtr<nsIMsgCopyService> copyService = do_GetService(NS_MSGCOPYSERVICE_CONTRACTID);
+                    if (copyService)
+                      rv = copyService->CopyFileMessage(tempFileSpec, destFolder,
                       /* nsIMsgDBHdr* msgToReplace */ nsnull,
                       PR_TRUE /* isDraftOrTemplate */,
-                      m_window,
-                      this);
+                        this,
+                        m_window);
                   }
                   else
                     m_curTempFile->Delete(PR_FALSE);
@@ -440,7 +445,9 @@ void nsImapOfflineSync::ProcessMoveOperation(nsIMsgOfflineImapOperation *current
               messages->AppendElement(iSupports);
             }
           }
-          destFolder->CopyMessages(m_currentFolder, messages, PR_TRUE, m_window, this, PR_FALSE, PR_FALSE);
+          nsCOMPtr<nsIMsgCopyService> copyService = do_GetService(NS_MSGCOPYSERVICE_CONTRACTID, &rv);
+          if (copyService)
+            copyService->CopyMessages(m_currentFolder, messages, destFolder, PR_TRUE, this, m_window, PR_FALSE);
         }
       }
     }
@@ -535,7 +542,9 @@ void nsImapOfflineSync::ProcessCopyOperation(nsIMsgOfflineImapOperation *current
               messages->AppendElement(iSupports);
             }
           }
-          destFolder->CopyMessages(m_currentFolder, messages, PR_FALSE, m_window, this, PR_FALSE, PR_FALSE);
+          nsCOMPtr<nsIMsgCopyService> copyService = do_GetService(NS_MSGCOPYSERVICE_CONTRACTID, &rv);
+          if (copyService)
+            copyService->CopyMessages(m_currentFolder, messages, destFolder, PR_FALSE, this, m_window, PR_FALSE);
         }
       }
     }
@@ -941,7 +950,7 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
     // Update the INBOX first so the updates on the remaining
     // folders pickup the results of any filter moves.
     nsCOMPtr<nsIMsgAccountManager> accountManager = 
-             do_GetService(kMsgAccountManagerCID, &rv);
+             do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
     if (NS_FAILED(rv)) return rv;
     nsCOMPtr<nsISupportsArray> servers;
   
@@ -1016,10 +1025,14 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
       imapFolder = do_QueryInterface(m_currentFolder);
     m_currentFolder->GetFlags(&folderFlags);
 		// need to check if folder has offline events, or is configured for offline
-		if (imapFolder && folderFlags & MSG_FOLDER_FLAG_OFFLINE)
-      return m_currentFolder->DownloadAllForOffline(this, m_window);
-    else
-      AdvanceToNextFolder();
+    if (imapFolder && folderFlags & MSG_FOLDER_FLAG_OFFLINE)
+    {
+      rv = m_currentFolder->DownloadAllForOffline(this, m_window);
+      if (NS_SUCCEEDED(rv) || rv == NS_BINDING_ABORTED)
+        return rv;
+      // if this fails and the user didn't cancel/stop, fall through to code that advances to next folder
+    }
+    AdvanceToNextFolder();
   }
   if (m_listener)
     m_listener->OnStopRunningUrl(nsnull, NS_OK);
