@@ -127,7 +127,6 @@ nsImapExtensionSinkProxy::nsImapExtensionSinkProxy(nsIImapExtensionSink* aImapEx
 {
     NS_ASSERTION (aImapExtensionSink, 
                   "nsImapExtensionSinkProxy: invalid aImapExtensionSink");
-    NS_INIT_REFCNT ();
     m_realImapExtensionSink = aImapExtensionSink;
     NS_IF_ADDREF (m_realImapExtensionSink);
 }
@@ -271,7 +270,6 @@ nsImapMiscellaneousSinkProxy::nsImapMiscellaneousSinkProxy(
 {
     NS_ASSERTION (aImapMiscellaneousSink, 
                   "nsImapMiscellaneousSinkProxy: invalid aImapMiscellaneousSink");
-    NS_INIT_REFCNT ();
     m_realImapMiscellaneousSink = aImapMiscellaneousSink;
     NS_IF_ADDREF (m_realImapMiscellaneousSink);
 }
@@ -464,61 +462,6 @@ nsImapMiscellaneousSinkProxy::PercentProgress(nsIImapProtocol* aProtocol,
     return res;
 }
 
-NS_IMETHODIMP
-nsImapMiscellaneousSinkProxy::TunnelOutStream(nsIImapProtocol* aProtocol,
-                                          msg_line_info* aInfo)
-{
-    nsresult res = NS_OK;
-    NS_PRECONDITION (aInfo, "Oops... null aInfo");
-    if(!aInfo)
-        return NS_ERROR_NULL_POINTER;
-    NS_ASSERTION (m_protocol == aProtocol, "Ooh ooh, wrong protocol");
-
-    if (PR_GetCurrentThread() == m_thread)
-    {
-        TunnelOutStreamProxyEvent *ev =
-            new TunnelOutStreamProxyEvent(this, aInfo);
-        if(nsnull == ev)
-            res = NS_ERROR_OUT_OF_MEMORY;
-        else
-            ev->PostEvent(m_eventQueue);
-    }
-    else
-    {
-        res = m_realImapMiscellaneousSink->TunnelOutStream(aProtocol, aInfo);
-    }
-    return res;
-}
-
-NS_IMETHODIMP
-nsImapMiscellaneousSinkProxy::ProcessTunnel(nsIImapProtocol* aProtocol,
-                                        TunnelInfo *aInfo)
-{
-    nsresult res = NS_OK;
-    NS_PRECONDITION (aInfo, "Oops... null aInfo");
-    if(!aInfo)
-        return NS_ERROR_NULL_POINTER;
-    NS_ASSERTION (m_protocol == aProtocol, "Ooh ooh, wrong protocol");
-
-    if (PR_GetCurrentThread() == m_thread)
-    {
-        ProcessTunnelProxyEvent *ev =
-            new ProcessTunnelProxyEvent(this, aInfo);
-        if(nsnull == ev)
-            res = NS_ERROR_OUT_OF_MEMORY;
-        else
-            ev->PostEvent(m_eventQueue);
-    }
-    else
-    {
-        res = m_realImapMiscellaneousSink->ProcessTunnel(aProtocol, aInfo);
-    }
-    return res;
-}
-
-
-///
-
 ////
 nsImapExtensionSinkProxyEvent::nsImapExtensionSinkProxyEvent(
     nsImapExtensionSinkProxy* aProxy)
@@ -838,66 +781,3 @@ PercentProgressProxyEvent::HandleEvent()
     return res;
 }
 
-TunnelOutStreamProxyEvent::TunnelOutStreamProxyEvent(
-    nsImapMiscellaneousSinkProxy* aProxy, msg_line_info* aInfo) :
-    nsImapMiscellaneousSinkProxyEvent(aProxy)
-{
-    NS_ASSERTION (aInfo, "Oops... a null message line info");
-    if (aInfo)
-    {
-        m_msgLineInfo.adoptedMessageLine = 
-            PL_strdup(aInfo->adoptedMessageLine);
-        m_msgLineInfo.uidOfMessage = aInfo->uidOfMessage;
-    }
-    else
-    {
-        m_msgLineInfo.adoptedMessageLine = nsnull;
-        m_msgLineInfo.uidOfMessage = 0xffffffff;
-    }
-}
-
-TunnelOutStreamProxyEvent::~TunnelOutStreamProxyEvent()
-{
-    if (m_msgLineInfo.adoptedMessageLine)
-        PL_strfree(m_msgLineInfo.adoptedMessageLine);
-}
-
-NS_IMETHODIMP
-TunnelOutStreamProxyEvent::HandleEvent()
-{
-    nsresult res = m_proxy->m_realImapMiscellaneousSink->TunnelOutStream(
-        m_proxy->m_protocol, &m_msgLineInfo);
-    if (m_notifyCompletion)
-        m_proxy->m_protocol->NotifyFEEventCompletion();
-    return res;
-}
-
-ProcessTunnelProxyEvent::ProcessTunnelProxyEvent(
-    nsImapMiscellaneousSinkProxy* aProxy, TunnelInfo *aInfo) :
-    nsImapMiscellaneousSinkProxyEvent(aProxy)
-{
-    NS_ASSERTION (aInfo, "Oops... a null tunnel info");
-    if (aInfo)
-    {
-        m_tunnelInfo = *aInfo;
-    }
-    else
-    {
-        memset(&m_tunnelInfo, 0, sizeof(TunnelInfo));
-    }
-}
-
-ProcessTunnelProxyEvent::~ProcessTunnelProxyEvent()
-{
-}
-
-NS_IMETHODIMP
-ProcessTunnelProxyEvent::HandleEvent()
-{
-  nsresult res = m_proxy->m_realImapMiscellaneousSink->ProcessTunnel(
-      m_proxy->m_protocol, &m_tunnelInfo);
-  if (m_notifyCompletion)
-      m_proxy->m_protocol->NotifyFEEventCompletion();
-  return res;
-}
-
