@@ -292,12 +292,9 @@ nsImapMailboxSpec::nsImapMailboxSpec()
 
 nsImapMailboxSpec::~nsImapMailboxSpec()
 {
-	if (allocatedPathName)
-		nsCRT::free(allocatedPathName);
-	if (unicharPathName)
-		nsCRT::free(unicharPathName);
-	if (hostName)
-		nsCRT::free(hostName);
+	nsCRT::free(allocatedPathName);
+	nsCRT::free(unicharPathName);
+	nsCRT::free(hostName);
 }
 
 NS_IMPL_GETSET(nsImapMailboxSpec, Folder_UIDVALIDITY, PRInt32, folder_UIDVALIDITY);
