@@ -1,24 +1,39 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
- */
+ * Contributor(s):
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 #include "nsOEScanBoxes.h"
 #include "nsIComponentManager.h"
 #include "nsIServiceManager.h"
@@ -37,10 +52,8 @@
 
 */
 
-static NS_DEFINE_CID(kImportServiceCID,		NS_IMPORTSERVICE_CID);
 static NS_DEFINE_IID(kISupportsIID,			NS_ISUPPORTS_IID);
 
-
 nsOEScanBoxes::nsOEScanBoxes()
 {
 	m_pFirst = nsnull;
@@ -48,11 +61,19 @@ nsOEScanBoxes::nsOEScanBoxes()
 
 nsOEScanBoxes::~nsOEScanBoxes()
 {
-	int max = m_entryArray.Count();
-	for (int i = 0; i < max; i++) {
-		MailboxEntry *pEntry = (MailboxEntry *) m_entryArray.ElementAt( i);
+	int i, max;
+  MailboxEntry *pEntry;
+	for (i = 0, max = m_entryArray.Count(); i < max; i++) {
+		pEntry = (MailboxEntry *) m_entryArray.ElementAt( i);
 		delete pEntry;
 	}
+  // Now free the unprocessed child entries (ie, those without parents for some reason).
+  for (i = 0, max = m_pendingChildArray.Count(); i < max; i++)
+  {
+    pEntry = (MailboxEntry *) m_pendingChildArray.ElementAt(i);
+    if (!pEntry->processed)
+      delete pEntry;
+  }
 }
 
 
@@ -62,7 +83,7 @@ void nsOEScanBoxes::ConvertToUnicode(const char *pStr, nsString &dist)
 	nsresult rv = NS_OK;
 
 	if (!mService)
-		mService = do_GetService(kImportServiceCID);
+		mService = do_GetService(NS_IMPORTSERVICE_CONTRACTID);
 
 	if (mService)
 		rv = mService->SystemStringToUnicode(pStr, dist);
@@ -95,7 +116,7 @@ PRBool nsOEScanBoxes::Find50Mail( nsIFileSpec *pWhere)
 			key += (const char *)pBytes;
 			nsOERegUtil::FreeValueBytes( pBytes);
 			key += "\\Software\\Microsoft\\Outlook Express\\5.0";
-			if (::RegOpenKeyEx( HKEY_CURRENT_USER, key, 0, KEY_QUERY_VALUE, &sKey) == ERROR_SUCCESS) {
+			if (::RegOpenKeyEx( HKEY_CURRENT_USER, key.get(), 0, KEY_QUERY_VALUE, &sKey) == ERROR_SUCCESS) {
 				pBytes = nsOERegUtil::GetValueBytes( sKey, "Store Root");
 				if (pBytes) {
 					pWhere->SetNativePath((char *)pBytes);
@@ -280,7 +301,7 @@ PRBool nsOEScanBoxes::FindMailBoxes( nsIFileSpec* descFile)
 		#endif
 		
 		pEntry->fileName.Right( ext, 4);
-		if (Compare(ext, mbxExt))
+		if (!ext.Equals(mbxExt))
 			pEntry->fileName.Append( ".mbx");		
 		
 		m_entryArray.AppendElement( pEntry);
@@ -370,7 +391,8 @@ PRBool nsOEScanBoxes::Find50MailBoxes( nsIFileSpec* descFile)
 		while (dataOffset < strOffset) {
 			tag = (PRUint8) pBytes[dataOffset];
 
-			nsCRT::memcpy( &data, &(pBytes[dataOffset + 1]), 3);
+      data = 0; // make sure all bytes are 0 before copying 3 bytes over.
+			memcpy( &data, &(pBytes[dataOffset + 1]), 3);
 			switch( tag) {
 				case 0x80: // id record
 					id = data;
@@ -396,7 +418,13 @@ PRBool nsOEScanBoxes::Find50MailBoxes( nsIFileSpec* descFile)
 		// now build an entry if necessary!
 		if (pDataSource) {
 			if (!nsCRT::strcasecmp( pDataSource, "LocalStore"))
+      {
 				localStoreId = id;	
+        // See if we have any child folders that need to be added/processed for this top level parent.
+        ProcessPendingChildEntries(localStoreId, localStoreId, m_pendingChildArray);
+        // Clean up the pending list.
+        RemoveProcessedChildEntries();
+      }
 		}
 		else if (id && localStoreId && parent) {
 			// veryify that this mailbox is in the local store
@@ -410,23 +438,38 @@ PRBool nsOEScanBoxes::Find50MailBoxes( nsIFileSpec* descFile)
 			}
 			if (data == localStoreId) {
 				// Create an entry for this bugger
-				pEntry = new MailboxEntry();
-				pEntry->index = id;
-				pEntry->parent = parent;
-				pEntry->child = 0;
-				pEntry->type = 0;
-				pEntry->sibling = -1;
-				ConvertToUnicode((const char *) (pBytes + strOffset), pEntry->mailName);
-				if (pFileName)
-					pEntry->fileName = pFileName;
-				AddChildEntry( pEntry, localStoreId);
+        pEntry = NewMailboxEntry(id, parent, (const char *) (pBytes + strOffset), pFileName);
+        if (pEntry)
+        {
+				  AddChildEntry( pEntry, localStoreId);
+          pEntry->processed =  PR_TRUE;
+          // See if we have any child folders that need to be added/processed.
+          ProcessPendingChildEntries(id, localStoreId, m_pendingChildArray);
+          // Clean up the pending list.
+          RemoveProcessedChildEntries();
+        }
 			}
+      else
+      {
+        // Put this folder into child array and process it when its parent shows up.
+        pEntry = NewMailboxEntry(id, parent, (const char *) (pBytes + strOffset), pFileName);
+        if (pEntry)
+          m_pendingChildArray.AppendElement(pEntry);
+      }
 		}
+    else if (pFileName)
+    {
+      // Put this folder into child array and process it when its parent shows up.
+      // For some reason, it's likely that child folders come before their parents.
+      pEntry = NewMailboxEntry(id, parent, (const char *) (pBytes + strOffset), pFileName);
+      if (pEntry)
+        m_pendingChildArray.AppendElement(pEntry);
+    }
 
 		delete [] pBytes;
 	}
-
 	
+
 	delete [] pIndex;
 
 	if (m_entryArray.Count())
@@ -435,6 +478,55 @@ PRBool nsOEScanBoxes::Find50MailBoxes( nsIFileSpec* descFile)
 		return( PR_FALSE);
 }
 
+nsOEScanBoxes::MailboxEntry *nsOEScanBoxes::NewMailboxEntry(PRUint32 id, PRUint32 parent, const char *prettyName, char *pFileName)
+{
+  MailboxEntry *pEntry = new MailboxEntry();
+  if (!pEntry)
+    return nsnull;
+
+  pEntry->index = id;
+  pEntry->parent = parent;
+  pEntry->child = 0;
+  pEntry->type = 0;
+  pEntry->sibling = -1;
+  pEntry->processed =  PR_FALSE;
+  ConvertToUnicode(prettyName, pEntry->mailName);
+  if (pFileName)
+	  pEntry->fileName = pFileName;
+  return pEntry;
+}
+
+void nsOEScanBoxes::ProcessPendingChildEntries(PRUint32 parent, PRUint32 rootIndex, nsVoidArray	&childArray)
+{
+  PRInt32 i, max;
+  MailboxEntry *pEntry;
+  for (i = 0, max = childArray.Count(); i < max; i++)
+  {
+    pEntry = (MailboxEntry *) childArray.ElementAt(i);
+    if ((!pEntry->processed) && (pEntry->parent == parent))
+    {
+      AddChildEntry(pEntry, rootIndex);
+      pEntry->processed =  PR_TRUE; // indicate it's been processed.
+      // See if there are unprocessed child folders for this child in the
+      // array as well (ie, both child and grand-child are on the list).
+      ProcessPendingChildEntries(pEntry->index, rootIndex, childArray);
+    }
+  }
+}
+
+void nsOEScanBoxes::RemoveProcessedChildEntries()
+{
+  // Remove already processed entries from the pending list. Note that these entries are also
+  // on 'm_entryArray' list so we don't want to deallocate the space for the entries now.
+  MailboxEntry * pEntry;
+  PRInt32 i;
+  for (i = m_pendingChildArray.Count()-1; i >= 0; i--)
+  {
+    pEntry = (MailboxEntry *) m_pendingChildArray.ElementAt(i);
+    if (pEntry->processed)
+      m_pendingChildArray.RemoveElementAt(i);
+  }
+}
 
 void nsOEScanBoxes::AddChildEntry( MailboxEntry *pEntry, PRUint32 rootIndex)
 {
@@ -519,7 +611,7 @@ PRBool nsOEScanBoxes::Scan50MailboxDir( nsIFileSpec * srcDir)
 				pLeaf = nsnull;
 				rv = spec->GetLeafName( &pLeaf);
 				if (NS_SUCCEEDED( rv) && pLeaf && 
-					((sLen = nsCRT::strlen( pLeaf)) > 4) && 
+					((sLen = strlen( pLeaf)) > 4) && 
 					(!nsCRT::strcasecmp( pLeaf + sLen - 3, "dbx"))) {
 					// This is a *.dbx file in the mail directory
 					if (nsOE5File::IsLocalMailFile( spec)) {
@@ -591,7 +683,7 @@ void nsOEScanBoxes::ScanMailboxDir( nsIFileSpec * srcDir)
 				pLeaf = nsnull;
 				rv = spec->GetLeafName( &pLeaf);
 				if (NS_SUCCEEDED( rv) && pLeaf && 
-					((sLen = nsCRT::strlen( pLeaf)) > 4) && 
+					((sLen = strlen( pLeaf)) > 4) && 
 					(!nsCRT::strcasecmp( pLeaf + sLen - 3, "mbx"))) {
 					// This is a *.mbx file in the mail directory
 					pEntry = new MailboxEntry;
@@ -694,7 +786,7 @@ void nsOEScanBoxes::BuildMailboxList( MailboxEntry *pBox, nsIFileSpec * root, PR
 	nsISupports *					pInterface;
 	PRUint32						size;
 	
-	nsCOMPtr<nsIImportService> impSvc(do_GetService(kImportServiceCID, &rv));
+	nsCOMPtr<nsIImportService> impSvc(do_GetService(NS_IMPORTSERVICE_CONTRACTID, &rv));
 	if (NS_FAILED( rv))
 		return;
 	
@@ -707,7 +799,7 @@ void nsOEScanBoxes::BuildMailboxList( MailboxEntry *pBox, nsIFileSpec * root, PR
 			if (pBox->fileName.Length() > 0) {
 				pID->GetFileSpec( &file);
 				file->FromFileSpec( root);
-				file->AppendRelativeUnixPath( (const char *)pBox->fileName);
+				file->AppendRelativeUnixPath( pBox->fileName.get());
 				size = 0;
 				file->GetFileSize( &size);
 				pID->SetSize( size);
