@@ -63,7 +63,6 @@
 // that doesn't allow you to call ::nsISupports::GetIID() inside of a class
 // that multiply inherits from nsISupports
 static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
-static NS_DEFINE_CID(kUrlListenerManagerCID, NS_URLLISTENERMANAGER_CID);
 static NS_DEFINE_CID(kCMailDB, NS_MAILDB_CID);
 
 // this is totally lame and MUST be removed by M6
@@ -387,7 +386,6 @@ nsresult nsMailboxUrl::ParseSearchPart()
 		m_messageID = extractAttributeValue(searchPart.get(),"messageid=");
 		if (messageKey)
 			m_messageKey = atol(messageKey); // convert to a long...
-		if (messageKey || m_messageID)
 		
     PR_FREEIF(msgPart);
 		PR_FREEIF(messageKey);
