@@ -1,26 +1,41 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
+ * Contributor(s):
  * Adam D. Moss <adam@gimp.org>
  * Seth Spitzer <sspitzer@netscape.com>
- */
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #if defined(DEBUG_adam) || defined(DEBUG_seth)
 #define MOVEMAIL_DEBUG
@@ -53,15 +68,14 @@
 #include "nsMsgBaseCID.h"
 #include "nsXPIDLString.h"
 #include "nsCOMPtr.h"
+#include "nsMsgFolderFlags.h"
 
 #define PREF_MAIL_ROOT_MOVEMAIL "mail.root.movemail"
 
 static NS_DEFINE_CID(kPrefCID,           NS_PREF_CID);
-static NS_DEFINE_CID(kMsgMailSessionCID, NS_MSGMAILSESSION_CID);
 
 nsMovemailService::nsMovemailService()
 {
-    NS_INIT_REFCNT();
 #ifdef MOVEMAIL_DEBUG
     fprintf(stderr, "*** MURRR, new nsMovemailService\n");
 #endif
@@ -121,13 +135,13 @@ PRBool ObtainSpoolLock(const char *spoolnameStr,
     if (NS_FAILED(rv))
         return PR_FALSE;
     // Create nsFileSpec and nsILocalFile for the spool.mozlock file
-    nsFileSpec tmplocspec(mozlockstr);
+    nsFileSpec tmplocspec(mozlockstr.get());
     nsCOMPtr<nsILocalFile> tmplocfile;
     rv = NS_FileSpecToIFile(&tmplocspec, getter_AddRefs(tmplocfile));
     if (NS_FAILED(rv))
         return PR_FALSE;
     // Create nsFileSpec and nsILocalFile for the spool.lock file
-    nsFileSpec locklocspec(lockstr);
+    nsFileSpec locklocspec(lockstr.get());
     nsCOMPtr<nsILocalFile> locklocfile;
     rv = NS_FileSpecToIFile(&locklocspec, getter_AddRefs(locklocfile));
     if (NS_FAILED(rv))
@@ -218,7 +232,7 @@ PRBool YieldSpoolLock(const char *spoolnameStr)
     nsresult rv;
 
     // Create nsFileSpec and nsILocalFile for the spool.lock file
-    nsFileSpec locklocspec(lockstr);
+    nsFileSpec locklocspec(lockstr.get());
     nsCOMPtr<nsILocalFile> locklocfile;
     rv = NS_FileSpecToIFile(&locklocspec, getter_AddRefs(locklocfile));
     if (NS_FAILED(rv))
@@ -326,27 +340,27 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
 
                     wholeboxname = "/var/spool/mail/";
                     wholeboxname += boxfilename;
-                    spoolfile = Probe_SpoolFilePath((const char *)wholeboxname,
+                    spoolfile = Probe_SpoolFilePath(wholeboxname.get(),
                                                     &found_spool_but_it_is_locked);
                     
                     if ((!spoolfile) && (!found_spool_but_it_is_locked)) {
                         wholeboxname = "/usr/spool/mail/";
                         wholeboxname += boxfilename;
-                        spoolfile = Probe_SpoolFilePath((const char *)wholeboxname,
+                        spoolfile = Probe_SpoolFilePath(wholeboxname.get(),
                                                         &found_spool_but_it_is_locked);
                     }
 
                     if ((!spoolfile) && (!found_spool_but_it_is_locked)) {
                         wholeboxname = "/var/mail/";
                         wholeboxname += boxfilename;
-                        spoolfile = Probe_SpoolFilePath((const char *)wholeboxname,
+                        spoolfile = Probe_SpoolFilePath(wholeboxname.get(),
                                                         &found_spool_but_it_is_locked);
                     }
 
                     if ((!spoolfile) && (!found_spool_but_it_is_locked)) {
                         wholeboxname = "/usr/mail/";
                         wholeboxname += boxfilename;
-                        spoolfile = Probe_SpoolFilePath((const char *)wholeboxname,
+                        spoolfile = Probe_SpoolFilePath(wholeboxname.get(),
                                                         &found_spool_but_it_is_locked);
                     }
                 }
@@ -375,8 +389,8 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
                 rv = NS_ERROR_OUT_OF_MEMORY;
             else {
                 if (!spoolfile->failed()) {
-                    nsIOFileStream* m_outFileStream;
-                    nsParseNewMailState *m_newMailParser;
+                    nsIOFileStream* outFileStream;
+                    nsParseNewMailState *newMailParser;
                         
                     nsCOMPtr<nsIFileSpec> mailDirectory;
                     rv = in_server->GetLocalPath(getter_AddRefs(mailDirectory));
@@ -386,10 +400,10 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
                     nsFileSpec fileSpec;
                     mailDirectory->GetFileSpec(&fileSpec);
                     fileSpec += "Inbox";
-                    m_outFileStream =
+                    outFileStream =
                         new nsIOFileStream(fileSpec /*, PR_CREATE_FILE */);
-                    if (m_outFileStream) {
-                        m_outFileStream->seek(fileSpec.GetFileSize());
+                    if (outFileStream) {
+                        outFileStream->seek(fileSpec.GetFileSize());
                     }
                     else {
                         rv = NS_ERROR_UNEXPECTED;
@@ -397,12 +411,13 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
                     }
                         
                     // create a new mail parser
-                    m_newMailParser = new nsParseNewMailState;
-                    if (m_newMailParser == nsnull) {
+                    newMailParser = new nsParseNewMailState;
+                    if (newMailParser == nsnull) {
                         rv = NS_ERROR_OUT_OF_MEMORY;
                         goto freebuff_and_unlock;
                     }
                         
+                    nsCOMPtr <nsIMsgParseMailMsgState> iParseMailMsgState = NS_STATIC_CAST(nsIMsgParseMailMsgState*, newMailParser);
                     nsCOMPtr <nsIFolder> serverFolder;
                     rv =
                         in_server->GetRootFolder(getter_AddRefs(serverFolder));
@@ -410,8 +425,20 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
                     if (NS_FAILED(rv))
                         goto freebuff_and_unlock;
                     
-                    rv = m_newMailParser->Init(serverFolder,
-                                               fileSpec, m_outFileStream);
+	            nsCOMPtr<nsIMsgFolder> inbox;
+	            if(NS_SUCCEEDED(rv))
+	            {
+		      nsCOMPtr<nsIMsgFolder> rootMsgFolder = do_QueryInterface(serverFolder);
+		      if(rootMsgFolder)
+		      {
+			      PRUint32 numFolders;
+			      rv = rootMsgFolder->GetFoldersWithFlag(MSG_FOLDER_FLAG_INBOX, 1,
+                                                         &numFolders,
+                                                         getter_AddRefs(inbox));
+                      }
+                    }
+                    rv = newMailParser->Init(serverFolder, inbox, 
+                                               fileSpec, outFileStream, nsnull);
                     if (NS_FAILED(rv))
                         goto freebuff_and_unlock;
                         
@@ -439,7 +466,7 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
                                 // then the file mysteriously ended) then abort
                                 // parsing.
                                 if (numlines == 0 &&
-                                    nsCRT::strlen(buffer) == 0 &&
+                                    !*buffer &&
                                     spoolfile->eof()) {
 #ifdef MOVEMAIL_DEBUG
                                     fprintf(stderr, "*** Utterly empty spool file\n");
@@ -451,8 +478,8 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
 
                                 //     fprintf(stderr, "%d: %s", numlines, buffer);
 
-                                m_newMailParser->HandleLine(buffer, PL_strlen(buffer));
-                                *m_outFileStream << buffer;
+                                newMailParser->HandleLine(buffer, PL_strlen(buffer));
+                                *outFileStream << buffer;
 
                                 numlines++;
 
@@ -460,14 +487,14 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
                                     !spoolfile->eof()) {
                                     PL_strcpy(buffer, "X-Mozilla-Status: 8000"
                                               MSG_LINEBREAK);
-                                    m_newMailParser->HandleLine(buffer,
+                                    newMailParser->HandleLine(buffer,
                                                                 PL_strlen(buffer));
-                                    *m_outFileStream << buffer;
+                                    *outFileStream << buffer;
                                     PL_strcpy(buffer, "X-Mozilla-Status2: 00000000"
                                               MSG_LINEBREAK);
-                                    m_newMailParser->HandleLine(buffer,
+                                    newMailParser->HandleLine(buffer,
                                                                 PL_strlen(buffer));
-                                    *m_outFileStream << buffer;
+                                    *outFileStream << buffer;
                                             
                                 }
                             }
@@ -475,18 +502,16 @@ nsMovemailService::GetNewMail(nsIMsgWindow *aMsgWindow,
                         
                         
                     // END
-                    m_outFileStream->flush();	// try this.
-                    m_newMailParser->OnStopRequest(nsnull, nsnull, NS_OK);
-                    if (m_outFileStream->is_open())
-                      m_outFileStream->close();
-                    delete m_outFileStream;
-                    m_outFileStream = 0;
-                    delete m_newMailParser;
-                    m_newMailParser = nsnull;
+                    outFileStream->flush();	// try this.
+                    newMailParser->OnStopRequest(nsnull, nsnull, NS_OK);
+                    if (outFileStream->is_open())
+                      outFileStream->close();
+                    delete outFileStream;
+                    outFileStream = 0;
 
                     // truncate the spool file here.
                     nsFileSpec * filespecForTrunc =
-                        new nsFileSpec((const char *)wholeboxname);
+                        new nsFileSpec(wholeboxname.get());
                     if (filespecForTrunc) {
                         filespecForTrunc->Truncate(0);
                         delete filespecForTrunc;
@@ -567,13 +592,9 @@ nsMovemailService::GetDefaultLocalPath(nsIFileSpec ** aResult)
     
     // Make the resulting nsIFileSpec
     // TODO: Convert arg to nsILocalFile and avoid this
-    nsXPIDLCString pathBuf;
-    rv = localFile->GetPath(getter_Copies(pathBuf));
-    if (NS_FAILED(rv)) return rv;
     nsCOMPtr<nsIFileSpec> outSpec;
-    rv = NS_NewFileSpec(getter_AddRefs(outSpec));
+    rv = NS_NewFileSpecFromIFile(localFile, getter_AddRefs(outSpec));
     if (NS_FAILED(rv)) return rv;
-    outSpec->SetNativePath(pathBuf);
     
     if (!havePref || !exists)
         rv = SetDefaultLocalPath(outSpec);
@@ -597,7 +618,7 @@ nsMovemailService::GetRequiresUsername(PRBool *aRequiresUsername)
         NS_ENSURE_ARG_POINTER(aRequiresUsername);
 
 #ifdef MOVEMAIL_DEBUG
-        fprintf(stderr, ">>> Yah, got asked for info\n");
+        fprintf(stderr, "XXX Yah, got asked for info\n");
 #endif
 
         *aRequiresUsername = PR_FALSE;
@@ -636,6 +657,14 @@ nsMovemailService::GetCanGetMessages(PRBool *aCanGetMessages)
     return NS_OK;
 }  
 
+NS_IMETHODIMP
+nsMovemailService::GetCanGetIncomingMessages(PRBool *aCanGetIncomingMessages)
+{
+    NS_ENSURE_ARG_POINTER(aCanGetIncomingMessages);
+    *aCanGetIncomingMessages = PR_TRUE;
+    return NS_OK;
+} 
+
 NS_IMETHODIMP
 nsMovemailService::GetCanDuplicate(PRBool *aCanDuplicate)
 {
