@@ -71,7 +71,6 @@
 #include "prprf.h"
 #include "nsEscape.h"
 #include "nsIMimeHeaders.h"
-
 #include "nsIMsgMdnGenerator.h"
 #include "nsMsgSearchCore.h"
 
@@ -162,39 +161,6 @@ NS_IMETHODIMP nsMsgMailboxParser::OnStopRequest(nsIRequest *request, nsISupports
         m_mailDB->RemoveListener(this);
     // and we want to mark ourselves for deletion or some how inform our protocol manager that we are 
     // available for another url if there is one....
-#ifdef DEBUG1
-    // let's dump out the contents of our db, if possible.
-    if (m_mailDB)
-    {
-        nsMsgKeyArray	keys;
-        nsCAutoString	author;
-        nsCAutoString	subject;
-
-    //	m_mailDB->PrePopulate();
-        m_mailDB->ListAllKeys(keys);
-        PRUint32 size = keys.GetSize();
-        for (PRUint32 keyindex = 0; keyindex < size; keyindex++)
-        {
-            nsCOMPtr<nsIMsgDBHdr> msgHdr;
-            nsresult ret =
-                m_mailDB->GetMsgHdrForKey(keys[keyindex],
-                                          getter_AddRefs(msgHdr));
-            if (NS_SUCCEEDED(ret) && msgHdr)
-            {
-                nsMsgKey key;
-
-                msgHdr->GetMessageKey(&key);
-                msgHdr->GetAuthor(&author);
-                msgHdr->GetSubject(&subject);
-#ifdef DEBUG_bienvenu
-                // leak nsString return values...
-                printf("hdr key = %d, author = %s subject = %s\n", key, author.get(), subject.get());
-#endif
-            }
-        }
-        m_mailDB->Close(PR_TRUE);
-    }
-#endif
 
     ReleaseFolderLock();
     // be sure to clear any status text and progress info..
@@ -248,6 +214,12 @@ NS_IMETHODIMP nsMsgMailboxParser::OnReadChanged(nsIDBChangeListener *instigator)
     return NS_OK;
 }
 
+/* void OnJunkScoreChanged (in nsIDBChangeListener instigator); */
+NS_IMETHODIMP nsMsgMailboxParser::OnJunkScoreChanged(nsIDBChangeListener *instigator)
+{
+    return NS_OK;
+}
+
 nsMsgMailboxParser::nsMsgMailboxParser() : nsMsgLineBuffer(nsnull, PR_FALSE)
 {
   Init();
@@ -368,10 +340,8 @@ void nsMsgMailboxParser::UpdateDBFolderInfo()
 // update folder info in db so we know not to reparse.
 void nsMsgMailboxParser::UpdateDBFolderInfo(nsIMsgDatabase *mailDB)
 {
-	// ### wrong - use method on db.
 	mailDB->SetSummaryValid(PR_TRUE);
 	mailDB->Commit(nsMsgDBCommitType::kLargeCommit);
-//	m_mailDB->Close();
 }
 
 // By default, do nothing
@@ -382,38 +352,38 @@ void nsMsgMailboxParser::FolderTypeSpecificTweakMsgHeader(nsIMsgDBHdr * /* tweak
 // Tell the world about the message header (add to db, and view, if any)
 PRInt32 nsMsgMailboxParser::PublishMsgHeader(nsIMsgWindow *msgWindow)
 {
-	FinishHeader();
-	if (m_newMsgHdr)
-	{
-		FolderTypeSpecificTweakMsgHeader(m_newMsgHdr);
-
-		PRUint32 flags;
-        (void)m_newMsgHdr->GetFlags(&flags);
-		if (flags & MSG_FLAG_EXPUNGED)
-		{
-			nsCOMPtr<nsIDBFolderInfo> folderInfo;
-			m_mailDB->GetDBFolderInfo(getter_AddRefs(folderInfo));
-            PRUint32 size;
-            (void)m_newMsgHdr->GetMessageSize(&size);
-            folderInfo->ChangeExpungedBytes(size);
-			m_newMsgHdr = nsnull;
-		}
-		else if (m_mailDB)
-		{
-			m_mailDB->AddNewHdrToDB(m_newMsgHdr, PR_TRUE);
-			m_newMsgHdr = nsnull;
-		}
-		else
-			NS_ASSERTION(PR_FALSE, "no database while parsing local folder");	// should have a DB, no?
-	}
-	else if (m_mailDB)
-	{
-		nsCOMPtr<nsIDBFolderInfo> folderInfo;
-		m_mailDB->GetDBFolderInfo(getter_AddRefs(folderInfo));
-		if (folderInfo)
-			folderInfo->ChangeExpungedBytes(m_position - m_envelope_pos);
-	}
-	return 0;
+  FinishHeader();
+  if (m_newMsgHdr)
+  {
+    FolderTypeSpecificTweakMsgHeader(m_newMsgHdr);
+    
+    PRUint32 flags;
+    (void)m_newMsgHdr->GetFlags(&flags);
+    if (flags & MSG_FLAG_EXPUNGED)
+    {
+      nsCOMPtr<nsIDBFolderInfo> folderInfo;
+      m_mailDB->GetDBFolderInfo(getter_AddRefs(folderInfo));
+      PRUint32 size;
+      (void)m_newMsgHdr->GetMessageSize(&size);
+      folderInfo->ChangeExpungedBytes(size);
+      m_newMsgHdr = nsnull;
+    }
+    else if (m_mailDB)
+    {
+      m_mailDB->AddNewHdrToDB(m_newMsgHdr, PR_TRUE);
+      m_newMsgHdr = nsnull;
+    }
+    else
+      NS_ASSERTION(PR_FALSE, "no database while parsing local folder");	// should have a DB, no?
+  }
+  else if (m_mailDB)
+  {
+    nsCOMPtr<nsIDBFolderInfo> folderInfo;
+    m_mailDB->GetDBFolderInfo(getter_AddRefs(folderInfo));
+    if (folderInfo)
+      folderInfo->ChangeExpungedBytes(m_position - m_envelope_pos);
+  }
+  return 0;
 }
 
 void nsMsgMailboxParser::AbortNewHeader()
@@ -1463,6 +1433,7 @@ nsParseNewMailState::Init(nsIFolder *rootFolder, nsIMsgFolder *downloadFolder, n
 	m_rootFolder = rootFolder;
 	m_inboxFileSpec = folder;
 	m_inboxFileStream = inboxFileStream;
+  m_msgWindow = aMsgWindow;
 	// the new mail parser isn't going to get the stream input, it seems, so we can't use
 	// the OnStartRequest mechanism the mailbox parser uses. So, let's open the db right now.
 	nsCOMPtr<nsIMsgDatabase> mailDB;
@@ -1502,6 +1473,13 @@ nsParseNewMailState::~nsParseNewMailState()
 #endif
 }
 
+// not an IMETHOD so we don't need to do error checking or return an error.
+// We only have one caller.
+void nsParseNewMailState::GetMsgWindow(nsIMsgWindow **aMsgWindow)
+{
+  NS_IF_ADDREF(*aMsgWindow = m_msgWindow);
+}
+
 
 // This gets called for every message because libnet calls IncorporateBegin,
 // IncorporateWrite (once or more), and IncorporateComplete for every message.
@@ -1528,7 +1506,7 @@ void nsParseNewMailState::DoneParsingFolder(nsresult status)
 
 PRInt32 nsParseNewMailState::PublishMsgHeader(nsIMsgWindow *msgWindow)
 {
-  PRBool		moved = PR_FALSE;
+  PRBool moved = PR_FALSE;
   
   FinishHeader();
   
@@ -1550,7 +1528,7 @@ PRInt32 nsParseNewMailState::PublishMsgHeader(nsIMsgWindow *msgWindow)
         if (!(oldFlags & MSG_FLAG_READ)) // don't mark read messages as new.
           m_newMsgHdr->OrFlags(MSG_FLAG_NEW, &newFlags);
         
-        m_mailDB->AddNewHdrToDB (m_newMsgHdr, PR_TRUE);
+        m_mailDB->AddNewHdrToDB(m_newMsgHdr, PR_TRUE);
       }
     }		// if it was moved by imap filter, m_parseMsgState->m_newMsgHdr == nsnull
     m_newMsgHdr = nsnull;
@@ -1673,7 +1651,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
 			    if ((const char*)actionTargetFolderUri &&
               nsCRT::strcasecmp(m_inboxUri, actionTargetFolderUri))
           {
-            nsresult err = MoveIncorporatedMessage(msgHdr, m_mailDB, (const char *) actionTargetFolderUri, filter, msgWindow);
+            nsresult err = MoveIncorporatedMessage(msgHdr, m_mailDB, actionTargetFolderUri, filter, msgWindow);
             if (NS_SUCCEEDED(err))
             {
               if (loggingEnabled)
@@ -1733,7 +1711,7 @@ int nsParseNewMailState::MarkFilteredMessageRead(nsIMsgDBHdr *msgHdr)
 
 nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr, 
 											   nsIMsgDatabase *sourceDB, 
-											   const char *destFolderUri,
+											   const nsACString& destFolderUri,
                                                       nsIMsgFilter *filter,
                                                       nsIMsgWindow *msgWindow)
 {
@@ -1921,11 +1899,10 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
     // update the folder size so we won't reparse.
     UpdateDBFolderInfo(destMailDB);
     if (destIFolder != nsnull)
-      destIFolder->SummaryChanged();
+      destIFolder->UpdateSummaryTotals(PR_TRUE);
     
     destMailDB->Commit(nsMsgDBCommitType::kLargeCommit);
   }  
-  
   return err;
 }
 
