@@ -2062,7 +2062,6 @@ nsresult GetMailNewsFont(MimeObject *obj, PRBool styleFixed,  PRInt32 *fontPixel
     nsCOMPtr<nsICharsetConverterManager2> charSetConverterManager2;
     nsCOMPtr <nsIAtom> charsetAtom;
     nsCOMPtr<nsIAtom> langGroupAtom;
-    const PRUnichar* langGroup = nsnull;
     nsCAutoString prefStr;
 
     ToLowerCase(charset);
@@ -2078,15 +2077,13 @@ nsresult GetMailNewsFont(MimeObject *obj, PRBool styleFixed,  PRInt32 *fontPixel
     rv = charSetConverterManager2->GetCharsetLangGroup(charsetAtom,getter_AddRefs(langGroupAtom));
     if (NS_FAILED(rv))
       return rv;
-    rv = langGroupAtom->GetUnicode(&langGroup);
+    rv = langGroupAtom->ToUTF8String(fontLang);
     if (NS_FAILED(rv))
       return rv;
 
-    fontLang.AssignWithConversion(langGroup);
-
     // get a font size from pref
     prefStr.Assign(!styleFixed ? "font.size.variable." : "font.size.fixed.");
-    prefStr.AppendWithConversion(langGroup);
+    prefStr.Append(fontLang);
     rv = prefs->GetIntPref(prefStr.get(), fontPixelSize);
     if (NS_FAILED(rv))
       return rv;
