@@ -1,24 +1,39 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
- */
+ * Contributor(s):
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #include "modmimee.h"
 #include "mimemsig.h"
@@ -30,6 +45,7 @@
 #include "nsMimeTypes.h"
 #include "msgCore.h"
 #include "nsMimeStringResources.h"
+#include "mimemoz2.h"
 
 
 #define MIME_SUPERCLASS mimeMultipartClass
@@ -102,9 +118,13 @@ MimeMultipartSigned_cleanup (MimeObject *obj, PRBool finalizing_p)
   mult->state = MimeMultipartEpilogue;  /* #58075.  Fix suggested by jwz */
   sig->state = MimeMultipartSignedEpilogue;
 
-#ifdef MOZ_SECURITY
-HG09003
-#endif /* MOZ_SECURITY */
+  if (finalizing_p && sig->crypto_closure) {
+    /* Don't free these until this object is really going away -- keep them
+       around for the lifetime of the MIME object, so that we can get at the
+       security info of sub-parts of the currently-displayed message. */
+    ((MimeMultipartSignedClass *) obj->clazz)->crypto_free (sig->crypto_closure);
+     sig->crypto_closure = 0;
+  }
 
   if (sig->sig_decoder_data)
 	{
@@ -113,7 +133,6 @@ HG09003
 	}
 }
 
-
 static int
 MimeMultipartSigned_parse_eof (MimeObject *obj, PRBool abort_p)
 {
@@ -129,9 +148,8 @@ MimeMultipartSigned_parse_eof (MimeObject *obj, PRBool abort_p)
 	  sig->state == MimeMultipartSignedSignatureLine ||
 	  sig->state == MimeMultipartSignedEpilogue)
 	{
-#ifdef MOZ_SECURITY
-    HG77782
-#endif /* MOZ_SECURITY */
+    status = (((MimeMultipartSignedClass *) obj->clazz)->crypto_signature_eof) (sig->crypto_closure, abort_p);
+    if (status < 0) return status;
 	}
 
   if (!abort_p)
@@ -271,14 +289,14 @@ MimeMultipartSigned_parse_line (char *line, PRInt32 length, MimeObject *obj)
 	case MimeMultipartSignedBodyHeaders:
 	case MimeMultipartSignedBodyLine:
 
-	  if (!sig->xlation_closure)
+	  if (!sig->crypto_closure)
 		{
 		  /* Set error change */
 		  PR_SetError(0, 0);
 		  /* Initialize the signature verification library. */
-		  sig->xlation_closure = (((MimeMultipartSignedClass *) obj->clazz)
-								 ->xlation_init) (obj);
-		  if (!sig->xlation_closure)
+		  sig->crypto_closure = (((MimeMultipartSignedClass *) obj->clazz)
+								 ->crypto_init) (obj);
+		  if (!sig->crypto_closure)
 			{
 			  status = PR_GetError();
 			  PR_ASSERT(status < 0);
@@ -309,7 +327,7 @@ MimeMultipartSigned_parse_line (char *line, PRInt32 length, MimeObject *obj)
 			 of after, except for the first line, which is not preceeded by a
 			 newline.
 
-			 For purposes of xlationgraphic hashing, we always hash line
+			 For purposes of cryptographic hashing, we always hash line
 			 breaks as CRLF -- the canonical, on-the-wire linebreaks, since
 			 we have no idea of knowing what line breaks were used on the
 			 originating system (SMTP rightly destroys that information.)
@@ -319,28 +337,28 @@ MimeMultipartSigned_parse_line (char *line, PRInt32 length, MimeObject *obj)
 		  if (length > 0 && line[length-1] == nsCRT::LF) length--;
 		  if (length > 0 && line[length-1] == nsCRT::CR) length--;
 
-		  PR_ASSERT(sig->xlation_closure);
+		  PR_ASSERT(sig->crypto_closure);
 
 		  if (!first_line_p)
 			{
 			  /* Push out a preceeding newline... */
 			  char nl[] = CRLF;
 			  status = (((MimeMultipartSignedClass *) obj->clazz)
-						->xlation_data_hash (nl, 2, sig->xlation_closure));
+						->crypto_data_hash (nl, 2, sig->crypto_closure));
 			  if (status < 0) return status;
 			}
 
 		  /* Now push out the line sans trailing newline. */
 		  if (length > 0)
 			status = (((MimeMultipartSignedClass *) obj->clazz)
-					  ->xlation_data_hash (line,length, sig->xlation_closure));
+					  ->crypto_data_hash (line,length, sig->crypto_closure));
 		  if (status < 0) return status;
 		}
 	  break;
 
 	case MimeMultipartSignedSignatureHeaders:
 
-	  if (sig->xlation_closure &&
+	  if (sig->crypto_closure &&
 		  old_state != mult->state)
 		{
 		  /* We have just moved out of the MimeMultipartSignedBodyLine
@@ -348,7 +366,7 @@ MimeMultipartSigned_parse_line (char *line, PRInt32 length, MimeObject *obj)
 			 reached the end of the signed data.
 		   */
 		  status = (((MimeMultipartSignedClass *) obj->clazz)
-					->xlation_data_eof) (sig->xlation_closure, PR_FALSE);
+					->crypto_data_eof) (sig->crypto_closure, PR_FALSE);
 		  if (status < 0) return status;
 		}
 	  break;
@@ -368,38 +386,38 @@ MimeMultipartSigned_parse_line (char *line, PRInt32 length, MimeObject *obj)
 	   */
 	  {
 		MimeDecoderData *(*fn) (nsresult (*) (const char*, PRInt32,void*), void*) = 0;
-		char *encoding = MimeHeaders_get (sig->sig_hdrs,
-										  HEADER_CONTENT_TRANSFER_ENCODING,
-										  PR_TRUE, PR_FALSE);
-		if (!encoding)
-		  ;
-		else if (!nsCRT::strcasecmp(encoding, ENCODING_BASE64))
-		  fn = &MimeB64DecoderInit;
-		else if (!nsCRT::strcasecmp(encoding, ENCODING_QUOTED_PRINTABLE))
-		  fn = &MimeQPDecoderInit;
-		else if (!nsCRT::strcasecmp(encoding, ENCODING_UUENCODE) ||
-				 !nsCRT::strcasecmp(encoding, ENCODING_UUENCODE2) ||
-				 !nsCRT::strcasecmp(encoding, ENCODING_UUENCODE3) ||
-				 !nsCRT::strcasecmp(encoding, ENCODING_UUENCODE4))
-		  fn = &MimeUUDecoderInit;
-
+    nsXPIDLCString encoding;
+    encoding.Adopt(MimeHeaders_get (sig->sig_hdrs,
+                   HEADER_CONTENT_TRANSFER_ENCODING,
+                   PR_TRUE, PR_FALSE));
+    if (encoding.IsEmpty())
+      ;
+    else if (!nsCRT::strcasecmp(encoding.get(), ENCODING_BASE64))
+      fn = &MimeB64DecoderInit;
+    else if (!nsCRT::strcasecmp(encoding.get(), ENCODING_QUOTED_PRINTABLE))
+      fn = &MimeQPDecoderInit;
+    else if (!nsCRT::strcasecmp(encoding.get(), ENCODING_UUENCODE) ||
+             !nsCRT::strcasecmp(encoding.get(), ENCODING_UUENCODE2) ||
+             !nsCRT::strcasecmp(encoding.get(), ENCODING_UUENCODE3) ||
+             !nsCRT::strcasecmp(encoding.get(), ENCODING_UUENCODE4))
+      fn = &MimeUUDecoderInit;
 		if (fn)
 		  {
 			sig->sig_decoder_data =
 			  fn (((nsresult (*) (const char *, PRInt32, void *))
 				   (((MimeMultipartSignedClass *) obj->clazz)
-					->xlation_signature_hash)),
-				  sig->xlation_closure);
+					->crypto_signature_hash)),
+				  sig->crypto_closure);
 			if (!sig->sig_decoder_data)
 			  return MIME_OUT_OF_MEMORY;
 		  }
 	  }
 
-	  /* Show these headers to the xlation module. */
+	  /* Show these headers to the crypto module. */
 	  if (hash_line_p)
 		{
 		  status = (((MimeMultipartSignedClass *) obj->clazz)
-					->xlation_signature_init) (sig->xlation_closure,
+					->crypto_signature_init) (sig->crypto_closure,
 											  obj, sig->sig_hdrs);
 		  if (status < 0) return status;
 		}
@@ -415,8 +433,8 @@ MimeMultipartSigned_parse_line (char *line, PRInt32 length, MimeObject *obj)
 			status = MimeDecoderWrite (sig->sig_decoder_data, line, length);
 		  else
 			status = (((MimeMultipartSignedClass *) obj->clazz)
-					  ->xlation_signature_hash (line, length,
-											   sig->xlation_closure));
+					  ->crypto_signature_hash (line, length,
+											   sig->crypto_closure));
 		  if (status < 0) return status;
 		}
 	  break;
@@ -582,7 +600,7 @@ MimeMultipartSigned_emit_child (MimeObject *obj)
   int status = 0;
   MimeObject *body;
 
-  PR_ASSERT(sig->xlation_closure);
+  NS_ASSERTION(sig->crypto_closure, "no crypto closure");
 
   /* Emit some HTML saying whether the signature was cool.
 	 But don't emit anything if in FO_QUOTE_MESSAGE mode.
@@ -592,17 +610,19 @@ MimeMultipartSigned_emit_child (MimeObject *obj)
 	  obj->options->write_html_p &&
 	  obj->options->output_fn &&
 	  obj->options->headers != MimeHeadersCitation &&
-	  sig->xlation_closure)
+	  sig->crypto_closure)
 	{
 	  char *html = (((MimeMultipartSignedClass *) obj->clazz)
-					->xlation_generate_html (sig->xlation_closure));
+					->crypto_generate_html (sig->crypto_closure));
+#if 0 // XXX For the moment, no HTML output. Fix this XXX //
 	  if (!html) return -1; /* MIME_OUT_OF_MEMORY? */
 
 	  status = MimeObject_write(obj, html, nsCRT::strlen(html), PR_FALSE);
 	  PR_Free(html);
 	  if (status < 0) return status;
+#endif
 
-	  /* Now that we have written out the xlation stamp, the outermost header
+	  /* Now that we have written out the crypto stamp, the outermost header
 		 block is well and truly closed.  If this is in fact the outermost
 		 message, then run the post_header_html_fn now.
 	   */
@@ -622,7 +642,7 @@ MimeMultipartSigned_emit_child (MimeObject *obj)
 		  obj->options->state->post_header_html_run_p = PR_TRUE;
 		  if (html)
 			{
-			  status = MimeObject_write(obj, html, nsCRT::strlen(html), PR_FALSE);
+			  status = MimeObject_write(obj, html, strlen(html), PR_FALSE);
 			  PR_Free(html);
 			  if (status < 0) return status;
 			}
@@ -649,6 +669,38 @@ MimeMultipartSigned_emit_child (MimeObject *obj)
   status = (((MimeMultipartClass *)(&MIME_SUPERCLASS))->create_child(obj));
   if (status < 0) return status;
 
+  // Notify the charset of the first part.
+  if (obj->options && !(obj->options->override_charset)) {
+    MimeObject *firstChild = ((MimeContainer*) obj)->children[0];
+    char *disposition = MimeHeaders_get (firstChild->headers,
+                                         HEADER_CONTENT_DISPOSITION, 
+                                         PR_TRUE,
+                                         PR_FALSE);
+    // check if need to show as inline
+    if (!disposition)
+    {
+      const char *content_type = firstChild->content_type;
+      if (!nsCRT::strcasecmp (content_type, TEXT_PLAIN) ||
+          !nsCRT::strcasecmp (content_type, TEXT_HTML) ||
+          !nsCRT::strcasecmp (content_type, TEXT_MDL) ||
+          !nsCRT::strcasecmp (content_type, MULTIPART_ALTERNATIVE) ||
+          !nsCRT::strcasecmp (content_type, MULTIPART_RELATED) ||
+          !nsCRT::strcasecmp (content_type, MESSAGE_NEWS) ||
+          !nsCRT::strcasecmp (content_type, MESSAGE_RFC822)) {
+        char *ct = MimeHeaders_get(mult->hdrs, HEADER_CONTENT_TYPE, PR_FALSE, PR_FALSE);
+        if (ct) {
+          char *cset = MimeHeaders_get_parameter (ct, "charset", NULL, NULL);
+          if (cset) {
+            mimeEmitterUpdateCharacterSet(obj->options, cset);
+            SetMailCharacterSetToMsgWindow(obj, cset);
+            PR_Free(cset);
+          }
+          PR_Free(ct);
+        }
+      }
+    }
+  }
+
   /* Retrieve the child that it created.
    */
   PR_ASSERT(cont->nchildren == 1);
