@@ -1,25 +1,40 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
  *       Ben Bucksch <mozilla@bucksch.org>
- */
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #include "mimetpfl.h"
 #include "mimebuf.h"
@@ -27,6 +42,7 @@
 #include "plstr.h"
 #include "mozITXTToHTMLConv.h"
 #include "nsString.h"
+#include "nsReadableUtils.h"
 #include "nsMimeStringResources.h"
 #include "nsIPref.h"
 #include "nsIServiceManager.h"
@@ -134,34 +150,31 @@ MimeInlineTextPlainFlowed_parse_begin (MimeObject *obj)
   // Get font
   // only used for viewing (!plainHTML)
   nsCAutoString fontstyle;
+  nsCAutoString fontLang;     // langgroup of the font
+
+
+  // generic font-family name ( -moz-fixed for fixed font and NULL for
+  // variable font ) is sufficient now that bug 105199 has been fixed.
+
+  if (exdata->fixedwidthfont)
+    fontstyle = "font-family: -moz-fixed";
+
   if (nsMimeOutput::nsMimeMessageBodyDisplay == obj->options->format_out ||
       nsMimeOutput::nsMimeMessagePrintOutput == obj->options->format_out)
   {
-    /* Use a langugage sensitive default font
-       (otherwise unicode font will be used since the data is UTF-8). */
-    char fontName[128];     // default font name
     PRInt32 fontSize;       // default font size
     PRInt32 fontSizePercentage;   // size percentage
     nsresult rv = GetMailNewsFont(obj, exdata->fixedwidthfont,
-                                  fontName, 128, &fontSize, &fontSizePercentage);
+                                  &fontSize, &fontSizePercentage, fontLang);
     if (NS_SUCCEEDED(rv))
     {
-      fontstyle = "font-family: ";
-      fontstyle += fontName;
-      fontstyle += "; font-size: ";
+      if ( ! fontstyle.IsEmpty() ) {
+        fontstyle += "; ";
+      }
+      fontstyle += "font-size: ";
       fontstyle.AppendInt(fontSize);
       fontstyle += "px;";
     }
-    else
-    {
-      if (exdata->fixedwidthfont)
-        fontstyle = "font-family: -moz-fixed";
-    }
-  }
-  else  // DELETEME: makes sense?
-  {
-    if (exdata->fixedwidthfont)
-      fontstyle = "font-family: -moz-fixed";
   }
 
   // Opening <div>.
@@ -177,6 +190,12 @@ MimeInlineTextPlainFlowed_parse_begin (MimeObject *obj)
       openingDiv += fontstyle;
       openingDiv += '"';
     }
+    if (!plainHTML && !fontLang.IsEmpty())
+    {
+      openingDiv += " lang=\"";
+      openingDiv += fontLang;
+      openingDiv += '\"';
+    }
     openingDiv += ">";
     status = MimeObject_write(obj, NS_CONST_CAST(char*, openingDiv.get()), openingDiv.Length(), PR_FALSE);
     if (status < 0) return status;
@@ -189,6 +208,7 @@ static int
 MimeInlineTextPlainFlowed_parse_eof (MimeObject *obj, PRBool abort_p)
 {
   int status = 0;
+  struct MimeInlineTextPlainFlowedExData *exdata = nsnull;
 
   PRBool quoting = ( obj->options
     && ( obj->options->format_out == nsMimeOutput::nsMimeMessageQuoting ||
@@ -206,7 +226,6 @@ MimeInlineTextPlainFlowed_parse_eof (MimeObject *obj, PRBool abort_p)
   // Look up and unlink "our" extended data structure
   // We do it in the beginning so that if an error occur, we can
   // just free |exdata|.
-  struct MimeInlineTextPlainFlowedExData *exdata;
   struct MimeInlineTextPlainFlowedExData **prevexdata;
   prevexdata = &MimeInlineTextPlainFlowedExDataList;
 
@@ -358,7 +377,7 @@ MimeInlineTextPlainFlowed_parse_line (char *line, PRInt32 length, MimeObject *ob
       // convert to unicode so it won't confuse ScanTXT.
       char *newcstr;
 
-      newcstr = lineSource.ToNewCString();      // lineSource uses nsString but the string is NOT unicode
+      newcstr = ToNewCString(lineSource);      // lineSource uses nsString but the string is NOT unicode
       if (!newcstr) return -1;
 
       nsAutoString ustr;
@@ -379,8 +398,8 @@ MimeInlineTextPlainFlowed_parse_line (char *line, PRInt32 length, MimeObject *ob
           Recycle(u);
           rv = nsMsgI18NConvertFromUnicode(mailCharsetStr, ustr, cstr);
           if (NS_SUCCEEDED(rv))
-            lineResult.AssignWithConversion(cstr);   // create nsString which contains NON unicode 
-                                                     // as the following code expecting it
+            lineResult.AssignWithConversion(cstr.get());   // create nsString which contains NON unicode 
+                                                           // as the following code expecting it
         }
       }
       if (NS_FAILED(rv))
@@ -418,8 +437,8 @@ MimeInlineTextPlainFlowed_parse_line (char *line, PRInt32 length, MimeObject *ob
       preface += " style=\"";
       preface += style;
       preface += '"';
-      PR_FREEIF(style);
     }
+    PR_FREEIF(style);
     preface += '>';
   }
   while(quoteleveldiff<0) {
@@ -440,8 +459,8 @@ MimeInlineTextPlainFlowed_parse_line (char *line, PRInt32 length, MimeObject *ob
         && lineSource[0] == '-'
         &&
         (
-          lineSource.EqualsWithConversion("-- \r", PR_FALSE, 4) ||
-          lineSource.EqualsWithConversion("-- \n", PR_FALSE, 4)
+          Substring(lineSource, 0, 4).Equals(NS_LITERAL_STRING("-- \r")) ||
+          Substring(lineSource, 0, 4).Equals(NS_LITERAL_STRING("-- \n"))
         )
       )
     {
@@ -472,10 +491,10 @@ MimeInlineTextPlainFlowed_parse_line (char *line, PRInt32 length, MimeObject *ob
 
   if (!(exdata->isSig && quoting))
   {
-    char* tmp = preface.ToNewCString();
+    char* tmp = ToNewCString(preface);
     status = MimeObject_write(obj, tmp, preface.Length(), PR_TRUE);
     Recycle(tmp);
-    tmp = lineResult2.ToNewCString();
+    tmp = ToNewCString(lineResult2);
     status = MimeObject_write(obj, tmp, lineResult2.Length(), PR_TRUE);
     Recycle(tmp);
     return status;
