@@ -51,11 +51,9 @@
 #include "nsIMsgAccountManager.h"
 #include "nsMsgFolderFlags.h"
 #include "nsIRequestObserver.h"
+
 // This file contains the news article download state machine.
 
-static NS_DEFINE_CID(kNntpServiceCID,	NS_NNTPSERVICE_CID);
-
-
 // if pIds is not null, download the articles whose id's are passed in. Otherwise,
 // which articles to download is determined by nsNewsDownloader object,
 // or subclasses thereof. News can download marked objects, for example.
@@ -92,7 +90,6 @@ nsNewsDownloader::nsNewsDownloader(nsIMsgWindow *window, nsIMsgDatabase *msgDB,
   // not the perfect place for this, but I think it will work.
   if (m_window)
     m_window->SetStopped(PR_FALSE);
-  NS_INIT_REFCNT();
 }
 
 nsNewsDownloader::~nsNewsDownloader()
@@ -304,7 +301,8 @@ NS_IMETHODIMP nsNewsDownloader::OnSearchDone(nsresult status)
     if (m_listener)
       return m_listener->OnStopRunningUrl(nsnull, NS_OK);
   }
-  nsresult rv = DownloadArticles(m_window, m_folder, &m_keysToDownload);
+  nsresult rv = DownloadArticles(m_window, m_folder, 
+                  /* we've already set m_keysToDownload, so don't pass it in */ nsnull);
   if (NS_FAILED(rv))
     if (m_listener)
       m_listener->OnStopRunningUrl(nsnull, rv);
@@ -347,7 +345,6 @@ DownloadMatchingNewsArticlesToNewsDB::~DownloadMatchingNewsArticlesToNewsDB()
 {
 }
 
-static NS_DEFINE_CID(kMsgAccountManagerCID, NS_MSGACCOUNTMANAGER_CID);
 static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
 
 NS_IMPL_ISUPPORTS1(nsMsgDownloadAllNewsgroups, nsIUrlListener)
@@ -355,7 +352,6 @@ NS_IMPL_ISUPPORTS1(nsMsgDownloadAllNewsgroups, nsIUrlListener)
 
 nsMsgDownloadAllNewsgroups::nsMsgDownloadAllNewsgroups(nsIMsgWindow *window, nsIUrlListener *listener)
 {
-  NS_INIT_REFCNT();
   m_window = window;
   m_listener = listener;
   m_downloaderForGroup = new DownloadMatchingNewsArticlesToNewsDB(window, nsnull, nsnull, this);
@@ -381,8 +377,16 @@ nsMsgDownloadAllNewsgroups::OnStopRunningUrl(nsIURI* url, nsresult exitCode)
   {
     if (m_downloadedHdrsForCurGroup)
     {
-      rv = DownloadMsgsForCurrentGroup();
+      PRBool savingArticlesOffline = PR_FALSE;
+      nsCOMPtr <nsIMsgNewsFolder> newsFolder = do_QueryInterface(m_currentFolder);
+      if (newsFolder)
+        newsFolder->GetSaveArticleOffline(&savingArticlesOffline);
+
       m_downloadedHdrsForCurGroup = PR_FALSE;
+      if (savingArticlesOffline) // skip this group - we're saving to it already
+        rv = ProcessNextGroup();
+      else
+        rv = DownloadMsgsForCurrentGroup();
     }
     else
     {
@@ -425,10 +429,9 @@ nsresult nsMsgDownloadAllNewsgroups::AdvanceToNextServer(PRBool *done)
 
   while (serverIndex < numServers)
   {
-    nsCOMPtr <nsISupports> serverSupports = getter_AddRefs(m_allServers->ElementAt(serverIndex));
+    nsCOMPtr<nsIMsgIncomingServer> server = do_QueryElementAt(m_allServers, serverIndex);
     serverIndex++;
 
-    nsCOMPtr<nsIMsgIncomingServer> server = do_QueryInterface(serverSupports);
     nsCOMPtr <nsINntpIncomingServer> newsServer = do_QueryInterface(server);
     if (!newsServer) // we're only looking for news servers
       continue;
