@@ -5,7 +5,7 @@
 /* *                                                                        * */
 /* * project   : libmng                                                     * */
 /* * file      : libmng_chunks.h           copyright (c) 2000 G.Juyn        * */
-/* * version   : 0.9.3                                                      * */
+/* * version   : 1.0.5                                                      * */
 /* *                                                                        * */
 /* * purpose   : Chunk structures (definition)                              * */
 /* *                                                                        * */
@@ -40,6 +40,13 @@
 /* *             0.9.3 - 10/16/2000 - G.Juyn                                * */
 /* *             - added JDAA chunk                                         * */
 /* *                                                                        * */
+/* *             1.0.5 - 08/19/2002 - G.Juyn                                * */
+/* *             - added HLAPI function to copy chunks                      * */
+/* *             1.0.5 - 09/14/2002 - G.Juyn                                * */
+/* *             - added event handling for dynamic MNG                     * */
+/* *             1.0.5 - 11/28/2002 - G.Juyn                                * */
+/* *             - fixed definition of iMethodX/Y for MAGN chunk            * */
+/* *                                                                        * */
 /* ************************************************************************** */
 
 #if defined(__BORLANDC__) && defined(MNG_STRICT_ANSI)
@@ -81,6 +88,10 @@ typedef mng_retcode (*mng_readchunk)    (mng_datap   pData,
 typedef mng_retcode (*mng_writechunk)   (mng_datap   pData,
                                          mng_chunkp  pChunk);
 
+typedef mng_retcode (*mng_assignchunk)  (mng_datap   pData,
+                                         mng_chunkp  pChunkto,
+                                         mng_chunkp  pChunkfrom);
+
 /* ************************************************************************** */
 
 typedef struct {                       /* generic header */
@@ -89,6 +100,7 @@ typedef struct {                       /* generic header */
            mng_cleanupchunk  fCleanup;
            mng_readchunk     fRead;
            mng_writechunk    fWrite;
+           mng_assignchunk   fAssign;
            mng_chunkp        pNext;    /* for double-linked list */
            mng_chunkp        pPrev;
         } mng_chunk_header;
@@ -730,19 +742,42 @@ typedef struct {                       /* MAGN */
            mng_chunk_header  sHeader;
            mng_uint16        iFirstid;
            mng_uint16        iLastid;
-           mng_uint16        iMethodX;
+           mng_uint8         iMethodX;
            mng_uint16        iMX;
            mng_uint16        iMY;
            mng_uint16        iML;
            mng_uint16        iMR;
            mng_uint16        iMT;
            mng_uint16        iMB;
-           mng_uint16        iMethodY;
+           mng_uint8         iMethodY;
         } mng_magn;
 typedef mng_magn * mng_magnp;
 
 /* ************************************************************************** */
 
+typedef struct {                       /* EvNT entry */
+           mng_uint8         iEventtype;
+           mng_uint8         iMasktype;
+           mng_int32         iLeft;
+           mng_int32         iRight;
+           mng_int32         iTop;
+           mng_int32         iBottom;
+           mng_uint16        iObjectid;
+           mng_uint8         iIndex;
+           mng_uint32        iSegmentnamesize;
+           mng_pchar         zSegmentname;
+        } mng_evnt_entry;
+typedef mng_evnt_entry * mng_evnt_entryp;
+
+typedef struct {                       /* EvNT */
+           mng_chunk_header  sHeader;
+           mng_uint32        iCount;
+           mng_evnt_entryp   pEntries;
+        } mng_evnt;
+typedef mng_evnt * mng_evntp;
+
+/* ************************************************************************** */
+
 typedef struct {                       /* unknown chunk */
            mng_chunk_header  sHeader;
            mng_uint32        iDatasize;
