@@ -37,7 +37,6 @@
 #include "prtypes.h"
 #include "prinrval.h"
 
-#include "nsRepository.h"
 #include "nsIComponentManager.h"
 #include "nsCOMPtr.h"
 #include "nsString.h"
@@ -54,6 +53,7 @@
 #include "nsISignatureVerifier.h"
 #include "nsIObserverService.h"
 #include "nsWeakReference.h"
+#include "nsIObserver.h"
 
 class nsIInputStream;
 class nsJARManifestItem;
@@ -82,9 +82,6 @@ class nsJAR : public nsIZipReader, public nsIJAR
 
     NS_DECL_NSIJAR
 
-    static NS_METHOD
-    Create(nsISupports *aOuter, REFNSIID aIID, void **aResult);
-
     PRIntervalTime GetReleaseTime() {
         return mReleaseTime;
     }
@@ -104,7 +101,8 @@ class nsJAR : public nsIZipReader, public nsIJAR
     void SetZipReaderCache(nsZipReaderCache* cache) {
       mCache = cache;
     }
-  
+
+    PRFileDesc* OpenFile();
   protected:
     //-- Private data members
     nsCOMPtr<nsIFile>        mZipFile;        // The zip/jar file on disk
@@ -116,7 +114,9 @@ class nsJAR : public nsIZipReader, public nsIJAR
     PRIntervalTime           mReleaseTime;    // used by nsZipReaderCache for flushing entries
     nsZipReaderCache*        mCache;          // if cached, this points to the cache it's contained in
 	PRLock*					 mLock;	
-
+    PRInt32                  mTotalItemsInManifest;
+    PRFileDesc*              mFd;
+    
     //-- Private functions
     nsresult ParseManifest(nsISignatureVerifier* verifier);
     void     ReportError(const char* aFilename, PRInt16 errorCode);
@@ -198,9 +198,6 @@ public:
   nsZipReaderCache();
   virtual ~nsZipReaderCache();
 
-  static NS_METHOD
-  Create(nsISupports *aOuter, REFNSIID aIID, void **aResult);
-
   nsresult ReleaseZip(nsJAR* reader);
 
 protected:
